/*******************************************************************************
*	acquire		(see manual, Command Ref., User Functions)	<fc>   *
*	         	Copies the map files from the specified path to        *
*			a temporary directory and loads the map data for use   *
*			by line trace.                                         *
*			                                                       *
*			Returns 1 if successful, 0 if not.                     *
*			j.dabritz, 5/31/89                                     *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for big endian support */

#include <string.h>
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <malloc.h>

#ifdef BIG_ENDIAN
#include "../../incl/big_end.h"
#endif

long acquire( mapdir )
	char *mapdir;
{	

	int filefd;
	long h,i,w;
	long hold_version;
	long row;
	long universe;

	char path[512];
	char *ptr;
	unsigned long raster_size, size;
	char string[512];

	OBJECTS objects;
	ATTRIBUTES v_attributes;


		/*  tell the user what is going on.               */
	message("acquiring map from:  ");
	append_msg(mapdir);
	append_msg("  . . . ");


		/*  remove existing temporary map file, if any     */
	remove_temp();


		/*  check to see if there is file space for map                    */
	if(fsfree(maps_path) < 5000000)
	{	message("  <<  not enough file space to acquire (5 megs.)");
		return(0);
	}

		
		/*  check ram to see if there is at least 1 meg of free space      */
	if(!(ramfree(RAMSPACE)))
	{	message("  <<  not enough RAM space free to acquire a map");
		return(0);
	}


		/*  Check if user has read/write access to maps dir */
	if( access( maps_path, 6) )
	{	message("You do not have read/write permission in -->  ");
		append_msg(maps_path);
		return(0);
	}

		/*  copy map file to temp directory                                */
	i=0;
	message("Making a copy of the map file for use.");
	sprintf(temp_map_path,"%s/temp%d",maps_path,i);
	for(;(i<100)  &&  makedir(temp_map_path);i++) /* make temporary directory */
		sprintf(temp_map_path,"%s/temp%d",maps_path,i);
	if(i >= 100)
	{	message("  <<  cannot create temp directory.");
		remove_temp();
		return(0);
	}
	if(copy_dir(mapdir,temp_map_path))
	{	message("  aborting acquire.");
		remove_temp();
		return(0);
	}


		/*  unpack/uncompress any packed files  */
	message("Checking/uncompressing map files.");
	sprintf(string,"un%s %s/\*",GENERAL_PACK, temp_map_path);
	system_ltp(string);
	sprintf(string,"un%s %s/\*",RASTER_PACK, temp_map_path);
	system_ltp(string);


		/*  set map_path & map_name globals                                */
	strcpy(map_path,mapdir);
	for(ptr = mapdir+strlen(mapdir)-1; (*ptr != '/') && (ptr>=mapdir); ptr--);
	ptr++;
	strcpy(map_name,ptr);

	

		/*  load fixed map globals (header/registration etc)               */
			/* header */
	message("Loading map sub-file:  header");
	sprintf(path,"%s/header",temp_map_path);
	if(get_header(path,&header,1))
	{	message("  <<  cannot read map header file.");
		remove_temp();
		return(0);
	}


			/* history */
	message("Loading map sub-file:  history");
	sprintf(path,"%s/history",temp_map_path);
	if(get_history(path))
	{	message("  <<  cannot read map history file.");
		remove_temp();
		return(0);
	}


			/* level_names */
	sprintf(path,"%s/level_names",temp_map_path);
	if(get_levelnms(path))
	{	message("  <<  cannot read map level_names file.");
		remove_temp();
		return(0);
	}


			/* job_stats */
	message("Loading map sub-file:  job_stats");
	sprintf(path,"%s/job_stats",temp_map_path);
	if(get_jobstats(path))
	{	message("  <<  cannot read job_stats map file.");
		remove_temp();
		job_stats[0] = 0;
		return(0);
	}
	hold_version = job_stats[0];
	job_stats[0] = 0;                   /* set to 0 until all file is loaded */
		/** reset VIEW ONLY flag **/
	job_stats[78] = 0;
		/*  set the raster_wincrement and raster_bincrement globals  */
	raster_wincrement = (job_stats[1] + 1) >> 5;
	raster_bincrement = (job_stats[1] + 1) >> 3;


			/* register */
	message("Loading map sub-file:  register");
	sprintf(path,"%s/register",temp_map_path);
	if(get_register(path))
	{	message("  <<  cannot read register map file.");
		remove_temp();
		return(0);
	}
		/*   remove actual corner registration from corners array **/
	for(i=0; i<4; i++)
	{	corner[4][i][0] = 0.0;
		corner[4][i][1] = 0.0;
		corner[5][i][0] = 0.0;
		corner[5][i][1] = 0.0;
	}
	sprintf(path,"%s/register",temp_map_path);
	if(put_register(path))
	{	message("Cannot write registration data to temp file.  Aborting.");
		return(0);
	}


/********** load raster ******************************************************/
	message("Loading map sub-file:  raster");
		/***********  calculate sizes **************/
	h = job_stats[2] + 1;             /* height of raster in rows  */
	w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */
	raster_size = h*w;                /* size of raster in bytes   */
	
	size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster/r */
		/***********  allocate memory **************/
	rasterm.malloc = (char *) calloc_ltp(size,1);
	if(!rasterm.malloc)
	{	message("  <<  cannot get enough RAM for raster.");
		remove_temp();
		return(0);
	}
		/********  calculate & set global pointers to raster ****************/

	rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
	rasterm.recn = rasterm.rec0 + raster_size;
	craster = (unsigned char **)(rasterm.malloc);
	lraster = (unsigned long **) (rasterm.malloc + 4*h);
	for(i=0; i<h; i++)
	{	craster[i] = (unsigned char *)rasterm.rec0 + w*i;
		lraster[i] = (unsigned long *) (craster[i]);
	}

		/***********  read raster from file *********************/
	sprintf(path,"%s/raster",temp_map_path);
	filefd = openf(path,READ);
	if(0 > filefd)
	{	message("  <<  cannot open raster map file.");
		remove_temp();
		return(0);
	}
	if(0 >= readb(rasterm.rec0,raster_size,1,filefd))
	{	message("  <<  cannot read raster map file.");
		closef(filefd);
		remove_temp();
		return(0);
	}
	closef(filefd);

#if (BIG_ENDIAN && GENERIC_FILE)
	/* if this is a big endian machine, swap the bytes */
	byte_swap((unsigned char *)rasterm.rec0,raster_size,SWAP_LONG);
printf("\ngeneric file hook activated...");
#endif


		/*  load mini-raster                         */
	message("Loading map sub-file:  mini-raster");
	sprintf(path,"%s/mini_raster",temp_map_path);
	filefd = openf(path,READ);
	if(0 > filefd)
	{	message("  <<  cannot open mini_raster map file.");
		remove_temp();
		return(0);
	}
	if(0 >= readb(mini_raster[0],sizeof(mini_raster),1,filefd))
	{	message("  <<  cannot read mini_raster map file.");
		closef(filefd);
		remove_temp();
		return(0);
	}
	closef(filefd);


		/*  load index_array                    */
	message("Loading map sub-file:  index_array");
		/***********  calculate sizes **************/
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = (h*w + 4*h);               /* size to malloc            */
		/***********  allocate memory **************/
	index_arraym.malloc = (char *) calloc_ltp(size,1);
	if(!index_arraym.malloc)
	{	message("  <<  cannot get enough RAM for index_array.");
		remove_temp();
		return(0);
	}
		/********  calculate & set global pointers to index_array ***********/
	index_arraym.rec0 = index_arraym.malloc;
	index_arraym.recn = (char *)(index_arraym.rec0 + h*w);
	index_array = (long **)index_arraym.recn;
	for(i=0; i<h; i++)
		index_array[i] = (long *) (index_arraym.malloc + w*i);

		/***********  read index_array from file *********************/
	sprintf(path,"%s/index_array",temp_map_path);
	filefd = openf(path,READ);
	if(0 > filefd)
	{	message("  <<  cannot open index_array map file.");
		remove_temp();
		return(0);
	}
	if(0 > readb(index_arraym.rec0,h*w,1,filefd))
	{	message("  <<  cannot read index_array map file.");
		closef(filefd);
		remove_temp();
		return(0);
	}
	closef(filefd);

		/************ check ram space *******************************/
	if(!ramfree(RAMSPACE))
	{	message("  <<  not enough ram space to load map.");
		remove_temp();
		return(0);
	}

		/*  load record type files if space allows or set up file pointers */
		/********* objects ***********/
		/*    open file        */
	message("Loading map sub-file:  objects");
	sprintf(path,"%s/objects",temp_map_path);
	objects_fd = openf(path,RDWR);
	if(0 > objects_fd)
	{	message("  <<  cannot open objects map file.");
		remove_temp();
		return(0);
	}
		/*  calculate size to malloc */
	size = objectsm.rec_size * job_stats[10];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(objectsm.malloc = (malloc_ltp(size)))
			{   objectsm.rec0 = objectsm.malloc;
				objectsm.recn = objectsm.rec0 + size;
				if(!readb(objectsm.rec0,size,1,objects_fd))
				{	message("  <<  cannot read objects map file.");
					remove_temp();
					return(0);
				}
			}
		}
	}
		/***** build node_lookup (node lookup array) *********/
		/*  free old space if any */
	if(node_lookup_ptr)
		free_ltp(node_lookup_ptr);
		/*  calloc new space  */
	node_lookup_ptr = calloc_ltp(job_stats[2]+1,4);
	if(!node_lookup_ptr)
	{	message("  <<  cannot calloc new space for node_lookup table.");
		remove_temp();
		return(0);
	}
	else
	{	node_lookup = (long *) node_lookup_ptr;
		for(i=0; i<job_stats[10]; i++)
		{	get_objects(&objects,i);
			if((objects.i12[0] < 8) && (objects.i12[0] != 5)) /* non-is node */
			{	row = objects.f4[1];
				if((i < node_lookup[row])  ||  !(node_lookup[row]))
					node_lookup[row] = i;
			}
		}
	}	

		/********* seg_list ***********/
		/*    open file        */
	message("Loading map sub-file:  seg_list");
	sprintf(path,"%s/seg_list",temp_map_path);
	seg_list_fd = openf(path,RDWR);
	if(0 > seg_list_fd)
	{	message("  <<  cannot open seg_list map file.");
		remove_temp();
		return(0);
	}
		/*  calculate size to malloc */
	size = seg_listm.rec_size * job_stats[12];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(seg_listm.malloc = (malloc_ltp(size)))
			{   seg_listm.rec0 = seg_listm.malloc;
				seg_listm.recn = seg_listm.rec0 + size;
				if(!readb(seg_listm.rec0,size,1,seg_list_fd))
				{	message("  <<  cannot read seg_list map file.");
					remove_temp();
					return(0);
				}
			}
		}
	}


		/********* coord_list ***********/
		/*    open file        */
	message("Loading map sub-file:  coord_list");
	sprintf(path,"%s/coord_list",temp_map_path);
	coord_list_fd = openf(path,RDWR);
	if(0 > coord_list_fd)
	{	message("  <<  cannot open coord_list map file.");
		remove_temp();
		return(0);
	}
		/*  calculate size to malloc */
	size = coord_listm.rec_size * job_stats[11];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(coord_listm.malloc = (malloc_ltp(size)))
			{   coord_listm.rec0 = coord_listm.malloc;
				coord_listm.recn = coord_listm.rec0 + size;
				if(!readb(coord_listm.rec0,size,1,coord_list_fd))
				{	message("  <<  cannot read coord_list map file.");
					remove_temp();
					return(0);
				}
			}
		}
	}


		/********* index_list ***********/
		/*    open file        */
	message("Loading map sub-file:  index_list");
	sprintf(path,"%s/index_list",temp_map_path);
	index_list_fd = openf(path,RDWR);
	if(0 > index_list_fd)
	{	message("  <<  cannot open index_list map file.");
		remove_temp();
		return(0);
	}
		/*  calculate size to malloc */
	size = index_listm.rec_size * job_stats[15];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(index_listm.malloc = (malloc_ltp(size)))
			{   index_listm.rec0 = index_listm.malloc;
				index_listm.recn = index_listm.rec0 + size;
				if(!readb(index_listm.rec0,size,1,index_list_fd))
				{	message("  <<  cannot read index_list map file.");
					remove_temp();
					return(0);
				}
			}
		}
	}


		/********* v_objects ***********/
		/*    open file        */
	message("Loading map sub-file:  v_objects");
	sprintf(path,"%s/v_objects",temp_map_path);
	v_objects_fd = openf(path,RDWR);
	if(0 > v_objects_fd)
	{	message("  <<  cannot open v_objects map file.");
		remove_temp();
		return(0);
	}
		/*  calculate size to malloc */
	size = v_objectsm.rec_size * job_stats[17];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(v_objectsm.malloc = (malloc_ltp(size)))
			{   v_objectsm.rec0 = v_objectsm.malloc;
				v_objectsm.recn = v_objectsm.rec0 + size;
				if(!readb(v_objectsm.rec0,size,1,v_objects_fd))
				{	message("  <<  cannot read v_objects map file.");
					remove_temp();
					return(0);
				}
			}
		}
	}


		/********* v_coordlist ***********/
		/*    open file        */
	message("Loading map sub-file:  v_coordlist");
	sprintf(path,"%s/v_coordlist",temp_map_path);
	v_coordlist_fd = openf(path,RDWR);
	if(0 > v_coordlist_fd)
	{	message("  <<  cannot open v_coordlist map file.");
		remove_temp();
		return(0);
	}
		/*  calculate size to malloc */
	size = v_coordlistm.rec_size * job_stats[18];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(v_coordlistm.malloc = (malloc_ltp(size)))
			{   v_coordlistm.rec0 = v_coordlistm.malloc;
				v_coordlistm.recn = v_coordlistm.rec0 + size;
				if(!readb(v_coordlistm.rec0,size,1,v_coordlist_fd))
				{	message("  <<  cannot read v_coordlist map file.");
					remove_temp();
					return(0);
				}
			}
		}
	}


		/********* attributes ***********/
		/*    open file        */
	message("Loading map sub-file:  attributes");
	sprintf(path,"%s/attributes",temp_map_path);
	attributes_fd = openf(path,RDWR);
	if(0 > attributes_fd)
	{	message("  <<  cannot open attributes map file.");
		remove_temp();
		return(0);
	}
		/*  calculate size to malloc */
	size = attributesm.rec_size * job_stats[13];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(attributesm.malloc = (malloc_ltp(size)))
			{   attributesm.rec0 = attributesm.malloc;
				attributesm.recn = attributesm.rec0 + size;
				if(!readb(attributesm.rec0,size,1,attributes_fd))
				{	message("  <<  cannot read attributes map file.");
					remove_temp();
					return(0);
				}
			}
		}
	}
		/** make sure attributes has at least one record **/
	if(!job_stats[13])
	{	*v_attributes.string = 0;
		v_attributes.next = 0;
		if(put_atrbt(&v_attributes,0))
			return(0);
	}


		/********* v_attributes ***********/
		/*    open file        */
	message("Loading map sub-file:  v_attributes");
	sprintf(path,"%s/v_attributes",temp_map_path);
	v_attributes_fd = openf(path,RDWR);
	if(0 > v_attributes_fd)
	{	message("  <<  cannot open v_attributes map file.");
		remove_temp();
		return(0);
	}
		/*  calculate size to malloc */
	size = v_attributesm.rec_size * job_stats[16];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(v_attributesm.malloc = (malloc_ltp(size)))
			{   v_attributesm.rec0 = v_attributesm.malloc;
				v_attributesm.recn = v_attributesm.rec0 + size;
				if(!readb(v_attributesm.rec0,size,1,v_attributes_fd))
				{	message("  <<  cannot read v_attributes map file.");
					remove_temp();
					return(0);
				}
			}
		}
	}
		/** make sure v_attributes has at least one record **/
	if(!job_stats[16])
	{	*v_attributes.string = 0;
		v_attributes.next = 0;
		if(put_v_attr(&v_attributes,0))
			return(0);
	}


		/********* errors ***********/
		/*    open file        */
	message("Loading map sub-file:  errors");
	sprintf(path,"%s/errors",temp_map_path);
	errors_fd = openf(path,RDWR);
	if(0 > errors_fd)
	{	message("  <<  cannot open errors map file.");
		remove_temp();
		return(0);
	}
		/*  calculate size to malloc */
	size = errorsm.rec_size * job_stats[14];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(errorsm.malloc = (malloc_ltp(size)))
			{   errorsm.rec0 = errorsm.malloc;
				errorsm.recn = errorsm.rec0 + size;
				if(!readb(errorsm.rec0,size,1,errors_fd))
				{	message("  <<  cannot read errors map file.");
					remove_temp();
					return(0);
				}
			}
		}
	}


		/*   load the desired menuset (indicated in the header file) */
	message("Loading menu-set & setting up grahpics display.");
	trim(header.menu_set);
	if(header.menu_set[0] != 0)
		menuset_get(header.menu_set);


		/*  set up graphics & display     */
	job_stats[0] = hold_version;
	for(i=0; i<4; i++)
	{	view[i].lower_leftx = 0;
		view[i].lower_lefty = 0;
	}
	view[0].zoom_factor = mind(2,  (double) 1.0, 
			(double) (400.0/(maxl(2,job_stats[1]+1,job_stats[2]+1))));
	view[1].zoom_factor = 0.25;
	view[2].zoom_factor = 1.0;
	view[3].zoom_factor = 8.0;


		/*** reset the undo array **********/
	for(i=0; i<32; i++)
		undo_array[i].mode = 0;


		/*** reset coordinate type to raster **/
	disp_coord_type = 0;

		/*** reset display to raster only **/
	disp_raster = 1;
	disp_normal = 0;
	disp_i1normal = 0;
	disp_i2normal = 0;
	disp_hglight = 0;
	disp_i1hglight = 0;
	disp_i2hglight = 0;

	dispgraphics((long) (job_stats[1]/2), (long) (job_stats[2]/2), 0);

	cursor(561,80);
	cursor(560,80);


		/*   display map name in upper left block, line 2 */
	strcpy(string,map_name);
	string[14] = 0;
	for(i=2; i<4; i++)
	{	swplane(i);
		if(windowxmin)
		{	bar_off(0,16,111,31);
			disptext(1,0,string);
		}
		else
		{	bar_off(528,416,639,431);
			disptext(26,66,string);
		}
	}


		/*   go out of dig absolute mode if necessary **/
	if(gr_dev == DIG1ABS)
		ginput(1,0,0);
	if(gr_dev == DIG2ABS)
		ginput(2,0,0);

		/*   change input mode to -none - */
	change_mode(0,0);

		/*   check last_map_tyep against map_type */
	if(map_type != job_stats[9])
		default_sets();

		/*   tell user what is going on  */
	append_msg(" done.");

		/*   update objects file to include set 31, 'universe' for file types
           prior to 2.18 */
	if(job_stats[0] < 218)
	{		/** add level name **/
		sprintf(level_names[31],"universe (all objects)");
			/** add set **/
		universe = (unsigned long) (1 << 31);
		for(i=0; i<job_stats[10]; i++)
		{	if(!(i % 250))
			{	sprintf(string,"updating objects file to type 218, at object %d.",i);
				message(string);
			}
			if(get_objects(&objects,i))
				break;
			objects.i12[4] |= universe;
			if(put_objects(&objects,i))
				break;
		}
			/*** update file version ***/
		job_stats[0] = ltp_version * 100.0 + 0.5;
	}

		/*   note the acquire in the diary */
	if(put_diary("map acquired."))
		return(0);

    return(1);
}
