/*******************************************************************************
*	assemble	(see manual, Command Ref., User Function)	<fc>   *
*			Assemble is the primary function of LTPlus.            *
*                                                                              *
*			It converts raster data into vector data in the        *
*			following steps:                                       *
*			1. Thins the raster lines to 1 pixel width.            *
*			2. Finds all 0,1,3,and 4 branch nodes.                 *
*			3. Vectorizes all segments starting (&ending) at       *
*			   those nodes.                                        *
*			4. Stores the raster,  then temporarily erases all     *
*			   segments traces so far.                             *
*			5. Finds all remaining 'on' pixel groups (simple       *
*			   islands) assigns the left-most lowest point in      *
*			   each an makes it a 2-branch node.  Then traces      *
*			   the single segment loop from that node & erases     *
*			   the loop.                                           *
*			6. Restores the full raster from storage into RAM.     *
*			7. Assembles all polygons using the nodes and          *
*			   segment records in the OBJECTS file.                *
*			8. (optionally) reduces vertex (segment) data if       *
*			   specified (see parameter below).                    *
*			9. Recalculate segment length and polygon area based   *
*			   on the reduced vertex set.                          *
*	       10. Updates all serial number by checking the serial    *
*			   number of the object under each new centroid.       *
*	       11. Updates all attributes by taking old object         *
*			   centroids and finding the new objects that they     *
*			   correspond to.                                      *
*	       12. Remove the old vector files and replace them with   *
*			   the new set.                                        *
*	       13. Index the new vectors into the index_array and      *
*			   index_list files.                                   *
*                                                                  *
*		Makes changes in the files:                                *
*			History                                                *
*			Job_stats                                              *
*			Raster                                                 *
*			Objects                                                *
*			Coord_list                                             *
*			Seg_list                                               *
*			Attribute                                              *
*			Index_array                                            *
*			Index_list                                             *
*		and the global variables:                                      *
*			History[]                                              *
*			Job_stats[]                                            *
*                                                                              *
*		Returns 0 on successful End Of Job [EOJ]                       *
*       J.Dabritz   7/11/89                                                    *
*******************************************************************************/
/* Modified 5/92 by DBS@CWU for ULTRIX support */

#include "../../incl/globals.h"
#include "../../incl/in.h"
#ifndef SV32
#include <sys/timeb.h>
#endif


long assemble(reduce)
	long reduce;  /* flag, 1=reduce segment vertices using routine which
			assumes raw vectorized input (8 directions).    */
{
	char temp2[512];
	char string[512];
	
	long i;
	long old_objects, old_coord_list;   /* number of records in old files */
	long x,y;
	long start_time, finish_time;
	long hours, minutes, seconds;


		/********** get start time *****************************************/
	start_time = time((long *) 0);

		/**********  check for current map *********************************/
	if(!job_stats[0])
		return(1);

		/************* check for VIEW ONLY raster **********************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}

		/**********  update index_list if necessary ************************/
	if(job_stats[49])
	{	if(index_map())
			return(1);
	}

	if(abort_cmd())
		return(1);


		/**********  stores old map to a temporary file ********************/
		/*  update map to current status   */
	message("updating temporary map file. . .");
	if(update_temp())
		return(1);
	append_msg(" done.");
		/*  create new temp directory  */
	message("copying temporary file to secondary temp file . . .");
	i=0;
	sprintf(temp2,"%s/temp%d",maps_path,i);
	for(;makedir(temp2);i++)     /* make temporary directory        */
		sprintf(temp2,"%s/temp%d",maps_path,i);

/* #if (ULTRIX || SUNOS) */
	sprintf(string,"%s %s/* %s",COPY_CMD, temp_map_path,temp2);

	if(system_ltp(string))
	{	message("  <<  cannot copy map to temp. space");
		return(1);
	}
	append_msg(" done.");

	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	
		/**********  1. Thins the raster lines to 1 pixel width.           */
	if(thin_lines(0,1))
	{	reset_temp(temp2);
		return(1);
	}


	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}

	
		/**********  1.2 store off old objects & coordlist files **/
	old_objects = job_stats[10];
	old_coord_list = job_stats[11];
	message(" setting up files for updating attributes later . . . ");
		/** next remove existing 'old' files (if any) **/
	sprintf(string,"%s/old_objects",temp_map_path);
	purgef(string);
	sprintf(string,"%s/old_coordlst",temp_map_path);
	purgef(string);
		/** next copy existing files into 'old' files **/
/* #if (ULTRIX || SUNOS) */
	sprintf(string,"%s %s/objects %s/old_objects", COPY_CMD, temp_map_path,temp_map_path);
	if(system_ltp(string))
	{	message("cannot copy objects file.  aborting assemble.");
		reset_temp(temp2);
		return(1);
	}

    /* ULTRIX/UNIX  support COPY_CMD define */
	sprintf(string,"%s %s/coord_list %s/old_coordlst", COPY_CMD, temp_map_path,temp_map_path);

	if(system_ltp(string))
	{	message("cannot copy seg_list file.  aborting assemble.");
		reset_temp(temp2);
		return(1);
	}
	append_msg(" done.");
	


	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	

		/**********  2. Finds all 0,1,3,and 4 branch nodes.                */
		/*  zero out all object/vertex/attribute data   */
	for(i=3; i<8; i++)
		job_stats[i] = 0;
	for(i=10; i<13; i++)
		job_stats[i] = 0;
		/*  find all 0, 1, 3, 4 branch nodes   */
	if(find_nodes())
	{	reset_temp(temp2);
		return(1);
	}


	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	
		/**********  3. Vectorizes all segments starting (&ending) at      */
		/**********     those nodes & all simple island nodes/segments.    */
	if(vectorize())
	{	reset_temp(temp2);
		return(1);
	}


	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	
		/**********  4. Assembles all polygons using the nodes and         */
		/**********     segment records in the OBJECTS file.               */
	if(map_type & 4)     /*  polygon map type **/
	{	if(asmbl_polys())
		{	reset_temp(temp2);
			return(1);
		}
	}


	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	

		/**********  5. centroid polygons                                  */
	if(job_stats[7])     /*  there are polygons  **/
	{	if(polycentroid())
		{	reset_temp(temp2);
			return(1);
		}
	}



	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	
		/**********  6. centroid segments                                  */
	if(job_stats[5] || job_stats[6])   /*  there are segments */
	{	if(segcentroid())
		{	reset_temp(temp2);
			return(1);
		}
	}

	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	

		/**********  8. (optionally) reduces vertex (segment) data if      */
		/**********     specified (see parameter below).                   */
		/**********     calculates segment and polygon length/perimeter    */
		/**********     and area.                                          */
	if(job_stats[5] || job_stats[6])
	{	if(reduce_rdata(reduce))
		{	reset_temp(temp2);
			return(1);
		}
	}

	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	

		/**********  9. Updates all serial numbers and attributes by 
						checking the old object which corrisponds.   */
	if(update_atrbt(old_objects,old_coord_list))
	{
		reset_temp(temp2);
		return(1);
	}

	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	

		/********** 12. Index the new vectors into the index_array and     */
		/**********     index_list files.                                  */
	if(index_map())
	{	reset_temp(temp2);
		return(1);
	}

	if(abort_cmd())
	{	reset_temp(temp2);
		return(1);
	}
	

		/********** 13. Remove the secondary temp(2) map files             */
	message("removing secondary temp file . . . ");
	if(purgef(temp2))
		message("Cannot remove secondary temp file.  It's ok, but takes up space.");
	else
		append_msg("done.");

		/********** 14. Store all map files to temp                        */
	message("Updating temp file and writing diary record . . . ");
	if(update_temp())
	{	reset_temp(temp2);
		return(1);
	}
		
		/*********** 15. update the default sets *********/
	if(default_sets())
		return(1);

		/*** set the assembled flag **/
	job_stats[45]++;
	job_stats[41] = job_stats[45];

		/*** note the assembly in the map diary **/
	sprintf(string,"map assembled (#%d), %d objects, map type = %s",
	 job_stats[41],job_stats[10],trim(header.maptype));
	if(put_diary(string))
		return(1);
	append_msg(" done.");

		/*** redisplay the map ****/
	if((disp_hglight || disp_normal))
	{		/************* call dispgraphics to re-display data *********/
		x = (rwindowxmin + rwindowxmax) / 2;
		y = (rwindowymin + rwindowymax) / 2;
		if(dispgraphics(x,y,resolution_code))
			return(1);
	}

		/*** set level names for levels 1-24 to empty ***/
	for(i=1; i<25; i++)
		sprintf(level_names[i],"set %d, - empty -",i);

		/********** get finish time & compute elapsed time **************/
	finish_time = time((long *) 0);
	seconds = finish_time - start_time;
	hours = seconds / 3600;
	seconds = seconds % 3600;
	minutes = seconds / 60;
	seconds = seconds % 60;
	sprintf(string," assemble, total elapsed clock time = %02d:%02d:%02d",hours,minutes,seconds);
	message(string);
	

	return(0);
}



/******************************************************************************/
/*        reset_temp                                                          */
/*           This function allows assemble to abort gracefully.  It           */
/*        restores the pre-assemble data to the program as if assemble        */
/*        had never occurred.  To do this, it acquires the temp2 file         */
/*        (which has all of the old data), then changed program variables     */
/*        to store it into the origional map space when saved.                */
/*                                                                            */
/*        All globals & files associated with the 'acquire' function          */
/*        are changed.                                                        */
/*                                                                            */
/*        Returns 0 on normal end-of-job, 1 on failure.                       */
/*        J.Dabritz   7/11/89                                                 */
/******************************************************************************/


reset_temp(temp2)
char *temp2;           /*  path of temp storage to be re-stored  */
{
	char origional_path[512];
	char origional_name[32];
	char string[512];


		/************ save map path & name ******************************/
	strcpy(origional_path,map_path);
	strcpy(origional_name,map_name);

		/************ acquire temp2 map *********************************/
	if(!acquire(temp2))
	{	job_stats[0] = 0;
		return(1);
	}

		/************ reset map path ************************************/
	strcpy(map_name,origional_name);
	strcpy(map_path,origional_path);

		/************ change name of map on display *********************/
	if(windowxmin)
	{	strcpy(string,map_name);
		string[14] = 0;
		swplane(2);
		bar_off(0,16,111,31);
		disptext(1,0,string);
		swplane(3);
		bar_off(0,16,111,31);
		disptext(1,0,string);
	}

		/************ return ********************************************/
	return(0);
}
