/*******************************************************************************
*      atrbt_copy      (see manual, Command Ref., User Function)        <fc>   *
*            Sets the graphics input mode to copy attributes from one object   *
*        type and level to another object type on level 0 using the input      *
*        attribute action.                                                     *
*                                                                              *
*        Parameters:   long fotype;  from object type:  XXT where:             *
*                                    XX = level number 0, 32-95                *
*                                    T = object type 0=node, 1=seg, 2=poly     *
*                      long totype;  to obect type (0-2 as in T above)         *
*                      long action;  attribute action (see asgn_atrbt.c)       *
*                                                                              *
*        Globals changed:   atrbt_action, input_mode.                          *
*                                                                              *
*        Values returned:  0=successful completion, 1=error.                   *
*           J.Dabritz 10/1/90                                                  *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long from_otype;
long from_level;

long atrbt_copy(fotype,totype,action)
	long fotype,totype,action;
{
	char string[STRLIST];


		/****** check from otype *******************************/
	from_otype = fotype % 10;
	from_level = fotype / 10;
	if((fotype <0) || (fotype > 952) || ((fotype>2) && (fotype<320)) || (from_otype <0) || (from_otype > 2))
	{	message("From Object Type must be 0-2 on level 0 or 32-95.  no action taken.");
		return(1);
	}
	
		/****** ************************************************/
	if((totype <0)  ||  (totype > 2))
	{	message("TO Object type must be in range 0 - 2.  no action taken.");
		return(1);
	}

		/****** check attribute action code ********************/
	if((action <0)  ||  (action > atrbt_action_limit))
	{	sprintf(string,"Action code must be in range 0 - %d.  no action taken.",atrbt_action_limit);
		message(string);
		return(1);
	}
	
	atrbt_otype = totype;
	atrbt_action = action;
	atrbt_string[0] = (char) 0;

	change_mode(34,0);
	
	return(0);
}


/*******************************************************************************
*       find_atrbt      (see manual, Internal Functions)                <in>   *
*    Finds objects using the input x,y location & search radius.  Finds        *
*    the first attribute string and sets atrbt_string global.                  *
*                                                                              *
*    Arguments:    float x,y,r:   raster coordinates and search radius.        *
*                                 (search radius used for segs & nodes)        *
*                                                                              *
*    Returned Values: (long) 0 = found object & set attribute string global.   *
*                            1 = didn't find object or didn't find attribute.  *
*                                                                              *
*    Global Variables Modified:  none.                                         *
*                                                                              *
*                      j.dabritz  8/9/89                                       *
*******************************************************************************/

long find_atrbt(x,y,r)
float x,y,r;
{	
	OBJECTS object;
	V_OBJECTS vobj;
	ATTRIBUTES atrbt;

	char string[STRLIST];

	long obj_no;


		/********* check for map ****************************************/
	if(!job_stats[0])
	{	message("no current map.");
		return(1);
	}

		/********* find object ******************************************/
	switch(from_otype)     {
	case 0:           /** nodes ***/
		obj_no = find_vnode(x,y,r,from_level);
		break;
	case 1:           /** segments  */
		obj_no = find_vseg(x,y,r,from_level);
		break;
	case 2:           /** polygons  */
		obj_no = find_vpoly(x,y,from_level);
		break;
	default:
		return(1);
	}

		/********* read attribute & set global **************************/
	if(obj_no >= 0)
	{	if(from_level < 32)
		{		/**** get object ****/
			if(get_objects(&object,obj_no))
			{	message("find_atrbt ERROR #1.  Attribute string NOT set.");
				return(1);
			}
				/**** set attribute string global ***/
			if(object.i12[2] == 0)
			{	sprintf(string,"Object found, but has NO attribute.  current atrbt = %s",atrbt_string);
				message(string);
				return(1);
			}
			if(get_attr(&atrbt,object.i12[2]))
			{	message("find_atrbt ERROR #2.  Attribute string NOT set.");
				return(1);
			}
			sprintf(atrbt_string,"%s",atrbt.string);
			atrbt_string[30] = (char) 0;
			sprintf(string,"New Attribute String:  %s",atrbt_string);
			message(string);
		}
		else    /*** import level ***/
		{	if(get_vobjects(&vobj,obj_no))
			{	message("find_atrbt ERROR #3.  Attribute string NOT set.");
				return(1);
			}
				/**** set attribute string global ***/
			if(vobj.atr == 0)
			{	sprintf(string,"Object found, but has NO attribute.  current atrbt = %s",atrbt_string);
				message(string);
				return(1);
			}
			if(get_v_attr(&atrbt,vobj.atr))
			{	message("find_atrbt ERROR #2.  Attribute string NOT set.");
				return(1);
			}
			sprintf(atrbt_string,"%s",atrbt.string);
			atrbt_string[30] = (char) 0;
			sprintf(string,"New Attribute String:  %s",atrbt_string);
			message(string);
		}
	}
	else
	{	message("No object found.");
sprintf(string,"x=%f,  y=%f,  r=%f,  from_level = %d",x,y,r,from_level);
message(string);

		return(1);
	}

		/********* return ***********************************************/
	return(0);
}
