/*******************************************************************************
*	atrbt_list (see manual, Command Ref., User Function)	<fc>   *
*			Sets the grahpics input mode to digitize attributes from a list.   *
*			Sets the object type, attribute action type, and reads the 1st        *
*        attribute from the list.                                              *
*                                                                              *
*			Returns zero upon normal end-of-job (EOJ).             *
*           J.Dabritz 10/2/90                                                  *
*******************************************************************************/
/* Modified 7/92 by DBS@CWU for multi user support */

#include "../../incl/globals.h"
#include "../../incl/in.h"

int atrbt_list_fd = -1;
long next_list_atr();

long atrbt_list(otype,action,fname)
	long otype;	/*   object type code to attribute: 0=node, 1=seg,
				2=polygon                                  */
	long action; 	/*   attribute action code  */
	char fname[];
{
	char string[STRLIST];
	char dummy[STRLIST];


	if((otype <0)  ||  (otype > 2))
	{	message("Object type must be in range 0 - 2.  no action taken.");
		return(1);
	}

		/****** check attribute action code ********************/
	if((action <0)  ||  (action > atrbt_action_limit))
	{	sprintf(string,"Action code must be in range 0 - %d.  no action taken.",atrbt_action_limit);
		message(string);
		return(1);
	}
	
	atrbt_otype = otype;
	atrbt_action = action;

	change_mode(35,0); /** note:  also closes atrbt_list_fd, if open **/

		/****** set file name & open file ***********************/
	trim(fname);

#ifndef MULTIUSER  /***** original code *****/
	if(fname[0] != '/') 
		sprintf(dummy,"%s/dictionaries/%s",ltp_path,fname);
	else
		sprintf(dummy,"%s",fname);

	atrbt_list_fd = openf(dummy,READ);
#else   /***** in multiuser look in home dir first *****/
	if(fname[0] != '/') {
		sprintf(dummy,"%s/.ltp/dictionaries/%s",home_path,fname);
	}
	else
		sprintf(dummy,"%s",fname);
	atrbt_list_fd = openf(dummy,READ);
	if(atrbt_list_fd < 0) {        /* now try the LTPlus dir */
		sprintf(dummy,"%s/dictionaries/%s",ltp_path,fname);
		atrbt_list_fd = openf(dummy,READ);
	}
#endif

	if(atrbt_list_fd < 0)
	{	sprintf(string,"atrbt_list, ERROR:  couldn't open file %s",dummy);
		message(string);
		return(1);
	}

		/****** get next (first) attribute from file & display **/
	next_list_atr();

	
	return(0);
}

/*******************************************************************************
*   atrbt_list_close                                                           *
*        This program closes the atrbt_list_fd                                 *
*                                                                              *
*******************************************************************************/

long atrbt_list_close()
{
	if(atrbt_list_fd >= 0)
		closef(atrbt_list_fd);

	atrbt_list_fd = -1;

	return(0);
}


/*******************************************************************************
*   next_list_atr                                                              *
*        This program reads the atrbt_list file and sets the next attribute.   *
*                                                                              *
*******************************************************************************/

long next_list_atr()
{
	char string[256];
	char *ptr;
	long count;


	if(next_list_atr < 0)
	{	message("next_list_atr, ERROR:  illegal file pointer.");
		return(1);
	}

	count = 0;
	for(atrbt_string[0] = (char) 0; !(*atrbt_string); )
	{	ptr = reada(atrbt_string,255,atrbt_list_fd);
		if(ptr == (char *) 0)
		{		/*** move file pointer to beginning of file ***/
			if(!count)  /** one time only, move file pointer to top **/
			{	message("***** DONE WITH LIST, Starting again at the top. *****");
				movefp(atrbt_list_fd,(long) 0, (int) 0);
				count++;
			}
			else
			{	message("next_list_atr, ERROR, cannot read file, Attribute NOT SET.");
				atrbt_string[0] = (char) 0;
				return(1);
			}
		}
		else if((*atrbt_string > (char) 32)  &&  (*atrbt_string < (char) 127))
		{	trim(atrbt_string);
			sprintf(string,"Next Attribute:  %s",atrbt_string);
			message(string);
			break;
		}
		else
			atrbt_string[0] = (char) 0;
	}
	

	return(0);
}
