/*******************************************************************************
*	atrbt_print	(see manual, Command Ref., User Functions)	<fc>   *
*			Description:  Prints a named attribute set on a        *
*			specific device.                                       *
*                                                                              *
*			Parameters:  long  set_num   (1 - 24)                              *
*                        long  dev       (device code)                         *
*                        char  *file     ( file name.)                         *
*                                                                              *
*			Returns zero upon normal end-of-job (EOJ).             *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long atrbt_print( set_num, dev , file)
	long set_num;
	long dev;
	char *file;
{
	OBJECTS object;
	ATTRIBUTES attribute;

	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	char string[256];
	char *ptr;
	char type[256];

	long i,j;
	long ok;

	int fd;
	
		/************** check for a current map & objects ***********/
	if(!job_stats[0])
	{	message("No map current acquired.  nothing to print or file.");
		return(1);
	}

		/************** check parameters ****************************/
	if(set_num < 0   ||  set_num > 31)
	{	message("Illegal set number, atrbt_print aborted.");
		return(1);
	}
	if(dev < 0   ||  dev > 2)
	{	message("Illegal device code, atrbt_print aborted.");
		return(1);
	}
	if(dev == 2   &&  !(*file)) 
	{	message("Illegal file name, atrbt_print aborted.");
		return(1);
	}

		/************** create & open file **************************/
	if(dev < 2)
	{	tmpnam( tmp_name );
		sprintf(file,"%s",tmp_name);
	}
	if(creatf( file ) == -1) 
	{	sprintf(string, "  Cannot create file %s, atrbt_print aborted.", file);
		message(string);
		return(1);
	}

	if((fd = openf( file, WRITE )) == -1) 
	{	sprintf(string, "  Cannot open file %s, atrbt_print aborted.", file);
		message(string);
		return(1);
	}

		/************** read object records & write file ************/
	std_header(string);       /*** get a standard header line ***/
	if(writea(string,fd))
		return(1);
	sprintf(string,"Attribute Print.\n\n");
	if(writea(string,fd))
		return(1);
	if(set_num)
	{	ptr = (char *)  (level_names[set_num]);
		sprintf(string,"\nSet printed (%d):  %s\n\n",set_num,ptr);
		if(writea(string,fd))
			return(1);
	}
	/*				         1         2         3         4            */
    /*              12345678901234567890123456789012345678901234567890  */
	sprintf(string,"obj num    serial   obj type   attribute\n");
	if(writea(string,fd))
		return(1);
	for(i=0, j=0; i<job_stats[10]; i++)
	{		/*** get object record ***/
		if(get_objects(&object,i))
			return(1);

			/*** see if it is in the correct set ***/
		ok = 0;
		if(set_num)   /** set ok = 1 if it is in the set **/
			ok = object.i12[4] & (1<<set_num);
		else         /** all objects (in in map type or have attribute) ***/
		{	if(object.i12[0] < 8)       /*** a node ***/
				ok = (map_type & 1);
			else if(object.i12[0] < 14)  /*** a segment **/
				ok = (map_type & 2);
			else if(object.i12[0] != 15)  /*** a polygon, (not exterior) **/
				ok = (map_type & 4);
			if(object.i12[2])            /** object has an attribute **/
				ok = 1;
		}

		if(ok)
		{		/*** print the object & attributes ****/
				/** set the object type string **/
			if(object.i12[0] < 8)
				sprintf(type,"node");
			else if(object.i12[0] < 14)
				sprintf(type,"segment");
			else if(object.i12[0] == 14)
				sprintf(type,"polygon");
			else if(object.i12[0] == 15)
				sprintf(type,"ext ply");
			else if(object.i12[0] == 16)
				sprintf(type,"map bdr");
			else
				type[0] = 0;
					
				/*** set the attribute string ***/
			if(object.i12[2])
			{	if(get_attr(&attribute,object.i12[2]))
					return(1);
			}
			else
			{	attribute.string[0] = 0;
				attribute.next = 0;
			}
				
				/*** write string into file **/
			if(!(j%5))
			{	if(writea("\n",fd))
					return(1);
			}
			sprintf(string,"%7d%10d%11s = %s\n",i,object.i12[1],type,
					attribute.string);
			if(writea(string,fd))
				return(1);
			j++;

				/*** write all subsiquent attributes of object in file **/
			for(; attribute.next; )
			{	if(get_attr(&attribute,attribute.next))
					return(1);
				sprintf(string,"                               %s\n",
						attribute.string);
				if(!(j%5))
				{	if(writea("\n",fd))
						return(1);
				}
				if(writea(string,fd))
					return(1);
				j++;
			}
		}
	}

	if(set_num  &&  !j)
	{	sprintf(string," ------- no objects in set -------\n");
		if(writea(string,fd))
			return(1);
	}
	

		/************** close file **********************************/
	closef(fd);

		/************** display or print file if necessary **********/
	if(dev == 0)   /** display on screen **/
		more(file);
	else if(dev == 1)   /** print file **/
		print_file(file);
	else             /*** print to file **/
	{	sprintf(string,"file  %s  of map attributes has been written.",file);
		message(string);
	}

		/************** return **************************************/
	return(0);
}
