/*******************************************************************************
*   atrbt_thin      (see manual, Command Ref., User Functions)  <fc>           *
*           Description:  Removes attributes in a variety of ways according    *
*			to the 'action' parameter.                                         *
*                                                                              *
*			Parameters:  long action;   attribute thinning action code:        *
*                                       -2 = remove all duplicate attributes   *
*                                       -1 = remove ALL attributes             *
*                                        0 = remove all except last attribute  *
*                                        1 - n = remove all except nth attrbt. *
*                        long s;        set number                             *
*                                                                              *
*           Globals/files changed:  job_stats[13], attributes & objects files. *
*                                                                              *
*           Returns zero upon normal end-of-job (EOJ).                         *
*                   J.Dabritz,   1/3/89                                        *
*******************************************************************************/

#include <string.h>
#include "../../incl/globals.h"
#include "../../incl/in.h"


long atrbt_thin( action, s )
	long action;
	long s;
{
	OBJECTS object;
	ATTRIBUTES atrbt;

	char string[256];
	char list[1000][32];

	long new_atrbts;
	long i,j,n;
	long otype;
	long thin;
	long atr_no;


		/**************** check for map ********************************/
	if(!job_stats[0])
	{	message("You must CREATE or ACQUIRE a map befor thinning attributes.");
		return(1);
	}

		/**************** check for assembly/objects *******************/
	if(!job_stats[10])
	{	message("You must ASSEMBLE the map befor thinning attributes.");
		return(1);
	}

		/**************** check for attributes *************************/
	if(job_stats[13] < 2)
	{	message("You must assign attributes befor thinning them.");
		return(1);
	}



		/**************** check all objects & thin attributes ***/
	new_atrbts = 0;
	for(i=0; i<job_stats[10]; i++)
	{		/**************** get object *******************************/
		if(get_objects(&object,i))
		{	message("aborting atrbt_thin, location 1.");
			return(1);
		}

			/**************** inform user ******************************/
		if(!(i % 100))
		{	sprintf(string,"checking objects for attribute thinning, object %d.",i);
			message(string);
		}

			/**************** check set ********************************/
		if(s == 0)        /** all map type objects ***/
		{	if(object.i12[0] < 8)        /** node **/
				otype = 1;
			else if(object.i12[0] < 14)  /** segment **/
				otype = 2;
			else
				otype = 4;
			thin = otype & map_type;
		}
		else
			thin = object.i12[4] & (1 << s);

		if(thin && object.i12[2])
		{	/**************** read through all attributes thinning */
			atrbt.string[0] = 0;
			atrbt.next = object.i12[2];
			if(action > 0)          /***  save nth attribute ***/
			{	for(n=0;  atrbt.next  &&  n != action;  n++)
				{	if(get_attr(&atrbt,atrbt.next))
					{	message("aborting atrbt_thin, location 2.");
						return(1);
					}
				}
				if(n < action)
					atrbt.string[0] = 0;
			}
			else if(action == 0)    /*** save last attribute ***/
			{	for(;  atrbt.next ;)
				{	if(get_attr(&atrbt,atrbt.next))
					{	message("aborting atrbt_thin, location 2.");
						return(1);
					}
				}
			}
			else if(action == -1)   /*** remove all attributes for object **/
				atrbt.string[0] = 0;
			else if(action == -2)   /*** remove redundant attributes *******/
			{	atr_no = 0;
				for(;  atrbt.next ;)
				{	if(get_attr(&atrbt,atrbt.next))
					{	message("aborting atrbt_thin, location 2.");
						return(1);
					}
						/**** check against previous attributes ****/
					for(j=0; j<atr_no  &&  j<1000 ; j++)
					{	if(!strncmp(list[j],atrbt.string,30))
							break;
					}
						/**** store if not a duplicate ***/
					if(atr_no == j)     /***** made it through list *****/
					{		/***** add to list *****/
						strncpy(list[atr_no],atrbt.string,30);
						
							/***** store attribute record ***/
						if(atr_no == 0)    /*** first attribute ***/
						{	if(asgn_atrbt(i,0,list[atr_no]))
							{	message("aborting atrbt_thin, location 3.");
								return(1);
							}
							new_atrbts++;
						}
						else if(atr_no == j)
						{	if(asgn_atrbt(i,6,list[atr_no]))
							{	message("aborting atrbt_thin, location 3.");
								return(1);
							}
							new_atrbts++;
						}
					
							/***** increment atr_no *******/
						atr_no++;
					}
				}
			}


			/**************** store attribute if there are changes *****/
			if(action != -2)
			{	if(atrbt.string[0] == 0)
				{	if(asgn_atrbt(i,7,atrbt.string))
					{	message("aborting atrbt_thin, location 3.");
						return(1);
					}
				}
				else
				{	if(asgn_atrbt(i,0,atrbt.string))
					{	message("aborting atrbt_thin, location 3.");
						return(1);
					}
				}
				new_atrbts++;
			}
		}
	}

		/**************** pack attributes ******************************/
	if(new_atrbts)
	{	if(pack_atrbts())
		{	message("aborting atrbt_thin, location 4.");
			return(1);
		}
	}
	

		/**************** return ***************************************/
	return(0);
}
