/*******************************************************************************
*	b_edit		(see manual, Command Ref., User Function)	<fc>   *
*			Batch edits ASCII files by using user specified        *
*			parameters.                                            *
*      	                                                                       *
*			Designed for use as a pre-processor for importing ASCII*
*			files.                                                 *
*                                                                              *
*			Note:  The fixed number of initial lines (b) are       *
*			deleted before any lines are checked for length (l).   *
*                                                                              *
*			Changes only the ASCII file edited.                    *
*******************************************************************************/
/* Modified 7/92 by DBS@CWU for multi user support */

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <sys/stat.h>
#include <errno.h>
#include <string.h>

long b_edit(ifl,ofl,cflag,eflag,a,b,l,c,ichar,dchar,jchar,f,s,t,g,replchr)
	char *ifl;   /*  file name of input file (default path = import dir) **/
	char *ofl;   /*  file name of output file (default path = import dir) */
	long cflag;  /*  flag, 1=delete all control characters (0-31)         */
	long eflag;  /*  flag, 1=delete all extension characters (128+)       */
	long a;      /*  Number of chars at which to add LF chars             */
	long b;      /*  Number of lines to delete from the beginning of file */
	long l;	    /*  Minimum line length accepted from input file         */
	long c;      /*  Number of characters to delete from each line (begin)*/
	unsigned char *ichar; /*  All characters to be deleted wherever found */
	unsigned char *dchar; /* All columns to be deleted */
	unsigned char *jchar; /*  all columns to be added.  */
	long f;      /*  Number of chars to fill to (with blanks)             */
	long s;      /*  Look S chars preceding to replace spaces (blanks)    */
	long t;      /*  Trim lines at T chars (if that long)                 */
	long g;      /*  Number of lines to group into each single line output*/
	unsigned char *replchr; /* Replacement characters for all characters 0-255 */
{
	unsigned char string[2048];
	unsigned char last_string[2048];
	char tmp[L_tmpnam];
	unsigned char *ptr, *ptr2, *ptr3;
	char ifactual[256], ofactual[256],ofl2[256];
	unsigned char readblock[2048], *readptr, *endptr;
	unsigned char ichars[256], dchars[256], jchars[256];

	int ifd, ofd;

	long i,n;
	long j,jmax; 
	long tmpjs;
	long lines_input, lines_output;
	long chars_input, chars_output;
	long gcount;
	long ignore_lf0;
	long done;
	long linelength, last_linelength;
	long col;
	long write_lf;
	long rcount;


		/************* check input parameters *************************/
	if(c > 2000)
	{	message("b_edit cannot delete more than 2000 characters from each line, aborting.");
		return(1);
	}

		/** check for f > 2000 **/
	if(f > 2000)
	{	message("b_edit cannot fill to more than 2000 characters, aborting.");
		return(1);
	}

		/************* build ordered lists of d,i,j values ************/
	for(i=0; i<256; i++)     /** init to 0 **/
	{	dchars[i] = (unsigned char) 0;
		ichars[i] = (unsigned char) 0;
		jchars[i] = (unsigned char) 0;
	}
	for(ptr=dchar; *ptr; ptr++)
		dchars[*ptr] = 1;
	for(ptr=ichar; *ptr; ptr++)
		ichars[*ptr] = 1;
	for(ptr=jchar, j=0, jmax = 0; *ptr; ptr++)
	{	i = (*ptr) - 1;
		if(!jchars[i])
		{	jchars[i] = 1;
			jmax = (*ptr > jmax) ?  *ptr : jmax;  /** jmax is 1 based **/
			j++;
		}
	}

		/************* count replacement characters *******************/
	for(rcount = 0, i=0; i<256; i++)
	{	if(replchr[i] != i)
			rcount++;
	}

		/************* get actual input filename **********************/
	message("b_edit, setting up files.");

		/************* find & asgn pointer to import file **************/
	tmpjs = job_stats[0];
	job_stats[0] = 1;
	if(get_impfile(ifl,ifactual))
	{	message("b_edit, abort, location 1.");
		job_stats[0] = tmpjs;
		return(1);
	}
	job_stats[0] = tmpjs;

	ifd = openf(ifactual,READ);
	if(ifd <= 0)
	{	message("Cannot open input file, abort, location 2.");
		return(1);
	}

		/************* check for actual output file name **************/
		/** check for null name (same as import) **/
	if(!(ofl[0]))
		strcpy(ofl2,ifl);
	else	
		strcpy(ofl2,ofl);
	trim(ofl2);

		/** check for DOS name (write to temp) **/
	if(strchr(ofl2,':') != ((char *) 0))
	{	message("b_edit will not output to a DOS device.");
		closef(ifd);
		return(1);
	}
	else
	{		/** check for no path  (assume import directory path) **/
		if(ofl2[0] != '/')
		{	trim(config.import_path);
			if(*config.import_path == (char) 0)
#ifdef MULTIUSER
				sprintf(config.import_path,"%s/.ltp/import",home_path); /*default value */
#else
				sprintf(config.import_path,"%s/import",ltp_path); /*default value */
#endif
			if(config.import_path[1] == ':')
			{	message("b_edit will not output to a DOS device.");
				closef(ifd);
				return(1);
			}
			else
				sprintf(ofactual,"%s/%s",config.import_path,ofl2);
		}
		else
			strcpy(ofactual,ofl2);
	}

		/** check for output == input **/
	if(!strcmp(ofactual,ifactual))
		tmpnam(tmp);
	else
		tmp[0] = (char) 0;

		/** create output file **/
	if(tmp[0])
		creatf(tmp);
	else
		creatf(ofactual);

		/** open output file **/
	if(tmp[0])
		ofd = openf(tmp,WRITE);
	else
		ofd = openf(ofactual,WRITE);

	if(ofd <= 0)
	{	message("Cannot open output file. aborting.");
		closef(ifd);
		return(1);
	}

		/************* Initialize counters ****************************/
	ptr = string;
	lines_input = lines_output = 0;
	chars_input = chars_output = 0;
	gcount = 0;
	endptr = string;
	readptr = endptr;
	ignore_lf0 = 0;      /** reset **/
	done = 0;
	last_string[0] = (unsigned char) 0;
	last_linelength = 0;

	message("b_editing file . . . ");
		/************* read all lines of input file & process *********/
	for(; !done ;)
	{	
/* 1. One line is read from the input file.  Control (-c) & extention (-e)    */
/*    characters are deleted if specified.  If a linefeed is not encountered  */
/*    first and there is a specified value (a=), a linefeed is inserted at the*/
/*    specified number of characters (not counting the characters deleted by  */
/*    -c & -e).  b_edit will not do line length processing of lines longer    */
/*    than 2000 characters, although other processing may be done.            */

		write_lf = 0;
		for(ptr=string, linelength=0, readptr++;;readptr++)
		{		/*  0.  read more data if necessary */
			if(readptr >= endptr)
			{	readptr = readblock;
				endptr = readptr + readb(readptr,2000,1,ifd);
				if(endptr <= readptr)
				{	done=1;
					break;
				}
			}

			chars_input++;

				/*** check for cntl ***/
			if((*readptr > 31) || (!cflag) || (*readptr == (unsigned char) 10))
			{		/*** check for extn ***/
				if((*readptr < 128) || (!eflag))
				{		/*  check for ignore_lf0 */
					if((ignore_lf0) && (!linelength) && (*ptr == (unsigned char) 10))
					{	lines_input++;
						ignore_lf0 = 0;
					}
					else
					{	ignore_lf0 = 0;      /** reset **/
							/*  insert character into *ptr  */
						*ptr = *readptr;
						if(*ptr == (unsigned char) 10)
						{	*ptr = (char) 0;
							lines_input++;
							write_lf = 1;
							break;
						}
	
						linelength++;
						ptr++;
	
						if(linelength == a)
						{	*ptr = (char) 0;
							ignore_lf0 = 1;
							write_lf = 1;
							break;
						}
						else if(linelength == 2000)
						{	*ptr = (char) 0;
							break;
						}
					}
				}
			}
		}
		ptr = string;

		if(done && !linelength)
			break;

/* 2. If there is a specified number of lines to delete from the top of the   */
/*    file (b=), then the line number of the read line is checked & the line  */
/*    is omitted from output if it is within the specified count.  If so,     */
/*    return to step 1 & read next line.                                      */

		if(b)
			b--;
		else
		{

/* 3. If a minimum line length is specified (l=), then the length of the line */
/*    is checked.  If it is less than the specified length, then it is        */
/*    disreguarded.  If so, Return to step 1 & read next line.                */

			if((linelength >= l) || !l)
			{
			
/* 4. If there is a specification to delete a number of characters from the   */
/*    beginning of each line (c=), then that number of characters is deleted  */
/*    from the line.  If there are fewer characters than specified under      */
/*    c=, then all characters of the line are deleted, but processing         */
/*    continues.                                                              */

				if(c)
				{	if(c < linelength)
					{	ptr += c;
						linelength -= c;
					}
					else
					{	ptr += linelength;
						linelength = 0;
					}
				}

/* 5. The line is scanned for occurrances of specified ascii characters to be */
/*    deleted (i=), if specified.  They are deleted if present.               */
		
				if(*ichar)
				{	for(ptr2=ptr, i=0; *ptr2; *ptr2++)
					{	if(ichars[*ptr2])     /** char to delete **/
							i++;
						else if(i)
							*(ptr2 - i) = *ptr2;
					}
					if(i)
					{	*(ptr2 - i) = (unsigned char) 0;
						linelength -= i;
					}
				}

/* 6. If column delete(s) are specified (d=) then those columns are deleted   */
/*    (if they exist) beginning with the column furthest to the RIGHT.        */
		
				if(*dchar)
				{	for(ptr2=ptr, i=0, col=1; *ptr2; *ptr2++, col++)
					{	if(dchars[col])     /** col to delete **/
							i++;
						else if(i)
							*(ptr2 - i) = *ptr2;
					}
					*(ptr2 - i) = (unsigned char) 0;
					linelength -= i;
				}

/* 7. If column insertion(s) are spedified (j=) then those columns are added  */
/*    beginning with the column furthest to the RIGHT.  If the string does    */
/*    not have enough columns, then it is filled with blanks up to the maximum*/
/*    column specified under j=.                                              */
		
				if(j)
				{		/** ensure adequate length **/
					if(jmax > linelength)
					{	i = jmax - linelength;
						for(ptr2 = ptr + linelength; i ; i--, linelength++,ptr2++)
							*ptr2 = ' ';
						*ptr2 = (unsigned char) 0;
					}
						/** add columns specified **/
					ptr2 = ptr + linelength + j;
					i = j;
					col = linelength + j;
					for( ; i && (col >= 0); ptr2--, col--)
					{
						*ptr2 = *(ptr2 - i);
						if(col < 256)
						{	if(jchars[col-i])  /** col to add **/
							{	ptr2--;         /** decrement pointer **/
								*ptr2 = ' ';    /** add column (blank) **/
								i--;            /** decrement column added counter **/
								col--;          /** decrement column counter **/
								linelength++;   /** increment line length **/
							}
						}
					}
				}

/* 8. If fill to a given length is specified (f=) then the line is filled     */
/*    to that length if it is shorter than specified.  If longer, then it     */
/*    is left alone.                                                          */

				if(f > linelength)
				{	for(ptr2 = ptr + linelength; f > linelength; ptr2++, linelength++)
						*ptr2 = ' ';
					*ptr2 = (unsigned char) 0;
				}

/* 9. If blanks are to be replaced by a preceding character (s=) then they    */
/*    are replaced by the specified preceding character.  If the specified    */
/*    preceding character is on the previous line, then it is used.  If there */
/*    is no preceding line, then the blank(s) are left.                       */
	
				if(s)
				{	ptr2 = ptr;
					for(i = 0; *ptr2; i++, ptr2++)
					{	if(*ptr2 == ' ')
						{	if(i >= s)
								*ptr2 = *(ptr2 - s);
							else
							{	n = s - i;
								if(last_linelength >= n)
								{	if(gcount  ||  !g) 
										*ptr2 = *(last_string + last_linelength - n);
								}
							}	
						}
					}
				}

/*10. If line truncation is specified (t=) then the line is truncated at that */
/*    length if it is longer.  Shorter lines are left alone.                  */

				if(t > 0)
				{	if(t < linelength)
					{	linelength = t;
						*(ptr + t) = (unsigned char) 0;
					}
				}

/*11. The grouping (g) parameter is checked and a linefeed character is added */
/*    to the end of the line if no grouping is specified, or if the specified */
/*    number of lines has been written according to the grouping parameter.   */


				if(!ichars[10])
				{	if(g) 
					{	if(write_lf)
						{	gcount = (gcount + 1) % g;
							if(!(gcount))
							{	*(ptr + linelength) = (unsigned char) 10;
								linelength++;
							}
							else
								write_lf = 0;
						}
					}
					else
					{	if(write_lf)
						{	*(ptr + linelength) = (unsigned char) 10;
							linelength++;
						}
					}
				}

/*12. All character replacement (xx=) is done.  Any number of characters      */
/*    (control, displayable, or extention) may be replaced for any other      */
/*    ASCII character in the range of 0 to 255.                               */
				if(rcount)
				{		/* check all characters for replacement */
					ptr3 = ptr + linelength;
					for(ptr2=ptr; ptr2 < ptr3; ptr2++)
						*ptr2 = replchr[(unsigned char) *ptr2];
				}

/*13. The line is written to the output file.                                 */
				if(linelength)
				{	if(writeb(ptr,linelength,1,ofd) < 0)
					{	message("Cannot write to output file.  aborting.");
						closef(ifd);
						closef(ofd);
						return(1);
					}
					sprintf(last_string,"%s",ptr);
					last_linelength = linelength;
				}

					/*** add to lines_output & chars_output count ***/
				ptr3 = ptr + linelength;
				for(ptr2=ptr; ptr2 < ptr3; ptr2++)
				{	if(*ptr2 == (unsigned char) 10)
						lines_output++;
				}
				chars_output += linelength;

				if((!(chars_output % 20000) && !lines_output)  ||  
					(lines_output && !(lines_output % 250)))
				{	sprintf(string,"Lines: in=%d, out=%d,  Chars: in=%d, out=%d",lines_input,lines_output,chars_input,chars_output);
					message(string);
					if(abort_cmd())
					{	message("b_edit aborted.");
						sleep(2);
						closef(ifd);
						closef(ofd);
						if(strstr(ifactual,ifl) == (char *) 0)
							purgef(ifactual);
						if(!strcmp(ifactual,ofactual))
							purgef(tmp);
						else
							purgef(ofactual);
						return(1);
					}
				}

/*14. Go to step 1.                                                           */

			}
		}
	}


	sprintf(string,"Lines: in=%d, out=%d,  Chars: in=%d, out=%d",lines_input,lines_output,chars_input,chars_output);
	message(string);
	message("closing & copying files . . . ");


		/************* close files ************************************/
	closef(ifd);
	closef(ofd);

		/************* purge temp input file **************************/
	if(strstr(ifactual,ifl) == (char *) 0)
		purgef(ifactual);

		/************* purge actual input file if same as output ******/
	if(!strcmp(ifactual,ofactual))
	{	purgef(ifactual);
			/************* copy output file if same as input **************/
		sprintf(string,"cp %s %s",tmp,ofactual);
		if(system_ltp(string))
		{	message("copy of tmp file into output file failed.");
			sprintf(string,"output file is at:  %s",tmp);
			message(string);
		}
	}

	append_msg("done.");

		/************* return *****************************************/

	return(0);
}
