/*******************************************************************************
*	continue    see manual, Command Ref., User Function)	<fc>   *
*			Continue an execution of the current graphics input    *
*			mode at the current x,y position.                      *
*                                                                              *
*			Sets first_point global to 0,                          *
*			Updates last_raster_x,y globals.,                      *
*		                                                               *
*		    Returns 1 on error, 0 on ok.                               *
*		                                                               *
*		    J.Dabritz  6/26/89                                         *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long continue_ltp(x,y)
	long x,y;  /*  raster coordinates at which to continue     */
{
	long word, bit;
	long dx,dy;
	long on;
	long i;
	long locale;
	long x1,x2,x3,y2;
	long ymin,ymax;
	long xmin,xmax;
	long hit;
	long spur[2][2];
	static long move_x[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
	static long move_y[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };

	double slope;


		/******************* check for map loaded **********************/
	if(!job_stats[0])
		return(1);

		/***************** make sure x & y are on raster ***************/
	if(x < 0)
		x = 0;
	else if(x > job_stats[1])
		x = job_stats[1];

	if(y < 0)
		y = 0;
	else if(y > job_stats[2])
		y = job_stats[2];

		/***************** take action according to mode *************/
		/*  set 'on' variable for modes 4,5,6,7,14,16  */
	if((input_mode < 8) || (input_mode == 16) || (input_mode == 14))
		on = 1;
	else
		on = 0;

	switch(input_mode)  {
 	case 0:          /* none                    */
		break;
 	case 1:          /* attribute contours      */
		message("Continue not used when in atrbt_contur mode.  Use start, increment or decrement.");
		return(1);
 	case 2:          /* attribute digitize      */
	case 34:         /* atrbt_copy              */
	case 35:         /* atrbt_list              */
		atrbt_xy((float) x,(float) y,(float) search_radius,atrbt_otype,atrbt_action);
		break;
 	case 3:         /* connect                 */
		if(!first_point)
			set_undo(3,0,0,last_raster_x,last_raster_y,x,y);
	case 25:        /* recall connect          */
		if(!first_point)
		{	dx = x - last_raster_x;
			dx = (dx > -dx)  ?  dx : -dx;
			dy = y - last_raster_y;
			dy = (dy > -dy)  ?  dy : -dy;
			if(dy > dx)
			{	dy = 0;
				dx = 1;
			}
			else
			{	dy = 1;
				dx = 0;
			}
				/** erase diagonal lines on both edges **/
			line_diag(last_raster_x+dx,last_raster_y+dy,x+dx,y+dy,0);
			line_diag(last_raster_x-dx,last_raster_y-dy,x-dx,y-dy,0);
				/** draw diagonal line separating edges **/
			if(input_mode == 3)
				line_diag(last_raster_x,last_raster_y,x,y,1);
			else
				line_diag(last_raster_x,last_raster_y,x,y,0);

				/** fill in the erased line corners to make them solid **/
			fill_concnrs(last_raster_x,last_raster_y,x,y);

				/** for recall mode, fill in old bits **/
			if(input_mode == 25)
				recall_mode(25,0,0,last_raster_x,last_raster_y,x,y);
		}
		else
		{	cursor_mode = 1;     /* rubber line  */
			first_point = 0;
		}
		anchorx = x;
		anchory = y;
		break;
 	case 4:          /* draw point              */
 	case 8:          /* erase point              */
		set_undo(4,0,0,0,0,x,y);
	case 18:         /* recall point  */
		pixel(x,y,on);
			/** for recall mode, fill in old bits **/
		if(input_mode == 18)
			recall_mode(18,0,0,0,0,x,y);
		break;
 	case 5:          /* draw line               */
 	case 9:          /* erase line              */
		if(!first_point)
			set_undo(input_mode,0,0,last_raster_x,last_raster_y,x,y);
	case 19:         /* recall line             */
		if(!first_point)
		{	if(input_mode == 5)
				line_diag(last_raster_x,last_raster_y,x,y,on);
			else
				line_solid(last_raster_x,last_raster_y,x,y,on);
				/** for recall mode, fill in old bits **/
			if(input_mode == 19)
				recall_mode(19,0,0,last_raster_x,last_raster_y,x,y);
		}
		else
		{	cursor_mode = 1;     /* rubber line  */
			first_point = 0;
		}
		anchorx = x;
		anchory = y;
		break;
 	case 6:          /* draw fat-line           */
 	case 10:         /* erase fat-line          */
		if(!first_point)
			set_undo(input_mode,0,0,last_raster_x,last_raster_y,x,y);
	case 20:         /* recall fat-line         */
		if(!first_point)
		{	dx = x - last_raster_x;
			dx = (dx > -dx)  ?  dx : -dx;
			dy = y - last_raster_y;
			dy = (dy > -dy)  ?  dy : -dy;
			if(dy > dx)
			{	dy = 0;
				dx = 1;
			}
			else
			{	dy = 1;
				dx = 0;
			}
			for(i= -1; i<2; i++)
			{	if(input_mode == 5)
					line_diag(last_raster_x+i*dx,last_raster_y+i*dy,x+i*dx,y+i*dy,on);
				else
					line_solid(last_raster_x+i*dx,last_raster_y+i*dy,x+i*dx,y+i*dy,on);
			}
				/** for recall mode, fill in old bits **/
			if(input_mode == 20)
				recall_mode(20,0,0,last_raster_x,last_raster_y,x,y);
		}
		else
		{	cursor_mode = 1;     /* rubber line  */
			first_point = 0;
		}
		anchorx = x;
		anchory = y;
		break;
 	case 7:          /* draw rectangle          */
		if(!first_point)
			set_undo(7,0,0,anchorx,anchory,x,y);
	case 21:         /* recall rectangle border */
		if(!first_point)
		{	line_diag(anchorx,anchory,anchorx,y,on);
			line_diag(anchorx,y,x,y,on);
			line_diag(x,y,x,anchory,on);
			line_diag(x,anchory,anchorx,anchory,on);
				/** for recall mode, fill in old bits **/
			if(input_mode == 21)
				recall_mode(21,0,0,anchorx,anchory,x,y);
			first_point = 1;
			cursor_mode = 0;  /*  cross hair   */
		}
		else
		{	anchorx = x;
			anchory = y;
			cursor_mode = 2;     /* rubber box  */
			first_point = 0;
		}
		break;
	case 11:          /* erase rectangle        */
		if(!first_point)
			set_undo(11,0,0,anchorx,anchory,x,y);
	case 22:          /* recall rectangle area  */
		if(!first_point)
		{	erase_rect(anchorx,anchory,x,y);
				/** for recall mode, fill in old bits **/
			if(input_mode == 22)
				recall_mode(22,0,0,anchorx,anchory,x,y);
			first_point = 1;
			cursor_mode = 0;  /*  cross hair   */
		}
		else
		{	anchorx = x;
			anchory = y;
			cursor_mode = 2;     /* rubber box  */
			first_point = 0;
		}
		break;
	case 12:          /* erase segment          */
		if(!erase_segment(x,y,&xmin,&ymin,&xmax,&ymax))
			set_undo(11,0,0,xmin,ymin,xmax,ymax);  /** enclosing rectangle **/
		break;
	case 13:          /* query                  */
		query_xy((float) x,(float) y,(float) search_radius,atrbt_otype);
		break;
	case 14:          /* separate               */
		if(!first_point)
			set_undo(14,0,0,last_raster_x,last_raster_y,x,y);
	case 24:          /* recall separate        */
		if(!first_point)
		{	dx = x - last_raster_x;
			dx = (dx > -dx)  ?  dx : -dx;
			dy = y - last_raster_y;
			dy = (dy > -dy)  ?  dy : -dy;
			if(dy > dx)
			{	dy = 0;
				dx = 1;
			}
			else
			{	dy = 1;
				dx = 0;
			}
				/** copy the 8 bits around beginning pt. **/
			if((last_raster_x > 0 ) && (last_raster_x < job_stats[1]-1) &&
			 (last_raster_y > 0 ) && (last_raster_y < job_stats[2]-1))
				locale = locale_xy(last_raster_x,last_raster_y);
			else
				locale = ~0;

				/** draw solid lines on both edges **/
			line_solid(last_raster_x+dx,last_raster_y+dy,x+dx,y+dy,on);
			line_solid(last_raster_x-dx,last_raster_y-dy,x-dx,y-dy,on);
				/** erase solid line separating edges **/
			line_solid(last_raster_x,last_raster_y,x,y,0);
				
				/** re-erase any cardinal pixels around beginning that were
					blank befor **/
			for(i=0; i<8 ; i += 2)
			{	if(!(locale & (1 << i)))
					pixel(last_raster_x+move_x[i], last_raster_y+move_y[i], 0);
			}
				/** for recall mode, fill in old bits **/
			if(input_mode == 24)
				recall_mode(24,0,0,last_raster_x,last_raster_y,x,y);
		}
		else
		{	cursor_mode = 1;     /* rubber line  */
			first_point = 0;
		}
		anchorx = x;
		anchory = y;
		break;
	case 15:          /* wink                   */
		set_undo(15,0,0,0,0,x,y);
		word = x >> 5;
		bit  = (7 & ~x) | (x & 24);
		on = (lraster[y][word] & (1 << bit)) ?  0 : 1;
		pixel(x,y,on);
		break;
	case 16:          /*    draw wedge      */
	case 17:          /*    erase wedge      */
		if(!first_point)
			set_undo(16,anchor2x,anchor2y,anchorx,anchory,x,y);
	case 23:          /*    recall wedge     */
		if(!first_point)
		{	line_diag(anchorx,anchory,x,y,on);
			line_diag(anchor2x,anchor2y,x,y,on);
			line_diag(anchor2x,anchor2y,anchorx,anchory,on);
				/* find y limits of triangle */
			ymin = ymax = y;
			if(anchory > ymax)
				ymax = anchory;
			if(anchor2y > ymax)
				ymax = anchor2y;

			if(anchory < ymin)
				ymin = anchory;
			if(anchor2y < ymin)
				ymin = anchor2y;


				/** erase horizontal lines within triangle from bottom to top */
			for(ymin++;  ymin < ymax; ymin++)
			{		/** find intersection of row with triangle sides **/
				hit = 0;
					/** side anchor to anchor2 **/
				if(((ymin <= anchory) && (ymin >= anchor2y))   ||
				   ((ymin <= anchor2y) && (ymin >= anchory)))
				{	dx = anchor2x - anchorx;
					dy = anchor2y - anchory;
					if(dy)
					{	slope = dx / (double) dy;
						x1 = x2 = anchorx + (ymin - anchory) * slope + 0.5;
						hit = 1;
					}
				}
					/** side anchor to x,y     **/
				if(((ymin <= anchory) && (ymin >= y))   ||
				   ((ymin <= y) && (ymin >= anchory)))
				{	dx = x - anchorx;
					dy = y - anchory;
					if(dy)
					{	slope = dx / (double) dy;
						x3 = anchorx + (ymin - anchory)* slope + 0.5;
						if(!hit)
						{	x1 = x2 = x3;
							hit = 1;
						}
						else
						{	x1 = (x3 < x1) ?   x3 : x1;
							x2 = (x3 > x2) ?   x3 : x2;
						}
					}
				}
					/** side anchor2 to x,y    **/
				if(((ymin <= anchor2y) && (ymin >= y))   ||
				   ((ymin <= y) && (ymin >= anchor2y)))
				{	dx = x - anchor2x;
					dy = y - anchor2y;
					if(dy)
					{	slope = dx / (double) dy;
						x3 = anchor2x + (ymin - anchor2y)* slope + 0.5;
						if(!hit)
						{	x1 = x2 = x3;
							hit = 1;
						}
						else
						{	x1 = (x3 < x1) ?   x3 : x1;
							x2 = (x3 > x2) ?   x3 : x2;
						}
					}
				}
					/** erase row between intersects **/
				if(hit)
				{	for(x3=x1; x3 <= x2; x3++)
						pixel(x3,ymin,on);
				}
				if(abort_cmd())
					break;
			}
				/** for recall mode, fill in old bits **/
			if(input_mode == 23)
				recall_mode(23,anchor2x,anchor2y,anchorx,anchory,x,y);
			anchor2x = x;
			anchor2y = y;
		}
		else
		{	cursor_mode = 3;     /* rubber wedge */
			first_point = 0;
			anchorx = anchor2x = x;
			anchory = anchor2y = y;
		}
		break;
 	case 26:          /* bridge_gap              */
		/*** find if 2 endpoints are within search radius **/
		if(find_gap(x,y,spur,search_radius,1))
		{		/*** draw line between endpoints ***/
			set_undo(5,0,0,spur[0][0],spur[0][1],spur[1][0],spur[1][1]);
			line_diag(spur[0][0],spur[0][1],spur[1][0],spur[1][1],1);
		}
		else 
			return(1);
		break;
	case 27:          /* seg_break                */
		if(first_point)
			return(start_ltp(x,y));
		if(seg_breaker(1,x,y))  /** breaks segment ********************/
		{	message("Break failed.  File may be CORRUPT.  Advise you re-acquire map WITHOUT saving.");
			return(1);            /** first point not found **/
		}
		cursor_mode = 0;   /* cross hair **/
		first_point = 1;   
		cursor(x+1,y);
		cursor(x,y);
		break;
	case 28:            /* draw spur **/
		if(!add_spur(x,y,&x2,&y2))
			set_undo(5,0,0,x2,y2,x,y);
		break;
	case 29:            /* atrbt_line */
			/*** check for first point ***/
		if(first_point)
			return(start_ltp(x,y));
				/*** attribute to node near x,y ***/
		if(line_atrbt(x,y,search_radius,0))
			return(1);
		set_undo(29,0,0,last_raster_x,last_raster_y,x,y);

				/*** reset first point & cursor type ***/
		cursor(x+1,y);
		cursor(x,y);
		break;
	case 30:             /* erase_obj */
		erase_xyobj((float) x,(float) y,(float) search_radius,atrbt_otype);
		cursor_mode = 0;
		break;
	case 31:             /* vertex_move */
		if(first_point)
			return(start_ltp(x,y));
		if(move_vertex(x,y))  /** move vertex ***************************/
			return(1);
		cursor_mode = 0;
		first_point = 1;
		break;
	case 32:             /* level_icopy */
		if(first_point)
			return(start_ltp(x,y));
		if(icopy(x,y))  /** level_icopy ***************************/
			return(1);
		first_point = 1;
		break;
	case 33:             /* set_dig */
		if(dig_set(x,y))
			return(1);
		break;
	}

		/***************** update last_raster_x,y ********************/
	last_raster_x = x;
	last_raster_y = y;

		/***************** return ************************************/
	return(0);
}
