/*******************************************************************************
*	create		(see manual, Command Ref., User Functions)	<fc>   *
*                                                                              *
*			Creates map file 'name' in the default map directory.  *
*			creates all map files.  Sets defualt values required.  *
*           Uses the default header file.                                      *
*                                                                              *
*			Returns 1 if successful, 0 otherwise.                  *
*           J.Dabritz 5/31/89                                                  *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for multi user support */

#include "../../incl/globals.h"
#include <time.h>

char *getlogin();

long create( name )
	char *name;
{
	char diary_rec[512];
	char file[512];
	char filepath[512];
	char nulls[STRLIST*2];
	char path[512];
	char tmp_buf[512];
	char string[512];

	int fd;

	long h;
	long i;
	long iend;
	long w;

	static char creat_files[][30] = { "attributes", "coord_list", "diary", 
		"errors", "header", "history","level_names","index_array","index_list",
		"job_stats", "mini_raster", "objects", "raster", "register", 
		"seg_list", "v_attributes", "v_objects", "v_coordlist", 0 };
	
/*********************** tell the user what is going on ****************/
	message("creating map:  ");
	append_msg(name);
	append_msg(" . . . ");

#ifdef MULTIUSER  /**** may need to create the maps directory ****/
	if (!file_exists(maps_path,"")) {
		if (!makedir(maps_path)) {
			append_msg("  << maps directory cannot be created.");
			return(0);
		}
	}
#endif

/*********************** close existing files & free malloc'd space ****/
	remove_temp();

/*********************** create directory ******************************/
		/*   check amount of disc space free  */
	if(fsfree(maps_path) < 5000000)
	{	append_msg("  ** disc has < 5,000,000 bytes free, abort create **");
		return(0);
	}
	trim(name);
	sprintf(path,"%s/%s",maps_path,name);
	if(makedir(path))
	{	append_msg("  << map directory cannot be created.");
		return(0);
	}
		/*   set global map path name  */ 
	sprintf(map_path, "%s/%s", maps_path, name);


/*********************** create all files ******************************/

	for(i=0 ; creat_files[i][0] ; i++)
	{	sprintf(path, "%s/%s", map_path, creat_files[i]);
		if(creatf(path) < 0)
		{	message("cannot create file:  ");
			append_msg(path);
			message("  ** aborting create **");
			return(0);
		}
	}
	sync();    /*   make sure that all files are on disc befor writing */


/*********************** store initial files required ******************/

/******  Initialize history file to nulls  */
	sprintf(file,"%s/history",map_path);
	if( (fd = openf(file,WRITE)) == -1)
	{	sprintf(tmp_buf, "create():  Cannot open file %s", file);
		message(tmp_buf);
		return(0);
	}
	for(i=0; i< 512; i++)		/* load up array with newlines */
		string[i] = 13;

	if(0 > writeb(string, sizeof(string), 1, fd))
	{ 	sprintf(tmp_buf, "create(): unable to write to %s", file );
		message(tmp_buf);
		closef(fd);
		return(0);
	}
	closef(fd);


/******  Initialize level_names file to nulls  */
	sprintf(level_names[0],"all map-type objects");
	for(i=1; i<25; i++)
		sprintf(level_names[i],"set %d, - empty -",i);
	sprintf(level_names[25],"objs with 1+ atrbt");
	sprintf(level_names[26],"objs with no atrbts");
	sprintf(level_names[27],"objs with 2+ atrbts");
	sprintf(level_names[28],"objs without 2+ atrbts");
	sprintf(level_names[29],"objs: atrbt 1 != atrbt 2");
	sprintf(level_names[30],"sgmnts: rt poly = lt poly");
	sprintf(level_names[31],"universe (all objects)");
	for(i=32; i<96; i++)
		sprintf(level_names[i],"");

	sprintf(file,"%s/level_names",map_path);
	if(put_levelnms(file))
		return(0);
		
		
/******  Initialize register globals to nulls  */
	for(i=0; i<STRLIST; i++)
		nulls[i] = 0;             /*  a string of nulls  */
	sprintf(file,"%s/register",map_path);
	if( (fd = openf(file,WRITE | O_TRUNC)) == -1)
	{	sprintf(tmp_buf, "create():  cannot open file %s", file);
		message(tmp_buf);
		return(0);
	}
	for(i=0; i < (30*4*2); i++)
	{	if(0 > writeb(nulls,8,1,fd))
		{ 	sprintf(tmp_buf, "create(): unable to write to %s", file );
			message(tmp_buf);
			closef(fd);
			return(0);
		}
	}
	closef(fd);

		
/******  Initialize attributes file with a dummy first record. ****/
	for(i=0; i<STRLIST; i++)
		nulls[i] = 0;             /*  a string of nulls  */
	sprintf(file,"%s/attributes",map_path);
	if( (fd = openf(file, WRITE | O_TRUNC)) == -1)
	{ 	sprintf(tmp_buf, "create(): cannot open %s", file );
		message(tmp_buf);
		return(0);
	}
	if(0 > writeb(nulls,36,1,fd))
	{ 	sprintf(tmp_buf, "create(): unable to write to %s", file );
		message(tmp_buf);
		closef(fd);
		return(0);
	}
	closef(fd);
	job_stats[13] = 1;

/******  Initialize diary file with map name and time and date created  */

	sprintf(filepath, "%s/%s", map_path, "diary" );
	if( (fd = openf( filepath, WRITE | O_TRUNC)) == -1 )
	{	sprintf(tmp_buf, "create(): unable to open %s", filepath );
		message(tmp_buf);
		return(0);
	}
	strcpy(map_name,name);
	if( writea( std_header(diary_rec), fd) )
	{	map_name[0] = 0;
		return(0);
	}
	map_name[0] = 0;
	if( writea("  > map created.\n\n",fd))
		return(0);
	closef( fd );

	
/******  get default header file, if any  ***/
#ifdef MULTIUSER
	sprintf(file,"%s/.ltp/config/header",home_path);
	if(!file_exists("",file))
		sprintf(file,"%s/config/header",ltp_path);
#else
	sprintf(file,"%s/config/header",ltp_path);
#endif
	get_header(file,&header,0);

/******  store header file        */
	sprintf(file,"%s/header",map_path);
	put_header(file,&header);


/******  store job_stats file        */
	sprintf(file,"%s/job_stats",map_path);
	if( (fd = openf(file, WRITE | O_TRUNC)) == -1)
	{ 	sprintf(tmp_buf, "create(): cannot open %s", file );
		message(tmp_buf);
		return(0);
	}

	job_stats[0] = ltp_version * 100 + 0.5;       /*  current ltp version **/
	if(0 > writeb((char *)job_stats,400,1,fd))
	{ 	sprintf(tmp_buf, "create(): unable to write to %s", file );
		message(tmp_buf);
		closef(fd);
		job_stats[0] = 0;    /*  leave flag that there is no map loaded. */
		return(0);
	}
	closef(fd);
	job_stats[0] = 0;    /*  leave flag that there is no map loaded. */


/******  store raster file        */
	sprintf(file,"%s/raster",map_path);
	if( (fd = openf(file, WRITE | O_TRUNC)) == -1)
	{ 	sprintf(tmp_buf, "create(): cannot open file %s", file );
		message(tmp_buf);
		return(0);
	}
	iend = 1 + ((job_stats[1]+1) * (job_stats[2]+1) / 8 /STRLIST);
	for(i=0; i <= iend; i++)
	{	if(0 > writeb(nulls,STRLIST,1,fd))
		{ 	sprintf(tmp_buf, "create(): unable to write to %s", file );
			message(tmp_buf);
			closef(fd);
			return(0);
		}
	}
	closef(fd);


/******  store mini_raster file   */
	sprintf(file,"%s/mini_raster",map_path);
	if( (fd = openf(file, WRITE | O_TRUNC)) == -1) 
	{ 	sprintf(tmp_buf, "create(): cannot open file %s", file );
		message(tmp_buf);
		return(0);
	}
	iend = 1 + (400 * 400 / 8 / STRLIST);
	for(i=0; i <= iend; i++)
	{	if(0 > writeb(nulls,STRLIST,1,fd))
		{ 	sprintf(tmp_buf, "create(): unable to write to %s", file );
			message(tmp_buf);
			closef(fd);
			return(0);
		}
	}
	closef(fd);


/******  store index_array file   */
	sprintf(file,"%s/index_array",map_path);
	if( (fd = openf(file, WRITE | O_TRUNC)) == -1)
	{ 	sprintf(tmp_buf, "create(): cannot open file %s", file );
		message(tmp_buf);
		return(0);
	}
	w = (job_stats[1]) / 32 +1;
	h = (job_stats[2]) / 32 +1;
	iend = 1 + (h * w * 4 / STRLIST);
	for(i=0; i <= iend; i++)
	{	if(0 > writeb(nulls,STRLIST,1,fd))
		{ 	sprintf(tmp_buf, "create(): unable to write to %s", file );
			message(tmp_buf);
			closef(fd);
			return(0);
		}
	}
	closef(fd);
	sync();

	append_msg("  done.");

		/********************* acquire map file          ********************/
sleep(1);
	sprintf(path,"%s/%s",maps_path,name);
	if(!acquire(path))
		message("  map created but cannot be acquired.");


	return(1);
}
