/*******************************************************************************
*     crit_exec       (see manual, Command Ref., User Functions)      <fc>     *
*                                                                              *
*         Creates a set of objects by testing all objects against all          *
*         accumulated criteria.  All objects which meet at least 1 criteria    *
*         of each criteria type specified are marked as being in the set.      *
*         Objects which don't are marked as excluded from the set.             *
*         The old set is written over.  After the set is created all criteria  *
*         are cleared.                                                         *
*                                                                              *
*         Parameters:    long set_num;   set number created (1-24)             *
*                        char descr[];   set name/description                  *
*                                                                              *
*         globals changed:  set_criteria[8], critserial[], objects file,       *
*                           level_names[]                                      *
*                                                                              *
*         Returns 0 upon normal execution, else                                *
*         returns -1 upon error.                                               *
*             J.Dabritz  1/16/90                                               *
*******************************************************************************/

#include <string.h>
#include "../../incl/globals.h"
#include "../../incl/in.h"

char *get_dict();
long contlook();


long crit_exec(set_num, descr)
	long set_num;
	char descr[];
{	
	OBJECTS object,segment,polygon,polygon2,island;
	ATTRIBUTES atrbt,atrbt2;

	char string[256];
	char *dictionary;
	char *ptr;

	long i,j;
	long x,y;
	long set_mask;
	long mask;
	long in_set;
	long hit;
	long s,s1,s2;
	long seg,poly;
	long ok1,ok3,ok5,ok9;
	long length;
	long sides;
	long no_chosen;


		/*********** check for map *************************************/
	if(!job_stats[0])
	{	message("You must Create or Acquire a map befor creating sets of objects.");
		return(1);
	}

		/*********** check input parameters ****************************/
	if(set_num<1  ||  set_num>24)
	{	message("The set to be created must be in the range 1-24.");
		return(1);
	}
	if(descr <= (char *) 0)
	{	message("Illegal string pointer.  crit_exec aboring.");
		return(1);
	}
	descr[79] = (char) 0;


		/*********** inform user ***************************************/
	message("Crit_exec, setting up internal variables for creating sets.");


		/*********** calc set mask *************************************/
	set_mask = 1 << set_num;


		/*********** set ? \ \? and \\ values for crit_arange **********/
	for(i=0; i<set_criteria[1]; i++)
	{	setup_crtstr(critarange[i][0]);
		setup_crtstr(critarange[i][1]);
	}

		/*********** convert crit_area values to pixels ****************/
	get_units();
	for(i=0; i<set_criteria[2]; i++)
	{	if(((double) 0.0)  ==  unit_factor[(long) critarea[i][2]][1])
		{	message("Your choice of area units is not available without additional registration.");
			sleep(3);
		}
		else
		{	critarea[i][0] = critarea[i][0] /unit_factor[(long) critarea[i][2]][1];
			critarea[i][1] = critarea[i][1] /unit_factor[(long) critarea[i][2]][1];
		}
	}

		/*********** calc begin and end string for crit_astring ********/
		/*********** set ? \ \? and \\ values for crit_asring **********/
	for(i=0; i<set_criteria[3]; i++)
		setup_crtstr(critastrg[i]);

		/*********** convert crit_length values to pixels **************/
	for(i=0; i<set_criteria[4]; i++)
	{	if((double) 0.0  ==  unit_factor[(long) critlength[i][2]][0])
		{	message("Your choice of length units is not available without additional registration.");
			sleep(3);
		}
		else
		{	critlength[i][0] = critlength[i][0] / unit_factor[(long) critlength[i][2]][0];
			critlength[i][1] = critlength[i][1] / unit_factor[(long) critlength[i][2]][0];
		}
	}

		/*********** get dictionary if needed **************************/
	if(set_criteria[5])
		dictionary = get_dict(header.dictionary);
	else
		dictionary = (char *) 0;


		/*********** get contour_interval if needed ********************/
	for(i=0; i<set_criteria[12]; i++)
	{	if(crittype[i] == 13)
		{	get_cont_int();
			break;
		}
	}


		/*********** check all objects for inclusion in set ************/
	no_chosen = 0;
	for(i=0; i<job_stats[10]; i++)
	{		/*********** inform user ***********************************/
		if(!(i % 100))
		{	sprintf(string,"Crit_exec, object: %d.",i);
			message(string);
		}

			/******** set default flag *********************************/
		in_set = 1;

			/*********** get object ************************************/
		if(get_objects(&object,i))
		{	message("crit_exec abort location #1.");
			sleep(3);
			break;
		}

			/******** check for map type object if type, record, & serial blank */
		if(in_set  && !set_criteria[12])
			in_set = object.i12[4] & 1;

			/******** 0. check crit_adj criteria ***********************/
		if(in_set  && set_criteria[0])
		{		/*** check to see if it is an interior or map_border polygon */
			if((object.i12[0] < 14)  ||  (object.i12[0] == 15))
				in_set = 0;
			else
			{	mask = 0;
				for(j = 0;  j<set_criteria[0]; j++)
					mask |= (1 << critadj[j]);

					/*** check all adjacent polygons     ***/
				s1 = object.i12[5];
				s2 = object.i12[6];

				hit = 0;
				for(s = s1; s<=s2  && !hit; s++)
				{	if(!get_seglist(&seg,s))
					{	if(seg < 0)
							seg = -seg;
						{	if(!get_objects(&segment,seg))
							{	if(segment.i12[7] == i)
									poly = segment.i12[8];
								else
									poly = segment.i12[7];
								if(!get_objects(&polygon,poly))
									hit = polygon.i12[4] & mask;
							}
						}
					}
				}
						/*** check islands, if any ******/
				if(object.i12[8] && !hit)
				{	island.i12[8] = object.i12[8];
					for(;island.i12[8]  && !hit; )
					{	if(!get_objects(&island,island.i12[8]))
						{	s1 = island.i12[5];
							s2 = island.i12[6];
							for(s = s1; s<=s2  && !hit; s++)
							{	if(!get_seglist(&seg,s))
								{	if(seg < 0)
										seg = -seg;
									if(!get_objects(&segment,seg))
									{	if(segment.i12[7] == i)
											poly = segment.i12[8];
										else
											poly = segment.i12[7];
										if(!get_objects(&polygon,poly))
											hit = polygon.i12[4] & mask;
									}
								}
							}
						}
					}
				}

				if(!hit)
					in_set = 0;
			}
		}


			/********    check attribute related criteria **************/
		if(in_set  &&  (set_criteria[1] || set_criteria[3] ||
						set_criteria[5] || set_criteria[9]    ))
		{
			if(set_criteria[1])
				ok1 = 0;
			else
				ok1 = 1;

			if(set_criteria[3])
				ok3 = 0;
			else
				ok3 = 1;

			if(set_criteria[5])
				ok5 = 0;
			else
				ok5 = 1;

			if(set_criteria[9])
				ok9 = 0;
			else
				ok9 = 1;


				/******** get strings out in succession ****************/
			atrbt.next = object.i12[2];
			for(; atrbt.next  &&  (!ok1 || !ok3 || !ok5 || !ok9); )
			{	if(!get_attr(&atrbt,atrbt.next))
				{		
						/******** 1. check crit_arange criteria ********/
					atrbt.string[30] = 0;
					if(!ok1)
					{	for(j=0; (j<set_criteria[1])  &&  !ok1; j++)
						{		/** compair strings               **/
							if((string_match(atrbt.string,critarange[j][0]) >= 0)  &&
							 (string_match(atrbt.string,critarange[j][1]) <= 0))
								ok1 = 1;
						}
					}

						/******** 3. check crit_astring criteria *******/
					if(!ok3)
					{	for(j=0; (j<set_criteria[3])  &&  !ok3; j++)
						{		/** compair strings               **/
							if(!string_match(atrbt.string,critastrg[j]))
								ok3 = 1;
						}
					}

						/******** 5. check crit_newatr criteria ********/
					if(!ok5)
					{	for(ptr = dictionary; *ptr ; ptr += 32)
						{	if(!strcmp(ptr,atrbt.string))
								break;
						}
						if(!(*ptr))    /** no match, must be a new attribute **/
							ok5 = 1;
					}

						/******** 9. check crit_slen criteria **********/
					if(!ok9)
					{	length = strlen(atrbt.string);
						for(j=0; (!ok9)  &&  (j < set_criteria[9]); j++)
						{	if((length>=critslen[j][0]) && (length<=critslen[j][1]))
								ok9 = 1;
						}
					}
				}
				else
					atrbt.next = 0;
			}

			if(ok1  &&  ok3  &&  ok5  &&  ok9)
				in_set = 1;
			else
				in_set = 0;
		}


			/******** 2. check crit_area criteria **********************/
		if(in_set  && set_criteria[2])
		{	in_set = 0;
			if(object.i12[0] >= 14)
			{	for(j=0; j<set_criteria[2]  &&  !in_set; j++)
				{	if((critarea[j][0] <= (double) object.f4[3])  &&
				 	(critarea[j][1] >= (double) object.f4[3]))
						in_set = 1;
				}
			}
		}

			/******** 4. check crit_length criteria ********************/
		if(in_set  && set_criteria[4])
		{	in_set = 0;
			if(object.i12[0] >= 8)
			{	for(j=0; j<set_criteria[4]  &&  !in_set; j++)
				{	if((critlength[j][0] <= (double) object.f4[2])  &&
				 	(critlength[j][1] >= (double) object.f4[2]))
						in_set = 1;
				}
			}
		}

			/******** 6. check crit_psides criteria ********************/
		if(in_set  && set_criteria[6])
		{	in_set = 0;
			if(object.i12[0] >= 14  &&  object.i12[0] != 15)
			{	sides = object.i12[6] - object.i12[5] + 1;
				island.i12[8] = object.i12[8];
				for(; island.i12[8]; )
				{	if(!get_objects(&island,island.i12[8]))
						sides += island.i12[6] - island.i12[5] + 1;
				}
				for(j=0; j<set_criteria[6]  &&  !in_set; j++)
				{	if((sides >= critpsides[j][0])  &&  (sides <= critpsides[j][1]))
						in_set = 1;
				}
			}
		}


			/******** 7. check crit_record criteria ********************/
		if(in_set  && set_criteria[7])
		{	in_set = 0;
			for(j=0; j<set_criteria[7]  &&  !in_set; j++)
			{	if((i >= critrecord[j][0])  &&  (i <= critrecord[j][1]))
					in_set = 1;
			}
		}


			/******** 8. check crit_serial criteria ********************/
		if(in_set  && set_criteria[8])
		{	in_set = 0;
			for(j=0; j<set_criteria[8]  &&  !in_set; j++)
			{	if((object.i12[1] >= critserial[j][0])  &&  
				 (object.i12[1] <= critserial[j][1]))
					in_set = 1;
			}
		}


			/******* 12. check crit_type criteria **********************/
		if(in_set  && set_criteria[12])
		{	in_set = 0;
			for(j=0; j<set_criteria[12]  &&  !in_set; j++)
			{	switch(crittype[j])  {
				case 0:           /***  any node              ***/
					if(object.i12[0] < 8)
						in_set = 1;
					break;
				case 1:           /***  any segment           ***/
					if((object.i12[0] >= 8)  &&  (object.i12[0] <=10))
						in_set = 1;
					break;
				case 2:           /***  any polygon           ***/
					if((object.i12[0] == 14)  ||  (object.i12[0] == 16))
						in_set = 1;
					break;
				case 3:           /***  0 branch node         ***/
				case 4:           /***  1 branch node         ***/
				case 5:           /***  2 branch node         ***/
				case 6:           /***  3 branch node         ***/
				case 7:           /***  4 branch node         ***/
				case 8:           /***  segment w 2 free ends ***/
				case 9:           /***  segment w 1 free end  ***/
				case 10:          /***  segment w 0 free ends ***/
					if(object.i12[0] == crittype[j])
						in_set = 1;
					break;
				case 11:          /***  segment single link to island ***/
					if((object.i12[0] == 10)  &&  (object.i12[7] == object.i12[8]))
						in_set = 1;
					break;
				case 12:          /***  segment between polys with same atrbt ***/
					if((object.i12[0] == 10) && (object.i12[7] != object.i12[8]))
					{	if(!get_objects(&polygon,object.i12[7]))
						{	if(!get_objects(&polygon2,object.i12[8]))
							{	if(polygon.i12[2]  &&  polygon2.i12[2])
								{	atrbt.next = polygon.i12[2];
									for(; atrbt.next  &&  !in_set; )
									{	if(!get_attr(&atrbt,atrbt.next))
										{	atrbt2.next = polygon2.i12[2];
											for(; atrbt2.next  &&  !in_set; )
											{	if(!get_attr(&atrbt2,atrbt2.next))
												{	if(!strcmp(atrbt.string,atrbt2.string))
														in_set = 1;
												}
												else
													atrbt2.next = 0;
											}
										}
										else
											atrbt.next = 0;
									}
								}
							}
						}
					}
					break;
				case 13:  /***  contour seg with neihbors not within 1 cont int **/
					if((object.i12[0] >= 8)  && (object.i12[8] <=10))  /* segment */
					{
						if(check_contur(i) == 1)
							in_set = 1;
					}
					break;
				case 14:          /***  interior polygon      ***/
				case 15:          /***  exterior polygon      ***/
				case 16:          /***  map border polygon    ***/
					if(object.i12[0] == crittype[j])
						in_set = 1;
					break;
				}
			}
		}

			/******* 10. check crit_subel criteria *********************/
		if(in_set  && set_criteria[10])
		{	in_set = 0;
			mask = 0;
			for(j = 0;  j<set_criteria[10]; j++)
				mask |= (1 << critsubel[j]);
			in_set = subelement(i,mask);
		}

			/******* 11. check crit_superel criteria *******************/
		if(in_set  && set_criteria[11])
		{	in_set = 0;
			mask = 0;
			for(j = 0;  j<set_criteria[11]; j++)
				mask |= (1 << critsuperel[j]);
			in_set = superelement(i,mask);
		}


			/******* mark set inclusion or exclusion for object ********/
		if(in_set)
		{	object.i12[4] |= set_mask;      /** raise flag for set **/
			no_chosen++;
		}
		else
			object.i12[4] &= ~set_mask;     /** lower flag for set **/

			/******* store object **************************************/
		if(put_objects(&object,i))
		{	message("crit_exec abort location #1.");
			sleep(3);
			break;
		}

			/*********** inform user ***********************************/
		if(!(i % 100))
		{	sprintf(string,"Crit_exec, checking object %d,  objects chosen = %d.",i,no_chosen);
			message(string);
		}
	}

		/*********** free dictionary space *****************************/
	if(dictionary != (char *) 0)
		free_ltp(dictionary);

		/*********** set level_name to set descriptor ******************/
	trim(descr);
	if(! (*descr))
		sprintf(descr,"   [ unlabeled set ]");
	sprintf(level_names[set_num],"%s",descr);

		/*********** change display to highlight created set only ******/
	disp_hglight = (1 << set_num);
	disp_centroid = 1;

		/*********** re-display map ************************************/
	x = (rwindowxmin + rwindowxmax) / 2;
	y = (rwindowymin + rwindowymax) / 2;
	if(dispgraphics(x,y,resolution_code))
		return(1);

		/*********** reset criteria to none ****************************/
	for(i=0; i<30; i++)
		set_criteria[i] = 0;

		/*********** inform user about no_obj, len/perim, area, label ***/
	report_1set(set_num);

		/*********** return ********************************************/
	return(0);
}


/*******************************************************************************
*    setup_crtstr                                                              *
*        Setup criteria string.  This program interprets an input string       *
*     for *, ?, \\,  \*, \? characters and processes the string as follows:    *
*       1. converts '*' (not \*) to char 254.                                  *
*       2. converts '?' (not \?) to char 255.                                  *
*       3. converts '\?' to '?', '\*' to '*', and '\\' to '\'                  *
*     Returns 0 upon completion.                                               *
*     J.Dabritz    1/16/90                                                     *
*******************************************************************************/


setup_crtstr(string)
	char string[];
{
	char *ptr, *ptr1;

		/******** convert special characters *******************/

	ptr = string;
	ptr1 = string;
	for(; (*ptr);  ptr++, ptr1++)
	{	switch(*ptr) {
		case 92 :             /** '\' convert next character **/
			ptr++;
			*ptr1 = *ptr;
			break;
		case '?':             /** convert to 255         **/
			*ptr1 = (unsigned char) 255;
			break;
		case '*':             /** convert to 254         **/
			*ptr1 = (unsigned char) 254;
			break;
		default:              /** move single character  **/
			*ptr1 = *ptr;
			break;
		}
	}

	*ptr1 = (char) 0;

	return(0);
}



/*******************************************************************************
*    string_match                                                              *
*        This program compairs a test string with a template string.           *
*     In the template an ascii 254 = the wild card character *, and an ascii   *
*     255 = the wild card character ?.                                         *
*                                                                              *
*     Parameters:   char string[];   string to be tested                       *
*                   char template[]; template to test against                  *
*                                                                              *
*     Values returned:  -1 = string < template                                 *
*                        0 = string == template                                *
*                        1 = string > template                                 *
*     J.Dabritz  1/17/90                                                       *
*******************************************************************************/

string_match(string,template)
	unsigned char string[];
	unsigned char template[];
{

	unsigned char string1[STRLIST];
	unsigned char *ptr;
	unsigned char *ptr1, *ptr2, *ptr3;

	long i;
	long length, len;


		/******** get string length and len past template character 254 ***/
	length = strlen(string);
	len = 0;
	for(i=0, ptr = template; *ptr; ptr++, len += i)
	{	if((unsigned char) *ptr == 254)
		{	i = 1;
			len = -1;
		}
	}

		/******** rewrite string accounting for wild card characters ******/
	ptr = template;
	ptr1 = string1;
	ptr2 = string;
	for( ; *ptr2;  ptr++,  ptr1++,  ptr2++)
	{	switch((unsigned char) *ptr) {
		case 254:                /** old '*'      ****/
			*ptr1 = 254;
			ptr3 = string + length - len - 1;
			if((unsigned char *) (ptr3+1) >= ptr2)
				ptr2 = ptr3;
			else
				*(ptr1 + 1) =  (unsigned char) 0;
			break;
		case 255:                /** old '?'      ****/
			*ptr1 = 255;
			break;
		case 0:
			ptr--;
			*ptr1 = *ptr2;
			break;
		default:                 /** old '*'      ****/
			*ptr1 = *ptr2;
			break;
		}
	}

		/******** add a 254, if necessary, to the end of the test string **/
	if((unsigned char) *ptr == (unsigned char) 254)
	{	*ptr1 = (unsigned char) 254;
		ptr1++;
	}

		/******** append a null to end the string *******/
	*ptr1 = (char) 0;

		/******** compair the strings & return **********/
	return(strcmp(string1,template));
}


/*******************************************************************************
*    subelement                                                                *
*         This program tests the input object number for inclusion in a set    *
*     indicated by the mask.   If it is not in the set, then the objects       *
*     to which it belongs are tested.                                          *
*                                                                              *
*     If the object (or the larger objects to which it belongs) are in a set   *
*     included in the mask, then a 1 is returned, otherwise a 0 is returned.   *
*                                                                              *
*     J.Dabritz  1/18/90                                                       *
*******************************************************************************/


subelement(obj,mask)
	long obj,mask;
{
	OBJECTS object;

	long seg;
	long j;


		/********** test the object itself *****************/
	if((obj < 0)  ||  (obj >= job_stats[10]))
		return(0);

	if(!get_objects(&object,obj))
	{	if(object.i12[4] & mask)
			return(1);

		if(object.i12[0] < 8)            /** node     ***/
		{		/******* check segments    ***/
			for(j=6; (object.i12[j])  &&  (j<10)  ; j++)
			{	seg = object.i12[j];
				if(seg < 0)
					seg = -seg;
				if(subelement(seg,mask))
					return(1);
			}
		}
		else if(object.i12[0] < 14)         /** segment  ***/
		{     /******* check polygons    ***/
			if(job_stats[7])
			{	if(subelement(object.i12[7],mask))
					return(1);
				else if(object.i12[7] != object.i12[8])
				{	if(subelement(object.i12[8],mask))
						return(1); 
				}
			}
		}
		else if(object.i12[0]  == 15)    /** exterior polygon  ***/
		{     /******* check parent      ***/
			if(subelement(object.i12[7],mask))
				return(1);
		}
	}

		/********** no hits, return 0 **********************/
	return(0);
}

/*******************************************************************************
*    superelement                                                              *
*         This program tests the input object number for inclusion in a set    *
*     indicated by the mask.   If it is not in the set, then the objects       *
*     which make it up are tested.                                             *
*                                                                              *
*     If the object (or the objects that make it up) are in a set included     *
*     in the mask, then a 1 is returned, otherwise a 0 is returned.            *
*                                                                              *
*     J.Dabritz  1/18/90                                                       *
*******************************************************************************/


superelement(obj,mask)
	long obj,mask;
{
	OBJECTS object;

	long seg;
	long j;


		/********** test the object itself *****************/
	if((obj < 0)  ||  (obj >= job_stats[10]))
		return(0);

	if(!get_objects(&object,obj))
	{	if(object.i12[4] & mask)
			return(1);

		if(object.i12[0] < 8)            /** node     ***/
			return(0);
		else if(object.i12[0] < 14)         /** segment  ***/
		{     /******* check nodes on both ends ***/
			if(superelement(object.i12[9],mask))
				return(1);
			else if(object.i12[9] != object.i12[10])
			{	if(superelement(object.i12[10],mask))
					return(1); 
			}
		}
		else                             /** polygon           ***/
		{     /******* check segments & islands *****/
			for(j=object.i12[5]; j<=object.i12[6]; j++)    /** check segments **/
			{	if(!get_seglist(&seg,j))
				{	if(seg < 0)
						seg = -seg;
					if(superelement(seg,mask))
						return(1);
				}
			}
				/***** check islands ******/
			if(object.i12[0] != 15)
			{	for(; object.i12[8]; )
				{	if(superelement(object.i12[8],mask))
						return(1);
					if(get_objects(&object,object.i12[8]))
						object.i12[8] = 0;
				}
			}
		}
	}

		/********** no hits, return 0 **********************/
	return(0);
}



/*******************************************************************************
*      check_contur                                                            *
*           This program recieves an object number of a segement as an         *
*      argument.  It checks this segment for a numeric first attribute.  If    *
*      if has such an attribute then it calls routines which return the object *
*      numbers of the segments which are adjacent (looking perpendicular to    *
*      the midpoint of the segment) to it.  If either of these adjacent        *
*      segments have numeric attributes which are not within 1 contour         *
*      interval of the origional segment then a  1  is returned.  Otherwise    *
*      a  0 (normal) or  -1 (error) is returned.                               *
*                                                                              *
*           parameters:   long seg;                segment object number       *
*                                                                              *
*           values returned:  1 (possible contour attribure error)             *
*                             0 (no error found)                               *
*                            -1 (program error)                                *
*           J.Dabritz  1/22/90                                                 *
*******************************************************************************/

check_contur(seg)
	long seg;
{
	OBJECTS object;
	ATTRIBUTES atrbt;

	long s;
	long elev, elev2;
	long difference;

	float x0,y0,x1,y1;


		/***************** get segment object **************************/
	if(get_objects(&object,seg))
		return(-1);

		/***************** get attribute *******************************/
	if(!object.i12[2])
		return(0);
	if(get_attr(&atrbt,object.i12[2]))
		return(-1);

		/***************** convert attribute to long *******************/
	elev = -99999999;
	if(!sscanf(atrbt.string,"%ld",&elev))
		return(0);
	if(elev == -99999999)
		return(0);

		/***************** find centerpoint & perpendicular direction **/
	x0 = object.f4[0];
	y0 = object.f4[1];
	if(find_perp(seg,x0,y0,&x1,&y1))
		return(-1);

		/***************** find object number of seg to rt *************/
	s = contlook(seg,x0,y0,x1,y1);
	if(s > 0)
	{	if(get_objects(&object,s))
			return(-1);
	}
		
		/***************** check attribute of segment to right *********/
	if(object.i12[2])
	{	if(get_attr(&atrbt,object.i12[2]))
			return(-1);
	}
	elev2 = -99999999;
	if(sscanf(atrbt.string,"%ld",&elev2))
	{
		if(elev2 != -99999999)
		{	difference = elev - elev2;
			if((difference > contour_interval)  ||  
			 (difference < -contour_interval))
				return(1);

		}
	}

		/***************** find segment to left ************************/
	s = contlook(seg,x0,y0,x0 - (x1 - x0),y0 - (y1 - y0));
	if(s > 0)
	{	if(get_objects(&object,s))
			return(-1);
	}
		
		/***************** check attribute of segment to left **********/
	if(object.i12[2])
	{	if(get_attr(&atrbt,object.i12[2]))
			return(-1);
	}
	elev2 = -99999999;
	if(sscanf(atrbt.string,"%ld",&elev2))
	{
		if(elev2 != -99999999)
		{	difference = elev - elev2;
			if((difference > contour_interval)  ||  
			 (difference < -contour_interval))
				return(1);

		}
	}

		/***************** return **************************************/
	return(0);
}


/*******************************************************************************
*     find_perp                                                                *
*         This program finds a point on a line which is through x0,y0, and     *
*     is perpendicular to the nearest point on segment seg.                    *
*                                                                              *
*     Parameters:   long seg;                segment number                    *
*                   float x,y;               point to go through               *
*                   float *xptr,*yptr;       point to be found                 *
*                                                                              *
*     Value returned:  0=point found & loaded,  1=error                        *
*     J.Dabritz  1/22/90                                                       *
*******************************************************************************/


find_perp(seg,x,y,xptr,yptr)
	long seg;
	float x,y,*xptr,*yptr;
{
	INDEX_LIST index_list;
	COORD_LIST coord_list;

	long row, col;
	long x1,y1,x2,y2;
	long i, v;

	float r;

	double vx, vy;
	double last_vx, last_vy;
	double array[2][3];
	double dx, dy;
	double dist_squared, min_square;


	r = 3.0;

		/*********** find index_array window ****************************/
	x1 = (x - r);          /* x min (pixels) */
	x1 = x1 >> 5;          /* x min (index_array coords) */
	x2 = (x + r);          /* x max (pixels) */
	x2 = x2 >> 5;          /* x max (index_array coords) */
	y1 = (y - r);          /* y min (pixels) */
	y1 = y1 >> 5;          /* y min (index_array coords) */
	y2 = (y + r);          /* y max (pixels) */
	y2 = y2 >> 5;          /* y max (index_array coords) */

		/** test for raster coverage *****************************/
	if((x2 < 0)  || (x1 > (job_stats[1] >> 5)))
		return(1);
	if((y2 < 0)  || (y1 > (job_stats[2] >> 5)))
		return(1);

		/** limit to range of index_array ***/
	if(x1 < 0) 
		x1 = 0;
	if(y1 < 0)
		y1 = 0;
	if(x2 > (job_stats[1] >> 5))
		x2 = (job_stats[1] >> 5);
	if(y2 > (job_stats[2] >> 5))
		y2 = (job_stats[2] >> 5);
	
		/*********** search for segment within r dist **********************/
		/** for each row & col of index array, search objects */
	min_square = 25.0;                      /** compair squares to save time **/
	for(row = y1; row <= y2; row++)
	{	for(col = x1; col <= x2; col++)
		{		/** search all indicated objects **/
			index_list.next = index_array[row][col];
			for(; index_list.next;)
			{		/** get index_list record **/
				if(get_indexl(&index_list,index_list.next))
					return(1);
				  /*** check for object type == segment ***/
				if(index_list.obj_rec == seg)
				{		/**** test every vertex & side for distance **/
					i = (index_list.v1 < index_list.v2)  ?  1 : -1;
					for(v = index_list.v1;      ; v += i)
					{		/*** get vertex coords ***/
						if(get_coordlst(&coord_list,v))
							return(1);

						vx = coord_list.x;
						vy = coord_list.y;
				
							/*** calculate distance to input point **/
						dx = x - vx;
						dy = y - vy;
						dist_squared = (dx * dx) + (dy * dy);
							
							/*** check against min dist ***/
						if(dist_squared < min_square)
						{	if(dist_squared > 0.001)    /** use point directly **/
							{	*xptr = vx;
								*yptr = vy;
								min_square = dist_squared;
							}
							else if(v != index_list.v1)  /** if not first point **/
							{	dx = vx - last_vx;
								dy = vy - last_vy;
								*xptr = x + dy;
								*yptr = y - dx;
								min_square = dist_squared;
							}
						}
					
							/*** if not first point, then get dist to vector **/
						if(v != index_list.v1)
						{	dx = vx - last_vx;
							dy = vy - last_vy;
							if((dx != 0.0) || (dy != 0.0))
							{	array[0][0] = dx;
								array[0][1] = dy;
								array[0][2] = x - last_vx;
								array[1][0] = dy;
								array[1][1] = -dx;
								array[1][2] = y - last_vy;
								if(!mat_solve(2,3,array))
								{		/*** see if pt perp. off side of vector **/
									if((array[0][2] > 0.0) && (array[0][2] < 1.0))
									{	dist_squared = array[1][2] * array[1][2] * 
                                     	((dx * dx) + (dy * dy));
											/*** check against min dist ***/
										if(dist_squared < min_square)
										{	*xptr = x + dy;
											*yptr = y - dx;
											min_square = dist_squared;
										}
									}
								}
							}
						}
					
							/*** check to see if verticies done ***/
						if(v == index_list.v2)
							break;
	
							/*** reset last_v coords ***/
						last_vx = vx;
						last_vy = vy;

					}
				}
			}
		}
	}
	
			/*********** return closest segment record number or -1 ************/
	if(min_square < 25.0)
		return(0);
	else
		return(1);
}



/*******************************************************************************
*    contlook                                                                  *
*         This program looks along a ray beginning at x0,y0 in the direction   *
*    of x1,y1 and returns the record number of the first segment that it       *
*    encounters which is not seg.                                              *
*                                                                              *
*    parameters:    long seg;             rec number of segment to ignore      *
*                   float x0,y0;          endpoint of ray                      *
*                   float x1,y1;          point on ray, not endpoint           *
*                                                                              *
*    returned values:   0=none found, >0=segment number, <0 = error            *
*    J.Dabritz  1/22/90                                                        *
*******************************************************************************/

#include <math.h>


long contlook(seg,x0,y0,x1,y1)
	long seg;
	float x0,y0,x1,y1;
{
	INDEX_LIST index_list;
	COORD_LIST coord_list;

	long cell_x, cell_y;
	long xinc,yinc;
	long cell_xinc, cell_yinc;
	long s;
	long v;

	double dx, dy;
	double old_factor, new_factor;
	double x_factor, y_factor;
	double x_above, x_below;
	double y_above, y_below;
	double last_x, last_y;
	double sx1,sy1,sx2,sy2;
	double matrix[2][3];
	double a,b;
	double min_dist;
	double length;
	double f1,f2;



		/************* find beginning cell *******************************/
	cell_x = x0 + 0.5;
	cell_x = cell_x >> 5;
	cell_y = y0 + 0.5;
	cell_y = cell_y >> 5;

	x_below = cell_x * 32.0 - 0.5;
	x_above = x_below + 32.0;
	y_below = cell_y * 32.0 - 0.5;
	y_above = y_below + 32.0;

		/************* find cell x & y increment values ******************/
	dx = x1 - x0;
	if(dx > 0.0)
	{	xinc = 32.0;
		cell_xinc = 1;
	}
	else
	{	xinc = -32.0;
		cell_xinc = -1;
	}

	dy = y1 - y0;
	if(dy > 0.0)
	{	yinc = 32.0;
		cell_yinc = 1;
	}
	else
	{	yinc = -32.0;
		cell_yinc = -1;
	}
	
		/************* find x & y factor & old & new factor **************/
	if((dx == 0.0)  &&  (dy == 0.0))
		return(0);

		/****** (normalize the vector) **/
	length = sqrt((dx * dx) + (dy * dy));
	dx = dx / length;
	dy = dy / length;

	if(dx == 0.0)
	{	old_factor =           -9999999999;
		new_factor  = x_factor =  9999999999;
	}
	else
	{	f1 = (x_above - x0)  /  dx;
		f2 = (x_below - x0)  /  dx;
		if(f1 > f2)
		{	old_factor = f2;
			x_factor = new_factor = f1;
			last_x = x_above;
		}
		else
		{	old_factor = f1;
			new_factor = x_factor = f2;
			last_x = x_below;
		}
	}
		
	if(dy == 0.0)
		y_factor =  9999999999;
	else
	{	f1 = (y_above - y0)  /  dy;
		f2 = (y_below - y0)  /  dy;
		if(f1 > f2)
		{	if(f2 > old_factor)
				old_factor = f2;
			y_factor = f1;
			if(f1 < new_factor)
				new_factor = f1;
			last_y = y_above;
		}
		else
		{	y_factor = f2;
			if(f2 < new_factor)
				new_factor = f2;
			if(f1 > old_factor)
				old_factor = f1;
			last_y = y_below;
		}
	}

		/************* step through all cells checking for seg crossing **/
	s = 0;
	for(;!s;)          /*** check cells until a hit found ****/
	{	index_list.next = index_array[cell_y][cell_x];
		for(; index_list.next ;)
		{		/** get index_list record **/
			if(get_indexl(&index_list,index_list.next))
				return(-1);
	
				/********* check object type for segment *******************/
			if((index_list.type == 1) && (index_list.obj_rec != seg))
			{		/********* go through coord_list checking for intersections */
				if(get_coordlst(&coord_list,index_list.v1))
					return(1);
				sx1 = coord_list.x;
				sy1 = coord_list.y;
				for(v=index_list.v1+1;  v <= index_list.v2; v++)
				{		/********* get vertex ******************************/
					if(get_coordlst(&coord_list,v))
						return(1);
						/********* calc matrix variables *******************/
					sx2 = coord_list.x;
					sy2 = coord_list.y;
					matrix[0][0] = dx;
					matrix[0][1] = sx1 - sx2;
					matrix[0][2] = sx1 - x0;
					matrix[1][0] = dy;
					matrix[1][1] = sy1 - sy2;
					matrix[1][2] = sy1 - y0;
	
						/********* solve for intersection ******************/
					if(!mat_solve(2,3,matrix[0]))
					{		/***** if intersection between old_ & new_factor ***/
							/***** and between verticies and a is positive   ***/
						a = matrix[0][2];
						b = matrix[1][2];
						if((a >= old_factor) && (a <= new_factor) &&
						 (b >= 0.0)  &&  (b <= 1.0)  &&  (a >= 0.0))
						{	/***** a good hit, check dist ******************/
							if((a < min_dist)  ||  !s)
							{	s = index_list.obj_rec;
								min_dist = a;
							}
						}
					}
						/**** increment vector verticies **/
					sx1 = sx2;
					sy1 = sy2;
				}
			}
		}
		old_factor = new_factor;
		if(new_factor == x_factor)
		{	last_x += xinc;
			cell_x += cell_xinc;
			if(((cell_x < 0)  ||  ((cell_x + 1) * 32) > job_stats[1]))
				break;
			x_factor = (last_x - x0)  /  dx;
		}
		else
		{	last_y += yinc;
			cell_y += cell_yinc;
			if(((cell_y < 0)  ||  ((cell_y + 1) * 32) > job_stats[2]))
				break;
			y_factor = (last_y - y0)  /  dy;
		}
		if(x_factor < y_factor)
			new_factor = x_factor;
		else
			new_factor = y_factor;
	}
		/************* return ********************************************/
	return(s);
}
