/*******************************************************************************
*     crit_report     (see manual, Command Ref., User Functions)      <fc>     *
*                                                                              *
*         Generates a report to the designated printer based                   *
*         on the report parameters received.                                   *
*         Parameters:                                                          *
*            long outdev   Display device type: 0=screen, 1=printer, 2=file    *
*            char fname    Name of file report writes to.                      *
*                                                                              *
*         Returns 0 upon normal execution, else                                *
*         returns -1 upon error.                                               *
*             J.Dabritz  1/10/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"


long crit_report(  outdev, fname )
	long outdev;
	char *fname;
{
	char tmp_name[L_tmpnam];
	char actual[256];
	char string[256];

	int fd;

	long i;


		/*********** check for map *************************/
	if(!job_stats[1])
	{	message("Cannot write a set_criteria report unless a map is acquired.");
		return(1);
	}

		/*********** check output device code **************/
	if((outdev < 0)  ||  (outdev > 2))
	{	message("Output device code must be in the range  0 - 2.");
		return(1);
	}

		/*********** check file name************************/
	message("setting up output files and variables.");
	if(outdev == 2)
	{	if(setup_ofile(fname,actual))
		{	exmossabort(11);
			return(1);
		}
	}
	else
	{	tmpnam( tmp_name);
		sprintf(actual,"%s",tmp_name);
	}
		
		/*********** create file ***************************/
	if( creatf( actual ) == -1 )
	{	message("crit_report: Cannot create temp file. Bye.");
		return(1);
	}
	
	if( (fd = openf( actual, RDWR)) == -1 )
	{	message("crit_report: Cannot open temp file. Bye.");
		return(1);
	}
	
		/*********** write to file *************************/
		/***** write header *********/
	if(writea("Crit_report \n",fd))
	{	message("crit_report: Cannot write to temp file. Bye.");
		closef(fd);
		return(1);
	}
	std_header(string);
	if(writea(string, fd))
	{	message("crit_report: Cannot write to temp file. Bye.");
		closef(fd);
		return(1);
	}
	if(writea("\n",fd))
	{	message("crit_report: Cannot write to temp file. Bye.");
		closef(fd);
		return(1);
	}

		/***** 0.  write critadj *********/
	if(set_criteria[0])
	{	for(i=0; i<set_criteria[0]; i++)
		{	sprintf(string,"crit_adj:        polygons adjacent to polygons in set %d\n",critadj[i]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 1.  write arange *********/
	if(set_criteria[1])
	{	for(i=0; i<set_criteria[1]; i++)
		{	sprintf(string,"crit_arange:     atrbts in range:  %s  to  %s\n",critarange[i][0],critarange[i][1]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 2.  write area *********/
	if(set_criteria[2])
	{	for(i=0; i<set_criteria[2]; i++)
		{	sprintf(string,"crit_area:       Area range %.2f  to  %.2f  coord_type %.0f\n",critarea[i][0],critarea[i][1],critarea[i][2]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 3.  write astring *********/
	if(set_criteria[3])
	{	for(i=0; i<set_criteria[3]; i++)
		{	sprintf(string,"crit_astring:    attribute template:  %s\n",critastrg[i]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}


		/***** 4.  write length        *********/
	if(set_criteria[4])
	{	for(i=0; i<set_criteria[4]; i++)
		{	sprintf(string,"crit_length:     Length range %.2f  to  %.2f  coord_type %.0f\n",critlength[i][0],critlength[i][1],critlength[i][2]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 5.  write new attribute *********/
	if(set_criteria[5])
	{	sprintf(string,"crit_newatr:     objects with attribute not in dictionary\n");
		if(writea(string, fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 6.  write poly sides  *********/
	if(set_criteria[6])
	{	for(i=0; i<set_criteria[6]; i++)
		{	sprintf(string,"crit_psides:     polygons with %d  to  %d sides.\n",critpsides[i][0],critpsides[i][1]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 7.  write record *********/
	if(set_criteria[7])
	{	for(i=0; i<set_criteria[7]; i++)
		{	sprintf(string,"crit_record:     objects with record %d to %d.\n",critrecord[i][0],critrecord[i][1]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 8.  write serial *********/
	if(set_criteria[8])
	{	for(i=0; i<set_criteria[8]; i++)
		{	sprintf(string,"crit_serial:     objects with serial %d to %d.\n",critserial[i][0],critserial[i][1]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 9.  write string length *********/
	if(set_criteria[9])
	{	for(i=0; i<set_criteria[9]; i++)
		{	sprintf(string,"crit_slen:       objects with attributes of length %d to %d\n",critslen[i][0],critslen[i][1]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 10.  write sub element *********/
	if(set_criteria[10])
	{	for(i=0; i<set_criteria[10]; i++)
		{	sprintf(string,"crit_subel:      sub elements of objects in set %d\n",critsubel[i]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 11.  write super element *********/
	if(set_criteria[11])
	{	for(i=0; i<set_criteria[11]; i++)
		{	sprintf(string,"crit_subel:      super element of objects in set %d\n",critsuperel[i]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}

		/***** 12.  write object type *********/
	if(set_criteria[12])
	{	for(i=0; i<set_criteria[12]; i++)
		{	sprintf(string,"crit_type:       object must be of type %d.\n",crittype[i]);
			if(writea(string, fd))
			{	message("crit_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
		if(writea("\n", fd))
		{	message("crit_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
	}


		/*********** close file ****************************/
	if( closef(fd))
	{	message("crit_report: Cannot open temp file. Bye.");
		return(1);
	}

		/*********** print or display file if necess *******/
	switch(outdev)
	{
	case 0:				/* Display on CRT */
		more(actual);
		break;
	case 1:				/* Send to printer1 */
		print_file(actual);
		break;
	case 2:				/* send to file     */
		break;
	default:			/* SHOULD NEVER HAPPEN */
		message("crit_report: Darned serious problem occurred. Good luck.");
		return(1);
	}

		/*********** return ********************************/
	return(0);
}
