/*******************************************************************************
*       del_points      (see manual, Command Ref., User Functions)     <fc>    *
*               Description:  Removes all isolated points from the             *
*               raster display.  An isolated point is defined as               *
*               a node with zero branches.  Assembly is not required.          *
*                                                                              *
*               Parameters:  None.                                             *
*                                                                              *
*               Globals changed:  lraster (craster)                            *
*                                                                              *
*               Returns zero upon normal end-of-job (EOJ).                     *
*               J.Dabritz   6/29/89                                            *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long del_points( )
{
	unsigned long *ptr, *end_ptr;
	long bit, word, x, y;
	long count;

	char string[STRLIST];

		/**************** check for loaded map **************************/
	if(!job_stats[0])
		return(0);

		/************* check for VIEW ONLY raster **********************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}

append_msg(" . . erasing points . . ");
		/**************** erase a 1 pixel column on rt edge of raster ***/
		/**************** (to avoid wrapping on raster in locale_ptr() **/
	erase_rect(job_stats[1],0,job_stats[1],job_stats[2]);
	
		/**************** search raster for isolated pixels & destroy ***/
	count = 0;
	end_ptr = lraster[job_stats[2]] + raster_wincrement;
	for(ptr = lraster[0]; ptr < end_ptr; ptr++)
	{	if(*ptr)
		{	for(bit = 0; bit < 32; bit++)
			{	if(*ptr & (1<<bit))
				{	if(!locale_ptr(ptr,bit))
					{	word = (ptr - lraster[0]) % raster_wincrement;
						x = (word << 5) | (7 & ~bit) | (bit & 24);
						y = (ptr - lraster[0]) / raster_wincrement;
						pixel(x,y,0);
						count++;
					}
				}
			}
		}
	}
	sprintf(string," done.   %d points erased.",count);
	append_msg(string);

		/**************** return ****************************************/
	return(0);
}
