/*******************************************************************************
*	del_spurs	(see manual, Command Ref., User Functions)	<fc>   *
*			Description:  Removes all spurs <= specified number of *
*			pixels in length from the raster display.  To save the *
*			changes, execute the save_raster function.  A spur is  *
*			defined as a segment with one free (unconnected) end.  *
*			Assembly is not required.                              *
*			Parameters:  Length of spurs to delete.                *
*									       *
*			Returns zero upon normal end-of-job (EOJ).             *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

static long dirx[] = {0, 1, 1, 1, 0,-1,-1,-1};
static long diry[] = {1, 1, 0,-1,-1,-1, 0, 1};

long del_spurs( length )
	long length;
{
	long count = 0;
	long del_cnt = 0;
	long direction = 0;
	long i = 0;
	long key = 0;
	long mask9, mask13;
	long spur_x, spur_y, spur_dir;
	long word = 0;
	long x = 0, y = 0;
	unsigned long *end_ptr, *ptr;
	static long bit[] = {7, 6, 5, 4, 3, 2, 1, 0, 15, 14, 13, 12, 11, 10, 
		9, 8, 23, 22, 21, 20, 19, 18, 17, 16, 31, 30, 29, 28, 27,
		26, 25, 24};
	char tmp_msg[32];

		/************* check for VIEW ONLY raster **********************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}
/***  Initialize variables  ***/
	mask9  = 1 << 9;
	mask13 = 1 << 13;
	end_ptr = lraster[job_stats[2]] + raster_wincrement;

/***  Erase right hand column of pixels to avoid wrapping ***/
	erase_rect(job_stats[1],0,job_stats[1],job_stats[2]);

/***	Find spurs in raster  ***/
	for( ptr = lraster[0]; ptr < end_ptr; ptr++ )
	{   if( *ptr )
	    {   for( i = 0; i < 32; i++ )
	    	    if(*ptr & (1 << bit[i]) )
		    {   key = tlocale[locale_ptr(ptr, bit[i])];
			if( key & mask9 )	/*** True if spur end ***/
			{
				/**** check for user abort **/
				if(abort_cmd())
					return(1);

			    for(; !(key & (1 << direction));
					direction = (direction + 1) & 7 );
/***  Find coordinates and direction.  ***/ 
			    word = (ptr - lraster[0]) % raster_wincrement;
			    x = (word << 5) | i;
			    y = (ptr - lraster[0]) / raster_wincrement;
/***  Save x and y and direction for deleting selected spurs.  ***/ 
			    spur_x = x;
			    spur_y = y;
			    spur_dir = direction;
			    count = 1;

			    do 
			    {	x += dirx[direction];	/*** Set new x y ***/
				y += diry[direction];
				key = tlocale[locale_xy(x, y)];
			     	count++;
				if( count > length )	/*** Spur too long  ***/
/***  If its a node and ***/	{   if( key & mask13 && !(key & mask9))
/***  NOT a spur end,   ***/		count--;
/***  decrement count   ***/	    break;
				}
				if( key & mask13 )	/*** Node found     ***/
				    break;
/***  Set direction for next pixel look-around  ***/
				direction = (direction + 5) & 7;
			    	for(; !(key & (1 << direction));
				            direction = (direction + 1) & 7 );
			    }
			    while( TRUE );		/*** Infinite loop  ***/

/***   Delete the spur ONLY if it is within allowable length  ***/
			    if( count <= length)
			    {
				x = spur_x;
				y = spur_y;
				direction = spur_dir;
				do
				{   x += dirx[direction];
				    y += diry[direction];
				    key = tlocale[locale_xy(x, y)];
				    if( key & mask13 )	/*** True if node ***/
				    {	pixel(spur_x, spur_y, 0);
					break;
				    }
				    pixel(spur_x, spur_y, 0);
				    spur_x = x;
				    spur_y = y;
/***  Find next adjacent pixel  ***/
				    direction = (direction + 5) & 7;
				    for(; !(key & (1 << direction));
					    direction = (direction + 1) & 7 );
				}
				while( TRUE );
/***  If the next pixel is a node end, turn the pixel off  ***/
			        if( key & mask9 )
				    pixel(x, y, 0);
			 	del_cnt++;
			    }
			}		    /**  No spur  **/
		    }    		/**  Empty bit  **/
	    }    		/**  Empty word  **/
	}		/**  End of for loop  **/
		
	sprintf(tmp_msg, "...done.  %d spurs deleted.", del_cnt);
	append_msg(tmp_msg);

	return(0);
}
