/*******************************************************************************
*    dem_view      (see manual, Command Ref., User Functions)    <fc>          *
*            This program reads a dem file and displays the topography on      *
*       the screen as if side-lit.                                             *
*                                                                              *
*       files/globals changed:  none                                           *
*                                                                              *
*       values returned:  0=file viewed,  1=file not viewed/error              *
*                                                                              *
*       parameters:   char *fname  filename                                    *
*                     long d       light direction code:                       *
*                                     0=nw, 1=ne, 2=se, 3=sw                   *
*                                                                              *
*            J.Dabritz   1/25/90                                               *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for X11 support and big endian */

#include "../../incl/globals.h"
#include "../../incl/in.h"
#ifdef LTP_X11
#include "../../incl/x11.h"
char *malloc_ltp();
#endif

#ifdef BIG_ENDIAN
#include "../../incl/big_end.h"
#endif

#define DE0   -25
#define DE1   -12
#define DE2   -6
#define DE3   -2
#define DE4   0
#define DE5   2
#define DE6   5
#define DE7   9
#define DE8   14
#define DE9   20
#define DE10  28
#define DE11  38

long dem_view( fname, d)
    char *fname;
    long d;
{
	char actual[256];
	unsigned char string[1056];
	unsigned char dem[4][480][BITPLN_WIDTH];
	unsigned char *ptr, *end_ptr;
	unsigned char *cptr[4];
	unsigned char mask;
	char msg[STRLIST];

	long col_index;
	long dem_col[3][480];    /* elevatins for 3 dem columns on the screen */
	long *c, *n, *ne, *e, *se, *s, *sw, *w, *nw;  /** ptrs to adj elevations */
	long *c0, *c1, *c2;  /** ptrs to adjacent elevs from lighting dir */
	long i,j;
	long increment;          /* hight of a dem cell */
	long base_y;             /* number of increment values at bottom of dem **/
	long no_cols;
	long new_col, col;
	long no_elevs;
	long col_base;
	long local_datum;
	long row;
	long elevs;
	long w_col, c_col, e_col;  /** indexes to dem columns **/
	long byte, bit;
	long de;                   /** difference in elevation ***/
	long color;                /** RGBrgb color number **/

	int fd;

	double min_y, y;
	double begin_y;
	double col_datum;
#ifdef LTP_X11
	char *to_ptr;
#endif


		/************* check parameter 'd' *****************************/
	if((d<0) || (d>3))
	{	message("illegal direction code for dem lighting.");
		return(1);
	}
	
		/************* zero out dem array ******************************/
	ptr = &(dem[0][0][0]);
	end_ptr = ptr + 4 * 480 * BITPLN_WIDTH;
	for(; ptr < end_ptr; ptr++)
		*ptr = (char) 0;

		/************* open file ***************************************/
	if(*fname != (char) '/')
		sprintf(actual,"%s/%s",config.export_path,fname);
	else
		sprintf(actual,"%s",fname);

	fd = openf(actual,READ);
	if(fd <= 0)
	{	sprintf(string,"Cannot open file  %s  , aborting");
		message(string);
		return(1);
	}


		/************* read file header ********************************/
	if(readb(string,1024,1,fd) > 0)
	{		/*---------- find south most corner ***********************/
		ptr = string + 570;
		ptr[24] = (unsigned char) 0;
		for(i=0; i<24; i++)
		{	if(ptr[i] == (unsigned char) 'D')
				ptr[i] = (unsigned char) 'E';
		}
		sscanf(ptr,"%lf",&min_y);
		for(j=1, ptr += 48; j<4; j++, ptr += 48)
		{	for(i=0; i<24; i++)
			{	if(ptr[i] == (unsigned char) 'D')
					ptr[i] = (unsigned char) 'E';
			}
			sscanf(ptr,"%lf",&y);
			if(y < min_y)
				min_y = y;
		}

			/*---------- find n increment value ***********************/
		ptr = string + 828;
		ptr[12] = (unsigned char) 0;
		sscanf(ptr,"%lf",&y);
		increment = y;

			/*---------- find base north of dem area ******************/
		base_y = min_y;
		base_y = base_y / increment;

			/*---------- find number of columns ***********************/
		no_cols = 0;
		ptr = string + 858;
		sscanf(ptr,"%ld",&no_cols);
		if(no_cols > 640)
			no_cols = 640;
		
	}

		/************* read columns & display **************************/
	col_index = 0;
	new_col = 1;
	col = 0;
	end_ptr = string + 1024 - 5;
	for( ;(col<no_cols) && (readb(string,1024,1,fd)>0) && (col<(8*BITPLN_WIDTH)); )
	{	ptr = string;
		string[1024] = (unsigned char) 0;
			/*--------- if new column, read column header ********/
		if(new_col)
		{
			if(!(col % 25))
			{	sprintf(msg,"reading dem column %d.",col);
				message(msg);
#ifdef LTP_X11  /***** for X11 we need to flush the outputto see message *****/
				XFlush(xdisplay);
#endif
			}
			ptr = string + 12;
			ptr[6] = (unsigned char) 0;
			if(1 > sscanf(ptr,"%ld",&no_elevs))   /** number of elevs in profile */
				break;
			
			ptr = string + 48;
			ptr[24] = (unsigned char) 0;
			for(i=0; i<24; i++)
			{	if(ptr[i] == (unsigned char) 'D')
					ptr[i] = (unsigned char) 'E';
			}
			if(1 > sscanf(ptr,"%lf",&begin_y))    /** beginning y coord        **/
				break;
			col_base = begin_y;
			col_base = col_base / increment;
			col_base = col_base - base_y;

			ptr = string + 72;
			ptr[24] = (unsigned char) 0;
			col_datum = 0.0;
			sscanf(ptr,"%lf",&col_datum);   /** local datum for profile **/
			local_datum = col_datum;

			row = col_base;
			elevs = 0;
			ptr = string + 144;

				/** 0 out column elevations **/
			for(i=0; i<480; i++)
				dem_col[col_index][i] = 0;
			
			new_col = 0;
		}


			/*--------- read elevations **************************/
		for(; (elevs<no_elevs) && (ptr<end_ptr) ; row++, elevs++, ptr+=6)
		{
			if(row < 480)
			{	sscanf(ptr,"%6ld",&dem_col[col_index][row]);
				dem_col[col_index][row] += local_datum;
			}
		}

			/*--------- if column done, display last column ******/
		if((elevs >= no_elevs)  &&  (col > 1))
		{	w_col = (col_index + 1) % 3;
			c_col = (col_index + 2) % 3;
			e_col = (col_index) % 3;

			sw = &(dem_col[w_col][0]);
			w = sw + 1;
			nw = w + 1;
			s = &(dem_col[c_col][0]);
			c = s + 1;
			n = c + 1;
			se = &(dem_col[e_col][0]);
			e = se + 1;
			ne = e + 1;

			switch(d)  {
			case 0:              /** lighted from nnw **/
				c0 = w;
				c1 = nw;
				c2 = n;
				break;
			case 1:              /** lighted from ene **/
				c0 = n;
				c1 = ne;
				c2 = e;
				break;
			case 2:              /** lighted from sse **/
				c0 = e;
				c1 = se;
				c2 = s;
				break;
			case 3:              /** lighted from wsw **/
				c0 = s;
				c1 = sw;
				c2 = w;
				break;
			}


			byte = col >> 3;
			cptr[0] = &(dem[0][478][byte]);
			cptr[1] = &(dem[1][478][byte]);
			cptr[2] = &(dem[2][478][byte]);
			cptr[3] = &(dem[3][478][byte]);
			bit = (~col) & 7;
			mask = 1 << bit;
			for(i=1; i<479; i++, c++,c0++,c1++,c2++)
			{		/*** calculate de, the difference in elevation in direction d */

				de = 4 * *c - *c0 - *c1 - *c1 - *c2;
					/*** calculate gray scale (approximate) color value **/
				if(de > DE6)
				{	if(de > DE9)
					{	if(de > DE11)
							color = 12;
						else if(de > DE10)
							color = 11;
						else
							color = 10;
					}
					else
					{	if(de > DE8)
							color = 9;
						else if(de > DE7)
							color = 8;
						else
							color = 7;
					}
				}
				else
				{	if(de > DE3)
					{	if(de > DE5)
							color = 6;
						else if(de > DE4)
							color = 5;
						else
							color = 4;
					}
					else
					{	if(de > DE2)
							color = 3;
						else if(de > DE1)
							color = 2;
						else if(de > DE0)
							color = 1;
						else
							color = 0;
					}
				}


					/**** add to all 4 bit-planes for single dem pixel **/
				for(j=0; j<4; j++)
				{	if(color & (1 << j))
					{
						*cptr[j] |= mask;
					}
				}

					/*** increment bit-plane pointers to next pixel up **/
				cptr[0] -= BITPLN_WIDTH;
				cptr[1] -= BITPLN_WIDTH;
				cptr[2] -= BITPLN_WIDTH;
				cptr[3] -= BITPLN_WIDTH;
			}
		}

			/*--------- if column done, reset new_col flag *******/
		if(elevs >= no_elevs)
		{	new_col = 1;
			col++;
			col_index = (col_index + 1) % 3;
		}
	}

		/************* close file **************************************/
	closef(fd);


		/************* save screen & clear screen **********************/
		/*    save graphics for redisplay later                  */
	for(i=0; i<4; i++)
	{	swplane(i);
#ifndef LTP_X11
		bitmap(BITPLN_WIDTH,479,BITPLN_WIDTH,BITPLN_WIDTH,vga_plane,scratch_pad[i]);
#else
                bitmap4x(WIDTH,479,0,0,xmainwin,0,0,xmainpix);
#endif
		bar_off(0,0,639,479);
	}

		/************* reset colors for gray scale *********************/
                        /***       dark light  ***/
                        /***      R G B r g b  ***/
                        /***     3216 8 4 2 1  ***/
	colorvga(0,0);       /***  0 0 0 0 0 0 0 0  ***/
	colorvga(1,8);       /***  0 0 0 0 1 0 0 0  ***/
	colorvga(2,16);      /***  0 0 0 1 0 0 0 0  ***/
	colorvga(3,24);      /***  0 0 0 1 1 0 0 0  ***/
	colorvga(4,56);      /***  0 0 1 1 1 0 0 0  ***/
	colorvga(5,49);      /***  0 0 1 1 0 0 0 1  ***/
	colorvga(6,42);      /***  0 0 1 0 1 0 1 0  ***/
	colorvga(7,35);      /***  0 0 1 0 0 0 1 1  ***/
	colorvga(8,7);       /***  0 0 0 0 0 1 1 1  ***/
	colorvga(9,15);      /***  0 0 0 0 1 1 1 1  ***/
	colorvga(10,23);     /***  0 0 0 1 0 1 1 1  ***/
	colorvga(11,31);     /***  0 0 0 1 1 1 1 1  ***/
	colorvga(12,63);     /***  0 0 1 1 1 1 1 1  ***/


		/************* display dem view on screen **********************/
#ifdef LTP_X11        /***** this code only for X11 *****/
        /* get begin pointer to temp space  */
        if ((to_ptr = malloc_ltp(BITPLN_WIDTH*480+1)) == NULL) {
            printf("Unable to allocate space needed to display requested graphic.");
        }
	else {
#endif
	    for(i=0; i<4; i++)
	    {	swplane(i);
#ifndef LTP_X11      /***** first the original VGA code *****/
		bitmap(BITPLN_WIDTH,480,BITPLN_WIDTH,BITPLN_WIDTH,dem[i],vga_plane);
#else                /***** now the X11 code *****/
                bitmap(BITPLN_WIDTH,480,BITPLN_WIDTH,BITPLN_WIDTH,dem[i],to_ptr);
                bit_swap(to_ptr,BITPLN_WIDTH*480+1);
                xtemppix = XCreateBitmapFromData(xdisplay,xmainwin,to_ptr,BITPLN_WIDTH*8,480);
                XCopyPlane(xdisplay,xtemppix,xmainwin,xgccopy,0,0,BITPLN_WIDTH*8,480,0,0,0x01);
                XFreePixmap(xdisplay, xtemppix);
#endif               /***** end of X11 code *****/

	    }

		/************* prompt for return *******************************/
	    swplane(3);
	    sprintf(string,"file: %s",fname);
	    disptext(27,50,string);
	    switch(d) {
	    case 0: 
		    disptext(28,50,"Lighted from the NW.");
		    break;
	    case 1:
		    disptext(28,50,"Lighted from the NE.");
		    break;
	    case 2:
		    disptext(28,50,"Lighted from the SE.");
		    break;
	    case 3:
		    disptext(28,50,"Lighted from the SW.");
		    break;
	    }
	    disptext(29,50,"Press [RETURN] to continue");
	    get_string(string);

		/************* reset colors for normal screen ******************/
	    for(i=0; i<16; i++)
		colorvga(i,colors[i]);
#ifdef LTP_X11       /***** this close brace only needed for X11 *****/
	}
#endif
		/************* reset screen ************************************/
	/* reset all bit planes                          */
	for(i=0; i<4; i++)
	{	swplane(i);
#ifndef LTP_X11
		bitmap(BITPLN_WIDTH,480,BITPLN_WIDTH,BITPLN_WIDTH,scratch_pad[i],vga_plane);
#else
                bitmap4x(WIDTH,479,0,0,xmainpix,0,0,xmainwin);
#endif
	}

		/************* return ******************************************/
	return(0);
}
#ifdef BIG_ENDIAN    /***** need to swap byte order for byte level ops *****/
/*			byte_swap(dem,4*480*BITPLN_WIDTH,SWAP_LONG);
*/
#endif
