/*******************************************************************************
*	erase_margin	(see manual, Command Ref., User Functions)	<fc>   *
*			Description:  Turns off all pixels in the currently    *
*			defined margin area.                                   *
*			Parameters:  None.                                     *
*									       *
*			Returns zero upon normal end-of-job (EOJ).             *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

#define VERTICAL 9999999999.0

long half_erase();


long erase_margin( )
{
	long last_mode, last_snap;
	long i;
	long sw,nw,ne,se;
	long min, max;

	double boundry[4][2];
	double dmin, dmax;


		/************** check for map ***********************************/
	if(!job_stats[0])
	{	message("You must Create or Acquire a map befor erasing margins.");
		return(1);
	}

		/************* check for VIEW ONLY raster **********************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}

		/************** check for raster registration *******************/
	if(!registered(0))
	{	message("You must Register the raster befor erasing margins.");
		return(1);
	}


		/************** save input mode and snap mode *******************/
	last_mode = input_mode;
	last_snap = snap_input;

		/************** find which corner is which & margin boundries ***/
		/** 0. initialize     **/
	sw = 0;
	nw = 1;
	ne = 2;
	se = 3;
		/** 1. separate n & s **/
	if(corner[0][nw][1] < corner[0][sw][1])
	{	i = nw;
		nw = sw;
		sw = i;
	}
	if(corner[0][nw][1] < corner[0][se][1])
	{	i = nw;
		nw = se;
		se = i;
	}
	if(corner[0][nw][1] < corner[0][ne][1])
	{	i = nw;
		nw = ne;
		ne = i;
	}
	if(corner[0][ne][1] < corner[0][sw][1])
	{	i = ne;
		ne = sw;
		sw = i;
	}
	if(corner[0][ne][1] < corner[0][se][1])
	{	i = ne;
		ne = se;
		se = i;
	}
		/** 2. separate e & w **/
	if(corner[0][ne][0] < corner[0][nw][0])
	{	i = ne;
		ne = nw;
		nw = i;
	}
	if(corner[0][se][0] < corner[0][sw][0])
	{	i = se;
		se = sw;
		sw = i;
	}
		/** 3. set margin boundries **/
	boundry[0][0] = corner[0][sw][0];
	boundry[0][1] = corner[0][sw][1];
	boundry[1][0] = corner[0][nw][0];
	boundry[1][1] = corner[0][nw][1];
	boundry[2][0] = corner[0][ne][0];
	boundry[2][1] = corner[0][ne][1];
	boundry[3][0] = corner[0][se][0];
	boundry[3][1] = corner[0][se][1];

		/************** erase rows outside margin boundries *************/
		/** 1.  find max & min rows used & erase top & bottom **/
	dmax = dmin = boundry[0][1];
	for(i=1; i<4; i++)
	{	if(dmin > boundry[i][1])
			dmin = boundry[i][1];
		if(dmax < boundry[i][1])
			dmax = boundry[i][1];
	}
	dmin -= 1.0;
	min = dmin;
	dmax += 1.0;
	max = dmax;

	change_mode(11,0);         /** erase rectangle **/
	if(min >= 0)
	{	start_ltp(0,0);
		continue_ltp(job_stats[1],min);
	}
	if(max <= job_stats[2])
	{	start_ltp(0,max);
		continue_ltp(job_stats[1],job_stats[2]);
	}
	
		/** 2.  for lines between, find lt & rt intersects with margin & erase */
		/** 2a find slope (m), and y intercept (b) for all 4 sides. **/
	half_erase((long)boundry[0][0],(long)boundry[0][1],(long)boundry[1][0],(long)boundry[1][1],1);

	if(boundry[1][1] > boundry[2][1])    /** line slopes down to rt **/
		half_erase((long)boundry[2][0],(long)boundry[2][1],(long)boundry[1][0],(long)boundry[1][1],0);
	else
		half_erase((long)boundry[2][0],(long)boundry[2][1],(long)boundry[1][0],(long)boundry[1][1],1);

	half_erase((long)boundry[2][0],(long)boundry[2][1],(long)boundry[3][0],(long)boundry[3][1],0);

	if(boundry[0][1] > boundry[3][1])    /** line slopes down to rt **/
		half_erase((long)boundry[0][0],(long)boundry[0][1],(long)boundry[3][0],(long)boundry[3][1],1);
	else
		half_erase((long)boundry[0][0],(long)boundry[0][1],(long)boundry[3][0],(long)boundry[3][1],0);
	

		/************** reset mode **************************************/	
	change_mode(last_mode,last_snap);


		/************** return ******************************************/
	return(0);
}

/*******************************************************************************
*       half_erase                                                             *
*                                                                              *
*    Purpose:    erase a half plane to the rt or left of a line segment.       *
*       (based on line_diag, uses same algorythm to calc line.)                *
*                                                                              *
*    Arguments:    long x1,y1,x2,t2  line segment endpoints, (raster coords)   *
*                  long max          flag:  0 = erase lt, 1 = erase rt.        *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified:  none                                          *
*                                                                              *
*                      j.dabritz  2/6/90                                       *
*******************************************************************************/

long half_erase(x1,y1,x2,y2,max)
	long x1,y1,x2,y2;
	long max;
{	
	long dx,dy;
	long e,e1,e2;
	long i;

		/*********   clip line to raster ***************************/
	if(clipl(&x1,&y1,&x2,&y2,0,job_stats[1],0,job_stats[2]))
		return(0);

		/*********   calc dx and switch points to make it non-negative.   */
	dx = x2 - x1;
	if(dx < 0) 
	{	i = x1;
		x1 = x2;
		x2 = i;
		i = y1;
		y1 = y2;
		y2 = i;
		dx = x2 - x1;
	}
		/*********   calc cy  */
	dy = y2 - y1;


		/*********   divide situation up into cases ****************/
	if(dy >= 0)              /* line has positive slope */
	{	if(dx > dy)          /* slope is < 1  */
		{	e  = (dy * 2) - dx;
			e1 = (dy - dx) * 2;
			e2 = dy * 2;
			for(i = -1; i != dx; i++, x1++)
			{	if(e > 0)
				{	if(!max)
					{	start_ltp(x1+1,y1);
						continue_ltp(job_stats[1],y1);
					}
					y1++;
					if(max)
					{	start_ltp(0,y1);
						continue_ltp(x1,y1);
					}
					e += e1;
				}
				else
					e += e2;
			}
		}
		else                  /* slope is > 1    */
		{	e  = (dx * 2) - dy;
			e1 = (dx - dy) * 2;
			e2 = dx * 2;
			for(i = -1; i != dy; i++, y1++)
			{	if(!max)
				{	start_ltp(x1+1,y1);
					continue_ltp(job_stats[1],y1);
				}
				if(max)
				{	start_ltp(0,y1);
					continue_ltp(x1-1,y1);
				}
				if(e > 0)
				{	x1++;
					e += e1;
				}
				else
					e += e2;
			}
		}
	}
	else                     /* slope is negative  */
	{	if(dx > -dy)          /* slope is > -1  */
		{	e  = ((-dy) * 2) - dx;
			e1 = ((-dy) - dx) * 2;
			e2 = (-dy) * 2;
			for(i = -1; i != dx; i++, x1++)
			{	if(e > 0)
				{	if(!max)
					{	start_ltp(x1+1,y1);
						continue_ltp(job_stats[1],y1);
					}
					y1--;
					if(max)
					{	start_ltp(0,y1);
						continue_ltp(x1,y1);
					}
					e += e1;
				}
				else
					e += e2;
			}
		}
		else                  /* slope is < -1   */
		{	e  = (dx * 2) + dy;
			e1 = (dx + dy) * 2;
			e2 = dx * 2;
			for(i = 1; i != dy; i--, y1--)
			{	if(!max)
				{	start_ltp(x1+1,y1);
					continue_ltp(job_stats[1],y1);
				}
				if(max)
				{	start_ltp(0,y1);
					continue_ltp(x1-1,y1);
				}
				if(e > 0)
				{	x1++;
					e += e1;
				}
				else
					e += e2;
			}
		}
	}

	return(0);
}
