/*******************************************************************************
*       excell         (see manual, Command Ref., User Functions)       <fc>   *
*                                                                              *
*           This program writes a generic cell formatted ascii file            *
*           according to the parameters specified.                             *
*                                                                              *
*           Parameters:  char *fname    name of the file to be written         *
*                        long t;        number of atrbt characters / cell      *
*                        long c;        coord_type code:  same as register     *
*                        double wd;     cell width                             *
*                        double hd;     cell height                            *
*                        char str[];    characters used for obj w/o atrbts     *
*                                                                              *
*           Files/Values changed:  the exported file and job_stats[59]         *
*                                                                              *
*           Returns zero upon normal execution, 1=failure.                     *
*           J.Dabritz,   8/2/90                                                *
*******************************************************************************/

#include "../../incl/globals.h"
#include <string.h>
#include "../../incl/in.h"

int fd;
long lines_written;

long excellabort();


long excell(fname,t,c,wd,hd,str)
	char *fname;
	long t,c;
	double wd, hd;
	char *str;
{
	OBJECTS object;
	ATTRIBUTES atrbt;

	char actual [256];
	char string [256];
	char *sptr;

	long i;
	long xbegin,xend,ybegin,yend;
	long izone;
	long last_p;
	long xint, yint;
	long p;

	double x,y;
	double yinout;
	double xmin,xmax,ymin,ymax;


        /************ init global **************************************/
	fd = -1;

        /************ check for map loaded *****************************/
    if(!job_stats[0])
    {    message("no map is acquired, cannot write cell file.");
        return(1);
    }

        /************ check for map assembled **************************/
    if(!job_stats[10])
    {    message("you must assemble the map befor exporting data.");
        return(1);
    }

        /************ check t parameter ********************************/
    if((t < 1)  ||  (t > 30))
    {    message("Number of characters must be from 1 to 30.  no file written.");
        return(1);
    }

        /************ check for map registration ***********************/
	if(!registered(0)    ||    !registered(c))
	{	message("This map needs to be registered (raster and/or output coords.) befor export.");
		return(1);
	}
		
        /************ find limits of cell grid *************************/
	xmin = xmax = corner[c][0][0];
	ymin = ymax = corner[c][0][1];
	for(i=1; i<4;i++)
	{		/*** check min x **/
		if(xmin > corner[c][i][0])
			xmin = corner[c][i][0];

			/*** check max x **/
		if(xmax < corner[c][i][0])
			xmax = corner[c][i][0];

			/*** check min y **/
		if(ymin > corner[c][i][1])
			ymin = corner[c][i][1];

			/*** check max y **/
		if(ymax < corner[c][i][1])
			ymax = corner[c][i][1];
	}

        /************ center limits on cell centers ********************/
	xbegin = xmin / wd + 0.5;
	xend = xmax / wd + 0.5;
	ybegin = ymax / wd + 0.5;
	yend = ymin / wd + 0.5;

        /************ create & ouput file ******************************/
	message("setting up output files and variables.");
	if(setup_ofile(fname,actual))
	{	excellabort(1);
		return(1);
	}

        /************ open file ****************************************/
	fd = openf(actual,WRITE);
	if(fd < 0)
	{	excellabort(2);
		return(1);
	}

	message("writing cell file.");
        /************ write file header ********************************/
		/** std LTPlus print header **/
	std_header(string);
	trim(string);
	string[80] = (char) 0;
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(41);
		return(1);
	}
		/**** map name *****************/
	sprintf(string,"map name:           %s",map_name);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(3);
		return(1);
	}
		/**** coordinate system name ***/
		/**** coordinate system zone ***/
	switch(c) {
	case 0:       /** raster pixels **/
		sprintf(string,"coord system name:  raster pixels");
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(4);
			return(1);
		}
		sprintf(string,"coord system zone:  %10d",0);
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(5);
			return(1);
		}
		break;
	case 1:       /** decimal degrees, geodetics **/
		sprintf(string,"coord system name:  geodetics (lon/lat)");
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(6);
			return(1);
		}
		sprintf(string,"coord system zone:  %10d",0);
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(7);
			return(1);
		}
		break;
	case 2:       /** utm **/
		sprintf(string,"coord system name:  utm meters");
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(8);
			return(1);
		}
		if( check_llref() )
		{	excellabort(42);
			return(1);
		}
		if(lon_ref < 0.0)          /*** take both + & - lon ref as West lon */
			izone = (186.0 + lon_ref ) / 6.0;
		else
			izone = (186.0 - lon_ref ) / 6.0;
		sprintf(string,"coord system zone:  %10d",izone);
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(9);
			return(1);
		}
		break;
	case 3:       /** state plane ***/
		sprintf(string,"coord system name:  state plane");
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(10);
			return(1);
		}
		sprintf(string,"coord system zone:  %s",header.stplzn);
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(11);
			return(1);
		}
		break;
	case 4:       /** dig 1 **/
		sprintf(string,"coord system name:  digitizer 1 coordinates");
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(12);
			return(1);
		}
		sprintf(string,"coord system zone:  %10d",0);
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(13);
			return(1);
		}
		break;
	case 5:       /** dig 2 **/
		sprintf(string,"coord system name:  digitizer 2 coordinates");
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(14);
			return(1);
		}
		sprintf(string,"coord system zone:  %10d",0);
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(15);
			return(1);
		}
		break;
	case 6:       /** map inches **/
		sprintf(string,"coord system name:  map inches");
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(16);
			return(1);
		}
		sprintf(string,"coord system zone:  %10d",0);
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(17);
			return(1);
		}
		break;
	default:      /** other **/
		sprintf(string,"coord system name:  non-standard user specified coordinates");
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(18);
			return(1);
		}
		sprintf(string,"coord system zone:  %10d",0);
		writea(strfill(string,80),fd);
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(19);
				return(1);
		}
		break;
	}
		/**** cell width  **************/
	sprintf(string,"cell width:         %17.6f",wd);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(20);
		return(1);
	}
	
		/**** cell height **************/
	sprintf(string,"cell height:        %17.6f",hd);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(21);
		return(1);
	}
	
		/**** min x  *******************/
	sprintf(string,"lt col cell cntr x: %17.6f",(double) xbegin * wd);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(22);
		return(1);
	}
	
		/**** max x  *******************/
	sprintf(string,"rt col cell cntr x: %17.6f",(double) xend * wd);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(23);
		return(1);
	}
	
		/**** max y  *******************/
	sprintf(string,"top row cel cntr y: %17.6f",(double) ybegin * hd);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(24);
		return(1);
	}
	
		/**** min y  *******************/
	sprintf(string,"btm row cel cntr y: %17.6f",(double) yend * hd);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(25);
		return(1);
	}
	
		/**** characters per cell ******/
	sprintf(string,"char per cell:      %10d",t);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(26);
		return(1);
	}
	
		/**** number of cell rows ******/
	sprintf(string,"cell rows:          %10d",(long) ybegin - yend + 1);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(27);
		return(1);
	}
	
		/**** number of cell columns ***/
	sprintf(string,"cell columns:       %10d",(long) xend - xbegin + 1);
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(28);
		return(1);
	}
	
		/**** end of header marker *****/
	sprintf(string,"END OF HEADER");
	writea(strfill(string,80),fd);
	if(writeb("\n",1,1,fd) < 0)
	{	excellabort(29);
		return(1);
	}
	

        /************ grid map & write cell attributes *****************/
		/*** write all cell rows  **/
	last_p = -1;
	sptr = str;
	strfill(sptr,t);
	for(yint = ybegin; yint >= yend; yint--)
	{		/***** inform user *******/
		if(!((ybegin-yint) % 5))
		{	sprintf(string,"export cell, writing cell row %d",(yint - yend));
			message(string);
		}
			/**** check for user abort ****/
		if(abort_cmd())
		{	message(" export interupted in middle of file.");
			break;
		}

			/** calc y value ***********************/
		y = yint * hd;
			/** write all cell columns in one row **/
		for(xint = xbegin; xint <= xend; xint++)
		{		/*** calc x value ***/
			x = xint * wd;
				/*** convert to pixel coordinates ***/
			yinout = y;
			transform(&x,&yinout,c,0);
				/*** find polygon number ***/
			p = find_vpoly((float) x,(float) yinout,(long) 0);
				/*** get attribute *********/
			if(p != last_p)
			{	last_p = p;
					/*** check for legit p **/
				if(p < 0)
					sptr = str;
				else
				{		/*** get object ***/
					if(get_objects(&object,p))
					{	excellabort(29);
						return(1);
					}
						/*** get 1st attribute ***/
						/*** set characters ********/
					if(object.i12[2])
					{		/*** get attribute ***/
						if(get_attr(&atrbt,object.i12[2]))
						{	excellabort(30);
							return(1);
						}
						sptr = atrbt.string;
							/*** ensure t characters ***/
						strfill(sptr,t);
					}
					else
						sptr = str;
				}
			}
				/*** write to cell file ****/
			if(writeb(sptr,t,1,fd) < 0)
			{	excellabort(31);
				return(1);
			}
		}
			/**** output linefeed (end of row) ****/
		if(writeb("\n",1,1,fd) < 0)
		{	excellabort(32);
			return(1);
		}
	}


	message("closing files and freeing RAM space.");
        /************ close file ***************************************/
    if(closef_ptr(&fd))
    {   sprintf(string,"cell file complete, but cannot close file:  %s",actual);
        message(string);
        return(0);
    }

        /************ if a DOS file, copy to DOS partition/media *******/
    if(strchr(fname,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        actual);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        message(string);
        sprintf(string,"doscp -m %s %s",actual,fname);
        if(system_ltp(string))
        {    sprintf(string,"copy failed, file is at %s",actual);
            message(string);
        }
        else
        {    purgef(actual);
            message("done.");
        }
#endif
    }
    else
    {    sprintf(string,"cell export done.  File is at:  %s",actual);
        message(string);
    }

        /************ set file for cell exported ************************/
	job_stats[59] = job_stats[45] + 1;

        /************ write diary entry for cell export *****************/
	sprintf(string,"cell file output (#%d) to %s",job_stats[58],actual);
	put_diary(string);


        /************ return *******************************************/
    return(0);
}


/*******************************************************************************
*      excellabort                                                             *
*          This program is a convenient way of closing the output file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long excellabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* print abort message ******************************/
	sprintf(string,"export cell, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
