/*******************************************************************************
*       excff          (see manual, Command Ref., User Functions)       <fc>   *
*                                                                              *
*           This program writes a cff  import/export formatted ascii file      *
*           according to the parameters specified.                             *
*                                                                              *
*           Parameters:  char *fname    name of the file to be written         *
*                        long c         coord_type code:  same as register     *
*                        long s         set number to be exported (0=all obj)  *
*                                                                              *
*           Files/Values changed:  the exported file and job_stats[55]         *
*                                                                              *
*           Returns zero upon normal execution, 1=failure.                     *
*           J.Dabritz,   10/10/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include <string.h>
#include "../../incl/in.h"

int fd;

long excffabort();


long excff( fname, c, s)
	char *fname;
	long c, s;
{
	OBJECTS object, poly, segment;
	ATTRIBUTES atrbt;
	COORD_LIST coordlist;

	char actual [256];
	char string [256];
	char str[5][32];

	double x,y;

	long i,j,v,v1,v2;
	long type;
	long no_verticies;
	long seg,aseg;
	long increment;
	long latdeg,latmin,latsec,londeg,lonmin,lonsec;
	long feature_code;
	long vtx;


        /************ init global **************************************/
	fd = -1;

        /************ check for map loaded *****************************/
    if(!job_stats[0])
    {    message("no map is acquired, cannot write cff file.");
        return(1);
    }

        /************ check for map assembled **************************/
    if(!job_stats[10])
    {    message("you must assemble the map befor exporting data.");
        return(1);
    }

        /************ check for map registration ***********************/
	if(!registered(0)    ||    !registered(c))
	{	message("This map needs to be registered (raster and/or output coords.) befor export.");
		return(1);
	}


        /************ create & ouput file ******************************/
	message("setting up output files and variables.");
	if(setup_ofile(fname,actual))
	{	excffabort(11);
		return(1);
	}

        /************ open file ****************************************/
	fd = openf(actual,WRITE);
	if(fd < 0)
	{	excffabort(12);
		return(1);
	}

	message("writing cff file.");
        /************ write file header lines (map corners) ************/
	for(i=0; i<4; i++)
	{	dms(corner[1][i][1],&latdeg,&latmin,&latsec);
		dms(corner[1][i][0],&londeg,&lonmin,&lonsec);
		sprintf(string," %02d%02d%03d%02d           %4d   3\n",latdeg,latmin,londeg,lonmin,1);
		writea(string,fd);
		
		x = corner[c][i][0];
		y = corner[c][i][1];
		sprintf(string," %8.0f%8.0f0\n",x,y);
		writea(string,fd);
	}
        /************ check all objects & write file  ******************/
	for(i=0; i<job_stats[10]; i++)
	{		/********* inform user *************************************/
		if(!(i % 100))
		{	sprintf(string,"export cff, checking object %d.",i);
			message(string);
		}

			/********* check to see if object is in file ***************/
		if(get_objects(&object,i))
		{	excffabort(1);
			return(1);
		}
		if(object.i12[0] < 8)
			type = 0;                      /** point     */
		else if(object.i12[0] < 14)
			type = 1;                      /** segment   */
		else 
			type = 2;                      /** polygon   */

		if(object.i12[0] == 15)    /**** exterior poly ****/
		{	if(get_objects(&poly,object.i12[7]))
			{	excffabort(1);
				return(1);
			}
			object.i12[2] = poly.i12[2];  /* atr rec = parent atr rec */
			object.i12[4] = poly.i12[4];  /* set affiliation = parent sets */
		}
		
		if(object.i12[4] & (1 << s))
		{		/********* count verticies *****************************/
			switch(type) {
			case 0:                         /* node */
				no_verticies = 1;
				break;
			case 1:                         /* segment */
				no_verticies = object.i12[6] - object.i12[5] + 1;
				break;
			case 2:                         /* polygon */
				no_verticies = 0;
				no_verticies++;          /* beginning node */
				for(j=object.i12[5]; j<=object.i12[6]; j++) /* count seg vert */
				{	if(get_seglist(&seg,j))	    /* get each seg number */
					{	excffabort(3);
						return(1);
					}
					aseg = abs(seg);
					if(get_objects(&segment,aseg))  /* get each segment */
					{	excffabort(4);
						return(1);
					}
					no_verticies += segment.i12[6] - segment.i12[5];
				}
				break;
			}

			
				/********* write header line ***************************/
				/****** get feature codes **********/
			j = 0;
			atrbt.next = object.i12[2];
			for(; j<5 && atrbt.next;  )
			{	if(get_attr(&atrbt,atrbt.next))   /*  get attribute */
				{	excffabort(5);
					return(1);
				}
				trim(atrbt.string);
				if((atrbt.string[0] >= '0')  &&  (atrbt.string[0] <= '9'))
				{	if(sscanf(atrbt.string,"%ld",&feature_code))
					{	feature_code = feature_code % 1000;
						sprintf(str[j],"%03d ",feature_code);
						j++;
					}
				}
			}
			for(; j<5; j++)            /* fill remaining slots with blanks */
				sprintf(str[j],"    ");

				/********* write header line ***************************/
			sprintf(string," %s%s%s%s%s%4d   4\n",str[0],str[1],str[2],str[3],str[4],no_verticies);
			writea(string,fd);
			
				/********* write  veritcies ****************************/
			switch(type) {
			case 0:                         /* node */
				x = object.f4[0];
				y = object.f4[1];
				transform(&x,&y,0,c);
				sprintf(string," %8.0f%8.0f1\n",x,y);
				writea(string,fd);
				break;

			case 1:                         /* segment */
				for(v=object.i12[5]; v<=object.i12[6]; v++)
				{	if(get_coordlst(&coordlist,v))
					{	excffabort(6);
						return(1);
					}
					x = coordlist.x;
					y = coordlist.y;
					transform(&x,&y,0,c);
					if((v == object.i12[5]) || (v == object.i12[6])) /* endpt */
						sprintf(string," %8.0f%8.0f1\n",x,y);
					else
						sprintf(string," %8.0f%8.0f2\n",x,y);
					writea(string,fd);
				}
				break;

			case 2:                         /* polygon */
				vtx = 0;
				for(j=object.i12[5]; j<=object.i12[6]; j++)
				{	if(get_seglist(&seg,j))	    /* get each seg number */
					{	excffabort(8);
						return(1);
					}
					aseg = abs(seg);
					if(get_objects(&segment,aseg))  /* get each segment */
					{	excffabort(9);
						return(1);
					}

					if(seg > 0)
					{	v1 = segment.i12[5];
						v2 = segment.i12[6];
						increment = 1;
					}
					else
					{	v1 = segment.i12[6];
						v2 = segment.i12[5];
						increment = -1;
					}

					for(v=v1; v != v2 ; v += increment)
					{	if(get_coordlst(&coordlist,v))
						{	excffabort(10);
							return(1);
						}
						x = coordlist.x;
						y = coordlist.y;
						transform(&x,&y,0,c);

						if((v == v1)  &&  !vtx)
							sprintf(string," %8.0f%8.0f1\n",x,y);
						else
							sprintf(string," %8.0f%8.0f2\n",x,y);
						writea(string,fd);
						vtx++;
					}

				}
					/******* write ending vertex ***********/
				if(get_coordlst(&coordlist,v))
				{	excffabort(16);
					return(1);
				}
				x = coordlist.x;
				y = coordlist.y;
				transform(&x,&y,0,c);
				sprintf(string," %8.0f%8.0f1\n",x,y);
				writea(string,fd);
			}
		}
	}



	message("closing files and freeing RAM space.");
        /************ close file ***************************************/
    if(closef_ptr(&fd))
    {   sprintf(string,"cff file complete, but cannot close file:  %s",actual);
        message(string);
        return(0);
    }

        /************ if a DOS file, copy to DOS partition/media *******/
    if(strchr(fname,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        actual);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        message(string);
        sprintf(string,"doscp -m %s %s",actual,fname);
        if(system_ltp(string))
        {    sprintf(string,"copy failed, file is at %s",actual);
            message(string);
        }
        else
        {    purgef(actual);
            message("done.");
        }
#endif
    }
    else
    {    sprintf(string,"cff export done.  File is at:  %s",actual);
        message(string);
    }

        /************ set file for cff exported ************************/
	job_stats[55] = job_stats[45] + 1;

        /************ write diary entry for cff export *****************/
	sprintf(string,"cff file output (#%d) to %s",job_stats[52],actual);
	put_diary(string);

        /************ return *******************************************/
    return(0);
}


/*******************************************************************************
*      excffabort                                                             *
*          This program is a convenient way of closing the output file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long excffabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* print abort message ******************************/
	sprintf(string,"export cff, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
