/*******************************************************************************
*       exdig          (see manual, Command Ref., User Functions)       <fc>   *
*                                                                              *
*           This program writes a dig (ERDAS input)  formatted ascii file      *
*           according to the parameters specified.                             *
*                                                                              *
*           Parameters:  char *fname    name of the file to be written         *
*                        long a         attribute code:  1=obj, 3=obj & attr   *
*                        long c         coord_type code:  same as register     *
*                        long s         set number to be exported (0=all obj)  *
*                                                                              *
*           Files/Values changed:  the exported file and job_stats[58]         *
*                                                                              *
*           Returns zero upon normal execution, 1=failure.                     *
*           J.Dabritz,   10/10/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include <string.h>
#include "../../incl/in.h"

int fd;
long lines_written;

long exdigabort();
long write_digcoords();


long exdig( fname, a, c, s)
	char *fname;
	long a, c, s;
{
	OBJECTS object, segment;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	char actual [256];
	char string [256];

	double x,y;
	double last_x, last_y;

	long i,j,v,v2;
	long seg,aseg;
	long munits, npts, gis_value, mode;
	long header_offset;
	long endj;
	long vinc;



        /************ init global **************************************/
	fd = -1;

        /************ check for map loaded *****************************/
    if(!job_stats[0])
    {    message("no map is acquired, cannot write dig file.");
        return(1);
    }

        /************ check for map assembled **************************/
    if(!job_stats[10])
    {    message("you must assemble the map befor exporting data.");
        return(1);
    }

        /************ check for map registration ***********************/
	if(!registered(0)    ||    !registered(c))
	{	message("This map needs to be registered (raster and/or output coords.) befor export.");
		return(1);
	}
	switch(c) {
	case 2:         /* utm */
		munits = 1;
		break;
	case 3:         /* state plane */
		munits = 2;
		break;
	case 1:         /* lon/lat     */
		munits = 100;
		break;
	default:
		munits = 99;    /* default, coord system with origin in lower left */
		break;
	}

        /************ check for attributes only ************************/
	if(a == 2)
	{	message("The only way to pass attributes in a dig file is with coordinate information.");
		message("The file exported will be the same as if both objects & attributes were wanted.");
		a = 3;
	}

        /************ create & ouput file ******************************/
	message("setting up output files and variables.");
	if(setup_ofile(fname,actual))
	{	exdigabort(11);
		return(1);
	}

        /************ open file ****************************************/
	fd = openf(actual,WRITE);
	if(fd < 0)
	{	exdigabort(12);
		return(1);
	}

	message("writing dig file.");
        /************ write file header ********************************/
        /************ headerheader line ********************************/
	lines_written = 0;
	npts = 4;
	gis_value = -1;
	mode = 1;
			/** note:  lines written will be overwritten later. **/
	sprintf(string,"%5d%5d%5d%5d%5d\n",npts,gis_value,munits,mode,lines_written);
	writea(string,fd);
	lines_written++;
	
        /************ header verticies *********************************/
	for(i=0; i<4; i++)
	{	x = corner[0][i][0];
		y = corner[0][i][1];
		write_digcoords(x,y,c);
	}

        /************ check all objects & write file  ******************/
	for(i=0; i<job_stats[10]; i++)
	{		/********* inform user *************************************/
		if(!(i % 100))
		{	sprintf(string,"export dig, checking object %d, lines written: %d",i,lines_written);
			message(string);
		}

			/********* check to see if object is in file ***************/
		if(get_objects(&object,i))
		{	exdigabort(1);
			return(1);
		}

			/********* see if object is in set to be written ***********/
		if(object.i12[4] & (1 << s))
		{		/********* get attribute string ************************/
			gis_value = 0;
			if(a & 2)
			{	if(object.i12[2])
				{	if(get_attr(&attr,object.i12[2]))   /*  get attribute */
					{	exdigabort(5);
						return(1);
					}
					sscanf(attr.string,"%ld",&gis_value);
					if((gis_value < -9999) || (gis_value > 99999))
						gis_value = 0;
				}
			}

			if(object.i12[0] < 8)             /** point    **/
			{		/**** construct header line *****/
				mode = 3;        /* point */
				npts = 1;
				sprintf(string,"%5d%5d%5d%5d    ;\n",npts,gis_value,munits,mode,lines_written);
					/**** write header line *********/
				writea(string,fd);
				lines_written++;
					/**** write vertex line *********/
				x = object.f4[0];
				x = object.f4[1];
				write_digcoords(x,y,c);
			}
			else if(object.i12[0] < 14)       /** segment  **/
			{		/**** construct header line *****/
				mode = 2;        /* segment */
				npts = 0;
				sprintf(string,"%5d%5d%5d%5d    ;\n",npts,gis_value,munits,mode,lines_written);
					/**** write header line (temp) *********/
				header_offset = lines_written * 26;
				writea(string,fd);
				lines_written++;

					/**** write vertex lines *********/
				j=object.i12[5];
				endj=object.i12[6];
				for(;j<=endj;j++)
				{		/** get coord pair **/
					if(get_coordlst(&coordlist,j))
					{	exdigabort(6);
						return(1);
					}
					x = coordlist.x;
					y = coordlist.y;
					if(!npts || ((x != last_x) || (y != last_y)))
					{	last_x = x;
						last_y = y;
						write_digcoords(x,y,c);
						npts++;
					}
				}

					/**** write header line (final) ********/
				sprintf(string,"%5d%5d%5d%5d    ;\n",npts,gis_value,munits,mode,lines_written);
					/* position file pointer */
				if(0 > movefp(fd,header_offset,0))
				{	exdigabort(6);
					return(1);
				}
					/* overwrite header */
				writea(string,fd);
					/* re-position file pointer */
				if(0 > movefp(fd,lines_written*26,0))
				{	exdigabort(6);
					return(1);
				}
			}
			else                              /** polygon  **/
			{		/**** construct header line *****/
				mode = 1;        /* polygon */
				npts = 0;
				sprintf(string,"%5d%5d%5d%5d    ;\n",npts,gis_value,munits,mode,lines_written);
					/**** write header line (temp) *********/
				header_offset = lines_written * 26;
				writea(string,fd);
				lines_written++;

					/**** write vertex lines *********/
				j=object.i12[5];
				endj=object.i12[6];
				for(;j<=endj;j++)
				{		/** get segment number **/
					if(get_seglist(&seg,j))	    /* get each seg number */
					{	exdigabort(8);
						return(1);
					}
					aseg = abs(seg);
					if(get_objects(&segment,aseg))  /* get each segment */
					{	exdigabort(9);
						return(1);
					}
		
					if(seg < 0)
					{	v = segment.i12[6];
						v2 = segment.i12[5];
						vinc = -1;
					}
					else
					{	v = segment.i12[5];
						v2 = segment.i12[6];
						vinc = 1;
					}

					for(;v != v2; v += vinc)
					{		/** get coord pair **/
						if(get_coordlst(&coordlist,v))
						{	exdigabort(6);
							return(1);
						}
						x = coordlist.x;
						y = coordlist.y;
						if(!npts || ((x != last_x) || (y != last_y)))
						{	last_x = x;
							last_y = y;
							write_digcoords(x,y,c);
							npts++;
						}
					}
				}
	
					/**** write header line (final) ********/
				sprintf(string,"%5d%5d%5d%5d    ;\n",npts,gis_value,munits,mode,lines_written);
					/* position file pointer */
				if(0 > movefp(fd,header_offset,0))
				{	exdigabort(6);
					return(1);
				}
					/* overwrite header */
				writea(string,fd);
					/* re-position file pointer */
				if(0 > movefp(fd,lines_written*26,0))
				{	exdigabort(6);
					return(1);
				}
			}
		}
	}

        /************ overwrite first line with new lines_written value */
	sprintf(string,"%5d%5d%5d%5d%5d\n",4,-1,munits,1,lines_written);
		/* position file pointer */
	if(0 > movefp(fd,0,0))
	{	exdigabort(6);
		return(1);
	}
		/* overwrite header */
	writea(string,fd);
		/* re-position file pointer */
	if(0 > movefp(fd,lines_written*26,0))
	{	exdigabort(6);
		return(1);
	}


	message("closing files and freeing RAM space.");
        /************ close file ***************************************/
    if(closef_ptr(&fd))
    {   sprintf(string,"dig file complete, but cannot close file:  %s",actual);
        message(string);
        return(0);
    }

        /************ if a DOS file, copy to DOS partition/media *******/
    if(strchr(fname,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        actual);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        message(string);
        sprintf(string,"doscp -m %s %s",actual,fname);
        if(system_ltp(string))
        {    sprintf(string,"copy failed, file is at %s",actual);
            message(string);
        }
        else
        {    purgef(actual);
            message("done.");
        }
#endif
    }
    else
    {    sprintf(string,"dig export done.  File is at:  %s",actual);
        message(string);
    }

        /************ set file for dig exported ************************/
	job_stats[58] = job_stats[45] + 1;

        /************ write diary entry for dig export *****************/
	sprintf(string,"dig file output (#%d) to %s",job_stats[58],actual);
	put_diary(string);

        /************ return *******************************************/
    return(0);
}


/*******************************************************************************
*      exdigabort                                                             *
*          This program is a convenient way of closing the output file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long exdigabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* print abort message ******************************/
	sprintf(string,"export dig, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}


/*******************************************************************************
*   write_digcoords                                                            *
*           convert x,y to output coords & write to file, increment            *
*           lines_written variable;                                            *
*                                                                              *
*******************************************************************************/

long write_digcoords(x,y,c)
double x,y;
long c;
{
	char string[STRLIST];


		/********* convert x,y to output coords **********************/
	transform(&x,&y,0,c);

		/********* format line to be written *************************/
	if(c == 1)        /** geodetic coords **/
		sprintf(string,"%12.6f%12.6f;\n",x,y);
	else
		sprintf(string,"%12.2f%12.2f;\n",x,y);

		/********* write line & increment lines_written **************/
	writea(string,fd);
	lines_written++;

		/********* return ********************************************/
	return(0);
}
