/*******************************************************************************
*       exdigit        (see manual, Command Ref., User Functions)       <fc>   *
*                                                                              *
*           This program writes a digit file set                               *
*           according to the parameters specified.                             *
*                                                                              *
*           Parameters:  char *fname    name of the file to be written         *
*                        long a         attribute code:  1=obj, 3=obj & attr   *
*                        long c         coord_type code:  same as register     *
*                        long s         set number to be exported (0=all obj)  *
*                                                                              *
*           Files/Values changed:  the exported file and job_stats[54]         *
*                                                                              *
*           Returns zero upon normal execution, 1=failure.                     *
*           J.Dabritz,   11/17/89                                              *
*   Modified 4/5/90 by C. Jonson to create a '.cat' file that contains the     *
*   attribute strings if they are non-numeric only.  The '.att' file will      *
*   contain a number that is linked to an attribute string (character) in      *
*   the '.cat' file.  This is for SCS-GRASS only.  Standard GRASS does not     *
*   use nor does it handle character attribute string.  If you use character   *
*   attributes, they will only exist in the '.cat' file!                       *
*******************************************************************************/

#include "../../incl/globals.h"
#include <string.h>
#include <time.h>
#include "../../incl/in.h"

int fd;

long exdigitabort();
char *digcheck;       /*   a long flag for each object:  bit 0 = Line 
                                                         bit 1 = Arc
                                                         bit 2 = attribute **/


long exdigit( fname, a, c, s)
	char *fname;
	long a, c, s;
{
	struct tm *tm_date;
	struct dig_cat {
		long num;
		char attr[36];
		}  *cat_ptr, *cat_ptr_base, *tptr;

	OBJECTS object,segment;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	char actual [256];
	char fascii[256];
	char *lognm;
	char *ptr;
	char record_512[512];
	char string [256];

	double x,y;
	double xmin,xmax,ymin,ymax;
	double ln;

	long i, j, k, v;
	long catagory;
	long clock;
	long end;
	long end_vloop;
	long num_cats_blk, num_cats;
	long num_loops;
	long seg,aseg;
	long verticies;
	long zone;



        /************ init global **************************************/
	fd = -1;

        /************ check for map loaded *****************************/
    if(!job_stats[0])
    {    message("no map is acquired, cannot write digit file.");
        return(1);
    }

        /************ check for map assembled **************************/
    if(!job_stats[5])
    {    message("you must have assembled lines or polygons befor exporting in digit format.");
        return(1);
    }

        /************ check for map registration ***********************/
	if(!registered(0)    ||    !registered(c))
	{	message("This map needs to be registered (raster and/or output coords.) befor export.");
		return(1);
	}

        /************ check for attributes only ************************/
	if(a == 2)
	{	message("The only way to pass attributes in a digit file is with coordinate information.");
		message("The file exported will be the same as if both objects & attributes were wanted.");
		a = 3;
		sleep(3);
	}


        /************ create & ouput file ******************************/
	message("setting up output files and variables.");
		/*** truncate at 8 characters & add .ascii *******/
	for(ptr = fname, i=0; fname[i]; i++)
	{	if(fname[i] == '/')
			ptr = fname + i+1;
	}
	ptr[8] = 0;       /* truncate at 8 characters */
	sprintf(fascii,"%s.ascii",fname);
	if(setup_ofile(fascii,actual))
	{	exdigitabort(1);
		return(1);
	}

        /************ open file ****************************************/
	fd = openf(actual,WRITE);
	if(fd < 0)
	{	exdigitabort(2);
		return(1);
	}

		/*** malloc space for check_list ***/
	digcheck = (char *) 0;
	digcheck = (char *) calloc_ltp(job_stats[10],1);
	if(digcheck == (char *) 0)
	{	exdigitabort(3);
		return(1);
	}
	
		/*** check objects to see which ones go into file **/
	for(i=0; i<job_stats[10] ; i++)
	{		/********* inform user *************************************/
		if(!(i % 100))
		{	sprintf(string,"export digit, checking object %d.",i);
			message(string);
		}

			/********* check to see if object is in file ***************/
		if(get_objects(&object,i))
		{	exdigitabort(4);
			return(1);
		}
	
			/********* check to see if in file. *********************/
		switch(object.i12[0])  {
		case 8:  case 9:  case 10:            /* segment */
			if((s==0 && (map_type & 2))  ||  object.i12[0] & (1 << s))
				digcheck[i] = 5;
			break;
		case 14:  case 15:  case 16:          /* polygon */
			if((s==0 && (map_type & 4))  ||  object.i12[0] & (1 << s))
				digcheck[i] = 4;
			if((digcheck[i] & 4) || (object.i12[7] && (object.i12[7] & (1 << s))))
			{	end = object.i12[6];
				for(j=object.i12[5] ; j<= end; j++)
				{	if(get_seglist(&seg,j))	    /* get each seg number */
					{	exdigitabort(5);
						return(1);
					}
					aseg = abs(seg);
					if(object.i12[0] != 16)    /** not map border **/
						digcheck[aseg] |= 2;
					else
					{	if(get_objects(&segment,aseg))
						{	exdigitabort(99);
							return(1);
						}
						if(segment.i12[7] != segment.i12[8])
							digcheck[aseg] |= 2;
					}
				}
			}
			break;
		}
	}


		/************ write header ****************************************/
		/**** orgaization ***********/
	sprintf(string,"ORGANIZATION: %s\n",header.org);
	string[44] = 10;
	string[45] = 0;
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** date ******************/
	time( &clock );
	tm_date = localtime( &clock );
	sprintf(string,"DIGIT DATE:   %d/%d/%d\n",tm_date->tm_mon + 1, tm_date->tm_mday, tm_date->tm_year);
	string[34] = 10;
	string[35] = 0;
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** person ****************/
	if( (lognm = (char *) getlogin()) == (char *) NULL)	/*  get user login  */
		sprintf(string,"DIGIT NAME:   unknown");
	else
		sprintf(string,"DIGIT NAME:   %s\n",lognm);
	string[34] = 10;
	string[35] = 0;
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** map name **************/
	sprintf(string,"MAP NAME:     %s\n",map_name);
	string[55] = 10;
	string[56] = 0;
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** source date ***********/
	sprintf(string,"MAP DATE:     %s\n",header.source_yr);
	string[25] = 10;
	string[26] = 0;
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** comments **************/
	sprintf(string,"OTHER INFO:   LTPlus ver %.2f, %s, last map src rev: %s\n",ltp_version,header.quad_unit,header.revision_yr);
	string[87] = 10;
	string[88] = 0;
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** scale *****************/
	sprintf(string,"MAP SCALE:    %.0f\n",map_scale);
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** zone ******************/
	zone = 0;
	if(c == 2)      /* utm */
	{	trim(header.utmln);
		ln = 0.0;
    	sscanf(header.utmln,"%lf\n",&ln);
		ln = (ln < 0.0) ?  -ln : ln;
		ln = 31 - (ln / 6.0);
		zone = ln;

		if((zone > 20) || (zone < 1))
		{	ln = 0.0;
			for(i=0, ln=0.0; i<4; i++)
				ln += (corner[1][i][0] / 4.0);
			ln = (ln < 0.0) ?  -ln : ln;
			ln = 31 - (ln / 6.0);
			zone = ln;
		}
	}	
	else if(c == 3)   /* stpln */
		sscanf(header.stplzn,"%ld\n",&zone);

	sprintf(string,"ZONE:         %d\n",zone);
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
	
			/**** find limits of raster in output coord system **/
	xmin  = (job_stats[1] + 1) / 2.0;     /** center of raster **/
	ymin = (job_stats[2] + 1) / 2.0;
	transform(&xmin,&ymin,0,c);
	xmax = xmin;
	ymax = ymin;

	x = 0.0;             /*** lower left corner of the raster **/
	y = 0.0;
	if(!transform(&x,&y,0,c))
	{	if(x < xmin)
			xmin = x;
		else if(x > xmax)
			xmax = x;
		if(y < ymin)
			ymin = y;
		else if(y > ymax)
			ymax = y;
	}

	x = 0.0;                 /*** upper left corner of the raster **/
	y = job_stats[2];
	if(!transform(&x,&y,0,c))
	{	if(x < xmin)
			xmin = x;
		else if(x > xmax)
			xmax = x;
		if(y < ymin)
			ymin = y;
		else if(y > ymax)
			ymax = y;
	}
	
	x = job_stats[1];        /*** upper right corner of the raster **/
	y = job_stats[2];
	if(!transform(&x,&y,0,c))
	{	if(x < xmin)
			xmin = x;
		else if(x > xmax)
			xmax = x;
		if(y < ymin)
			ymin = y;
		else if(y > ymax)
			ymax = y;
	}
	
	x = job_stats[1];        /*** lower right corner of the raster **/
	y = 0.0;
	if(!transform(&x,&y,0,c))
	{	if(x < xmin)
			xmin = x;
		else if(x > xmax)
			xmax = x;
		if(y < ymin)
			ymin = y;
		else if(y > ymax)
			ymax = y;
	}
	
		/**** west edge *************/
	sprintf(string,"WEST EDGE:    %.2f\n",xmin);
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** east edge *************/
	sprintf(string,"EAST EDGE:    %.2f\n",xmax);
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** south edge ************/
	sprintf(string,"SOUTH EDGE:   %.2f\n",ymin);
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** north edge ************/
	sprintf(string,"NORTH EDGE:   %.2f\n",ymax);
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** digitizing resolution */
	sprintf(string,"MAP THRESH:   0.00\n");
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}
		/**** end of header (VERTI) */
	sprintf(string,"VERTI:        \n");
	if(writea(string,fd))
	{	exdigitabort(7);
		return(1);
	}

        /************ check all segments & write .ascii file  *************/
	end = (job_stats[7]) ?   job_stats[7] : job_stats[10];
	for(i=job_stats[5]; i<end ; i++)
	{		/********* inform user *************************************/
		if(!(i % 100))
		{	sprintf(string,"writing digit ascii file.  at object %d",i);
			message(string);
		}

			/********* check to see if object is in file ***************/
		if(digcheck[i] & 3)
		{		/** get segment **/
			if(get_objects(&object,i))
			{	exdigitabort(6);
				return(1);
			}
				/** write header line **/
			verticies = object.i12[6] - object.i12[5] + 1;
			if(digcheck[i] & 2)               /** arc **/
				sprintf(string,"A %4d\n",verticies);
			else
				sprintf(string,"L %4d\n",verticies);
			if(writea(string,fd))
			{	exdigitabort(7);
				return(1);
			}

				/** write verticies **/
			end_vloop = object.i12[6] + 1;
			for(v=object.i12[5]; v<end_vloop; v++)
			{	if(get_coordlst(&coordlist,v))
				{	exdigitabort(8);
					return(1);
				}
				x = coordlist.x;
				y = coordlist.y;
				transform(&x,&y,0,c);
				sprintf(string," %12.2f %12.2f\n",y,x);
				if(writea(string,fd))
				{	exdigitabort(9);
					return(1);
				}
			}
		}
	}
		/** close file **/
    if(closef_ptr(&fd))
    {   sprintf(string,"digit ascii file complete, but cannot close file:  %s",actual);
        message(string);
        return(0);
    }

        /************ if a DOS file, copy to DOS partition/media *******/
    if(strchr(fname,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        actual);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        message(string);
        sprintf(string,"doscp -m %s %s",actual,fname);
        if(system_ltp(string))
        {	sprintf(string,"copy failed, file is at %s",actual);
            message(string);
        }
        else
        {	purgef(actual);
            message("done.");
        }
#endif
    }
    else
    {	sprintf(string,"digit ascii file writen.  File is at:  %s",actual);
        message(string);
    }


	if(a > 1)
	{		/** open .att file **/
		message("setting up output file for att file.");
		sprintf(fascii,"%s.att",fname);
		if(setup_ofile(fascii,actual))
		{	exdigitabort(10);
			return(1);
		}
	
        	/************ open file ****************************************/
		fd = openf(actual,WRITE);
		if(fd < 0)
		{	exdigitabort(11);
			return(1);
		}
	
			/*** Calloc space for the cat array ***/
		cat_ptr = cat_ptr_base = (struct dig_cat *)calloc_ltp(job_stats[10],
								sizeof(struct dig_cat) );
		if( cat_ptr_base == (struct dig_cat *)0 )
		{	exdigitabort(15);
			return(1);
		}

			/** write .att and .cat files **/
		for(i=job_stats[5]; i<job_stats[10] ; i++)
		{		/********* inform user *************************************/
			if(!(i % 100))
			{	sprintf(string,"writing digit att file.  at object %d",i);
				message(string);
			}
	
				/********* check to see if object is in file ***************/
			if(digcheck[i] & 4)
			{		/** get segment **/
				if(get_objects(&object,i))
				{	exdigitabort(12);
					return(1);
				}
					/** get attribute & convert to catagory number **/
				if(object.i12[2])
				{	if(get_attr(&attr,object.i12[2]))   /*  get attribute */
					{	exdigitabort(13);
						return(1);
					}
	
					trim(attr.string);
					if(sscanf(attr.string,"%ld",&catagory) > 0)
					{	/** write att line **/
						x = object.f4[0];
						y = object.f4[1];
						transform(&x,&y,0,c);
				
						if(digcheck[i] & 1)               /** line **/
							sprintf(string,"L  %12.2f  %12.2f  %8d          \n",x,y,catagory);
						else
							sprintf(string,"A  %12.2f  %12.2f  %8d          \n",x,y,catagory);
						if(writea(string,fd))
						{	exdigitabort(14);
							return(1);
						}
					}
					else		/*** Character attribute ***/
					{	catagory = 0;
						/*** If attr string assigned catagory number       ***/
						/*   previously, assign matching number, else        */
						/*** add catagory to array and assign cat number.  ***/
						for(tptr = cat_ptr_base; tptr < cat_ptr; tptr++)
						{	if( !strncmp(attr.string, tptr->attr,
													strlen(attr.string)) )
							{	catagory = tptr->num;
								break;
							}
						}
						if( !catagory )		/** No previous catagory ***/
						{	if( cat_ptr == cat_ptr_base )  /** 1st catagory **/
							{	catagory = cat_ptr->num = 1;
								strcpy(cat_ptr->attr, attr.string);
								cat_ptr++;
							}
							else
							{	catagory = cat_ptr->num = (cat_ptr-1)->num + 1;
								strcpy(cat_ptr->attr, attr.string);
								cat_ptr++;
							}
						}

						/** write att line **/
						x = object.f4[0];
						y = object.f4[1];
						transform(&x,&y,0,c);
				
						if(digcheck[i] & 1)               /** line **/
							sprintf(string,"L  %12.2f  %12.2f  %8d          \n",x,y,catagory);
						else
							sprintf(string,"A  %12.2f  %12.2f  %8d          \n",x,y,catagory);
						if(writea(string,fd))
						{	exdigitabort(16);
							return(1);
						}
					}
				}
			}
		}
	
			/** close attribute file **/
		message("closing file.");
        	/************ close file ***************************************/
    	if(closef_ptr(&fd))
    	{   sprintf(string,"digit attr file written; cannot close file:  %s",actual);
        	message(string);
        	return(0);
    	}
	
        	/************ if a DOS file, copy to DOS partition/media *******/
    	if(strchr(fname,':'))
    	{   
#ifdef SV32
        	sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        	actual);
        	message(string);
#else
        	sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        	message(string);
        	sprintf(string,"doscp -m %s %s",actual,fname);
        	if(system_ltp(string))
        	{    sprintf(string,"copy failed, file is at %s",actual);
            	message(string);
        	}
        	else
        	{    purgef(actual);
            	message("done.");
        	}
#endif
    	}
    	else
    	{	sprintf(string,"digit attr file writen.  File is at:  %s",actual);
        	message(string);
    	}

	/**** Write the digit.cat file ******/
	  num_cats = cat_ptr - cat_ptr_base;
	  if( num_cats )		/** Process a .cat file ***/
	  {
		message("setting up output file for digit \".cat\" file.");
		sprintf(fascii,"%s.cat",fname);
		if(setup_ofile(fascii,actual))
		{	exdigitabort(17);
			return(1);
		}
	
		/************ open file ****************************************/
		if( (fd = openf(actual,WRITE)) == -1)
		{	exdigitabort(18);
			return(1);
		}
	
		/*** Write cat header records ***/
/*		sprintf(string, "%d #catagories\n", num_cats); */	/** 1st rec **/   
		sprintf(record_512, "# %d catagories\n", num_cats);	/** 1st rec **/
/*		strcat(record_512, string);  **** & line above, jpd fix, 3/11/91 ****/
		sprintf(string, "%s\n\n", map_name);			/** 2nd & 3rd rec **/
		strcat(record_512, string);
		sprintf(string, "0.00 0.00 0.00 0.00\n");		/** 4th rec **/
		strcat(record_512, string);
		sprintf(string, "0:no data\n");					/** 5th rec **/
		strcat(record_512, string);
		if( writea(record_512, fd) == -1 )
		{	exdigitabort(19);
			return(1);
		}
		/*** NULL out the record_512 array ***/
		for( k = 0; k < 512; record_512[k++] = 0 );
	
		/*** Write the cat catagory records ***/
		num_cats_blk = sizeof(record_512) / sizeof(struct dig_cat);
		if( num_cats % num_cats_blk )
			num_loops = num_cats / num_cats_blk + 1;
		else
			num_loops = num_cats / num_cats_blk;

		for( tptr = cat_ptr_base, j = 0; j < num_loops; j++ )
		{	for( k = 0; k < num_cats_blk && tptr < cat_ptr; k++, tptr++ )
			{	sprintf(string, "%d:%s\n", tptr->num, tptr->attr);
				strcat(record_512, string);
			}
	
			if( writea(record_512, fd) == -1 )
			{	exdigitabort(20);
				return(1);
			}
			/*** NULL out the record_512 array ***/
			for( k = 0; k < 512; record_512[k++] = 0 );
	
		}
		
		/** close cat file **/
		message("closing file.");
        	/************ close file ***************************************/
    	if(closef_ptr(&fd))
    	{   sprintf(string,"digit cat file written; cannot close file:  %s",actual);
        	message(string);
        	return(0);
    	}
	
        	/************ if a DOS file, copy to DOS partition/media *******/
    	if(strchr(fname,':'))
    	{   
#ifdef SV32
        	sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        	actual);
        	message(string);
#else
        	sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        	message(string);
        	sprintf(string,"doscp -m %s %s",actual,fname);
        	if(system_ltp(string))
        	{	sprintf(string,"copy failed, file is at %s",actual);
            	message(string);
        	}
        	else
        	{	purgef(actual);
            	message("done.");
        	}
#endif
    	}
    	else
    	{	sprintf(string,"digit export done.  File is at:  %s",actual);
			message(string);
    	}
	  }
      else
      {	sprintf(string,"digit export done.  File is at:  %s",actual);
	    message(string);
      }
	}

	/**** Free up the malloced/calloced space ***/
	message("freeing RAM space.");
	if( digcheck != (char *)0 )
		free_ltp(digcheck); 
	if( cat_ptr_base != (struct dig_cat *)0 )
		free_ltp(cat_ptr_base);

        /************ set file for digit exported ************************/
	job_stats[54] = job_stats[45] + 1;

        /************ write diary entry for digit export *****************/
	sprintf(string,"digit file(s) output (#%d) to %s",job_stats[54],actual);
	put_diary(string);


        /************ return *******************************************/
    return(0);
}


/*******************************************************************************
*      exdigitabort                                                            *
*          This program is a convenient way of closing the output file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long exdigitabort(location)
	long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************ free malloc space ********************************/
	if( digcheck != (char *)0 )
		free_ltp(digcheck); 

		/************* print abort message *****************************/
	sprintf(string,"export digit, aborted, location %d",location);
	message(string);

		/************* return ******************************************/
	return(0);
}
