/*******************************************************************************
*       exdlg3        (see manual, Command Ref., User Functions)    <fc>       *
*          computes and writes a digital line graph formatted file from        *
*       all or a subset of a LTPlus file.                                      *
*                                                                              *
*       Parameters:  char fname[]     name of ascii file to be written         *
*                    long a           attributes code: 1=objects only          *
*                                                      2=attributes only       *
*                                                      3=both                  *
*                    long at          attribute format code:  1 = code pairs   *
*                                                             2 = strings      *
*                    long c           coord system code:  2 = utm              *
*                                                         3 = state plane      *
*                    long s           set to be exported (0=all objects)       *
*                                                                              *
*       files/globals changed:  file written, job_stats[50], diary file        *
*                                                                              *
*       Returns (long)   0 = file written,  1 = failure                        *
*       J.Dabritz  8/24/89                                                     *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU - major and minor were being used in macros
   somewhere in the included files, so I changed them to majorc and minorc
   */

#include "../../incl/globals.h"
#include "../../incl/in.h"

#include <string.h>

long codepair();
long count_cdprs();
long count_atrbt();
long write_atrbts();


long exdlg3( fname, a, at, c, s )
    char *fname;
    long a,at,c,s;
{
	OBJECTS object, island;
	COORD_LIST coordlst; 

	char string1[256],string2[256],string3[256],string4[256],string5[256];
   char string[256];
	char actual[256];
	char line[256];
	char edge[32];
	char catagory_name[256];
	static char cp_label[4][3] = { "SW", "NW", "NE", "SE" };

	int fd;

	long *index;
	long nodes,areas,lines,degen_lines,no_islands;
	long code_pairs;
	long degen_line1, degen_line2;
	long b,i,j;
	long in_set;
	long begin, done, hit;
	long seg;
	long gpr, zone;
	long units_code;
	long text_chr;
	long line_len;
	long rt_area, lt_area;
	long line_segs;
	long no_coord_prs;
	long ln_seg[4];

	double ln;
	double res;
	double x,y;


	fd = -1;

        /************ check for map loaded *****************************/
    if(!job_stats[0])
    {    message("no map is acquired, cannot write dlg file.");
        return(1);
    }

        /************ check for map assembled **************************/
    if(!job_stats[10])
    {    message("you must assemble the map befor exporting data.");
        return(1);
    }

        /************ check for map registration ***********************/
	if(!registered(0)    ||    !registered(c))
	{	message("This map needs to be registered (raster and/or output coords.) befor export.");
		return(1);
	}

        /************ check for complete dlg ***************************/
    if(s != 0)
    {    message("A dlg file of a LTPlus subset is not usable by all systems which accept ");
        message("complete dlg files.");
        sleep(3);
    }


        /************ check for attributes only ************************/
	if(a == 2)
	{	message("The only way to pass attributes in a dlg file is with coordinate information.");
		message("The file exported will be the same as if both objects & attributes were wanted.");
		a = 3;
		sleep(3);
	}



        /************ create & ouput file ******************************/
	message("setting up output files and variables.");
	if(setup_ofile(fname,actual))
	{	message("export dlg3, abort, location 1.0");
		return(1);
	}

        /************ flag & calculate index numbers for all objects ***/
        /************ to be exported ***********************************/
		/*  1.  allocate an array for indexing LTPlus<>dlg record numbers */
	index = (long *) malloc_ltp(job_stats[10] * 4);
	if(!index)
	{	message("unable to acquire enough ram to index objects for dlg output, aborting.");
printf("5.0 unable to acquire enough ram to index objects for dlg output, aborting.");
		return(1);
	}
		/*  2.  flag objects which are to be exported */
	nodes = 0;
	areas = 0;
	lines = 0;
	for(i=0; i<job_stats[10]; i++)
	{	index[i] = 0;                   /* default */
		if(get_objects(&object,i))
		{	message("export dlg3, abort, location 2.0");
			return(1);
		}
		in_set = 0;
		if(object.i12[4] & (1<<s))        /* object is in set */
			in_set = 1;
		else if(object.i12[0] >= 14)   /* check if parent poly is in set */
		{	if(object.i12[7])
			{	if(index[object.i12[7]])
					in_set = 1;
			}
		}
		if(in_set)
		{	if(object.i12[0] < 8)             /* node */
				index[i] = ++nodes;
			else if(object.i12[0] < 14)       /* segment  */
				index[i] = ++lines;
			else                              /* polygon  */
			{	if(object.i12[0] != 15)       /* (not an exterior polygon) */
					index[i] = ++areas;
					/*  3.  for each poly, make sure all lines are included */
				for(j=object.i12[5]; j<= object.i12[6]; j++)
				{	if(get_seglist(&seg,j))
					{	message("export dlg3, abort, location 3.0");
						return(1);
					}
					seg = (seg < 0) ?  -seg : seg;
					if(!index[seg])
						index[seg] = ++lines;
				}
			}
		}
	}

	degen_lines = 0;
	done = (job_stats[7]) ?   job_stats[7] : job_stats[10];
	for(i=0; i<done; i++)
	{	if(index[i])
		{	if(get_objects(&object,i))
			{	message("export dlg3, abort, location 4.0");
				return(1);
			}
			   /*  4. for each isolated point/node throw in a degenerate line */
			if(object.i12[0] < 8)           /** point **/
			{	for(b=0, hit=0; !hit  &&  b<4  && object.i12[5+b] ;  b++)
				{	if(index[abs(object.i12[5+b])])
						hit = 1;
				}
				if(!hit)       /** an isolated node (point) **/
					degen_lines++;
			}
				/*  5.  for each line, make sure both endpoints included */
			else                            /** line  **/
			{	if(!index[object.i12[9]])
					index[object.i12[9]] = ++nodes;
				if(!index[object.i12[10]])
					index[object.i12[10]] = ++nodes;
			}
		}
	}
	degen_line1 = degen_line2 = lines + 1;
	
        /************ open file ****************************************/
	fd = openf(actual,WRITE);
	if(fd < 0)
	{	message("export dlg3, abort, location 5.0");
		return(1);
	}

	message("writin header portion of dlg3 file.");
        /************ write header, line 1 - banner - ******************/
	trim(header.head1);
	if(*header.head1)
		sprintf(line,"%-72s\n",header.head1);
	else
	{	trim(std_header(string));
		sprintf(line,"%-72s\n",string);
	}
	if(writea(line, fd))
	{	message("export dlg3, abort, location 6.0");
		closef_ptr(&fd);
		return(1);
	}

        /************ write header, line 2 - cartog. unit, scale, ******/
        /************ sectional indicator ******************************/
	sprintf(string1,"%s",header.quad_unit);
	trim(string1);
	string1[36] = (char) 0;
	sprintf(string2,"%s",header.state);
	trim(string2);
	string2[2] = (char) 0;
	sprintf(string3,"%s",header.source_yr);
	trim(string3);
	string3[4] = (char) 0;
	sprintf(string4,"%s",header.revision_yr);
	trim(string4);
	string4[4] = (char) 0;
	sprintf(string5,"%s",header.scale);
	trim(string5);
	string5[8] = (char) 0;
	sprintf(line,"%36s, %2s %4s, %4s %8s            \n",
		string1,string2,string3,string4,string5);
	if(writea(line, fd))
	{	message("export dlg3, abort, location 7.0");
		closef_ptr(&fd);
		return(1);
	}
	
        /************ write header, line 3, edge flags (blank = unchecked) */
	sprintf(edge,"        ");
	lowerc(header.edgematch);
	if(strchr(header.edgematch,'w'))
		edge[0] = edge[1] = '0';        /* mark w edge as passed edgmatching */
	if(strchr(header.edgematch,'n'))
		edge[2] = edge[3] = '0';        /* mark n edge as passed edgmatching */
	if(strchr(header.edgematch,'e'))
		edge[4] = edge[5] = '0';        /* mark e edge as passed edgmatching */
	if(strchr(header.edgematch,'s'))
		edge[6] = edge[7] = '0';        /* mark s edge as passed edgmatching */
	sprintf(line,"%72s\n",edge);
	if(writea(line, fd))
	{	message("export dlg3, abort, location 8.0");
		closef_ptr(&fd);
		return(1);
	}

        /************ write header, line 4, map/ground references ******/
	gpr = (c == 2) ?  1 : 2;   /* ground planimetric reference, 1=utm, 2=stpln*/

	zone = 0;
	if(gpr == 1)      /* utm */
		zone = utm_zone();
	else              /* stpln */
		sscanf(header.stplzn,"%ld",&zone);
	
	units_code = ((gpr == 1) || (zone == 9999)) ?  2 : 1;    /* 2=m, 1=ft */
	
	if(get_units())
	{	message("export dlg3, abort, location 9.0");
		closef_ptr(&fd);
		return(1);
	}
	res = (gpr == 1) ?  unit_factor[5][0] : unit_factor[2][0]; /* resolution */

	sprintf(line,"%6d%6d%6d%6d%18.11f%6d%6d%6d%6d      \n",3,gpr,zone,
		units_code, res,4,0,4,1);
	if(writea(line, fd))
	{	message("export dlg3, abort, location 10.0");
		closef_ptr(&fd);
		return(1);
	}
		

        /************ write header lines 5-9, transformation params ****/
	if(gpr == 1)       /* utm */
	{	ln = -1000000 * (180 - (zone * 6.0 - 3.0));
		sprintf(line,"  % .15E   0.450000000000000D+08   0.000000000000000D+00\n"			, ln);
		line[20] = 'D';
		if(writea(line, fd))      /*  line 5  */
		{	message("export dlg3, abort, location 11.0");
			closef_ptr(&fd);
			return(1);
		}
	
		sprintf(line,"   0.000000000000000D+00   0.000000000000000D+00   0.000000000000000D+00\n");
		if(writea(line, fd))      /*  line 6  */
		{	message("export dlg3, abort, location 12.0");
			closef_ptr(&fd);
			return(1);
		}
		if(writea(line, fd))      /*  line 7  */
		{	message("export dlg3, abort, location 13.0");
			closef_ptr(&fd);
			return(1);
		}
		if(writea(line, fd))      /*  line 8  */
		{	message("export dlg3, abort, location 14.0");
			closef_ptr(&fd);
			return(1);
		}
		if(writea(line, fd))      /*  line 9  */
		{	message("export dlg3, abort, location 15.0");
			closef_ptr(&fd);
			return(1);
		}
	}
	else               /* state plane */
	{	for(i=1; i<16; i+=3)
		{	sprintf(line,"  % .15E  % .15E  % .15E\n",stpln_params[i],
				stpln_params[i+1], stpln_params[i+2]);
			line[20] = 'D';
			line[44] = 'D';
			line[68] = 'D';
			if(writea(line, fd))
			{	message("export dlg3, abort, location 16.0");
				closef_ptr(&fd);
				return(1);
			}
		}
	}

        /************ write header line 10, internal transformation ****/
        /************ parameters ***************************************/
	sprintf(line," 0.10000000000D+01 0.00000000000D+00 0.00000000000D+00 0.00000000000D+00\n");
	if(writea(line, fd))
	{	message("export dlg3, abort, location 17.0");
		closef_ptr(&fd);
		return(1);
	}

        /************ write header line 11-14, corner references *******/
	for(i=0; i<4; i++)
	{	sprintf(line,"%s    %12.6f%12.6f      %12.2f%12.2f            \n",
			cp_label[i],corner[1][i][1],corner[1][i][0],
			corner[c][i][0], corner[c][i][1]);
		if(writea(line, fd))
		{	message("export dlg3, abort, location 18.0");
			closef_ptr(&fd);
			return(1);
		}
	}

        /************ write header line 15, catagory items *************/
	sprintf(string1,"%s",header.layer_name);
	string1[20] = (char) 0;
	sprintf(catagory_name,"%-20s",string1);
	sprintf(line,"%-20s%4d%6d%6d %1d%1d %6d%6d %1d%1d%1d%6d%6d   %1d\n",
		catagory_name,0,nodes,nodes,0,1,areas,areas,0,1,0,
		lines + degen_lines, lines+degen_lines,1);
	if(writea(line, fd))
	{	message("export dlg3, abort, location 19.0");
		closef_ptr(&fd);
		return(1);
	}


    /*================ end of header, begin object records =================*/
        /************ write node records *******************************/
        /************ write area records *******************************/
	message("writing node and area records.");
	for(i=0; i<job_stats[10]; i++)
	{		/*---------- check to see if object is to be exported ------*/
		if(index[i])
		{		/*  get object record **/
			if(get_objects(&object,i))
			{	message("export dlg3, abort, location 20.0");
				closef_ptr(&fd);
				return(1);
			}

				/** get x & y centroid values **/
			if((object.i12[0] < 8)  ||  (object.i12[0] > 13))
			{	x = object.f4[0];
				y = object.f4[1];
				if(transform(&x,&y,0,c))        /** convert to utm / stpln **/
				{	message("export dlg3, abort, location 21.0");
					closef_ptr(&fd);
					return(1);
				}
			}

				/*---------- if node, write node type record ---------------*/
			if(object.i12[0] < 8)         /** a node **/
			{		/** count code pair attributes **/
				if((at & 1) && (a & 2))
					code_pairs = count_cdprs(object.i12[2]);
				else
					code_pairs = 0;

					/** count text characters      **/
				if((at & 2) && (a & 2))
					text_chr = 72 * count_atrbt(object.i12[2]);
				else
					text_chr = 0;

					/** count line seg list        **/
				line_segs = 0;
				for(j=6; j<10; j++)
				{	if(object.i12[j])
					{	if(index[abs(object.i12[j])])
						{	if(object.i12[j] >= 0)
								ln_seg[line_segs] = index[object.i12[j]];
							else
								ln_seg[line_segs] = -index[abs(object.i12[j])];
							line_segs++;
						}
					}
				}
				if(line_segs == 0)
				{	ln_seg[0] = degen_line1++;
					ln_seg[1] = -ln_seg[0];
					line_segs = 2;
					index[i] = -index[i];    /* flag node for degen line */
				}

					/** write header line          **/
				sprintf(line,"N%5d%12.2f%12.2f%6d%6d%6d%6d%6d%6d\n",
				 abs(index[i]),x,y,0,line_segs,0,code_pairs, text_chr,0);
				if(writea(line, fd))
				{	message("export dlg3, abort, location 22.0");
					closef_ptr(&fd);
					return(1);
				}

					/** write line-seg list        **/
				for(j=0; j<line_segs; j++)
				{	sprintf(line,"%6d",ln_seg[j]);
					if(writea(line, fd))
					{	message("export dlg3, abort, location 23.0");
						closef_ptr(&fd);
						return(1);
					}
				}
				if(writea("\n", fd))
				{	message("export dlg3, abort, location 24.0");
					closef_ptr(&fd);
					return(1);
				}
				
					/** write code pair attributes **/
				if(a & 2)
					write_atrbts(object,at,fd);

			}

				/*---------- if area, write area type record ---------------*/
			else if(object.i12[0] > 13)   /** a polygon **/
			{		/** count code pair attributes **/
				if((at & 1) && (a & 2))
				{	code_pairs = count_cdprs(object.i12[2]);
					if(object.i12[0] == 16)
						code_pairs++;  /* add 1 for map border attribute */
				}
				else
					code_pairs = 0;

					/** count text characters      **/
				if((at & 2) && (a & 2))
					text_chr = 72 * count_atrbt(object.i12[2]);
				else
					text_chr = 0;

					/** count line seg list        **/
				line_segs = 0;
				line_segs = object.i12[6] - object.i12[5] + 1;
				no_islands = 0;
				if(object.i12[8])
				{	island.i12[8] = object.i12[8];
					for(; island.i12[8]; )
					{	if(get_objects(&island,island.i12[8]))
						{	message("export dlg3, abort, location 25.0");
							closef_ptr(&fd);
							return(1);
						}
						no_islands++;
						line_segs += island.i12[6] - island.i12[5] + 2;
					}
				}

					/** write header line          **/
				sprintf(line,"A%5d%12.2f%12.2f%6d%6d%6d%6d%6d%6d\n",
				 abs(index[i]),x,y,0,line_segs,0,code_pairs, 
				 text_chr,no_islands);
				if(writea(line, fd))
				{	message("export dlg3, abort, location 26.0");
					closef_ptr(&fd);
					return(1);
				}

					/** write line-seg list        **/
				line_len = 0;
				for(j=object.i12[5]; j <= object.i12[6]; j++)
				{	if(get_seglist(&seg,j))
					{	message("export dlg3, abort, location 27.0");
						closef_ptr(&fd);
						return(1);
					}
					if(seg > 0)
						sprintf(line,"%6d",index[seg]);
					else
						sprintf(line,"%6d",-index[-seg]);
					if(writea(line, fd))
					{	message("export dlg3, abort, location 28.0");
						closef_ptr(&fd);
						return(1);
					}
					line_len += 6;
					if(!(line_len % 72))
						writea("\n",fd);
				}
					/** add to line-seg list for islands **/
				island.i12[8] = object.i12[8];
				for(; island.i12[8] ;)
				{	if(get_objects(&island,island.i12[8]))
					{	message("export dlg3, abort, location 30.0");
						closef_ptr(&fd);
						return(1);
					}
					if(writea("     0", fd))
					{	message("export dlg3, abort, location 31.0");
						closef_ptr(&fd);
						return(1);
					}
					line_len += 6;
					if(!(line_len % 72))
						writea("\n",fd);
					for(j=island.i12[5]; j <= island.i12[6]; j++)
					{	if(get_seglist(&seg,j))
						{	message("export dlg3, abort, location 32.0");
							closef_ptr(&fd);
							return(1);
						}
						if(seg > 0)
							sprintf(line,"%6d",index[seg]);
						else
							sprintf(line,"%6d",-index[-seg]);
						if(writea(line, fd))
						{	message("export dlg3, abort, location 33.0");
							closef_ptr(&fd);
							return(1);
						}
						line_len += 6;
						if(!(line_len % 72))
							writea("\n",fd);
					}
				}
				if(line_len % 72)
					writea("\n",fd);

				
					/** write attributes **/
				if(a & 2)
					write_atrbts(object,at,fd);
			}

				/*---------- (skip for now if line) ------------------------*/
		}
	}

        /************ write line records *******************************/
	message("writing line records.");
	begin = (job_stats[5]) ?   job_stats[5] : job_stats[6];
	done = (job_stats[7]) ?    job_stats[7] : job_stats[10];
	for(i=begin; i<done; i++)
	{		/*---------- check to see if object is to be exported ------*/
		if(index[i])
		{		/*  get object record **/
			if(get_objects(&object,i))
			{	message("export dlg3, abort, location 34.0");
				closef_ptr(&fd);
				return(1);
			}
				/*---------- if segment write record -----------------------*/
			if((object.i12[0] >= 8)  &&  (object.i12[0] < 14))   /** segment **/
			{		/** count code pair attributes **/
				if((at & 1) && (a & 2))
					code_pairs = count_cdprs(object.i12[2]);
				else
					code_pairs = 0;

					/** count text characters      **/
				if((at & 2) && (a & 2))
					text_chr = 72 * count_atrbt(object.i12[2]);
				else
					text_chr = 0;

					/** get rt & lt areas          **/
				if(job_stats[7])
				{	rt_area = index[object.i12[7]];
					lt_area = index[object.i12[8]];
				}
				else
					rt_area = lt_area = 0;
		
					/** count coordinates          **/
				no_coord_prs = object.i12[6] - object.i12[5] + 1;

					/** write header line          **/
				sprintf(line,"L%5d%6d%6d%6d%6d%12s%6d%6d%6d\n",
				 index[i],index[object.i12[9]],index[object.i12[10]],
				 lt_area,rt_area," ",no_coord_prs,code_pairs,text_chr);
				if(writea(line, fd))
				{	message("export dlg3, abort, location 35.0");
					closef_ptr(&fd);
					return(1);
				}

					/** write coordinate list *******/
				line_len = 0;
				for(j = object.i12[5]; j <= object.i12[6]; j++)
				{	if(get_coordlst(&coordlst,j))
					{	message("export dlg3, abort, location 36.0");
						closef_ptr(&fd);
						return(1);
					}
					x = coordlst.x;
					y = coordlst.y;
					if(transform(&x,&y,0,c))   /** convert to utm / stpln **/
					{	message("export dlg3, abort, location 37.0");
						closef_ptr(&fd);
						return(1);
					}
					sprintf(line,"%12.2f%12.2f",x,y);
					if(writea(line, fd))
					{	message("export dlg3, abort, location 38.0");
						closef_ptr(&fd);
						return(1);
					}
					line_len += 24;
					if((line_len % 72) == 0)
					{	if(writea("\n", fd))
						{	message("export dlg3, abort, location 39.0");
							closef_ptr(&fd);
							return(1);
						}
					}
				}
				if((line_len % 72) != 0)
				{	if(writea("\n", fd))
					{	message("export dlg3, abort, location 40.0");
						closef_ptr(&fd);
						return(1);
					}
				}

					/** write attributes **/
				if(a & 2)
					write_atrbts(object,at,fd);

			}
		}
	}
	

        /************ write degenerate line records ********************/
	done = (job_stats[7]) ?   job_stats[7] : job_stats[10];
	for(i=0; i<done; i++)
	{	if(index[i] < 0)
		{	if(get_objects(&object,i))
			{	message("export dlg3, abort, location 41.0");
				closef_ptr(&fd);
				return(1);
			}

				/** write header line          **/
			sprintf(line,"L%5d%6d%6d%6d%6d%12s%6d%6d%6d\n",
			 degen_line2++,-index[i],-index[i],0,0," ",2,0,0);
			if(writea(line, fd))
			{	message("export dlg3, abort, location 42.0");
				closef_ptr(&fd);
				return(1);
			}
			
				/** get x & y centroid values **/
			x = object.f4[0];
			y = object.f4[1];
			if(transform(&x,&y,0,c))        /** convert to utm / stpln **/
			{	message("export dlg3, abort, location 43.0");
				closef_ptr(&fd);
				return(1);
			}

				/** write coord list record **/
			sprintf(line,"%12.2f%12.2f%12.2f%12.2f\n",x,y,x,y);
			if(writea(line, fd))
			{	message("export dlg3, abort, location 44.0");
				closef_ptr(&fd);
				return(1);
			}

		}
	}
	

    /*================ end of object records, file complete ================*/


	message("closing files and freeing RAM space.");
        /************ free malloc space ********************************/
	free_ltp((char *) index);

        /************ close file ***************************************/
    if(closef_ptr(&fd))
    {   sprintf(string,"dlg file complete, but cannot close file:  %s",actual);
        message(string);
        return(0);
    }

        /************ if a DOS file, copy to DOS partition/media *******/
    if(strchr(fname,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        actual);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        message(string);
        sprintf(string,"doscp -m %s %s",actual,fname);
        if(system_ltp(string))
        {    sprintf(string,"copy failed, file is at %s",actual);
            message(string);
        }
        else
        {    purgef(actual);
            message("done.");
        }
#endif
    }
    else
    {    sprintf(string,"dlg+ export done.  File is at:  %s",actual);
        message(string);
    }

        /************ set file for dlg exported ************************/
	job_stats[50] = job_stats[45] + 1;

        /************ write diary entry for dlg export *****************/
	sprintf(string,"dlg3 file output (#%d) to %s",job_stats[50],actual);
	put_diary(string);

        /************ return *******************************************/
    return(0);
}

/*******************************************************************************
*     codepair                                                                 *
*         This program looks at the string passed in and determines if it      *
*     can be interpreted as a dlg code pair.  To be able to, the leading       *
*     characters must be up to 7 digits, unsigned.  (Less than 7 will be 0     *
*     filled on the left.)  A 0 is returned if it can be a code pair,          *
*     otherwise a 1 is returned.  The input majorc, minorc are set if it can be  *
*     a code pair.                                                             *
*                                                                              *
*     Parameters  char string[30]                                              *
*                                                                              *
*     Returned values:   1 =  not a code pair candidate                        *
*                        0 =  the code pair has been set (majorc,minorc)         *
*     J.Dabritz   8/25/89                                                      *
*******************************************************************************/


long codepair(string,majorc,minorc)
	char *string;
	long *majorc, *minorc;
{
	long code_pair;

	if(!string)
		return(1);

	trim(string);
	if(sscanf(string,"%ld",&code_pair))
	{	if(code_pair >= 0)
		{	code_pair += 100000000;
			*majorc = (code_pair / 10000) % 10000;
			*minorc = code_pair % 10000;
			return(0);
		}
	}

	return(1);
}

/*******************************************************************************
*     count_cdprs                                                              *
*         This program counts the number of code pairs attributes that can     *
*     can be got from a linked list of LTPlus object attribute strings.        *
*     It starts with the record number of the first attribute and returns      *
*     the number of code pairs.                                                *
*                                                                              *
*     Parameters  long rec_no                                                  *
*                                                                              *
*     Returned values:   long code_pairs (number of code pairs)                *
*     J.Dabritz   8/25/89                                                      *
*******************************************************************************/

long count_cdprs(rec_no)
	long rec_no;
{
	ATTRIBUTES atrbt;
	long code_pairs;
	long majorc, minorc;

	atrbt.next = rec_no;
	for(code_pairs = 0; atrbt.next;)
	{	if(get_attr(&atrbt,atrbt.next))
			break;
		if(codepair(atrbt.string,&majorc,&minorc) == 0)
			code_pairs++;
	}

	return(code_pairs);
}


/*******************************************************************************
*     count_atrbt                                                              *
*         This program counts the number of LTPlus attribuets that an object   *
*     has.                                                                     *
*                                                                              *
*     Parameters  long rec_no                                                  *
*                                                                              *
*     Returned values:   long no_atrbts  (number of attributes)                *
*     J.Dabritz   8/25/89                                                      *
*******************************************************************************/

long count_atrbt(rec_no)
	long rec_no;
{
	ATTRIBUTES atrbt;
	long no_atrbts; 

	atrbt.next = rec_no;
	for(no_atrbts = 0 ; atrbt.next; no_atrbts++)
	{	if(get_attr(&atrbt,atrbt.next))
			break;
	}

	return(no_atrbts);
}

/*******************************************************************************
*     write_atrbts                                                             *
*         This program writes out the attribute and text lines for a dlg3      *
*     optional file.                                                           *
*                                                                              *
*     Parameters  OBJECTS object     object structure of object for which      *
*                                    attributes/strings are being written      *
*                 long at            atribute type, bit1=code prs, bit2 = text *
*                 int fd             file pointer                              *
*                                                                              *
*     Returned values:   long 0 = done, 1=error                                *
*     J.Dabritz   8/25/89                                                      *
*******************************************************************************/

long write_atrbts(object,at,fd)
	OBJECTS object;
	long at;
	int fd;
{
	long line_len;
	long majorc,minorc;
	ATTRIBUTES atrbt;
	char line[256];

	if(at & 1)
	{	line_len = 0;
		if(object.i12[0] == 16)        /* map border polygon */
		{	sprintf(line,"%6d%6d",0,0);
			if(writea(line, fd))
				return(1);
			line_len += 12;
		}
		for(atrbt.next = object.i12[2]; atrbt.next; )
		{	if(get_attr(&atrbt,atrbt.next))
				return(1);
			if(codepair(atrbt.string,&majorc,&minorc) == 0)
			{	sprintf(line,"%6d%6d",majorc,minorc);
				if(writea(line, fd))
					return(1);
				line_len += 12;
				if(!(line_len % 72))
				{	if(writea("\n", fd))
						return(1);
				}
			}
		}
		if(line_len % 72)
		{	if(writea("\n", fd))
				return(1);
		}
	}

		/** write text string(s)       **/
	if(at & 2)
	{	for(atrbt.next = object.i12[2]; atrbt.next; )
		{	if(get_attr(&atrbt,atrbt.next))
				return(1);
			sprintf(line,"%-72s\n",atrbt.string);
			if(writea(line, fd))
				return(1);
		}
	}

	return(0);
}
