/*******************************************************************************
*       exdxf         (see manual, Command Ref., User Functions)        <fc>   *
*                                                                              *
*             Writes a dxf export file of the specified map objects.           *
*             Points are written as points, segments as open polylines,        *
*             and polygons as closed polylines.   If attributes are given,     *
*             they are passed as text values at the centroid location(s).      *
*             No other header, block, tables etc. records are written.         *
*                                                                              *
*             Parameters:  char fname;     filename of dxf file to be written. *
*                          long a;         atrbt code:1=obj,2=att,3=both,4=3D  *
*                          long c;         coord type code 0-19;               *
*                          long s;         set to be exported: 0-31.           *
*                                                                              *
*             Globals changed:  job_stats[57] (dxf exported flag)              *
*                                                                              *
*             Values returned:   0 = export complete, 1=error                  *
*             J.Dabritz   7/13/90                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>
#include <string.h>
#include "../../incl/in.h"

int fd;
long first_vertex;
long ctype; 
char layer[STRLIST];

long exdxfabort();
long write_dxfseg();


long exdxf( fname, a, c, set)
	char *fname;
	long a, c, set;
{
	OBJECTS object, poly;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	char actual [256];
	char string [256];

	double x,y,z;
	double txt_ht;

	long count;
	long i,j;
	long next;
	long seg;


        /************ init global **************************************/
	fd = -1;

        /************ check for map loaded *****************************/
    if(!job_stats[0])
    {    message("no map is acquired, cannot write dxf file.");
        return(1);
    }

        /************ check for map assembled **************************/
    if(!job_stats[10])
    {    message("you must assemble the map befor exporting data.");
        return(1);
    }

        /************ check for map registration ***********************/
	if(!registered(0)    ||    !registered(c))
	{	message("This map needs to be registered (raster and/or output coords.) befor export.");
		return(1);
	}
	ctype = c;

			/*********** get text ht, if needed ***************************/
	if(a & 2)
	{	y = (corner[c][1][1] - corner[c][0][1]);
		x = (corner[c][1][0] - corner[c][0][0]);
		txt_ht = (sqrt(x*x + y*y)) / 200.0;
		if(txt_ht <= 0.0)
			txt_ht = 0.1;
	}


        /************ create & ouput file ******************************/
	message("setting up output files and variables.");
	if(setup_ofile(fname,actual))
	{	exdxfabort(11);
		return(1);
	}

        /************ open file ****************************************/
	fd = openf(actual,WRITE);
	if(fd < 0)
	{	exdxfabort(12);
		return(1);
	}

	message("writing dxf file.");
			/*********** write header lines ********************************/
	writea("  0\nSECTION\n  2\nHEADER\n  0\nENDSEC\n",fd);
			/*********** write tabels lines ********************************/
	writea("  0\nSECTION\n  2\nTABLES\n  0\nENDSEC\n",fd);
			/*********** write blocks lines ********************************/
	writea("  0\nSECTION\n  2\nBLOCKS\n  0\nENDSEC\n",fd);
			/*********** write entities lines ******************************/
	writea("  0\nSECTION\n  2\nENTITIES\n",fd);
        /************ check all objects & write file  ******************/
	count = 0;
	for(i=0; i<job_stats[10]; i++)
	{		/********* inform user *************************************/
		if(!(i % 100))
		{	sprintf(string,"export dxf, checking object %d, %d objects written to file.",i,count);
			message(string);
		}

			/********* get object record *******************************/
		if(get_objects(&object,i))
		{	exdxfabort(1);
			return(1);
		}

			/******** if 3D, get elevation ******************************/
		if(a & 4)
		{	z = 0.0;
			attr.next = object.i12[2];
			for(;attr.next;)
			{		/*** get attribute ***/
				if(get_attr(&attr,attr.next))   /*  get attribute */
				{	exdxfabort(17);
					return(1);
				}
				trim(attr.string);
				if(sscanf(attr.string,"%lf",&z) > 0)
					break;
			}
		}
		
			/******** check to see if object is in set ******************/
		if(object.i12[4] & (1 << set))
		{		/**** write lines according to object type *****/
			if(a & 5)        /*** objects wanted. ***/
			{	if(object.i12[0] < 8)             /** point     */
				{	writea("  0\nPOINT\n",fd); /* write object header */
					writea("  8\nLTPpoint\n",fd); /* write layer name    */
					x = object.f4[0];
					y = object.f4[1];
					transform(&x,&y,0,c);     /*** convert to output coords ***/
					if(a & 4)
						sprintf(string," 10\n%f\n 20\n%f\n 30\n%f\n",x,y,z);
					else
						sprintf(string," 10\n%f\n 20\n%f\n",x,y);
					writea(string,fd);
				}
				else if(object.i12[0] < 14)       /** segment   */
				{	writea("  0\nPOLYLINE\n",fd);     /** write object header **/
					sprintf(layer,"LTPseg");
					writea("  8\nLTPseg\n",fd); /* write layer name    */
					writea(" 66\n1\n",fd);      /* entities follow flag */
						/*** write polyline segment verticies ***/
					first_vertex = 1;
					if(write_dxfseg(i,a,z))
					{	exdxfabort(1);
						return(1);
					}
						/** terminate polyline **/
					writea("  0\nSEQEND\n",fd);
				}
				else                              /** polygon   */
				{		/*** write vertex list for parent polygon & island(s) ***/
						/* do vert for all polys/isl */
					for(next=i ; next ; next = poly.i12[8])
					{		/*** get polygon/island object ***/

						if(get_objects(&poly,next))	    /* get each poly/island */
						{	exdxfabort(7);
							return(1);
						}
							/*** write polyline header ***/
						writea("  0\nPOLYLINE\n",fd);
						sprintf(layer,"LTPpoly");
						writea("  8\nLTPpoly\n",fd); /* write layer name    */
						writea(" 66\n1\n",fd);      /* entities follow flag */
							/*** read seglist & write vertex list ***/
						first_vertex = 1;
						for(j=poly.i12[5]; j<=poly.i12[6]; j++) /* do each polyg  */
						{	if(get_seglist(&seg,j))	    /* get each seg number */
							{	exdxfabort(8);
								return(1);
							}
								/*** write polyline segment vertex list ***/
							if(write_dxfseg(seg,a,z))
							{	exdxfabort(1);
								return(1);
							}
						}
							/** terminate polyline **/
						writea("  0\nSEQEND\n",fd);
					}
				}
			}

			if(a & 2)        /*** attribute(s) wanted. ***/
			{	if(object.i12[2])   /** object has first attribute **/
				{		/*** get attribute ***/
					if(get_attr(&attr,object.i12[2]))   /*  get attribute */
					{	exdxfabort(5);
						return(1);
					}
						/**** write attribute text at each centroid ***/
						/****  primary centroid first ***/
					x = object.f4[0];
					y = object.f4[1];
					next = object.i12[3];
					for(;;)
					{	writea("  0\nTEXT\n",fd);     /** write object header **/
						writea("  8\nLTPatrbt\n",fd); /* write layer name    */
						transform(&x,&y,0,c);       /** convert to output coords **/
						sprintf(string," 10\n%f\n 20\n%f\n",x,y);
						writea(string,fd);          /** write coords        **/
						sprintf(string," 40\n%f\n",txt_ht);
						writea(string,fd);          /** write text ht.      **/
						sprintf(string,"  1\n%s\n",attr.string);
						writea(string,fd);          /** write text value    **/
							/*** check to see if last centroid reached ***/
						if(!next)
							break;
							/*** get next centroid ***/
						if(get_coordlst(&coordlist,next))
						{	exdxfabort(6);
							return(1);
						}
						x = coordlist.x;
						y = coordlist.y;
							/*** set next centroid record value (0=done) ***/
						next++;
						if(next > object.i12[11])
							next = 0;
					}
				}
			}
			count++;
		}
	}

		/******* write endsec ***********************************/
	writea("  0\nENDSEC\n",fd);
		/******* write end of file section **********************/
	writea("  0\nEOF\n",fd);
			

	message("closing files and freeing RAM space.");
        /************ close file ***************************************/
    if(closef_ptr(&fd))
    {   sprintf(string,"dxf file complete, but cannot close file:  %s",actual);
        message(string);
        return(0);
    }

        /************ if a DOS file, copy to DOS partition/media *******/
    if(strchr(fname,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        actual);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        message(string);
        sprintf(string,"doscp -m %s %s",actual,fname);
        if(system_ltp(string))
        {    sprintf(string,"copy failed, file is at %s",actual);
            message(string);
        }
        else
        {    purgef(actual);
            message("done.");
        }
#endif
    }
    else
    {    sprintf(string,"dxf export done.  File is at:  %s",actual);
        message(string);
    }

        /************ set file for dxf exported ************************/
	job_stats[57] = job_stats[45] + 1;

        /************ write diary entry for dxf export *****************/
	sprintf(string,"dxf file output (#%d) to %s",job_stats[57],actual);
	put_diary(string);

        /************ return *******************************************/
    return(0);
}


/*******************************************************************************
*      exdxfabort                                                             *
*          This program is a convenient way of closing the output file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long exdxfabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* print abort message ******************************/
	sprintf(string,"export dxf, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}

/*******************************************************************************
*   write_dxfseg     writes a vertex list for a dxf polyline                   *
*       Parameters:   long seg;         record number of seg to be written:    *
*                                      (if seg < 0, write in reverse order)    *
*                     long a;           attribute code                         *
*                     double z;         elevation                              *
*       Values returned:  0=segment written, 1=error                           *
*******************************************************************************/

long write_dxfseg(seg,a,z)
long seg;
long a;
double z;
{
	OBJECTS segment;
	COORD_LIST coordlist;

	char string[STRLIST];
	long s;
	long i,end,inc;
	
	static float last_x, last_y;

	double x,y;


		/********** get positive segment number ******************/
	s = (seg >= 0)?  seg: -seg;

		/********** read segment record **************************/
	if(get_objects(&segment,s))
	{	exdxfabort(1);
		return(1);
	}

		/********** set loop values ******************************/
	if(seg > 0)
	{	i = segment.i12[5];
		end = segment.i12[6] + 1;
		inc = 1;
	}
	else
	{	i = segment.i12[6];
		end = segment.i12[5] - 1;
		inc = -1;
	}

		/********** if first set & 3d, write out first x,y,z *****/
		/********** read & write verticies ***********************/
	for(;i!=end; i += inc)
	{		/*** read vertex ***/
		if(get_coordlst(&coordlist,i))
		{	exdxfabort(6);
			return(1);
		}
		x = coordlist.x;
		y = coordlist.y;

			/*** check for first vertex & 3d ***/
		if((a & 4) && (first_vertex))
		{		/*** transform coords ***/
			transform(&x,&y,0,ctype);
				/*** write coords ***/
			sprintf(string," 10\n%f\n 20\n%f\n 30\n%f\n",x,y,z);
			writea(string,fd);
				/*** reset x,y ***/
			x = coordlist.x;
			y = coordlist.y;
		}
			/*** check for first vertex, or changed coord ***/
		if((first_vertex) || (x != last_x) || (y != last_y))
		{		/*** write vertex header **/
			writea("  0\nVERTEX\n",fd);
			sprintf(string,"  8\n%s\n",layer);
			writea(string,fd); /* write layer name    */
				/*** transform coords ***/
			transform(&x,&y,0,ctype);
				/*** write coords ***/
			if(a & 4)
				sprintf(string," 10\n%f\n 20\n%f\n 30\n%f\n",x,y,z);
			else
				sprintf(string," 10\n%f\n 20\n%f\n",x,y);
			writea(string,fd);
				/*** reset last_x,y, first_vertex values ***/
			last_x = coordlist.x;
			last_y = coordlist.y;
			first_vertex = 0;
		}
	}

		/********** return ***************************************/
	return(0);
}
