/*******************************************************************************
*       exmoss         (see manual, Command Ref., User Functions)       <fc>   *
*                                                                              *
*           This program writes a moss import/export formatted ascii file      *
*           according to the parameters specified.                             *
*                                                                              *
*           Parameters:  char *fname    name of the file to be written         *
*                        long a         attribute code:  1=obj, 3=obj & attr   *
*                        long c         coord_type code:  same as register     *
*                        long otype     object type code: 0=node, 1=seg, 2=poly*
*                        long s         set number to be exported (0=all obj)  *
*                                                                              *
*           Files/Values changed:  the exported file and job_stats[52]         *
*                                                                              *
*           Returns zero upon normal execution, 1=failure.                     *
*           J.Dabritz,   10/10/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include <string.h>
#include "../../incl/in.h"

int fd;

long exmossabort();


long exmoss( fname, a, c, otype, s)
	char *fname;
	long a, c, otype, s;
{
	OBJECTS object, poly, segment;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	char actual [256];
	char string [256];

	double x,y;

	long item_no;
	long i,j,v,v1,v2;
	long type;
	long no_verticies;
	long next;
	long seg,aseg;
	long island;
	long increment;



        /************ init global **************************************/
	fd = -1;

        /************ check for map loaded *****************************/
    if(!job_stats[0])
    {    message("no map is acquired, cannot write moss file.");
        return(1);
    }

        /************ check for map assembled **************************/
    if(!job_stats[10])
    {    message("you must assemble the map befor exporting data.");
        return(1);
    }

        /************ check for map registration ***********************/
	if(!registered(0)    ||    !registered(c))
	{	message("This map needs to be registered (raster and/or output coords.) befor export.");
		return(1);
	}

        /************ check for attributes only ************************/
	if(a == 2)
	{	message("The only way to pass attributes in a moss file is with coordinate information.");
		message("The file exported will be the same as if both objects & attributes were wanted.");
		a = 3;
		sleep(3);
	}



        /************ create & ouput file ******************************/
	message("setting up output files and variables.");
	if(setup_ofile(fname,actual))
	{	exmossabort(11);
		return(1);
	}

        /************ open file ****************************************/
	fd = openf(actual,WRITE);
	if(fd < 0)
	{	exmossabort(12);
		return(1);
	}

	message("writing moss file.");
        /************ check all objects & write file  ******************/
	item_no = 1;
	for(i=0; i<job_stats[10]; i++)
	{		/********* inform user *************************************/
		if(!(i % 100))
		{	sprintf(string,"export moss, checking object %d, %d objects written to file.",i,item_no - 1);
			message(string);
		}

			/********* check to see if object is in file ***************/
		if(get_objects(&object,i))
		{	exmossabort(1);
			return(1);
		}
		if(object.i12[0] < 8)
			type = 0;                      /** point     */
		else if(object.i12[0] < 14)
			type = 1;                      /** segment   */
		else if(object.i12[0] != 15)
			type = 2;                      /** polygon   */
		else
			type = -1;
		
		if((type == otype)  &&  (object.i12[4] & (1 << s)))
		{		/********* count verticies *****************************/
			switch(type) {
			case 0:                         /* node */
				no_verticies = 1;
				break;
			case 1:                         /* segment */
				no_verticies = object.i12[6] - object.i12[5] + 1;
				break;
			case 2:                         /* polygon */
				no_verticies = 0;
				for(next=i ; next ;)            /* count total for polygon */
				{	if(get_objects(&poly,next))	    /* get each poly/island */
					{	exmossabort(2);
						return(1);
					}
					no_verticies++;          /* beginning node */
					for(j=poly.i12[5]; j<=poly.i12[6]; j++) /* count seg vert */
					{	if(get_seglist(&seg,j))	    /* get each seg number */
						{	exmossabort(3);
							return(1);
						}
						aseg = abs(seg);
						if(get_objects(&segment,aseg))  /* get each segment */
						{	exmossabort(4);
							return(1);
						}
						no_verticies += segment.i12[6] - segment.i12[5];
					}
					
					next = poly.i12[8];
				}
				break;
			}

			
				/********* get attribute string ************************/
			attr.string[0] = 0;         /* set to null */

			if(a & 2)
			{	if(object.i12[2])
				{	if(get_attr(&attr,object.i12[2]))   /*  get attribute */
					{	exmossabort(5);
						return(1);
					}
				}
			}

				/********* check no_verticies **************************/
			if(no_verticies > 32767)
			{	message("This file contains an object with more than 32767 verticeis and may not be ");
				message("readable by MOSS.  You may want to reduce verticies.");
				sleep(5);
			}

				/********* write header line ***************************/
			if(c == 1)    /** if geodetic output, then make item_no negative */
				sprintf(string,"%5d          %-30s     %5d\n", -item_no,attr.string,no_verticies);
			else
				sprintf(string,"%5d          %-30s     %5d\n", item_no,attr.string,no_verticies);

			writea(string,fd);
			item_no++;

			
				/********* write  veritcies ****************************/
			switch(type) {
			case 0:                         /* node */
				x = object.f4[0];
				y = object.f4[1];
				transform(&x,&y,0,c);
				if(c == 1)               /** geodetic coords **/
					sprintf(string,"%10.5f%10.5f 0\n",x,y);
				else
					sprintf(string,"%11.2f%11.2f\n",x,y);
				writea(string,fd);
				break;

			case 1:                         /* segment */
				for(v=object.i12[5]; v<=object.i12[6]; v++)
				{	if(get_coordlst(&coordlist,v))
					{	exmossabort(6);
						return(1);
					}
					x = coordlist.x;
					y = coordlist.y;
					transform(&x,&y,0,c);
					if(c == 1)               /** geodetic coords **/
						sprintf(string,"%10.5f%10.5f 0\n",x,y);
					else
						sprintf(string,"%11.2f%11.2f\n",x,y);
					writea(string,fd);
				}
				break;

			case 2:                         /* polygon */
				next = i;         
				island = 0;
				no_verticies = 0;
				for(next=i ; next ;island = 1)   /* do vert for all polys/isl */
				{	if(get_objects(&poly,next))	    /* get each poly/island */
					{	exmossabort(7);
						return(1);
					}
					for(j=poly.i12[5]; j<=poly.i12[6]; j++) /* do each polyg  */
					{	if(get_seglist(&seg,j))	    /* get each seg number */
						{	exmossabort(8);
							return(1);
						}
						aseg = abs(seg);
						if(get_objects(&segment,aseg))  /* get each segment */
						{	exmossabort(9);
							return(1);
						}

						if(seg > 0)
						{	v1 = segment.i12[5];
							v2 = segment.i12[6];
							increment = 1;
						}
						else
						{	v1 = segment.i12[6];
							v2 = segment.i12[5];
							increment = -1;
						}

						for(v=v1; v != v2 ; v += increment)
						{	if(get_coordlst(&coordlist,v))
							{	exmossabort(10);
								return(1);
							}
							x = coordlist.x;
							y = coordlist.y;
							transform(&x,&y,0,c);

							if(island)	/* make first island xvertex negative */
							{	if(c == 1)          /** geodetic coords **/
									sprintf(string,"%10.5f%10.5f 1\n",x,y);  
								else
									sprintf(string,"%11.2f%11.2f\n",-x,y);
								island = 0;
							}
							else
							{	if(c == 1)          /** geodetic coords **/
									sprintf(string,"%10.5f%10.5f 0\n",x,y);  
								else
									sprintf(string,"%11.2f%11.2f\n",x,y);
							}

							writea(string,fd);
						}

					}
						/******* write ending vertex ***********/
					if(get_coordlst(&coordlist,v))
					{	exmossabort(16);
						return(1);
					}
					x = coordlist.x;
					y = coordlist.y;
					transform(&x,&y,0,c);
					if(c == 1)          /** geodetic coords **/
						sprintf(string,"%10.5f%10.5f 0\n",x,y);  
					else
						sprintf(string,"%11.2f%11.2f\n",x,y);
					writea(string,fd);

					next = poly.i12[8];
				}
				break;
			}
		}
	}



	message("closing files and freeing RAM space.");
        /************ close file ***************************************/
    if(closef_ptr(&fd))
    {   sprintf(string,"moss file complete, but cannot close file:  %s",actual);
        message(string);
        return(0);
    }

        /************ if a DOS file, copy to DOS partition/media *******/
    if(strchr(fname,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        actual);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        message(string);
        sprintf(string,"doscp -m %s %s",actual,fname);
        if(system_ltp(string))
        {    sprintf(string,"copy failed, file is at %s",actual);
            message(string);
        }
        else
        {    purgef(actual);
            message("done.");
        }
#endif
    }
    else
    {    sprintf(string,"moss export done.  File is at:  %s",actual);
        message(string);
    }

        /************ set file for moss exported ************************/
	job_stats[52] = job_stats[45] + 1;

        /************ write diary entry for moss export *****************/
	sprintf(string,"moss file output (#%d) to %s",job_stats[52],actual);
	put_diary(string);

        /************ count subjects for user **************************/
	sprintf(string,"There are %d subjects in the output MOSS import/export file.",count_subj(s));
	message(string);

        /************ return *******************************************/
    return(0);
}


/*******************************************************************************
*      exmossabort                                                             *
*          This program is a convenient way of closing the output file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long exmossabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* print abort message ******************************/
	sprintf(string,"export moss, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
