/*******************************************************************************
*       exrdef2        (see manual, Command Ref., User Functions)       <fc>   *
*                                                                              *
*           This program writes an rdef2 formatted ascii file                  *
*           according to the parameters specified.                             *
*                                                                              *
*           Parameters:  char *fname    name of the file to be written         *
*                        long a         attribute code:  1=obj, 3=obj & attr   *
*                        long r         register code: 0=omit, 1=include       *
*                                                                              *
*           Files/Values changed:  the exported file and job_stats[53]         *
*                                                                              *
*           Returns zero upon normal execution, 1=failure.                     *
*           J.Dabritz,   10/12/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include <string.h>
#include "../../incl/in.h"

int fd;

long exrdef2abort();


long exrdef2( fname, a, r)
	char *fname;
	long a, r;
{
	OBJECTS object;
	ATTRIBUTES attr;

	char actual[256];
	char string[256];
	char zone[64];
	char typestring[32];
	char *ptr, *send_ptr;
	
	long attribute_lines;
	long registration_lines;
	long deg, min, sec;
	long c, i, j;
	long ix, iy;
	long row;
	long value;
	long last_c;
	long word, bit;
	
	long run_length;
	unsigned long *raster_row;

	double x,y;

        /************ init global **************************************/
	fd = -1;

        /************ check for map loaded *****************************/
    if(!job_stats[0])
    {    message("no map is acquired, cannot write rdef2 file.");
        return(1);
    }

        /************ create & ouput file ******************************/
	message("setting up output files and variables.");
	if(setup_ofile(fname,actual))
	{	exrdef2abort(1);
		return(1);
	}

        /************ open file ****************************************/
	fd = openf(actual,WRITE);
	if(fd < 0)
	{	exrdef2abort(2);
		return(1);
	}

	message("collecting header data.");
        /************ if attributes included, count attribute lines ****/
	attribute_lines = 0;
	if(a & 2)
	{	for(i=0; i<job_stats[10]; i++)
		{		/****** get object record ****/
			if(get_objects(&object,i))
			{	exrdef2abort(3);
				return(1);
			}

			if(object.i12[2])
				attribute_lines++;
		}
	}

        /************ set number of registration lines *****************/
	if(r)
		registration_lines = 7;
	else
		registration_lines = 0;

        /************ write file header line ***************************/
	message("writing header line.");
	if(registration_lines)
		sprintf(string,"%-30s%6d%6d%6d%6d\n",map_name,job_stats[1]+1,job_stats[2]+1,attribute_lines,registration_lines);
	else
		sprintf(string,"%-30s%6d%6d%6d\n",map_name,job_stats[1]+1,job_stats[2]+1,attribute_lines);
	writea(string,fd);

        /************ write registration data lines ********************/
	if(r)
	{	message("writing registration lines.");
			/** state plane zone **/
		sprintf(zone,"%s",header.stplzn);
		trim(zone);
		zone[6] = 0;
		trim(zone);
		sprintf(string,"STPLZ=%s\n",zone);
		writea(string,fd);
			/** scale **/
		sprintf(string,"SCL=%s\n",(char *) trim(header.scale));
		writea(string,fd);
			/** geodetics **/
		writea("GEOCP=",fd);
		for(i=0; i<4; i++)
		{	dms(corner[1][i][1],&deg,&min,&sec);
			sprintf(string,"%d:%d:%d,",deg,min,sec);
			writea(string,fd);
			x = corner[1][i][0];
			for(; x>0; x -= 360.0);  /** get West longitude **/
			dms(x,&deg,&min,&sec);
			if(i<3)
				sprintf(string,"%d:%d:%d,",deg,min,sec);
			else
				sprintf(string,"%d:%d:%d;\n",deg,min,sec);
			writea(string,fd);
		}
			/** columns **/
		sprintf(string,"COLS=%d\n",job_stats[1]+1);
		writea(string,fd);
			/** rows **/
		sprintf(string,"ROWS=%d\n",job_stats[2]+1);
		writea(string,fd);
			/** map inch 100ths  **/
		writea("MAPCP=",fd);
		for(i=0; i<4; i++)
		{	
			x = corner[6][i][0]*100.0+0.5;
			j = x;
			sprintf(string,"%d,",j);
			writea(string,fd);
			y = corner[6][i][1]*100.0+0.5;
			j = y;
			if(i<3)
				sprintf(string,"%d,",j);  
			else
				sprintf(string,"%d;\n",j);
			writea(string,fd);
		}
			/** map raster coords **/
		writea("GRDCP=",fd);
		for(i=0; i<4; i++)
		{	x = corner[0][i][0]+1.5;
			j = x;
			sprintf(string,"%d,",j);
			writea(string,fd);
			y = 1 + job_stats[2] - corner[0][i][1] + 0.5;
			j = y;
			if(i<3)
				sprintf(string,"%d,",j);
			else
				sprintf(string,"%d;\n",j);
			writea(string,fd);
		}
	}

        /************ write attribute journal lines ********************/
	if(a & 2)
	{	typestring[1] = 0;
		for(i=0; i<job_stats[10]; i++)
		{		/****** get object record ****/
			if(get_objects(&object,i))
			{	exrdef2abort(4);
				return(1);
			}

			if(object.i12[2])
			{		/****** find object type *****/
				if(object.i12[0] < 8)	        /* point  */
					typestring[0] = 'N';
				else if(object.i12[0] < 14)     /* segment */
					typestring[0] = 'L';
				else                            /* polygon */
					typestring[0] = 'A';
				
					/*** get position ***/
				x = 10 * (object.f4[0] + 1) + 0.5;
				ix = x;
				y = 10 * (job_stats[2] + 1 - object.f4[1]) + 0.5;
				iy = y;

					/*** get attribute string ***/
				if(get_attr(&attr,object.i12[2]))
				{	exrdef2abort(5);
					return(1);
				}

					/*** write journal line **/
				sprintf(string,"%s %6d%6d %s\n",typestring,ix,iy,attr.string);
				writea(string,fd);
			}
		}
	}


        /************ write raster header line *************************/
	sprintf(string,"RUN LENGTH ENCODED  %6d%6d%6d%6d\n",1,job_stats[1]+1,
														1,job_stats[2]+1);
	writea(string,fd);


        /************ write raster data body ***************************/
	send_ptr = string + 80;
	for(row = job_stats[2]; (a & 1) && (row >= 0); row--)
	{		/*** inform user ***/
		if(!(row % 25))
		{	sprintf(string,"Writing run length encoded raster data, row %d",row);
			message(string);
		}
			/*** write line number **/
		sprintf(string,"%6d",1 + job_stats[2] - row);
		ptr = string + 6;
			/*** set variables for raster row **/
		*send_ptr = '\n';
		*(send_ptr + 1) = 0;
		raster_row = (unsigned long *) lraster[row];
		value = 0;         /** off **/
		last_c = 0;    
		for(c=0; c<=job_stats[1]; )
		{	word = c >> 5;
			if((raster_row[word] == 0) && (value == 0))
				c = (c + 32) & ~31;
			else
			{	bit = (7 & ~c) | (c & 24);
					/**** check for changed value *****/
				if(value != ((raster_row[word] >> bit) & 1))
				{	run_length = c - last_c;
					for(; ; )
					{	*ptr = (run_length > 94) ?  126 : run_length+32;
						ptr++;
						if(ptr == send_ptr)
						{	writea(string,fd);
							sprintf(string,"      ");
							ptr = string + 6;
						}
						run_length -= 94;
						if(run_length > 0)
						{	*ptr = 32;
							ptr++;
							if(ptr == send_ptr)
							{	writea(string,fd);
								sprintf(string,"      ");
								ptr = string + 6;
							}
						}
						else
						{	last_c = c;
							break;
						}
					}
					value ^= 1;
				}
				c++;
			}
		}
		run_length = c - last_c;
		for(; run_length > 0; )
		{	*ptr = (run_length > 94) ?  126 : run_length+32;
			ptr++;
			if(ptr == send_ptr)
			{	writea(string,fd);
				sprintf(string,"      ");
				ptr = string + 6;
			}
			run_length -= 94;
			if(run_length > 0)
			{	*ptr = 32;
				ptr++;
				if(ptr == send_ptr)
				{	writea(string,fd);
					sprintf(string,"      ");
					ptr = string + 6;
				}
			}
			else
				break;
		}

		if(ptr > string + 6)
		{	*ptr = 10;
			ptr++;
			*ptr = 0;
			writea(string,fd);
		}

	}

        /************ write EOF ****************************************/
	writea("EOF\n",fd);


        /************ close file ***************************************/
	message("closing files.");
    if(closef_ptr(&fd))
    {   sprintf(string,"rdef2 file complete, but cannot close file:  %s",actual);
        message(string);
        return(0);
    }

        /************ if a DOS file, copy to DOS partition/media *******/
    if(strchr(fname,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  file is at: %s",
        actual);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual,fname);
        message(string);
        sprintf(string,"doscp -m %s %s",actual,fname);
        if(system_ltp(string))
        {    sprintf(string,"copy failed, file is at %s",actual);
            message(string);
        }
        else
        {    purgef(actual);
            message("done.");
        }
#endif
    }
    else
    {    sprintf(string,"rdef2 export done.  File is at:  %s",actual);
        message(string);
    }

        /************ set file for rdef2 exported ************************/
	job_stats[53] = job_stats[45] + 1;

        /************ write diary entry for rdef2 export *****************/
	sprintf(string,"rdef2 file output (#%d) to %s",job_stats[52],actual);
	put_diary(string);

        /************ if attributes included, check for thinned data ***/
	if(a == 3)
	{	message("checking for thinned data.");
		if(not_thinned(1))
		{	if(attribute_lines)
			{	wait_ltp("The RDEF2 file has UNTHINNED raster data which may not match the ATTRIBUTE data.");
				message("The RDEF2 file has UNTHINNED raster data which may not match the ATTRIBUTE data.");
			}
			else
			{	pause_ltp(3,"The RDEF2 file has UNTHINNED raster data.");
				message("The RDEF2 file has UNTHINNED raster data.");
			}
		}
	}

        /************ return *******************************************/
    return(0);
}


/*******************************************************************************
*      exrdef2abort                                                             *
*          This program is a convenient way of closing the output file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long exrdef2abort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* print abort message ******************************/
	sprintf(string,"export rdef2, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
