/*******************************************************************************
*       exsnap         (see manual, Command Ref., User Functions)       <fc>   *
*                                                                              *
*           This program writes a pair of PC-Snap (USDS-FS) formated files.    *
*           The pair can be for polygons or lines.                             *
*                                                                              *
*           Parameters:  char *fname    name of the file to be written         *
*                        long otype     object type code: 1=seg, 2=poly        *
*                        long c         coord_type code:  same as register     *
*                        long s         set number to be exported (0=all obj)  *
*                                                                              *
*           Files/Values changed:  the exported file and job_stats[60]         *
*                                                                              *
*           Returns zero upon normal execution, 1=failure.                     *
*           J.Dabritz,   10/10/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include <string.h>
#include "../../incl/in.h"

int fd1,fd2;

long exsnapabort();

	/******* Timber unit (polygon) attribute major code definitions ******/
#define SILV_PRESC        1
#define STATUS_CODE       100
#define FIXED_COST        5
#define SPECIES_CODE0     7
#define SPECIES_CODE1     12
#define SPECIES_CODE2     17
#define GROSS_MBF0        8
#define GROSS_MBF1        13
#define GROSS_MBF2        18
#define DEFECT_PERCENT0   9
#define DEFECT_PERCENT1   14
#define DEFECT_PERCENT2   19
#define QUALITY_PREM0     10      /*** used to be piece size0 **/
#define QUALITY_PREM1     15      /*** used to be piece size1 **/
#define QUALITY_PREM2     20      /*** used to be piece size2 **/
#define BEG_SERAL_STAGE   101     /*** used to be ecosystem ***/
#define HARVEST_SYSTEM0   4
#define HARVEST_SYSTEM1   102
#define HARVEST_SYSTEM2   103
#define ENTRY_NODE0       104
#define ENTRY_NODE1       105
#define ENTRY_NODE2       106
#define HARVEST_COST0     107
#define HARVEST_COST1     108
#define HARVEST_COST2     109
#define ACCUMULATOR       6

	/******* Road (segment) attribute major code definitions ******/
#define ROAD_STATUS       110
#define TERRAIN_TYPE      111
#define ACL_CODE          24
#define ROAD_TYPE         112
#define MAINT_CLASS       113
#define AVERAGE_GRADE     997
#define AVERAGE_SPEED     998
#define AVG_GRND_SLOPE    999


long exsnap( fname, otype, c, s)
	char *fname;
	long otype, c, s;
{
	OBJECTS object, node, segment;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	char string [256],*ptr1,*ptr2;
	char from_node[256],to_node[256];
	char fname1[256],actual1[256];
	char fname2[256],actual2[256];
	char parsed[3][STRLIST];
	char unit_label[32];
	char entry_node[3][32];

	double x,y;
	double last_x, last_y;
	double gross_mbf[3];
	double quality_prem[3];
	double harvest_cost[3];
	double fixed_cost;
	double simple_vertex[30][2];
	double average_grade, average_speed;
	double defect_percent[3];

	long no_points;
	long vlen;
	long i2;
	long road_status;
	long terrain_type;
	long acl_code;
	long road_type;
	long maint_class;
	long avg_grnd_slope;
	long zone;
	long no_params;
	long code;
	long silv_presc;
	long status_code;
	long species_code[3];
	long harvest_system[3];
	long beg_seral_stage;
	long accumulator[20];
	long next_accum;
	long deficit;
	long simple_verticies;
	long item_no;
	long i,j,v,v1,v2,n;
	long type;
	long seg,aseg;
	long increment;
	long is_label;

        /************ init global **************************************/
	fd1 = -1;
	fd2 = -1;

        /************ check for map loaded *****************************/
    if(!job_stats[0])
    {    message("no map is acquired, cannot write snap file.");
        return(1);
    }

        /************ check for map assembled **************************/
    if(!job_stats[10])
    {    message("you must assemble the map befor exporting data.");
        return(1);
    }

        /************ check otype parameter ******************/
	if((otype < 1) || (otype > 2))
	{	message("Snap output supports ONLY segment(1) or polygon(2) otypes.");
		return(1);
	}

        /************ check coordinate type parameter ******************/
	if((c < 2) || (c > 3))
	{	message("Snap output supports ONLY utm (2) or state_plane (3) coordinate types.");
		return(1);
	}

        /************ check for map registration ***********************/
	if(!registered(0)    ||    !registered(c))
	{	message("This map needs to be registered (raster and/or output coords.) befor export.");
		return(1);
	}

        /************ check setparameter ******************/
	if((s < 0) || (s > 31))
	{	message("The Set parameter must be in range 0-31.");
		return(1);
	}

		/** coordinat system code   & zone **/
	zone = -99;
	if(c == 3)   /** state plane **/
		sscanf(header.stplzn,"%ld",&zone);
	else         /** assume utm  **/	
		zone = utm_zone();

		/*** divide processing by otype ***/
	if(otype == 1)   /** roads ***/
   {     /************ create ouput files ******************************/
			/*** 	set up .lsa [ascii link stick](snap header) file ***/
		fname[8] = (char) 0;
		sprintf(fname1,"%s.lsa",fname);
		message("setting up output files and variables.");
		if(setup_ofile(fname1,actual1))
		{	exsnapabort(1);
			return(1);
		}
			/*** 	set up .lpa [ascii link point] (snap vector) file ***/
		sprintf(fname2,"%s.lpa",fname);
		if(setup_ofile(fname2,actual2))
		{	exsnapabort(2);
			return(1);
		}

        /************ open files ****************************************/
		fd1 = openf(actual1,WRITE);
		if(fd1 < 0)
		{	exsnapabort(3);
			return(1);
		}
		fd2 = openf(actual2,WRITE);
		if(fd2 < 0)
		{	exsnapabort(4);
			return(1);
		}

		message("writing snap files (.lsa & .lpa)");
		get_units();
		  /**** write blanks for holding number of links in file, later **/
		sprintf(string,"    \n");
		writea(string,fd2);
			/** end of record delimeter **/
		sprintf(string,"----------------\n");
		writea(string,fd2);

        /************ check all objects & write files ******************/
		item_no = 1;
		for(i=0; i<job_stats[10]; i++)
		{		/********* inform user *************************************/
			if(!(i % 100))
			{	sprintf(string,"export snap, checking object %d, %d objects written to files.",i,item_no - 1);
				message(string);
			}

				/********* check to see if object is in file ***************/
			if(get_objects(&object,i))
			{	exsnapabort(5);
				return(1);
			}
			if(object.i12[0] < 8)
				type = 0;                      /** point     */
			else if(object.i12[0] < 14)
				type = 1;                      /** segment   */
			else if(object.i12[0] == 14)
				type = 2;                      /** polygon   */
			else
				type = -1;
		
			simple_verticies = 0;
			if((type == otype)  &&  ((s == 0)  ||  (object.i12[4] & (1 << s))))
			{		/** find from node **/
					/* get node **/
				if(get_objects(&node,object.i12[9]))
				{	exsnapabort(8);
					return(1);
				}
					/* read 1st attribute */
				if(get_attr(&attr,node.i12[2]))
				{	exsnapabort(9);
					return(1);
				}
				attr.string[30] = (char) 0;
				trim(attr.string);
				if(*attr.string)
					sprintf(from_node,"%s\n",attr.string);
				else
					sprintf(from_node,"-99\n");
		
					/** to node **/
					/* get node **/
				if(get_objects(&node,object.i12[10]))
				{	exsnapabort(10);
					return(1);
				}
					/* read 1st attribute */
				if(get_attr(&attr,node.i12[2]))
				{	exsnapabort(11);
					return(1);
				}
				attr.string[30] = (char) 0;
				trim(attr.string);
				if(*attr.string)
					sprintf(to_node,"%s\n",attr.string);
				else
					sprintf(to_node,"-99\n");

					/*** write node names in 'nice' file ***/
				writea(from_node,fd2);
				writea(to_node,fd2);

					/*** count & write no of verticies (points) in 'nice' file ***/
				no_points = object.i12[6] - object.i12[5] + 1;
				sprintf(string,"%-5d\n",no_points);
				writea(string,fd2);

					/***** write 'nice' vertex list , set begin,end ptr ***/
				last_x = -1.0;
				last_y = -1.0;
					/*** write verticies ***/
				for(v=object.i12[5]; ; v++ )
				{	if(get_coordlst(&coordlist,v))
					{	exsnapabort(6);
						return(1);
					}
					x = coordlist.x;
					y = coordlist.y;
					if((x != last_x)  ||  (y != last_y))
					{	transform(&x,&y,0,c);
						sprintf(string,"%-7.0f             %-7.0f             %-7.0f\n",x,y,(double) -99.0);
						writea(string,fd2);
					}
					if(v == object.i12[6])
						break;
				}
					/**** write end of record delimeter in 'nice' file **/
				sprintf(string,"----------------\n");
				writea(string,fd2);
				

					/***** read attributes & fill header values ***/
					/** zero out values **/
				road_status = -99;
				terrain_type = -99;
				acl_code = -99; 
				road_type = -99;
				maint_class = -99;
				average_grade = -99.0;
				average_speed = -99.0;
				avg_grnd_slope = -99;

					/*** read attributes & fill header values ***/
				for(attr.next=object.i12[2]; attr.next; )
				{		/*** get attribute record ***/
					if(get_attr(&attr,attr.next))   /*  get attribute */
					{	exsnapabort(7);
						return(1);
					}
						/*** check attribute for initial digit & single parse **/
						/*** check for a code pair, or label *******************/
					no_params = parse(trim(attr.string)," ",parsed,3);
					is_label = 1;     /** default, a label (not a code pair) **/
					switch(no_params) {
					case 1:      /*** only a code pair if 5-7 digits ***/
						n = strlen(attr.string);
						if((n > 4) && (n < 8))   /** check for 5-7 characters **/
						{	for(ptr1 = attr.string; *ptr1; ptr1++)
							{	if((*ptr1 < '0') || (*ptr1 > '9'))
									break;
							}
							if(*ptr1 == (char) 0) /** all digits, ok, it'a a code pr */
								is_label = 0;
						}
						break;
					case 2:   /** must have a 1-3 digit group & a 1-4 digit group */
						is_label = 2;
						for(j=0; j<2; j++)   /** check both groups **/
						{		/** check length **/
							n = strlen(parsed[j]);
							if((n > 0) && (n < (4 + j))) /** check for 1-3/4 chars **/
							{		/*** check for all digits ***/
								for(ptr1 = parsed[j]; *ptr1; ptr1++)
								{	if((*ptr1 < '0') || (*ptr1 > '9'))
										break;
								}
								if(*ptr1 == (char) 0)
									is_label--;
							}
						}
						break;
					}

						/*** if a solid 5-7 digit number, break into maj/min codes **/
					if((no_params == 1) && (is_label == 0))
					{		/** move last 4 digits of parsed[0] into parsed[1] **/
						ptr1 = parsed[0] + n - 4;
						for(ptr2 = parsed[1];*ptr1; ptr1++, ptr2++)
							*ptr2 = *ptr1;

							/** end both strings with a null **/
						ptr1 = parsed[0] + n - 4;
						*ptr1 = (char) 0;
						*ptr2 = (char) 0;
							/** set number of substrings to 2 **/
						no_params = 2;
               }

						/** decode numeric code pairs ***/
					if(is_label == 0)
					{		/**** coded pair, decode & set header value ****/
							/**** find code *****/
						sscanf(parsed[0],"%ld",&code);
						switch(code) {
						case ROAD_STATUS:
							sscanf(parsed[1],"%ld",&road_status);
							break;
						case TERRAIN_TYPE:
							sscanf(parsed[1],"%ld",&terrain_type);
							break;
						case ACL_CODE:
							sscanf(parsed[1],"%ld",&acl_code);
							break;
						case ROAD_TYPE:
							sscanf(parsed[1],"%ld",&road_type);
							break;
						case MAINT_CLASS:
							sscanf(parsed[1],"%ld",&maint_class);
							break;
						case AVERAGE_GRADE:
							sscanf(parsed[1],"%lf",&average_grade);
							break;
						case AVERAGE_SPEED:
							sscanf(parsed[1],"%lf",&average_speed);
							break;
						case AVG_GRND_SLOPE:
							sscanf(parsed[1],"%ld",&avg_grnd_slope);
							break;
						}
					}
				}
				
					/***** write header (link stick) record ****/
					/** from node **/
				writea(from_node,fd1);
					/** to node **/
				writea(to_node,fd1);
					/** road status **/
				sprintf(string,"%-3d\n",road_status);
				writea(string,fd1);
					/** distance **/
				sprintf(string,"%-6.2f\n",object.f4[2]*unit_factor[3][0]);
				writea(string,fd1);
					/** terrain_type **/
				sprintf(string,"%-3d\n",terrain_type);
				writea(string,fd1);
					/** acl_code **/
				sprintf(string,"%-3d\n",acl_code);
				writea(string,fd1);
					/** road_type **/
				sprintf(string,"%-3d\n",road_type);
				writea(string,fd1);
					/** maint_class **/
				sprintf(string,"%-3d\n",maint_class);
				writea(string,fd1);
					/** average_grade **/
				sprintf(string,"%-5.1f\n",average_grade);
				writea(string,fd1);
					/** average_speed **/
				sprintf(string,"%-5.1f\n",average_speed);
				writea(string,fd1);
					/** avg_grnd_slope **/
				sprintf(string,"%-3d\n",avg_grnd_slope);
				writea(string,fd1);
					/** coordinat system code & zone  **/
				sprintf(string,"%1d                   %-6d\n",4-c,zone);
				writea(string,fd1);
					/** from coords, x,y,z **/
				if(get_coordlst(&coordlist,object.i12[5]))
				{	exsnapabort(12);
					return(1);
				}
				x = coordlist.x;
				y = coordlist.y;
				transform(&x,&y,0,c);

				sprintf(string,"%-7.0f             %-7.0f             %-7.0f\n",x,y,(double) -99.0);
				writea(string,fd1);
					/** to coords, x,y,z **/
				if(get_coordlst(&coordlist,object.i12[6]))
				{	exsnapabort(13);
					return(1);
				}
				x = coordlist.x;
				y = coordlist.y;
				transform(&x,&y,0,c);

				sprintf(string,"%-7.0f             %-7.0f             %-7.0f\n",x,y,(double) -99.0);
				writea(string,fd1);
					/** label x coord **/
				x = object.f4[0];
				y = object.f4[1];
				transform(&x,&y,0,c);

				sprintf(string,"%-7.0f\n",x);
				writea(string,fd1);
					/** label y coord **/
				sprintf(string,"%-7.0f\n",y);
				writea(string,fd1);
					/** end of record delimeter **/
				sprintf(string,"----------------\n");
				writea(string,fd1);
					/** increment item number written **/
				item_no++;
			}
		}
			/**** now, reposition pointer in 'nice' file to write no_links **/
		if(0 > movefp(fd2,0,0))
		{	exsnapabort(102);
			return(1);
		}
			/*** calc & write no_links (item_no) **/
		item_no--;
		sprintf(string,"%-4d\n",item_no);
		writea(string,fd2);

	}
	else             /** otype = 2, polygons **/
   {     /************ create ouput files ******************************/
			/*** 	set up .usa [ascii unit stick] (snap header) file ***/
		fname[8] = (char) 0;
		sprintf(fname1,"%s.usa",fname);
		message("setting up output files and variables.");
		if(setup_ofile(fname1,actual1))
		{	exsnapabort(14);
			return(1);
		}
			/*** 	set up .upa [ascii unit point] (snap vector) file ***/
		sprintf(fname2,"%s.upa",fname);
		if(setup_ofile(fname2,actual2))
		{	exsnapabort(15);
			return(1);
		}

        /************ open files ****************************************/
		fd1 = openf(actual1,WRITE);
		if(fd1 < 0)
		{	exsnapabort(16);
			return(1);
		}
		fd2 = openf(actual2,WRITE);
		if(fd2 < 0)
		{	exsnapabort(17);
			return(1);
		}

		message("writing snap files (.usa & .upa)");
		get_units();

		  /**** write blanks for holding number of units in file, later **/
		sprintf(string,"    \n");
		writea(string,fd2);
			/** end of record delimeter **/
		sprintf(string,"----------------\n");
		writea(string,fd2);

        /************ check all objects & write files ******************/
		item_no = 1;
		for(i=0; i<job_stats[10]; i++)
		{		/********* inform user *************************************/
			if(!(i % 100))
			{	sprintf(string,"export snap, checking object %d, %d objects written to files.",i,item_no - 1);
				message(string);
			}

				/********* check to see if object is in file ***************/
			if(get_objects(&object,i))
			{	exsnapabort(18);
				return(1);
			}
			if(object.i12[0] < 8)
				type = 0;                      /** point     */
			else if(object.i12[0] < 14)
				type = 1;                      /** segment   */
			else if(object.i12[0] == 14)
				type = 2;                      /** polygon   */
			else
				type = -1;
		
			simple_verticies = 0;
			if((type == otype)  &&  ((s == 0)  ||  (object.i12[4] & (1 << s))))
			{		/***** read attributes & fill header values ***/
					/** zero out values **/
				sprintf(unit_label,"-99");
				silv_presc = -99;
				status_code = -99;
				fixed_cost = -99.0;
				species_code[0] = -99;
				species_code[1] = -99;
				species_code[2] = -99;
				gross_mbf[0] = -99.0;
				gross_mbf[1] = -99.0;
				gross_mbf[2] = -99.0;
				defect_percent[0] = -99.0;
				defect_percent[1] = -99.0;
				defect_percent[2] = -99.0;
				quality_prem[0] = -99.0;
				quality_prem[1] = -99.0;
				quality_prem[2] = -99.0;
				beg_seral_stage = -99;
				harvest_system[0] = -99;
				harvest_system[1] = -99;
				harvest_system[2] = -99;
				sprintf(entry_node[0],"-99");
				sprintf(entry_node[1],"-99");
				sprintf(entry_node[2],"-99");
				harvest_cost[0] = -99.0;
				harvest_cost[1] = -99.0;
				harvest_cost[2] = -99.0;
				for(j=0; j<15; j++)
					accumulator[j] = 0;
				next_accum = 0;
					/*** read attributes & fill header values ***/
				for(attr.next=object.i12[2]; attr.next; )
				{		/*** get attribute record ***/
					if(get_attr(&attr,attr.next))   /*  get attribute */
					{	exsnapabort(22);
						return(1);
					}
						/*** check for a code pair, or label *******************/
					no_params = parse(trim(attr.string)," ",parsed,3);
					is_label = 1;     /** default, a label (not a code pair) **/
					switch(no_params) {
					case 1:      /*** only a code pair if 5-7 digits ***/
						n = strlen(attr.string);
						if((n > 4) && (n < 8))   /** check for 5-7 characters **/
						{	for(ptr1 = attr.string; *ptr1; ptr1++)
							{	if((*ptr1 < '0') || (*ptr1 > '9'))
									break;
							}
							if(*ptr1 == (char) 0) /** all digits, ok, it'a a code pr */
								is_label = 0;
						}
						break;
					case 2:   /** must have a 1-3 digit group & a 1-4 digit group */
						is_label = 2;
						for(j=0; j<2; j++)   /** check both groups **/
						{		/** check length **/
							n = strlen(parsed[j]);
							if((n > 0) && (n < (4 + j))) /** check for 1-3/4 chars **/
							{		/*** check for all digits ***/
								for(ptr1 = parsed[j]; *ptr1; ptr1++)
								{	if((*ptr1 < '0') || (*ptr1 > '9'))
										break;
								}
								if(*ptr1 == (char) 0)
									is_label--;
							}
						}
						break;
					}

						/*** if a solid 5-7 digit number, break into maj/min codes **/
					if((no_params == 1) && (is_label == 0))
					{		/** move last 4 digits of parsed[0] into parsed[1] **/
						ptr1 = parsed[0] + n - 4;
						for(ptr2 = parsed[1];*ptr1; ptr1++, ptr2++)
							*ptr2 = *ptr1;

							/** end both strings with a null **/
						ptr1 = parsed[0] + n - 4;
						*ptr1 = (char) 0;
						*ptr2 = (char) 0;
							/** set number of substrings to 2 **/
						no_params = 2;
               }

						/** decode numeric code pairs ***/
					if(is_label == 0)
					{		/**** coded pair, decode & set header value ****/
							/**** find code *****/
						sscanf(parsed[0],"%ld",&code);
						switch(code) {
						case SILV_PRESC:
							sscanf(parsed[1],"%ld",&silv_presc);
							break;
						case STATUS_CODE:
							sscanf(parsed[1],"%ld",&status_code);
							break;
						case FIXED_COST:
							sscanf(parsed[1],"%lf",&fixed_cost);
							break;
						case SPECIES_CODE0:
							sscanf(parsed[1],"%ld",&species_code[0]);
							break;
						case SPECIES_CODE1:
							sscanf(parsed[1],"%ld",&species_code[1]);
							break;
						case SPECIES_CODE2:
							sscanf(parsed[1],"%ld",&species_code[2]);
							break;
						case GROSS_MBF0:
							sscanf(parsed[1],"%lf",&gross_mbf[0]);
							break;
						case GROSS_MBF1:
							sscanf(parsed[1],"%lf",&gross_mbf[1]);
							break;
						case GROSS_MBF2:
							sscanf(parsed[1],"%lf",&gross_mbf[2]);
							break;
						case DEFECT_PERCENT0:
							sscanf(parsed[1],"%lf",&defect_percent[0]);
							break;
						case DEFECT_PERCENT1:
							sscanf(parsed[1],"%lf",&defect_percent[1]);
							break;
						case DEFECT_PERCENT2:
							sscanf(parsed[1],"%lf",&defect_percent[2]);
							break;
						case QUALITY_PREM0:
							sscanf(parsed[1],"%lf",&quality_prem[0]);
							break;
						case QUALITY_PREM1:
							sscanf(parsed[1],"%lf",&quality_prem[1]);
							break;
						case QUALITY_PREM2:
							sscanf(parsed[1],"%lf",&quality_prem[2]);
							break;
						case BEG_SERAL_STAGE:
							sscanf(parsed[1],"%ld",&beg_seral_stage);
							break;
						case HARVEST_SYSTEM0:
							sscanf(parsed[1],"%ld",&harvest_system[0]);
							break;
						case HARVEST_SYSTEM1:
							sscanf(parsed[1],"%ld",&harvest_system[1]);
							break;
						case HARVEST_SYSTEM2:
							sscanf(parsed[1],"%ld",&harvest_system[2]);
							break;
						case ENTRY_NODE0:
							sprintf(entry_node[0],"%s",parsed[1]);
							entry_node[0][20] = (char) 0;
							break;
						case ENTRY_NODE1:
							sprintf(entry_node[1],"%s",parsed[1]);
							entry_node[1][20] = (char) 0;
							break;
						case ENTRY_NODE2:
							sprintf(entry_node[2],"%s",parsed[1]);
							entry_node[2][20] = (char) 0;
							break;
						case HARVEST_COST0:
							sscanf(parsed[1],"%lf",&harvest_cost[0]);
							break;
						case HARVEST_COST1:
							sscanf(parsed[1],"%lf",&harvest_cost[1]);
							break;
						case HARVEST_COST2:
							sscanf(parsed[1],"%lf",&harvest_cost[2]);
							break;
						case ACCUMULATOR:
							if(next_accum < 15)
							{	sscanf(parsed[1],"%ld",&accumulator[next_accum]);
								next_accum++;
							}
							break;
						default:
							break;
						}
					}
					else  /**** string, assume that it is a unit label *****/
						sprintf(unit_label,"%s",attr.string);
				}

				
					/***** write unit label in 'nice' file ***/
				sprintf(string,"%s\n",unit_label);
				writea(string,fd2);

					/***** count nice points **********************/
				no_points = 0;
				for(j=object.i12[5]; j<=object.i12[6]; j++) /* count each seg  */
				{	if(get_seglist(&seg,j))	    /* get each seg number */
					{	exsnapabort(19);
						return(1);
					}
					aseg = abs(seg);
					if(get_objects(&segment,aseg))  /* get each segment */
					{	exsnapabort(20);
						return(1);
					}
					no_points += segment.i12[6] - segment.i12[5];
				}
					/***** write nice points **********************/
				sprintf(string,"%-5d\n",no_points);
				writea(string,fd2);
				
					/***** write 'nice' vertex list , set begin,end ptr ***/
				last_x = -1.0;
				last_y = -1.0;
				for(j=object.i12[5]; j<=object.i12[6]; j++) /* do each segment */
				{	if(get_seglist(&seg,j))	    /* get each seg number */
					{	exsnapabort(19);
						return(1);
					}
					aseg = abs(seg);
					if(get_objects(&segment,aseg))  /* get each segment */
					{	exsnapabort(20);
						return(1);
					}

					if(seg > 0)
					{	v1 = segment.i12[5];
						v2 = segment.i12[6];
						increment = 1;
					}
					else
					{	v1 = segment.i12[6];
						v2 = segment.i12[5];
						increment = -1;
					}

					for(v=v1; v != v2 ; v += increment)
					{	if(get_coordlst(&coordlist,v))
						{	exsnapabort(21);
							return(1);
						}
						x = coordlist.x;
						y = coordlist.y;
						if((x != last_x)  ||  (y != last_y))
						{	transform(&x,&y,0,c);
							if((v == v1) && (simple_verticies < 25))
							{	simple_vertex[simple_verticies][0] = x;
								simple_vertex[simple_verticies][1] = y;
								simple_verticies++;
							}
							sprintf(string,"%-7.0f             %-7.0f             %-7.0f\n",x,y,(double) -99.0);
							writea(string,fd2);
						}
					}
				}
					/** end of record delimeter **/
				sprintf(string,"----------------\n");
				writea(string,fd2);

					/***** generate simple vertex list, count verticies **/
				if(simple_verticies < 3)
				{		/*** find amount of deficit ***/
					deficit = 3 - simple_verticies;

						/*** get verticies from first segment long enough **/
					for(j=object.i12[5]; j<=object.i12[6]; j++) /* do each segment */
					{	if(get_seglist(&seg,j))	    /* get each seg number */
						{	exsnapabort(23);
							return(1);
						}
						aseg = abs(seg);
						if(get_objects(&segment,aseg))  /* get each segment */
						{	exsnapabort(24);
							return(1);
						}
	
						vlen = segment.i12[6] - segment.i12[5] + 1;
						if(vlen > 2)
						{	i2 = vlen / (deficit + 1);
							if(seg > 0)
							{	v1 = segment.i12[5];
								v2 = segment.i12[6];
								increment = i2;
							}
							else
							{	v1 = segment.i12[6];
								v2 = segment.i12[5];
								increment = -i2;
							}
	
							v = v1 + increment;
							for(i2=0; i2<deficit; i2++,v+=increment)
							{	if(get_coordlst(&coordlist,v))
								{	exsnapabort(25);
									return(1);
								}
								x = coordlist.x;
								y = coordlist.y;
								transform(&x,&y,0,c);
								simple_vertex[simple_verticies][0] = x;
								simple_vertex[simple_verticies][1] = y;
								simple_verticies++;
								if(simple_verticies > 25)
									simple_verticies = 25;
							}
							break;
						}
					}
				}

					/***** write header record ****/
					/** unit label **/
				sprintf(string,"%s\n",unit_label);
				writea(string,fd1);
					/** unit_area **/
				sprintf(string,"%-7.2f\n",object.f4[3]*unit_factor[2][1]);
				writea(string,fd1);
					/** silvecultural perscription **/
				sprintf(string,"%-3d\n",silv_presc);
				writea(string,fd1);
					/** status_code **/
				sprintf(string,"%-3d\n",status_code);
				writea(string,fd1);
					/** fixed_cost **/
				sprintf(string,"%-6.0f\n",fixed_cost);
				writea(string,fd1);
					/** species_codes **/
				sprintf(string,"%-5d               %-5d               %-5d\n",species_code[0],species_code[1],species_code[2]);
				writea(string,fd1);
					/** gross_mbf **/
				sprintf(string,"%-5.1f               %-5.1f               %-5.1f\n",gross_mbf[0],gross_mbf[1],gross_mbf[2]);
				writea(string,fd1);
					/** defect_percent **/
				sprintf(string,"%-5.1f               %-5.1f               %-5.1f\n",defect_percent[0],defect_percent[1],defect_percent[2]);
				writea(string,fd1);
					/** quality_prem  **/
				sprintf(string,"%-6.2f              %-6.2f              %-6.2f\n",quality_prem[0],quality_prem[1],quality_prem[2]);
				writea(string,fd1);
					/** beg_seral_stage **/
				sprintf(string,"%-3d\n",beg_seral_stage);
				writea(string,fd1);
					/** harvest_system **/
				sprintf(string,"%-3d                 %-3d                 %-3d\n",harvest_system[0],harvest_system[1],harvest_system[2]);
				writea(string,fd1);
					/** entry_node **/
				sprintf(string,"%-20s%-20s%-20s\n",entry_node[0],entry_node[1],entry_node[2]);
				writea(string,fd1);
					/** harvest_cost **/
				sprintf(string,"%-7.2f             %-7.2f             %-7.2f\n",harvest_cost[0],harvest_cost[1],harvest_cost[2]);
				writea(string,fd1);
					/** number of accumulators (attributes) **/
				if(next_accum)
					sprintf(string,"%-3d\n",next_accum);
				else
					sprintf(string,"-99\n",next_accum);
				writea(string,fd1);
					/** accumulators (attributes) **/
				for(j=0; j<next_accum; j++)
				{	sprintf(string,"%-3d\n",accumulator[j]);
					writea(string,fd1);
				}
					/** coordinat system code   & zone **/
				sprintf(string,"%1d                   %-6d\n",4-c,zone);
				writea(string,fd1);
					/** label x coord **/
				x = object.f4[0];
				y = object.f4[1];
				transform(&x,&y,0,c);

				sprintf(string,"%-7.0f\n",x);
				writea(string,fd1);
					/** label y coord **/
				sprintf(string,"%-7.0f\n",y);
				writea(string,fd1);
					/** number of simple verticies **/
				sprintf(string,"%-3d\n",simple_verticies);
				writea(string,fd1);
					/** simple verticies **/
				for(j=0; j<simple_verticies; j++)
				{	sprintf(string,"%-7.0f             %-7.0f             %-7.0f\n",simple_vertex[j][0],simple_vertex[j][1],(double) -99.0);
					writea(string,fd1);
				}
					/** end of record delimeter **/
				sprintf(string,"----------------\n");
				writea(string,fd1);
					/** increment item number written **/
				item_no++;
			}
		}
			/**** now, reposition pointer in 'nice' file to write no_links **/
		if(0 > movefp(fd2,0,0))
		{	exsnapabort(102);
			return(1);
		}
			/*** calc & write no_links (item_no) **/
		item_no--;
		sprintf(string,"%-4d\n",item_no);
		writea(string,fd2);

	}

	message("closing files.");
        /************ close file ***************************************/
    if(closef_ptr(&fd1))
    {   sprintf(string,"snap header file complete, but cannot close file:  %s",actual1);
        message(string);
    }
    if(closef_ptr(&fd2))
    {   sprintf(string,"snap vertex file complete, but cannot close file:  %s",actual2);
        message(string);
    }

        /************ if a DOS file, copy to DOS partition/media *******/
    if(strchr(fname1,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  header file is at: %s",
        actual1);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual1,fname1);
        message(string);
        sprintf(string,"doscp -m %s %s",actual1,fname1);
        if(system_ltp(string))
        {    sprintf(string,"copy failed, header file is at %s",actual1);
            message(string);
        }
        else
        {    purgef(actual1);
            message("done.");
        }
#endif
    }
    else
    {    sprintf(string,"export done.  header file at:  %s",actual1);
        message(string);
    }

    if(strchr(fname2,':'))
    {   
#ifdef SV32
        sprintf(string,"Cannot copy to DOS under ATT Unix.  vertex file is at: %s",
        actual2);
        message(string);
#else
        sprintf(string,"Now copying file %s to DOS file %s.",actual2,fname2);
        message(string);
        sprintf(string,"doscp -m %s %s",actual2,fname2);
        if(system_ltp(string))
        {    sprintf(string,"copy failed, vertex file is at %s",actual2);
            message(string);
        }
        else
        {    purgef(actual2);
            message("done.");
        }
#endif
    }
    else
    {    sprintf(string,"export done.  vertex file at:  %s",actual2);
        message(string);
    }

        /************ set file for snap exported ************************/
	job_stats[60] = job_stats[45] + 1;

        /************ write diary entry for snap export *****************/
	sprintf(string,"snap file output (#%d) to %s",job_stats[52],actual1);
	put_diary(string);

        /************ return *******************************************/
    return(0);
}


/*******************************************************************************
*      exsnapabort                                                             *
*          This program is a convenient way of closing the output file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long exsnapabort(location)
long location;
{	
	char string[256];


		/************ close file fd's if open ****************************/
	if(fd1 > 0)
		closef_ptr(&fd1);
	if(fd2 > 0)
		closef_ptr(&fd2);

		/************* print abort message ******************************/
	sprintf(string,"export snap, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
