/*******************************************************************************
*       fill_holes      (see manual, Command Ref., User Functions)     <fc>    *
*               Description:  Fills all 'open' locations which are             *
*               surrounded in all of the 4 cardinal directions.                *
*                                                                              *
*               Parameters:  None.                                             *
*                                                                              *
*               Globals changed:  lraster (craster)                            *
*                                                                              *
*               Returns zero upon normal end-of-job (EOJ).                     *
*               J.Dabritz   6/29/89                                            *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long fill_holes( )
{
	unsigned long *ptr, *end_ptr;
	long bit, word, x, y;
	long count;

	char string[STRLIST];

		/**************** check for loaded map **************************/
	if(!job_stats[0])
		return(0);

		/************* check for VIEW ONLY raster **********************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}

append_msg(" . . filling holes . . ");
		/**************** erase a 1 pixel column on rt edge of raster ***/
		/**************** (to avoid wrapping on raster in locale_ptr()) */
	erase_rect(job_stats[1],0,job_stats[1],job_stats[2]);
	
		/**************** search raster for holes & fill 'em ************/
	count = 0;
	end_ptr = lraster[job_stats[2]];
	for(ptr = lraster[1]; ptr < end_ptr; ptr++)
	{	if(*ptr)
		{	for(bit = 0; bit < 32; bit++)
			{	switch(bit) {
				case 7:                      /*  left most bit   */
					if((*ptr & 64) && !(*ptr & 128) && (*(ptr-1) & (1<<24)))
					{	if(85 == (85 & locale_ptr(ptr,bit)))
						{	word = (ptr - lraster[0]) % raster_wincrement;
							x = (word << 5) | (7 & ~bit) | (bit & 24);
							y = (ptr - lraster[0]) / raster_wincrement;
							pixel(x,y,1);
							count++;
						}
					}
					break;
				case 24:                     /*  right most bit  */
					if((*ptr & (1<<25)) && !(*ptr & (1<<24)) && 
						(*(ptr+1) & 128) )
					{	if(85 == (85 & locale_ptr(ptr,bit)))
						{	word = (ptr - lraster[0]) % raster_wincrement;
							x = (word << 5) | (7 & ~bit) | (bit & 24);
							y = (ptr - lraster[0]) / raster_wincrement;
							pixel(x,y,1);
							count++;
						}
					}
					break;
				default:
					switch(bit % 8)  {
					case 7:
						if((*ptr & (1<<(bit-15))) && !(*ptr & (1<<bit)) &&
							(*ptr & (1<<(bit-1))) )
						{	if(85 == (85 & locale_ptr(ptr,bit)))
							{	word = (ptr - lraster[0]) % raster_wincrement;
								x = (word << 5) | (7 & ~bit) | (bit & 24);
								y = (ptr - lraster[0]) / raster_wincrement;
								pixel(x,y,1);
								count++;
							}
						}
						break;
					case 0:
						if((*ptr & (1<<(bit+1))) && !(*ptr & (1<<bit)) &&
							(*ptr & (1<<(bit+15))) )
						{	if(85 == (85 & locale_ptr(ptr,bit)))
							{	word = (ptr - lraster[0]) % raster_wincrement;
								x = (word << 5) | (7 & ~bit) | (bit & 24);
								y = (ptr - lraster[0]) / raster_wincrement;
								pixel(x,y,1);
								count++;
							}
						}
						break;
					default:
						if((*ptr & (1<<(bit+1))) && !(*ptr & (1<<bit)) &&
							(*ptr & (1<<(bit-1))) )
						{	if(85 == (85 & locale_ptr(ptr,bit)))
							{	word = (ptr - lraster[0]) % raster_wincrement;
								x = (word << 5) | (7 & ~bit) | (bit & 24);
								y = (ptr - lraster[0]) / raster_wincrement;
								pixel(x,y,1);
								count++;
							}
						}
						break;
					}
				}
			}
		}
	}
	sprintf(string," done.   %d holes filled.",count);
	append_msg(string);

		/**************** return ****************************************/
	return(0);
}
