/*******************************************************************************
*    frame_map     (see manual, Command Ref., User Functions)        <fc>      *
*        Description:  Draws lines from SW to NW to NE to SE                   *
*        raster corners, thereby creating a frame, or border,                  *
*        aound the active map.  The frame may be projected on                  *
*        to a geodetic reference.  The frame may be erased.                    *
*        Parameters:  Action type (erase/draw) and projection                  *
*				     (raster/geodetic).                                        *
*                                                                              *
*        Returns 0 upon normal end-of-job (EOJ), else 1.                       *
*        C. Jonson 12/18/89                                                    *
*******************************************************************************/
#include "../../incl/globals.h"
#include "../../incl/in.h"

long frame_map( draw_frame, geo_frame)
	long draw_frame;            /* 0 is erase; 1 is draw       */
	long geo_frame;             /* 0 is raster; 1 is geodetic  */
{
	long ret,  i;

/*** Make sure map is registered before framing it.  ***/
	if( geo_frame )				/* True is geodetic */
	{	if( !registered(1) )            /* Check for geodetic registration */
		{	message("Register geodetics before framing map!  Bye.");
			return(1);
		}
	}
	else
	{	if( !registered(0) )            /* Check for geodetic registration */
		{	message("Register raster before framing map!  Bye.");
			return(1);
		}
	}

/*** Use the level_dup user function to frame the map, or use ***/
/*** level_clear to erase a map.                              ***/
	if( draw_frame )
	{	if( geo_frame )
			ret = add_rm_frame(32, 1);
		else
			ret = add_rm_frame(33, 1);
	}
	else
	{	if( geo_frame )
			ret = add_rm_frame(32, 0);
		else
			ret = add_rm_frame(33, 0);
	}
	
	return(ret);
}
