/*******************************************************************************
*	ginput		(see manual, Command Ref., User Function)	<fc>   *
*			Resets the active graphics graphics input device.      *
*                                                                              *
*			Returns zero upon normal end-of-job, 1 on failure.     *
*******************************************************************************/
/* Modified 5/92 by DBS@CWU for X11 support */

#include "../../incl/globals.h"
#include "../../incl/in.h"

long ginput(dev, abs_mode, debug)
	long abs_mode, dev, debug;  
{
/**  Locals  **/
	char string[256];

	double rms;

	long input_code;
	long new_dev;


	switch( dev )
	{
		case 3:		/*  Mouse  */
#ifndef LTP_X11   /***** here is the non-X11 code *****/
			if(!(strcmp(config.mouse_type,"ltsmse")))
				new_dev = LTSMSE;
			else if(!(strcmp(config.mouse_type,"mssmse")))
				new_dev = MSSMSE;
			else if(!(strcmp(config.mouse_type,"attbmse")))
				new_dev = ATTBMSE;
#else   /***** here is the X11 code *****/
			if(!(strcmp(config.mouse_type,"x11mse")))
			        new_dev = X11MSE;
#endif
			else
			{	message("config file has illegal mouse type:  ");
				append_msg(config.mouse_type);
				return(1);
			}
			break;
		case 1:		/*  Dig1   */
			if( abs_mode )
				new_dev = DIG1ABS;
			else
				new_dev = DIG1REL;
			break;
		case 2:		/*  Dig2   */ 
			if( abs_mode )
				new_dev = DIG2ABS;
			else
				new_dev = DIG2REL;
			break;
	}

		/***** see if registration is within tolerance ****/
	if((new_dev == DIG1ABS)  ||  (new_dev == DIG2ABS))
	{	get_toler();
		if(tolerance > 0.0)
		{		/*** check for digitizer registration ***/
			if(new_dev == DIG1ABS)
			{	if(!registered(4))
				{	message("you cannot go into absolute mode until the digitizer is registered.");
					return(1);
				}
				input_code = 4;
			}
			else
			{	if(!registered(5))
				{	message("you cannot go into absolute mode until the digitizer is registered.");
					return(1);
				}
				input_code = 5;
			}

				/*** check for utm registration ***/
			if(!registered(2))
			{	message("you cannot go into absolute mode until geodetics are registered.");
				return(1);
			}

				/*** check for utm registration ***/
			if(!registered(0))
			{	message("you cannot go into absolute mode until the raster is registered.");
				return(1);
			}

				/*** get rms between digitizer & utm ***/
			rms = corner_rms(input_code,2);
			if(rms > 0.0)
			{		/*** convert rms to feet ****/
				rms = rms * 3937.0 / 1200.0;

					/*** compair with tolerance ***/
					/*** bomb out if tolerance not met. ***/
				if(rms > tolerance)
				{	sprintf(string,"rms error (%.2f ft.) (dig-utm) exceeds tolerance (%.2f ft.)",rms,tolerance);
					message(string);
					message("ginput dig absolute NOT ALLOWED until digitizer is registered within tolerance.");
					return(1);
				}
			}
		}
	}

	ginput_reset();    /**  Close previous graphics dev  **/
	
	if(ginput_setup(new_dev))
	{	message("ginput failed, you must use the keyboard.");
		return(1);
	}
	else
	{	if(debug)
			dig_debug();
	}

#ifndef LTP_X11  /***** use this for non-X11 environment *****/
		/** kick the logitech serial mouse (c-9) twice ***/
	if(new_dev == LTSMSE)
	{		/*** reset mouse **/
		ginput_reset();    /**  Close previous graphics dev  **/
			/*** setup_mouse again ***/
		if(ginput_setup(new_dev))
		{	message("ginput failed, you must use the keyboard.");
			return(1);
		}
	}
#endif

	return(0);
}
