/*******************************************************************************
*	goto_ltp    (see manual, Command Ref., User Function)	<fc>               *
*			repositions the cursore and screen display according to the        *
*			parameters:                                                        *
*                                                                              *
*           Parameters:   long gtype:    0 = object , arg1 = object number     *
*                                        1 = serial,  arg1 = serial number     *
*                                        2 = coords,  arg1=x, arg2=y           *
*                                        3 = error log, arg1:  -2 = last record*
*                                                              -1 = next record*
*                                                              >=0 = record #  *
*                         double arg1,arg2:   as stated above.                 *
*                                                                              *
*           Updates the map global job_stats[75]                               *
*                                                                              *
*           Returns 0 on successful completion, 1 on failure.                  *
*                                                                              *
*           J.Dabritz    8/19/89                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <math.h>

long goto_ltp(gtype,arg1,arg2)
	long gtype;
	double arg1, arg2;
{	
	ERRORS error;
	OBJECTS object;

	long x,y;
	long rec, serial;
	long margin;

	double tx, ty;

	char string[256];


		/************ check for map **********************************/
	if(!job_stats[0])
	{	message("cannot goto errors until a map is acquired.");
		return(1);
	}

		/************ find coords & build string according to type ***/
	switch(gtype)  {
	case 0:             /****  record  ****/
		rec = arg1;
		if((rec < 0)  ||  (rec >= job_stats[10]))
		{	message("record number is out of range for this map.");
			return(1);
		}
		if(get_objects(&object,rec))
			return(1);
		x = object.f4[0] + 0.5;
		y = object.f4[1] + 0.5;
		sprintf(string,"object %d.",rec);
		break;
	case 1:             /****  serial  ****/
		serial = arg1;
		for(rec=0; rec < job_stats[10]; rec++)
		{	if(get_objects(&object,rec))
				return(1);
			if(serial == object.i12[1])
				break;
		}
		if(serial != object.i12[1])
		{	sprintf(string,"No object with serial number %d exists on this map.",serial);
			message(string);
			return(1);
		}
		x = object.f4[0] + 0.5;
		y = object.f4[1] + 0.5;
		sprintf(string,"object %d,  serial number %d",rec,serial);
		break;
	case 2:             /****  coords  ****/
			/*** convert to pixel coords ***/
		tx = arg1;
		ty = arg2;
		if(disp_coord_type == 1)   /*** lat/lon ***/
			tx = -fabs(tx);
		if(disp_coord_type > 0)
			transform(&tx,&ty,disp_coord_type,0);
		x = tx + 0.5;
		y = ty + 0.5;
		sprintf(string,"at location %f, %f",arg1,arg2);
		break;
	case 3:             /****  errors  ****/
		if(arg1 == -2.0)             /**  -  **/
			rec = job_stats[75] - 2;
		else if(arg1 == -1.0)             /**  +  **/
			rec = job_stats[75];
		else if(arg1 >= 0.0)              /**  errors file record number  **/
			rec = arg1;
		else
		{	message("Error log record number out of range.");
			return(1);
		}

		if(rec < 0)
		{	message("Can't read a negative errors log file record number.");
			return(1);
		}
		if(rec >= job_stats[14])
		{	message("Can't go beyond the end of the errors file log.");
			return(1);
		}
		if(get_errors(&error,rec))
			return(1);
		job_stats[75] = rec + 1;
		x = error.x  + 0.5;
		y = error.y  + 0.5;
		if((error.type >= 0)  &&  (error.type <=31))
			sprintf(string,"member of logged set %d",error.type);
		else
		{	switch(error.type)   {
			case 36:            /** hole **/
				sprintf(string,"logged hole location,  error %d of %d",job_stats[75],job_stats[14]);
				break;
			case 37:            /** spur end **/
				sprintf(string,"logged spur end location,  error %d of %d",job_stats[75],job_stats[14]);
				break;
			case 33:            /** isolated point **/
				sprintf(string,"logged point location,  error %d of %d",job_stats[75],job_stats[14]);
				break;
			case 34:            /** junction       **/
				sprintf(string,"logged junction location,  error %d of %d",job_stats[75],job_stats[14]);
				break;
			default:
				sprintf(string,"logged location,  error %d of %d",job_stats[75],job_stats[14]);
				break;
			}
		}
		break;
	default:
		message("goto, bad parameter, no action taken");
		return(1);
	}


		/************ move window & cursor according to parameters ***/
		/*** move window to indicated locatino **/
	margin = (rwindowymax - rwindowymin) / 12;
	if((rwindowxmin > 0) || (rwindowxmax < job_stats[1]) ||
      (rwindowymin > 0) || (rwindowymax < job_stats[2]))
	{	if(( x < rwindowxmin + margin)  ||  (x > rwindowxmax - margin)  ||
	   	( y < rwindowymin + margin)  ||  (y > rwindowymax - margin))
			view_ltp(x,y,resolution_code);
	}

		/*** if not in ginput dig absolute mode, move cursor too.  ***/
	if((gr_dev != DIG1ABS)  &&  (gr_dev != DIG2ABS))
	{	ginfo.tile_num = 0;
		ginfo.tcoord[0] = x;
		ginfo.tcoord[1] = y;
		rstr_to_scrn(&x,&y);
		ginfo.scoord[0] = x;
		ginfo.scoord[1] = y;
		cursor(x,y);
	}
		/********* display message ****/
	message(string);

		/************ return *****************************************/
	return(0);
}
