/*******************************************************************************
*       grow_spurs      (see manual, Command Ref., User Functions)     <fc>    *
*               Description:  locates all spur ends and points on the raster   *
*               and attempts to close them using bridge_gaps logic.            *
*                                                                              *
*               Parameters:  long length;  number of pixels to extend spur     *
*                                    into non-spur object.  Close gaps between *
*                                    2 spurs up to twice this length.          *
*                                                                              *
*               Globals changed:  lraster (craster)                            *
*                                                                              *
*               Returns zero upon normal end-of-job (EOJ).                     *
*               J.Dabritz   6/29/89                                            *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long grow_spurs(length)
	long length;
{

	static long bit[32] = {7, 6, 5, 4, 3, 2, 1, 0, 15, 14, 13, 12, 11, 10, 9, 8,
       23, 22, 21, 20, 19, 18, 17, 16, 31, 30, 29, 28, 27, 26, 25, 24 };
	long count;
	long x,y;
	long word;
	long key;
	long i;
	unsigned long *ptr, *end_ptr;
	long spur[2][2];

	char string[STRLIST];


		/**************** check for loaded map **************************/
	if(!job_stats[0])
		return(0);

		/************* check for VIEW ONLY raster **********************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}

append_msg(" . . closing gaps . . ");

		/**************** erase a 1 pixel column on rt edge of raster ***/
		/**************** (to avoid wrapping on raster in locale_ptr() **/
	erase_rect(job_stats[1],0,job_stats[1],job_stats[2]);
	
		/****** double input length for bridge_gaps logic ***/
	length *= 2;

		/****** find spurs & fix *****/
	count = 0;
	end_ptr = lraster[job_stats[2]] + raster_wincrement;
	for(ptr = lraster[0]; ptr < end_ptr; ptr++)  /* search bottom to top */
	{	if(*ptr)
		{	for(i = 0; i < 32; i++)
			{	if(*ptr & (1<<bit[i]))
				{	key = tlocale[locale_ptr(ptr,bit[i])];    /* see tables.h */
						/****** if this is a spur end . . . ********/
					if(key & (3 << 8))     /* bit 9 = spur, bit 8 = point  */
					{		/*** find xy coords of spur ***/
						word = (ptr - lraster[0]) % raster_wincrement;
						x = (word << 5) | i;
						y = (ptr - lraster[0]) / raster_wincrement;
							/*** try a bridge_gap at this location ***/
							/*** find if 2 endpoints are within search radius **/
						if(find_gap(x,y,spur,length,0))
						{		/*** draw line between endpoints ***/
							set_undo(5,0,0,spur[0][0],spur[0][1],spur[1][0],spur[1][1]);
							line_diag(spur[0][0],spur[0][1],spur[1][0],spur[1][1],1);
							count++;
							if(key & (1 << 8)) /** for points, bridge twice **/
								i--;
						}
					}
				}
			}
		}
	}

	sprintf(string," done.   %d gaps closed.",count);
	append_msg(string);

		/**************** return ****************************************/
	return(0);
}
