/*******************************************************************************
*       impasn          (see manual, Command Ref., User Functions)      <fc>   *
*            This program imports a strip type runlength encoded raster        *
*        ascii file which has been produced by the USFS  GRAMS program.        *
*        All vector type files, (map and import levels) are erased as the      *
*        the new raster forms a new coordinate basis for all files.            *
*                                                                              *
*        The asn format:  (ascii, strip rle)                                   *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   10/14/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>


long impasn( fname )
	char *fname;
{
	char actual[256];
	char string[256];
	unsigned char *ptr;

	int fd;

	long i,h,n,w;
	long width, width0, strip_width;
	long height, height0;
	long size;
	long x,y,x0,xmax,dx;
	long strip;
	long value;
	long wrd,bit;
	long raster_size;


	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

	message("import asn, setting up variables.");
		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import asn, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open input file. aborting.");
		return(1);
	}

		/**** set display coordinate type to 0 (raster pixels) ******/
	disp_coord_type = 0;

		/************* read 4 header ascii lines:              *********/
		/*************  1. file (raster) width
						2. file (raster) height
						3. (raster) strip width
						4. plotter type code (unused) of scanner  ********/
	reada(string,254,fd);             /*  1.  */
	height = 0;
	sscanf(string,"%ld",&height);
	height0 = height;

	reada(string,254,fd);             /*  2.  */
	width = 0;
	sscanf(string,"%ld",&width);
	width0 = width;

	reada(string,254,fd);             /*  3.  */
	strip_width = 0;
	sscanf(string,"%ld",&strip_width);

	reada(string,254,fd);             /*  4.  */
	/** ignore **/

		/****** make width an integral multiple of 32 **/
	if(width % 32)
	{	n = width / 32 + 1;
		width = n * 32;
	}


	message("resetting map vairables & allocating space for new raster.");
		/************* reset job_stats *********************************/
	job_stats[1] = width - 1;
	job_stats[2] = height - 1;
	for(i=3; i<=7; i++)
		job_stats[i] = 0;
	for(i=10; i<=18; i++)
	{	if((i==13)  ||  (i==15)  ||  (i==16))
			job_stats[i] = (job_stats[i])  ?    1 : 0;
		else
			job_stats[i] = 0;
	}
	job_stats[45]++;                          /** asn import flag **/
	job_stats[23] = job_stats[45];
	job_stats[49] = 0;
	job_stats[75] = 0;


		/********** free old space *************************************/
	free_ltp(rasterm.malloc);
	rasterm.malloc = (char *) 0;

		/***********  calculate sizes **************/
	h = job_stats[2] + 1;             /* height of raster in rows  */
	w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */
	raster_size = h*w;                /* size of raster in bytes   */
	size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster/r */

		/***********  allocate memory **************/
	rasterm.malloc = (char *) calloc_ltp(size,1);
	if(rasterm.malloc == (char *) 0)
	{	message("  <<  cannot get enough RAM for raster.  You must re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);

	}


		/********  calculate & set global pointers to raster ****************/
	rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
	rasterm.recn = rasterm.rec0 + raster_size;
	craster = (unsigned char **)(rasterm.malloc);
	lraster = (unsigned long **) (rasterm.malloc + 4*h);
	for(i=0; i<h; i++)
	{	craster[i] = (unsigned char *)rasterm.rec0 + w*i;
		lraster[i] = (unsigned long *) (craster[i]);
	}


		/************* read asn file one scan strip at a time  ********/
	for(x0 = 0, strip = 1; x0 < width0 ; x0 += strip_width, strip++)
	{		/*********inform user **************/
		sprintf(string,"impasn, reading in scan strip %d",strip);
		message(string);

			/************ read & decode 1 scan strip ****************/
		for( y=0; y < height; y++)
		{		/*** set x0 *************************************/
			x = x0;
			xmax = x0 + strip_width;

				/*** read ascii line ****************************/
			reada(string,253,fd);
			if( *string == 121)
				break;
			
				/*** decode ascii line **************************/
			ptr = (unsigned char *) string;
			for(value=0; *ptr; ptr++,  value ^= 1)
			{	if((*ptr) > 13)
				{	dx = *ptr & 127;
					if(value)
					{
						for( ; dx && (x < xmax) ; x++, dx--)
						{	wrd = x >> 5;              /* find 32 bit word */
							bit = (7 & ~x) | (x & 24); /* find bit within word*/
							lraster[y][wrd] |= (1 << bit);
						}
					}
					else
						x += dx;
				}
			}
		}
		if(*string != 121)
			reada(string,253,fd);
			
	}
			

		/************* close file  *************************************/
	closef_ptr(&fd);


		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);



		/* create index_arary in ram            */
		/* free existing array space */
	free_ltp(index_arraym.malloc);
	index_arraym.malloc = (char *) 0;
		/***********  calculate sizes **************/
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = (h*w + 4*h);               /* size to malloc            */
		/***********  allocate memory **************/
	index_arraym.malloc = (char *) calloc_ltp(size,1);
	if(!index_arraym.malloc)
	{	message("  <<  cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}
		/********  calculate & set global pointers to index_array ***********/
	index_arraym.rec0 = index_arraym.malloc;
	index_arraym.recn = (char *)(index_arraym.rec0 + h*w);
	index_array = (long **)index_arraym.recn;
	for(i=0; i<h; i++)
		index_array[i] = (long *) (index_arraym.malloc + w*i);

		/************* un-register raster ******************************/
	for(i=0; i<4; i++)
	{	corner[0][i][0] = 0.0;
		corner[0][i][1] = 0.0;
	}

		/************* write diary record ******************************/
	sprintf(string,"imported asn file %s  (%d x %d)",fname,job_stats[2]+1,job_stats[1]+1);
	put_diary(string);

		/************* redisplay map ***********************************/
		/*  set the raster_wincrement and raster_bincrement globals  */
	raster_wincrement = (job_stats[1] + 1) >> 5;
	raster_bincrement = (job_stats[1] + 1) >> 3;
		/** recompute view[] global ******/
	for(i=0; i<4; i++)
	{	view[i].lower_leftx = 0;
		view[i].lower_lefty = 0;
	}
	view[0].zoom_factor = mind(2,  (double) 1.0, 
			(double) (400.0/(maxl(2,job_stats[1]+1,job_stats[2]+1))));
	view[1].zoom_factor = 0.25;
	view[2].zoom_factor = 1.0;
	view[3].zoom_factor = 8.0;


	message("building new map display.");
	redisp_map();


		/*** reset undo array    **********/
	for(i=0; i<32; i++)
		undo_array[i].mode = 0;


		/************* create rasterb **********************************/
	brstr_create();

		/************* return ******************************************/
	return(0);
}
