/*******************************************************************************
*       impcell        (see manual, Command Ref., User Functions)      <fc>   *
*            This program imports a cell formatted ascii file.                *
*        The old raster is replaced by the new raster, which may be a          **        different size.  All vector type files, (map and import levels)       *
*        are erased as the the new raster forms a new coordinate basis         *
*        for all files.                                                        *
*                                                                              *
*            Parameters:   char *fname       the name of the cell file to      *
*                                            import.                           *
*                          long n            ratio of import file chars        *
*                                            to cells.                         *
*                          long a            attribute code:  1=get objects,   *
*                                            3= get objects & attributes.      *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist, raster, mini-raster, attributes, errors,        *
*                 register                                                     *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   5/31/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

int fd, fd2;
char *malloc_pt;

long impcellabort();

long impcell( fname, n, a )
	char *fname;
	long n,a;
{
	OBJECTS object;

	char actual[256];
	char string[256];
	char *ptr;
	char new_file[256];
	char raster3[256];
	char *top_row;
	char *cntr_row;
	char *end_cntr;
	char *input_str[2];
	char *new_ptr, *old_ptr;
	

	long width;                  /* width of 1 raster row (pixels) */
	long h,i,j,w;
	long size,size1;
	long rows;                   /* number of raster rows */
	long x;
	long byte,bit;
	long offset;
	long stringlength, cellwidth;
	long new, old;
	long eol;
	long filerow,filechar;

	float floatx,floaty;
	
	double coord_pairs[4][2];
	double ffilerow,ffilechar;


		/************* initialize fd & malloc globals ******************/
	fd = -1;
	fd2 = -1;
	malloc_pt = (char *) 0;

		/************* check input parameters **************************/
	if((n < 1)  ||  (n > 30))
	{	message("illegal number of characters per cell.  aborting.");
		return(1);
	}

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

	message("import cell, setting up variables.");
		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import cell, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open input file, abort, location 2.");
		return(1);
	}


		/**** set display coordinate type to 0 (raster pixels) ******/
	disp_coord_type = 0;


		/*************** free current map malloc space *****************/
	message("Updating temporary map file.");
	update_temp();
	free_malloc();


	message("Decoding file width.");
		/************* find file width *********************************/
	*string = (char) 10;
	for(i=0; ; i++)
	{	if(readb(string,1,1,fd) == -1)
			break;
			/*** look for carraige return or line feed ****/
		if((*string == (char) 10)  ||  (*string == (char) 13))
			break;
	}

	stringlength = i;
	cellwidth = i / n;
	width =  9 + 2 * cellwidth ;   /** 2 pixels per cell + 1 + 4 pixel buffer **/


		/****** make width an integral multiple of 32 **/
	if(width % 32)
	{	i = width / 32 + 1;
		width = i * 32;
	}


	message("Creating temporary files.");
		/************* create new raster file & asgn pointer ***********/
	sprintf(new_file,"%s/raster3",temp_map_path);
	purgef(new_file);
	sprintf(new_file,"%s/raster2",temp_map_path);
	purgef(new_file);
	if(-1 == creatf(new_file))
	{	message("Cannot create new raster file. aborting.");
		impcellabort(4);
		return(1);
	}
	fd2 = openf(new_file,WRITE);
	if(fd2 < 0)
	{	message("Cannot open new raster file. aborting.");
		impcellabort(5);
		return(1);
	}
	
		/************* malloc enough space for a raster row ************/
	size = width / 8;
	size1 = stringlength + 15;
	malloc_pt = (char *) malloc_ltp(2* size + 2 * size1);
	if(malloc_pt == (char *) 0)
	{	message("Not enough RAM available to import cell.  Aborting.");
		impcellabort(6);
		return(1);
	}
	top_row = malloc_pt;
	cntr_row = malloc_pt + size;
	end_cntr = malloc_pt + 2 * size;
	input_str[0] = malloc_pt + 2 * size + 4;
	input_str[1] = malloc_pt + 2 * size + 4 + size1;
	new = 0;
	old = 1;

		/************* store 4 blank rows *****************************/
			/****** clear raster rows **************************/
	for(ptr=top_row; ptr<end_cntr; ptr++)
		*ptr = (char) 0;
	for(rows=0; rows<4; rows+=2)
	{	if(writeb(malloc_pt,size,2,fd2) <= 0)
		{	message("write failed when storing a row of raster data.  Aborting.");
			impcellabort(7);
			return(1);
		}
	}

		/*** reset file pointer to beginning ***/
	if(0 > movefp(fd,0,0))
	{	impcellabort(20);
		message("Move file pointer failed.  Aborting.");
		return(1);
	}

		/************* read cell file *********************************/
	for( ; reada(input_str[new],stringlength+2,fd); new^=1,old^=1)
	{		/****** clear raster rows **************************/
		for(ptr=top_row; ptr<end_cntr; ptr++)
			*ptr = (char) 0;

		if(!(rows % 64))
		{	sprintf(string,"Importing cell row %d.",(rows - 4) / 2);
			message(string);
		}

		new_ptr=input_str[new];
		old_ptr=input_str[old];

			/****** check for differences above & left ***/
		for(i=0, x=5; i < stringlength; i+=n, x+=2, new_ptr+=n, old_ptr+=n)
		{	if(strncmp(new_ptr,old_ptr,n))   /** difference above ******/
			{	for(j=x-1; j<x+2; j++)
				{	byte = j >> 3;
					bit = 7 & (~j);
					top_row[byte] |= (char) 1 << bit;
				}
			}
			if(strncmp(new_ptr,new_ptr-n,n))   /** difference left  ******/
			{	byte = (x-1) >> 3;
				bit = 7 & (~(x-1));
				top_row[byte] |= (char) 1 << bit;
				cntr_row[byte] |= (char) 1 << bit;
			}
		}
		
			/****** store row rows  ****************************/
		if(writeb(top_row,size,1,fd2) <= 0)
		{	message("write failed when storing a row of raster data.  Aborting.");
			impcellabort(8);
			return(1);
		}
		if(writeb(cntr_row,size,1,fd2) <= 0)
		{	message("write failed when storing a row of raster data.  Aborting.");
			impcellabort(8);
			return(1);
		}
		rows+=2;

	}
		/************* store 5 blank rows *****************************/
			/****** clear raster rows **************************/
	for(ptr=top_row; ptr<end_cntr; ptr++)
		*ptr = (char) 0;
	for(i=0; i<5; i++, rows++)
	{	if(writeb(malloc_pt,size,1,fd2) <= 0)
		{	message("write failed when storing a row of raster data.  Aborting.");
			impcellabort(9);
			return(1);
		}
	}


		/************* close files *************************************/
	closef_ptr(&fd);
	closef_ptr(&fd2);

		/************* create final raster file ************************/
	message("impcell, re-reflecting new raster & storing in raster file.");
	sprintf(raster3,"%s/raster3",temp_map_path);
	if(-1 == creatf(raster3))
	{	message("Cannot create raster3 raster file. aborting.");
		impcellabort(10);
		return(1);
	}
	fd = openf(raster3,WRITE);
	if(fd < 0)
	{	message("Cannot open raster3 raster file. aborting.");
		impcellabort(11);
		return(1);
	}
	
		/************* open new_file (upside down raster) **************/
	fd2 = openf(new_file,READ);
	if(fd2 < 0)
	{	message("Cannot open new_file raster file. aborting.");
		impcellabort(12);
		return(1);
	}
	
		/************* read upside down raster & store right side up ***/
	for(offset = (rows - 1) * size; offset >= 0; offset -= size)
	{		/******************** position file read pointer ***************/
		if(0 > movefp(fd2,offset,0))
		{	impcellabort(13);
			message("Move file pointer failed while reflecting cell file.  Aborting.");
			return(1);
		}

			/******************** read  record from file 1 ******************/
		if(0 > readb(malloc_pt, size, 1, fd2))
		{	impcellabort(14);
			message("Read failed while reflecting cell file.  Aborting.");
			return(1);
		}

			/******************** store record in file 2 ********************/
		if(0 > writeb(malloc_pt, size, 1, fd))
		{	impcellabort(15);
			message("Write failed while reflecting cell file.  Aborting.");
			return(1);
		}
	}

		/************* close files & free malloc space *****************/
	closef_ptr(&fd);
	closef_ptr(&fd2);
	purgef(new_file);

		/************* move raster file ********************************/
	sprintf(new_file,"%s/raster",temp_map_path);
	if(move_file(raster3, new_file))
	{	message("Move failed when attempting to move new raster into old file, Aborting.");
		impcellabort(16);
		return(1);
	}

	message("resetting map vairables & loading new raster into display RAM.");
		/************* reset job_stats *********************************/
	job_stats[1] = size * 8 - 1;
	job_stats[2] = rows - 1;
	for(i=3; i<=7; i++)
		job_stats[i] = 0;
	for(i=10; i<=18; i++)
	{	if((i==13)  ||  (i==15)  ||  (i==16))
			job_stats[i] = (job_stats[i])  ?    1 : 0;
		else
			job_stats[i] = 0;
	}
	job_stats[45]++;                   /** cell import flag **/
	job_stats[29] = job_stats[45];
	job_stats[49] = 0;
	job_stats[75] = 0;



/********* fix 2 *************************************************************/
		/* store new sized index_array file     */
		/***********  calculate sizes **************/
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = (h*w);                     /* size to calloc            */
		/***********  allocate memory **************/
	index_arraym.malloc = (char *) calloc_ltp(1,size);
	if(index_arraym.malloc == (char *) 0)
	{	message("  <<  cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}

		/***********  write index_array to file *********************/
	sprintf(string,"%s/index_array",temp_map_path);
	fd = openf(string,WRITE);
	if(0 > fd)
	{	message("  <<  cannot open index_array map file.");
		return(0);
	}
	if(0 > writeb(index_arraym.malloc,size,1,fd))
	{	message("  <<  cannot write index_array map file.");
		closef_ptr(&fd);
		return(0);
	}
	closef_ptr(&fd);

	free_ltp(index_arraym.malloc);
	index_arraym.malloc = (char *) 0;

		/***********  write job_stats to file ***********************/
	sprintf(string,"%s/job_stats",temp_map_path);
	if(put_jobstats(string))
		{	message("Cannot write job_stats to temp file.  Aborting.");
			return(1);
		}

		/************* re-acquire temp file ****************************/
	if(reacquire())
	{	message("You must re-acquire the map & start over.");
		job_stats[0] = 0;
		return(1);
	}


/********* end fix 2 *********************************************************/


		/************* un-register raster ******************************/
	for(i=0; i<4; i++)
	{	corner[0][i][0] = 0.0;
		corner[0][i][1] = 0.0;
	}

		/***** register raster at cell file corners ***/
	coord_pairs[0][0] = 4.0;                /** sw **/
	coord_pairs[0][1] = 4.0;

	coord_pairs[1][0] = 4.0;                /** nw **/
	coord_pairs[1][1] = job_stats[2] - 4.0;

	coord_pairs[2][0] = cellwidth * 2 + 4.0;    /** ne **/
	coord_pairs[2][1] = job_stats[2] - 4.0;

	coord_pairs[3][0] = cellwidth * 2 + 4.0;    /** se **/
	coord_pairs[3][1] = 4.0;

	register_ltp(0, 0L, coord_pairs, 4L, (char) 0);

		/*** display map raster   ***/
/*	graphics(0,1,0,0,0,0,0,0,0);   */

		/***** frame map ****/
	if(frame_map(1,0))
	{	impcellabort(17);
		message("Cannot frame map.  aborting.");
		return(1);
	}

		/***** set maptype to polygon ****/
	if(!header_ltp(0,"maptype","polygon"))
	{	impcellabort(18);
		message("Cannot set map type to polygon.  aborting.");
		return(1);
	}

		/************* write diary record ******************************/
	sprintf(string,"imported cell file %s  (%d x %d)",fname,job_stats[2]+1,job_stats[1]+1);
	put_diary(string);


		/******* re-open file                         *************/
	if(a > 1)
	{	fd = openf(actual,READ);
		if(fd <= 0)
			message("Cannot open input file. registration and/or attribute data not read.");
		else
		{
				/***** assemble map ***/
			if(assemble(1))
			{	impcellabort(19);
				message("Cannot assemble map.  aborting.");
				return(1);
			}

				/*** display vectors & highlight objects not attributed ***/
/*			graphics(0,0,1,0,0,(1 << 26),0,0,1);    */

				/*** blank out old string ****************************/
			for(old_ptr = input_str[old], i=0; i<stringlength; i++, old_ptr++)
				*old_ptr = (char) 0;

				/*** find out if 1 or 2 characters (lf or drlf) are at eol ***/
			if(0 > movefp(fd,stringlength,0))
			{	impcellabort(21);
				message("Move file pointer failed.  Aborting.");
				return(1);
			}
				/**** read 5 bytes at end of first string *****/
			if(5 > readb(string,5,1,fd))
			{	impcellabort(22);
				message("Read file failed.  Aborting.");
				return(1);
			}
				/**** count number of consec control characters ***/
			string[5] = (char) 0;
			for(eol=0; string[eol] < (char) 32; eol++);

				/*** read through all objects attributing interior polys   ***/
			for(i=job_stats[7]; i<job_stats[10]; i++)
			{		/*** get object record ***/
				if(get_objects(&object,i))
				{	impcellabort(23);
					return(1);
				}
					/*** check to see if it is an interior polygon ***/
				if(object.i12[0] == 14)
				{		/*** find it's centroid ***/
					floatx = object.f4[0] + 0.01;
					floaty = object.f4[1] + 0.01;

						/*** calculate file coordinates of centroid ***/
					ffilerow = (job_stats[2] - 5.0 - floaty) / 2.0 + 0.1;
					filerow = ffilerow;
					ffilechar = ((floatx - 5.0) / 2.0 ) * n + 0.1;
					filechar = ffilechar;
					filechar = filechar - (filechar % n);  /* new code 9/19/90 */
					offset = (stringlength + eol) * filerow + filechar;
					
						/*** move file pointer & read attribute ***/
					if(0 > movefp(fd,offset,0))
					{	impcellabort(24);
						message("Move file pointer failed.  Aborting.");
						return(1);
					}
					if(0 > readb(string,n,1,fd))
					{	impcellabort(25);
						message("Read file failed.  Aborting.");
						return(1);
					}
					string[n] = (char) 0;

						/*** assign attribute ***/
					if(asgn_atrbt(i,1,string))
					{	impcellabort(26);
						message("Read file failed.  Aborting.");
						return(1);
					}
				}
			}
		}
			/*** close file ***/
		closef_ptr(&fd);
	}
	else
	{		/************* re-build map display ****************************/
		redisp_map();
		view_ltp(0,0,0);
	}


		/************* free malloc space *****************/
	free_ltp(malloc_pt);
	malloc_pt = (char *) 0;

		/*** reset undo array    **********/
	for(i=0; i<32; i++)
		undo_array[i].mode = 0;


		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);

		/************* create rasterb **********************************/
	brstr_create();

		/************* return ******************************************/
	return(0);
}



/*******************************************************************************
*    impcellabort                                                             *
*        This program closes files and frees malloc space and prints an abort  *
*    message on the screen which identifies the abort location.                *
*    Parameter:                long location                                   *
*    Files & globals changed:  none                                            *
*    Value returned:  long 0;                                                  *
*    J.Dabritz    10/13/89                                                     *
*******************************************************************************/

long impcellabort(location)
	long location;
{
	char string[256];

		/************ close files **************************************/
	if(fd > 0)
		closef_ptr(&fd);
	if(fd2 > 0)
		closef_ptr(&fd2);

		/************ free malloc space ********************************/
	if(malloc_pt != (char *) 0)
	{	free_ltp(malloc_pt);
		malloc_pt = (char *) 0;
	}

		/************ print message ************************************/
	sprintf(string,"Aborting import cell, location %d.",location);
	message(string);


		/************ return *******************************************/
	return(0);
}
