/*******************************************************************************
*   impcff      (see manual, Command Ref., User Functions)  <fc>               *
*                                                                              *
*       This program reads a cff  import/export formatted ascii file and       *
*       stores the data on the specified 'level' (35 - 95)                     *
*                                                                              *
*            Parameters:   char *fname       the name of the dlg3 file to      *
*                                            import.                           *
*                          long level        the level number to import to     *
*                                            (35  - 95  inclusive)             *
*                          long c            coord_type code (same as in the   *
*                                            register file)                    *
*                          char *lname       new name for the level imported   *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist,   (varaibles)  [bunches]                        *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   10/10/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

int fd;
long last_js16, last_js17, last_js18;

long impcffabort();
char *reada();


long impcff( fname, level, c, lname)
char *fname;
long level,c;
char *lname;
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	char actual[256];
	char *ptr;
	char string[256];
	char msg[256];
	char parsed[5][STRLIST];

	double x,y;

	long i,n;
	long length;
	long line;
	long no_verticies;
	long v;
	long otype;
	long int_level;
	long levels[100];


	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check for raster registration *******************/
	if(!registered(0))
	{	message("You must REGISTER the Raster befor importing a vector file.");
		return(1);
	}

		/************* check for geodetic registration *****************/
	if(!registered(c))
	{	message("You must REGISTER the input coordinate system befor importing.");
		return(1);
	}

		/************* check input parameters **************************/
	if((level < 35)  ||  (level > 95))
	{	message("Illegal level number.  Level number must be between 35 and 95.");
		return(1);
	}
	if((c < 0)  ||  (c > 19))
	{	message("Illegal coordinate type code.  Code must be between 0 and 19 inclusive.");
		return(1);
	}


		/******* set up variables *************************************/
	message("import cff, setting up variables.");
	last_js16 = job_stats[16];
	last_js17 = job_stats[17];
	last_js18 = job_stats[18];
	fd = -1;


		/************* output v_attribute record 0 dummy record ********/
	if(!job_stats[16])
	{	attr.next = 0;
		attr.string[0] = 0;
		if(put_v_attr(&attr,0))
		{	message("import cff, abort, location 0.");
			return(1);
		}
	}

		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import cff, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open (1) file. aborting.");
		return(1);
	}
	
	for(i=0; i<100; i++)
		levels[i] = 0;

		/************* read all records ********************************/
	for(line=0 ; reada(string,STRLIST,fd) > (char *) 0; line++)   /*  header */
	{
		if(!(line %25))
		{	sprintf(msg,"Reading & storing cff data, cff object %d.",line+1);
			message(msg);
		}

			/***** check for adequate header line length ******/
		if((length = strlen(string)) >= 29)
		{	/*** decode header ***/
				/*************** get intergraph level nmber, if any ******/
			if(length >= 33)
			{	ptr = string + 29;
				if(sscanf(ptr,"%ld",&int_level))
					int_level = (abs(int_level)) % 100;
				else
					int_level = 1;
			}
			else
				int_level = 1;

			if(int_level == 0)
				int_level = 1;
			for(i=0; i<99  &&  levels[i]  &&  (levels[i] != int_level); i++);
			if(!levels[i])
			{	levels[i] = int_level;
				if((level + i) <= 95)
					sprintf(level_names[level + i],"cff import from %s, intgrph level %d",fname,int_level);
			}
			if((level + i) > 95)
				i = 95 - level;
			otype = 10 * (level+i);

				/*************** get number of points ********************/
			string[25] = (char) 0;
			ptr = string + 21;
			sscanf(ptr,"%ld",&no_verticies);
			if(no_verticies < 1)
			{	message("import cff, negative number of verticies.  Aborting");
				impcffabort(1);
				return(1);
			}

				/*************** get feature codes ***********************/
			ptr = string;
			ptr[21] = (char) 0;
			trim(ptr);
			v_object.atr = 0;
			n = parse(ptr,delim_buf,parsed,5);
			for(i = n-1 ;   i >= 0   ; i--)
			{	strcpy(attr.string,parsed[i]);
				attr.next = v_object.atr;
				v_object.atr = job_stats[16];
				if(put_v_attr(&attr,job_stats[16]))
				{	impcffabort(2);
					return(1);
				}
			}

			if(no_verticies > 1)
				v_object.type = otype + 1;     /***** segment ******/
			else
				v_object.type = otype;         /***** point   ******/

			v_object.v1 = job_stats[18];

				/************* get verticies ************************/
			for(v = 1; v <= no_verticies; v++)
			{	if(reada(string,STRLIST,fd) == (char *) 0)
				{	message("import cff,  Arrived at end of file unexpectedly, aborting.");
					impcffabort(3);
					return(1);
				}
				string[17] = (char) 0;
				ptr = string + 9;
				sscanf(ptr,"%lf",&y);

				*ptr = 0;
				ptr = string;
				sscanf(ptr,"%lf",&x);
				transform(&x,&y,c,0);
				coordlist.x = x;
				coordlist.y = y;
				if(put_v_coord(&coordlist,job_stats[18]))
				{	impcffabort(5);
					return(1);
				}
			}
			v_object.v2 = job_stats[18] - 1;
			if(put_vobjects(&v_object,job_stats[17]))
			{	impcffabort(4);
				return(1);
			}
		}
		else
			message("Read a non-header line when expecting a header.  Continuing.");
	}


	message("import cff, closing files.");
		/************* close file **************************************/
	closef_ptr(&fd);

		/************* if a copied dos file, purge it ******************/
#ifdef SV32
	/* can't access dos anyway */
#else
	if(((char *) 0) != strchr(fname,':'))     /** a dos file **/
		purgef(actual);                 /** purge the xenix copy **/
#endif

		/************* index v_objects imported ************************/
	if(index_vobj(last_js17))
	{	job_stats[17] = last_js17;
		return(1);
	}

		/************* write diary *************************************/
	sprintf(string,"import cff file %s to level %d.",fname,level);
	put_diary(string);

		/************* set job_stats ***********************************/
	job_stats[45]++;
	job_stats[34] = job_stats[45];

		/************* set level name **********************************/
	if((levels[1] == 0)  &&  (levels[0] == 1))
	{	if(*lname)
		{	lname[STRLIST - 2] = 0;
			sprintf(level_names[level],"%s",lname);
		}
		else
			sprintf(level_names[level],"imported cff file: %s",fname);
	}


		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);

		/*********** change display to add imported level **************/
	if(level < 64)
		disp_i1hglight |= (1 << (level % 32));
	else
		disp_i2hglight |= (1 << (level % 32));

		/*********** re-display map ************************************/
	dispgraphics((long) ((rwindowxmin + rwindowxmax) / 2),(long) ((rwindowymin + rwindowymax) / 2),resolution_code);

		/************* return ******************************************/
	return(0);
}


/*******************************************************************************
*      impcffabort                                                            *
*          This program is a convenient way of closing the input  file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long impcffabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* reset job_stats variables ************************/
	job_stats[16] = last_js16;
	job_stats[17] = last_js17;
	job_stats[18] = last_js18;

		/************* print abort message ******************************/
	sprintf(string,"import cff, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
