/*******************************************************************************
*   impdem      (see manual, Command Ref., User Functions)  <fc>               *
*                                                                              *
*       This program reads a DEM file and import the elevations as ponits      *
*       with elevations as attributes.                                         *
*                                                                              *
*            Parameters:   char *fname       the name of the dem  file to      *
*                                            import.                           *
*                          long level        level number to import into       *
*                          char *lname       level name for input level        *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist,   (varaibles)  [bunches]                        *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   10/4/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

int fd;
long last_js16, last_js17, last_js18;

long impdemabort();
char *reada();


long impdem( fname, level, lname)
char *fname;
long level;
char *lname;
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	char actual[256];
	unsigned char string[1056];
	unsigned char *ptr, *end_ptr;
	char mstring[256];

	long elevation;
	long col_index;
	long c;
	long i;
	long no_cols;
	long new_col, col;
	long no_elevs;
	long local_datum;
	long elevs;

	double col_datum;
	double increment;          /* hight of a dem cell */
	double x,y;
	double begin_x,begin_y;
	double xinout, yinout;

		/************* init file pointer *******************************/
	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check for raster registration *******************/
	if(!registered(0))
	{	message("You must REGISTER the Raster befor importing a dem file.");
		return(1);
	}

		/************* check for geodetic registration *****************/
	if(!registered(2))
	{	message("You must REGISTER geodetics or utm befor importing a dem.");
		return(1);
	}

		/************* check input parameters **************************/
	if((level < 35)  ||  (level > 95))
	{	message("Illegal level number.  Level number must be between 35 and 95.");
		return(1);
	}

		/******* set up variables *************************************/
	message("import dem, setting up variables.");
	last_js16 = job_stats[16];
	last_js17 = job_stats[17];
	last_js18 = job_stats[18];
	fd = -1;


		/************* output v_attribute record 0 dummy record ********/
	if(!job_stats[16])
	{	attr.next = 0;
		attr.string[0] = 0;
		if(put_v_attr(&attr,0))
		{	message("import dem, abort, location 0.");
			return(1);
		}
	}

		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import dem, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open (1) file. aborting.");
		return(1);
	}
	

	v_object.type = 10 * level;

		/************* read file header ********************************/
	if(readb(string,1024,1,fd) > 0)
	{	
			/*---------- find number of columns ***********************/
		no_cols = 0;
		ptr = string + 858;
		string[864] = (char) 0;
		sscanf(ptr,"%ld",&no_cols);
		if(no_cols > 640)
			no_cols = 640;
		
			/*---------- find vertical increment **********************/
		increment = 0.0;
		ptr = string + 828;
		string[840] = (char) 0;
		sscanf(ptr,"%lf",&increment);
		
			/*---------- find coordinate system ***********************/
		c = -1;
		ptr = string + 156;
		string[162] = (char) 0;
		sscanf(ptr,"%ld",&c);
		if((c < 0) || (c > 2))
			c = 1;
		c++;
		
	}

		/************* read columns & store ****************************/
	col_index = 0;
	new_col = 1;
	col = 0;
	end_ptr = string + 1024 - 5;
	for( ;(col<no_cols) && (readb(string,1024,1,fd)>0); )
	{	ptr = string;
		string[1024] = (unsigned char) 0;
			/*--------- if new column, read column header ********/
		if(new_col)
		{
			if(!(col % 10))
			{	sprintf(mstring,"importing dem, col %d  of  %d.",col,no_cols);
				message(mstring);
			}

			ptr = string + 12;
			ptr[6] = (unsigned char) 0;
			if(1 > sscanf(ptr,"%ld",&no_elevs))   /** number of elevs in profile */
				break;
			
			ptr = string + 24;
			ptr[24] = (unsigned char) 0;
			for(i=0; i<24; i++)
			{	if(ptr[i] == (unsigned char) 'D')
					ptr[i] = (unsigned char) 'E';
			}
			if(1 > sscanf(ptr,"%lf",&begin_x))    /** beginning x coord        **/
				break;

			ptr = string + 48;
			*ptr = (unsigned char) 32;
			ptr[24] = (unsigned char) 0;
			for(i=0; i<24; i++)
			{	if(ptr[i] == (unsigned char) 'D')
					ptr[i] = (unsigned char) 'E';
			}
			if(1 > sscanf(ptr,"%lf",&begin_y))    /** beginning y coord        **/
				break;

			ptr = string + 72;
			*ptr = (unsigned char) 32;
			ptr[24] = (unsigned char) 0;
			col_datum = 0.0;
			if(1 > sscanf(ptr,"%lf",&col_datum))   /** local datum for profile **/
				break;
			local_datum = col_datum;

			elevs = 0;
			ptr = string + 144;

			new_col = 0;

			x = begin_x;
			y = begin_y;
		}


			/*--------- read elevations **************************/
		for(; (elevs<no_elevs) && (ptr<end_ptr) ; y += increment, elevs++, ptr+=6)
		{
			sscanf(ptr,"%6ld",&elevation);
			elevation += local_datum;
				/*** convert coordinates & store vector point ***/
			xinout = x;
			yinout = y;
			transform(&xinout,&yinout,c,0);
			if((xinout >= 0.0) && (xinout <= (double) job_stats[1]) &&
			   (yinout >= 0.0) && (yinout <= (double) job_stats[2]))
         {	
				coordlist.x = xinout;
				coordlist.y = yinout;
				if(put_v_coord(&coordlist,job_stats[18]))
				{	impdemabort(5);
					return(1);
				}
	
				v_object.v1 = v_object.v2 = job_stats[18] - 1;
				sprintf(attr.string,"%d",elevation);
				attr.next = 0;
				v_object.atr = job_stats[16];
				if(put_v_attr(&attr,job_stats[16]))
				{	impdemabort(2);
					return(1);
				}
				if(put_vobjects(&v_object,job_stats[17]))
				{	impdemabort(4);
					return(1);
				}
			}
		}

			/*--------- if column done, reset new_col flag *******/
		if(elevs >= no_elevs)
		{	new_col = 1;
			col++;
			col_index = (col_index + 1) % 3;
		}
	}

		/************* close file **************************************/
	closef_ptr(&fd);

		/************* if a copied dos file, purge it ******************/
#ifdef SV32
	/* can't access dos anyway */
#else
		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);
#endif

		/************* index v_objects imported ************************/
	if(index_vobj(last_js17))
	{	job_stats[17] = last_js17;
		return(1);
	}

		/************* write diary *************************************/
	sprintf(string,"import dem file %s to level %d.",fname,level);
	put_diary(string);

		/************* set job_stats ***********************************/
	job_stats[45]++;
	job_stats[38] = job_stats[45];

		/************* set level name **********************************/
	if(*lname)
	{	lname[STRLIST - 2] = 0;
		sprintf(level_names[level],"%s",lname);
	}
	else
		sprintf(level_names[level],"imported dem: %s",fname);

		/*********** change display to add imported level **************/
	if(level < 64)
		disp_i1normal |= (1 << (level % 32));
	else
		disp_i2normal |= (1 << (level % 32));

		/*********** re-display map ************************************/
	dispgraphics((long) ((rwindowxmin + rwindowxmax) / 2),(long) ((rwindowymin + rwindowymax) / 2),resolution_code);

		/************* return ******************************************/
	return(0);
}


/*******************************************************************************
*      impdemabort                                                            *
*          This program is a convenient way of closing the input  file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long impdemabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* reset job_stats variables ************************/
	job_stats[16] = last_js16;
	job_stats[17] = last_js17;
	job_stats[18] = last_js18;

		/************* print abort message ******************************/
	sprintf(string,"import dem, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
