/*******************************************************************************
*   impdigit_asc(see manual, Command Ref., User Functions)  <fc>               *
*                                                                              *
*       This program reads a GRASS digit.ascii  formatted ascii file and       *
*       stores the data on the specified 'level' (35 - 95)                     *
*                                                                              *
*            Parameters:   char *fname       the name of the dlg3 file to      *
*                                            import.                           *
*                          long level        the level number to import to     *
*                                            (35  - 95  inclusive)             *
*                          long c            coord_type code (same as in the   *
*                                            register file)                    *
*                          char *lname       new name for the level imported   *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist,   (varaibles)  [bunches]                        *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   12/29/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

int fd;
long last_js16, last_js17, last_js18;

long impdigiaabort();
char *reada();


long impdigit_asc( fname, level, c, lname)
char *fname;
long level,c;
char *lname;
{
	V_OBJECTS v_object;
	COORD_LIST coordlist;

	char actual[256];
	char *ptr;
	char string[256];
	char msg[256];
	char ftype;

	double x,y;
	double old_ref;
	double old_params[16];

	long line;
	long no_verticies;
	long v;
	long zone, zonehit;
	long i,n;


	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check for raster registration *******************/
	if(!registered(0))
	{	message("You must REGISTER the Raster befor importing a vector file.");
		return(1);
	}

		/************* check input parameters **************************/
	if((level < 35)  ||  (level > 95))
	{	message("Illegal level number.  Level number must be between 35 and 95.");
		return(1);
	}
	if((c < 0)  ||  (c > 19))
	{	message("Illegal coordinate type code.  Code must be between 0 and 19 inclusive.");
		return(1);
	}


		/******* set up variables *************************************/
	message("import digit.ascii, setting up variables.");
	last_js16 = job_stats[16];
	last_js17 = job_stats[17];
	last_js18 = job_stats[18];
	fd = -1;


		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import digit.ascii, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open (1) file. aborting.");
		return(1);
	}
	
			/***** decode header section of digit.ascii file **/
	zone = 0;
	zonehit = 0;
	for( ; ; )
	{	if(!reada(string,100,fd))
		{	message("File ran out befor end of header reached.  aborting.");
			impdigiaabort(1);
			return(1);
		}
		string[99] = 0;
		trim(string);
		lowerc(string);
		if(!strcmp("verti:",string))
			break;
		string[5] = 0;
		if(!strcmp("zone:",string))
		{	ptr = string + 6;
			trim(ptr);
			zonehit = sscanf(ptr,"%ld",&zone);
		}
	}

		/*********** decode utm/state plane zone *****************/

	if(c == 2)         /******* utm coords **************/
	{	if((zone < 1)  ||  (zone > 20) ||  (!zonehit))
		{	message("Illegal zone in header portion of file.  The default utm zone will be used.");
			for(i=0; i<4; i++)
			{	corner[8][i][0] = corner[2][i][0];
				corner[8][i][1] = corner[2][i][1];
			}
			sleep(1);
		}
		else
		{	old_ref = lon_ref;
			lon_ref = -(183.0 - 6.0 * zone);    /** lon at zone center **/
				/*********** load input corner coords into corner[8] **/
			for(i=0; i<4; i++)
				geo_to_utm(corner[1][i][0],corner[1][i][1],&corner[8][i][0],&corner[8][i][1]);            
			lon_ref = old_ref;
		}
	}


	else if(c == 3)    /******* state plane coords ******/
	{	for(i=0; i<16; i++)
			old_params[i] = stpln_params[i];
		if(stpln_data(zone))
		{	for(i=0; i<16; i++)
				stpln_params[i] = old_params[i];
			if(registered(3))
			{	message("Illegal zone in header portion of file.  The default utm zone will be used.");
				for(i=0; i<4; i++)
				{	corner[8][i][0] = corner[3][i][0];
					corner[8][i][1] = corner[3][i][1];
				}
			}
			else
			{	message("Illegal zone in header & no default zone in map.  Import aborted.");
				return(1);
			}
		}
		else
		{	for(i=0; i<4; i++)
				geo_to_stpln(corner[1][i][0],corner[1][i][1],&corner[8][i][0],&corner[8][i][1]);            
			for(i=0; i<16; i++)
				stpln_params[i] = old_params[i];
		}
	}
	else
	{	if(!registered(c))
		{	message("There is no map registration for this coordinate type.  Aborting.");
			impdigiaabort(2);
			return(1);
		}
		for(i=0; i<4; i++)
		{	corner[8][i][0] = corner[c][i][0];
			corner[8][i][1] = corner[c][i][1];
		}
	}


	v_object.type = 10 * level + 1;
		/************* read all records ********************************/
	for(line=0 ; reada(string,STRLIST,fd) > (char *) 0; line++)   /*  header */
	{
		if(!(line %25))
		{	sprintf(msg,"Reading & storing digit.ascii data, digit.ascii object %d.",line+1);
			message(msg);
		}

			/***** check for adequate header line length ******/
		trim(string);
		if(strlen(string) <= 6)
		{		/*** decode header ***/
			n = sscanf(string,"%c %ld",&ftype,&no_verticies);
			if(n < 2)
			{	message("Illegal header line within Arc section of import file.  Aborting.");
				impdigiaabort(3);
				return(1);
			}
			if(no_verticies < 1)
			{	message("import digit.ascii, negative number of verticies.  Aborting");
				impdigiaabort(4);
				return(1);
			}


			v_object.v1 = job_stats[18];
			if(ftype > 'Z')
			{	for(v = 1; v <= no_verticies; v++)
				{	if(reada(string,STRLIST,fd) == (char *) 0)
					{	message("import digit.ascii,  Arrived at end of file unexpectedly, aborting.");
						impdigiaabort(5);
						return(1);
					}
				}
			}
			else
			{	for(v = 1; v <= no_verticies; v++)
				{	if(reada(string,STRLIST,fd) == (char *) 0)
					{	message("import digit.ascii,  Arrived at end of file unexpectedly, aborting.");
						impdigiaabort(6);
						return(1);
					}
	
					string[13] = 0;
					ptr = string + 14;
					sscanf(string,"%lf",&y);
					sscanf(ptr,"%lf",&x);
					transform(&x,&y,8,0);
					coordlist.x = x;
					coordlist.y = y;
					if(put_v_coord(&coordlist,job_stats[18]))
					{	impdigiaabort(7);
						return(1);
					}
				}
				v_object.v2 = job_stats[18] - 1;
				if(put_vobjects(&v_object,job_stats[17]))
				{	impdigiaabort(8);
					return(1);
				}
			}
		}
		else
			message("Read a non-header line when expecting a header.  Continuing.");
	}


	message("import digit.ascii, closing files.");
		/************* close file **************************************/
	closef_ptr(&fd);

		/************* if a copied dos file, purge it ******************/
#ifdef SV32
	/* can't access dos anyway */
#else
		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);
#endif

		/************* index v_objects imported ************************/
	if(index_vobj(last_js17))
	{	job_stats[17] = last_js17;
		return(1);
	}

		/************* write diary *************************************/
	sprintf(string,"import digit.ascii file %s to level %d.",fname,level);
	put_diary(string);

		/************* set job_stats ***********************************/
	job_stats[45]++;
	job_stats[32] = job_stats[45];

		/************* set level name **********************************/
	if(*lname)
	{	lname[STRLIST - 2] = 0;
		sprintf(level_names[level],"%s",lname);
	}
	else
		sprintf(level_names[level],"imported digit.ascii file: %s",fname);

		/*********** change display to add imported level **************/
	if(level < 64)
		disp_i1hglight |= (1 << (level % 32));
	else
		disp_i2hglight |= (1 << (level % 32));

		/*********** re-display map ************************************/
	dispgraphics((long) ((rwindowxmin + rwindowxmax) / 2),(long) ((rwindowymin + rwindowymax) / 2),resolution_code);

		/************* return ******************************************/
	return(0);
}


/*******************************************************************************
*      impdigiaabort                                                            *
*          This program is a convenient way of closing the input  file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long impdigiaabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* reset job_stats variables ************************/
	job_stats[16] = last_js16;
	job_stats[17] = last_js17;
	job_stats[18] = last_js18;

		/************* print abort message ******************************/
	sprintf(string,"import digit.ascii, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
