/*******************************************************************************
*   impdigit_att(see manual, Command Ref., User Functions)  <fc>               *
*                                                                              *
*       This program reads a GRASS digit.arr formatted ascii file and          *
*       assigns the attributes to objects in supplied set.                     *
*                                                                              *
*            Parameters:   char *fname       the name of the dlg3 file to      *
*                                            import.                           *
*                          long c            coord_type code (same as in the   *
*                                            register file)                    *
*                          long s            set number of set to attribute    *
*                          long ac           attribute action code             *
*                                                                              *
*            files/globals changed:  (files) job_stats, objects,               *
*                 attributes                                                   *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   1/2/90                                                *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>


char *reada();


long impdigit_att( fname, c, s, ac)
char *fname;
long c, s, ac;
{
	char actual[256];
	char string[256];
	char msg[256];
	char ftype;
	char parsed[5][STRLIST];

	double x,y;

	float fx,fy,r;

	long line;
	long n;
	long catagory;
	long obj;

	int fd;


	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check for raster registration *******************/
	if(!registered(0))
	{	message("You must REGISTER the Raster befor importing a digit.att file.");
		return(1);
	}

		/************* check for input  registration *******************/
	if(!registered(c))
	{	message("You must REGISTER the Input coordinate system befor importing a digit.att file.");
		return(1);
	}

		/******* set up variables *************************************/
	message("import digit.att, setting up variables.");
	fd = -1;


		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import digit.att, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open (1) file. aborting.");
		return(1);
	}
	
		/************* read all records ********************************/
	r = 2.0;
	for(line=0 ; reada(string,STRLIST,fd) > (char *) 0; line++)
	{
		if(!(line %25))
		{	sprintf(msg,"Reading import file line %d.",line+1);
			message(msg);
		}

		obj = -1;        /*** default, none **/

		n = parse(string," ",parsed,4);
		if(n == 4)
		{	sscanf(parsed[0],"%c",&ftype);
			sscanf(parsed[1],"%12lf",&x);
			sscanf(parsed[2],"%12lf",&y);
			sscanf(parsed[3],"%8ld",&catagory);
			transform(&x,&y,c,0);
			fx = x;
			fy = y;
			switch(ftype)  {
			case 'A':                               /*** arc     ***/
				obj = find_vpoly(fx,fy,s);
				break;
			case 'L':                               /*** segment ***/
				obj = find_vseg(fx,fy,r,s);
				break;
			case 'a':                               /*** deleted arc ***/
			case 'l':                               /*** deleted segment **/
				break;
			default:
				message("Illegal feature type encountered, ignoring.");
				break;
			}

			if(obj >= 0)
			{	sprintf(string,"%d",catagory);
				asgn_atrbt(obj,ac,string);
			}
		}
		else
			message("Read a incomplete line, ignoring it & proceding.");
	}


	message("import digit.att, closing files.");
		/************* close file **************************************/
	closef_ptr(&fd);

		/************* if a copied dos file, purge it ******************/
#ifdef SV32
	/* can't access dos anyway */
#else
		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);
#endif


		/************* write diary *************************************/
	sprintf(string,"import digit.att file %s to set %d.",fname,s);
	put_diary(string);

		/************* set job_stats ***********************************/
	job_stats[45]++;
	job_stats[33] = job_stats[45];

		/************* return ******************************************/
	return(0);
}
