/*******************************************************************************
*       impdlg3         (see manual, Command Ref., User Functions)      <fc>   *
*            This program imports a dlg3 optinoal onto an LTPlus level.        *
*                                                                              *
*            Parameters:   char *fname       the name of the dlg3 file to      *
*                                            import.                           *
*                          char *lname       new name for the level imported   *
*                          long level        the level number to import to     *
*                                            (33  - 95  inclusive)             *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist                                                  *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   9/12/89                                               *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU - major was being used in macros
   somewhere in the included files, so I changed it to majorc.
   */

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

char *malloc_ptr1, *malloc_ptr2;
int fd1, fd2;
long line_count1, line_count2;
long last_js15, last_js16, last_js17, last_js18;

long get_dlg_line();
long decode_dlgan();
long decode_dlgln();
long close_imdlg3();
long decode_dlgqr();
long decode_dlgcd();
long decode_dlgat();
long decode_dlgtx();

long parameters;
double parameter[4];


long impdlg3( fname, lname, level )
	char *fname, *lname;
	long level;
{
	V_OBJECTS v_object;
	V_OBJECTS v_object2;
	ATTRIBUTES atrbt;
	COORD_LIST coord_list;

	char actual[256];
	char line[256];
	char cartog_unit[256];
	char *ptr;
	char catagory[256];
	char item[256];
	char string[256];

	long i,n,q;
	long level0;
	long coord_sys, zone;
	long control_pts, q_catagories;
	long size;
	long *lptr;
	long **q_cat;
	long *line_index;
	long line_max;
	long line_no;
	long lines,areas,nodes;
	long id, an_list, ln_list, cd_list;
	long ats, txt, islands;

	double save_lon_ref;
	double save_stpl[16];
	double nx, ny;
	double x,y;


	fd1 = -1;
	fd2 = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check for raster registration *******************/
	if(!registered(0))
	{	message("You must REGISTER the Raster befor importing a vector file.");
		return(1);
	}

		/************* check for geodetic registration *****************/
	if(!registered(1))
	{	message("You must REGISTER the Geodetics befor importing a vector file.");
		return(1);
	}

		/************* check input parameters **************************/
	if((level < 33)  ||  (level > 95))
	{	message("Illegal level number.  Level number must be between 33 and 95.");
		return(1);
	}
	level0 = level;


	message("import dlg3, setting up variables.");
		/************* initialize globals ******************************/
	line_count1 = line_count2 = 0;

		/************* output v_attribute record 0 dummy record ********/
	if(!job_stats[16])
	{	atrbt.next = 0;
		atrbt.string[0] = 0;
		if(put_v_attr(&atrbt,0))
		{	message("import dlg3, abort, location 0.");
			return(1);
		}
	}

		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import dlg3, abort, location 1.");
		return(1);
	}
	fd1 = openf(actual,READ);
	if(fd1 <= 0)
	{	message("Cannot open (1) file. aborting.");
		return(1);
	}
	fd2 = openf(actual,READ);
	if(fd2 <= 0)
	{	message("Cannot open (2) file. aborting.");
		closef_ptr(&fd1);
		return(1);
	}
	

	message("import dlg3, reading file header.");
		/************* read file header ********************************/
	if(get_dlg_line(line,1))
	{	message("import dlg3, abort, location 2.");
		return(1);
	}
	if(get_dlg_line(cartog_unit,1))
	{	message("import dlg3, abort, location 3.");
		return(1);
	}

		/** get name of cartographic unit **/
	cartog_unit[40] = 0;
	trim(cartog_unit);

			/** find line 4 **/
	for(i=0;i<100;i++)
	{	if(get_dlg_line(line,1))
		{	message("import dlg3, abort, location 4.");
			return(1);
		}
		if(line != (char *) 0)
		{	line[79] = 0;
			if((strstr(line,"     3     ") == line)  &&  (strlen(line) >= 24))
				break;        /** found dlg level code (level 3) **/
		}
	}
		/** check for bad pointers / no data **/
	if(i >= 100)
	{	message("Cannot find dlg level code in first 100 lines, aborting.");
		close_imdlg3();
		return(1);
	}

		/** get coord type code **/
	ptr = line + 11;
	if(*ptr == '1')
		coord_sys = 2;        /* utm */
	else if(*ptr == '2')
		coord_sys = 3;        /* state plane */
	else
	{	message("Improper code for planimetric reference, aborting.");
		close_imdlg3();
		return(1);
	}
		/** get zone ***/
	ptr = line + 12;
	line[18] = 0;
	sscanf(ptr,"%d",&zone);
	line[18] = ' ';
		/** get zone parameters **/
	check_llref();
	save_lon_ref = lon_ref;
	for(i=0; i<16; i++)
		save_stpl[i] = stpln_params[i];
	if(coord_sys == 2)     /* utm */
	{	lon_ref= 183.0 - zone * 6.0;
		if((lon_ref< 0.0)  ||  (lon_ref > 180.0))
		{	message("Illegal utm zone in dlg file.  Assuming coordinates are in same zone as map.");	
			for(i=0; i<4; i++)
			{	corner[8][i][0] = corner[2][i][0];
				corner[8][i][1] = corner[2][i][1];
			}
		}
		else
		{	for(i=0; i<4; i++)
			{	if(geo_to_utm(corner[1][i][0],corner[1][i][1],
				          &corner[8][i][0],&corner[8][i][1]))
				{	message("Cannot calculate utm coords of map corners.");
					close_imdlg3();
						/* reset parameters */
					lon_ref = save_lon_ref;
					for(i=0; i<16; i++)
						stpln_params[i] = save_stpl[i];
					return(1);
				}
			}
		}
	}
	else                   /* stpln */
	{	if(stpln_data(zone))
		{	message("Illegal state plane zone in dlg file.  Assuming same zone as map.");	
			for(i=0; i<4; i++)
			{	corner[8][i][0] = corner[3][i][0];
				corner[8][i][1] = corner[3][i][1];
			}
		}
		else
		{	for(i=0; i<4; i++)
			{	if(geo_to_stpln(corner[1][i][0],corner[1][i][1],
				          &corner[8][i][0],&corner[8][i][1]))
				{	message("Cannot calculate state plane coords of map corners.");
					close_imdlg3();
						/* reset parameters */
					lon_ref = save_lon_ref;
					for(i=0; i<16; i++)
						stpln_params[i] = save_stpl[i];
					return(1);
				}
			}
		}
	}
		/* reset parameters */
	lon_ref = save_lon_ref;
	for(i=0; i<16; i++)
		stpln_params[i] = save_stpl[i];

		/** get no of control points and number of q catagories */
	control_pts = 4;       /* default */
	q_catagories = 1;      /* default */
	if(strlen(line) >= 66)
	{	ptr = line + 60;
		line[66] = 0;
		sscanf(ptr,"%d",&q_catagories);
	}
	if(strlen(line) >= 60)
	{	ptr = line + 54;
		line[60] = 0;
		sscanf(ptr,"%d",&control_pts);
	}

		/** read through projection parameters **/
	for(i=0; i<6; i++)
	{	if(get_dlg_line(line,1))
		{	message("import dlg3, abort, location 5.");
			return(1);
		}
	}

		/*** decode internal file-to-map projection transformation parameters **/
		/*** change D's to E's ***/
	for(ptr=line;*ptr;ptr++)
	{	if(*ptr == 'D')
			*ptr = 'E';
	}
		/*** convert parameters ***/
	ptr = line + 54;
	parameters = 1;
	for(i=3; i >= 0; i--, ptr -= 18)
	{	*(ptr + 18) = (char) 0;
		parameter[i] = 0.00;
		if(!sscanf(ptr,"%lf",&parameter[i]))
			parameters = 0;
	}
		/**** check for meaningful parameters *****/
	if(((parameter[0] == 1.00) || (parameter[0] == 0.0)) && (parameter[1] == 0.00) && (parameter[2] == 0.00)  &&  (parameter[3] == 0.00))
		parameters = 0;

		
		/** read through control point records **/
	for(i=0; i<control_pts; i++)
	{	if(get_dlg_line(line,1))
		{	message("import dlg3, abort, location 6.");
			return(1);
		}
	}

		/** malloc space for q_catagories */
	size = 4 * 11 * q_catagories;
	malloc_ptr1 = malloc_ltp(size);
	q_cat = (long **) malloc_ptr1;
	lptr = (long *) (q_cat + q_catagories);
	for(i=0 ; i<= q_catagories; i++, lptr += 10)
		q_cat[i] = lptr;

		/** read catagory records **/
	for(i=0; i<q_catagories; i++)
	{	if(get_dlg_line(line,1))
		{	message("import dlg3, abort, location 7.");
			return(1);
		}

		if(decode_dlgqr(line,catagory, q_cat[i]))
		{	message("import dlg3, abort, location 8.");
			return(1);
		}
		sprintf(level_names[level+i],"%s,  map %s",catagory,cartog_unit);
	}


		/************* setup vairables for line index ******************/
	last_js15 = job_stats[15];
	last_js16 = job_stats[16];
	last_js17 = job_stats[17];
	last_js18 = job_stats[18];

		/** malloc space for line index **/
	n = q_cat[0][7];                   /** find maximum line referenced **/
	for(i=0; i<q_catagories; i++)
	{	if(n < q_cat[i][7])
			n = q_cat[i][7];
		if(n < q_cat[i][8])
			n = q_cat[i][8];
	}
	n++;
	line_max = n;

		/*** malloc space for line index ***/
	size = 4 * n;
	malloc_ptr2 = malloc_ltp(size);
	if(malloc_ptr2 == (char *) 0)
	{	close_imdlg3();
		message("Not enough RAM to index dlg lines, aborting.");
		return(1);
	}

		/************* move second file pointer up to line/area/nodes **/
	for(; line_count2 < line_count1; )
	{	if(get_dlg_line(line,2))
		{	message("import dlg3, abort, location 9.");
			return(1);
		}
	}

		/************* set line_index up *******************************/
	line_index = (long *) malloc_ptr2;

		/************* read in all q catagories ************************/
	for(q = 0; q < q_catagories; q++, level++)
	{
			/** initialize line_index **/
		for(i=1; i < line_max; i++)
			line_index[i] = 0;

			/************* read line records *******************************/
		for(lines=0; lines < q_cat[q][8]; lines++)
		{		/** find next line record header line **/
			if(!(lines % 10))
			{	sprintf(string,"import dlg3, reading q catagory %d, line %d",
				 q,lines);
				message(string);
			}

			for(;;)
			{	if(get_dlg_line(line,1))
				{	message("import dlg3, abort, location 10.");
					return(1);
				}

					/** read past if it is a area or node record **/
				if((*line=='A') || (*line=='N') )
				{
					if(decode_dlgan(line,&id,&nx,&ny,&an_list,&ln_list,&cd_list,&ats,&txt,&islands))
					{	message("import dlg3, abort, location 11.");
						return(1);
					}
					 /* read past area-node linkage list */
					for(i=0; i<an_list; i += 12 )
					{	if(get_dlg_line(line,1))
						{	message("import dlg3, abort, location 12.");
							return(1);
						}
					}

					 /* read past line segment list */
					for(i=0; i<ln_list; i += 12 )
					{	if(get_dlg_line(line,1))
						{	message("import dlg3, abort, location 13.");
							return(1);
						}
					}
					
					 /* read past area coord   list */
					for(i=0; i<cd_list; i += 3 )
					{	if(get_dlg_line(line,1))
						{	message("import dlg3, abort, location 14.");
							return(1);
						}
					}
					
					 /* read past attributes        */
					for(i=0; i<ats; i += 6 )
					{	if(get_dlg_line(line,1))
						{	message("import dlg3, abort, location 15.");
							return(1);
						}
					}
					
					 /* read past text characters   */
					for(i=0; i<txt; i += 72 )
					{	if(get_dlg_line(line,1))
						{	message("import dlg3, abort, location 16.");
							return(1);
						}
					}
				}
					/** read it if it is a line record **/
				else if(*line=='L')
					break;
				else    /** else abort **/
				{	message("(1) Non-header line found in header's place, aborting.");
					close_imdlg3();
					return(1);
				}
			}
			if(decode_dlgln(line,&id,&cd_list,&ats,&txt))
			{	message("import dlg3, abort, location 17.");
				return(1);
			}

				/*** set line index ***/
			if(id >= line_max)
			{	close_imdlg3();
				message("Line id greater than specified in catagory record.  Aborting.");
				return(1);
			}
			else
				line_index[id] = job_stats[17];
			
				/*** set up v_object record ***/
			v_object.type = level * 10 + 1;
			v_object.v1 = job_stats[18];
			v_object.v2 = job_stats[18] + cd_list - 1;
			v_object.atr = 0;

				/*** store coordinate list ***/
			for(; (cd_list>0) ; )
			{	if(get_dlg_line(line,1))
				{	message("import dlg3, abort, location 18.");
					return(1);
				}
				 /*** get coordinates ***/
				if(decode_dlgcd(line,&cd_list))
				{	message("import dlg3, abort, location 19.");
					return(1);
				}
			}

				/*** store code_pair attributes ****/
			if(ats)
			{	for(; (ats>0) ; )
				{	if(get_dlg_line(line,1))
					{	message("import dlg3, abort, location 20.");
						return(1);
					}
				 	/*** get code pair attributes ***/
					if(decode_dlgat(line,&ats,&v_object.atr))
					{	message("import dlg3, abort, location 21.");
						return(1);
					}
				}
			}

				/*** store txt attributes ***/
			if(txt)
			{	for(; (txt>0) ; )
				{	if(get_dlg_line(line,1))
					{	message("import dlg3, abort, location 22.");
						return(1);
					}
				 	/*** get text attributes ***/
					if(decode_dlgtx(line,&txt,&v_object.atr))
					{	message("import dlg3, abort, location 23.");
						return(1);
					}
				}
			}

				/*** store v_objects record ***/
			if(put_vobjects(&v_object,job_stats[17]))
			{	message("import dlg3, abort, location 24.");
				return(1);
			}
		}
		

			/************* read area and node records **********************/
		for(areas=0,nodes=0; (areas<q_cat[q][4]) || (nodes<q_cat[q][2]); )
		{
			if((areas == 0) && !(nodes % 10))
			{	sprintf(string,"import dlg3, reading q catagory %d, node %d",
				 q,nodes);
				message(string);
			}
			else if((nodes == q_cat[q][2]) && !(areas % 10))
			{	sprintf(string,"import dlg3, reading q catagory %d, area %d",
				 q,areas);
				message(string);
			}

				/** find next area or node record header line **/
			for(;;)
			{	if(get_dlg_line(line,2))
				{	message("import dlg3, abort, location 25.");
					return(1);
				}

					/** read past if it is a line record **/
				if(*line=='L')
				{		/*** decode header line ***/
					if(decode_dlgln(line,&id,&cd_list,&ats,&txt))
					{	message("import dlg3, abort, location 26.");
						return(1);
					}

						/*** read past coordinate list ***/
					for(; (cd_list>0) ; cd_list -= 3)
					{	if(get_dlg_line(line,2))
						{	message("import dlg3, abort, location 27.");
							return(1);
						}
					}

						/*** read past code_pair attributes ****/
					for(; (ats>0) ; ats -= 6)
					{	if(get_dlg_line(line,2))
						{	message("import dlg3, abort, location 28.");
							return(1);
						}
					}

						/*** read past txt attributes ***/
					for(; (txt>0) ; txt -= 72)
					{	if(get_dlg_line(line,2))
						{	message("import dlg3, abort, location 29.");
							return(1);
						}
					}
				}
				else if((*line == 'A') || (*line == 'N'))
					break;
				else
				{
					message("(2) Non-header line found in header's place, aborting.");
					message(line);
					close_imdlg3();
					return(1);
				}
			}

				/** if it is a area record ****/
			if(*line=='A')       /** area header **/
			{
				if(decode_dlgan(line,&id,&nx,&ny,&an_list,&ln_list,&cd_list,&ats,&txt,&islands))
				{	message("import dlg3, abort, location 30.");
					return(1);
				}

					/*** set up v_object record ***/
				v_object.type = level * 10 + 2;
				if(txt || ats)
					v_object.atr = job_stats[16];
				else
					v_object.atr = 0;


				 /* read past area-node linkage list */
				for(i=0; i<an_list; i += 12 )
				{	if(get_dlg_line(line,2))
					{	message("import dlg3, abort, location 31.");
						return(1);
					}
				}

				 /* read line segment list */
				for(i=0; i<ln_list; i++ )
				{	if((i % 12) == 0)
					{	if(get_dlg_line(line,2))
						{	message("import dlg3, abort, location 32.");
							return(1);
						}
					}
						/*** line segment number **************/
					strncpy(item,(char *) (line+(i%12)*6), 6);
					*(item + 6) = 0;
					if(0 >= sscanf(item,"%ld",&line_no))
					{	close_imdlg3();
						message("line segment list too short, aborting.");
						return(1);
					}
						/** check line record no **/
					n = abs(line_no);
					if(n != 0)       /** break between islands **/
					{	if(n > q_cat[q][7])
						{
							close_imdlg3();
							message("Illegal line number, aborting.");
							return(1);
						}
	
							/**  read line record **/
						n = line_index[n];
						if(get_vobjects(&v_object2,n))
						{	message("import dlg3, abort, location 33.");
							return(1);
						}

							/*** set new vertex coords in area record **/
						if(line_no > 0)
						{	v_object.v1 = v_object2.v1;
							v_object.v2 = v_object2.v2;
						}
						else
						{	v_object.v1 = v_object2.v2;
							v_object.v2 = v_object2.v1;
						}
					
							/*** store v_objects record ***/
						if(put_vobjects(&v_object,job_stats[17]))
						{	message("import dlg3, abort, location 34.");
							return(1);
						}
					}
				}
				
				 /* read area coord list only if there is no line list */
				if(cd_list && !ln_list)
				{	v_object.v1 = job_stats[18];
					v_object.v2 = job_stats[18] + cd_list - 1;

					for(i=0; i<cd_list; i += 3 )
					{	if(get_dlg_line(line,2))
						{	message("import dlg3, abort, location 35.");
							return(1);
						}
					 	/*** get coordinates ***/
						if(decode_dlgcd(line,&cd_list))
						{	message("import dlg3, abort, location 36.");
							return(1);
						}
					}
						/*** store v_objects record ***/
					if(put_vobjects(&v_object,job_stats[17]))
					{	message("import dlg3, abort, location 37.");
						return(1);
					}
				}


					/*** store code_pair attributes ****/
				v_object.atr = 0;
				for(; (ats>0) ; )
				{	if(get_dlg_line(line,2))
					{	message("import dlg3, abort, location 38.");
						return(1);
					}
				 	/*** get code pair attributes ***/
					if(decode_dlgat(line,&ats,&v_object.atr))
					{	message("import dlg3, abort, location 39.");
						return(1);
					}
				}

					/*** store txt attributes ***/
				for(; (txt>0) ; )
				{	if(get_dlg_line(line,2))
					{	message("import dlg3, abort, location 40.");
						return(1);
					}
				 	/*** get text attributes ***/
					if(decode_dlgtx(line,&txt,&v_object.atr))
					{	message("import dlg3, abort, location 41.");
						return(1);
					}
				}
				areas++;
			}
			else if(*line=='N')                   /** node header **/
			{		/*** decode header line ***/
				if(decode_dlgan(line,&id,&nx,&ny,&an_list,&ln_list,&cd_list,&ats,&txt,&islands))
				{	message("import dlg3, abort, location 42.");
					return(1);
				}

					/*** calc object record ***/
				v_object.type = level * 10;
				if(txt || ats)
					v_object.atr = job_stats[16];
				else
					v_object.atr = 0;
				v_object.v1 = v_object.v2 = job_stats[18];
					/*** store v_objects record ***/
				if(put_vobjects(&v_object,job_stats[17]))
				{	message("import dlg3, abort, location 43.");
					return(1);
				}

					/*** transform coordinates ***/
				if(parameters)
				{	x = nx;
					y = ny;
					nx = parameter[0] * x + parameter[1] * y + parameter[2];
					ny = parameter[0] * y - parameter[1] * x + parameter[3];
				}
				transform(&nx,&ny,8,0);
					/*** store coordinates ***/
				coord_list.x = nx;
				coord_list.y = ny;
				if(put_v_coord(&coord_list,job_stats[18]))
				{	close_imdlg3();
					message("import dlg3, abort, location 44.");
					return(1);
				}

					/* read past area-node linkage list */
				for(i=0; i<an_list; i += 12 )
				{	if(get_dlg_line(line,2))
					{	message("import dlg3, abort, location 45.");
						return(1);
					}
				}

					/* read past line segment list */
				for(i=0; i<ln_list; i += 12 )
				{	if(get_dlg_line(line,2))
					{	message("import dlg3, abort, location 46.");
						return(1);
					}
				}

					/*** store code_pair attributes ****/
				v_object.atr = 0;
				for(; (ats>0) ; )
				{	if(get_dlg_line(line,2))
					{	message("import dlg3, abort, location 47.");
						return(1);
					}
				 	  /*** get code pair attributes ***/
					if(decode_dlgat(line,&ats,&v_object.atr))
					{	message("import dlg3, abort, location 48.");
						return(1);
					}
				}

					/*** store txt attributes ***/
				for(; (txt>0) ; )
				{	if(get_dlg_line(line,2))
					{	message("import dlg3, abort, location 49.");
						return(1);
					}
				 	/*** get text attributes ***/
					if(decode_dlgtx(line,&txt,&v_object.atr))
					{	message("import dlg3, abort, location 50.");
						return(1);
					}
				}
					
				nodes++;
			}
		}

			/************* match fd1 & fd2 pointers ************************/
		if((q+1) < q_catagories)
		{
			message("import dlg3, matching file pointers.");
			for(; line_count1 < line_count2; )
			{	if(get_dlg_line(line,1))
				{	message("import dlg3, abort, location 51.");
					return(1);
				}
			}

				/** bring fd2 up to fd1 **/
			for(; line_count2 < line_count1; )
			{	if(get_dlg_line(line,2))
				{	message("import dlg3, abort, location 52.");
					return(1);
				}
			}
		}
	}


	message("import dlg3, closing files.");
		/************* close file **************************************/
	closef_ptr(&fd1);
	closef_ptr(&fd2);

		/************* if a copied dos file, purge it ******************/
#ifdef SV32
	/* can't access dos anyway */
#else
		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);
#endif

		/************* index v_objects imported ************************/
	if(index_vobj(last_js17))
	{	job_stats[17] = last_js17;
		return(1);
	}

		/************* write diary *************************************/
	sprintf(string,"import dlg3 file %s to level %d.",fname,level0);
	put_diary(string);

		/************* set job_stats ***********************************/
	job_stats[45]++;
	job_stats[30] = job_stats[45];

		/*********** change display to add imported level **************/
	if(level0 < 64)
		disp_i1hglight |= (1 << (level0 % 32));
	else
		disp_i2hglight |= (1 << (level0 % 32));

		/*********** re-display map ************************************/
	dispgraphics((long) ((rwindowxmin + rwindowxmax) / 2),(long) ((rwindowymin + rwindowymax) / 2),resolution_code);

		/************* return ******************************************/
	return(0);
}


/*******************************************************************************
*    get_dlg3_line                                                             *
*        reads 1 line from a dlg3 file, increments counters, displays an       *
*    error message upon failure and closes the files.                          *
*                                                                              *
*    Parameters  char *line     (ascii line read)                              *
*                long fdnum     fd number (1 or 2)                             *
*                                                                              *
*    Globals changed (file globals only, fd1,fd2,line_count1,line_count2)      *
*                                                                              *
*    Returned (long) 0=ok,  1 = failure on read, files closed, err msg disp.   *
*    J.Dabritz  9/14/89                                                        *
*******************************************************************************/

long get_dlg_line(line,fdnum)
	char *line;
	long fdnum;
{
	if(fdnum == 1)
	{	if(reada(line,96,fd1) == (char *) 0)
		{	message("Read failed while attempting to read dlg3 ascii record.");
			close_imdlg3();
			return(1);
		}
		line[95] = 0;
		line_count1++;
	}
	else if(fdnum == 2)
	{	if(reada(line,96,fd2) == (char *) 0)
		{	message("Read failed while attempting to read dlg3 ascii record.");
			close_imdlg3();
			return(1);
		}
		line[95] = 0;
		line_count2++;
	}
	else
	{	message("bad fdnum in get_dlg_line, aborting.");
		close_imdlg3();
		return(1);
	}
	
	return(0);
}

/*******************************************************************************
*    decode_dlgan                                                              *
*        This program decodes a dlg area or node header record.                *
*                                                                              *
*    parameters:  char *line       line to be decoded                          *
*                 long *id         file internal id number                     *
*                 double *nx       file node x coordinate                      *
*                 double *ny       file node y coordinate                      *
*                 long *an_list    file number of items in area/node list      *
*                 long *ln_list    file number of items in line list           *
*                 long *cd_list    file number of items in coordinate list     *
*                 long *ats        file number of attributes listed            *
*                 long *txt        file number of text characters listed       *
*                 long *islands    file number of islands in area listed       *
*                                                                              *
*    no globals or files are changed                                           *
*                                                                              *
*    returned values:  long 0 = ok, 1=header corrupted/absent, aborting        *
*    J.Dabritz  9/14/89                                                        *
*******************************************************************************/

long decode_dlgan(line,id,nx, ny, an_list, ln_list, cd_list, ats, txt, islands)
	char *line;
	long *id;
	double *nx, *ny;
	long *an_list, *ln_list, *cd_list, *ats, *txt, *islands;
{
	char item[256];
	long i,n;


		/************ initialize return values ***********************/
	*id = *an_list = *ln_list = *cd_list = *ats = *txt = *islands = 0;
	*nx = 0.0;
	*ny = 0.0;

		/************ check for 'A' or 'N' in column 1 ***************/
	if(*line == 'A')
	{		/************ check length ***********************************/
		n = strlen(line);
		if(n<66)
		{	close_imdlg3();
			message("DLG3 area header line shorter than expected.  Aborting.");
			return(1);
		}

			/************* get id number ******************************/
		strncpy(item,line+1,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",id);
		if(*id < 1)
		{	close_imdlg3();
			message("DLG area header has no/illegal id number, aborting.");
			return(1);
		}

			/************* number of elements in node list ************/
		strncpy(item,line+30,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",an_list);
		if(*an_list < 0)
		{	close_imdlg3();
			message("DLG area header has illegal number of node_list elements, aborting");
			return(1);
		}

			/************* number of elements in line seg list ********/
		strncpy(item,line+36,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",ln_list);
		if(*ln_list < 0)
		{	close_imdlg3();
			message("DLG area header has illegal number of seg_list elements, aborting");
			return(1);
		}

			/************* number of points in area coord list ********/
		strncpy(item,line+42,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",cd_list);
		if(*cd_list < 0)
		{	close_imdlg3();
			message("DLG area header has illegal number of cord_list elements, aborting");
			return(1);
		}

			/************* number of attributes ***********************/
		strncpy(item,line+48,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",ats);
		if(*ats < 0)
		{	close_imdlg3();
			message("DLG area header has illegal number of attribute elements, aborting");
			return(1);
		}

			/************* number of text characters listed ***********/
		strncpy(item,line+54,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",txt);
		if(*txt < 0)
		{	close_imdlg3();
			message("DLG area header has illegal number of text chatacter elements, aborting");
			return(1);
		}

			/************* number of islands within area **************/
		strncpy(item,line+60,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",islands);
		if(*islands < 0)
		{	close_imdlg3();
			message("DLG area header has illegal number of islands listed, aborting");
			return(1);
		}

	}
	else if(*line == 'N')
	{		/************ check length ***********************************/
		n = strlen(line);
		if(n<54)
		{	close_imdlg3();
			message("DLG3 node header line shorter than expected.  Aborting.");
			return(1);
		}

			/************* get id number ******************************/
		strncpy(item,line+1,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",id);
		if(*id < 1)
		{	close_imdlg3();
			message("DLG node header has no/illegal id number, aborting.");
			return(1);
		}

			/************* x coordinate of node ***********************/
		strncpy(item,(char *) (line+6),12);
		*(item + 12) = 0;
		i = sscanf(item,"%lf",nx);
		if(i < 1)
		{	close_imdlg3();
			message("DLG node header has no x coordinate, aborting.");
			return(1);
		}

			/************* y coordinate of node ***********************/
		strncpy(item,(char *) (line+18),12);
		*(item + 12) = 0;
		i = sscanf(item,"%lf",ny);
		if(i < 1)
		{	close_imdlg3();
			message("DLG node header has no y coordinate, aborting.");
			return(1);
		}

			/************* number of elements in area list ************/
		strncpy(item,line+30,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",an_list);
		if(*an_list < 0)
		{	close_imdlg3();
			message("DLG node header has illegal number of node_list elements, aborting");
			return(1);
		}

			/************* number of elements in line seg list ********/
		strncpy(item,line+36,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",ln_list);
		if(*ln_list < 0)
		{	close_imdlg3();
			message("DLG node header has illegal number of seg_list elements, aborting");
			return(1);
		}

			/************* number of points in node coord list ********/
		strncpy(item,line+42,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",cd_list);
		if(*cd_list < 0)
		{	close_imdlg3();
			message("DLG node header has illegal number of cord_list elements, aborting");
			return(1);
		}

			/************* number of attributes ***********************/
		strncpy(item,line+48,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",ats);
		if(*ats < 0)
		{	close_imdlg3();
			message("DLG node header has illegal number of attribute elements, aborting");
			return(1);
		}

			/************* number of text characters listed ***********/
		if(n < 55)
			*txt = 0;
		else
		{	strncpy(item,line+54,6);
			*(item + 6) = 0;
			sscanf(item,"%ld",txt);
			if(*txt < 0)
			{	close_imdlg3();
				message("DLG node header has illegal number of text chatacter elements, aborting");
				return(1);
			}
		}

	}
	else
	{	message("Illegal area or node header line, aborting.");
		return(1);
	}

	return(0);
	
}


/*******************************************************************************
*    decode_dlgln                                                              *
*        This program decodes a dlg line header record.                        *
*                                                                              *
*    parameters:  char *line       line to be decoded                          *
*                 long *id         file internal id number                     *
*                 long *cd_list    file number of coordinate pairs listed      *
*                 long *ats        file number of attributes listed            *
*                 long *txt        file number of text characters listed       *
*                                                                              *
*    no globals or files are changed                                           *
*                                                                              *
*    returned values:  long 0 = ok, 1=not a line header/failed to get all info *
*    J.Dabritz  9/14/89                                                        *
*******************************************************************************/

long decode_dlgln(line,id,cd_list,ats,txt)
	char *line;
	long *id, *cd_list, *ats, *txt;
{
	long n;
	char item[256];

		/************ initialize return values ***********************/
	*id = *cd_list = *ats = *txt = 0;

		/************ check for 'L'        in column 1 ***************/
	if(*line == 'L')
	{		/************ check length ***********************************/
		n = strlen(line);
		if(n<54)
		{	close_imdlg3();
			message("DLG3 line header line shorter than expected.  Aborting.");
			return(1);
		}

			/************* get id number ******************************/
		strncpy(item,line+1,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",id);
		if(*id < 1)
		{	close_imdlg3();
			message("DLG line header has no/illegal id number, aborting.");
			return(1);
		}


			/************* number of points in line coord list ********/
		strncpy(item,line+42,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",cd_list);
		if(*cd_list < 0)
		{	close_imdlg3();
			message("DLG line header has illegal number of cord_list elements, aborting");
			return(1);
		}

			/************* number of attributes ***********************/
		strncpy(item,line+48,6);
		*(item + 6) = 0;
		sscanf(item,"%ld",ats);
		if(*ats < 0)
		{	close_imdlg3();
			message("DLG line header has illegal number of attribute elements, aborting");
			return(1);
		}

			/************* number of text characters listed ***********/
		if(n >= 60)
		{	strncpy(item,line+54,6);
			*(item + 6) = 0;
			sscanf(item,"%ld",txt);
			if(*txt < 0)
			{	close_imdlg3();
				message("DLG line header has illegal number of text chatacter elements, aborting");
				return(1);
			}
		}

		return(0);
	}
	else
	{	message("Illegal line header, aborting.");
		return(1);
	}


}


/*******************************************************************************
*    close_imdlg3                                                              *
*        closes files and frees malloc space for impdlg3.c                     *
*                                                                              *
*                                                                              *
*                                                                              *
*                                                                              *
*******************************************************************************/

long close_imdlg3()
{
	long n;

		/********* close files ***************************************/
	if(fd1 > 0)
		closef_ptr(&fd1);

	if(fd2 > 0)
		closef_ptr(&fd2);

		/********* free malloc space *********************************/
	n = (long) malloc_ptr1;
	if(n > 0)
		free_ltp(malloc_ptr1);

	n = (long) malloc_ptr2;
	if(n > 0)
		free_ltp(malloc_ptr2);

		/********* return ********************************************/
	return(0);
}


/*******************************************************************************
*    decode_dlgqr                                                              *
*        This program decodes a dlg catagory (q) records                       *
*                                                                              *
*    parameters:  char *line       line to be decoded                          *
*                 char *catagory   catagory name                               *
*                 long *qptr       pointer to q_cat array row:                 *
*                           [0] =  attribute format code                       *
*                           [1] =  number of nodes referenced in catagory      *
*                           [2] =  number of nodes in file for catagory        *
*                           [3] =  number of areas referenced in catagory      *
*                           [4] =  number of areas in file for catagory        *
*                           [5] =  flag for area to line linkage records       *
*                           [6] =  flag for area coordinate list records       *
*                           [7] =  number of lines referenced in catagory      *
*                           [8] =  number of lines in file for catagory        *
*                           [9] =  flag for line coordinate list records       *
*                                                                              *
*    no globals or files are changed                                           *
*                                                                              *
*    returned values:  long 0 = ok, 1=not a line header/failed to get all info *
*    J.Dabritz  9/14/89                                                        *
*******************************************************************************/

long decode_dlgqr(line,catagory,qptr)
	char *line, *catagory;
	long *qptr;
{
	long n;
	char item[256];


		/************ initialize return values ***********************/
	catagory[0] = 0;
	for(n=0; n<10; n++)
		qptr[0] = 0;


		/************* check line length **************************/
	n = strlen(line);
	if(n < 72)
	{	close_imdlg3();
		message("DLG catagory record is too short, aborting.");
		return(1);
	}

		/************* get catagory name **************************/
	strncpy(catagory ,line,20);
	trim(catagory);

		/************* attribute format code **********************/
	strncpy(item,line+20,4);
	*(item + 4) = 0;
	sscanf(item,"%ld",qptr);
	if((qptr[0] < 0)  ||  (qptr[0] > 1))
	{	close_imdlg3();
		message("DLG catagory record has illegal attribute format code.");
		return(1);
	}

		/************* nodes referenced ***************************/
	strncpy(item,line+24,6);
	*(item + 6) = 0;
	sscanf(item,"%ld",qptr+1);
	if(qptr[1] < 0)
	{	close_imdlg3();
		message("DLG catagory record has illegal number of nodes referenced.");
		return(1);
	}

		/************* nodes in file    ***************************/
	strncpy(item,line+30,6);
	*(item + 6) = 0;
	sscanf(item,"%ld",qptr+2);
	if(qptr[2] < 0)
	{	close_imdlg3();
		message("DLG catagory record has illegal number of nodes.");
		return(1);
	}

		/************* areas referenced ***************************/
	strncpy(item,line+40,6);
	*(item + 6) = 0;
	sscanf(item,"%ld",qptr+3);
	if(qptr[3] < 0)
	{	close_imdlg3();
		message("DLG catagory record has illegal number of areas referenced.");
		return(1);
	}

		/************* areas in file    ***************************/
	strncpy(item,line+46,6);
	*(item + 6) = 0;
	sscanf(item,"%ld",qptr+4);
	if(qptr[4] < 0)
	{	close_imdlg3();
		message("DLG catagory record has illegal number of areas.");
		return(1);
	}

		/************* area to line link flag *********************/
	if(*(line+54) == '1')
		qptr[5] = 1;

		/************* area coordinate list flag ******************/
	if(*(line+55) == '1')
		qptr[6] = 1;

		/************* lines referenced ***************************/
	strncpy(item,line+56,6);
	*(item + 6) = 0;
	sscanf(item,"%ld",qptr+7);
	if(qptr[7] < 0)
	{	close_imdlg3();
		message("DLG catagory record has illegal number of lines referenced.");
		return(1);
	}

		/************* lines in file    ***************************/
	strncpy(item,line+62,6);
	*(item + 6) = 0;
	sscanf(item,"%ld",qptr+8);
	if(qptr[8] < 0)
	{	close_imdlg3();
		message("DLG catagory record has illegal number of lines.");
		return(1);
	}

		/************* line coordinate list flag ******************/
	if(*(line+71) == '1')
		qptr[9] = 1;

	return(0);

}

/*******************************************************************************
*     decode_dlgcd                                                             *
*         This program decodes and stores dlg coordinate lines.                *
*                                                                              *
*     Parameters:    char *line   input string                                 *
*                    long *cd_list  number of coord pairs left for this obj.   *
*                                                                              *
*     Files changed:  v_coordlist file, job_stats[18]                          *
*                                                                              *
*     Returned:  long 0=ok, 1=abort                                            *
*     J.Dabritz   9/15/89                                                      *
*******************************************************************************/

long decode_dlgcd(line,cd_list)
	char *line;
	long *cd_list;
{
	char item[256];
	COORD_LIST coord_list;
	long i,j;
	double x,y;
	double nx,ny;


	for(j=0; (*cd_list > 0)  &&  (j < 3);  j++, *cd_list = *cd_list - 1)
	{		/******* x coordinate ********/
		strncpy(item,(char *) (line+(j*24)),12);
		*(item + 12) = 0;
		i = sscanf(item,"%lf",&x);
		if(i < 1)
		{	close_imdlg3();
			message("Coordinate list x missing, aborting.");
			return(1);
		}
			/******* y coordinate ********/
		strncpy(item,(char *) (line+(j*24+12)),12);
		*(item + 12) = 0;
		i = sscanf(item,"%lf",&y);
		if(i < 1)
		{	close_imdlg3();
			message("Coordinate list y missing, aborting.");
			return(1);
		}
			/*** transform coordinates to raster based **/
			/*** transform coordinates ***/
		if(parameters)
		{	nx = x;
			ny = y;
			x = parameter[0] * nx + parameter[1] * ny + parameter[2];
			y = parameter[0] * ny - parameter[1] * nx + parameter[3];
		}
		transform(&x,&y,8,0);
			/*** store coordinates ***/
		coord_list.x = x;
		coord_list.y = y;
		if(put_v_coord(&coord_list,job_stats[18]))
		{	close_imdlg3();
			return(1);
		}
	}

	return(0);
}

/*******************************************************************************
*     decode_dlgat                                                             *
*         This program decodes and stores dlg attribute lines.                 *
*                                                                              *
*     Parameters:    char *line   input string                                 *
*                    long *ats  number of attribute code pairs left for this   *
*                               object                                         *
*                    long *atr_ptr pointer to first atrbt in v_obj rec.        *
*                                                                              *
*     Files changed:  v_attributes file, job_stats[16]                         *
*                                                                              *
*     Returned:  long 0=ok, 1=abort                                            *
*     J.Dabritz   9/15/89                                                      *
*******************************************************************************/

long decode_dlgat(line,ats,atr_ptr)
	char *line;
	long *ats;
	long *atr_ptr;
{
	char item[256];
	char majorc[32];
	char *ptr;

	ATTRIBUTES atrbt; 

	long j;
	long rec, save_next;


	save_next = 0;

	for(j=0; (*ats>0)  && (j<6);  j++, *ats = *ats - 1)
	{		/*** check string length **/
		if(strlen(line) < (j*12 + 12))
		{	close_imdlg3();
			message("Attribute codes missing, aborting.");
			return(1);
		}

			/******* major code   ********/
		strncpy(majorc,line+(j*12),6);
		*(majorc + 6) = 0;
		trim(majorc);

			/******* minor code   ********/
		strcpy(item,"0000");
		ptr = (char *) (item + 4);
		strncpy(ptr,line+(j*12 + 6),6);
		*(ptr + 6) = 0;
		trim(ptr);
		ptr = (char *) (item + (strlen(item) - 4));

			/******* store attribute *****/
		sprintf(atrbt.string,"%s%s",majorc,ptr);
		if((*ats>1)  && (j<5))
			atrbt.next = job_stats[16] + 1;
		else
			atrbt.next = 0;

		if(!save_next)
			save_next = job_stats[16];
		if(put_v_attr(&atrbt,job_stats[16]))
			return(1);
	}

		/*** Add new list to end of current object's list ***/
		/**  read to end of current list (if any) & add ptr to new list **/
	if(save_next)
	{	if(*atr_ptr)		/** there is an existing attribute **/
		{		/** read to end of list **/
			atrbt.next = *atr_ptr;
			for(;atrbt.next;)
			{	rec = atrbt.next;
				if(get_v_attr(&atrbt,rec))
					return(1);
			}
			atrbt.next = save_next;
			if(put_v_attr(&atrbt,rec))
				return(1);
		}
		else              /** the list is currently empty **/
		{	*atr_ptr = save_next;
		}
	}


	return(0);
}

/*******************************************************************************
*     decode_dlgtx                                                             *
*         This program decodes and stores dlg text lines as attributes         *
*                                                                              *
*     Parameters:    char *line   input string                                 *
*                    long *txt  number of text characters left for this obj.   *
*                    long *atr_ptr pointer to first atrbt in v_obj rec.        *
*                                                                              *
*     Files changed:  v_attributes file, job_stats[16]                         *
*                                                                              *
*     Returned:  long 0=ok, 1=abort                                            *
*     J.Dabritz   9/15/89                                                      *
*******************************************************************************/

long decode_dlgtx(line,txt,atr_ptr)
	char *line;
	long *txt;
	long *atr_ptr;
{
	char *ptr;
	ATTRIBUTES atrbt; 
	long rec, save_next;


	save_next = 0;
	trim(line);
	atrbt.next = 1;
	for(ptr = line; atrbt.next ; )
	{
			/**** copy from line to attribute ***/
		strncpy(atrbt.string,ptr,30);
		atrbt.string[30] = 0;
		trim(atrbt.string);

			/******* see if there is more to store ***/
		if(strlen(ptr) > 30)
		{	ptr += 30;
			trim(ptr);
			if(*ptr)
				atrbt.next = job_stats[16] + 1;
			else
				atrbt.next = 0;
		}
		else
			atrbt.next = 0;

		if(!save_next)
			save_next = job_stats[16];
		if(put_v_attr(&atrbt,job_stats[16]))
			return(1);
	}

		/** reset txt amount **/
	if(*txt > 72)
		*txt -= 72;
	else
		*txt = 0;


		/*** Add new list to end of current object's list ***/
		/**  read to end of current list (if any) & add ptr to new list **/
	if(save_next)
	{	if(*atr_ptr)		/** there is an existing attribute **/
		{		/** read to end of list **/
			atrbt.next = *atr_ptr;
			for(;atrbt.next;)
			{	rec = atrbt.next;
				if(get_v_attr(&atrbt,rec))
					return(1);
			}
			atrbt.next = save_next;
			if(put_v_attr(&atrbt,rec))
				return(1);
		}
		else              /** the list is currently empty **/
			*atr_ptr = save_next;
	}

	return(0);
}
