/*******************************************************************************
*      impdxf            (see manual, Command Ref., User Functions)      <fc>  *
*                                                                              *
*          Imports vector data from dxf (Autocad) format.                      *
*                                                                              *
*            Parameters:   char *fname       the name of the dxf file to       *
*                                            import.                           *
*                          long level        the level number to import to     *
*                                            (35  - 95  inclusive)             *
*                          long c            coord_type code (same as in the   *
*                                            register file)                    *
*                          char *lname       new name for the level imported   *
*                                                                              *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist,   (varaibles)  [bunches]                        *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   6/22/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>
#include <math.h>

long last_js16, last_js17, last_js18;

long dxfreadpair();
long impdxfabort();
long dxfpolyline();
long dxfline();
long dxfpoint();
long dxftext();
long dxfcircle();
long dxfarc();
long dxftrace();
char *reada();
long otype;
long input_system;
int fd;


long impdxf( fname, level, c, lname)
char *fname;
long level,c;
char *lname;
{
	ATTRIBUTES attr;
	char actual[256];
	char string[256];
	char string1[STRLIST],string2[STRLIST];

	long hit;
	long i,last_i;


	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check for raster registration *******************/
	if(!registered(0))
	{	message("You must REGISTER the Raster befor importing a vector file.");
		return(1);
	}

		/************* check for import coordinate system registration */
	if(!registered(c))
	{	message("You must REGISTER the input coordinate system befor importing.");
		return(1);
	}
	input_system = c;

		/************* check input parameters **************************/
	if((level < 35)  ||  (level > 95))
	{	message("Illegal level number.  Level number must be between 35 and 95.");
		return(1);
	}
	if((c < 0)  ||  (c > 19))
	{	message("Illegal coordinate type code.  Code must be between 0 and 19 inclusive.");
		return(1);
	}


		/******* set up variables *************************************/
	message("import dxf, setting up variables.");
	last_js16 = job_stats[16];
	last_js17 = job_stats[17];
	last_js18 = job_stats[18];
	fd = -1;
	otype = level * 10;


		/************* output v_attribute record 0 dummy record ********/
	if(!job_stats[16])
	{	attr.next = 0;
		attr.string[0] = 0;
		if(put_v_attr(&attr,0))
		{	message("import dxf, abort, location 0.");
			return(1);
		}
	}

		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import dxf, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open (1) file. aborting.");
		return(1);
	}
	

		/*********** read through header to ENTITIES section of file */
	hit = 0;
	for(;;)
	{	if(dxfreadpair(string1,string2))
			return(1);

		if(*string1 == '0')
		{	if(!strcmp(string2,"SECTION"))
				hit = 1;
			else
				hit = 0;
		}
		else if(hit && (*string1 == '2'))
		{	if(!strcmp(string2,"ENTITIES"))
				break;
			else
				hit = 0;
		}
		else
			hit = 0;
	}

		/*********** Read remainder of file to ENDSEC ****************/
	last_i = 0;
	message("import dxf, reading entities. . .");
	for(i=0;;)
	{		/*********** read a line pair *****************************/
		if(dxfreadpair(string1,string2))
			return(1);

			/*********** check for ENDSEC *****************************/
		if(*string1 == '0')
		{	if(!strcmp(string2,"ENDSEC"))
				break;
		

				/*********** check for & enter lines, points etc. *********/
			if(!strcmp(string2,"POLYLINE"))
			{	if(dxfpolyline())
					return(1);
				i++;
			}
			else if(!strcmp(string2,"LINE"))
			{	if(dxfline())
					return(1);
				i++;
			}
			else if(!strcmp(string2,"POINT"))
			{	if(dxfpoint())
					return(1);
				i++;
			}
			else if(!strcmp(string2,"CIRCLE"))
			{	if(dxfcircle())
					return(1);
				i++;
			}
			else if(!strcmp(string2,"ARC"))
			{	if(dxfarc())
					return(1);
				i++;
			}
			else if((!strcmp(string2,"TRACE")) || (!strcmp(string2,"SOLID")))
			{	if(dxftrace())
					return(1);
				i++;
			}
			else if((!strcmp(string2,"INSERT")) || (!strcmp(string2,"SHAPE")))
			{	if(dxfpoint())
					return(1);
				i++;
			}
			else if((!strcmp(string2,"TEXT")) || (!strcmp(string2,"ATTRIB")))
			{	if(dxftext())
					return(1);
				i++;
			}
	
			if((i != last_i))
			{	sprintf(string,"import dxf, read entity %d.",i);
				message(string);
				last_i = i;
			}
		}
	}


	message("import dxf, closing files.");
		/************* close file **************************************/
	closef_ptr(&fd);

		/************* if a copied dos file, purge it ******************/
#ifdef SV32
	/* can't access dos anyway */
#else
		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);
#endif

		/************* index v_objects imported ************************/
	if(index_vobj(last_js17))
	{	job_stats[17] = last_js17;
		return(1);
	}

		/************* write diary *************************************/
	sprintf(string,"import dxf file %s to level %d.",fname,level);
	put_diary(string);

		/************* set job_stats ***********************************/
	job_stats[45]++;
	job_stats[37] = job_stats[45];

		/************* set level name **********************************/
	if(lname[1] == (char) 0)
		sprintf(level_names[level],"imported dxf file: %s",fname);
	else
	{	lname[STRLIST - 2] = 0;
		sprintf(level_names[level],"imported dxf file: %s",lname);
	}

		/*********** change display to add imported level **************/
	if(level < 64)
		disp_i1hglight |= (1 << (level % 32));
	else
		disp_i2hglight |= (1 << (level % 32));

		/*********** re-display map ************************************/
	dispgraphics((long) ((rwindowxmin + rwindowxmax) / 2),(long) ((rwindowymin + rwindowymax) / 2),resolution_code);

		/************* return ******************************************/
	return(0);
}


/*******************************************************************************
*      impdxfabort                                                            *
*          This program is a convenient way of closing the input  file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long impdxfabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* reset job_stats variables ************************/
	job_stats[16] = last_js16;
	job_stats[17] = last_js17;
	job_stats[18] = last_js18;

		/************* print abort message ******************************/
	sprintf(string,"import dxf, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}




/*******************************************************************************
*    dxfreadpair   reads a pair of entries in the dxf file                     *
*                      char *string1;    first string to be read               *
*                      char *string2;    second string to be read              *
*******************************************************************************/
long dxfreadpair(string1,string2)
char *string1, *string2;
{
	if(reada(string1,256,fd) == (char *) 0)
	{	message("Ran out of data befor anticipated data found.");
		impdxfabort(1);
		return(1);
	}
	if(reada(string2,256,fd) == (char *) 0)
	{	message("Ran out of data befor anticipated data found.");
		impdxfabort(1);
		return(1);
	}

	trim(string1);
	trim(string2);
	
	return(0);
}



/*******************************************************************************
*    dxfpolyline   reads one polyline record in the dxf file                   *
*******************************************************************************/
long dxfpolyline()
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	float first_x, first_y;
	long verticies;

	char string1[256],string2[256];

	long closed;
	long hit;


		/********** init ********************************************/
	closed = 0;

		/********** read & check flags (70) *************************/
	for(;;)
	{	if(dxfreadpair(string1,string2))
			return(1);

		if(*string1 == '0')
		{	if(!strcmp(string2,"VERTEX"))
				break;
			if(!strcmp(string2,"SEQEND"))
				return(1);
		}
		else if(!strcmp(string1,"1"))
		{	if(!strcmp(string2,"1"))
				closed = 1;
		}
	}


		/********** set up v_object record **************************/
	v_object.v1 = job_stats[18];
	v_object.atr = 0;


		/********** read all verticies & store v_object *************/
	attr.string[0] = (char) 0;
	hit = 0;
	for(;;)
	{	if(dxfreadpair(string1,string2))
			return(1);

			/******** check for end of object ************/
		if(*string1 == '0')
		{	if(!strcmp(string2,"SEQEND"))
				break;
		}		/********* check for x or y coordinate ********/
		else if((!strcmp(string1,"10"))  ||  (!strcmp(string1,"11")))
		{	sscanf(string2,"%f",&coordlist.x);      /*** x coordinate ***/
			hit |= 1;
		}
		else if((!strcmp(string1,"20"))  ||  (!strcmp(string1,"21")))
		{	sscanf(string2,"%f",&coordlist.y);      /*** y coordinate ***/
			hit |= 2;
		}
		else if((!strcmp(string1,"30")) || (!strcmp(string1,"31")) ||
		        (!strcmp(string1,"1")) || (!strcmp(string1,"2")) ||
		        (!strcmp(string1,"3")) || (!strcmp(string1,"4")) ||
		        (!strcmp(string1,"5")) || (!strcmp(string1,"8")))
		{	strncpy(attr.string,string2,30); /* text, symbol name or other atrbt */
			attr.string[30] = (char) 0;
		}

			/*********** if both coordinates entered, store & reset hit flag **/
		if(hit == 3)
		{	if(v_object.v1 == job_stats[18])    /** save first vertex for close **/
			{	first_x = coordlist.x;
				first_y = coordlist.y;
			}
			if(put_vcdxf(&coordlist))
			{	impdxfabort(5);
				return(1);
			}
			hit = 0;                  /** reset hit flag **/
		}
	}

		/********** store attribute record, if there was an elevation **/
	if(attr.string[0])
	{	attr.next = 0;
		v_object.atr = job_stats[16];
		if(put_v_attr(&attr,job_stats[16]))
		{	impdxfabort(2);
			return(1);
		}
	}
	
		/********** store object record *****************************/
	verticies = job_stats[18] - v_object.v1;
	switch(verticies) {
	case 0:                             /*** no verticies written ****/
		return(0);
	case 1:                             /*** point ***/
		v_object.type = otype;       /*** point on level otype **/
		break;
	case 2:                             /*** line  ***/
		v_object.type = otype + 1;   /*** line  on level otype **/
		break;
	default:
		if(closed)
		{		/*** close polygon if needed ***/
			if((first_x != coordlist.x) || (first_y != coordlist.y))
			{	coordlist.x = first_x;
				coordlist.y = first_y;
				if(put_vcdxf(&coordlist))
				{	impdxfabort(5);
					return(1);
				}
			}
			v_object.type = otype + 2;   /*** polygon on level otype **/
		}
		else
			v_object.type = otype + 1;   /*** line  on level otype **/
		break;
	}

		/***** set ending vertex *******/
	v_object.v2 = job_stats[18] - 1;

		/***** store v_object record ****/
	if(put_vobjects(&v_object,job_stats[17]))
	{	impdxfabort(4);
		return(1);
	}

		/********** return ******************************************/
	return(0);
}




/*******************************************************************************
*    dxfline       reads one line     record in the dxf file                   *
*******************************************************************************/
long dxfline()
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	long verticies;

	char string1[256],string2[256];

	long hit;


		/********** set up v_object record **************************/
	v_object.v1 = job_stats[18];
	v_object.atr = 0;


		/********** read all verticies & store v_object *************/
	attr.string[0] = (char) 0;
	hit = 0;
	for(verticies = 0;  verticies < 2;)
	{	if(dxfreadpair(string1,string2))
			return(1);

				/********* check for x or y coordinate ********/
		if((!strcmp(string1,"10"))  ||  (!strcmp(string1,"11")))
		{	sscanf(string2,"%f",&coordlist.x);      /*** x coordinate ***/
			hit |= 1;
		}
		else if((!strcmp(string1,"20"))  ||  (!strcmp(string1,"21")))
		{	sscanf(string2,"%f",&coordlist.y);      /*** y coordinate ***/
			hit |= 2;
		}
		else if((!strcmp(string1,"30")) || (!strcmp(string1,"31")) ||
		        (!strcmp(string1,"2")) || (!strcmp(string1,"3")) ||
		        (!strcmp(string1,"4")) || (!strcmp(string1,"5")) ||
		        (!strcmp(string1,"8")) || (!strcmp(string1,"1")))
		{	strncpy(attr.string,string2,30); /* text, symbol name or other atrbt */
			attr.string[30] = (char) 0;
		}

			/*********** if both coordinates entered, store & reset hit flag **/
		if(hit == 3)
		{	if(put_vcdxf(&coordlist))
			{	impdxfabort(5);
				return(1);
			}
			hit = 0;                  /** reset hit flag **/
			verticies++;
		}
	}

		/********** check number of verticies ********/
	if(verticies != 2)
		return(1);

		/********** store attribute record, if there was an elevation **/
	if(attr.string[0])
	{	attr.next = 0;
		v_object.atr = job_stats[16];
		if(put_v_attr(&attr,job_stats[16]))
		{	impdxfabort(2);
			return(1);
		}
	}
	
		/********** store object record *****************************/
	v_object.type = otype + 1;   /*** line  on level otype **/

		/***** set ending vertex *******/
	v_object.v2 = job_stats[18] - 1;

		/***** store v_object record ****/
	if(put_vobjects(&v_object,job_stats[17]))
	{	impdxfabort(4);
		return(1);
	}

		/********** return ******************************************/
	return(0);
}


/*******************************************************************************
*    dxfpoint      reads one piont    record in the dxf file                   *
*******************************************************************************/
long dxfpoint()
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	long verticies;

	char string1[256],string2[256];

	long hit;


		/********** set up v_object record **************************/
	v_object.v1 = job_stats[18];
	v_object.atr = 0;


		/********** read  vertex & store v_object *************/
	attr.string[0] = (char) 0;
	hit = 0;
	for(verticies = 0;  verticies < 1;)
	{	if(dxfreadpair(string1,string2))
			return(1);

				/********* check for x or y coordinate ********/
		if((!strcmp(string1,"10"))  ||  (!strcmp(string1,"11")))
		{	sscanf(string2,"%f",&coordlist.x);      /*** x coordinate ***/
			hit |= 1;
		}
		else if((!strcmp(string1,"20"))  ||  (!strcmp(string1,"21")))
		{	sscanf(string2,"%f",&coordlist.y);      /*** y coordinate ***/
			hit |= 2;
		}
		else if((!strcmp(string1,"30")) || (!strcmp(string1,"31")) ||
		        (!strcmp(string1,"2")) || (!strcmp(string1,"3")) ||
		        (!strcmp(string1,"4")) || (!strcmp(string1,"5")) ||
		        (!strcmp(string1,"8")) || (!strcmp(string1,"1")))
		{	strncpy(attr.string,string2,30); /* text, symbol name or other atrbt */
			attr.string[30] = (char) 0;
		}

			/*********** if both coordinates entered, store & reset hit flag **/
		if(hit == 3)
		{	if(put_vcdxf(&coordlist))
			{	impdxfabort(5);
				return(1);
			}
			hit = 0;                  /** reset hit flag **/
			verticies++;
			break;
		}
	}

		/********** check number of verticies ********/
	if(verticies != 1)
		return(1);

		/********** store attribute record, if there was an elevation **/
	if(attr.string[0])
	{	attr.next = 0;
		v_object.atr = job_stats[16];
		if(put_v_attr(&attr,job_stats[16]))
		{	impdxfabort(2);
			return(1);
		}
	}
	
		/********** store object record *****************************/
	v_object.type = otype;   /*** point on level otype **/

		/***** set ending vertex *******/
	v_object.v2 = job_stats[18] - 1;

		/***** store v_object record ****/
	if(put_vobjects(&v_object,job_stats[17]))
	{	impdxfabort(4);
		return(1);
	}

		/********** return ******************************************/
	return(0);
}



/*******************************************************************************
*    dxftext       reads one text     record in the dxf file                   *
*******************************************************************************/
long dxftext()
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	long verticies;

	char string1[256],string2[256];

	long hit;


		/********** set up v_object record **************************/
	v_object.v1 = job_stats[18];
	v_object.atr = 0;


		/********** read  vertex & store v_object *************/
	attr.string[0] = (char) 0;
	hit = 0;
	for(verticies = 0;  verticies < 1;)
	{	if(dxfreadpair(string1,string2))
			return(1);

				/********* check for x or y coordinate ********/
		if((!strcmp(string1,"10"))  ||  (!strcmp(string1,"11")))
		{	sscanf(string2,"%f",&coordlist.x);      /*** x coordinate ***/
			hit |= 1;
		}
		else if((!strcmp(string1,"20"))  ||  (!strcmp(string1,"21")))
		{	sscanf(string2,"%f",&coordlist.y);      /*** y coordinate ***/
			hit |= 2;
		}
		else if(!strcmp(string1,"1"))
		{	strncpy(attr.string,string2,30); /* text vaule ***********************/
			attr.string[30] = (char) 0;
			hit |= 4;
		}
		else if((!strcmp(string1,"30")) || (!strcmp(string1,"31")) ||
		        (!strcmp(string1,"2")) || (!strcmp(string1,"3")) ||
		        (!strcmp(string1,"4")) || (!strcmp(string1,"5")) ||
		        (!strcmp(string1,"8")) || (!strcmp(string1,"1")))
		{	strncpy(attr.string,string2,30); /* text, symbol name or other atrbt */
			attr.string[30] = (char) 0;
		}

			/*********** if both coordinates entered, store & reset hit flag **/
		if(hit == 7)
		{	if(put_vcdxf(&coordlist))
			{	impdxfabort(5);
				return(1);
			}
			hit = 0;                  /** reset hit flag **/
			verticies++;
			break;
		}
	}

		/********** check number of verticies ********/
	if(verticies != 1)
		return(1);

		/********** store attribute record, if present *****************/
	if(attr.string[0])
	{	attr.next = 0;
		v_object.atr = job_stats[16];
		if(put_v_attr(&attr,job_stats[16]))
		{	impdxfabort(2);
			return(1);
		}
	}
	
		/********** store object record *****************************/
	v_object.type = otype;   /*** point on level otype **/

		/***** set ending vertex *******/
	v_object.v2 = job_stats[18] - 1;

		/***** store v_object record ****/
	if(put_vobjects(&v_object,job_stats[17]))
	{	impdxfabort(4);
		return(1);
	}

		/********** return ******************************************/
	return(0);
}



/*******************************************************************************
*    dxfcircle     reads one circle   record in the dxf file                   *
*******************************************************************************/
long dxfcircle()
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	float centerx,centery;
	float radius;

	double a, da;

	long verticies;
	long i;
	long hit;

	char string1[256],string2[256];


		/********** set up v_object record **************************/
	v_object.v1 = job_stats[18];
	v_object.atr = 0;


		/********** read  vertex & store v_object *************/
	attr.string[0] = (char) 0;
	hit = 0;
	for(verticies = 0;  verticies < 1;)
	{	if(dxfreadpair(string1,string2))
			return(1);

				/********* check for x or y coordinate ********/
		if((!strcmp(string1,"10"))  ||  (!strcmp(string1,"11")))
		{	sscanf(string2,"%f",&centerx);      /*** x coordinate ***/
			hit |= 1;
		}
		else if((!strcmp(string1,"20"))  ||  (!strcmp(string1,"21")))
		{	sscanf(string2,"%f",&centery);      /*** y coordinate ***/
			hit |= 2;
		}
		else if((!strcmp(string1,"40"))  ||  (!strcmp(string1,"41")))
		{	sscanf(string2,"%f",&radius);      /*** radius       ***/
			hit |= 4;
		}
		else if((!strcmp(string1,"30")) || (!strcmp(string1,"31")) ||
		        (!strcmp(string1,"2")) || (!strcmp(string1,"3")) ||
		        (!strcmp(string1,"4")) || (!strcmp(string1,"5")) ||
		        (!strcmp(string1,"8")) || (!strcmp(string1,"1")))
		{	strncpy(attr.string,string2,30); /* text, symbol name or other atrbt */
			attr.string[30] = (char) 0;
		}

			/*********** if both coordinates entered, store & reset hit flag **/
		if(hit == 7)
		{		/*** divide circle into 100 segments ***/
			da = 3.1415927 * 2.0 / 100;
			for(a=0.0, i=0; i< 100; a+=da, i++)
			{	coordlist.x = centerx + radius * cos(a);
				coordlist.y = centery + radius * sin(a);
				if(put_vcdxf(&coordlist))
				{	impdxfabort(5);
					return(1);
				}
			}
			coordlist.x = centerx + radius * cos(0.0);
			coordlist.y = centery + radius * sin(0.0);
			if(put_vcdxf(&coordlist))
			{	impdxfabort(5);
				return(1);
			}
			hit = 0;                  /** reset hit flag **/
			verticies = 101;
			break;
		}
	}

		/********** check number of verticies ********/
	if(verticies != 101)
		return(1);

		/********** store attribute record, if there was an elevation **/
	if(attr.string[0])
	{	attr.next = 0;
		v_object.atr = job_stats[16];
		if(put_v_attr(&attr,job_stats[16]))
		{	impdxfabort(2);
			return(1);
		}
	}
	
		/********** store object record *****************************/
	v_object.type = otype + 2;   /*** polygon on level otype **/

		/***** set ending vertex *******/
	v_object.v2 = job_stats[18] - 1;

		/***** store v_object record ****/
	if(put_vobjects(&v_object,job_stats[17]))
	{	impdxfabort(4);
		return(1);
	}

		/********** return ******************************************/
	return(0);
}



/*******************************************************************************
*    dxfarc        reads one arc      record in the dxf file                   *
*******************************************************************************/
long dxfarc()
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	float centerx,centery;
	float radius;
	float a_begin, a_end;

	double a, da;

	long verticies;
	long hit;

	char string1[256],string2[256];


		/********** set up v_object record **************************/
	v_object.v1 = job_stats[18];
	v_object.atr = 0;


		/********** read  vertex & store v_object *************/
	attr.string[0] = (char) 0;
	hit = 0;
	for(verticies = 0;  verticies < 1;)
	{	if(dxfreadpair(string1,string2))
			return(1);

				/********* check for x or y coordinate ********/
		if((!strcmp(string1,"10"))  ||  (!strcmp(string1,"11")))
		{	sscanf(string2,"%f",&centerx);      /*** x coordinate ***/
			hit |= 1;
		}
		else if((!strcmp(string1,"20"))  ||  (!strcmp(string1,"21")))
		{	sscanf(string2,"%f",&centery);      /*** y coordinate ***/
			hit |= 2;
		}
		else if((!strcmp(string1,"40"))  ||  (!strcmp(string1,"41")))
		{	sscanf(string2,"%f",&radius);      /*** radius        ***/
			hit |= 4;
		}
		else if(!strcmp(string1,"50"))     
		{	sscanf(string2,"%f",&a_begin);      /*** begin angle  ***/
			hit |= 8;
		}
		else if(!strcmp(string1,"51"))     
		{	sscanf(string2,"%f",&a_end );      /*** end   angle  ***/
			hit |= 16;
		}
		else if((!strcmp(string1,"30")) || (!strcmp(string1,"31")) ||
		        (!strcmp(string1,"2")) || (!strcmp(string1,"3")) ||
		        (!strcmp(string1,"4")) || (!strcmp(string1,"5")) ||
		        (!strcmp(string1,"8")) || (!strcmp(string1,"1")))
		{	strncpy(attr.string,string2,30); /* text, symbol name or other atrbt */
			attr.string[30] = (char) 0;
		}

			/*********** if both coordinates entered, store & reset hit flag **/
		if(hit == 31)
		{		/*** divide arc into segments **********/
				/*** convert angle to radians ***/
			a_begin = a_begin / 360.0 * 2.0 * 3.1415927;
			a_end = a_end / 360.0 * 2.0 * 3.1415927;
			if(a_begin > a_end)
			{	da = a_begin;
				a_begin = a_end;
				a_end = da;
			}
			da = 3.1415927 * 2.0 / 100;
			for(a=a_begin; a<a_end; a+=da)
			{	coordlist.x = centerx + radius * cos(a);
				coordlist.y = centery + radius * sin(a);
				if(put_vcdxf(&coordlist))
				{	impdxfabort(5);
					return(1);
				}
			}
			coordlist.x = centerx + radius * cos(a_end);
			coordlist.y = centery + radius * sin(a_end);
			if(put_vcdxf(&coordlist))
			{	impdxfabort(5);
				return(1);
			}
			hit = 0;                  /** reset hit flag **/
			verticies = 101;
			break;
		}
	}

		/********** check number of verticies ********/
	if(verticies != 101)
		return(1);

		/********** store attribute record, if there was an elevation **/
	if(attr.string[0])
	{	attr.next = 0;
		v_object.atr = job_stats[16];
		if(put_v_attr(&attr,job_stats[16]))
		{	impdxfabort(2);
			return(1);
		}
	}
	
		/********** store object record *****************************/
	v_object.type = otype + 1;   /*** segment on level otype **/

		/***** set ending vertex *******/
	v_object.v2 = job_stats[18] - 1;

		/***** store v_object record ****/
	if(put_vobjects(&v_object,job_stats[17]))
	{	impdxfabort(4);
		return(1);
	}

		/********** return ******************************************/
	return(0);
}


/*******************************************************************************
*    dxftrace      reads one trace    record in the dxf file                   *
*******************************************************************************/
long dxftrace()
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	float x1, y1;
	long verticies;

	char string1[256],string2[256];

	long hit;


		/********** set up v_object record **************************/
	v_object.v1 = job_stats[18];
	v_object.atr = 0;


		/********** read all verticies & store v_object *************/
	attr.string[0] = (char) 0;
	hit = 0;
	for(verticies = 0;  verticies < 4;)
	{	if(dxfreadpair(string1,string2))
			return(1);

				/********* check for x or y coordinate ********/
		if((!strcmp(string1,"10"))  ||  (!strcmp(string1,"11")) ||
		   (!strcmp(string1,"12"))  ||  (!strcmp(string1,"13")))
		{	sscanf(string2,"%f",&coordlist.x);      /*** x coordinate ***/
			hit |= 1;
		}
		else if((!strcmp(string1,"20"))  ||  (!strcmp(string1,"21"))
		     || (!strcmp(string1,"22"))  ||  (!strcmp(string1,"23")))
		{	sscanf(string2,"%f",&coordlist.y);      /*** y coordinate ***/
			hit |= 2;
		}
		else if((!strcmp(string1,"30")) || (!strcmp(string1,"31")) ||
		        (!strcmp(string1,"2")) || (!strcmp(string1,"3")) ||
		        (!strcmp(string1,"4")) || (!strcmp(string1,"5")) ||
		        (!strcmp(string1,"8")) || (!strcmp(string1,"1")))
		{	strncpy(attr.string,string2,30); /* text, symbol name or other atrbt */
			attr.string[30] = (char) 0;
		}

			/*********** if both coordinates entered, store & reset hit flag **/
		if(hit == 3)
		{	if(put_vcdxf(&coordlist))
			{	impdxfabort(5);
				return(1);
			}
			hit = 0;                  /** reset hit flag **/
			verticies++;
			if(verticies == 1)        /** save first vertex **/
			{	x1 = coordlist.x;
				y1 = coordlist.y;
			}
		}
	}

		/********** check number of verticies ********/
	if(verticies != 4)
		return(1);

		/********** close polygon   ******************/
	coordlist.x = x1;
	coordlist.y = y1;
	if(put_vcdxf(&coordlist))
	{	impdxfabort(5);
		return(1);
	}

		/********** store attribute record, if there was an elevation **/
	if(attr.string[0])
	{	attr.next = 0;
		v_object.atr = job_stats[16];
		if(put_v_attr(&attr,job_stats[16]))
		{	impdxfabort(2);
			return(1);
		}
	}
	
		/********** store object record *****************************/
	v_object.type = otype + 2;   /*** polygon on level otype **/

		/***** set ending vertex *******/
	v_object.v2 = job_stats[18] - 1;

		/***** store v_object record ****/
	if(put_vobjects(&v_object,job_stats[17]))
	{	impdxfabort(4);
		return(1);
	}

		/********** return ******************************************/
	return(0);
}

/*******************************************************************************
*    put_vcdxf  stores one pair of vector coordinates                          *
*******************************************************************************/

put_vcdxf(coordlist)
COORD_LIST *coordlist;
{
	double x,y;

		/******** transform coords to pixels *********************/
	x = coordlist->x;
	y = coordlist->y;
	transform(&x,&y,input_system,0);
	coordlist->x = x;
	coordlist->y = y;

		/******** write coords to vertex file ********************/
	if(put_v_coord(coordlist,job_stats[18]))  /** store coords **/
		return(1);

		/******** return *****************************************/
	return(0);
}

