/*******************************************************************************
*       impgis         (see manual, Command Ref., User Functions)      <fc>    *
*            This program imports a gis  formatted ascii file.                 *
*        The old raster is replaced by the new raster, which may be a          **        different size.  All vector type files, (map and import levels)       *
*        are erased as the the new raster forms a new coordinate basis         *
*        for all files.                                                        *
*                                                                              *
*            Parameters:   char *fname       the name of the gis  file to      *
*                                            import.                           *
*                          long a            attribute flag:                   *
*                                             1=raser obj, 2=atrbts, 3=both    *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist, raster, mini-raster, attributes, errors,        *
*                 register                                                     *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   6/19/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

int fd, fd2;
char *malloc_pt;

long impgisabort();

long impgis( fname , a)
   char *fname;
	long a;
{
   OBJECTS object;

   char actual[256],actual2[256];
   char string[256];
	char path[256];
   char *ptr, *endptr;
   unsigned char *top_row;
   unsigned char *cntr_row;
   char *check_str[2];
   char *new_ptr, *old_ptr;
   char s1[96];
	unsigned char record[132], *record_ptr;
	char fname2[256];
	char labels[384], *label[12];

   long width;                  /* width of 1 raster row (pixels) */
   long h,i,j,w;
   long size;
   long rows;                   /* number of raster rows */
   long x,y;
   long byte,bit;
   long new, old;
   long ipack;
   long row, col, rcols;
	long maptyp, height;
	long raster_size;
	long rbyte, rshift;
	long hit;
	long offset, offset_rows, offset_cols;
	long value;

   float floatx,floaty;
   float *fptr;
   float xmap,ymap,xcell,ycell;
   
   double coord_pairs[4][2];
	double rowlen;
	double foffset;


      /************* initialize fd & malloc globals ******************/
   fd = -1;
   fd2 = -1;
   malloc_pt = (char *) 0;


      /************* check for map present ***************************/
   if(!job_stats[0])
   {   message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
      return(1);
   }


   message("import gis, setting up variables.");
      /************* find & asgn pointer to import file **************/
   if(get_impfile(fname,actual))
   {   message("import gis, abort, location 1.");
      return(1);
   }
   fd = openf(actual,READ);
   if(fd <= 0)
   {   message("Cannot open input file, abort, location 2.");
      return(1);
   }

		/**** set display coordinate type to 0 (raster pixels) ******/
	disp_coord_type = 0;


   message("Decoding file header.");
      /************* find raster ht & wd *****************************/
      /************* read file header ********************************/
   readb(string, 128, 1, fd);
   ptr = string;
   ptr[512] = (char) 0;

      /** read "header" **/
   strncpy(s1,ptr,6);
   s1[6] = (char) 0;
   if(strcmp("HEADER",s1))
   {      /**** notify user & abort ****/
      message("File does not have \"HEADER\" as lead string, aborting.");
      return(1);
   }

      /** read ipack type **/
   ipack = (long) string[6];
      /** check ipack ***/
   if((ipack < 0) || (ipack > 2))
   {      /**** notify user & abort ****/
      message("File has illegal ipack type, abortig.");
      return(0);
   }

      /** read rcols, rows **/
   fptr = (float *) (string + 16);
   rcols = (long) *fptr;
   fptr = (float *) (string + 20);
   rows = (long) *fptr;

   if(rows < 1  ||  rcols < 1)
   {      /**** notify user & abort ****/
      message("File has illegal number of rows or columns, abortig.");
      return(0);
   }


      /** read maptype (coord type) **/
   maptyp = string[88];        /** 0=none, 1=utm, 2=state plane, 3=albers **/

      /** read xmap & ymap **/
   fptr = (float *) (string + 112);
   xmap = *fptr;
   fptr = (float *) (string + 116);
   ymap = *fptr;
   
      /** read xcell & ycell **/
   fptr = (float *) (string + 120);
   xcell = *fptr;
   fptr = (float *) (string + 124);
   ycell = *fptr;
  

	if(a & 1)   /***** if raster data is wanted ****/
   {	   /*************** free current map malloc space *****************/
   	message("Updating temporary map file.");
		if(changes())
   		update_temp();
   	free_malloc();
	
      	/****** make width an integral multiple of 32 **/
   	width = (rcols * 2) + 9;
   	height = rows * 2 + 9;
   	if(width % 32)
   	{   i = width / 32 + 1;
      	width = i * 32;
   	}
   	job_stats[1] = width - 1;
   	job_stats[2] = height;
	
	
      	/************* malloc space for raster ************/
      	/***********  calculate sizes **************/
   	h = job_stats[2] + 1;             /* height of raster in rows  */
   	w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */
   	raster_size = h*w;                /* size of raster in bytes   */
  	 
   	size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster/r */
      	/***********  allocate memory **************/
   	rasterm.malloc = (char *) calloc_ltp(size,1);
   	if(!rasterm.malloc)
   	{   message("  <<  cannot get enough RAM for raster.");
      	remove_temp();
      	return(0);
   	}
	
      	/********  calculate & set global pointers to raster ****************/
   	rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
   	rasterm.recn = rasterm.rec0 + raster_size;
   	craster = (unsigned char **)(rasterm.malloc);
   	lraster = (unsigned long **) (rasterm.malloc + 4*h);
   	for(i=0; i<h; i++)
   	{   craster[i] = (unsigned char *)rasterm.rec0 + w*i;
      	lraster[i] = (unsigned long *) (craster[i]);
   	}
	
	
      	/************ malloc space for input rows ***********************/
   	malloc_pt = calloc_ltp(2,rcols+1);
   	if(malloc_pt == (char *) 0)
   	{		/**** inform user & abort ***/
			impgisabort(1);
      	return(1);
   	}
	
      	/************ set pointers **************************************/
   	check_str[0] = malloc_pt+1;
   	check_str[1] = malloc_pt + rcols+2;
   	new = 0;
   	old = 1;
	
	
   	rbyte=128;
   	for(row=job_stats[2] - 4; row > 4; row -= 2, new ^= 1, old ^=1)
   	{      /************ fill new row   **********************/
      	new_ptr=check_str[new];
      	old_ptr=check_str[old];
	
			if(!(row % 100))
			{	sprintf(string," entering gis file, row %d",row);
				message(string);
			}
	
      	switch(ipack) {
      	case 1:                 /*** 4 bit ***/
         	for(col=0; col < rcols; col++, new_ptr++)
         	{      /*** read more data if necessary ***/
            	if(rbyte >= 128)
            	{   readb(record, 128, 1, fd);
               	rbyte=0;
						record_ptr = record;
               	rshift=0;
            	}
               	/*** expand data to fill row ***/
            	if(rshift)
            	{  *new_ptr = (*record_ptr) >> 4;
               	rbyte++;
               	record_ptr++;
					}
            	else
               	*new_ptr = *record_ptr & 15;
            	rshift ^= 1;
         	}
         	break;
      	case 0:              /*** 8 bit ***/
         	for(col=0; col < rcols; col++, new_ptr++)
         	{      /*** read more data if necessary ***/
            	if(rbyte >= 128)
            	{   readb(record, 128, 1, fd);
               	rbyte=0;
						record_ptr = record;
               	rshift=1;
            	}
               	/*** expand data to fill row ***/
            	*new_ptr = *record_ptr;
            	rbyte++;
            	record_ptr++;
         	} 
         	break;
      	case 2:              /** 16 bit ***/
         	for(col=0; col < rcols; col++, new_ptr++)
         	{      /*** read more data if necessary ***/
            	if(rbyte >= 128)
            	{   readb(record, 128, 1, fd);
               	rbyte=0;
						record_ptr = record;
               	rshift=1;
            	}
               	/*** expand data to fill row ***/
            	*new_ptr = *(record_ptr+1);
            	rbyte+=2;
            	record_ptr+=2;
         	} 
         	break;
      	}
     	 
         	/*** reset pointers ***/
      	new_ptr = check_str[new];
			top_row = craster[row + 1];
			cntr_row= craster[row];
	
         	/****** check for differences above & left ***/
      	for(i=0, x=5; i < rcols; i++, x+=2, new_ptr++, old_ptr++)
      	{   if(*new_ptr != *old_ptr)   /** difference above ******/
         	{   for(j=x-1; j<x+2; j++)
            	{   byte = j >> 3;
               	bit = 7 & (~j);
               	top_row[byte] |= (char) 1 << bit;
            	}
         	}
         	if(*new_ptr != *(new_ptr-1))   /** difference left  ******/
         	{   byte = (x-1) >> 3;
            	bit = 7 & (~(x-1));
            	top_row[byte] |= (char) 1 << bit;
            	cntr_row[byte] |= (char) 1 << bit;
         	}
      	}
   	}
	
      	/************* close files *************************************/
   	closef_ptr(&fd);
   	fd = -1;
		if(malloc_pt > (char *) 0)
		{	free_ltp(malloc_pt);
			malloc_pt = (char *) 0;
		}
	
      	/*******   write raster to temp file  *****/
		message("writting raster to temp file.");
      	/*  open file   */
   	sprintf(path,"%s/raster",temp_map_path);
   	if((fd = openf(path,WRITE)) <= 0)
   	{   message("Cannot open raster temp file.  Aborting.");
      	return(1);
   	}
      	/*  write all records  */
   	size = (job_stats[1]+1)  *  (job_stats[2]+1)  /  8;
   	if(0 > writeb(rasterm.rec0, size , 1, fd))
   	{   message("Cannot write raster.  You must re-acquire map & start over.");
      	closef_ptr(&fd);
			fd = -1;
      	job_stats[0] = 0;
      	return(1);
   	}

      	/*  close file  */
   	closef_ptr(&fd);
		fd = -1;

		
   	message("resetting map vairables & loading new raster into display RAM.");
      	/************* reset job_stats *********************************/
   	for(i=3; i<=7; i++)
      	job_stats[i] = 0;
   	for(i=10; i<=18; i++)
   	{   if((i==13)  ||  (i==15)  ||  (i==16))
         	job_stats[i] = (job_stats[i])  ?    1 : 0;
      	else
         	job_stats[i] = 0;
   	}
   	job_stats[45]++;
   	job_stats[27] = job_stats[45];
   	job_stats[49] = 0;
   	job_stats[75] = 0;
	

      	/* store new sized index_array file     */
      	/***********  calculate sizes **************/
   	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
   	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
   	size = (h*w);                     /* size to calloc            */
      	/***********  allocate memory **************/
   	index_arraym.malloc = (char *) calloc_ltp(1,size);
   	if(index_arraym.malloc == (char *) 0)
   	{   message("  <<  cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
      	job_stats[0] = 0;
      	map_name[0] = 0;
      	return(1);
   	}
	
      	/***********  write index_array to file *********************/
   	sprintf(string,"%s/index_array",temp_map_path);
   	fd = openf(string,WRITE);
   	if(0 > fd)
   	{   message("  <<  cannot open index_array map file.  Re-ACQUIRE map & start over.");
      	job_stats[0] = 0;
      	return(0);
   	}
   	if(0 > writeb(index_arraym.malloc,size,1,fd))
   	{   message("  <<  cannot write index_array map file.  Re-ACQUIRE map & start over.");
      	job_stats[0] = 0;
      	closef_ptr(&fd);
			fd = -1;
      	return(0);
   	}
   	closef_ptr(&fd);
		fd = -1;
	
   	free_ltp(index_arraym.malloc);
   	index_arraym.malloc = (char *) 0;
	
      	/***********  write job_stats to file ***********************/
   	sprintf(string,"%s/job_stats",temp_map_path);
   	if(put_jobstats(string))
      	{   message("Cannot write job_stats to temp file.    Re-ACQUIRE map & start over.");
      		job_stats[0] = 0;
         	return(1);
      	}

      	/************* re-acquire temp file ****************************/
   	if(reacquire())
   	{   message("You must re-acquire the map & start over.");
      	job_stats[0] = 0;
      	return(1);
   	}
	

      	/************* un-register raster ******************************/
   	for(i=0; i<4; i++)
   	{   corner[0][i][0] = 0.0;
      	corner[0][i][1] = 0.0;
   	}

      	/***** register raster at gis file corners ***/
   	coord_pairs[0][0] = 4.0;                /** sw **/
   	coord_pairs[0][1] = 4.0;
	
   	coord_pairs[1][0] = 4.0;                /** nw **/
   	coord_pairs[1][1] = job_stats[2] - 3.0;
	
   	coord_pairs[2][0] = rcols * 2 + 4.0;    /** ne **/
   	coord_pairs[2][1] = job_stats[2] - 3.0;
	
   	coord_pairs[3][0] = rcols * 2 + 4.0;    /** se **/
   	coord_pairs[3][1] = 4.0;
	
   	register_ltp(0, 0L, coord_pairs, 4L, (char) 0);

      	/***** utm or state plane registration *******/
   	coord_pairs[1][0] = xmap - xcell / 2.0;      /** nw **/
   	coord_pairs[1][1] = ymap + ycell / 2.0;
	
   	coord_pairs[0][0] = coord_pairs[1][0];       /** sw **/
   	coord_pairs[0][1] = coord_pairs[1][1] - rows * ycell;
	
   	coord_pairs[2][0] = coord_pairs[1][0] + rcols * xcell;     /** ne **/
   	coord_pairs[2][1] = coord_pairs[1][1];
	
   	coord_pairs[3][0] = coord_pairs[2][0];       /** se **/
   	coord_pairs[3][1] = coord_pairs[0][1];
	
		switch(maptyp) {
		case 1:             /** utm **/
   		register_ltp(2, 0L, coord_pairs, 4L, (char) 0);
			break;
		case 2:             /** state plane **/
   		register_ltp(3, 0L, coord_pairs, 4L, (char) 0);
			break;
		}
	

			/***** frame map  *****/
   	if(frame_map(1,0))
   	{   impgisabort(2);
      	message("Cannot frame map.  aborting.");
      	return(1);
   	}

      	/***** set maptype to polygon ****/
   	if(!header_ltp(0,"maptype","polygon"))
   	{   impgisabort(3);
      	message("Cannot set map type to polygon.  aborting.");
      		return(1);
   	}

      	/************* write diary record ******************************/
   	sprintf(string,"imported gis file %s  (%d x %d)",fname,job_stats[2]+1,job_stats[1]+1);
   	put_diary(string);
	}
	else
   {  	/************* close files *************************************/
   	closef_ptr(&fd);
   	fd = -1;
	}
	

	if(a & 2)   /***  attributes are wanted ****/
	{		/**************** init labels to be numeric codes **********/
		for(i=0; i<12; i++)
		{	label[i] = labels + i*32;
			if(i < 10)
				sprintf(label[i],"0%d",i);
			else
				sprintf(label[i],"%d",i);
		}
	
	
	
			/**************** get .TRL file if possible ****************/
   	message("looking for .TRL (trailer) file of attribute codes . . . ");
      	/************* find & asgn pointer to import file **************/
		strcpy(fname2,fname);
		ptr = fname2 + strlen(fname2) - 3;
		hit = 0;
		if(ptr >= fname2)
		{	if(!strcmp(ptr,"gis"))
			{	strcpy(ptr,"trl");
				hit = 1;
			}
			else if(!strcmp(ptr,"GIS"))
			{	strcpy(ptr,"TRL");
				hit = 1;
			}
		}
	
			
		if(hit)
		{	if(get_impfile(fname2,actual2))
				message("TRL file not found, attributes will be numbers 0 - 11.");
			else
   		{	fd = openf(actual2,READ);
   			if(fd <= 0)
					message("TRL file cannot be opened, attributes will be numbers 0 - 11.");
				else
				{		/*** read past records 1-16 ***/
					for(i=1; i<17; i++)
						readb(string,128,1,fd);
	
						/*** read class name records 17, 18, 19  ***/
					readb(labels,128,3,fd);
	
						/*** add nulls to break apart ***/
					for(i=0; i<12; i++)
						label[i][30] = (char) 0;
	
						/*** set '~' to null ***/
					endptr = labels + 32 * 12;
					for(ptr = labels; ptr < endptr; ptr++)
					{	if(*ptr == '~')
							*ptr = (char) 0;
					}
	
						/*** close .TRL file ***/
					closef_ptr(&fd);
					fd = -1;
				}

					/************* remove temp file, if any ************************/
				imp_rm_tmp(fname2,actual2);

			}
		}
	
	
      	/******* re-open file                         *************/
   	fd = openf(actual,READ);
   	if(fd <= 0)
      	message("Cannot open input file. registration and/or attribute data not read.");
   	else
   	{
			if(a == 3)  /** user wants both raster & attributes **/
         {		/***** assemble map ***/
      		if(assemble(1))
      		{   impgisabort(4);
         		message("Cannot assemble map.  aborting.");
         		return(1);
      		}
			}
	
	
				/*** set rowlen variable ***/
  			switch(ipack) {
			case 1:    /** 4 bits **/
				rowlen = rcols / 2.0;
				break;
			case 0:    /** 8 bits **/
				rowlen = rcols;
				break;
			case 2:    /** 16 bts **/
				rowlen = rcols * 2;
				break;
			}
	
         	/*** read through all objects attributing interior polys   ***/
      	for(i=job_stats[7]; i<job_stats[10]; i++)
			{		/*** check for user abort ***/
				if(abort_cmd())
					break;

      	      /*** get object record ***/
         	if(get_objects(&object,i))
         	{   impgisabort(7);
            	return(1);
         	}
            	/*** check to see if it is an interior polygon ***/
         	if(object.i12[0] == 14)
         	{      /*** find it's centroid ***/
            	floatx = object.f4[0] + 0.01;
            	floaty = object.f4[1] + 0.01;
	
					x = floatx;
					y = floaty;
	
						/*** calculate the position in the .gis file ***/
					offset_rows = (job_stats[2] - 4 - y) / 2;
					offset_cols = (x - 5) / 2;
      			switch(ipack) {
					case 1:    /** 4 bits **/
						foffset = 128.0 + offset_rows * rowlen + offset_cols / 2.0;
						offset = foffset;
						rshift = ((rcols & offset_rows) ^ offset_cols) & 1;
						break;
					case 0:    /** 8 bits **/
						offset = 128 + offset_rows * rowlen + offset_cols;
						break;
					case 2:    /** 16 bts **/
						offset = 128 + offset_rows * rowlen + offset_cols * 2;
						break;
					}
	
						/*** move file pointer ***/
					if(0 > movefp(fd,offset,0))
					{	impcellabort(24);
						message("Move file pointer failed.  Aborting.");
						return(1);
					}
	
						/*** read .gis file ***/
   				readb(record, 2, 1, fd);
	
						/*** get attribute code ***/
      			switch(ipack) {
					case 1:    /** 4 bits **/
            		if(rshift)
            	   	value = *record >> 4;
            		else
               		value = *record & 15;
						break;
					case 0:    /** 8 bits **/
						value = *record;
						break;
					case 2:    /** 16 bts **/
						value = *record + ((record[1]) << 8);
						break;
					}
	
					if((value < 0)  ||  (value > 11))
					{	message("Encountered illegal value, set value to 0.");
						value = 0;
					}
					
						/*** assign attribute ***/
					if(asgn_atrbt(i,1,label[value]))
					{	impcellabort(26);
						message("Read file failed.  Aborting.");
						return(1);
					}
         	}
      	}
   		/*** close file ***/
   		closef_ptr(&fd);
   		fd = -1;
   	}
	}
	
     	/*** reset undo array    **********/
  	for(i=0; i<32; i++)
     	undo_array[i].mode = 0;
	

		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);

	
		/************* create rasterb **********************************/
	brstr_create();

      /************* return ******************************************/
   return(0);
}



/*******************************************************************************
*    impgisabort                                                             *
*        This program closes files and frees malloc space and prints an abort  *
*    message on the screen which identifies the abort location.                *
*    Parameter:                long location                                   *
*    Files & globals changed:  none                                            *
*    Value returned:  long 0;                                                  *
*    J.Dabritz    10/13/89                                                     *
*******************************************************************************/

long impgisabort(location)
   long location;
{
   char string[256];

      /************ close files **************************************/
   if(fd > 0)
      closef_ptr(&fd);
   if(fd2 > 0)
      closef_ptr(&fd2);

      /************ free malloc space ********************************/
   if(malloc_pt != (char *) 0)
   {   free_ltp(malloc_pt);
      malloc_pt = (char *) 0;
   }

      /************ print message ************************************/
   sprintf(string,"Aborting import gis, location %d.",location);
   message(string);


      /************ re-acquire temp file *****************************/
   if(reacquire())
   {   message("You must re-acquire the map & start over.");
      job_stats[0] = 0;
      return(1);
   }


      /************ return *******************************************/
   return(0);
}
