/*******************************************************************************
*   impmap      (see manual, Command Ref., User Functions)  <fc>               *
*                                                                              *
*       This program reads an LTPLus map file/directory and stores the         *
*       data on the specified import 'level' (35 - 95)                         *
*                                                                              *
*            Parameters:   char *fname       the name of the map file to       *
*                                            import.                           *
*                          long level        the level number to import to     *
*                                            (35  - 95  inclusive)             *
*                          char *lname       new name for the level imported   *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist,   (varaibles)  [bunches]                        *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   2/12/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

long impmapabort();
long get_vmapobj();
set_map_attr();

int map_objects_fd;
int map_coord_list_fd;
int map_attr_fd;
long last_js16, last_js17, last_js18;


long impmap( fname, level,lname)
char *fname;
long level;
char *lname;
{
	OBJECTS object,obj_rt,obj_lt;
	V_OBJECTS v_object,poly_rt,poly_lt;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	char actual[256];
	char string[256];
	char job_stats_path[256];
	char objects_path[256];
	char coord_list_path[256];
	char register_path[256];
	char attributes_path[256];

	double x,y;
	double last_x, last_y;
	double map_corner[30][4][2];

	int fd;
	long v;
	long map_job_stats[100];
	long i;
	long node_type, seg_type, poly_type;
	long obj;
	long storing, stored;
	long offset;


	map_objects_fd = -1;
	map_coord_list_fd = -1;
	map_attr_fd = -1;
	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check for raster registration *******************/
	if(!registered(0))
	{	message("You must REGISTER the Raster befor importing a vector file.");
		return(1);
	}

		/************* check for geodetic registration *****************/
	if(!registered(1))
	{	message("You must REGISTER geodetics befor importing.");
		return(1);
	}

		/************* check input parameters **************************/
	if((level < 35)  ||  (level > 95))
	{	message("Illegal level number.  Level number must be between 35 and 95.");
		return(1);
	}


		/******* set up variables *************************************/
	message("import map, setting up variables.");
	last_js16 = job_stats[16];
	last_js17 = job_stats[17];
	last_js18 = job_stats[18];
	fd = -1;


		/************* output v_attribute record 0 dummy record ********/
	if(!job_stats[16])
	{	attr.next = 0;
		attr.string[0] = 0;
		if(put_v_attr(&attr,0))
		{	message("import map, abort, location 0.");
			return(1);
		}
	}

		/************* find & asgn pointer to import files **************/
	trim(fname);
	if(*fname == '/')
		sprintf(actual,"%s",fname);
	else
		sprintf(actual,"%s/%s",maps_path,fname);

	sprintf(job_stats_path,"%s/job_stats",actual);
	sprintf(register_path,"%s/register",actual);
	sprintf(objects_path,"%s/objects",actual);
	sprintf(coord_list_path,"%s/coord_list",actual);
	sprintf(attributes_path,"%s/attributes",actual);
	
		/********* get the job_stats file from the map to be imported. ******/
		/*    open the job_stats fd                               */
	if(0 > (fd = openf(job_stats_path, READ)))
	{	message("Can't open the map/header file.  Aborting import.");
		return(1);
	}
	if(0 > readb((char *)map_job_stats,400,1,fd))
	{	closef_ptr(&fd);
		message("Can't read the map/header file.  Aborting import.");
	 	return(1);
	}
	closef_ptr(&fd);

		/********* get the register file from the map to be imported. ****/
		/*    open the register file                             */
	if(0 > (fd = openf(register_path, READ)))
	{	message("Can't open the map/register file.  Aborting import.");
		return(1);
	}
		/*   read corner array  */
	if(0 > readb((char *)map_corner[0][0], 8, (30 * 4 * 2) ,fd))
	{	closef_ptr(&fd);
		message("Can't read the map/register file.  Aborting import.");
	 	return(1);
	}
	closef_ptr(&fd);

		/*************** check map_job_stats for objects *******************/
	if(!map_job_stats[10])
	{	message("Import map has no vector objects.  Aborting import.");
		return(1);
	}

		/*************** check map_register for geodetics & raster reg. ****/
	if(map_corner[0][0][0] == map_corner[0][2][0])
	{	message("You must register the import map raster befor importing.  Aborting import.");
		return(1);
	}
	if(map_corner[1][0][0] == map_corner[1][2][0])
	{	message("You must register the import map geodetics befor importing.  Aborting import.");
		return(1);
	}

		/*************** set import map raster corners in corner array *****/
	for(i=0; i<4; i++)
	{	corner[8][i][0] = map_corner[0][i][0];
		corner[8][i][1] = map_corner[0][i][1];
	}
		/*************** set import map corners in current raster coords ***/
	for(i=0; i<4; i++)
	{	x = map_corner[1][i][0];
		y = map_corner[1][i][1];
		transform(&x,&y,1,0);
		corner[9][i][0] = x;
		corner[9][i][1] = y;
	}

		/*************** open objects,coord_list,&attributes files *********/
	if(0 > (map_objects_fd = openf(objects_path,READ)))
	{	message("Can't open the map/objects file.  Aborting import.");
		return(1);
	}
	if(0 > (map_coord_list_fd = openf(coord_list_path,READ)))
	{	message("Can't open the map/coord_list file.  Aborting import.");
		closef_ptr(&map_objects_fd);
		return(1);
	}
	if(0 > (map_attr_fd = openf(attributes_path,READ)))
	{	message("Can't open the map/attributes file.  Aborting import.");
		closef_ptr(&map_objects_fd);
		closef_ptr(&map_coord_list_fd);
		return(1);
	}

		/*************** read all map objects & import *********************/
	node_type = 10 * level;
	seg_type = 10 * level + 1;
	poly_type = 10 * level + 2;

	poly_rt.type = poly_type;
	poly_lt.type = poly_type;
	for(obj=0; obj<map_job_stats[10]; obj++)
	{
			/******************** tell user what the hell is happening ******/
		if(!(obj % 50))
		{	sprintf(string,"import map, at object %d.",obj);
			message(string);
		}

			/******************** read  record ******************************/
		if(get_vmapobj(&object,obj))
		{	impmapabort(1);
			return(1);
		}

			/************ if polygon, we're done ****************************/
		if(object.i12[0] > 10)
			break;

			/********** store node or segment/polygon records ***************/
		if(object.i12[0] < 8)       /** node ****/
		{		/*********** if node, check if on raster ********************/
			x = object.f4[0];
			y = object.f4[1];
			transform(&x,&y,8,9);
			if((x >= 0.0) && (x <= (double) job_stats[1])  &&
           	(y >= 0.0) && (y <= (double) job_stats[2]))
			{	v_object.type = node_type;
				v_object.v1 = job_stats[18];
				v_object.v2 = job_stats[18];
					/*********** set attribute record(s) ************************/
				v_object.atr = set_map_attr(object.i12[2]);

					/*********** store v_coordlist record **************************/
				coordlist.x = x;
				coordlist.y = y;
				if(put_v_coord(&coordlist,job_stats[18]))
				{	impmapabort(2);
					return(1);
				}
					/*********** store v_objects record ****************************/
				if(put_vobjects(&v_object,job_stats[17]))
				{	impmapabort(3);
					return(1);
				}
			}
		}	
		else  /*********** segment *************/
		{	v_object.type = seg_type;

				/*********** check verticies for on map *******/
			last_x = 0.0;
			last_y = 0.0;
			storing = 0;
			stored = 0;
			for(v = object.i12[5]; v<= object.i12[6]; v++)
			{		/******** read vertex **********************/
				offset = v * coord_listm.rec_size;
					/**************** position file pointer ***************/
				if(0 > movefp(map_coord_list_fd,offset,0))
					return(1);
					/********* read  record ******************************/
				if(0 > readb(&coordlist, coord_listm.rec_size ,1 ,map_coord_list_fd))
				{	impmapabort(4);
					return(1);
				}

					/******** check to see if vertex on map ****/
				x = coordlist.x;
				y = coordlist.y;
				transform(&x,&y,8,9);
					/*** if OFF raster ****/
				if((x < 0.0) || (x > (double) job_stats[1])  ||
           		(y < 0.0) || (y > (double) job_stats[2]))
				{	if(storing)		/* store v_obj records */
					{		/******** store ending vertex **************/
						coordlist.x = x;
						coordlist.y = y;
						if(put_v_coord(&coordlist,job_stats[18]))
						{	impmapabort(5);
							return(1);
						}
							/******** set end vertex pointers **********/
						v_object.v2 = job_stats[18] - 1;
						if(object.i12[7])
						{	poly_rt.v2 = job_stats[18] - 1;
							poly_lt.v1 = job_stats[18] - 1;
						}
							/******** store v_object record(s);*********/
						if(put_vobjects(&v_object,job_stats[17]))
						{	impmapabort(6);
							return(1);
						}
						if(object.i12[7])  /** store lt & rt polygon records **/
						{	if(put_vobjects(&poly_rt,job_stats[17]))
							{	impmapabort(7);
								return(1);
							}
							if(put_vobjects(&poly_lt,job_stats[17]))
							{	impmapabort(8);
								return(1);
							}
						}
							/******** reset storing variables **********/
						storing = 0;
						stored++;
					}
				}
				else      /*** vertex is on raster ****/
				{		/******** check to see if last vertex stored **/
					if(storing == 0)
					{		/** set segment & poly v_object recs **/
						v_object.v1 = job_stats[18];
						if(!stored)
						{	v_object.atr = set_map_attr(object.i12[2]);
							if(object.i12[7])  /** set lt & rt polygon records **/
							{	if(get_vmapobj(&obj_rt,object.i12[7]))
								{	impmapabort(9);
									return(1);
								}
								poly_rt.v1 = job_stats[18];
								poly_rt.atr = set_map_attr(obj_rt.i12[2]);

								if(get_vmapobj(&obj_lt,object.i12[8]))
								{	impmapabort(10);
									return(1);
								}
								poly_lt.v2 = job_stats[18];
								poly_lt.atr = set_map_attr(obj_lt.i12[2]);
							}
								/** store last vertex, if necessary **/
							if(v > object.i12[5])
							{	coordlist.x = last_x;
								coordlist.y = last_y;
								if(put_v_coord(&coordlist,job_stats[18]))
								{	impmapabort(11);
									return(1);
								}
							}
						}
						else
						{	if(object.i12[7])  /** set lt & rt polygon records **/
							{	
								poly_rt.v1 = job_stats[18];
								poly_lt.v2 = job_stats[18];
							}
							coordlist.x = last_x;
							coordlist.y = last_y;
							if(put_v_coord(&coordlist,job_stats[18]))
							{	impmapabort(12);
								return(1);
							}
						}
							/******** set storing flag ********/
						storing = 1;
					}

						/******** store current vertex *************/
					coordlist.x = x;
					coordlist.y = y;
					if(put_v_coord(&coordlist,job_stats[18]))
					{	impmapabort(13);
						return(1);
					}
						/********* check for last vertex *************/
					if(v == object.i12[6])
					{		/******** set end vertex pointers **********/
						v_object.v2 = job_stats[18] - 1;
						if(object.i12[7])
						{	poly_rt.v2 = job_stats[18] - 1;
							poly_lt.v1 = job_stats[18] - 1;
						}
							/******** store v_object record(s);*********/
						if(put_vobjects(&v_object,job_stats[17]))
						{	impmapabort(14);
							return(1);
						}
						if(object.i12[7])  /** store lt & rt polygon records **/
						{	if(put_vobjects(&poly_rt,job_stats[17]))
							{	impmapabort(15);
								return(1);
							}
							if(put_vobjects(&poly_lt,job_stats[17]))
							{	impmapabort(16);
								return(1);
							}
						}
						stored++;
					}
				}
				last_x = x;
				last_y = y;
			}
		}
	}

	message("import map, closing files.");
		/************* close file **************************************/
	closef_ptr(&map_objects_fd);
	closef_ptr(&map_attr_fd);
	closef_ptr(&map_coord_list_fd);


		/************* index v_objects imported ************************/
	if(index_vobj(last_js17))
	{	job_stats[17] = last_js17;
		return(1);
	}

		/************* write diary *************************************/
	sprintf(string,"import map file %s to level %d.",fname,level);
	put_diary(string);

		/************* set job_stats ***********************************/
	job_stats[45]++;
	job_stats[36] = job_stats[45];

		/************* set level name **********************************/
	if(*lname)
	{	lname[STRLIST - 2] = 0;
		sprintf(level_names[level],"%s",lname);
	}
	else
		sprintf(level_names[level],"imported map file: %s",fname);


		/*********** change display to add imported level **************/
	if(level < 64)
		disp_i1hglight |= (1 << (level % 32));
	else
		disp_i2hglight |= (1 << (level % 32));

		/*********** re-display map ************************************/
	dispgraphics((long) ((rwindowxmin + rwindowxmax) / 2),(long) ((rwindowymin + rwindowymax) / 2),resolution_code);

		/************* return ******************************************/
	return(0);
}


/*******************************************************************************
*      impmapabort                                                            *
*          This program is a convenient way of closing the input  file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long impmapabort(location)
long location;
{	
	char string[256];


		/************ close file    if open ****************************/
	closef_ptr(&map_objects_fd);
	closef_ptr(&map_attr_fd);
	closef_ptr(&map_coord_list_fd);

		/************* reset job_stats variables ************************/
	job_stats[16] = last_js16;
	job_stats[17] = last_js17;
	job_stats[18] = last_js18;

		/************* print abort message ******************************/
	sprintf(string,"import map, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}


/*******************************************************************************
*   set_map_attr                                                               *
*         This program reads the import map attributes and copies attribute    *
*      records into the v_attributes file for a single object.                 *
*                                                                              *
*      Parameters:  long n;  record number of attribute in import map          *
*                                                                              *
*      Value returned:  long p;  record number of attribute in current map     *
*      J.Dabritz  2/12/90                                                      *
*******************************************************************************/

set_map_attr(n)
{
	ATTRIBUTES attr;

	long p;
	long offset;
	long next;


	if(n <= 0)
		return(0);

	p = job_stats[16];

	next = n;
	for(; next; )
	{		/*********** read import map attribute record ********/
			/***  Calculate offset into attributes file    ***/
		offset = next * attributesm.rec_size;
			/*** read import map attribute record ***/
		if( 0 > movefp( map_attr_fd, offset, 0 ) )
			return(0);
		if( attributesm.rec_size != readb((char *)&attr,attributesm.rec_size, 1, map_attr_fd ))
			return(0);
			/*********** set attr.string *************************/
		attr.string[30] = (char) 0;
			/*********** set attr.next ***************************/
		next = attr.next;
		if(next)
				attr.next = job_stats[16] + 1;
		else
				attr.next = 0;
			/*********** store v_attribute ***********************/
		if(put_v_attr(&attr,job_stats[16]))
			return(0);
	}

	return(p);
}

/*******************************************************************************
*   get_vmapobj                                                                *
*        Reads one record from the map_objects_fd file.                        *
*        Returns 0 if record read, 1 on failure.                               *
*        J.Dabritz  2/12/90                                                    *
*******************************************************************************/

long get_vmapobj(ptr,record_number)
OBJECTS *ptr;                /*  pointer to object structure to be got    */
long record_number;          /*  record number for get.                   */
{
	long offset;

		/******************** calculate offset **************************/
	offset = record_number * objectsm.rec_size;

		/******************** position file write pointer ***************/
	if(0 > movefp(map_objects_fd,offset,0))
		return(1);
		/******************** read  record ******************************/
	if(0 > readb((char *)ptr, objectsm.rec_size ,1 ,map_objects_fd))
		return(1);

		/******************** return ************************************/
	return(0);
}
