/*******************************************************************************
*   impmoss     (see manual, Command Ref., User Functions)  <fc>               *
*                                                                              *
*       This program reads a MOSS import/export formatted ascii file and       *
*       stores the data on the specified 'level' (35 - 95)                     *
*                                                                              *
*            Parameters:   char *fname       the name of the dlg3 file to      *
*                                            import.                           *
*                          long level        the level number to import to     *
*                                            (35  - 95  inclusive)             *
*                          long c            coord_type code (same as in the   *
*                                            register file)                    *
*                          long otype        obj type code: 0=node, 1=seg,     *
*                                                             2= polygon       *
*                          char *lname       new name for the level imported   *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist,   (varaibles)  [bunches]                        *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   10/10/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

int fd;
long last_js16, last_js17, last_js18;

long impmossabort();
char *reada();


long impmoss( fname, level, c, otype, lname)
char *fname;
long level,c,otype;
char *lname;
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST coordlist;

	char actual[256];
	char *ptr;
	char string[256];
	char msg[256];

	double x,y;

	long line;
	long no_verticies;
	long v;
	long island;


	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check for raster registration *******************/
	if(!registered(0))
	{	message("You must REGISTER the Raster befor importing a vector file.");
		return(1);
	}

		/************* check for geodetic registration *****************/
	if(!registered(c))
	{	message("You must REGISTER the input coordinate system befor importing.");
		return(1);
	}

		/************* check input parameters **************************/
	if((level < 35)  ||  (level > 95))
	{	message("Illegal level number.  Level number must be between 35 and 95.");
		return(1);
	}
	if((c < 0)  ||  (c > 19))
	{	message("Illegal coordinate type code.  Code must be between 0 and 19 inclusive.");
		return(1);
	}
	if((otype < 0)  ||  (otype > 2))
	{	message("Illegal object type code.  Type must be between 0 and 2 inclusive.");
		return(1);
	}


		/******* set up variables *************************************/
	message("import moss, setting up variables.");
	last_js16 = job_stats[16];
	last_js17 = job_stats[17];
	last_js18 = job_stats[18];
	fd = -1;


		/************* output v_attribute record 0 dummy record ********/
	if(!job_stats[16])
	{	attr.next = 0;
		attr.string[0] = 0;
		if(put_v_attr(&attr,0))
		{	message("import moss, abort, location 0.");
			return(1);
		}
	}

		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import moss, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open (1) file. aborting.");
		return(1);
	}
	

	v_object.type = 10 * level + otype;
		/************* read all records ********************************/
	for(line=0 ; reada(string,STRLIST,fd) > (char *) 0; line++)   /*  header */
	{
		if(!(line %25))
		{	sprintf(msg,"Reading & storing moss data, moss object %d.",line+1);
			message(msg);
		}

			/***** check for adequate header line length ******/
		if(strlen(string) >= 55)
		{		/*** decode header ***/
			ptr = string + 50;
			ptr[5] = 0;
			sscanf(ptr,"%ld",&no_verticies);
			if(no_verticies < 1)
			{	message("import moss, negative number of verticies.  Aborting");
				impmossabort(1);
				return(1);
			}
			v_object.v1 = job_stats[18];
			
			ptr = string + 15;
			ptr[30] = 0;
			trim(ptr);
			if(*ptr)
			{	strcpy(attr.string,ptr);
				attr.next = 0;
				v_object.atr = job_stats[16];
				if(put_v_attr(&attr,job_stats[16]))
				{	impmossabort(2);
					return(1);
				}
			}
			else
				v_object.atr = 0;

			for(v = 1; v <= no_verticies; v++)
			{	if(reada(string,STRLIST,fd) == (char *) 0)
				{	message("import moss,  Arrived at end of file unexpectedly, aborting.");
					impmossabort(3);
					return(1);
				}
				if(c == 1)               /*** geodetic coordinates ***/
				{	ptr = string + 20;
					trim(ptr);
					if(*ptr == '1')
						island = 1;
					else
						island = 0;

					ptr = string + 10;
					ptr[20] = 0;
					sscanf(ptr,"%lf",&y);
	
					ptr = string;
					ptr[10] = 0;
					sscanf(ptr,"%lf",&x);
				}
				else
				{	ptr = string + 11;
					ptr[22] = 0;
					sscanf(ptr,"%lf",&y);
	
					ptr = string;
					ptr[11] = 0;
					sscanf(ptr,"%lf",&x);

					if(x < 0)
					{	island = 1;
						x = -x;
					}
					else
						island = 0;
				}


				if(island)
				{	v_object.v2 = job_stats[18] - 1;
					if(put_vobjects(&v_object,job_stats[17]))
					{	impmossabort(4);
						return(1);
					}
					v_object.v1 = job_stats[18];
				}
				transform(&x,&y,c,0);
				coordlist.x = x;
				coordlist.y = y;
				if(put_v_coord(&coordlist,job_stats[18]))
				{	impmossabort(5);
					return(1);
				}
			}
			v_object.v2 = job_stats[18] - 1;
			if(put_vobjects(&v_object,job_stats[17]))
			{	impmossabort(4);
				return(1);
			}
		}
		else
			message("Read a non-header line when expecting a header.  Continuing.");
	}


	message("import moss, closing files.");
		/************* close file **************************************/
	closef_ptr(&fd);

		/************* if a copied dos file, purge it ******************/
#ifdef SV32
	/* can't access dos anyway */
#else
		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);
#endif

		/************* index v_objects imported ************************/
	if(index_vobj(last_js17))
	{	job_stats[17] = last_js17;
		return(1);
	}

		/************* write diary *************************************/
	sprintf(string,"import moss file %s to level %d.",fname,level);
	put_diary(string);

		/************* set job_stats ***********************************/
	job_stats[45]++;
	job_stats[31] = job_stats[45];

		/************* set level name **********************************/
	if(*lname)
	{	lname[STRLIST - 2] = 0;
		sprintf(level_names[level],"%s",lname);
	}
	else
		sprintf(level_names[level],"imported moss file: %s",fname);

		/*********** change display to add imported level **************/
	if(level < 64)
		disp_i1hglight |= (1 << (level % 32));
	else
		disp_i2hglight |= (1 << (level % 32));

		/*********** re-display map ************************************/
	dispgraphics((long) ((rwindowxmin + rwindowxmax) / 2),(long) ((rwindowymin + rwindowymax) / 2),resolution_code);

		/************* return ******************************************/
	return(0);
}


/*******************************************************************************
*      impmossabort                                                            *
*          This program is a convenient way of closing the input  file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long impmossabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* reset job_stats variables ************************/
	job_stats[16] = last_js16;
	job_stats[17] = last_js17;
	job_stats[18] = last_js18;

		/************* print abort message ******************************/
	sprintf(string,"import moss, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
