/*******************************************************************************
*       impoptr9        (see manual, Command Ref., User Functions)      <fc>   *
*            This program imports a type 9 optronics REL scan file             *
*        potduced by the optronics full format scanner at GSC.                 *
*        All vector type files, (map and import levels) are erased as the      *
*        the new raster forms a new coordinate basis for all files.            *
*                                                                              *
*        Parameters:  char  fname       name of file to be imported            *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   12/12/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>


long impoptr9( fname )
	char *fname;
{
	char actual[256];
	unsigned char string[1056], string2[256];
	unsigned char *ptr, *endptr;
	char msg_string[256];

	int  file_fd;

	long i,h,n,w;
	long width;
	long height;
	long size;
	long x,y;
	long bit_value;
	long wrd;
	long raster_size;
	long bytes;
	long scan_height;
	long scan_width;
	long pixel;
	long *lptr;
	long *longptr, runlength;
	long word_width;


	file_fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

	message("import optr9, setting up variables.");
		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import optr9, abort, location 1.");
		return(1);
	}
	file_fd = openf(actual,READ);
	if(file_fd <= 0)
	{	message("Cannot open input file. aborting.");
	return(1);
	}


		/**** set display coordinate type to 0 (raster pixels) ******/
	disp_coord_type = 0;


		/*************** free current map malloc space *****************/
	message("Updating temporary map file.");
	update_temp();
	free_malloc();


		/************* read the header bytes *****************************/
	bytes = readb(string,512,1,file_fd);
	if(bytes < 256)
	{	closef_ptr(&file_fd);
		message("Not Enough data in file for header.  Aborting.");
		return(1);
	}

		/** find format type ***/
	if(string[1] != 9)
	{	message("format of file does not match optronics type 9 rle file.  Aborting import.");
		closef_ptr(&file_fd);
		return(1);
	}
		
		/** find map height **/
	longptr = (long *) (string + 184);
	*longptr &= 65535;
	scan_height = *longptr;

		/** find map width **/
	longptr = (long *) (string + 188);
	*longptr &= 65535;
	scan_width = *longptr;



		/************* calculate map height & width ************************/
	height = scan_height;

	width = scan_width;

	sprintf(string2,"scan is %d high by %d wide.");

		/****** make width an integral multiple of 32 **/
	if(width % 32)
	{	n = width / 32 + 1;
		width = n * 32;
	}


	sprintf(string2,"scan is %d high by %d wide.",height,width);
	message(string2);
	message("resetting map vairables & allocating space for new raster.");
		/************* reset job_stats *********************************/
	job_stats[1] = width - 1;
	job_stats[2] = height - 1;
	for(i=3; i<=7; i++)
		job_stats[i] = 0;
	for(i=10; i<=18; i++)
	{	if((i==13)  ||  (i==15)  ||  (i==16))
			job_stats[i] = (job_stats[i])  ?    1 : 0;
		else
			job_stats[i] = 0;
	}
	job_stats[45]++;                          /** optr9 import flag **/
	job_stats[25] = job_stats[45];
	job_stats[49] = 0;
	job_stats[75] = 0;


	message("calculating new raster size.");
		/***********  calculate sizes **************/
	h = job_stats[2] + 1;             /* height of raster in rows  */
	w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */
	raster_size = h*w;                /* size of raster in bytes   */
	size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster/r */

		/***********  allocate memory **************/
	sprintf(msg_string,"New raster size = %d bytes",size);
	message(msg_string);
	rasterm.malloc = (char *) calloc_ltp(1,size);
	if(rasterm.malloc == (char *) 0)
	{	message("  <<  cannot get enough RAM for raster.  You must re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);

	}
		/*  set the raster_wincrement and raster_bincrement globals  */
	raster_wincrement = (job_stats[1] + 1) >> 5;
	raster_bincrement = (job_stats[1] + 1) >> 3;


	message("Setting pointers for RAM raser array.");
		/********  calculate & set global pointers to raster ****************/
	rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
	rasterm.recn = rasterm.rec0 + raster_size;
	craster = (unsigned char **)(rasterm.malloc);
	lraster = (unsigned long **)(rasterm.malloc + 4*h);
	for(i=0; i<h; i++)
	{	craster[i] = (unsigned char *)rasterm.rec0 + w*i;
		lraster[i] = (unsigned long *) (craster[i]);
	}


		/************* read past second header block if present **********/
	longptr = (long *) (string);
	*longptr = (*longptr) >> 16;
	for(i= *longptr - 254; i>0 ; i--)
		readb(string,2,1,file_fd);


		/************* read scan file *********************************/
	word_width = (1 + job_stats[1]) >> 5;
	longptr = (long *) string;
	endptr = string + 1023;
	ptr = endptr + 1;
	for(x=0; x<scan_width; x++)        /** loop once for each scan line **/
	{		/** inform user **/
		if(!(x % 25))
		{	sprintf(string2,"reading scan file, column %d.",x);
			message(string2);
		}
		wrd = x >> 5;
		pixel = 1 << ((7 & ~x) | (x & 24));

		ptr += 8;          /* read past scan-line header */
		bit_value = 0;     /* set initial pixel value to off */
		for(y=job_stats[2]; y>= 0; bit_value ^= 1)
		{	if(ptr > endptr)
			{	if(!readb(string,1024,1,file_fd))
				{	message("Ran out of scan data befor scan completed.  Proceding to next step of import.");
					y = -1;
					x = scan_width;
					break;
				}
				ptr -= 1024;
			}
			
			runlength = *ptr;
			ptr++;
			runlength |= (*ptr) << 8;
			ptr++;

			if(y < runlength)
				runlength = y + 1;
			if(bit_value)
			{	lptr = (long *) (lraster[y] + wrd);
				for(i = runlength;  ;   i--,  lptr -= word_width)
				{	*lptr |=  pixel;
					if(i <= 0)
						break;
				}
			}
			y -= runlength;
		}
	
			/* check for ending run = off */
		if(!bit_value)
			ptr += 2;
	}
		

		/************* close file  *************************************/
	closef_ptr(&file_fd);

		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);


		/*******   write raster to temp file  *****/
		/*  open file   */
	message("writing raster to temp. file.");
	sprintf(string,"%s/raster",temp_map_path);
	if((file_fd = openf(string,WRITE)) <= 0)
	{	message("Cannot open raster temp file.  Aborting.");
		return(1);
	}
		/*  write all records  */
	size = (job_stats[1]+1)  *  (job_stats[2]+1)  /  8;
	if(0 > writeb(rasterm.rec0, size , 1, file_fd))
	{	message("Cannot write all raster recs to temp file.  Aborting.");
		closef_ptr(&file_fd);
		return(1);
	}
		/*  close file  */
	closef_ptr(&file_fd);
/*  free malloc space */
free_ltp(rasterm.malloc);
rasterm.malloc = (char *) 0;



		/* store new sized index_array file     */
		/***********  calculate sizes **************/
	message("allocating new index_array in RAM.");
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = (h*w);                     /* size to calloc            */
		/***********  allocate memory **************/
	index_arraym.malloc = (char *) calloc_ltp(1,size);
	if(index_arraym.malloc == (char *) 0)
	{	message("  <<  cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}

		/***********  write index_array to file *********************/
	message("writing new index_array file.");
	sprintf(string,"%s/index_array",temp_map_path);
	file_fd = openf(string,WRITE);
	if(0 > file_fd)
	{	message("  <<  cannot open index_array map file.");
		return(0);
	}
	if(0 > writeb(index_arraym.malloc,size,1,file_fd))
	{	message("  <<  cannot write index_array map file.");
		closef_ptr(&file_fd);
		return(0);
	}
	closef_ptr(&file_fd);
/*  free malloc space */
free_ltp(index_arraym.malloc);
index_arraym.malloc = (char *) 0;

		/***********  write job_stats to file ***********************/
	message("writing new job_stats file.");
	sprintf(string,"%s/job_stats",temp_map_path);
	if(put_jobstats(string))
		{	message("Cannot write job_stats to temp file.  Aborting.");
			return(1);
		}

		/************* re-acquire temp file ****************************/
	if(reacquire())
	{	message("You must re-acquire the map & start over.");
		job_stats[0] = 0;
		return(1);
	}

		/************* un-register raster ******************************/
	for(i=0; i<4; i++)
	{	corner[0][i][0] = 0.0;
		corner[0][i][1] = 0.0;
	}

		/************* write diary record ******************************/
	sprintf(string,"imported optr9 file %s  (%d x %d)",fname,job_stats[2]+1,job_stats[1]+1);
	put_diary(string);


		/*** reset undo array    **********/
	for(i=0; i<32; i++)
		undo_array[i].mode = 0;

		/************* create rasterb **********************************/
	brstr_create();

	return(0);
}
