/*******************************************************************************
*       impras          (see manual, Command Ref., User Functions)      <fc>   *
*            This program imports a strip type runlength encoded raster        *
*        file which has been produced by the HI scanner plotter.               *
*        All vector type files, (map and import levels) are erased as the      *
*        the new raster forms a new coordinate basis for all files.            *
*                                                                              *
*        Parameters:  char  fname       name of file to be imported            *
*                     long  r           linear reduction factor                *
*                     long  t           threshold, pixels                      *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   10/15/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>


long impras( fname,r,t )
	char *fname;
	long r,t;
{
	char actual[256];
	unsigned char string[256];
	char str[256];
	unsigned char *ptr;

	int fd;

	long i,h,n,w;
	long width;
	long height;
	long size;
	long x,y,x0,dx,xmax;
	long strip;
	long value;
	long wrd,bit;
	long raster_size;
	long bytes;
	long low, high;
	long scan_height;
	long scan_width;
	long strip_width, strip_width0;
	long density_x, density_y;
	long density[256];
	long *lptr;


	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

	message("import ras, setting up variables.");
		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import ras, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open input file. aborting.");
		return(1);
	}


		/**** set display coordinate type to 0 (raster pixels) ******/
	disp_coord_type = 0;


		/*************** free current map malloc space *****************/
	message("Updating temporary map file.");
	update_temp();
	free_malloc();


		/************* read the 6 header bytes *****************************/
	bytes = readb(string,6,1,fd);
	if(bytes < 6)
	{	closef_ptr(&fd);
		message("Not Enough data in file for header.  Aborting.");
		return(1);
	}

		/** find strip_length **/
	low = string[0];
	high = string[1];
	scan_height = (high << 8) | low;

		/** find map width **/
	low = string[2];
	high = string[3];
	scan_width = (high << 8) | low;

		/** find strip_width **/
	strip_width0 = string[4];


		/************* calculate map height & width ************************/
	height = scan_height / r;
	if(scan_height % r)
		height++;

	width = scan_width / r;
	if(scan_width % r)
		width++;

	strip_width = strip_width0 / r;

	sprintf(string,"%d  %d  %d,  reduct = %d,   %d %d %d",
			scan_height,scan_width,strip_width0,
			r,height, width, strip_width     );
	message(string);



		/****** make width an integral multiple of 32 **/
	if(width % 32)
	{	n = width / 32 + 1;
		width = n * 32;
	}


	message("resetting map vairables & allocating space for new raster.");
		/************* reset job_stats *********************************/
	job_stats[1] = width - 1;
	job_stats[2] = height - 1;
	for(i=3; i<=7; i++)
		job_stats[i] = 0;
	for(i=10; i<=18; i++)
	{	if((i==13)  ||  (i==15)  ||  (i==16))
			job_stats[i] = (job_stats[i])  ?    1 : 0;
		else
			job_stats[i] = 0;
	}
	job_stats[45]++;                          /** ras import flag **/
	job_stats[24] = job_stats[45];
	job_stats[49] = 0;
	job_stats[75] = 0;


	message("calculating new raster size.");
		/***********  calculate sizes **************/
	h = job_stats[2] + 1;             /* height of raster in rows  */
	w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */
	raster_size = h*w;                /* size of raster in bytes   */
	size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster/r */

		/***********  allocate memory **************/
	rasterm.malloc = (char *) calloc_ltp(1,size);
	if(rasterm.malloc == (char *) 0)
	{	message("  <<  cannot get enough RAM for raster.  You must re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);

	}
		/*  set the raster_wincrement and raster_bincrement globals  */
	raster_wincrement = (job_stats[1] + 1) >> 5;
	raster_bincrement = (job_stats[1] + 1) >> 3;


	message("Setting pointers for RAM raser array.");
		/********  calculate & set global pointers to raster ****************/
	rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
	rasterm.recn = rasterm.rec0 + raster_size;
	craster = (unsigned char **)(rasterm.malloc);
	lraster = (unsigned long **)(rasterm.malloc + 4*h);
	for(i=0; i<h; i++)
	{	craster[i] = (unsigned char *)rasterm.rec0 + w*i;
		lraster[i] = (unsigned long *) (craster[i]);
	}


	message("Setting loop vairables.");
		/************* read scan file *********************************/
	x0 = 0;
	y = 0;
	bytes = 0;
	density_y = 0;
	density_x = 0;
	value = 0;
	for(i=0; i<strip_width; i++)
		density[i] = 0;

	strip = 1;
	message("Reading scan strip 1.");
	for(ptr=(unsigned char *) string ;    ;ptr++, bytes-- )
	{		/** check for data to interpret **/
		if(!bytes)
		{	bytes = readb(string,250,1,fd);
				/** check for data finished **/
			if(bytes <= 0)
				break;
			ptr = (unsigned char *) string;
		}

			/** interpret 1 byte */
		if(*ptr > (unsigned char) 120)			/** end of scan line **/
		{	density_y = (density_y + 1) % r;
			density_x = 0;
			value = 0;
			if(! density_y)
			{	if((y >= 0)  &&  (y <= job_stats[2]))
				{	xmax = x0 + strip_width;
					if(xmax > job_stats[1])
						xmax = job_stats[1] + 1;
					for(x=x0, lptr=density; x<xmax ; x++, lptr++)
					{
						if(*lptr >= t)
						{
							wrd = x >> 5;              /* find 32 bit word */
							bit = (7 & ~x) | (x & 24); /* find bit within word*/
							lraster[y][wrd] |= (1 << bit);
						}
						*lptr = 0;
					}
				}
				y++;
			}

				/** check for end of scan strip **/
			if(*ptr > 128)
			{	y = 0;
				x0 += strip_width;
				strip++;
				sprintf(str,"Reading scan strip %d.",strip);
				message(str);
			}
		}
		else       /***************** run length ************************/
		{	dx = *ptr;
			if(value)
			{	for( ; dx && (density_x < strip_width0) ; dx--, density_x++)
					density[density_x / r] += 1;
			}
			else
				density_x += dx;
			value = (value + 1) & 1;
		}
	}


		/************* close file  *************************************/
	closef_ptr(&fd);

		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);


		/*******   write raster to temp file  *****/
		/*  open file   */
	sprintf(string,"%s/raster",temp_map_path);
	if((fd = openf(string,WRITE)) <= 0)
	{	message("Cannot open raster temp file.  Aborting.");
		return(1);
	}
		/*  write all records  */
	size = (job_stats[1]+1)  *  (job_stats[2]+1)  /  8;
	if(0 > writeb(rasterm.rec0, size , 1, fd))
	{	message("Cannot write all raster recs to temp file.  Aborting.");
		closef_ptr(&fd);
		return(1);
	}
		/*  close file  */
	closef_ptr(&fd);
/*  free malloc space */
free_ltp(rasterm.malloc);
rasterm.malloc = (char *) 0;




		/* store new sized index_array file     */
		/***********  calculate sizes **************/
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = (h*w);                     /* size to calloc            */
		/***********  allocate memory **************/
	index_arraym.malloc = (char *) calloc_ltp(1,size);
	if(index_arraym.malloc == (char *) 0)
	{	message("  <<  cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}

		/***********  write index_array to file *********************/
	sprintf(string,"%s/index_array",temp_map_path);
	fd = openf(string,WRITE);
	if(0 > fd)
	{	message("  <<  cannot open index_array map file.");
		return(0);
	}
	if(0 > writeb(index_arraym.malloc,size,1,fd))
	{	message("  <<  cannot write index_array map file.");
		closef_ptr(&fd);
		return(0);
	}
	closef_ptr(&fd);
/*  free malloc space */
free_ltp(index_arraym.malloc);
index_arraym.malloc = (char *) 0;

		/***********  write job_stats to file ***********************/
	sprintf(string,"%s/job_stats",temp_map_path);
	if(put_jobstats(string))
		{	message("Cannot write job_stats to temp file.  Aborting.");
			return(1);
		}

		/************* re-acquire temp file ****************************/
	if(reacquire())
	{	message("You must re-acquire the map & start over.");
		job_stats[0] = 0;
		return(1);
	}

		/************* un-register raster ******************************/
	for(i=0; i<4; i++)
	{	corner[0][i][0] = 0.0;
		corner[0][i][1] = 0.0;
	}

		/************* write diary record ******************************/
	sprintf(string,"imported ras file %s  (%d x %d)",fname,job_stats[2]+1,job_stats[1]+1);
	put_diary(string);


		/*** reset undo array    **********/
	for(i=0; i<32; i++)
		undo_array[i].mode = 0;

		/************* create rasterb **********************************/
	brstr_create();

	return(0);
}
