/*******************************************************************************
*       imprdef2        (see manual, Command Ref., User Functions)      <fc>   *
*            This program imports a rdef2 formatted ascii file.                *
*        The old raster is replaced by the new raster, which may be a          **        different size.  All vector type files, (map and import levels)       *
*        are erased as the the new raster forms a new coordinate basis         *
*        for all files.                                                        *
*                                                                              *
*            Parameters:   char *fname       the name of the rdef2 file to      *
*                                            import.                           *
*                          long a            attribute code:  1=get objects,   *
*                                            3= get objects & attributes.      *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist, raster, mini-raster, attributes, errors,        *
*                 register                                                     *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   10/12/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

int fd, fd2;
char *malloc_pt;

long imprdef2abort();

long imprdef2( fname, a )
	char *fname;
	long a;
{
	char actual[256];
	char string[256];
	char string2[256];
	char *ptr;
	char index[256];
	char new_file[256];
	char part1[256], part2[256];
	char delim[256];
	char parsed[40][STRLIST];
	char raster3[256];
	

	long register_lines, attribute_lines;
	long height;
	long width;                  /* width of 1 raster row (pixels) */
	long h,i,j,n,w;
	long size;                   /* bytes in 1 raster row */
	long *rdef2_row;              /* pointer to malloc space */
	long rdef2_words;             /* number of 32 bit words in a raster row */
	long rows;                   /* number of raster rows */
	long done;
	long *lptr;
	long *rdef2_row_end;          /* ptr just beyond rdef2_row */
	long x,y;
	long value;
	long no_params;
	long deg,min,sec;
	long wrd,bit,dx;
	long offset;
	
	
	long otype;

	float floatx, floaty, floatr;

	double coord_pairs[4][2];


		/************* initialize fd & malloc globals ******************/
	fd = -1;
	fd2 = -1;
	malloc_pt = (char *) 0;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

	message("import rdef2, setting up variables.");
		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import rdef2, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open input file. aborting.");
		return(1);
	}


		/**** set display coordinate type to 0 (raster pixels) ******/
	disp_coord_type = 0;


		/*************** free current map malloc space *****************/
	message("Updating temporary map file.");
	update_temp();
	free_malloc();


	message("Decoding header.");
		/************* decode header line ******************************/
	for(; reada(string,254,fd) ; )
	{	if(strlen(string) >= 48)
			break;
	}

		/** get number of register lines **/
	ptr = string + 48;
	*(ptr + 6) = 0;
	trim(ptr);
	register_lines = 0;
	sscanf(ptr,"%ld",&register_lines);
	
		/** get number of attribute journal lines **/
	*ptr = 0;
	ptr = string + 42;
	trim(ptr);
	attribute_lines = 0;
	sscanf(ptr,"%ld",&attribute_lines);
	
		/** get raster height **/
	*ptr = 0;
	ptr = string + 36;
	trim(ptr);
	sscanf(ptr,"%ld",&height);
	
		/** get raster width ***/
	*ptr = 0;
	ptr = string + 30;
	trim(ptr);
	sscanf(ptr,"%ld",&width);

		/** check header values **/
	if((width < 32) || (height < 32))
	{	message("Illegal width or height.  Both must be >= 32 pixels.");
		imprdef2abort(1);
		return(1);
	}
		/** check header values **/
	if((register_lines < 0) || (attribute_lines < 0))
	{	message("Illegal number of register or attribute lines.  Both must be >= 0.");
		imprdef2abort(2);
		return(1);
	}

	
		/************* read through registration lines *****************/
	for(i=0; i<register_lines; i++)
	{	reada(string,254,fd);
	}

		/************* read through attribute journal lines ************/
	for(i=0; i<attribute_lines; i++)
	{	reada(string,254,fd);
	}

		/************* check for RUN LENGTH ENCODED in line ************/
	reada(string,254,fd);
	string[18] = 0;
	if(strcmp(string,"RUN LENGTH ENCODED") !=  0)
	{	imprdef2abort(3);
		message("Raster portion of file is misplaced or absent.  Aborting.");
		return(1);
	}
	
		/****** make width an integral multiple of 32 **/
	if(width % 32)
	{	n = width / 32 + 1;
		width = n * 32;
	}


		/************* create new raster file & asgn pointer ***********/
	sprintf(new_file,"%s/raster3",temp_map_path);
	purgef(new_file);
	sprintf(new_file,"%s/raster2",temp_map_path);
	purgef(new_file);
	if(-1 == creatf(new_file))
	{	message("Cannot create new raster file. aborting.");
		imprdef2abort(20);
		return(1);
	}
	fd2 = openf(new_file,WRITE);
	if(fd2 < 0)
	{	message("Cannot open new raster file. aborting.");
		imprdef2abort(21);
		return(1);
	}
	
		/************* malloc enough space for a raster row ************/
	size = width / 8;
	malloc_pt = (char *) malloc_ltp(size);
	if(malloc_pt == (char *) 0)
	{	message("Not enough RAM available to import rdef2.  Aborting.");
		imprdef2abort(4);
		return(1);
	}
	rdef2_row = (long *)malloc_pt;
	rdef2_words = size / 4 ;
	rdef2_row_end = (long *) (rdef2_row + rdef2_words);
	

		/************* read rdef2 file raster portion ******************/
	ptr = reada(string,254,fd);
	for(rows = 0, done = 0; !done ; rows++)
	{		/*********inform user **************/
		if(!(rows % 25))
		{	sprintf(string2,"imprdef2, reading in raster row %d",rows);
			message(string2);
		}

			/************ zero out storage row *************************/
		for(lptr = rdef2_row; lptr < rdef2_row_end; lptr++)
			*lptr = 0;

			/************ read & decode 1 raster row ****************/
		for(x=0, value=0 ;  ; ptr = reada(string,254,fd) )
		{		/** check string pointer **/
			if(ptr == (char *) 0)
			{	done = 1;
				break;
			}
				/** check string length **/
			if(strlen(string) < 6)
			{	done = 1;
				break;
			}
				/** check for EOF **/
			for(i=0; i<6; i++)
				index[i] = string[i];
			index[6] = 0;
			trim(index);
			if(!strcmp(index,"EOF"))
			{	done = 1;
				break;
			}
				/** check for line number **/
			if(*index  &&  x > 0)
				break;

				/** interpret data **/
			ptr = string + 6;
			for(ptr = string+6;   *ptr; ptr++)
			{		/** get run length **/
				dx = (*ptr) - 32;
				if(dx >= 0)
				{	if(value)
					{	for( ; dx && (x < width) ; x++, dx--)
						{	wrd = x >> 5;                  /* find  word */
							bit = (7 & ~x) | (x & 24);     /* find bit   */
							rdef2_row[wrd] |= (1 << bit);  /* turn on bit*/
						}
					}
					else
						x += dx;
					value = (value + 1) & 1;
				}
			}
		}
			/*** store raster row ***/
		if(x > 0)
		{	if(writeb(malloc_pt,size,1,fd2) <= 0)
			{	message("write failed when storing a row of raster data.  Aborting.");
				imprdef2abort(5);
				return(1);
			}
		}
	}


		/************* close files *************************************/
	closef_ptr(&fd);
	closef_ptr(&fd2);

		/************* create final raster file ************************/
	message("imprdef2, re-reflecting new raster & storing in raster file.");
	sprintf(raster3,"%s/raster3",temp_map_path);
	if(-1 == creatf(raster3))
	{	message("Cannot create raster3 raster file. aborting.");
		imprdef2abort(6);
		return(1);
	}
	fd = openf(raster3,WRITE);
	if(fd < 0)
	{	message("Cannot open raster3 raster file. aborting.");
		imprdef2abort(7);
		return(1);
	}
	
		/************* open new_file (upside down raster) **************/
	fd2 = openf(new_file,READ);
	if(fd2 < 0)
	{	message("Cannot open new_file raster file. aborting.");
		imprdef2abort(8);
		return(1);
	}
	
		/************* read upside down raster & store right side up ***/
	for(offset = (rows - 1) * size; offset >= 0; offset -= size)
	{		/******************** position file read pointer ***************/
		if(0 > movefp(fd2,offset,0))
		{	imprdef2abort(9);
			message("Move file pointer failed while reflecting rdef2 file.  Aborting.");
			return(1);
		}

			/******************** read  record from file 1 ******************/
		if(0 > readb(malloc_pt, size, 1, fd2))
		{	imprdef2abort(999);
			message("Read failed while reflecting rdef2 file.  Aborting.");
			return(1);
		}

			/******************** store record in file 2 ********************/
		if(0 > writeb(malloc_pt, size, 1, fd))
		{	imprdef2abort(10);
			message("Write failed while reflecting rdef2 file.  Aborting.");
			return(1);
		}
	}

		/************* close files & free malloc space *****************/
	free_ltp(malloc_pt);
	malloc_pt = (char *) 0;
	closef_ptr(&fd);
	closef_ptr(&fd2);
	purgef(new_file);

		/************* move raster file ********************************/
	sprintf(new_file,"%s/raster",temp_map_path);
	if(move_file(raster3, new_file))
	{	message("Move failed when attempting to move new raster into old file, Aborting.");
		imprdef2abort(22);
		return(1);
	}

	message("resetting map vairables & loading new raster into display RAM.");
		/************* reset job_stats *********************************/
	job_stats[1] = size * 8 - 1;
	job_stats[2] = rows - 1;
	for(i=3; i<=7; i++)
		job_stats[i] = 0;
	for(i=10; i<=18; i++)
	{	if((i==13)  ||  (i==15)  ||  (i==16))
			job_stats[i] = (job_stats[i])  ?    1 : 0;
		else
			job_stats[i] = 0;
	}
	job_stats[45]++;                   /** rdef2 import flag **/
	job_stats[22] = job_stats[45];
	job_stats[49] = 0;
	job_stats[75] = 0;



/********* fix 2 *************************************************************/
		/* store new sized index_array file     */
		/***********  calculate sizes **************/
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = (h*w);                     /* size to calloc            */
		/***********  allocate memory **************/
	index_arraym.malloc = (char *) calloc_ltp(1,size);
	if(index_arraym.malloc == (char *) 0)
	{	message("  <<  cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}

		/***********  write index_array to file *********************/
	sprintf(string,"%s/index_array",temp_map_path);
	fd = openf(string,WRITE);
	if(0 > fd)
	{	message("  <<  cannot open index_array map file.");
		return(0);
	}
	if(0 > writeb(index_arraym.malloc,size,1,fd))
	{	message("  <<  cannot write index_array map file.");
		closef_ptr(&fd);
		return(0);
	}
	closef_ptr(&fd);
free_ltp(index_arraym.malloc);
index_arraym.malloc = (char *) 0;

		/***********  write job_stats to file ***********************/
	sprintf(string,"%s/job_stats",temp_map_path);
	if(put_jobstats(string))
		{	message("Cannot write job_stats to temp file.  Aborting.");
			return(1);
		}

		/************* re-acquire temp file ****************************/
	if(reacquire())
	{	message("You must re-acquire the map & start over.");
		job_stats[0] = 0;
		return(1);
	}





/********* end fix 2 *********************************************************/


		/************* un-register raster ******************************/
	for(i=0; i<4; i++)
	{	corner[0][i][0] = 0.0;
		corner[0][i][1] = 0.0;
	}


		/************* write diary record ******************************/
	sprintf(string,"imported rdef2 file %s  (%d x %d)",fname,job_stats[2]+1,job_stats[1]+1);
	put_diary(string);


		/******* re-open file & read past header line *************/
	if(register_lines || (attribute_lines && (a & 2)))
	{	fd = openf(actual,READ);
		if(fd <= 0)
			message("Cannot open input file. registration and/or attribute data not read.");
		else
		{
				/******* read past header line ***/
			if(reada(string,254,fd) == (char *) 0)
			{	imprdef2abort(11);
				return(1);
			}
				/******* read registration data (if any) & register map ***/
			for(j=0; j<register_lines; j++)
			{	if(reada(string,254,fd) == (char *) 0)
				{	imprdef2abort(11);
					return(1);
				}
				strsplit(string, '=', part1, part2);
				trim(part1);
				trim(part2);
				if(strstr(string, "="))
				{	if(!(strcmp(part1,"STPLZ")))      /** state plane zone **/
					{	sprintf(raw_buf,"header stplzn = %s",part2);
						fj();
					}
					else if(!(strcmp(part1,"SCL")))   /** scale ****/
					{	sprintf(raw_buf,"header scale = %s",part2);
						fj();
					}
					else if(!(strcmp(part1,"GEOCP"))) /** geodetic corners ***/
					{	
							/** parse string **/
						strcpy(delim,delim_buf);
						delim[0] = 32;   /* blank */
						delim[1] = 44;   /* comma */
						delim[2] = 58;   /* colon */
						delim[3] = 59;   /* semi-colon */

						no_params = parse(trim(part2),delim,parsed,25);
							/** register geodetics **/
						for(i=0; i<4 ; i+=1)
						{		/*** get latitude for 1 corner **/
							deg = min = sec = 0;
							sscanf(parsed[6*i],"%ld",&deg);
							sscanf(parsed[6*i+1],"%ld",&min);
							sscanf(parsed[6*i+2],"%ld",&sec);
							coord_pairs[i][1] = deg + min/60.0 + sec / 3600.0;
								/*** get longitude for 1 corner **/
							deg = min = sec = 0;
							sscanf(parsed[6*i+3],"%ld",&deg);
							sscanf(parsed[6*i+4],"%ld",&min);
							sscanf(parsed[6*i+5],"%ld",&sec);
							coord_pairs[i][0] = -(deg + min/60.0 + sec/3600.0);
								/*** register corner ***/
						}
						register_ltp(1,0L,coord_pairs,4L,(char) 0);
					}
					else if(!(strcmp(part1,"GRDCP"))) /** map raster corners **/
					{		/** parse string **/
						strcpy(delim,delim_buf);
						delim[0] = 32;   /* blank */
						delim[1] = 44;   /* comma */
						delim[2] = 58;   /* colon */
						delim[3] = 59;   /* semi-colon */
						no_params = parse(trim(part2),delim,parsed,9);
						no_params = parse(trim(part2),delim,parsed,25);
							/** register raster **/
						for(i=0; i<4 ; i+=1)
						{		/*** get x value ****************/
							sscanf(parsed[2*i],"%ld",&x);
							x--;
								/*** get y value *****************/
							sscanf(parsed[2*i+1],"%ld",&y);
							y = job_stats[2] + 1 - y;
								/*** register corner ***/
							coord_pairs[i][0] = x;
							coord_pairs[i][1] = y;
						}
						register_ltp(0, 0L, coord_pairs, 4L, (char) 0);
					}
				}
			}


				/***** read attribute journal lines ***********/
			if(attribute_lines && (a & 2))
			{		/*** assemble map so objects can be attributed **/
				sprintf(raw_buf,"assemble");
				fj();

				floatr = 1.5;
				for(;attribute_lines; attribute_lines--)
				{	if(reada(string,254,fd) == (char *) 0)
					{	imprdef2abort(12);
						message("File read error.  Not All attribute data has been read.");
						return(1);
					}
						/** check string length **/
					if(strlen(string) > 16)
					{		/** get attribute string **/
						ptr = string + 15;
						trim(ptr);
						sprintf(atrbt_string,"%s",ptr);
						*ptr = 0;
							/** get y coordinate **/
						ptr = string + 8;
						floaty = 0.0;
						sscanf(ptr,"%f",&floaty);
						floaty = floaty / 10.0;
						floaty = floaty - 1;
						floaty = job_stats[2] - floaty;
						*ptr = 0;
							/** get x coordinate **/
						ptr = string + 2;
						floatx = 0.0;
						sscanf(ptr,"%f",&floatx);
						floatx = floatx / 10.0;
						floatx = floatx - 1;
							/** get object type **/
						switch(*string) {
						case 'A' :
							otype = 2;       /** polygon (Area) **/
							break;
						case 'L' :
							otype = 1;       /** segment (Line) **/
							break;
						case 'N' :
							otype = 0;       /** node    (Point) */
							break;
						default :
							otype = -1;
							break;
						}
						if(otype >= 0)
							atrbt_xy(floatx,floaty,floatr,otype,6);
					}
				}
			}
		}
	}

		/*** close file ***/
	closef_ptr(&fd);
	

		/*** reset undo array    **********/
	for(i=0; i<32; i++)
		undo_array[i].mode = 0;


		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);


		/************* create rasterb **********************************/
	brstr_create();

		/************* return ******************************************/
	return(0);
}



/*******************************************************************************
*    imprdef2abort                                                             *
*        This program closes files and frees malloc space and prints an abort  *
*    message on the screen which identifies the abort location.                *
*    Parameter:                long location                                   *
*    Files & globals changed:  none                                            *
*    Value returned:  long 0;                                                  *
*    J.Dabritz    10/13/89                                                     *
*******************************************************************************/

long imprdef2abort(location)
	long location;
{
	char string[256];

		/************ close files **************************************/
	if(fd > 0)
		closef_ptr(&fd);
	if(fd2 > 0)
		closef_ptr(&fd2);

		/************ free malloc space ********************************/
	if(malloc_pt != (char *) 0)
		free_ltp(malloc_pt);

		/************ print message ************************************/
	sprintf(string,"Aborting import rdef2, location %d.",location);
	message(string);

		/************ return *******************************************/
	return(0);
}
