/*******************************************************************************
*       imprstr         (see manual, Command Ref., User Functions)      <fc>   *
*            This program imports a bit-mapped raster file.  The old map       *
*        raster is replaced by the new raster, which may be a different size.  *
*        All vector type files, (map and import levels) are erased as the      *
*        the new raster forms a new coordinate basis for all files.            *
*                                                                              *
*            Parameters:   char *fname       the name of the rstr file to      *
*                                            import.                           *
*                          long bits_used    bits used (bit mask) 1 - 255.     *
*                                            for each byte in the file, only   *
*                                            ones which match this mask are    *
*                                            taken as data.                    *
*                          long a_s_ratio    ASCII/scan ratio.  This is the    *
*                                            number of ascii lines which will  *
*                                            be used to build each scan row.   *
*                                            Scan rows are taken to begin at   *
*                                            the top left and go horizontally. *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist, raster, mini-raster, attributes, errors,        *
*                 register                                                     *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   9/22/89                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>


long imprstr( fname, bits_used, a_s_ratio )
	char *fname;
	long bits_used,a_s_ratio;
{
	char actual[256];
	char new_file[256];
	char line[10240];
	char raster3[256];
	char file[256];
	char nulls[256];
	char path[256];
	char string[256];
	char tmp_buf[256];
	char *malloc_ptr;
	char *end_ptr;
	char *ptr;
	unsigned char bit_mask;

	int fd, fd1, fd2, filefd;

	long charbits;               /* # fo bits/byte that are used */
	long position[8];            /* raster difference for char bits read */
	long ascii_lines;            /* counter */
	long byte_width;             /* ascii bytes input per raster row */
	long i,j,h,n,w;
	long width;                  /* width of 1 raster row (pixels) */
	long size;                   /* bytes in 1 raster row */
	long *rstr_row;              /* pointer to malloc space */
	long rows;                   /* number of raster rows */
	long rstr_words;             /* number of 32 bit words in a raster row */
	long *rstr_row_end;          /* ptr just beyond rstr_row */
	long *lptr;
	long max;
	long done;
	long bit_count;
	long wrd,bit,x;
	long offset;
	long iend;
	long sizel;
	long raster_size;


	fd = -1;
	fd1 = -1;
	fd2 = -1;
	filefd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check bits_used parameter ***********************/
	if(bits_used<1  ||  bits_used>255)
	{	message("bits used code must be between 1 and 255.  Aborting.");
		return(1);
	}

		/************* check a_s_ratio parameter ***********************/
	if(a_s_ratio<1  ||  a_s_ratio>255)
	{	message("ASCII/scan ratio must be between 1 and 255.  Aborting.");
		return(1);
	}

	message("import rstr, setting up variables.");
		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import rstr, abort, location 1.");
		return(1);
	}
	fd1 = openf(actual,READ);
	if(fd1 <= 0)
	{	message("Cannot open input file. aborting.");
		return(1);
	}


		/**** set display coordinate type to 0 (raster pixels) ******/
	disp_coord_type = 0;


		/************* count bits used in 1 character ******************/
	for(i=0, charbits=0; i<8; i++)
		charbits += (1 & (bits_used >> i));

		/************* build position array ****************************/
	for(i=0, j=charbits-1; i<8; i++)
	{	if(bits_used & (1 << i))
			position[i] = j--;
		else
			position[i] = 0;
	}
	
		/************* read top raster line & get raster width *********/
	ascii_lines = 0;
	for(byte_width = 0 ; reada(line,9999,fd1) ; )
	{	n = strlen(line);
		if(((ascii_lines && ((ascii_lines + 1) == a_s_ratio)))  ||  (n > 15))
		{	byte_width += n-1;
			ascii_lines++;
		}
		if(ascii_lines == a_s_ratio)
			break;
	}
		/****** close input file. ***/
	closef_ptr(&fd1);
		/****** find raster width & make it an integral multiple of 32 **/
	if(ascii_lines == a_s_ratio)
	{	width = byte_width * charbits;
		if(width % 32)
		{	n = width / 32 + 1;
			width = n * 32;
		}
	}
	else
	{	message("Cannot find enough ascii data > 15 characters/line to read rstr file.  Aborting.");
		return(1);
	}


		/************* create new raster file & asgn pointer ***********/
	sprintf(new_file,"%s/raster3",temp_map_path);
	purgef(new_file);
	sprintf(new_file,"%s/raster2",temp_map_path);
	purgef(new_file);
	if(-1 == creatf(new_file))
	{	message("Cannot create new raster file. aborting.");
		return(1);
	}
	fd2 = openf(new_file,WRITE);
	if(fd2 < 0)
	{	message("Cannot open new raster file. aborting.");
		return(1);
	}
	
		/************** re-open input (rstr) file **********************/
	fd1 = openf(actual,READ);
	if(fd1 <= 0)
	{	message("Cannot open input file. aborting.");
		closef_ptr(&fd2);
		return(1);
	}


		/************* malloc enough space for a raster row ************/
	size = width / 8;
	malloc_ptr = malloc_ltp(size);
	if(malloc_ptr == (char *) 0)
	{	message("Not enough RAM available to import rstr.  Aborting.");
		closef_ptr(&fd1);
		closef_ptr(&fd2);
		return(1);
	}
	rstr_row = (long *) malloc_ptr;
	rstr_words = size / 4 ;
	rstr_row_end = (long *) (rstr_row + rstr_words);
	

		/************* read rstr file one raster line at a time ********/
	line[9999] = 0;
	bit_mask = (unsigned char) (bits_used & 255);
	end_ptr = line + byte_width;
	for(rows = 0, done = 0; !done ; rows++)
	{	if(!(rows % 25))
		{	sprintf(string,"imprstr, reading in raster row %d",rows);
			message(string);
		}
			/************ read ascii lines for 1 raster row ************/
		for(i=0, ptr=line, max=9999; max  &&  (i < a_s_ratio) ; )
		{	if((char *) 0 == reada(ptr,max,fd1))
			{	done = 1;
				break;
			}
			n = strlen(ptr);
			if((n > 15) || (i  &&  ((i+1) == a_s_ratio)))
			{	i++;
				ptr = ptr + n -1 ;
				max = max - n;
			}
		}
		if(done)
			break;

			/************ zero out storage row *************************/
		for(lptr = rstr_row; lptr < rstr_row_end; lptr++)
			*lptr = 0;

			/************ decode ascii into raster data ****************/
		for(ptr=line, bit_count = 0; ptr<end_ptr; bit_count += charbits, ptr++)
		{	*ptr &= bit_mask;                   /** mask out unused pixels **/
			if(*ptr)                            /** test for any 'on' pixels */
			{	for(i=0; i<8; i++)              /** check all pixels **/
				{	if(*ptr & (1<<i))                  /* found 'on' pixel */
					{	x = bit_count + position[i];   /* find x position */
						wrd = x >> 5;                  /* find 32 bit word */
						bit = (7 & ~x) | (x & 24);     /* find biw within word*/
						rstr_row[wrd] |= (1 << bit);   /* turn on rstr pixel */
					}
				}
			}
		}
			/************ store raster row in temp file ****************/
		if(writeb(malloc_ptr,size,1,fd2) <= 0)
		{	message("write failed when storing a row of raster data.  Aborting.");
			closef_ptr(&fd1);
			closef_ptr(&fd2);
			free_ltp(malloc_ptr);
			return(1);
		}
	}

		/************* close files *************************************/
	closef_ptr(&fd1);
	closef_ptr(&fd2);

		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);


		/************* create final raster file ************************/
	message("imprstr, re-reflecting new raster & storing in raster file.");
	sprintf(raster3,"%s/raster3",temp_map_path);
	if(-1 == creatf(raster3))
	{	message("Cannot create raster3 raster file. aborting.");
		free_ltp(malloc_ptr);
		return(1);
	}
	fd2 = openf(raster3,WRITE);
	if(fd2 < 0)
	{	message("Cannot open raster3 raster file. aborting.");
		free_ltp(malloc_ptr);
		return(1);
	}
	
		/************* open new_file (upside down raster) **************/
	fd1 = openf(new_file,READ);
	if(fd1 < 0)
	{	message("Cannot open new_file raster file. aborting.");
		free_ltp(malloc_ptr);
		closef_ptr(&fd2);
		return(1);
	}
	
		/************* read upside down raster & store right side up ***/
	for(offset = (rows - 1) * size; offset >= 0; offset -= size)
	{		/******************** position file read pointer ***************/
		if(0 > movefp(fd1,offset,0))
		{	free_ltp(malloc_ptr);
			closef_ptr(&fd1);
			closef_ptr(&fd2);
			message("Move file pointer failed while reflecting rstr file.  Aborting.");
			return(1);
		}

			/******************** read  record from file 1 ******************/
		if(0 > readb(malloc_ptr, size, 1, fd1))
		{	free_ltp(malloc_ptr);
			closef_ptr(&fd1);
			closef_ptr(&fd2);
			message("Read failed while reflecting rstr file.  Aborting.");
			return(1);
		}

			/******************** store record in file 2 ********************/
		if(0 > writeb(malloc_ptr, size, 1, fd2))
		{	free_ltp(malloc_ptr);
			closef_ptr(&fd1);
			closef_ptr(&fd2);
			message("Write failed while reflecting rstr file.  Aborting.");
			return(1);
		}
	}

		/************* close files & free malloc space *****************/
	free_ltp(malloc_ptr);
	closef_ptr(&fd1);
	closef_ptr(&fd2);
	purgef(new_file);

		/************* move raster file ********************************/
	sprintf(new_file,"%s/raster",temp_map_path);
	if(move_file(raster3, new_file))
	{	message("Move failed when attempting to move new raster into old file, Aborting.");
		return(1);
	}

	message("resetting map vairables & loading new raster into display RAM.");
		/************* reset job_stats *********************************/
	job_stats[1] = size * 8 - 1;
	job_stats[2] = rows - 1;
	for(i=3; i<=7; i++)
		job_stats[i] = 0;
	for(i=10; i<=18; i++)
	{	if((i==13)  ||  (i==15)  ||  (i==16))
			job_stats[i] = (job_stats[i])  ?    1 : 0;
		else
			job_stats[i] = 0;
	}
	job_stats[45]++;                   /** rstr import flag **/
	job_stats[20] = job_stats[45];
	job_stats[49] = 0;
	job_stats[75] = 0;
		/*  set the raster_wincrement and raster_bincrement globals  */
	raster_wincrement = (job_stats[1] + 1) >> 5;
	raster_bincrement = (job_stats[1] + 1) >> 3;

		/************* create new index_array file *********************/
		/** index_array **/
	for(i=0; i<= STRLIST; i++)
		nulls[i] = 0;
	sprintf(file,"%s/index_array",temp_map_path);
	fd = openf(file,WRITE);
	w = (job_stats[1]) / 32 +1;
	h = (job_stats[2]) / 32 +1;
	iend = 1 + (h * w * 4 / STRLIST);
	for(i=0; i <= iend; i++)
	{	if(0 > writeb(nulls,STRLIST,1,fd))
		{ 	sprintf(tmp_buf, "imprstr(): unable to write to %s", file );
			message(tmp_buf);
			closef_ptr(&fd);
			return(1);
		}
	}
	closef_ptr(&fd);

		/************* acquire raster & index_array ********************/
		/********** free old space *************************************/
	free_ltp(rasterm.malloc);
	rasterm.malloc = (char *) 0;
		/***********  calculate sizes **************/
	h = job_stats[2] + 1;             /* height of raster in rows  */
	w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */
	raster_size = h*w;                /* size of raster in bytes   */
	sizel = raster_size / 4;          /* size of raster in longs (4 bytes ea.)*/
	size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster/r */
		/***********  allocate memory **************/
	rasterm.malloc = (char *) calloc_ltp(size,1);
	if(!rasterm.malloc)
	{	message("  <<  cannot get enough RAM for raster.  You must re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}
		/********  calculate & set global pointers to raster ****************/

	rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
	rasterm.recn = rasterm.rec0 + raster_size;
	craster = (unsigned char **)(rasterm.malloc);
	lraster = (unsigned long **) (rasterm.malloc + 4*h);
	for(i=0; i<h; i++)
	{	craster[i] = (unsigned char *)rasterm.rec0 + w*i;
		lraster[i] = (unsigned long *) (craster[i]);
	}

		/***********  read raster from file *********************/
	sprintf(path,"%s/raster",temp_map_path);
	filefd = openf(path,READ);
	if(0 > filefd)
	{	message("  <<  cannot open raster map file.  You must re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}
	if(0 >= readb(rasterm.rec0,raster_size,1,filefd))
	{	message("  <<  cannot read raster map file.  You must re-ACQUIRE map & start over.");
		closef_ptr(&filefd);
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}


		/*  load index_array                    */
		/* free existing array space */
	free_ltp(index_arraym.malloc);
	index_arraym.malloc = (char *) 0;
		/***********  calculate sizes **************/
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = (h*w + 4*h);               /* size to malloc            */
		/***********  allocate memory **************/
	index_arraym.malloc = (char *) calloc_ltp(size,1);
	if(!index_arraym.malloc)
	{	message("  <<  cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}
		/********  calculate & set global pointers to index_array ***********/
	index_arraym.rec0 = index_arraym.malloc;
	index_arraym.recn = (char *)(index_arraym.rec0 + h*w);
	index_array = (long **)index_arraym.recn;
	for(i=0; i<h; i++)
		index_array[i] = (long *) (index_arraym.malloc + w*i);

		/***********  read index_array from file *********************/
	sprintf(path,"%s/index_array",temp_map_path);
	filefd = openf(path,READ);
	if(0 > filefd)
	{	message("  <<  cannot open index_array map file.  You must re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}
	if(0 > readb(index_arraym.rec0,h*w,1,filefd))
	{	message("  <<  cannot read index_array map file.  You must re-ACQUIRE map & start over.");
		closef_ptr(&filefd);
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}
	closef_ptr(&filefd);

		/************* un-register raster ******************************/
	for(i=0; i<4; i++)
	{	corner[0][i][0] = 0.0;
		corner[0][i][1] = 0.0;
	}

		/************* write diary record ******************************/
	sprintf(string,"imported rstr file %s  (%d x %d)",fname,job_stats[2]+1,job_stats[1]+1);
	put_diary(string);

		/************* redisplay map ***********************************/
	for(i=0; i<4; i++)
	{	view[i].lower_leftx = 0;
		view[i].lower_lefty = 0;
	}
	view[0].zoom_factor = mind(2,  (double) 1.0, 
			(double) (400.0/(maxl(2,job_stats[1]+1,job_stats[2]+1))));
	view[1].zoom_factor = 0.25;
	view[2].zoom_factor = 1.0;
	view[3].zoom_factor = 8.0;

	redisp_map();
/*	dispgraphics((long) (job_stats[1]/2), (long) (job_stats[2]/2), 0);   */

		/*** reset undo array    **********/
	for(i=0; i<32; i++)
		undo_array[i].mode = 0;


		/************* create rasterb **********************************/
	brstr_create();

		/************* return ******************************************/
	return(0);
}
