/*******************************************************************************
*       imptan          (see manual, Command Ref., User Functions)      <fc>   *
*            This program imports a raster from 2 files output by the          *
*        tangent scanner (.cmd & .img files).                                  *
*                                                                              *
*            Parameters:   char *fname       the name of the tangent file to   *
*                          long r;           reduction factor                  *
*                          long threshold;   threshold                         *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist, raster, mini-raster, attributes, errors,        *
*                 register                                                     *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            C. Jonson   3/29/90                                               *
*            J.Dabritz   10/3/90                                               *
*******************************************************************************/
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <errno.h>
#include <string.h>

long imptan( fname, r, threshold )
	char *fname;
	long r, threshold;
{
	struct tangent_cmd 		/* layout of header info less length field */
	{	char lut_file[256];
		char filter_file[256];
		unsigned short y_start;		/* Dimensions are 1/1000 of an inch */
		unsigned short y_end;		/* So 5 inches == 5000              */
		unsigned short x_start;
		unsigned short x_end;
		unsigned short data_packing;	/* Scan type:  5 is 2-color     */
		unsigned short filter_range;
		unsigned short scan_density;	/* Either 250 or 500            */
		unsigned short output_density; /* 1 - scan_density */
		unsigned short total_lines;		/* Number of rows in map        */
		unsigned short swath;
		char data_file1[256];		/* Name of the IMG (data) file      */
		char data_file2[256];		/* Ignore the remaining files       */
		char data_file3[256];
		char data_file4[256];
		char data_file5[256];
	};
	struct cmd		/* Actual layout of the header or "cmd" file */
	{	long cmd_size;
		struct tangent_cmd hdr;
	}	cmd;

   char actual[256];
   unsigned char string[256];
	unsigned char *ptr;
	unsigned char *end_scan_row;
	unsigned char scan_counter[50000];
	unsigned char scan_row[12500];

   int fd,fdr;

   long i,h,n,w;
   long width;
   long height;
   long x,y,y0,xmax;
   long byte,bit,offset;
   long bytes,rbytes;
   long scan_height;
   long scan_width;


		/************ init fd & malloc ptrs ****************************/
	fd = -1;
	fdr = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

	message("import tangent raster, setting up variables.");
		/************ get cmd filename *********************************/
		/** copy fname to string ****/
	sprintf(string,"%s",fname);
		/** remove any .    ending **/
	trim(string);
	i = strlen(string);
	for(ptr = (unsigned char *)(string + i - 1); ptr  >= string; ptr--)
	{	if((*ptr == '/') || (*ptr == ':') || (*ptr == 92))
			break;
		if(*ptr == '.')
		{	*ptr = (char) 0;
			break;
		}
	}
		/** add .cmd ending *********/
	ptr = (unsigned char *) string + strlen(string);
	sprintf(ptr,".cmd");

		/************* find & asgn pointer to "cmd" header file **************/
	if(get_impfile(string,actual))
	{	message("import tangent, abort, location 1.");
		return(1);
	}
	if( (fd = openf(actual,READ)) == -1 )
	{	sprintf(string,"Cannot open input file %s.  aborting.",actual);
		message(string);
		return(1);
	}

/*** Read the cmd file.  Struct tangent_cmd + long  ***/
	if( readb((char *)&cmd, (unsigned)(sizeof(struct cmd)), 1, fd) == -1 )
	{	sprintf(string,"Cannot read %s.  UNIX errno is %d", actual, errno);
		message(string);
		return(-1);
	}

/*** Compare the size of the struct with the size read from the header ***/
	if( cmd.cmd_size != sizeof(struct tangent_cmd) )
	{	sprintf(string, "WARNING: Tangent header contains %d bytes.\
  Expected %d bytes.", cmd.cmd_size, sizeof(struct tangent_cmd) );
		message(string);
		sprintf(string, "Will try to read Tangent scan regardless.");
		message(string);
	}
	
      /************* close file  *************************************/
   closef_ptr(&fd);
	closef_ptr(&fdr);

		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);

	
		/************ display cmd file data ****************************/
	sprintf(string,"y_start = %d",cmd.hdr.y_start);
	message(string);
	sprintf(string,"y_end = %d",cmd.hdr.y_end);
	message(string);
	sprintf(string,"x_start = %d",cmd.hdr.x_start);
	message(string);
	sprintf(string,"x_end = %d",cmd.hdr.x_end);
	message(string);
	sprintf(string,"data_packing = %d",cmd.hdr.data_packing);
	message(string);
	sprintf(string,"filter_range = %d",cmd.hdr.filter_range);
	message(string);
	sprintf(string,"scan_density = %d",cmd.hdr.scan_density);
	message(string);
	sprintf(string,"output_density = %d",cmd.hdr.output_density);
	message(string);
	sprintf(string,"total_lines = %d",cmd.hdr.total_lines);
	message(string);
	sprintf(string,"swath = %d",cmd.hdr.swath);
	message(string);
		/************ display ht & wd **********************************/
		/************ set import variables *****************************/
      /** find scan ht.     **/
   scan_height = cmd.hdr.y_end - cmd.hdr.y_start;
	if(62 == cmd.hdr.scan_density)
		scan_height = (scan_height * (cmd.hdr.scan_density + 0.5)) / 1000;
	else
		scan_height = (scan_height * cmd.hdr.scan_density) / 1000;

      /** find scan width **/
   scan_width = cmd.hdr.x_end - cmd.hdr.x_start;
	if(62 == cmd.hdr.scan_density)
		scan_width = (scan_width * (cmd.hdr.scan_density + 0.5)) / 1000;
	else
		scan_width = (scan_width * cmd.hdr.scan_density) / 1000;


      /************* calculate map height & width ************************/
   height = scan_height / r;
   if(scan_height % r)
      height++;

   width = scan_width / r;
   if(scan_width % r)
      width++;


      /****** make width an integral multiple of 32 **/
   if(width % 32)
   {   n = width / 32 + 1;
      width = n * 32;
   }

   sprintf(string,"scan h/w: %d %d,  reduct: %d,  rstr h/w:  %d %d",
         scan_height,scan_width,r,height, width);
   message(string);

		/************ check width **************************************/
	if(100000 < (cmd.hdr.x_end - cmd.hdr.x_start))
	{	message("Imptangent, scan width greater than 100,000 pixels.  Exceeds program limits.");
		return(1);
	}

		/************ get img filename *********************************/
		/** copy fname to string ****/
	sprintf(string,"%s",fname);
		/** remove any .cmd ending **/
	trim(string);
	i = strlen(string);
	for(ptr = (unsigned char *) string + i - 1; ptr  >= string; ptr--)
	{	if((*ptr == '/') || (*ptr == ':') || (*ptr == 92))
			break;
		if(*ptr == '.')
		{	*ptr = (char) 0;
			break;
		}
	}
		/** add .img ending *********/
	ptr = (unsigned char *) string + strlen(string);
	sprintf(ptr,".img");


		/************* find & asgn pointer to "img" header file **************/
	if(get_impfile(string,actual))
	{	message("import tangent, abort, location 1.");
		return(1);
	}
	if( (fd = openf(actual,READ)) == -1 )
	{	sprintf(string,"%s",cmd.hdr.data_file1);	/* Name of the IMG (data) file*/
		if(get_impfile(string,actual))
		{	message("import tangent, abort, location 1.");
			return(1);
		}
		if( (fd = openf(actual,READ)) == -1 )
		{	sprintf(string,"Cannot open input file %s.  aborting.",actual);
			message(string);
			return(1);
		}
	}


      /*************** free current map malloc space *****************/
   message("Updating temporary map file.");
   update_temp();
   free_malloc();

   message("resetting map vairables.");
      /************* reset job_stats *********************************/
   job_stats[1] = width - 1;
   job_stats[2] = height - 1;
   for(i=3; i<=7; i++)
      job_stats[i] = 0;
   for(i=10; i<=18; i++)
   {   if((i==13)  ||  (i==15)  ||  (i==16))
         job_stats[i] = (job_stats[i])  ?    1 : 0;
      else
         job_stats[i] = 0;
   }
   job_stats[45]++;
   job_stats[28] = job_stats[45];
   job_stats[49] = 0;
   job_stats[75] = 0;


   message("calculating new raster size.");
      /***********  calculate sizes **************/
   h = job_stats[2] + 1;             /* height of raster in rows  */
   w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */

   xmax = job_stats[1];

		/************ write blank scan *********************************/
	rbytes = (job_stats[1] >> 3) + 1;
	for(x=0; x<rbytes; x++)
		scan_row[x] = 0;
	sprintf(string,"%s/raster",temp_map_path);
	purgef(string);
	if(0 > creatf(string))
	{	job_stats[0] = 0;
		map_name[0] = 0;
		message("imptangent ERROR, cannot create new raster file.  You must re-acquire map.");
   	closef_ptr(&fd);
		closef_ptr(&fdr);
		return(1);
	}
	if( (fdr = openf(string,WRITE)) == -1 )
	{	job_stats[0] = 0;
		map_name[0] = 0;
		message("imptangent ERROR, cannot open new raster file.  You must re-acquire map.");
   	closef_ptr(&fd);
		closef_ptr(&fdr);
		return(1);
	}
	sprintf(string,"Writing new sized blank raster, row %d.",job_stats[2]);
	message(string);
	for(y=job_stats[2]; y>= 0; y--)
	{	if(!(y % 100))
		{	sprintf(string,"Writing new sized blank raster, row %d.",y);
			message(string);
		}
		if(1 > writeb(scan_row,rbytes,1,fdr))
		{	job_stats[0] = 0;
			map_name[0] = 0;
			message("imptangent ERROR, cannot open write raster file.  You must re-acquire map.");
   		closef_ptr(&fd);
			closef_ptr(&fdr);
			return(1);
		}
	}
	
		/************ import & store scan ******************************/
	bytes = (scan_width / 8);
	end_scan_row = scan_row + rbytes;
   y0=job_stats[2];
   sprintf(string,"importing tangent raster file.  At row %d.",y0);
   message(string);
   for(y=scan_height-1;(y>=0); y--)
   {	readb(scan_row,bytes,1,fd);
			/*** read bits & increment density array ***/
		if(r > 1)
		{	for(ptr=scan_row, i=0; i <= job_stats[1]; ptr++, i++)
			{	if(*ptr)
				{	for(bit = 7; bit >= 0; bit--)
					{	if(*ptr & (1 << bit))
							scan_counter[(long) (((i << 3) + 7 - bit) / r)]++;
					}
				}
			}
		}

        	/*** check for r complete ***/
     	if((r==1) || !(y % r))
     	{
			if(r > 1)
			{	for(ptr=scan_row; ptr<end_scan_row; ptr++)
					*ptr = (unsigned char) 0;

           		/*** set raster row & reset density array ***/
        		for(x=0; x<= xmax ; x++)
        		{  if(scan_counter[x] >= (unsigned char) threshold)
           		{	byte = x >> 3;   /* find byte */
              		bit = (7 & ~x); /* find bit within byte */
              		scan_row[byte] |= (1 << bit);
           		}
           		scan_counter[x] = 0;
        		}
			}
				/*** move file pointer ***/
			offset = y0 * rbytes;
			movefp(fdr,offset,0);
				/*** write raster row ***/
			if(1 > writeb(scan_row,rbytes,1,fdr))
			{	job_stats[0] = 0;
				map_name[0] = 0;
				message("imptangent ERROR, cannot write raster file.  You must re-acquire map.");
   			closef_ptr(&fd);
				closef_ptr(&fdr);
				return(1);
			}
           	/*** decrement y0 ***/
        	y0--;
        	if(!(y0 % 100))
        	{   sprintf(string,"importing tangent raster file.  At row %d.",y0);
           	message(string);
        	}
           	/**** check for scan complete (there is other data following) **/
        	if(y0 <= 0)
           	break;
     	}
  	}
   message("closing files and updating raster file.");

      /************* close file  *************************************/
   closef_ptr(&fd);
	closef_ptr(&fdr);

		/************ remove temp file, if any *************************/
	imp_rm_tmp(fname,actual);

      /* store new sized index_array file     */
		/** re-blank scan row **/
	for(ptr=scan_row; ptr<end_scan_row; ptr++)
		*ptr = (unsigned char) 0;

      /***********  calculate sizes **************/
   h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
   w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/

      /***********  write index_array to file *********************/
   sprintf(string,"%s/index_array",temp_map_path);
   fd = openf(string,WRITE);
   if(0 > fd)
   {   message("  <<  cannot open index_array map file.");
      return(0);
   }
	for(i=0; i<h; i++)
   {	if(0 > writeb(scan_row,w,1,fd))
   	{   message("  <<  cannot write index_array map file.");
      	closef_ptr(&fd);
      	return(0);
   	}
	}
   closef_ptr(&fd);

      /***********  write job_stats to file ***********************/
   sprintf(string,"%s/job_stats",temp_map_path);
   if(put_jobstats(string))
      {   message("Cannot write job_stats to temp file.  Aborting.");
         return(1);
      }

      /************* re-acquire temp file ****************************/
   if(reacquire())
   {   message("You must re-acquire the map & start over.");
      job_stats[0] = 0;
      return(1);
   }
   view_ltp(0,0,0);

      /************* un-register raster ******************************/
   for(i=0; i<4; i++)
   {   corner[0][i][0] = 0.0;
      corner[0][i][1] = 0.0;
   }

      /************* write diary record ******************************/
   sprintf(string,"imported tangent file %s  (%d x %d)",fname,job_stats[2]+1,job_stats[1]+1);
   put_diary(string);


      /*** reset undo array    **********/
   for(i=0; i<32; i++)
      undo_array[i].mode = 0;

		/************* create rasterb **********************************/
	brstr_create();

   return(0);
}
