/*******************************************************************************
*   impxy       (see manual, Command Ref., User Functions)  <fc>               *
*                                                                              *
*       This program reads a generic/user defined xy vector ascii file and     *
*       stores the data on the specified 'level' (35 - 95)                     *
*                                                                              *
*            Parameters:   char *fname       the name of the file to import    *
*                          long level        the level number to import to     *
*                          long c            coord_type code                   *
*                          long otype        obj tyep code:0=n,1=s,2=p,3=auto  *
*                          long ht           header type code                  *
*                          long in           item number position              *
*                          long vc           vertex count position             *
*                          long at           attribute position                *
*                          long x1           x or d/m/s position               *
*                          long x2           1/60 x position                   *
*                          long x3           1/3600 x position                 *
*                          long y1           y or d/m/s position               *
*                          long y2           1/60 y position                   *
*                          long y3           1/3600 y position                 *
*                                                                              *
*            files/globals changed:  (files) job_stats, index_array,           *
*                 index_list, level_names, v_attributes, v_objects,            *
*                 v_coordlist,   (varaibles)  [bunches]                        *
*                                                                              *
*            Values returned:  (long) 0 = successful import                    *
*                                     1 = failure                              *
*                                                                              *
*            J.Dabritz   2/23/91                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

int fd;
long last_js16, last_js17, last_js18;

long xy_item_check();
long impxyabort();
long xy_store_vertex();
long xy_decode_dec();
char *reada();

long impxy(fname,level,c,otype,ht,in,vc,at,x1,x2,x3,y1,y2,y3)
char *fname;
long level,c,otype;
long ht,in,vc,at,x1,x2,x3,y1,y2,y3;
{
	V_OBJECTS v_object;
	ATTRIBUTES attr;
	COORD_LIST crdlst, crdlst1;

	char actual[256];
	char string[256];
	char msg[256];
	unsigned char xydelims[256];
	char parsed[20][STRLIST];
	char last_atrstr[32];

	long line;
	long i, v;
	long header_flag;
	long vertex_count;
	long item_no;
	long object_count;
	long len;
	long no_params;
	long last_item_no;
	long last_len;
	long last_no_params;
	long max_param;

		/************* init file pointer *******************************/
	fd = -1;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor IMPORTing a file.");
		return(1);
	}

		/************* check for raster registration *******************/
	if(!registered(0))
	{	message("You must REGISTER the Raster befor importing a vector file.");
		return(1);
	}

		/************* check for geodetic registration *****************/
	if(!registered(c))
	{	message("You must REGISTER the input coordinate system befor importing.");
		return(1);
	}

		/************* check input parameters **************************/
	if((level < 35)  ||  (level > 95))
	{	message("Illegal level number.  Level number must be between 35 and 95.");
		return(1);
	}
	if((c < 0)  ||  (c > 19))
	{	message("Illegal coordinate type code.  Code must be between 0 and 19 inclusive.");
		return(1);
	}

	if(xy_item_check("header type",ht,1,6))
		return(1);
	if(xy_item_check("object no. pos.",in,0,20))
		return(1);
	if(xy_item_check("vertex count pos.",vc,0,20))
		return(1);
	if(xy_item_check("attribute pos.",at,0,20))
		return(1);
	if(xy_item_check("x coordinate pos.",x1,1,20))
		return(1);
	if(xy_item_check("x minutes pos.",x2,0,20))
		return(1);
	if(xy_item_check("x seconds pos.",x3,0,20))
		return(1);
	if(xy_item_check("y coordinate pos.",y1,1,20))
		return(1);
	if(xy_item_check("y minutes pos.",y2,0,20))
		return(1);
	if(xy_item_check("y seconds pos.",y3,0,20))
		return(1);

		/******* set up variables *************************************/
	message("import xy, setting up variables.");
	last_js16 = job_stats[16];
	last_js17 = job_stats[17];
	last_js18 = job_stats[18];
	fd = -1;
	for(i=1, v=0; i<256; i++)
	{	if(((i < '0') || (i > '9')) && (i != '.'))
		{	xydelims[v] = (unsigned char) i;
			v++;
		}
	}
	xydelims[v] = (unsigned char) 0;

	if(ht == 1)   /** only vertex lines **/
	{	max_param = (in > vc) ?   in : vc;
		max_param = (at > max_param) ?  at : max_param;
		max_param = (x1 > max_param) ?  x1 : max_param;
		max_param = (x2 > max_param) ?  x2 : max_param;
		max_param = (x3 > max_param) ?  x3 : max_param;
		max_param = (y1 > max_param) ?  y1 : max_param;
		max_param = (y2 > max_param) ?  y2 : max_param;
		max_param = (y3 > max_param) ?  y3 : max_param;
	}

		/************* output v_attribute record 0 dummy record ********/
	if(!job_stats[16])
	{	attr.next = 0;
		attr.string[0] = 0;
		if(put_v_attr(&attr,0))
		{	message("import xy, abort, location 0.");
			return(1);
		}
	}

		/************* find & asgn pointer to import file **************/
	if(get_impfile(fname,actual))
	{	message("import xy, abort, location 1.");
		return(1);
	}
	fd = openf(actual,READ);
	if(fd <= 0)
	{	message("Cannot open (1) file. aborting.");
		return(1);
	}
	

		/************* read all records ********************************/
	header_flag = ht - 1;	
	v = 0;
	vertex_count = 0;
	item_no = -1;
	object_count = 0;
	last_item_no = -1;
	last_len = -1;
	last_no_params = -1;
	for(line=0 ; reada(string,STRLIST,fd) > (char *) 0; line++, header_flag = 0)
	{	
		if(!(line % 250))
		{	sprintf(msg,"Reading & storing xy data, line %d, xy object %d.",line,object_count);
			message(msg);
		}

			/***** find line length ********/
		len = strlen(string);

			/***** parse into items ********/
		no_params = parse(trim(string),delim_buf,parsed,20);

			/***** if vertex lines only, check for not enough params ****/
		if((ht==1) && (no_params < max_param))
		{	header_flag = -1;    /** skips lines without enough data **/
			line--;
		}

			/***** check for header line or not ******/
		if(!header_flag) /** don't know if it is a header line or not **/
		{	if((otype == 0) && v)  /** node & a vertex stored **/	
				header_flag = 8;
			else if(ht == 2)           /** header only **/
				header_flag = 1;
			else if(vc)
			{	if(v >= vertex_count)
					header_flag = 2;
			}
			else if(ht == 1)      /** no header, chk item, at change   **/
			{	if(in)             /** check item number **/
				{	if(no_params >= in)
					{	if(sscanf(parsed[in-1],"%ld",&item_no))
							if(last_item_no != item_no)
								header_flag = 3;
					}
				}
				else if(at)
				{	if(no_params >= at)
					{	sprintf(attr.string,"%s",parsed[at - 1]);
						if(strcmp(last_atrstr,attr.string))
							header_flag = 4;
					}
				}
			}
			else if(in && ((ht == 4) || (ht == 6))) /** item number in xy line **/
			{	if(no_params >= in)
				{	if(sscanf(parsed[in-1],"%ld",&item_no))
						if(last_item_no != item_no)
							header_flag = 5;
				}
			}
			else if(at && ((ht == 5) || (ht == 6))) /** attribute in xy line **/
			{	if(no_params >= at)
				{	sprintf(attr.string,"%s",parsed[at - 1]);
					if(strcmp(last_atrstr,attr.string))
						header_flag = 6;
				}
			}
			else    /*** differentiate by line length or number of items ***/
			{	if((v) && ((len != last_len) || (no_params != last_no_params)))
					header_flag = 7;
			}
		}
			/*** set 'last' items ***/
		if(header_flag >= 0)
		{	last_len = len;
			last_no_params = no_params;
		}

		if(header_flag > 0)		/***** process header line ******/
		{		/**** store last object ****/
			if(line)
			{		/**** set ending vertex ****/
				v_object.v2 = job_stats[18] - 1;
					/**** determine object type ****/
				if(v_object.v1 > v_object.v2)   /** don't store ***/
				{	message("import xy, no verticies for object. Aborting");
					impxyabort(1);
					return(1);
				}
				if(v_object.v1 == v_object.v2)   /** point object **/
					v_object.type = 10 * level;
				else
				{	if(otype == 1)  /** segment **/
						v_object.type = 10 * level + 1;
					else
					{   /** read beginning & ending verticies to check for match **/
						if(get_v_coord(&crdlst1,v_object.v1))
						{	impxyabort(2);
							return(1);
						}
						if(get_v_coord(&crdlst,v_object.v2))
						{	impxyabort(3);
							return(1);
						}
						if(otype == 3)  /** auto **/
						{	if((crdlst.x == crdlst1.x) && (crdlst.y == crdlst1.y))
								v_object.type = 10 * level + 2;
							else
								v_object.type = 10 * level + 1;
						}
						else if(otype == 2)  /** polygon **/
						{		/*** check for closure ***/
							v_object.type = 10 * level + 2;
							if((crdlst.x != crdlst1.x) || (crdlst.y != crdlst1.y))
							{	if(put_v_coord(&crdlst1,job_stats[18]))
								{	impxyabort(3);
									return(1);
								}
								v_object.v2 = job_stats[18] - 1;
							}
						}
						else   /** segment **/
							v_object.type = 10 * level + 1;
					}
				}
					/**** store last object record *****/
				if(put_vobjects(&v_object,job_stats[17]))
				{	impxyabort(4);
					return(1);
				}
				object_count++;
			}
				/**** set-up new object record *****/
			v_object.type = 0;
			v_object.v1 = job_stats[18];
			v_object.v2 = 0;
			v_object.atr = 0;
				/**** set new attribute ***/
			if(at && (at <= no_params) && (ht < 5))
			{	sprintf(attr.string,"%s",parsed[at - 1]);
				v_object.atr = job_stats[16];
				attr.next = 0;
				if(put_v_attr(&attr,job_stats[16]))
				{	impxyabort(5);
					return(1);
				}
				sprintf(last_atrstr,"%s",attr.string);
			}
			else
				last_atrstr[0] = (char) 0;

				/**** set last item_no ****/
			last_item_no = item_no;

				/**** set vertex count  ****/
			if(vc && (no_params >= vc) && (ht != 2))
			{	if(!sscanf(parsed[vc-1],"%ld",&vertex_count))
				{	message("import xy, illegal vertex count for object. Aborting");
					impxyabort(6);
					return(1);
				}
			}
			v = 0;

				/**** set item number   ****/
			if(in && (no_params >= in) && (ht==3 || ht == 5))
			{	if(!sscanf(parsed[in-1],"%ld",&item_no))
				{	message("import xy, illegal vertex count for object. Aborting");
					impxyabort(7);
					return(1);
				}
			}
			else
				item_no = -1;

				/**** reset 'header' for vertex only header type ***/
			if(ht <= 2)
				header_flag = 0;
		}

		if(!header_flag)    /***** process vertex line ******/
		{		/**** get x & y values & store vertex ****/
			if(xy_store_vertex(parsed,xydelims,x1,x2,x3,y1,y2,y3,c))
			{	impxyabort(8);
				return(1);
			}
			v++;
				/**** get other values if present ****/
			if(in && (no_params >= in) && (ht==4 || ht == 6))
			{	if(!sscanf(parsed[in-1],"%ld",&item_no))
				{	message("import xy, illegal vertex count for object. Aborting");
					impxyabort(9);
					return(1);
				}
			}
			if(at && (at <= no_params) && (ht >= 5))
			{	sprintf(attr.string,"%s",parsed[at - 1]);
				v_object.atr = job_stats[16];
				attr.next = 0;
				if(put_v_attr(&attr,job_stats[16]))
				{	impxyabort(10);
					return(1);
				}
				sprintf(last_atrstr,"%s",attr.string);
			}
		}
	}

		/**** store last object record *****/
	if(v)
	{  /*** check ending & beginning coords to see if they match (poly) */
		v_object.v2 = job_stats[18] - 1;
		if(v_object.v1 == v_object.v2)   /** point object **/
			v_object.type = 10 * level;
		else
		{	if(otype == 1)  /** segment **/
				v_object.type = 10 * level + 1;
			else
			{   /** read beginning & ending verticies to check for match **/
				if(get_v_coord(&crdlst1,v_object.v1))
				{	impxyabort(2);
					return(1);
				}
				if(get_v_coord(&crdlst,v_object.v2))
				{	impxyabort(3);
					return(1);
				}
				if(otype == 3)  /** auto **/
				{	if((crdlst.x == crdlst1.x) && (crdlst.y == crdlst1.y))
						v_object.type = 10 * level + 2;
					else
						v_object.type = 10 * level + 1;
				}
				else if(otype == 2)  /** polygon **/
				{		/*** check for closure ***/
					v_object.type = 10 * level + 2;
					if((crdlst.x != crdlst1.x) || (crdlst.y != crdlst1.y))
					{	if(put_v_coord(&crdlst1,job_stats[18]))
						{	impxyabort(3);
							return(1);
						}
						v_object.v2 = job_stats[18] - 1;
					}
				}
				else   /** segment **/
					v_object.type = 10 * level + 1;
			}
		}

			/**** store last object record *****/
		if(put_vobjects(&v_object,job_stats[17]))
		{	impxyabort(13);
			return(1);
		}
		object_count++;
	}

	message("import xy, closing files.");
		/************* close file **************************************/
	closef_ptr(&fd);

		/************* if a copied dos file, purge it ******************/
#ifdef SV32
	/* can't access dos anyway */
#else
		/************* remove temp file, if any ************************/
	imp_rm_tmp(fname,actual);
#endif

		/************* index v_objects imported ************************/
	if(index_vobj(last_js17))
	{	job_stats[17] = last_js17;
		return(1);
	}

		/************* write diary *************************************/
	sprintf(string,"import xy file %s to level %d.",fname,level);
	put_diary(string);

		/************* set job_stats ***********************************/
	job_stats[45]++;
	job_stats[39] = job_stats[45];

		/************* set level name **********************************/
	sprintf(level_names[level],"imported xy (user defined) file: %s",fname);

		/*********** change display to add imported level **************/
	if(level < 64)
		disp_i1hglight |= (1 << (level % 32));
	else
		disp_i2hglight |= (1 << (level % 32));

		/*********** re-display map ************************************/
	dispgraphics((long) ((rwindowxmin + rwindowxmax) / 2),(long) ((rwindowymin + rwindowymax) / 2),resolution_code);

		/************* return ******************************************/

	return(0);
}


/*******************************************************************************
*      impxyabort                                                            *
*          This program is a convenient way of closing the input  file and     *
*      writing a message to the user when the program aborts.  no system       *
*      globals or files are changed.                                           *
*                                                                              *
*      Parameter:   long  location    unique abort location to be printed.     *
*******************************************************************************/

long impxyabort(location)
long location;
{	
	char string[256];


		/************ close file fd if open ****************************/
	if(fd > 0)
		closef_ptr(&fd);

		/************* reset job_stats variables ************************/
	job_stats[16] = last_js16;
	job_stats[17] = last_js17;
	job_stats[18] = last_js18;

		/************* print abort message ******************************/
	sprintf(string,"import xy, aborted, location %d",location);
	message(string);

		/************* return *******************************************/
	return(0);
}
/*******************************************************************************
*      xy_item_check                                                         *
*          This program check item number   specifications.                    *
*******************************************************************************/

long xy_item_check(name,pos,min,max)
char *name;
long pos, min, max;
{
	char string[96];

		/*** check for legal values (min - max) ***/
	if((pos < min) || (pos > max))
	{	sprintf(string,"Import xy: Illegal position spec. for %s.  Aborting.",name);
		return(1);
	}

	return(0);
}


/*******************************************************************************
*  xy_store_vertex                                                             *
*       reads vertex from parsed parameters, converts to pixel coords, stores  *
*    1 vertex.                                                                 *
*******************************************************************************/


long xy_store_vertex(parsed,xydelims,x1,x2,x3,y1,y2,y3,c)
char parsed[][STRLIST];
char *xydelims;
long x1,x2,x3,y1,y2,y3,c;
{
	COORD_LIST coordlist;
	double x,y;

		/************ decode x into decimal value *******************/
	if(xy_decode_dec(parsed,xydelims,x1,x2,x3,&x))
	{	message("import xy, cannot decode x value. aborting");
		return(1);
	}

		/************ decode y into decimal value *******************/
	if(xy_decode_dec(parsed,xydelims,y1,y2,y3,&y))
	{	message("import xy, cannot decode x value. aborting");
		return(1);
	}

		/************ make longitude negative ***********************/
	if(c == 1)   /** lat lon **/
	{	if(x < 0.0)
			x = -x;
	}

		/************ transform x,y into pixel coordinates **********/
	if(transform(&x,&y,c,0))
		return(1);

		/************ store vertex **********************************/
	coordlist.x = x;
	coordlist.y = y;
	if(put_v_coord(&coordlist,job_stats[18]))
		return(1);
	
		/************ return ****************************************/
	return(0);
}


/*******************************************************************************
*  xy_decode_dec                                                               *
*       decodes one x or y value which may be in several forms:  dd dm dms     *
*******************************************************************************/

long xy_decode_dec(parsed,xydelims,z1,z2,z3,z)
char parsed[][STRLIST];
char *xydelims;
long z1,z2,z3;
double *z;
{
	char zparsed[3][STRLIST];
	long i, n;
	long neg;
	double k;

		/************ init value of z *******************************/
	*z = 0.0;

		/************ check for '-' (negative value)    *************/
	if(parsed[z1 - 1][0] == '-')
	{	neg = 1;
		parsed[z1 - 1][0] = (char) 32;
	}
	else
		neg = 0;

		/************ if no z2, attempt to parse string *************/
	if(!z2)
	{	n = parse(parsed[z1-1],xydelims,zparsed,3);
		for(i=n-1; i>=0; i--)
		{	*z = *z / 60.0;
			if(sscanf(zparsed[i],"%lf",&k))
				*z += k;
		}
	}
	else   /*** otherwise use all parsed values present ***/
	{	k = 0.0;
		sscanf(parsed[z1-1],"%lf",z);
		sscanf(parsed[z2-1],"%lf",&k);
		*z = *z + k/60.0;
		if(z3)
		{	k = 0.0;
			sscanf(parsed[z3-1],"%lf",&k);
			*z = *z + k/3600.0;
		}
	}

	if(neg)
		*z = -(*z);

		/************ return ****************************************/
	return(0);
}
