/*******************************************************************************
*    level_clear   (see manual, Command Ref., User Functions)    <fc>          *
*            This program clears a range of levels of import data.             *
*                                                                              *
*       files/globals changed:  job_stats, index_array, index_list,            *
*        v_objects, v_coordlist, v_attributes                                  *
*                                                                              *
*       values returned:  long 0= ok,  1=failure                               *
*                                                                              *
*       parameters:    long level1   beginning of range to be cleared.         *
*                      long level2   end of range to be cleared.               *
*                                                                              *
*            J.Dabritz   9/20/89                                               *
*******************************************************************************/

#include "../../incl/globals.h"

int vobj2fd;
int vatr2fd;
int vcdl2fd;
int new_index_list_fd;

long lcabort();
char *malloc_ltp();

long level_clear(level1, level2)
	long level1, level2;
{
	OBJECTS object;
	V_OBJECTS v_object;
	ATTRIBUTES v_attr;
	COORD_LIST coordlist;
	INDEX_LIST index_list;

	char path[256];
	char from[256], to[256];
	char new_path[256];
	char string[256], string2[256];

	long obj_rec, cdl_rec, atr_rec;
	long c,i,j,r;
	long object_level;
	long coordlist1;
	long increment;
	long next;
	long size;
	long new_idx_ary_recs;
	long rows;
	long cols;
	long x,y;
	long mask;

	

		/*********** check for map & v_objects file *******************/
	if(!job_stats[0])
	{	message("You must acquire a map befor  level_clear  function can be used.");
		return(1);
	}

		/*********** check input parameters ***************************/
	if((level1 > level2) || (level1 < 0)  || (level2 > 95))
	{	message("Illegal level number.  Both beginning and ending levels must be in range 0-95.");
		return(1);
	}

		/*********** clear any set levels 1-24 *****************************/
	if(level1 < 25)
	{	mask = 0;
		for(i=level1; i<25  &&  i<=level2;  i++)
		{	if(i >= 1)
				mask |= (1 << i);
		}
	
		if(mask)
		{	mask = ~mask;
				/**************** clear set flags ***********************/
			for(i=0; i<job_stats[10]; i++)
			{	if(get_objects(&object,i))
				{	message("Can't get object, aborting command.");
					return(1);
				}
				object.i12[4] &= mask;
				if(put_objects(&object,i))
				{	message("Can't store object, aborting command.");
					return(1);
				}
			}
		}
	
		for(i=level1; i<25  &&  i<=level2;  i++)
		{	if(i >= 1)
				sprintf(level_names[i],"set %d, - empty -",i);
		}
	}


		/*********** clear levels 32+ **************************************/
	if(level2 >= 32)
	{
		
		if(!job_stats[17])
			return(0);


		message("Level_clear, setting up files and variables.");
			/*********** initialize file pointer variables ***********************/
		vobj2fd = -1;
		vatr2fd = -1;
		vcdl2fd = -1;

			/*********** set up new v_objects, v_attributes, & v_coordlist files */
		sprintf(path,"%s/v_objects2",temp_map_path);
		purgef(path);
		if(creatf(path) < 0)
		{	message("Unable to create temporary files.  aborting");
			return(1);
		}
		vobj2fd = openf(path,WRITE);
		if(vobj2fd < 0)
		{	message("Unable to open temporary files.  aborting");
			return(1);
		}
		obj_rec = 0;

		sprintf(path,"%s/v_coordlst2",temp_map_path);
		purgef(path);
		if(creatf(path) < 0)
		{	message("Unable to create temporary files.  aborting");
			lcabort(0);
			return(1);
		}
		vcdl2fd = openf(path,WRITE);
		if(vcdl2fd < 0)
		{	message("Unable to open temporary files.  aborting");
			lcabort(0);
			return(1);
		}
		cdl_rec = 0;

		sprintf(path,"%s/v_attr2",temp_map_path);
		purgef(path);
		if(creatf(path) < 0)
		{	message("Unable to create temporary files.  aborting");
			lcabort(0);
			return(1);
		}
		vatr2fd = openf(path,WRITE);
		if(vatr2fd < 0)
		{	message("Unable to open temporary files.  aborting");
			lcabort(0);
			return(1);
		}
		v_attr.next = 0;
		v_attr.string[0] = 0;
		if(1 > writeb((char *) &v_attr, v_attributesm.rec_size, 1, vatr2fd))
		{	message("unable to write to temporary file.  aborting.");
			lcabort(0);
			return(1);
		}
		atr_rec = 1;

			/*********** re_write old v_  files, eliminating levels *******/
		for(i=0; i<job_stats[17]; i++)
		{		/**** read v_objects rec *****************/
			if(get_vobjects(&v_object,i))
			{	lcabort(0);
				return(1);
			}
			
			if(!(i % 250))
			{	sprintf(string,"Deleting level(s) from vector files, object %d",i);
				message(string);
			}

				/**** if another level, copy to new files */
			object_level = v_object.type / 10;
			if((object_level < level1)  ||  (object_level > level2))
			{		/**** copy coordlist recs ****************/
				coordlist1 = cdl_rec;
				if(v_object.v2 >= v_object.v1)
					increment = 1;
				else
					increment = -1;

				for(j=v_object.v1; (j>=0)  &&  (j<job_stats[18]) ; j += increment)
				{
					if(get_v_coord(&coordlist,j))
					{	lcabort(0);
						return(1);
					}

					if(1 > writeb((char *) &coordlist, v_coordlistm.rec_size, 1, vcdl2fd))
					{	message("unable to write to temporary file.  aborting.");
						lcabort(0);
						return(1);
					}
					cdl_rec++;

					if(j == v_object.v2)
						break;
				}
				v_object.v1 = coordlist1;
				v_object.v2 = cdl_rec - 1;

					/**** copy attribute recs ****************/
				if(v_object.atr)
				{	next = v_object.atr;
					v_object.atr = atr_rec;
					for(; next;)
					{	if(get_v_attr(&v_attr,next))
						{	lcabort(0);
							return(1);
						}	
						next = v_attr.next;
						if(next)
							v_attr.next = atr_rec + 1;
						if(1 > writeb((char *) &v_attr, v_attributesm.rec_size, 1, vatr2fd))
						{	message("unable to write to temporary file.  aborting.");
							lcabort(0);
							return(1);
						}
						atr_rec++;
						
					}
				}

					/**** write object rec ********************/
				if(1 > writeb((char *) &v_object, v_objectsm.rec_size, 1, vobj2fd))
				{	message("unable to write to temporary file.  aborting.");
					lcabort(0);
					return(1);
				}
				obj_rec++;
			}
		}
		closef(vobj2fd);
		closef(vcdl2fd);
		closef(vatr2fd);

		message("Replacing old files with the new ones.");
			/*********** replace old files with the new ones **************/
			/***** 1.  v_objects file *******/
		if(v_objects_fd > 0)
		{	closef(v_objects_fd);
			v_objects_fd = -1;
		}
		if(v_objectsm.malloc)
		{	free_ltp(v_objectsm.malloc);
			v_objectsm.malloc = (char *) 0;
		}

		sprintf(from,"%s/v_objects2",temp_map_path);
		sprintf(to,"%s/v_objects",temp_map_path);
		if(move_file(from,to))
		{	message("unable to displace old import files.  aborting");
			lcabort(0);
			return(1);
		}
		job_stats[17] = obj_rec;
		

			/********* v_objects ***********/
			/*    open file        */
		sprintf(path,"%s/v_objects",temp_map_path);
		v_objects_fd = openf(path,RDWR);
		if(0 > v_objects_fd)
		{	message("  <<  cannot open v_objects map file.");
			lcabort(1);
			return(1);
		}
			/*  calculate size to malloc */
		size = v_objectsm.rec_size * job_stats[17];
			/*  malloc space & set pointer or leave file (stream) pointer  */
		if(size)
		{	if(ramfree(size + RAMSPACE))
			{	if(v_objectsm.malloc = (malloc_ltp(size)))
				{   v_objectsm.rec0 = v_objectsm.malloc;
					v_objectsm.recn = v_objectsm.rec0 + size;
					if(!readb(v_objectsm.rec0,size,1,v_objects_fd))
					{	message("  <<  cannot read v_objects map file.");
						lcabort(1);
						return(1);
					}
				}
			}
		}


			/***** 2.  v_coordlist file *******/
		if(v_coordlist_fd > 0)
		{	closef(v_coordlist_fd);
			v_coordlist_fd = -1;
		}
		if(v_coordlistm.malloc)
		{	free_ltp(v_coordlistm.malloc);
			v_coordlistm.malloc = (char *) 0;
		}

		sprintf(from,"%s/v_coordlst2",temp_map_path);
		sprintf(to,"%s/v_coordlist",temp_map_path);
		if(move_file(from,to))
		{	message("unable to displace old import files.  aborting");
			lcabort(1);
			return(1);
		}
		job_stats[18] = cdl_rec;

			/********* v_coordlist ***********/
			/*    open file        */
		sprintf(path,"%s/v_coordlist",temp_map_path);
		v_coordlist_fd = openf(path,RDWR);
		if(0 > v_coordlist_fd)
		{	message("  <<  cannot open v_coordlist map file.");
			lcabort(1);
			return(1);
		}
			/*  calculate size to malloc */
		size = v_coordlistm.rec_size * job_stats[18];
			/*  malloc space & set pointer or leave file (stream) pointer  */
		if(size)
		{	if(ramfree(size + RAMSPACE))
			{	if(v_coordlistm.malloc = (malloc_ltp(size)))
				{   v_coordlistm.rec0 = v_coordlistm.malloc;
					v_coordlistm.recn = v_coordlistm.rec0 + size;
					if(!readb(v_coordlistm.rec0,size,1,v_coordlist_fd))
					{	message("  <<  cannot read v_coordlist map file.");
						lcabort(1);
						return(1);
					}
				}
			}
		}


			/***** 3.  v_attributes file *******/
		if(v_attributes_fd > 0)
		{	closef(v_attributes_fd);
			v_attributes_fd = -1;
		}
		if(v_attributesm.malloc)
		{	free_ltp(v_attributesm.malloc);
			v_attributesm.malloc = (char *) 0;
		}

		sprintf(from,"%s/v_attr2",temp_map_path);
		sprintf(to,"%s/v_attributes",temp_map_path);
		if(move_file(from,to))
		{	message("unable to displace old import files.  aborting");
			lcabort(1);
			return(1);
		}
		job_stats[16] = atr_rec;

			/********* v_attributes ***********/
			/*    open file        */
		sprintf(path,"%s/v_attributes",temp_map_path);
		v_attributes_fd = openf(path,RDWR);
		if(0 > v_attributes_fd)
		{	message("  <<  cannot open v_attributes map file.");
			lcabort(1);
			return(1);
		}
			/*  calculate size to malloc */
		size = v_attributesm.rec_size * job_stats[16];
			/*  malloc space & set pointer or leave file (stream) pointer  */
		if(size)
		{	if(ramfree(size + RAMSPACE))
			{	if(v_attributesm.malloc = (malloc_ltp(size)))
				{   v_attributesm.rec0 = v_attributesm.malloc;
					v_attributesm.recn = v_attributesm.rec0 + size;
					if(!readb(v_attributesm.rec0,size,1,v_attributes_fd))
					{	message("  <<  cannot read v_attributes map file.");
						lcabort(1);
						return(1);
					}
				}
			}
		}
			/** make sure v_attributes has at least one record **/
		if(!job_stats[16])
		{	*v_attr.string = 0;
			v_attr.next = 0;
			if(put_v_attr(&v_attr,0))
				return(1);
		}



			/*********** re-write index_list eliminating v_object references */
		sprintf(new_path,"%s/index_list2",temp_map_path);
		purgef(new_path);
		creatf(new_path);
		new_index_list_fd = openf(new_path,RDWR);
		if(new_index_list_fd <= 0)
		{	message(" cannot open new index_list file.  aborting.");
			lcabort(1);
			return(1);
		}
		new_idx_ary_recs = 0;
			/** write a blank first record so that real recs start at 1 */
		if(writeb((char *)&index_list,index_listm.rec_size,1,new_index_list_fd) < 0)
		{	message("unable to write into new index_list file.  aborting process.");
			lcabort(1);
			return(1);
		}
		new_idx_ary_recs++;
		rows = (job_stats[2] >> 5);
		cols = (job_stats[1] >> 5);
		for(r = 0; r <= rows; r++)
		{
			if(!(r % 5))
			{	sprintf(string,"Re-writing index_list excluding import levels, row %d.",r);
				message(string);
			}
			for(c = 0; c <= cols; c++)
			{	j = index_array[r][c];
				index_array[r][c] = 0;
				for(;j;)
				{	if(get_indexl(&index_list,j))
					{	message("abort, index_map, location 3");
						return(1);
					}
					j = index_list.next;
					if(index_list.type < 320)
					{	i = index_array[r][c];
						index_array[r][c] = new_idx_ary_recs++;
						index_list.next = i;
						if(writeb((char *)&index_list,index_listm.rec_size,1
						 ,new_index_list_fd) < 0)
						{	message("unable to write into new index_list file.  aborting process.");
							return(1);
						}
					}
				}
			}
		}
		job_stats[15] = new_idx_ary_recs;
		
			/********* remove old file **************************************/
		if(index_list_fd > 0)
		{	closef(index_list_fd);
			index_list_fd = -1;
		}

		if(index_listm.malloc)
		{	free_ltp(index_listm.malloc);
			index_listm.malloc = (char *) 0;            
		}

		sprintf(string,"%s/index_list",temp_map_path);
		purgef(string);

			/********* move new file ****************************************/
		if(new_index_list_fd > 0)
			closef(new_index_list_fd);
		sprintf(string,"%s/index_list2",temp_map_path);
		sprintf(string2,"%s/index_list",temp_map_path);
		if(move_file(string,string2))
		{	message("can't move new index_list file.  aborting.");
			lcabort(1);
			return(1);
		}

			/*    open file        */
			/********* check memory & load new file or set pointer to it ****/
		sprintf(path,"%s/index_list",temp_map_path);
		index_list_fd = openf(path,RDWR);
		if(0 >= index_list_fd)
		{	message("  <<  cannot open index_list map file.");
			lcabort(1);
			return(1);
		}
			/*  calculate size to malloc */
		size = index_listm.rec_size * job_stats[15];
			/*  malloc space & set pointer or leave file (stream) pointer  */
		if(size)
		{	if(ramfree(size + RAMSPACE))
			{	if(index_listm.malloc = (char *) malloc_ltp(size))
				{   index_listm.rec0 = index_listm.malloc;
					index_listm.recn = index_listm.rec0 + size;
					if(!readb(index_listm.rec0,size,1,index_list_fd))
					{	message("  <<  cannot read index_list map file.");
						lcabort(1);
						return(0);
					}
				}
			}
		}


			/*********** re-index all v_objects ***************************/
		message("Indexing all remaining import levels.");
		if(index_vobj(0))
		{	lcabort(1);
			return(1);
		}
		

			/*********** clear level name *********************************/
		for(i=level1; i<=level2; i++)
		{	if((i >= 32) && (i <= 95))
				sprintf(level_names[i]," - unused -",i);
		}

	}


		/*********** redisplay graphics *******************************/
	x = (rwindowxmin + rwindowxmax) / 2;
	y = (rwindowymin + rwindowymax) / 2;
	if(dispgraphics(x,y,resolution_code))
		return(1);


		/*********** return *******************************************/
	return(0);
}

long lcabort(corrupt)
	long corrupt;
{
	if(vobj2fd > 0)
		closef(vobj2fd);

	if(vatr2fd > 0)
		closef(vatr2fd);

	if(vcdl2fd > 0)
		closef(vcdl2fd);

	if(new_index_list_fd > 0)
		closef(new_index_list_fd);

	if(corrupt)
	{	message("The level_clear function has aborted in a place that may leave the temporary");
		message("map file corrupted.  It is STRONGLY suggested that you acquire the map again ");
		message("WITHOUT saveing changes.");
		sleep(10);
	}

	return(0);
}
