/*******************************************************************************
*    level_clip   (see manual, Command Ref., User Functions)    <fc>          *
*            This program clips a range of levels of import data.             *
*                                                                              *
*       files/globals changed:  job_stats, index_array, index_list,            *
*        v_objects, v_coordlist, v_attributes                                  *
*                                                                              *
*       values returned:  long 0= ok,  1=failure                               *
*                                                                              *
*       parameters:    long level1   beginning of range to be cliped.         *
*                      long level2   end of range to be cliped.               *
*                                                                              *
*            J.Dabritz   9/20/89                                               *
*******************************************************************************/

#include "../../incl/globals.h"

int vobj2fd;
int vatr2fd;
int vcdl2fd;
int new_index_list_fd;

long lcpabort();
char *malloc_ltp();
long checkpair();
long code_outside();

long level_clip(level1, level2)
	long level1, level2;
{
	V_OBJECTS v_object;
	ATTRIBUTES v_attr;
	COORD_LIST coordlist,crdlst1,crdlst2;
	INDEX_LIST index_list;

	char path[256];
	char from[256], to[256];
	char new_path[256];
	char string[256], string2[256];

	long obj_rec, cdl_rec, atr_rec;
	long c,i,j,r;
	long object_level;
	long coordlist1;
	long increment;
	long next;
	long size;
	long new_idx_ary_recs;
	long rows;
	long cols;
	long xi,yi;
	long v1, v2; 
	long begin_orec;
	long inside,last_inside;
	long cross;

	double x,y,last_x,last_y;


		/*********** check for map & v_objects file *******************/
	if(!job_stats[0])
	{	message("You must acquire a map befor  level_clip  function can be used.");
		return(1);
	}

		/*********** check input parameters ***************************/
	if((level1 > level2) || (level1 < 35)  || (level2 > 95))
	{	message("Illegal level number.  Both beginning and ending levels must be in range 35-95.");
		return(1);
	}

		/*********** check check for level data ***********************/
	if(!job_stats[17])
	{	message("you must IMPORT vector data befor you can level_clip it.");
		return(1);
	}

		/*********** check registration *******************************/
	if(!(registered(0)))
	{	message("you must have a REGISTERED RASTER befor you can use level_clip.");
		return(1);
	}


	message("Level_clip, setting up files and variables.");
		/*********** initialize file pointer variables ***********************/
	vobj2fd = -1;
	vatr2fd = -1;
	vcdl2fd = -1;

		/*********** set-up registration for clip checking *******************/
	corner[8][0][0] = 0.0;     /*** a 1 unit box ***/
	corner[8][0][1] = 0.0;
	corner[8][1][0] = 0.0;
	corner[8][1][1] = 1.0;
	corner[8][2][0] = 1.0;
	corner[8][2][1] = 1.0;
	corner[8][3][0] = 1.0;
	corner[8][3][1] = 0.0;

		/*********** set up new v_objects, v_attributes, & v_coordlist files */
	sprintf(path,"%s/v_objects2",temp_map_path);
	purgef(path);
	if(creatf(path) < 0)
	{	message("Unable to create temporary files.  aborting");
		return(1);
	}
	vobj2fd = openf(path,WRITE);
	if(vobj2fd < 0)
	{	message("Unable to open temporary files.  aborting");
		return(1);
	}
	obj_rec = 0;

	sprintf(path,"%s/v_coordlst2",temp_map_path);
	purgef(path);
	if(creatf(path) < 0)
	{	message("Unable to create temporary files.  aborting");
		lcpabort(0);
		return(1);
	}
	vcdl2fd = openf(path,WRITE);
	if(vcdl2fd < 0)
	{	message("Unable to open temporary files.  aborting");
		lcpabort(0);
		return(1);
	}
	cdl_rec = 0;

	sprintf(path,"%s/v_attr2",temp_map_path);
	purgef(path);
	if(creatf(path) < 0)
	{	message("Unable to create temporary files.  aborting");
		lcpabort(0);
		return(1);
	}
	vatr2fd = openf(path,WRITE);
	if(vatr2fd < 0)
	{	message("Unable to open temporary files.  aborting");
		lcpabort(0);
		return(1);
	}
	v_attr.next = 0;
	v_attr.string[0] = 0;
	if(1 > writeb((char *) &v_attr, v_attributesm.rec_size, 1, vatr2fd))
	{	message("unable to write to temporary file.  aborting.");
		lcpabort(0);
		return(1);
	}
	atr_rec = 1;

		/*********** re_write old v_  files, clipping indicated levels *******/
	for(i=0; i<job_stats[17]; i++)
	{		/**** read v_objects rec *****************/
		if(get_vobjects(&v_object,i))
		{	lcpabort(0);
			return(1);
		}
		
		coordlist1 = cdl_rec;
		v1 = v_object.v1;
		v2 = v_object.v2;
		if(v2 >= v1)
			increment = 1;
		else
			increment = -1;

		if(!(i % 250))
		{	sprintf(string,"clipping level(s), object %d",i);
			message(string);
		}

			/**** see if object is on a level to be clipped/checked ***/
		object_level = v_object.type / 10;
		if((object_level >= level1) && (object_level <= level2)) /* check level */
		{		/**** count new obj recs stored for this old object ***/
			begin_orec = obj_rec;
			next = v_object.atr;
			if(next)
				v_object.atr = atr_rec;
			else
				v_object.atr = 0;

				/**** read 1st coord & set inside/outside flag ***/
			if(get_v_coord(&coordlist,v1))
			{	lcpabort(0);
				return(1);
			}
			j = v1 + increment;
			x = coordlist.x;
			y = coordlist.y;
			if(transform(&x,&y,0,8))
			{	lcpabort(0);
				return(1);
			}
			if((x < 0.0) || (x > 1.0) || (y < 0.0) || (y > 1.0))
				inside = 0;
			else
			{	inside = 1;
				v_object.v1 = coordlist1;
				if(1 > writeb((char *) &coordlist,v_coordlistm.rec_size,1,vcdl2fd))
				{	message("unable to write to temporary file.  aborting.");
					lcpabort(0);
					return(1);
				}
				cdl_rec++;
			}


				/**** read & copy coordlist recs ****************/
			if(v1 == v2)  /** one point **/
			{
				if(inside)
				{	v_object.v2 = cdl_rec - 1;
						/**** write object rec ********************/
					if(1 > writeb((char *) &v_object,v_objectsm.rec_size,1,vobj2fd))
					{	message("unable to write to temporary file.  aborting.");
						lcpabort(0);
						return(1);
					}
					obj_rec++;
				}
			}
			else
			{	for(j=v1+increment; (j>=0)  &&  (j<job_stats[18]) ; j += increment)
				{		/*** read next coords ***/
					last_x = coordlist.x;
					last_y = coordlist.y;
					if(get_v_coord(&coordlist,j))
					{	lcpabort(0);
						return(1);
					}
					x = coordlist.x;
					y = coordlist.y;
	
						/*** see if path has any part within map frame ***/
					last_inside = inside;
					inside = checkpair(x,y,last_x,last_y,&crdlst1,&crdlst2,&cross);
					if(inside < 0)
					{	lcpabort(0);
						return(1);
					}

						/*** store last coordlst if necessary **/
					if(cross && !last_inside)
					{	if(1>writeb((char *)&crdlst2,v_coordlistm.rec_size,1,vcdl2fd))
						{	message("unable to write to temporary file.  aborting.");
							lcpabort(0);
							return(1);
						}
						v_object.v1 = cdl_rec;
						cdl_rec++;
					}

						/*** store this coordlst if necessary **/
					if(cross)
					{	if(1>writeb((char *)&crdlst1,v_coordlistm.rec_size,1,vcdl2fd))
						{	message("unable to write to temporary file.  aborting.");
							lcpabort(0);
							return(1);
						}
						cdl_rec++;
					}

						/*** store object record if necessary **/
					if(cross && ((j == v2) || !inside))
					{	/**** write object rec ********************/
						v_object.v2 = cdl_rec - 1;
						if(1 > writeb((char *) &v_object, v_objectsm.rec_size, 1, vobj2fd))
						{	message("unable to write to temporary file.  aborting.");
							lcpabort(0);
							return(1);
						}
						obj_rec++;
					}

					if(j == v2)
						break;
				}
			}

				/**** write attribute recs if object recs stored. ***/
			if((begin_orec < obj_rec) && (v_object.atr))
			{	for(; next;)
				{	if(get_v_attr(&v_attr,next))
					{	lcpabort(0);
						return(1);
					}	
					next = v_attr.next;
					if(next)
						v_attr.next = atr_rec + 1;
					if(1 >writeb((char *) &v_attr,v_attributesm.rec_size,1,vatr2fd))
					{	message("unable to write to temporary file.  aborting.");
						lcpabort(0);
						return(1);
					}
					atr_rec++;
				}
			}
		}
		else
		{		/**** read & copy coordlist recs ****************/
			for(j=v_object.v1; (j>=0)  &&  (j<job_stats[18]) ; j += increment)
			{	if(get_v_coord(&coordlist,j))
				{	lcpabort(0);
					return(1);
				}

				if(1 > writeb((char *) &coordlist,v_coordlistm.rec_size,1,vcdl2fd))
				{	message("unable to write to temporary file.  aborting.");
					lcpabort(0);
					return(1);
				}
				cdl_rec++;

				if(j == v_object.v2)
					break;
			}

			v_object.v1 = coordlist1;
			v_object.v2 = cdl_rec - 1;

				/**** copy attribute recs ****************/
			if(v_object.atr)
			{	next = v_object.atr;
				v_object.atr = atr_rec;
				for(; next;)
				{	if(get_v_attr(&v_attr,next))
					{	lcpabort(0);
						return(1);
					}	
					next = v_attr.next;
					if(next)
						v_attr.next = atr_rec + 1;
					if(1 >writeb((char *) &v_attr,v_attributesm.rec_size,1,vatr2fd))
					{	message("unable to write to temporary file.  aborting.");
						lcpabort(0);
						return(1);
					}
					atr_rec++;
				}
			}

				/**** write object rec ********************/
			if(1 > writeb((char *) &v_object, v_objectsm.rec_size, 1, vobj2fd))
			{	message("unable to write to temporary file.  aborting.");
				lcpabort(0);
				return(1);
			}
			obj_rec++;
		}
	}
	closef(vobj2fd);
	closef(vcdl2fd);
	closef(vatr2fd);

	message("Replacing old files with the new ones.");
		/*********** replace old files with the new ones **************/
		/***** 1.  v_objects file *******/
	if(v_objects_fd > 0)
	{	closef(v_objects_fd);
		v_objects_fd = -1;
	}
	if(v_objectsm.malloc)
	{	free_ltp(v_objectsm.malloc);
		v_objectsm.malloc = (char *) 0;
	}

	sprintf(from,"%s/v_objects2",temp_map_path);
	sprintf(to,"%s/v_objects",temp_map_path);
	if(move_file(from,to))
	{	message("unable to displace old import files.  aborting");
		lcpabort(0);
		return(1);
	}
	job_stats[17] = obj_rec;
	

		/********* v_objects ***********/
		/*    open file        */
	sprintf(path,"%s/v_objects",temp_map_path);
	v_objects_fd = openf(path,RDWR);
	if(0 > v_objects_fd)
	{	message("  <<  cannot open v_objects map file.");
		lcpabort(1);
		return(1);
	}
		/*  calculate size to malloc */
	size = v_objectsm.rec_size * job_stats[17];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(v_objectsm.malloc = (malloc_ltp(size)))
			{   v_objectsm.rec0 = v_objectsm.malloc;
				v_objectsm.recn = v_objectsm.rec0 + size;
				if(!readb(v_objectsm.rec0,size,1,v_objects_fd))
				{	message("  <<  cannot read v_objects map file.");
					lcpabort(1);
					return(1);
				}
			}
		}
	}


		/***** 2.  v_coordlist file *******/
	if(v_coordlist_fd > 0)
	{	closef(v_coordlist_fd);
		v_coordlist_fd = -1;
	}
	if(v_coordlistm.malloc)
	{	free_ltp(v_coordlistm.malloc);
		v_coordlistm.malloc = (char *) 0;
	}

	sprintf(from,"%s/v_coordlst2",temp_map_path);
	sprintf(to,"%s/v_coordlist",temp_map_path);
	if(move_file(from,to))
	{	message("unable to displace old import files.  aborting");
		lcpabort(1);
		return(1);
	}
	job_stats[18] = cdl_rec;

		/********* v_coordlist ***********/
		/*    open file        */
	sprintf(path,"%s/v_coordlist",temp_map_path);
	v_coordlist_fd = openf(path,RDWR);
	if(0 > v_coordlist_fd)
	{	message("  <<  cannot open v_coordlist map file.");
		lcpabort(1);
		return(1);
	}
		/*  calculate size to malloc */
	size = v_coordlistm.rec_size * job_stats[18];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(v_coordlistm.malloc = (malloc_ltp(size)))
			{   v_coordlistm.rec0 = v_coordlistm.malloc;
				v_coordlistm.recn = v_coordlistm.rec0 + size;
				if(!readb(v_coordlistm.rec0,size,1,v_coordlist_fd))
				{	message("  <<  cannot read v_coordlist map file.");
					lcpabort(1);
					return(1);
				}
			}
		}
	}


		/***** 3.  v_attributes file *******/
	if(v_attributes_fd > 0)
	{	closef(v_attributes_fd);
		v_attributes_fd = -1;
	}
	if(v_attributesm.malloc)
	{	free_ltp(v_attributesm.malloc);
		v_attributesm.malloc = (char *) 0;
	}

	sprintf(from,"%s/v_attr2",temp_map_path);
	sprintf(to,"%s/v_attributes",temp_map_path);
	if(move_file(from,to))
	{	message("unable to displace old import files.  aborting");
		lcpabort(1);
		return(1);
	}
	job_stats[16] = atr_rec;

		/********* v_attributes ***********/
		/*    open file        */
	sprintf(path,"%s/v_attributes",temp_map_path);
	v_attributes_fd = openf(path,RDWR);
	if(0 > v_attributes_fd)
	{	message("  <<  cannot open v_attributes map file.");
		lcpabort(1);
		return(1);
	}
		/*  calculate size to malloc */
	size = v_attributesm.rec_size * job_stats[16];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(v_attributesm.malloc = (malloc_ltp(size)))
			{   v_attributesm.rec0 = v_attributesm.malloc;
				v_attributesm.recn = v_attributesm.rec0 + size;
				if(!readb(v_attributesm.rec0,size,1,v_attributes_fd))
				{	message("  <<  cannot read v_attributes map file.");
					lcpabort(1);
					return(1);
				}
			}
		}
	}
		/** make sure v_attributes has at least one record **/
	if(!job_stats[16])
	{	*v_attr.string = 0;
		v_attr.next = 0;
		if(put_v_attr(&v_attr,0))
			return(1);
	}



		/*********** re-write index_list eliminating v_object references */
	sprintf(new_path,"%s/index_list2",temp_map_path);
	purgef(new_path);
	creatf(new_path);
	new_index_list_fd = openf(new_path,RDWR);
	if(new_index_list_fd <= 0)
	{	message(" cannot open new index_list file.  aborting.");
		lcpabort(1);
		return(1);
	}
	new_idx_ary_recs = 0;
		/** write a blank first record so that real recs start at 1 */
	if(writeb((char *)&index_list,index_listm.rec_size,1,new_index_list_fd) < 0)
	{	message("unable to write into new index_list file.  aborting process.");
		lcpabort(1);
		return(1);
	}
	new_idx_ary_recs++;
	rows = (job_stats[2] >> 5);
	cols = (job_stats[1] >> 5);
	for(r = 0; r <= rows; r++)
	{
		if(!(r % 5))
		{	sprintf(string,"Re-writing index_list excluding import levels, row %d.",r);
			message(string);
		}
		for(c = 0; c <= cols; c++)
		{	j = index_array[r][c];
			index_array[r][c] = 0;
			for(;j;)
			{	if(get_indexl(&index_list,j))
				{	message("abort, index_map, location 3");
					return(1);
				}
				j = index_list.next;
				if(index_list.type < 320)
				{	i = index_array[r][c];
					index_array[r][c] = new_idx_ary_recs++;
					index_list.next = i;
					if(writeb((char *)&index_list,index_listm.rec_size,1
					 ,new_index_list_fd) < 0)
					{	message("unable to write into new index_list file.  aborting process.");
						return(1);
					}
				}
			}
		}
	}
	job_stats[15] = new_idx_ary_recs;
	
		/********* remove old file **************************************/
	if(index_list_fd > 0)
	{	closef(index_list_fd);
		index_list_fd = -1;
	}

	if(index_listm.malloc)
	{	free_ltp(index_listm.malloc);
		index_listm.malloc = (char *) 0;            
	}

	sprintf(string,"%s/index_list",temp_map_path);
	purgef(string);

		/********* move new file ****************************************/
	if(new_index_list_fd > 0)
		closef(new_index_list_fd);
	sprintf(string,"%s/index_list2",temp_map_path);
	sprintf(string2,"%s/index_list",temp_map_path);
	if(move_file(string,string2))
	{	message("can't move new index_list file.  aborting.");
		lcpabort(1);
		return(1);
	}

		/*    open file        */
		/********* check memory & load new file or set pointer to it ****/
	sprintf(path,"%s/index_list",temp_map_path);
	index_list_fd = openf(path,RDWR);
	if(0 >= index_list_fd)
	{	message("  <<  cannot open index_list map file.");
		lcpabort(1);
		return(1);
	}
		/*  calculate size to malloc */
	size = index_listm.rec_size * job_stats[15];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(index_listm.malloc = (char *) malloc_ltp(size))
			{   index_listm.rec0 = index_listm.malloc;
				index_listm.recn = index_listm.rec0 + size;
				if(!readb(index_listm.rec0,size,1,index_list_fd))
				{	message("  <<  cannot read index_list map file.");
					lcpabort(1);
					return(0);
				}
			}
		}
	}


		/*********** re-index all v_objects ***************************/
	message("Indexing all remaining import levels.");
	if(index_vobj(0))
	{	lcpabort(1);
		return(1);
	}
	

		/*********** redisplay graphics *******************************/
	xi = (rwindowxmin + rwindowxmax) / 2;
	yi = (rwindowymin + rwindowymax) / 2;
	if(dispgraphics(xi,yi,resolution_code))
		return(1);


		/*********** return *******************************************/
	return(0);
}

long lcpabort(corrupt)
	long corrupt;
{
	if(vobj2fd > 0)
		closef(vobj2fd);

	if(vatr2fd > 0)
		closef(vatr2fd);

	if(vcdl2fd > 0)
		closef(vcdl2fd);

	if(new_index_list_fd > 0)
		closef(new_index_list_fd);

	if(corrupt)
	{	message("The level_clip function has aborted in a place that may leave the temporary");
		message("map file corrupted.  It is STRONGLY suggested that you acquire the map again ");
		message("WITHOUT saveing changes.");
		sleep(10);
	}

	return(0);
}


/*******************************************************************************
*    checkpair      This program checks a line segment between two verticies   *
*       to see if any part of it falls within the map frame.  If it does,      *
*       then vertex structures are set and a 'cross' parameter is set to 1.    *
*       If not, then 'cross' is set to 0.  If the first vertex is within       *
*       the map frame the 1 is returned.  If it is outside the frame then      *
*       0 is returned.  If there is an error, then -1 is returned.             *
*                                                                              *
*     Parameters:  double x; x value of first coordinate pair                  *
*                  double y; y value of first coordinate pair                  *
*                  double last_x; x value of second coordinate pair            *
*                  double last_y; y value of second coordinate pair            *
*                  struct *crdlst1; coordlist structure for return of 1st cd pr*
*                  struct *crdlst2; coordlist structure for return of 2nd cd pr*
*                  long *cross;  ptr to cross flag.                            *
*******************************************************************************/

long checkpair(x,y,last_x,last_y,crdlst1,crdlst2,cross)
double x,y,last_x,last_y;
COORD_LIST *crdlst1, *crdlst2;
long *cross;
{
	long code1,code2;
	long inside;

	double dx,dy,slope;


		/********* set crdlst1 & 2 defaults *******************************/
	crdlst1->x = x;
	crdlst1->y = y;
	crdlst2->x = last_x;
	crdlst2->y = last_y;

		/********* change coords to type 8 (1 unit box) *******************/
	if(transform(&x,&y,0,8))
		return(-1);
	if(transform(&last_x,&last_y,0,8))
		return(-1);

		/********* code coord pairs as to which sides they are inside *****/
	code1 = code_outside(x,y);
	code2 = code_outside(last_x,last_y);

		/********* if both points inside, set cross & return **************/
	if(!(code1 + code2))
	{	*cross = 1;
		return(1);
	}

		/********* set inside (return) value ******************************/
	if(code1)
		inside = 0;
	else
		inside = 1;

		/********* check for both points outside on the same side *********/
	if(code1 & code2)
	{	*cross = 0;
		return(0);
	}

		/********* clip on w & e sides ************************************/
	if(5 & (code1 | code2))
	{	if(code1 & 1)       /** point1 outside to the west **/
		{	dx = last_x - x;
			dy = last_y - y;
			slope = dy / dx;
			y = y + (0.0 - x) * slope;
			x = 0.0;
		}
		else if(code1 & 4)  /** point1 outside to the east **/
		{	dx = last_x - x;
			dy = last_y - y;
			slope = dy / dx;
			y = y + (1.0 - x) * slope;
			x = 1.0;
		}


		if(code2 & 1)       /** point2 outside to the west **/
		{	dx = last_x - x;
			dy = last_y - y;
			slope = dy / dx;
			last_y = last_y + (0.0 - last_x) * slope;
			last_x = 0.0;
		}
		else if(code2 & 4)  /** point2 outside to the east **/
		{	dx = last_x - x;
			dy = last_y - y;
			slope = dy / dx;
			last_y = last_y + (1.0 - last_x) * slope;
			last_x = 1.0;
		}

			/****** check again ********************************************/
		code1 = code_outside(x,y);
		code2 = code_outside(last_x,last_y);

		if(code1 & code2)   /*** both pts outside **/
		{	*cross = 0;
			return(0);
		}
	}

		/********* clip on n & south sides ********************************/
	if(10 & (code1 | code2))
	{	if(code1 & 2)       /** point1 outside to the north **/
		{	dx = last_x - x;
			dy = last_y - y;
			slope = dx / dy;
			x = x + (1.0 - y) * slope;
			y = 1.0;
		}
		else if(code1 & 8)  /** point1 outside to the south **/
		{	dx = last_x - x;
			dy = last_y - y;
			slope = dx / dy;
			x = x + (0.0 - y) * slope;
			y = 0.0;
		}


		if(code2 & 2)       /** point2 outside to the north **/
		{	dx = last_x - x;
			dy = last_y - y;
			slope = dx / dy;
			last_x = last_x + (1.0 - last_y) * slope;
			last_y = 1.0;
		}
		else if(code2 & 8)  /** point2 outside to the south **/
		{	dx = last_x - x;
			dy = last_y - y;
			slope = dx / dy;
			last_x = last_x + (0.0 - last_y) * slope;
			last_y = 0.0;
		}

			/****** check again ********************************************/
		code1 = code_outside(x,y);
		code2 = code_outside(last_x,last_y);

		if(code1 & code2)   /*** both pts outside **/
		{	*cross = 0;
			return(0);
		}
	}

		/********* change coords back to raster ***************************/
	*cross = 1;
	if(transform(&x,&y,8,0))
		return(-1);
	if(transform(&last_x,&last_y,8,0))
		return(-1);

		/********* set coordlist structure ********************************/
	crdlst1->x = x;
	crdlst1->y = y;
	crdlst2->x = last_x;
	crdlst2->y = last_y;

		/********* return *************************************************/
	return(inside);
}


/*******************************************************************************
*  code_outside     This program looks at an xy pair & returns a long value    *
*      with 4 bit flags bassed on its position as follows:                     *
*         if it is < x=0.0   then  bit0=1   (outside to the west)              *
*         if it is > y=1.0   then  bit1=1   (outside to the north)             *
*         if it is > x=1.0   then  bit2=1   (outside to the east)              *
*         if it is < y=0.0   then  bit3=1   (outside to the south)             *
*******************************************************************************/
long code_outside(x,y)
double x,y;
{
	long code;

	code = 0;
	if(x < 0.0)         /** outside to the west **/
		code |= 1;
	else if(x > 1.0)    /** outside to the east **/
		code |= 4;

	if(y > 1.0)         /** outside to the north **/
		code |= 2;
	else if(y < 0.0)    /** outside to the south **/
		code |= 8;

	return(code);
}
