/*******************************************************************************
*    level_dup     (see manual, Command Ref., User Functions)    <fc>          *
*            This program duplicates the data from an import level onto the    *
*       map raster and/or attributes.                                          *
*                                                                              *
*       files/globals changed:  all, if both are specified and the map is      *
*       assembled.                                                             *
*                                                                              *
*       values returned:  long 0= ok,  1=failure                               *
*                                                                              *
*       parameters:    long level  level number to be duplicated on the map    *
*                                  (32 - 95)                                   *
*                      long itc    information type code, specified type of    *
*                                  data to duplicate on the map:               *
*                                  1 = objects only                            *
*                                  2 = attributes only                         *
*                                  3 = both                                    *
*                                  4 = level pt attributes to map segments     *
*                                  5 = level pt attributes to map polygons     *
*                                                                              *
*            J.Dabritz   9/20/89                                               *
*******************************************************************************/

#include "../../incl/globals.h"

long level_dup(level,itc)
	long level;
	long itc;
{
	OBJECTS object;
	V_OBJECTS vobj;
	COORD_LIST vcoord;
	ATTRIBUTES atrbt;

	long i,j,v1,v2;
	long x1,y1,x2,y2;


		/************* check for map **********************************/
	if(!job_stats[0])
	{	message("You must CREATE or ACQUIRE a map befor level_dup.");
		return(1);
	}

		/************* check for VIEW ONLY raster **********************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}
		/************* check level parameter **************************/
	if((level < 32) || (level > 95))
	{	message("Illegal level number.  Aborting.");
		return(1);
	}

		/************* check itc parameter ****************************/
	if((itc <1) || (itc > 5))
	{	message("Illegal information type parameter.  Aborting.");
		return(1);
	}

		/************* check for attribute copy (4/5) *****************/
	if(itc > 3)
	{		/********** check all objects on level specified ********/
		for(i=0; i < job_stats[17]; i++)
		{	if(get_vobjects(&vobj,i))
				return(1);

				/********** if point object & has attribute *************/
			if(((vobj.type / 10) == level) && ((vobj.type % 10) == 0))
			{		/********** get location ********************************/
				if(get_v_coord(&vcoord,vobj.v1))
					return(1);
					/********** then assign attribute(s) ********************/
				atrbt.next = vobj.atr;
				for(; atrbt.next;)
				{	if(get_v_attr(&atrbt,atrbt.next))
						return(1);
					sprintf(atrbt_string,"%s",atrbt.string);
					if(atrbt_xy(vcoord.x,vcoord.y,(float) 2.5,itc - 3,6))
						break;
				}
			}
		}
		return(0);
	}
		/************* if objects specified ***************************/
	if(itc & 1)
	{		/************* duplicate objects onto raster **************/
		for(i=0; i < job_stats[17]; i++)
		{		/************* (search all v_objects, checking level & ****/
				/*************  drawing them onto the raster ) ************/
			if(get_vobjects(&vobj,i))
				return(1);
			if((vobj.type / 10) == level)
			{	if(vobj.v1 == vobj.v2)     /* node */
				{	if(get_v_coord(&vcoord,vobj.v1))
						return(1);
					x1 = vcoord.x + 0.5;
					y1 = vcoord.y + 0.5;
					pixel(x1,y1,1);
				}
				else			/** segment or polygon **/
				{	if(vobj.v1 < vobj.v2)
					{	v1 = vobj.v1;
						v2 = vobj.v2;
					}
					else                /** reverse order ***/
					{	v1 = vobj.v2;
						v2 = vobj.v1;
					}

					if(get_v_coord(&vcoord,v1))
						return(1);
					x1 = vcoord.x + 0.5;
					y1 = vcoord.y + 0.5;
					
					for(v1++; v1 <= v2; v1++)
					{	if(get_v_coord(&vcoord,v1))
							return(1);
						x2 = vcoord.x + 0.5;
						y2 = vcoord.y + 0.5;
						line_diag(x1,y1,x2,y2,1);

						x1 = x2;
						y1 = y2;
					}
				}
			}
			if(abort_cmd())
				return(1);
		}
	}

		/************* if attributes specified ************************/
	if(itc & 2)
	{
			/************* check for assembly & assemble if necess ****/
		if(itc == 3)
		{	if(assemble(1))
				return(1);
		}

			/************* read through map objects finding corrisp. **/
			/*************  level objects & assigning attributes ******/
		for(i=0; i<job_stats[10]; i++)
		{	if(get_objects(&object,i))
				return(1);

			  /********* find matching object on desired level ****/
			if(object.i12[0] < 8)           /* node */
				j = find_vnode(object.f4[0],object.f4[1],(float) 2.0,level);
			else if(object.i12[0] < 14)     /* segment */
				j = find_vseg(object.f4[0],object.f4[1],(float) 2.0,level);
			else if(object.i12[0] == 14)    /* interior polygon */
				j = find_vpoly(object.f4[0],object.f4[1],level);
			else
				j = -1;

				/******** copy attribute(s) **********************/
			if(j >= 0)
			{	if(get_vobjects(&vobj,j))
					return(1);
				atrbt.next = vobj.atr;
				for(; atrbt.next;)
				{	if(get_v_attr(&atrbt,atrbt.next))
						return(1);
					if(asgn_atrbt(i,6,atrbt.string))
						return(1);
				}
			}
		}
	}


		/************* return *****************************************/
	return(0);
}
