/*******************************************************************************
*    level_snap    (see manual, Command Ref., User Functions)    <fc>          *
*            This program snaps the objects in the specified set to the        *
*       objects in the specified level.                                        *
*                                                                              *
*       files/globals changed:  coord_list                                     *
*                                                                              *
*       values returned:  long 0= ok,  1=failure                               *
*                                                                              *
*       parameters:    long s      set number of object to snap (0=all)        *
*                      long level1 level number, lower bound of levels to      *
*                                  snap to (32 - 95)                           *
*                      long level2 level number, upper bound of levels to      *
*                                  snap to (32 - 95)                           *
*                      double r    snap radius, in units of code below         *
*                      long c      units code, same choices as in report cmd.  *
*                      long p      vertex preference code:  1=snap preferen-   *
*                                    1=snap to line or vertex                  *
*                                    2=snap preferentially to vertices.        *
*                                    3=snap preferentially to vertices & add   *
*                                      vertices for a perfect match.           *
*                                    4=snap only to a node, or segment endpoint*
*            J.Dabritz   10/16/89 modified 8/8/90                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long tail_search();

long level_snap(s,level1,level2,r,c,p)
   long s,level1,level2;
   double r;
   long c,p;
{
   OBJECTS object,poly,segment,node;
   COORD_LIST coordlist;

   char string[256];

   double dx,dy;
   double rr,rx,ry;
   double dist;
   double mat[2][3];
   double r2;

   float x,y;
   float last_x, last_y;

   long i,j,n,v,v2;
   long in_set;
   long seg;
   long v_begin, v_end;
   long ro,last_ro;
   long rv1,rv2;
   long last_rv1,last_rv2;
   long rl,rf;
   long rmax,rmin,last_rmax,last_rmin;
   long increment;
   long v2_begin,v2_end;


      /************* check for map loaded ************************/
   if(!job_stats[0])
   {  message("You must Create or Acquire a map befor executing the Level_snap command.");
      return(1);
   }

      /************* check set number ****************************/
   if((s < 0) || (s > 25))
   {  message("Illegal set number.  Aborting.");
      return(1);
   }

      /************* check snapping preference code **************/
   if((p < 1) || (p > 4))
   {  message("Illegal snapping preference code.  Aborting.");
      return(1);
   }

      /************* check level number **************************/
   if(level1 > level2)
   {  i = level1;
      level1 = level2;
      level2 = i;
   }
   if((level1 < 32) || (level1 > 95)  ||  (level2 < 32) || (level2 > 95))
   {  message("Illegal level number.  Aborting.");
      return(1);
   }


      /************* check radius & units code *******************/
   if(r < (double) 0.0)
   {  message("Snap radius must be greater than zero.  Sorry Charlie.");
      return(1);
   }
   if((c < 0) || (c > 6))
   {  message("Illegal units code.  Aborting.");
      return(1);
   }
   get_units();
   {  if(unit_factor[c][0] == (double) 0.0)
      {  message("You must REGISTER the GEODETICS and RASTER befor choosing non-pixel units.");
         return(1);
      }
   }
   r = r / unit_factor[c][0];    /** find radius in pixels **/
   r2 = r/2.0;


   switch(p) {
   case 1:         /** snap to line or vertex, whichever is closer */
   case 2:         /** snap preferentially to a vertex, even if line closer */
	case 4:         /** snap only to node or end of segment **/
         /************* for each object in the map ******************/
      for(i=0; i<job_stats[10]; i++)
      {     /************* inform user *****************************/
         if(!(i % 25))
         {  sprintf(string,"Level_snap:  checking verticies on object %d.",i);
            message(string);
         }
   
            /************* see if object is in set *****************/
         if(get_objects(&object,i))
         {  message("level_snap, abort, location 1.");
            return(1);
         }
   
         in_set = 0;                      /** default **/
         if(object.i12[0] < 8)            /** node **/
         {  if(!s  ||  object.i12[4] & (1 << s))
               in_set = 1;
         }
         else if(object.i12[0] < 14)      /** segment **/
         {  if(!s  ||  object.i12[4] & (1 << s))
               in_set = 1;
            else
            {  for(j=7; j<9; j++)
               {  if(object.i12[j])
                  {  if(get_objects(&poly,object.i12[j]))
                     {  message("level_snap, abort, location 2.");
                        return(1);
                     }
                     if(!s  ||  poly.i12[4] & (1 << s))
                        in_set = 1;
                  }
               }
            }
         }         /** polygons checked as part of segments **/
         else
            break;          /** first poly is end of segments, done! */
   
         if(in_set)
         {     /************* switch by object type ***************/
            if(object.i12[0] < 8)       /** node **/
            {  /************* NODES: check position ***************/
               x = object.f4[0];
               y = object.f4[1];
               if(snap_vertex(&x,&y,level1,level2,r,p-1) == 1)
               {     /************* if hit, move node *******************/
                  object.f4[0] = x;
                  object.f4[1] = y;
                  if(put_objects(&object,i))
                  {  message("level_snap, abort, location 3.");
                     return(1);
                  }
                     /************* then    move branch endpoints *******/
                  for(j=6; j<10; j++)
                  {  seg = object.i12[j];
                     if(seg)
                     {  seg = (seg < 0) ?  -seg: seg;
                        if(get_objects(&segment,seg))
                        {  message("level_snap, abort, location 4.");
                           return(1);
                        }
                        if(object.i12[j] > 0)
                           v = segment.i12[5];
                        else
                           v = segment.i12[6];
                        coordlist.x = x;
                        coordlist.y = y;
                        if(put_coordlst(&coordlist,v))
                        {  message("level_snap, abort, location 5.");
                           return(1);
                        }
                     }
                     else
                        break;
                  }
               }
            }
            else
            {  /************* SEGMENTS: check all verticies *******/
               v_end = object.i12[6];
               for(v = object.i12[5];  v <= v_end; v++)
               {  if(get_coordlst(&coordlist,v))
                  {  message("level_snap, abort, location 6.");
                     return(1);
                  }
   
                  x = coordlist.x; 
                  y = coordlist.y; 
                  if(snap_vertex(&x,&y,level1,level2,r,p-1) == 1)
                  {     /********* move vertex ******************/
                     coordlist.x = x;
                     coordlist.y = y;
                     if(put_coordlst(&coordlist,v))
                     {  message("level_snap, abort, location 7.");
                        return(1);
                     }
                        /****** if end vertex, move node ********/
                     if(v == object.i12[5])   /*** beginning vertex **/
                        n = object.i12[9];
                     else if(v == object.i12[6])  /** ending vertex **/
                        n = object.i12[10];
                     else                       /** middle vertex **/
                        n = -1;
   
                     if(n >= 0)
                     {  if(get_objects(&node,n))
                        {  message("level_snap, abort, location 8.");
                           return(1);
                        }
                           /** move node *******************/
                        node.f4[0] = x;
                        node.f4[1] = y;
                        if(put_objects(&node,n))
                        {  message("level_snap, abort, location 9.");
                           return(1);
                        }
                           /***** then move branch endpoints *******/
                        for(j=6; j<10; j++)
                        {  seg = node.i12[j];
                           if(seg)
                           {  seg = (seg < 0) ?  -seg: seg;
                              if(seg != i)
                              {  if(get_objects(&segment,seg))
                                 {  message("level_snap, abort, location 10.");
                                    return(1);
                                 }
                                 if(node.i12[j] > 0)
                                    v2 = segment.i12[5];
                                 else
                                    v2 = segment.i12[6];
                                 coordlist.x = x;
                                 coordlist.y = y;
                                 if(put_coordlst(&coordlist,v2))
                                 {  message("level_snap, abort, location 11.");
                                    return(1);
                                 }
                              }
                           }
                           else
                              break;
                        }
                     }
                  }
               }
            }
         }
      }
      break;
   case 3:    /** snap to vertex or line, then add vertices for perf. match */
         /************* for each object in the map ******************/
      for(i=0; i<job_stats[10]; i++)
      {     /************* inform user *****************************/
         if(!(i % 25))
         {  sprintf(string,"Level_snap:  checking verticies on object %d.",i);
            message(string);
         }
   
            /************* see if object is in set *****************/
         if(get_objects(&object,i))
         {  message("level_snap, abort, location 12.");
            return(1);
         }
   
         in_set = 0;                      /** default **/
         if(object.i12[0] < 8)            /** node **/
         {  if(!s  ||  object.i12[4] & (1 << s))
               in_set = 1;
         }
         else if(object.i12[0] < 14)      /** segment **/
         {  if(!s  ||  object.i12[4] & (1 << s))
               in_set = 1;
            else
            {  for(j=7; j<9; j++)
               {  if(object.i12[j])
                  {  if(get_objects(&poly,object.i12[j]))
                     {  message("level_snap, abort, location 13.");
                        return(1);
                     }
                     if(!s  ||  poly.i12[4] & (1 << s))
                        in_set = 1;
                  }
               }
            }
         }         /** polygons checked as part of segments **/
         else
            break;          /** first poly is end of segments, done! */
   
         if(in_set)
         {     /************* switch by object type ***************/
            if(object.i12[0] < 8)       /** node **/
            {  /************* NODES: check position ***************/
               x = object.f4[0];
               y = object.f4[1];
               if(snap_vertex(&x,&y,level1,level2,r,1) == 1)
               {     /************* if hit, move node *******************/
                  object.f4[0] = x;
                  object.f4[1] = y;
                  if(put_objects(&object,i))
                  {  message("level_snap, abort, location 14.");
                     return(1);
                  }
                     /************* then    move branch endpoints *******/
                  for(j=6; j<10; j++)
                  {  seg = object.i12[j];
                     if(seg)
                     {  seg = (seg < 0) ?  -seg: seg;
                        if(get_objects(&segment,seg))
                        {  message("level_snap, abort, location 15.");
                           return(1);
                        }
                        if(object.i12[j] > 0)
                           v = segment.i12[5];
                        else
                           v = segment.i12[6];
                        coordlist.x = x;
                        coordlist.y = y;
                        if(put_coordlst(&coordlist,v))
                        {  message("level_snap, abort, location 16.");
                           return(1);
                        }
                     }
                     else
                        break;
                  }
               }
            }
            else
            {  /************* SEGMENTS: check all verticies *******/
               /*** & rewrite verticies to end of vertex_list **/
               v_end = object.i12[6];
               v_begin = object.i12[5];
               object.i12[5] = job_stats[11];
               last_ro = -1;
               for(v = v_begin;  v <= v_end; v++)
               {  if(get_coordlst(&coordlist,v))
                  {  message("level_snap, abort, location 17.");
                     return(1);
                  }
   
                  x = coordlist.x; 
                  y = coordlist.y; 
                  if(!find_vertex((double) x,(double) y,r,level1,level2,1,
                                  &rr,&rx,&ry,&ro,&rv1,&rv2,&rl,&rf))
                  {
                        /*** check snap vertex range ************/
                     if(rv1 < rv2)
                     {  rmax = rv2;
                        rmin = rv1;
                     }
                     else
                     {  rmax = rv1;
                        rmin = rv2;
                     }

                        /*** check if last vertex was hit by same obj **/
                     if(ro == last_ro)
                     {     /*** set loop variables **/
                        if((rmin < last_rmin) || (rmax < last_rmax))
                        {  increment = -1;
                           v2_begin = last_rmax - 1;
                           v2_end = rmin;
                        }
                        else if((rmin > last_rmin) || (rmax > last_rmax))
                        {  increment = 1;
                           v2_begin = last_rmin + 1;
                           v2_end = rmax;
                        }
                        else
                           increment = 0;


                           /*** check all snap level verticies in *********/
                           /*** vertex range & copy if within r ***********/
                           /*** of map obj being checked ******************/
                           /* 1. find vector x,y - last_x,y */
                        dx = x - last_x;
                        dy = y - last_y;
                           /* 2. normalize vector           */
                        dist = sqrt(dx*dx + dy*dy);
                        if(dist > (double) 0.0)
                        {  dx = dx / dist;
                           dy = dy / dist;
                        }
                        
                        if(dist > (double) 0.0)
                        {     /* set up matrix to find distance */
                           if(increment)
                           {  for(v2=v2_begin; v2 != v2_end; v2 += increment)
                              {     /*** read level object vertex **/
                                 if(get_v_coord(&coordlist,v2))
                                 {  message("level_snap, abort, location 18.");
                                    return(1);
                                 }
                                    /*** find dist to map object segment ***/
                                    /* set up matrix to find distance */
                                 mat[0][0] = dx;
                                 mat[0][1] = dy;
                                 mat[0][2] = coordlist.x - last_x;
                                 mat[1][0] = dy;
                                 mat[1][1] = -dx;
                                 mat[1][2] = coordlist.y - last_y;
                                 if(!mat_solve(2,3,mat[0]))
                                 {     /*** copy if within snapping dist ***/
                                    if((mat[0][2] >= -r2) &&
                                       (mat[0][2] <= dist + r2) &&
                                       (mat[1][2] <= r) && (mat[1][2] >= -r))
                                    {  if(put_coordlst(&coordlist,job_stats[11]))
                                       {  message("level_snap, abort, location 19.");
                                          return(1);
                                       }
                                    }
                                 }
                              }
                           }
                        }
                     }
                     else  /** snapping to different vector objects **/
                     {     /** look for snapping to tails of vector objects **/
									/** if not first vertex of object **/
									/** if a vtx has been snapped  **/
								if(v != v_begin)
								{
                           	/****** look for tail ahead of last vertex **/
                        	if((last_ro >= 0) && (v > (v_begin + 1)))
                           	tail_search((double) r,&last_x,&last_y,x,y,last_ro,last_rv1, last_rv2,1);
	
                           	/****** look for tail back of new vertex ****/
									if(v != v_end)
                        		tail_search((double) r,&last_x,&last_y,x,y,ro,rv1,rv2,-1);
								}

                     }
                        /*** reset last_ro,rmin,rmax **/
                     last_ro = ro;  /** vector obj snapped to **/
                     last_rmin = rmin;
                     last_rmax = rmax;
                        /********* store vertex ******************/
                     coordlist.x = rx;
                     coordlist.y = ry;
                     if(put_coordlst(&coordlist,job_stats[11]))
                     {  message("level_snap, abort, location 20.");
                        return(1);
                     }
                        /****** if end vertex, move node ********/
                     if(v == v_begin)   /*** beginning vertex **/
                        n = object.i12[9];
                     else if(v == v_end)  /** ending vertex **/
                        n = object.i12[10];
                     else                       /** middle vertex **/
                        n = -1;
   
                     if(n >= 0)
                     {  if(get_objects(&node,n))
                        {  message("level_snap, abort, location 21.");
                           return(1);
                        }
                           /** move node *******************/
                        node.f4[0] = rx;
                        node.f4[1] = ry;
                        if(put_objects(&node,n))
                        {  message("level_snap, abort, location 22.");
                           return(1);
                        }
                           /***** then move branch endpoints *******/
                        for(j=6; j<10; j++)
                        {  seg = node.i12[j];
                           if(seg)
                           {  seg = (seg < 0) ?  -seg: seg;
                              if(seg != i)
                              {  if(get_objects(&segment,seg))
                                 {  message("level_snap, abort, location 23.");
                                    return(1);
                                 }
                                 if(node.i12[j] > 0)
                                    v2 = segment.i12[5];
                                 else
                                    v2 = segment.i12[6];
                                 coordlist.x = rx;
                                 coordlist.y = ry;
                                 if(put_coordlst(&coordlist,v2))
                                 {  message("level_snap, abort, location 24.");
                                    return(1);
                                 }
                              }
                           }
                           else
                              break;
                        }
                     }
                        /*** copy snapped vertex **/
                     coordlist.x = rx;
                     coordlist.y = ry;
                     if(put_coordlst(&coordlist,job_stats[11]))
                     {  message("level_snap, abort, location 25.");
                        return(1);
                     }
                        /*** set last_ values ***/
                     last_rv1 = rv1;
                     last_rv2 = rv2;
                  }
                  else
                  {
                          /* if last vertex snapped  & is greater than v1 */
                     if((last_ro >= 0) && (v > (v_begin + 1)))
                     {     /****** look for tail ahead of last vertex **/
                        tail_search(r,&last_x,&last_y,x,y,last_ro,last_rv1,
                                    last_rv2,1);
                     }
                     last_ro = -1;  /** not snapped **/
                        /** copy map object vertex **/
                     if(put_coordlst(&coordlist,job_stats[11]))
                     {  message("level_snap, abort, location 26.");
                        return(1);
                     }
                  }
                     /*** reset last coords **********/
                  last_x = x;
                  last_y = y;
               }
                  /*** set end coord_list record in object record **/
               object.i12[6] = job_stats[11] - 1;
                  /*** re-store object record **/
               if(put_objects(&object,i))
               {  message("level_snap, abort, location 27.");
                  return(1);
               }
            }
         }
      }
      break;
   }

      /**** pack coord_list file ****/
   if(vertex_pack())
   {  message("level_snap, abort, location 28.");
      return(1);
   }
      
      /************* recalculate lengths & areas *****************/
   recalc_area();

      /************* recalculate line centroids ******************/
      /* start here , not currently done. */
      /************* recalculate polygon centroids ***************/
      /* start here , not currently done. */


      /************* re-index map objects ************************/
   if(index_map())
   {  message("level_snap, abort, location 29.");
      return(1);
   }

      /************* re-set max snap dist in job_stats file ******/
	rr = 100.0 * job_stats[77];
	if(rr < r)
	{	rr = 100 * r + 0.5;
		job_stats[77] = rr;
	}
	
      /*** redisplay the map ****/
   if((disp_hglight || disp_normal))
   {     /************* call dispgraphics to re-display data *********/
      i = (rwindowxmin + rwindowxmax) / 2;
      j = (rwindowymin + rwindowymax) / 2;
      if(dispgraphics(i,j,resolution_code))
         return(1);
   }

      /************* return **************************************/
   return(0);
}


/*******************************************************************************
*     tail_search                                                              *
*           This program is called when the same map objects snaps to a new    *
*   to a new vector object.  It looks on the last vector object (if any) for   *
*   an unduplicated tail, or on the beginning of the new vector object for     *
*   an unduplicated tail.  If found, such tial verticies are copied onto the   *
*   end of the current coord_list file.   The last_x, last_y are reset         *
*   if any verticies are written.                                              *
*                                                                              *
*   Parameters:  double r;              search radius limit, tolerance         *
*                float *last_x, *last_y;  coordinates of older map vertex      *
*                float new_x,new_y;     coordinates of newer map vertex        *
*                long vobj;             vector object number                   *
*                long v1,v2;            v_coordlist rec nos of bounding vtcies *
*                long dir;              1 = look out from older vertex,        *
*                                       -1 = look toward newer  vertex         *
*   Value returned:  long 0 = ok,  1=error                                     *
*     J.Dabritz  9/22/90                                                       *
*******************************************************************************/

long tail_search(r,last_x,last_y,new_x,new_y,vobj,v1,v2,dir)
double r;
float *last_x,*last_y;
float new_x,new_y;
long vobj;
long v1,v2;
long dir;
{
   COORD_LIST coordlist;
   V_OBJECTS v_object;

   double mat[2][3];
   double x,y;
   double x1,x2,y1,y2;
   double dx,dy,dx1,dy1,dx2,dy2;
   double length, dist;
   double a,b;

   long begin, end, increment;
   long v;


   
      /*********** check v1 & v2 for different verticies **************/
   if((v1 < 0) || (v2 < 0) || (v1 == v2))
      return(0);

      /*********** get vector object **********************************/
   if(get_vobjects(&v_object,vobj))
      return(1);

      /*********** check for node or only 1 vertex ********************/
   if((v_object.type % 3) == 0)      /** node **/
      return(0);
   if(v_object.v1 == v_object.v2)    /** only 1 vertex **/
      return(0);

      /*********** check new & old x,y for same point *****************/
   if((new_x == *last_x) && (new_y == *last_y))
      return(0);

      /*********** sort v1 & v2 ***************************************/
   if(v1 > v2)
   {   v = v1;
      v1 = v2;
      v2 = v;
   }

      /*********** get vertex1 & 2 values *****************************/
   if(get_v_coord(&coordlist,v1))
      return(1);
   x1 = coordlist.x;
   y1 = coordlist.y;

   if(get_v_coord(&coordlist,v2))
      return(1);
   x2 = coordlist.x;
   y2 = coordlist.y;
   
      /*********** find direction of vobj on map object ***************/
   dx1 = new_x - *last_x;  /*** map obj vector **/
   dy1 = new_y - *last_y;
   dx2 = x2 - x1;         /*** import object vector ***/
   dy2 = y2 - y1;
   
   mat[0][0] = dx1;   /** set up 2 X 3 matrix for solution **/
   mat[0][1] = dy1;
   mat[0][2] = dx2;
   mat[1][0] = dy1;
   mat[1][1] = -dx1;
   mat[1][2] = dy2;

   if(mat_solve(2,3,mat[0]))   /** no solution, return **/
      return(0);
   
      /*********** find verticies & direction to search ***************/
   if(mat[0][2] == 0.0)   /**** vectors are perpendicular, abort search **/
      return(0);
   if(mat[0][2] > 0)      /**** vectors in same general direction **/
   {   increment = 1;
      if(v_object.v1 < v_object.v2)    /** vector object in increasing order **/
      {   if(dir > 0)      /**** look ahead from last vertex *****/
         {   begin = v2;
            end = v_object.v2;
         }
         else             /**** look toward new vertex *****/
         {   begin = v_object.v1;
            end = v1;
         }
      }
      else                /** vector object in decreasing order **/
      {   if(dir > 0)      /**** look ahead from last vertex *****/
         {   begin = v2;
            end = v_object.v1;
         }
         else             /**** look toward new vertex *****/
         {   begin = v_object.v2;
            end = v1;
         }
      }
   }
   else                   /**** vectors on opposite general direction **/
   {   increment = -1;
      if(v_object.v1 < v_object.v2)    /** vector object in increasing order **/
      {   if(dir > 0)      /**** look ahead from last vertex *****/
         {   begin = v1;
            end = v_object.v1;
         }
         else             /**** look toward new vertex *****/
         {   begin = v_object.v2;
            end = v2;
         }
      }
      else                /** vector object in decreasing order **/
      {   if(dir > 0)      /**** look ahead from last vertex *****/
         {   begin = v1;
            end = v_object.v2;
         }
         else             /**** look toward new vertex *****/
         {   begin = v_object.v1;
            end = v2;
         }
      }
   }

      /*********** search verticies ***********************************/
   for(v=begin; ; v += increment)
   {   /*********** set some stable matrix variables *******************/
   	dx = new_x - *last_x;
   	dy = new_y - *last_y;
   	length = sqrt(dx*dx + dy*dy);      /** get vector length **/
   	if(length == 0.0)
      	return(0);
   	dx = dx / length;                  /** normalize vector **/
   	dy = dy / length;

         /**** get vertex coords ****/
      if(get_v_coord(&coordlist,v))
         return(1);
      x = coordlist.x;
      y = coordlist.y;

         /**** set matrix values for finding distance ***/
      mat[0][0] = dx;
      mat[0][1] = dy;
      mat[0][2] = x - *last_x;
      mat[1][0] = dy;
      mat[1][1] = -dx;
      mat[1][2] = y - *last_y;

         /**** find distance to line segment & add vertex if within r ***/
      if(!mat_solve(2,3,mat[0]))
      {   a = mat[0][2];
         b = mat[1][2];
         if((a >= 0.0) && (a <= length))
            dist = fabs(b);
         else if(a < 0.0)
         {   dx1 = x - *last_x;
            dy1 = y - *last_y;
            dist = sqrt(dx1*dx1 + dy1*dy1);
         }
         else      /** (a > length) **/
         {   dx1 = x - new_x;
            dy1 = y - new_y;
            dist = sqrt(dx1*dx1 + dy1*dy1);
         }

         if(dist <= r)     /***** within snap tolerance, insert vertex *****/
         {  if(put_coordlst(&coordlist,job_stats[11]))
            {  message("level_snap, abort, location 79.");
               return(1);
            }
				*last_x = coordlist.x;
				*last_y = coordlist.y;
         }
      }
         /**** check for end of search *****/
      if(v == end)
         break;
   }

      /*********** return *********************************************/
   return(0);
}
