/*******************************************************************************
*	log_ltp (see manual, Command Ref., User Function)	<fc>                   *
*			Logs (stores locations in the errors file) all                     *
*			locations (of objects/occurrences) of the specified                *
*			error type.  Also used to clear the errors file.                   *
*                                                                              *
*        ltype codes:    0-31 = log set indicated                              *
*                          32 =  clear error log                               *
*                          33 =  log points                                    *
*                          34 =  log junctions                                 *
*                          36 =  log holes                                     *
*                          37 =  log spurs                                     *
*                                                                              *
*           Changes the map 'errors' file.  Sets globals job_stats[14] and     *
*           job_stats[75].                                                     *
*                                                                              *
*           Returns 0 on successful completion, 1 on failure.                  *
*                                                                              *
*           J.Dabritz    8/19/89                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long log_ltp(ltype)    
long ltype;
{	
	ERRORS error;
	OBJECTS object;

	unsigned long *ptr, *end_ptr;

	long bt, word, x, y;
	long count;
	long key;
	long i;
	long mask9;
	static long bit[] = {7, 6, 5, 4, 3, 2, 1, 0, 15, 14, 13, 12, 11, 10, 
		9, 8, 23, 22, 21, 20, 19, 18, 17, 16, 31, 30, 29, 28, 27,
		26, 25, 24};

	char string[STRLIST];


		/************ check for map **********************************/
	if(!job_stats[0])
	{	message("cannot log errors until a map is acquired.");
		return(1);
	}

		/************ check parameter & log errors *******************/
	error.type = ltype;
	count = 0;

	switch(ltype)   {
	case 32:                /*** clear ***/
		job_stats[14] = 0;
		job_stats[75] = 0;
		message("error file has been cleared . . . ");
		break;
	case 36:               /*** holes ***/
		message("logging holes . . ");
			/**************** search raster for holes & log 'em  ************/
		count = 0;
		end_ptr = lraster[job_stats[2]];
		for(ptr = lraster[1]; ptr < end_ptr; ptr++)
		{	if(*ptr)
			{	for(bt = 0; bt < 32; bt++)
				{	switch(bt) {
					case 7:                      /*  left most bt   */
						if((*ptr & 64) && !(*ptr & 128) && (*(ptr-1) & (1<<24)))
						{	if(85 == (85 & locale_ptr(ptr,bt)))
							{	word = (ptr - lraster[0]) % raster_wincrement;
								error.x = (word << 5) | (7 & ~bt) | (bt & 24);
								error.y= (ptr - lraster[0]) / raster_wincrement;
								if(put_errors(&error,job_stats[14]))
									return(1);
								count++;
							}
						}
						break;
					case 24:                     /*  right most bt  */
						if((*ptr & (1<<25)) && !(*ptr & (1<<24)) && 
							(*(ptr+1) & 128) )
						{	if(85 == (85 & locale_ptr(ptr,bt)))
							{	word = (ptr - lraster[0]) % raster_wincrement;
								error.x = (word << 5) | (7 & ~bt) | (bt & 24);
								error.y= (ptr - lraster[0]) / raster_wincrement;
								if(put_errors(&error,job_stats[14]))
									return(1);
								count++;
							}
						}
						break;
					default:
						switch(bt % 8)  {
						case 7:
							if((*ptr & (1<<(bt-15))) && !(*ptr & (1<<bt)) &&
								(*ptr & (1<<(bt-1))) )
							{	if(85 == (85 & locale_ptr(ptr,bt)))
								{	word = (ptr - lraster[0]) % raster_wincrement;
									error.x = (word << 5) | (7 & ~bt) | (bt & 24);
									error.y = (ptr - lraster[0]) / raster_wincrement;
									if(put_errors(&error,job_stats[14]))
										return(1);
									count++;
								}
							}
							break;
						case 0:
							if((*ptr & (1<<(bt+1))) && !(*ptr & (1<<bt)) &&
								(*ptr & (1<<(bt+15))) )
							{	if(85 == (85 & locale_ptr(ptr,bt)))
								{	word = (ptr - lraster[0]) % raster_wincrement;
									error.x = (word << 5) | (7 & ~bt) | (bt & 24);
									error.y = (ptr - lraster[0]) / raster_wincrement;
									if(put_errors(&error,job_stats[14]))
										return(1);
									count++;
								}
							}
							break;
						default:
							if((*ptr & (1<<(bt+1))) && !(*ptr & (1<<bt)) &&
								(*ptr & (1<<(bt-1))) )
							{	if(85 == (85 & locale_ptr(ptr,bt)))
								{	word = (ptr - lraster[0]) % raster_wincrement;
									error.x = (word << 5) | (7 & ~bt) | (bt & 24);
									error.y = (ptr - lraster[0]) / raster_wincrement;
									if(put_errors(&error,job_stats[14]))
										return(1);
									count++;
								}
							}
							break;
						}
					}
				}
			}
		}
		break;
	case 37:               /*** spurs ***/
		message("logging spur endpoints . . . ");
		end_ptr = lraster[job_stats[2]] + raster_wincrement;
		mask9 = 1 << 9;
			/***	Find spurs in raster  ***/
		for( ptr = lraster[0]; ptr < end_ptr; ptr++ )
		{   if( *ptr )
	    	{   for( i = 0; i < 32; i++ )
	    	    {	if(*ptr & (1 << bit[i]) )
		    		{   key = tlocale[locale_ptr(ptr, bit[i])];
						if( key & mask9 )	/*** True if spur end ***/
						{   word = (ptr - lraster[0]) % raster_wincrement;
			    			error.x = (word << 5) | i;
			    			error.y = (ptr - lraster[0]) / raster_wincrement;
							if(put_errors(&error,job_stats[14]))
								return(1);
							count++;
						}
			    	}
		    	}
	    	}
		}
		break;
	case 33:               /*** points ***/
		message("logging points . . . ");
			/**************** search raster for isolated pixels & destroy ***/
		count = 0;
		end_ptr = lraster[job_stats[2]] + raster_wincrement;
		for(ptr = lraster[0]; ptr < end_ptr; ptr++)
		{	if(*ptr)
			{	for(bt = 0; bt < 32; bt++)
				{	if(*ptr & (1<<bt))
					{	if(!locale_ptr(ptr,bt))
						{	word = (ptr - lraster[0]) % raster_wincrement;
							error.x = (word << 5) | (7 & ~bt) | (bt & 24);
							error.y = (ptr - lraster[0]) / raster_wincrement;
							if(put_errors(&error,job_stats[14]))
								return(1);
							count++;
						}
					}
				}
			}
		}
		break;
	case 34:               /*** junctions ***/
		message("logging junctions . . . ");
		end_ptr = lraster[job_stats[2]] + raster_wincrement;
		mask9 = 3 << 11;             /** a 3 or 4 branch node **/
			/***	Find junctions in raster  ***/
		for( ptr = lraster[0]; ptr < end_ptr; ptr++ )
		{   if( *ptr )
	    	{   for( i = 0; i < 32; i++ )
	    	    {	if(*ptr & (1 << bit[i]) )
		    		{   key = tlocale[locale_ptr(ptr, bit[i])];
						if( key & mask9 )	/*** True if a junction ***/
						{   word = (ptr - lraster[0]) % raster_wincrement;
			    			error.x = (word << 5) | i;
			    			error.y = (ptr - lraster[0]) / raster_wincrement;
							if(put_errors(&error,job_stats[14]))
								return(1);
							count++;
						}
			    	}
		    	}
	    	}
		}
		break;
	default:               /*** log a set ***/
		if((ltype >= 0)  &&  (ltype <= 31))
		{	if(!job_stats[10])
			{	message("you must assemble the map befor there are any objects to make sets of.");
				return(1);
			}
			sprintf(string,"logging set %d in errors file . . . ",ltype);
			message(string);
			for(i=0; i<job_stats[10]; i++)
			{	if(get_objects(&object,i))
					return(1);
				if(object.i12[4] & (1 << ltype))
				{	error.x = object.f4[0];
					error.y = object.f4[1];
					if(put_errors(&error,job_stats[14]))
						return(1);
					count++;
				}
			}
		}
		else
		{	message("illegal log-errors type code.  No action taken.");
			return(1);
		}
	}
		/************ inform user ************************************/
	sprintf(string,"%d new errors logged.  %d total errors",count,job_stats[14]);
	append_msg(string);

		/************ return *****************************************/
	return(0);
}
