/*******************************************************************************
*	macro_hist	(see manual, Command Ref., User Functions)	<fc>   *
*			Description:  Constructs a macro file from the history *
*			file.                                                  *
*			Parameters:  New macro name, beginning/ending records  *
*				     of history file extract and descriptor.   *
*									       *
*			Returns zero upon normal end-of-job (EOJ).             *
*******************************************************************************/
/* Modified 5/92 by DBS@CWU for Multiuser support */

#include "../../incl/globals.h"
#include "../../incl/in.h"

long macro_hist( mac_name, begin_rec, end_rec, descript )
	char *mac_name;
	long begin_rec;
	long end_rec;
	char *descript;
{
	char fname[256];
	char string[256];

	long n,i,j;


	/************* check for map loaded ******************************/
	if(!job_stats[0])
	{	message("You must Create or Acquire a map befor using the macro_hist command.");
		return(1);
	}

	/************* check beginning and ending records ****************/
	if(begin_rec > end_rec)
	{	n = begin_rec;
		begin_rec = end_rec;
		end_rec = n;
	}
	if((begin_rec < 0)  ||  (end_rec > 511))
	{	message("Beginning and ending record numbers must be in range 0 - 511.");
		return(1);
	}

	/************* create file ***************************************/
#ifdef MULTIUSER
	sprintf(fname,"%s/.ltp/macros/%s",home_path,mac_name);
#else
	sprintf(fname,"%s/%s",macros_path,mac_name);
#endif
	if(1 > creatf(fname))
	{	message("Cannot create macro file, aborting command.");
		return(1);
	}

	/************* open file *****************************************/
	macro_file = openf(fname,WRITE);
	if(macro_file <= 0)
	{	purgef(fname);
		return(1);
	}

	/************* store description as a comment ********************/
	sprintf(string,"#  %s\n",descript);
	writea(string, macro_file);


	/************* store a blank line ********************************/
	writea("\n",macro_file);

	/************* store macro lines *********************************/
	n = end_rec - begin_rec + 1;
	for(i=(next_hist + begin_rec) % 512,j=0; j<n; j++, i = (i + 1) % 512)
	{	sprintf(string,"%s\n",history[i]);
		if(string[STRLIST - 2])
		{	string[STRLIST - 2] = (char) 10;
			string[STRLIST - 1] = (char) 0;
		}
		writea(string, macro_file);
	}
	
	/************* close file ****************************************/
	closef(macro_file);
	macro_file = -1;

	/************* inform user ***************************************/
	sprintf(string,"macro  %s  has been stored.",mac_name);
	message(string);

	/************* return ********************************************/
	return(0);
}
