/*******************************************************************************
*	margin		(see manual, Command Ref., User Function)	<fc>   *
*			Forces the raster margin to specified minimum width.   *
*			This may be more or less than the current margin.      *
*                                                                              *
*			Changes the job_stats, Raster, and mini-raster files   *
*			and associated common variables.  If the map has been  *
*			assembled, then all files which hold coordinate data   *
*			are updated to the new coordinate system.              *
*                                                                              *
*			Note:  This function will update all vectorized data   *
*			coordinate to match the new registration.              *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

long margin(wd)
	long wd;  /* width (in pixels) to make the margin (envelope 
			outside the raster registered map corners).      */
{
	char new_file[256];
	char raster2[256];
	char path[256];
	char string[256];
	char *mallocptr;

	int  fd2, filefd;

	long i,j,h,w,y;
	long *mrstr_row;
	long width;                  /* width of 1 raster row (pixels) */
	long size;                   /* bytes in 1 raster row */
	long rows;                   /* number of raster rows */
	long first_word,last_word;
	long xmin,xmax,ymin,ymax;
	long offset;
	long raster_size;
	long create_braster;

		/************* check for map present ***************************/
	if(!job_stats[0])
	{	message("You must ACQUIRE or CREATE a map befor setting the margin.");
		return(1);
	}

		/************* check for VIEW ONLY *****************************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}

		/************* check for raster registered *********************/
	if(!registered(0))
	{	message("You must REGISTER the raster befor setting the margin width.");
		return(1);
	}

		/************* check for braster *******************************/
	if(israsterb())
		create_braster = 1;
	else
		create_braster = 0;

		/************* find new raster limits **************************/
	xmin = xmax = corner[0][0][0];
	ymin = ymax = corner[0][0][1];
	for(i=1; i<4; i++)
	{	if(corner[0][i][0] < xmin)
			xmin = corner[0][i][0];
		if(corner[0][i][0] > xmax)
			xmax = corner[0][i][0];
		if(corner[0][i][1] < ymin)
			ymin = corner[0][i][1];
		if(corner[0][i][1] > ymax)
			ymax = corner[0][i][1];
	}

	ymin -= wd;
	ymax += wd;
	xmin -= wd;
	xmax += wd;
	rows = ymax - ymin + 1;

		/************** find words to create & store for each row *****/
	first_word = xmin >> 5;
	last_word = xmax >> 5;

	if(first_word < 0)
	{	offset = -first_word;
		first_word = 0;
	}
	else
		offset = 0;

	if(last_word > (job_stats[1] >> 5))
		last_word = (job_stats[1] >> 5);


		/****** make width an integral multiple of 32 **/
	width = ((xmax >> 5) - (xmin >> 5) + 1) * 32;

		/************* create new raster file & asgn pointer ***********/
	sprintf(raster2,"%s/raster2",temp_map_path);
	purgef(raster2);
	if(-1 == creatf(raster2))
	{	message("Cannot create new raster file. aborting.");
		return(1);
	}
	fd2 = openf(raster2,WRITE);
	if(fd2 < 0)
	{	message("Cannot open new raster file. aborting.");
		return(1);
	}
	


		/************* malloc enough space for a raster row ************/
	size = width / 8;
	mallocptr = calloc_ltp(size,1);
	if(mallocptr == (char *) 0)
	{	message("Not enough RAM available to reset margin.  Aborting.");
		closef(fd2);
		return(1);
	}
	mrstr_row = (long *) mallocptr;

		/************* write raster file with new margin ***************/
	for(y = ymin; y <= ymax; y++)
	{	if(!((y-ymin) % 100))
		{	sprintf(string,"storing new raster, row %d",(y-ymin));
			message(string);
		}

		if(y == job_stats[2] + 1)
		{	/************ zero out storage row *************************/
			for(i=first_word; i<=last_word; i++)
				mrstr_row[i + offset] = 0;

		}
		else if((y >=  0) && (y <= job_stats[2]))
		{	for(i=0, j=first_word; j<=last_word; j++, i++)
				mrstr_row[i + offset] = lraster[y][j];
		}

			/************ store raster row in temp file ****************/
		if(writeb(mallocptr,size,1,fd2) <= 0)
		{	message("write failed when storing a row of raster data.  Aborting.");
			closef(fd2);
			free_ltp(mallocptr);
			return(1);
		}
	}

		/************* close files *************************************/
	closef(fd2);

		/************* close files & free malloc space *****************/
	free_ltp(mallocptr);

		/************* move raster file ********************************/
	sprintf(new_file,"%s/raster",temp_map_path);
	if(move_file(raster2, new_file))
	{	message("Move failed when attempting to move new raster into old file, Aborting.");
		return(1);
	}

	message("resetting map vairables & loading new raster into display RAM.");
		/************* reset job_stats *********************************/
	job_stats[1] = size * 8 - 1;
	job_stats[2] = rows - 1;
	for(i=3; i<=7; i++)
		job_stats[i] = 0;
	for(i=10; i<=18; i++)
	{	if((i==13)  ||  (i==15)  ||  (i==16))
			job_stats[i] = (job_stats[i])  ?    1 : 0;
		else
			job_stats[i] = 0;
	}
	job_stats[49] = 0;
	job_stats[75] = 0;


		/********** free old space *************************************/
	free_ltp(rasterm.malloc);
		/***********  calculate sizes **************/
	h = job_stats[2] + 1;             /* height of raster in rows  */
	w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */
	raster_size = h*w;                /* size of raster in bytes   */
	size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster/r */
		/***********  allocate memory **************/
	rasterm.malloc = (char *) calloc_ltp(size,1);
	if(!rasterm.malloc)
	{	message("  <<  cannot get enough RAM for raster.  You must re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}
		/********  calculate & set global pointers to raster ****************/

	rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
	rasterm.recn = rasterm.rec0 + raster_size;
	craster = (unsigned char **)(rasterm.malloc);
	lraster = (unsigned long **) (rasterm.malloc + 4*h);
	for(i=0; i<h; i++)
	{	craster[i] = (unsigned char *)rasterm.rec0 + w*i;
		lraster[i] = (unsigned long *) (craster[i]);
	}

		/***********  read raster from file *********************/
	sprintf(path,"%s/raster",temp_map_path);
	filefd = openf(path,READ);
	if(0 > filefd)
	{	message("  <<  cannot open raster map file.  You must re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}
	if(0 >= readb(rasterm.rec0,raster_size,1,filefd))
	{	message("  <<  cannot read raster map file.  You must re-ACQUIRE map & start over.");
		closef(filefd);
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}


		/* create index_arary in ram            */
		/* free existing array space */
	free_ltp(index_arraym.malloc);
		/***********  calculate sizes **************/
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = (h*w + 4*h);               /* size to malloc            */
		/***********  allocate memory **************/
	index_arraym.malloc = (char *) calloc_ltp(size,1);
	if(!index_arraym.malloc)
	{	message("  <<  cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
		job_stats[0] = 0;
		map_name[0] = 0;
		return(1);
	}
		/********  calculate & set global pointers to index_array ***********/
	index_arraym.rec0 = index_arraym.malloc;
	index_arraym.recn = (char *)(index_arraym.rec0 + h*w);
	index_array = (long **)index_arraym.recn;
	for(i=0; i<h; i++)
		index_array[i] = (long *) (index_arraym.malloc + w*i);

		/************* move raster registration ************************/
	for(i=0; i<4; i++)
	{	corner[8][i][0] = corner[0][i][0] + (32.0 * (offset - first_word));
		corner[8][i][1] = corner[0][i][1] - ymin;
	}
	register_ltp(0,0,corner[8],4,(char) 0);
	for(i=0; i<4; i++)
	{	corner[8][i][0] = 0.0;
		corner[8][i][1] = 0.0;
	}


		/************* clear level names *******************************/
	for(i=1; i<96; i++)
	{	if((i<25)  ||  (i>34))
			sprintf(level_names[i]," - unused -");
	}

		/************* write diary record ******************************/
	sprintf(string,"set new raster margin  (%d x %d)",job_stats[2]+1,job_stats[1]+1);
	put_diary(string);

		/************* redisplay map ***********************************/
		/*  set the raster_wincrement and raster_bincrement globals  */
	raster_wincrement = (job_stats[1] + 1) >> 5;
	raster_bincrement = (job_stats[1] + 1) >> 3;
		/** recompute view[] global ******/
	for(i=0; i<4; i++)
	{	view[i].lower_leftx = 0;
		view[i].lower_lefty = 0;
	}
	view[0].zoom_factor = mind(2,  (double) 1.0, 
			(double) (400.0/(maxl(2,job_stats[1]+1,job_stats[2]+1))));
	view[1].zoom_factor = 0.25;
	view[2].zoom_factor = 1.0;
	view[3].zoom_factor = 8.0;


	message("building new map display.");
	redisp_map();

		/************* reacquire map ***********************************/
	if(reacquire2())
		return(1);

		/************* re-create braster *******************************/
	if(create_braster)
		brstr_create();

		/************* return ******************************************/
	return(0);
}
