/*******************************************************************************
*       mlist_create     (see manual, Command Ref., User Function)      <fc>   *
*                       Creates map-list files for use by other functions.     *
*                       These ASCII files contain map names, 1 per line and    *
*                       may contain the wild card characters ? and *  (for     *
*                       single character or multiple characters).              *
*                                                                              *
*                       Parameters: char *fname;  name of mlist file to create *
*                                   char *list;   pointer to beginning of list *
*                                                 (null string terminated)     *
*                                                 (up to 50 items)             *
*                                                                              *
*                       No map files are changed.  The map-list file created   *
*                       is not a map file.  It can be accessed from any map.   *
*                       It resides under the map_list directory.               *
*                                                                              *
*                       Value returned:  0 = done, 1=couldn't create file.     *
*                       J.Dabritz   2/6/90                                     *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for multi user support */

#include "../../incl/globals.h"
#include "../../incl/in.h"

mlist_create(fname,list)
	char *fname;
	char *list;
{
	char path[512];
	char *ptr;
	
	long i;

	int fd;


		/************** check for at least 1 map name *****************/
	if(!(*list))
	{	message("You must have at least 1 map name on the list to create a mlist file.");
		return(1);
	}

		/************** create file ***********************************/
	trim(fname);
#ifdef MULTIUSER
	sprintf(path,"%s/.ltp/map_lists/%s",home_path,fname);
#else
	sprintf(path,"%s/map_lists/%s",ltp_path,fname);
#endif
	if(0 > creatf(path))
	{	message("file > ");
		append_msg(path);
		message("Can not create the mlist file.  Aborting command.");
		return(1);
	}

		/************** open file *************************************/
	fd = openf(path,WRITE);
	if(0 > fd)
	{	message("file > ");
		append_msg(path);
		message("Can not open the mlist file.  Aborting command.");
		return(1);
	}
	
		/************** write names ***********************************/
	for(i=0, ptr = list; *ptr && (i<50) ; i++, ptr += STRLIST)
	{	trim(ptr);
		if(writea(ptr,fd))
		{	message("Cannot complete writing list (at ");
			append_msg(ptr);
			message(")");
			message("saving what I have.");
			break;
		}
		if(writea("\n",fd))
		{	message("Cannot complete writing list (at ");
			append_msg(ptr);
			message(")");
			message("saving what I have.");
			break;
		}
	}

		/************** close file ************************************/
	closef(fd);

		/************** return ****************************************/
	return(0);
}
