/*******************************************************************************
*    next_step     (see manual, Command Ref., User Functions)    <fc>          *
*            This program checks the job_stats file and makes a best guess     *
*       at the logical next processing step for the current map based on the   *
*       status flags that it finds.  If the parm is a valid ptr, no message    *
*       will be displayed, but the parm will be loaded with the message.       *
*                                                                              *
*       files/globals changed:  none                                           *
*       Parameter:  ptr - char ptr to array of STRLIST bytes.                  *
*                                                                              *
*       values returned:  long 0;                                              *
*                                                                              *
*       parameters:    - none -                                                *
*                                                                              *
*            J.Dabritz   9/9/89                                                *
*******************************************************************************/

#include "../../incl/globals.h"

char *gbl_cptr;

long next_step(ptr)
	char *ptr;
{
	long a,i;
	void message_mgr();

		/***  Initialize global pointer ***/
	gbl_cptr = ptr;

		/************* check for map **********************************/
	if(!job_stats[0])
	{	message_mgr("CREATE or ACQUIRE a map.");
		return(0);
	}

		/************* check for imported raster data *****************/
	for(i=20; i<30; i++)
	{	if(job_stats[45] && (job_stats[i] >= job_stats[45]))
		{	if((registered(0) && registered(1)))
			{	message_mgr("Newly imported Raster data.  Edit it (DRAW, ERASE, DEL_.., LOG, GOTO, MACROS etc");
				return(0);
			}
			else if(registered(1))
			{	message_mgr("Map has newly imported Raster data.  REGISTER raster.");
				return(0);
			}
			else if(registered(0))
			{	message_mgr("Map has newly imported Raster data.  REGISTER geodetics.");
				return(0);
			}
			else
			{	message_mgr("Map has newly imported Raster data.  REGISTER raster and geodetics.");
				return(0);
			}
		}
	}

		/************* check for imported vector data *****************/
	for(i=30; i<40; i++)
	{	if(job_stats[i] > job_stats[45])
		{	message_mgr("Newly imported vector data.  Add it to raster (see LEVEL cmds) or compair it.");
				return(0);
		}
	}

		/************* check for data in map **************************/
	get_toler();

	if(job_stats[45] == 0)
	{	if(registered(0) && registered(1))
		{	if(registered(4)  ||  registered(5))
			{	if((gr_dev == 1) || (gr_dev == 3))    /* dig absolute mode */
				{	message_mgr("IMPORT data from a file, or DRAW data on map from digitizer");
					return(0);
				}
				else
				{	message_mgr("IMPORT data from a file, or change (GINPUT -a 1) to digitizer absolute input.");
					return(0);
				}
			}
			else
			{	message_mgr("Map is ready for IMPORT  or  REGISTER digitizer and DRAW data onto map.");
				return(0);
			}
		}
		else if((registered(0) && registered(4)) ||
		 (registered(0) && registered(5)))
		{	if((gr_dev == 1) || (gr_dev == 3))    /* dig absolute mode */
			{	message_mgr("REGISTER gedetics (for Import), or DRAW data on map from digitizer");
				return(0);
			}
			else if(tolerance == 0.0)
			{	message_mgr("IMPORT data from a file, or change (GINPUT -a 1) to digitizer absolute input.");
				return(0);
			}
			else 
			{	message_mgr("IMPORT data from a file, or REGISTER geodetics for digitizer (Draw) input.");
				return(0);
			}
		}
		else
		{	message_mgr("A new map, set HEADER, or IMPORT raster data, or REGISTER geodetetics.");
			return(0);
		}
	}


		/************* check for editing ******************************/
	if(job_stats[40] > job_stats[41])    /* edited since last assembly */
	{	if(job_stats[14] > job_stats[75])
		{	message_mgr("Finish checking (GOTO +) the log errors file, or clear the file (LOG CLEAR)");
			return(0);
		}
		else
		{	message_mgr("Finish editing (DRAW, ERASE, DEL_.., LOG, GOTO, MACROS...), or ASSEMBLE map.");
			return(0);
		}
	}


		/************* check for exported data ************************/
	for(i=50; i<70; i++)
	{	if(job_stats[i] >= job_stats[45])
		{	message_mgr("Data Exported since last data change.  Change something or ACQUIRE a new map.");
			return(0);
		}
	}


		/************* check for attributes & plot & report ***********/
	if(job_stats[42] <= job_stats[41])     /* not plotted since last asmbl  */
	{	if(job_stats[44] <= job_stats[41]) /* not attributed since last asmbl */
		{	message_mgr("Map just Assembled.  PLOT, attribute (ATRBT_...), REPORT, or EXPORT data.");
			return(0);
		}
		else
		{	message_mgr("Map attributed, finish attributing (ATRBT_..), PLOT, REPORT, or EXPORT data.");
			return(0);
		}
	}
	else           /** map has been ploted since last assembly **/
	{	if(job_stats[44] <= job_stats[45]) /*not attributed since last change */
		{	message_mgr("Map just Plotted.  Attribute (ATRBT_...), REPORT, or EXPORT data.");
			return(0);
		}
		else
		{	message_mgr("Map attributed, finish attributing (ATRBT_..), REPORT, or EXPORT data.");
			return(0);
		}
	}

}

void message_mgr(msg)
	char *msg;
{
	if( gbl_cptr != (char *) 0)
		strcpy(gbl_cptr, msg);
	else
		message(msg);
	return;
}
