/*******************************************************************************
*   open_solids     (see manual, Command Ref., User Functions)         <fc>    *
*                   Description:   Erases the interior pixels of all solid     *
*                   areas within the raster.   Assembly is not required.       *
*                   Parameters:   None.                                        *
*                   Values returned:   0 = done, 1=aborted.                    *
*                   J.Dabritz   2/5/90                                         *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long open_solids( )
{
	long *old_row_array;	/*** pointer to array of 2 old raster rows			*/
	long *old_row0_ptr;	 /*** pointer to old current row						 */
	long *old_row1_ptr;	 /*** pointer to old row above current row		 */

	long *row_ptr;				/*** pointer to current raster row						*/
	long *end_raster_ptr; /*** pointer just beyond end of raster (last row)	 */

	long new_old_incr0; /*** difference (increment) from new to old row0 ptrs */
	long new_old_incr1; /*** difference (increment) from new to old row1 ptrs */

	long *ptr, *end_ptr;	/*** pointers for temporary use, initializing etc.		*/

	long *word_ptr;			 /*** pointer to current word in raster row			*/
	long *end_row_ptr;		/*** pointer just beyond end of current raster row	*/
	long word;						/*** resolved word_ptr value							*/

	long *ptr0;					 /*** pointer to word in old row 0 (current row) */
	long *ptr1;					 /*** pointer to word above in old array			 */

		/***	array of all bit numbers of a word in RAM order	*/
	static long bits[32] = { 7,6,5,4,3,2,1,0,	15,14,13,12,11,10,9,8,
		23,22,21,20,19,18,17,16,	31,30,29,28,27,26,25,24 };
	long *bit_ptr;				/*** pointer to current bit number						*/
	long *end_bit_ptr;		/*** pointer just beyond bits array					 */

	long locale;			/*** locale variable coding 8 surrounding bits*/
	long old_locale;		/*** same for 'old' raster of previous pass		*/

	long count;					 /*** count of pixels opened on a single pass	 */

	long bit;						 /*** bit number (0-31) of pixel in question	 */
	long lt_bit;					/*** bit number (0-31) of bit to the left		 */

	long x, y;						/*** parameters to pass to pixel			*/
	long row;

	char string[STRLIST];


		/***************** check for map present ***************************/
	if(!job_stats[0])
	{	message("No map current acquired.	command aborted.");
		return(1);
	}

		/************* check for VIEW ONLY raster **********************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}
		/***************** set up old row array ****************************/
	old_row_array = (long *) calloc_ltp(3 + 2 * raster_wincrement,4);
	if(old_row_array <= 0)
	{	message("Open_solids aborted.	Not enough RAM space available.");
		return(1);
	}
	old_row0_ptr = old_row_array + 1;								 /*** space for 1 row	*/
	old_row1_ptr = old_row0_ptr + raster_wincrement + 1; /*space for 1 more row */


		/***************** clear right hand column *************************/
		/****************** (to avoid wrapping on raster in locale_ptr() ***/
	erase_rect(job_stats[1],0,job_stats[1],job_stats[2]);

		/***************** open solids for each line ***********************/
	end_raster_ptr = (long *) (lraster[0] - 1);
	end_bit_ptr = bits + 32;

	count = 0;
		/***	reset old row above to zeros (empty is assumed around raster) */
	end_ptr = old_row0_ptr + raster_wincrement;
	for(ptr = old_row0_ptr;	ptr < end_ptr	;	ptr++)
		*ptr = 0;
		/***	set pointer to row						*/
	row_ptr = (long *) lraster[job_stats[2]];
		/***	open each row, top to bottom	*/
	for( row=0; row_ptr > end_raster_ptr ; row++, row_ptr -= raster_wincrement)
	{		/*** reset old row pointers */
		ptr = old_row1_ptr;
		old_row1_ptr = old_row0_ptr;
		old_row0_ptr = ptr;
			/*** load current row into old_row0 */
		memcpy((char *) old_row0_ptr, (char *) row_ptr, raster_bincrement);
			/*** check for user abort ***/
		if(abort_cmd())
			return(1);
			/*** calc new increment from current to old row pointers */
		new_old_incr0 = old_row0_ptr - row_ptr;
		new_old_incr1 = old_row1_ptr - row_ptr;
				/***************** open by word, left to right *************/
		end_row_ptr = row_ptr + raster_wincrement;
		for(word_ptr = row_ptr; word_ptr < end_row_ptr; word_ptr++)
		{		/***	check for any on bits in word	*/
			if(*word_ptr)
			{	 /*** set word variable (for repeted use, faster than *word_ptr) */
				word = *word_ptr;
					/*** open word by bit, left to right ******/
				for(bit_ptr = bits; bit_ptr < end_bit_ptr; bit_ptr++)
				{		/***	check for 'on' bit	*/
					if(word & (1 << *bit_ptr))
					{		/***************** calc new locale *************/
						locale = locale_ptr(word_ptr,*bit_ptr);
							/*** calc locale for old raster at this location	*/
						old_locale = locale & 60;	 /* clear bits 6,7,0,1	*/
						ptr0 = word_ptr + new_old_incr0;
						ptr1 = word_ptr + new_old_incr1;
						bit = *bit_ptr;
							/******* calc the old_locale variable *********/
						switch(bit & 7)	 {
						case 0:						 /*** on rt edge of byte	*/
							lt_bit = bit + 1;
							if(bit == 24)		 /*** on rt edge of word	*/
							{	old_locale |= ((*ptr0 >> lt_bit) & 1) << 6;
								old_locale |= ((*ptr1 >>		bit) & 1)		 ;
								ptr1++;
							}
							else							/*** not on rt edge of word */
							{	old_locale |= ((*ptr0 >> lt_bit) & 1) << 6;
								old_locale |= ((*ptr1 >>		bit) & 1)		 ;
							}
							break;
						case 7:						 /*** on lt edge of byte	*/
							if(bit == 7)			/*** on lt edge of word	*/
							{	lt_bit = 24;
								ptr0--;
								old_locale |= ((*ptr0 >> lt_bit) & 1) << 6;
								old_locale |= ((*ptr1 >>		bit) & 1)		 ;
								ptr1--;
							}
							else							/*** not on lt edge of word */
							{	lt_bit = bit - 15;
								old_locale |= ((*ptr0 >> lt_bit) & 1) << 6;
								old_locale |= ((*ptr1 >>		bit) & 1)		 ;
							}
							break;
						default:							/*** in middle of byte	 */
							lt_bit = bit + 1;
							old_locale |= ((*ptr0 >> lt_bit) & 1) << 6;
							old_locale |= ((*ptr1 >>		bit) & 1)		 ;
							break;
						}

							/*************	check for solid			***********/
						if((old_locale & 85) == 85)
						{	 *word_ptr ^= (1 << *bit_ptr);
							count++;
						}
					}
				}
			}
		}
		if(!(row % 100))
		{	sprintf(string,"opening solids, row %d,  %d pixels erased.",row,count);
			message(string);
		}
	}
	sprintf(string,"opening solids, %d pixels erased (total).",count);
	message(string);


		/***************** free malloc'd space *****************************/
	free_ltp((char *) old_row_array);


		/***************** update display **********************************/
	redisp_map();
	if(resolution_code)
	{	x = ((rwindowxmin + rwindowxmax) >> 1) + 1;
		y = ((rwindowymin + rwindowymax) >> 1) + 1;
		dispgraphics(x,y,resolution_code);
	}


		/***************** update edited variable **************************/
	if((job_stats[40] < job_stats[45]) || !job_stats[45])
	{	job_stats[45]++;
		job_stats[40] = job_stats[45];
	}

		/***************** return ******************************************/
	return(0);
}
