/*******************************************************************************
*   p_create   (see manual, Command Ref., User Functions)   <fc>               *
*                                                                              *
*        Writes pen-to-set assignment values (in pname) to the pens.h array,   *
*        and stores those values in a pen file called  pfile which is created  *
*        under the (default data path)/pens directory. The pen-to-set values   *
*        are written one per record to the file, the 32 records representing   *
*        sets 0-31 respectively. A 0 pen value for a set will produce no plot  *
*        for that set.                                                         *
*        Parameters:  fname - name of the penfile to create                    *
*                     pen_set - character array of 'digits' for sets to plot   *
*        Globals Modified:  none, but a new file is writted to the pens dir.   *
*                                                                              *
*        Returns zero upon normal execution, else returns -1.                  *
*           C. Jonson 2/13/90                                                  *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for multi user support */

#include "../../incl/globals.h"

long p_create( fname, pen_set )
	char *fname;
	char *pen_set;
{
	extern int errno;
	int        fd;
	char       pname[STRLIST];
	char       string[STRLIST];
	long i;

/*** check pen_set string ****/
	for(i=0; i<32  &&  pen_set[i] ; i++)
	{	if((pen_set[i] < '0') || (pen_set[i] > '9'))
			pen_set[i] = '0';
	}
	for(; i<32; i++)
		pen_set[i] = '0';
	pen_set[32] = (char) 0;
	
/*** Open the penfile ***/
#ifdef MULTIUSER
	sprintf(pname, "%s/.ltp/pens", home_path);
	if (!file_exists(pname,fname)) {
		char orig_path[512];
		sprintf(orig_path, "%s/pens", ltp_path);
		if (file_exists(orig_path, fname))
			copy_file(orig_path, fname, pname);
	}
	strcat(pname, "/");
	strcat(pname, fname);
#else
	sprintf(pname, "%s/pens/%s", ltp_path, fname);
#endif

	if( creatf(pname) == -1 )
	{	sprintf(string, "Cannot create %s.  UNIX errno is %d", pname, errno);
		message(string);
		return(-1);
	}

	if( (fd = openf(pname, WRITE)) == -1)
	{	sprintf(string, "Cannot open %s.  UNIX errno is %d", pname, errno);
		message(string);
		return(-1);
	}

/*** Write the string to the penfile ***/
	sprintf(string, "%s\n", pen_set);
	if( writea(string, fd)  == -1 )
	{	closef(fd);
		return(-1);
	}

/*** Close file and return ***/
	closef(fd);
	return(0);
}
