/*******************************************************************************
*   pers_plot    (see manual, Command Ref., User Functions)     <fc>           *
*                                                                              *
*           Generates a perspective contour plot of the current,               *
*           active map (found in files.h) to the screen map                    *
*           area (tile0) only.                                                 *
*                                                                              *
*           Parameters:   long d;      view direction code: 0=s,1=w,2=n,3=e    *
*                         double esf;  Elevation Sensitivity Factor            *
*                                      (elevation exaduration factor)          *
*                         double yvf;  Y Vertical Factor (vertical offset      *
*                                      for each unit in the y direction)       *
*                         double ysf;  Y Skew Factor (horizontal offset        *
*                                      for each unit in the y direction)       *
*                                                                              *
*           Globals changed:   - none -                                        *
*                                                                              *
*           Returns zero upon normal execution.                                *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for X11 support */

#include <math.h>
#include "../../incl/globals.h"
#include "../../incl/in.h"
#ifdef LTP_X11
#include "../../incl/x11.h"
#endif

long pplot_trans();

long sx_0, sy_0;
long x_0, y_0, e0;
double y_to_sx;
double x_to_sx;
double y_to_sy;
double x_to_sy;
double e_to_sy;


long pers_plot( d, esf, yvf, ysf)
	long d;
	double esf, yvf, ysf;
{
	OBJECTS object;
	INDEX_LIST index_list;
	ATTRIBUTES atrbt;
	COORD_LIST crdlst;

	char string[256];

	long cx1,cx2,cy_1,cy2;
	long first_elev;
	long row, col;
	long i;
	long elev, min_elev, max_elev;
	long width, depth, height;
	long screenx, screeny;
	long xlimit;
	long plane;
	long x1,x2,x3,x4,x5,x6,x7,x8;
	long y_1,y2,y3,y4,y5,y6,y7,y8;
	long elevations;
	long lx1, lx2, ly_1, ly2;
	long v, v1, v2;

	double x,y,e;
	double fx,fy;
	double ratio, sratio;
	double r_to_s_factor;



/*============================================================================*/


		/******* check for batch mode *********************************/
	if(batch_mode)
		return(0);

		/******* check for map ****************************************/
	if(!job_stats[0])
	{	message("You must acquire a map befor using pers_plot.");
		return(1);
	}

		/******* check for assembled data *****************************/
	if(!job_stats[5])
	{	message("You must have assembled segments befor using pers_plot.");
		return(1);
	}

		/******* check for attributes *********************************/
	if(!job_stats[13])
	{	message("You must have assigned elevation attributes befor using pers_plot.");
		return(1);
	}


		/******** get ground reference factors ************************/
	get_units();
	if(unit_factor[2][0] == (double) 0.0)
	{	message("Cannot plot perspective without ground reference.  Raster and Geodetics must");
		message("be registered.");
		return(1);
	}



/*=============== calculate raster-to-screen factors for view ================*/

		/******* find elevation limits of view ************************/
		/* find index limits of view */

		/******************* find cell limits of current display ***/
	cx1 = rwindowxmin >> 5;
	cx2 = (rwindowxmax <= job_stats[1]) ?  rwindowxmax : job_stats[1];
	cx2 = cx2 >> 5;
	cy_1 = rwindowymin >> 5;
	cy2 = (rwindowymax <= job_stats[2]) ?  rwindowymax : job_stats[2];
	cy2 = cy2 >> 5;

		/* read all segments & check first attributes */
	first_elev = 1;             /** flag for first elevation **/
	for(row = cy_1; row <= cy2; row++)
	{	for(col = cx1; col <= cx2; col++)
		{		
				/*********** for each cell check all index_list recs  */
			index_list.next = index_array[row][col];
			for(; index_list.next; )
			{		
					/*** get next indext_list record ****/
				if(get_indexl(&index_list,index_list.next))
				{	message("abort pers_plot, location 0.");
					return(1);
				}

					/*********** check index_list object for plotting **/
				if(index_list.type == 1)          /* segment */
				{		/** get object data **/
					if(get_objects(&object,index_list.obj_rec))
					{	message("abort pers_plot, location 1.");
						return(1);
					}

						/***** check first attribute ****/
					if(object.i12[2])
					{	if(get_attr(&atrbt,object.i12[2]))
						{	message("abort pers_plot, location 2.");
							return(1);
						}
							/** convert to elevation **/
						trim(atrbt.string);
						if(*atrbt.string >= '0'   &&  *atrbt.string <= '9')
						{	sscanf(atrbt.string,"%ld",&elev);
							if(first_elev)
							{	max_elev = elev;
								min_elev = elev;
								first_elev = 0;
							}
							else
							{	if(elev < min_elev)
									min_elev = elev;
								else if(elev > max_elev)
									max_elev = elev;
							}
						}
					}
				}
			}
		}
	}
		/******* check for any elevations *****************************/
	if(first_elev)
	{	message("There are no segments which have elevation attributes in the current view.");
		message("No Perspective Plot is possible.");
		return(1);
	}


		/******* calculate limits *************************************/
		/** (next 3 are in pixels) **/
	width = (d & 1) ?    rwindowymax-rwindowymin+1 :  rwindowxmax-rwindowxmin+1;
	depth = (d & 1) ?    rwindowxmax-rwindowxmin+1 :  rwindowymax-rwindowymin+1;
	x = ((max_elev - min_elev) / unit_factor[2][0]) + 0.5;
	height = x;


	x = width + (depth * fabs(ysf));
	y = (height * esf) + (depth * yvf);
	if(x == 0.0)
	{	message("abort pers_plot, location 3");
		return(1);
	}
	ratio = y / x;


	screenx = windowxmax - windowxmin - 9;
	screeny = windowymax - windowymin - 9;
	if(screenx == 0.0)
	{	message("abort pers_plot, location 4");
		return(1);
	}
	sratio = screeny / (double) screenx;


	xlimit = (ratio > sratio) ?    0 : 1;     /*  1 = x is limiting dimention */
	r_to_s_factor = (xlimit) ?     screenx / x :   screeny / y;

	if(ysf >= 0.0)
		sx_0 = windowxmin + 4;
	else
		sx_0 = windowxmin + 4 - (depth * r_to_s_factor * ysf);
	sy_0 = windowymax - 4;

	switch(d)   {
	case(0) :                 /** view from south **/
		x_0 = rwindowxmin;
		y_0 = rwindowymin;
		e0 = min_elev;
		
		y_to_sx = r_to_s_factor * ysf;
		x_to_sx = r_to_s_factor;
		
		y_to_sy = -r_to_s_factor * yvf;
		x_to_sy = 0.0;
		e_to_sy = r_to_s_factor * -esf / unit_factor[2][0];
		break;
	case(1) :                 /** view from west  **/
		x_0 = rwindowxmin;
		y_0 = rwindowymax;
		e0 = min_elev;
		
		y_to_sx = - r_to_s_factor;
		x_to_sx = r_to_s_factor * ysf;
		
		y_to_sy = 0.0;
		x_to_sy = -r_to_s_factor * yvf;
		e_to_sy = r_to_s_factor * -esf / unit_factor[2][0];
		break;
	case(2) :                 /** view from north  **/
		x_0 = rwindowxmax;
		y_0 = rwindowymax;
		e0 = min_elev;
		
		y_to_sx = -r_to_s_factor * ysf;
		x_to_sx = -r_to_s_factor;
		
		y_to_sy = r_to_s_factor * yvf;
		x_to_sy = 0.0;
		e_to_sy = r_to_s_factor * -esf / unit_factor[2][0];
		break;
	case(3) :                 /** view from east   **/
		x_0 = rwindowxmax;
		y_0 = rwindowymin;
		e0 = min_elev;
		
		y_to_sx = r_to_s_factor;
		x_to_sx = -r_to_s_factor * ysf;
		
		y_to_sy = 0.0;
		x_to_sy = r_to_s_factor * yvf;
		e_to_sy = r_to_s_factor * -esf / unit_factor[2][0];
		break;
	}


		/******* save current screen data & clear screen **************/
	for(i=0; i<4; i++)
	{	swplane(i);
#ifndef LTP_X11
		bitmap(BITPLN_WIDTH,480,BITPLN_WIDTH,BITPLN_WIDTH,vga_plane,scratch_pad[i]);
#else
                bitmap4x(WIDTH,480,0,0,xmainwin,0,0,xmainpix);
#endif
	}


		/******************* remove old graphics display ***********/
	for(plane=0; plane<4; plane++)
	{	swplane(plane);
		bar_off(windowxmin,windowymin,windowxmax,windowymax);
		logo_on = 0;
	}

		/******* draw box *********************************************/
	x = rwindowxmin;
	y = rwindowymin;
	e = min_elev;
	pplot_trans(x,y,e,&x1,&y_1);
	x = rwindowxmax;
	pplot_trans(x,y,e,&x2,&y2);
	y = rwindowymax;
	pplot_trans(x,y,e,&x3,&y3);
	x = rwindowxmin;
	pplot_trans(x,y,e,&x4,&y4);
	y = rwindowymin;
	e = max_elev;
	pplot_trans(x,y,e,&x5,&y5);
	x = rwindowxmax;
	pplot_trans(x,y,e,&x6,&y6);
	y = rwindowymax;
	pplot_trans(x,y,e,&x7,&y7);
	x = rwindowxmin;
	pplot_trans(x,y,e,&x8,&y8);	
	swplane(2);
	line_on(x1,y_1,x2,y2);
	line_on(x1,y_1,x4,y4);
	line_on(x1,y_1,x5,y5);
	line_on(x2,y2,x3,y3);
	line_on(x2,y2,x6,y6);
	line_on(x3,y3,x4,y4);
	line_on(x3,y3,x7,y7);
	line_on(x4,y4,x8,y8);

	
		/******* define box limits for user ***************************/
	switch(d)  {
	case(0):
		sprintf(string,"View from Bottom,  Elevation limits:  %d  to  %d.",min_elev,max_elev);
		break;
	case(1):
		sprintf(string,"View from Left,  Elevation limits:  %d  to  %d.",min_elev,max_elev);
		break;
	case(2):
		sprintf(string,"View from Top,  Elevation limits:  %d  to  %d.",min_elev,max_elev);
		break;
	case(3):
		sprintf(string,"View from Right,  Elevation limits:  %d  to  %d.",min_elev,max_elev);
		break;
	}
	message(string);

		/******* draw contours (attribute 1 & 2) **********************/
	swplane(2);
	for(row = cy_1; (row <= cy2)  &&  (!abort_cmd()); row++)
	{	for(col = cx1; col <= cx2; col++)
		{		
				/*********** for each cell check all index_list recs  */
			index_list.next = index_array[row][col];
			for(; index_list.next; )
			{		
					/*** get next indext_list record ****/
				if(get_indexl(&index_list,index_list.next))
				{	message("abort pers_plot, location 0.");
					return(1);
				}

					/*********** check index_list object for plotting **/
				if(index_list.type == 1)          /* segment */
				{		/** get object data **/
					if(get_objects(&object,index_list.obj_rec))
					{	message("abort pers_plot, location 1.");
						return(1);
					}

						/***** check first attribute ****/
					elevations = 0;
					if(object.i12[2])
					{	if(get_attr(&atrbt,object.i12[2]))
						{	message("abort pers_plot, location 2.");
							return(1);
						}
							/** convert to elevation **/
						trim(atrbt.string);
						if(*atrbt.string >= '0'   &&  *atrbt.string <= '9')
						{	sscanf(atrbt.string,"%ld",&elev);
							elevations++;
						}
					}

					if(elevations)
					{		/***** plot first elevation */
						if(get_coordlst(&crdlst,index_list.v1))
							return(1);
						fx = crdlst.x + 0.5;
						lx2 = fx;
						fy = crdlst.y + 0.5;
						ly2 = fy;
						e = elev;
	
						v = index_list.v1 + 1;
						v2 = index_list.v2;
						for( ; v <= v2 ;v++)
						{	if(get_coordlst(&crdlst,v))
								return(1);
							fx = crdlst.x + 0.5;
							lx1 = fx;
							fy = crdlst.y + 0.5;
							ly_1 = fy;
							if(!(clipl(&lx1,&ly_1,&lx2,&ly2, rwindowxmin,rwindowxmax,
						 	rwindowymin,rwindowymax)))
							{	x = lx1;
								y = ly_1;
								pplot_trans(x,y,e,&x1,&y_1);
								x = lx2;
								y = ly2;
								pplot_trans(x,y,e,&x2,&y2);
								line_on(x1,y_1,x2,y2);
							}
	
								/** update ending vertex **/
							lx2 = fx;
							ly2 = fy;
						}
					}
				}
			}
		}
	}


		/******* prompt user to press return to continue with LTPlus **/
	message("Perspective Plot complete.  Press [RETURN] when ready to continue.");
	get_string(string);

		/******* replace screen graphics ******************************/
		/* reset all bit planes                          */
	for(i=0; i<4; i++)
	{	swplane(i);
#ifndef LTP_X11
		bitmap(BITPLN_WIDTH,480,BITPLN_WIDTH,BITPLN_WIDTH,scratch_pad[i],vga_plane);
#else
                bitmap4x(WIDTH,480,0,0,xmainpix,0,0,xmainwin);
#endif
	}

		/******* return ***********************************************/
	return(0);
}



/*******************************************************************************
*  pplot_trans      This program transforms x,y,e[levation] raster & feet      *
*      coordinates into screen (pixel) coordinates.                            *
*                                                                              *
*      paramerers:  double x,y,e;    raster,raster,feet coordinates input      *
*                   long  *xptr,*yptr;   pointers to screen x & y output       *
*                                                                              *
*      returns      long 0;                                                    *
*         J.Dabritz,  11/24/89                                                 *
*******************************************************************************/

long pplot_trans(x,y,e,xptr,yptr)	
	double x,y,e;
	long *xptr, *yptr;
{
	double dx,dy,de,x1,y_1;


	dx = x - x_0;
	dy = y - y_0;
	de = e - e0;

	x1 = sx_0 + (dy * y_to_sx) + (dx * x_to_sx) + 0.5;
	y_1 = sy_0 + (dy * y_to_sy) + (dx * x_to_sy) + (de * e_to_sy) + 0.5;

	*xptr = x1;
	*yptr = y_1;
	
	return(0);
}
