/*******************************************************************************
*       plot_ltp   (see manual, Command Ref., User Functions)   <fc>   *
*                                                                              *
*                       Generates a plot to the specified (dev) plotter of     *
*                       the current, active map (found in files.h), using      *
*                       label type ltype and text size sizen, if included.     *
*                       If pen_path is a  null string, plot everything, else   *
*                       produce a plot of the selected objects only.           *
*                                                                              *
*         Returns 0 upon normal execution, else returns -1.      *
*         C. Jonson  9/1/89    modified 7/10/90 J.Dabritz        *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for multi user support */

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

long plot_xhair();
long plot_label();

static int  fd = 0;      /*  temp fd used for writing plotter cmds */

long plot_ltp( dev, ltype, sizen, scale, pen_path, fil_nam )
	long dev,ltype;
	double sizen;
	long scale;
   char *pen_path, *fil_nam;
{
   double beg_x, beg_y;
   double tmp_x, tmp_y;

   long   end_orec;
   int    fd_ascii;                         /* generic fd              */
   long   first_time;
   long   i, j, k;
   unsigned long   penmask[8];              /* bitmask for set/pen rel */
   long   plot_rotated = FALSE;             /* rotational flag         */
   long   *rec_plotted;
	long   save_dispn;

   char   p_cmd[512];
   char   *cptr;
   char   s_hdr[STRLIST];
   char   cat_str[STRLIST];
   char   string[256];
   char   user_str[STRLIST];
   OBJECTS    orec;
   COORD_LIST crec;


      /*** check for user abort ***/
   if(abort_cmd())
      return(-1);

/*** See if pen path is not null.  If true, get pen file record ***/
   if( *pen_path != '\0')
   {   if( get_penfile(pen_path, penmask) )
      {   message("Cannot read penfile.  Bye.");
         return(-1);
      }
   }
   else
   {   penmask[0] = 1;
      for( i = 1; i < 8; penmask[i++] = 0);
   }

      /*** check for user abort ***/
   if(abort_cmd())
      return(-1);

		/*** check for screen plot ***/
	if( dev == 0)
	{		/*** reset screen ***/
		graphics(0,1,0,0,0,0,0,0,0);
			/*** view map ***/
		view_ltp(0,0,0);
			/*** reset normal display parameters using penmask flags ***/
		j = 0;
		for(i=0; i<8; i++)       /** for each penmask **/
			j |= penmask[i];
			/*** add centroids if polygons plotted & labels wanted ***/
		if((map_type & 4) && ltype)
			i=1;
		else
			i=0;
			/*** display vector data ***/
		graphics(0,0,j,0,0,0,0,0,i);
			/*** return              ***/
		return(0);
	}

/*** Select fd to write to                   ***/
   if( dev == 3 )
	{	if(fil_nam[0] != '/')
#ifdef MULTIUSER
			sprintf(string,"%s/.ltp/export/%s",home_path,fil_nam);
#else
			sprintf(string,"%s/export/%s",ltp_path,fil_nam);
#endif
		else
			sprintf(string,"%s",fil_nam);

      if( creatf( string) == -1 )
      {   sprintf(cat_str,"Cannot create %s for plot",string);
         message(cat_str);
         return(-1);
      }
      if( (fd_ascii = openf(string, RDWR)) == -1)
      {   sprintf(cat_str, "Cannot open %s for plotting",string);
         message(cat_str);
         return(-1);
      }
      fd = fd_ascii;           /*** Write to ascii file ***/
   }
   else
      fd = fd_plot;            /*** Write to plotter device ***/

      /*** check for user abort ***/
   if(abort_cmd())
   {   closef(fd);
      if( dev == 1 || dev == 2 )   /* plotter was used & closed */
         fd_plot = -1;
      return(-1);
   }

/*** Initialize the plotter and select pen 1  ***/
   strcpy(p_cmd, "IN;SP1;");
   if( put_plot(p_cmd, fd, dev) == -1)
   {   message("PLOT: Cannot initialize plotter or select pen.");
      return(-1);
   }

      /*** check for user abort ***/
   if(abort_cmd())
   {   closef(fd);
      if( dev == 1 || dev == 2 )   /* plotter was used & closed */
         fd_plot = -1;
      return(-1);
   }

/*** See if axis rotation is necessary.  (compare x to y)  ***/
   if( corner[8][2][0] < corner[8][2][1] )
   {   strcpy(p_cmd, "RO90;");
      if( put_plot(p_cmd, fd, dev) == -1)
      {   message("PLOT:  Cannot send to plotter.  Bye.");
         return(0);
      }
      plot_rotated = TRUE;
   }

      /*** check for user abort ***/
   if(abort_cmd())
   {   closef(fd);
      if( dev == 1 || dev == 2 )   /* plotter was used & closed */
         fd_plot = -1;
      return(-1);
   }

/*** Draw corner markers (200 pu by 200 pu)  ***/
   for( i = 0; i < 4; i++ )
      if( plot_xhair(dev,corner[8][i][0],corner[8][i][1], 200L, 200L))
         return(-1);

      /*** check for user abort ***/
   if(abort_cmd())
   {   closef(fd);
      if( dev == 1 || dev == 2 )   /* plotter was used & closed */
         fd_plot = -1;
      return(-1);
   }

/*** Draw legend and whatnot  ***/
   std_header( s_hdr );

      /*** check for user abort ***/
   if(abort_cmd())
   {   closef(fd);
      if( dev == 1 || dev == 2 )   /* plotter was used & closed */
         fd_plot = -1;
      return(-1);
   }

/*** Manipulate the standard header to place the user name on another line ***/
   cptr = strstr(s_hdr, "User:");
   strcpy(user_str, cptr);
   *cptr = CNULL;            /* truncate 'User:' string from header */
   
   if( plot_rotated )
   {   tmp_x = corner[8][1][0];
      tmp_y = corner[8][1][1] + 800.0;    /** use raster size!!! */
   }
   else
   {   tmp_x = corner[8][1][0];
      tmp_y = corner[8][1][1] + 800.0;
   }
   sprintf(p_cmd, "PU;PA%d,%d;LB%s\03;PU;PA%d,%d;LB%s%d  %s\03;PU;", 
      (long)tmp_x, (long)tmp_y, s_hdr, (long)tmp_x,
         (long)(tmp_y - 300), "PLOT SCALE     1:", scale, user_str );
   if( put_plot(p_cmd, fd, dev) == -1 )
   {   message("PLOT:  Cannot send to plotter.  Bye.");
      return(0);
   }

/*** Calculate number of object records ***/
   end_orec = job_stats[10];        /* view polys to get label info */

      /*** check for user abort ***/
   if(abort_cmd())
   {   closef(fd);
      if( dev == 1 || dev == 2 )   /* plotter was used & closed */
         fd_plot = -1;
      return(-1);
   }

/*** Calloc space for array of 'objects plotted' flags ***/
   rec_plotted = (long *)calloc_ltp(end_orec, sizeof(long));
   if( rec_plotted == (long *)NULL )
   {   sprintf(string, "Cannot calloc %d bytes.  Plot aborting.",
               end_orec * sizeof(long) );
      message(string);
      return(-1);
   }

      /*** check for user abort ***/
   if(abort_cmd())
   {   free_ltp(rec_plotted);
      closef(fd);
      if( dev == 1 || dev == 2 )   /* plotter was used & closed */
         fd_plot = -1;
      return(-1);
   }


/*** Read objects file until EOF and format plotter commands ***/
   for( i = 0; i < 8; i++ )              /* Loop for all pens */
	{		/*** check if any sets plotted in this pen ****/
      if(penmask[i])
		{		/*** change pen ****************************/
         sprintf(p_cmd, "SP%d;", i + 1);
         if( put_plot(p_cmd, fd, dev) == -1)
         {   message("PLOT: Cannot select pen.");
            return(-1);
         }
   
				/***** check all object records *******/
         for(j = 0; j < end_orec; j++ )
         {		/*** See if object has been previously plotted ***/
            if( rec_plotted[j] )
               continue;
   
               /*** check for user abort ***/
            if(abort_cmd())
            {   free_ltp(rec_plotted);
               closef(fd);
               if( dev == 1 || dev == 2 )   /* plotter was used & closed */
                  fd_plot = -1;
               return(-1);
            }
   
					/**** See if object should be plotted ***/
				save_dispn = disp_normal;
				disp_normal = penmask[i];
				if((disp_object(j) & 1) == 0)
				{	disp_normal = save_dispn;
					continue;
				}
				disp_normal = save_dispn;
				
					/**** get object record *********/
            if( get_objects( &orec, j ))
            {   message("Plot:  Cannot read objects file");
               return(-1);
            }
   
         /*** Label object as having been plotted and record pen number ***/
            rec_plotted[j] = TRUE;
   
         /*** Node is to be plotted... ***/
            if( orec.i12[0] <= 7 )
            {  beg_x = (double)orec.f4[0];
               beg_y = (double)orec.f4[1];
               if(transform( (double *)&beg_x, (double *)&beg_y, 0L, 8L))
               {   message("0th transform failed");
                  return(-1);
               }
               if( plot_xhair(dev, beg_x, beg_y, 20L, 20L))
                  return(-1);
               if(plot_label( dev, &orec, j, ltype, sizen) )
                  return(-1);
            }
         /*** Segments are to be plotted (includes polygons) ***/
            else if( orec.i12[0] <= 10 )
            {   strcpy(p_cmd, "PU;");
               if( put_plot(p_cmd, fd, dev) == -1 )
               {   message("Cannot send to plotter. Bye.");
                  return(-1);
               }
               for( first_time = 1, k = orec.i12[5]; 
                     k <= orec.i12[6]; k++ )
               {   if(get_coordlst(&crec, k))
                  {   message("problem with clist");
                     break;
                  }
                  beg_x = (double)crec.x;
                  beg_y = (double)crec.y;
                  if(transform( (double *)&beg_x, (double *)&beg_y, 0L, 8L))
                  {   message("1st transform failed");
                     return(-1);
                  }
                  if( first_time )
                  {   sprintf(p_cmd, "PA%d,%d;PD;",
                         (long)beg_x, (long)beg_y);
                     first_time = 0;
                  }
                  else
                      sprintf(p_cmd, "PA%d,%d;",
                         (long)beg_x, (long)beg_y);
   
                  if( put_plot(p_cmd, fd, dev) == -1 )
                  {   message("Cannot send to plotter.  Bye.");
                     return(-1);
                  }
               }
            }
   
					/**** plot labels as necessary ****/
         	if( orec.i12[4] & penmask[i] )
         	{  if(plot_label(dev, &orec, j, ltype, sizen))
               	message("no label produced");
         	}

            strcpy(p_cmd, "PU;");
            if( put_plot(p_cmd, fd, dev) == -1 )
            {   message("Cannot send to plotter. Bye.");
               return(-1);
            }
         }
      }
   }

      /*** feed paper to next position ***/
   sprintf(p_cmd, "FR;");
   put_plot(p_cmd, fd, dev);

      /*** free ram & close file/device ***/
   free_ltp(rec_plotted);
   closef(fd);
   if( dev == 1 || dev == 2 )   /* plotter was used & closed */
      fd_plot = -1;


   return(0);
}
/*****************************************************************************
*    plot_xhair                                                              *
*             This function will generate the HPGL commands to plot a cross- *
*             hair.  The finished command is sent to the plotter.            *
*             Parameters:  dv- device to write to                            *
*                          x - x coordinate for center of xhair              *
*                          y - y coordinate for center of xhair              *
*                          ht - height of xhair in plotter units from center *
*                               of xhair.                                    *
*                          wt - width of xhair in plotter units from center  *
*                               of xhair.                                    *
*             Returns 0 if all went well, else returns -1.                   *
*****************************************************************************/

long plot_xhair( dv, x, y, ht, wd )
   long dv;
   double x, y;
   long   ht, wd;
{
   double xbeg, xend, ybeg, yend;
   char   p_cmd[256];

/*** Draw corner markers  ***/
   ybeg = y + ht;
   yend = y - ht;
   sprintf(p_cmd, "PU;PA%d,%d;PD;PA%d,%d;", (long)x, (long)ybeg, 
      (long)x, (long)yend);
   if( put_plot(p_cmd, fd, dv) == -1 )
   {   message("PLOTM:  Cannot send to plotter.  Bye.");
      return(-1);
   }

   xbeg = x + wd;
   xend = x - wd;
   sprintf(p_cmd, "PU;PA%d,%d;PD;PA%d,%d;PU;", (long)xbeg, (long)y,
         (long)xend, (long)y);
   if( put_plot(p_cmd, fd, dv) == -1 )
   {   message("PLOTM:  Cannot send to plotter.  Bye.");
      return(-1);
   }
   return(0);
}

/*****************************************************************************
*    plot_label                                                              *
*             This function will generate the HPGL commands to plot a        *
*             label. The finished command is sent to the plotter.            *
*             Parameters:  dv     - device to write to                       *
*                          optr   - pointer to current objects record        *
*                          recnum - record number of current objects record  *
*                          ltype  - label user desires                       *
*                          lsize  - height of label in centimeters           *
*****************************************************************************/

long plot_label( dv, optr, recnum, ltype, lsize ) 
   long dv;
   OBJECTS *optr;
   long     recnum;
   long     ltype;
   double   lsize;
{
   double    width;
   double    tmpx, tmpy;
   long      i;
   char      label[32];
   char      p_cmd[STRLIST];
   char      p_tmp[STRLIST];
   ATTRIBUTES arec;
   COORD_LIST crec;

/*** If label size present, calculate ratio of width to height.  If no ***/
/*** size is present, use default settings of .285 wt and .375 ht.     ***/
   if( lsize != 0.0 )
   {   width = 0.76 * lsize;         /* .285/.375 = .76  */
      sprintf(p_cmd, "SI%f,%f;", width, lsize );
   }
   else
      strcpy( p_cmd, "SI;" );

   if( put_plot(p_cmd, fd, dv) == -1)
   {   message("PLOT_LTP: Cannot write to plotter. Bye.");
      return(-1);
   }

/*** Convert raster coordinates into plotter units!       ***/
   tmpx = optr->f4[0];
   tmpy = optr->f4[1];
   transform( &tmpx, &tmpy, 0, 8 );
      
/*** Generate label and plot it.                          ***/

   switch( ltype )
   {   
      case 0:          /** No label desired **/
         return(0);
      case 1:          /** Record num used for label **/
         sprintf( p_cmd, "PU;PA%d,%d;LO15;LB%d\03;PU;",
         (long)tmpx, (long)tmpy, recnum );
         sprintf(label, "%d", recnum);
         break;
      case 2:          /** Serial num used for label **/
         sprintf( p_cmd, "PU;PA%d,%d;LO15;LB%d\03;PU;",
         (long)tmpx, (long)tmpy, optr->i12[1]);
         sprintf(label, "%d", optr->i12[1]);
         break;
      case 3:          /** First attr used for label **/
         if( optr->i12[2] == 0 )
            return(0);
         get_attr(&arec, optr->i12[2]);
         sprintf( p_cmd, "PU;PA%d,%d;LO15;LB%s\03;PU;",
         (long)tmpx, (long)tmpy, arec.string);
         sprintf(label, "%s", arec.string);
         break;
      case 4:          /** Concat all attr for label **/
         for(get_attr(&arec, optr->i12[2]); arec.next;
               get_attr(&arec, arec.next) );
            strcat( p_tmp, arec.string );
         sprintf( p_cmd, "PU;PA%d,%d;LO15;LB%s\03;PU;",
         (long)tmpx, (long)tmpy, p_tmp);
         sprintf(label, "%s", p_tmp);
         break;
      default:          /** Should not be here! **/
         return(-1);
   }

   if( put_plot(p_cmd, fd, dv) == -1 )
   {   message("PLOT_LTP: Cannot write to plotter. Bye.");
      return(-1);
   }

/*** Label the secondary centroids (if present) ***/
   if(optr->i12[3])
   {   for( i = optr->i12[3]; i <= optr->i12[11]; i++ )
      {   if(get_coordlst( &crec, i ))
         {   message("PLOT_LTP:  Cannot get coordlist. aborting");
            return(-1);
         }
         tmpx = crec.x;
         tmpy = crec.y;
         transform( &tmpx, &tmpy, 0, 8 );
         sprintf( p_cmd, "PU;PA%d,%d;LO15;LB%s\03;PU;",
               (long)tmpx, (long)tmpy, label);
         if( put_plot(p_cmd, fd, dv) == -1 )
         {   message("PLOT_LTP: Cannot write to plotter. Bye.");
            return(-1);
         }
      }
   }
      
   return(0);
}
