/*******************************************************************************
*	raw_print	(see manual, Command Ref., User Functions)	<fc>   *
*			Description:  Prints the entire named raw data file    *
*			or specified portion thereof.                          *
*			Parameters:  Raw data file type and beginning/ending   *
*				     record numbers. A flag for crt display    *
*				     is passed too.  If true, no print will    *
*				     occur, just a display on the crt.         *
*									       *
*			Returns zero upon normal end-of-job (EOJ).             *
*                      CEJ        3/7/89 - 5/8/89                              *
*******************************************************************************/

#include "../../incl/globals.h" 
#include "../../incl/in.h"
#include <errno.h>
#include <string.h>

long history_prt();
long header_prt();
long diary_prt();
long errors_prt();
long job_sta_prt();
long registr_prt();
long objects_prt();
long seg_lis_prt();
long coord_l_prt();
long attribu_prt();
long index_a_prt();
long index_l_prt();
long v_attri_prt();
long v_objec_prt();
long v_coord_prt();
long cmd_line_hist();
long open_files();
long malloc_state();

/***  Static globals  ***/
	static char tmp_str[96];
	static long disp_on_crt;

long raw_print( choice, beg_rec, end_rec, disp_flag )
	long choice;
	long beg_rec;
	long end_rec;
	long disp_flag;
{
/*  Locals  */
	static long (*prt_ptr[])() = { attribu_prt, coord_l_prt, diary_prt, 
		errors_prt, header_prt, history_prt, index_a_prt, index_l_prt,
		job_sta_prt, objects_prt, registr_prt, seg_lis_prt, v_attri_prt,
		v_coord_prt, v_objec_prt, cmd_line_hist, open_files, malloc_state};

/**  check for map acquired if printing map files **/
	if((choice < 16)  &&  !job_stats[0])
	{	message("You must CREATE or ACQUIRE a map befor raw_print'ing map data files.");
		return(1);
	}

/**  Call the appropriate print function based upon choice  **/
/**  Use choice - 1 to compensate for 0th array entry       **/

	disp_on_crt = disp_flag;	/***  Set global display flag ***/

	if( (*prt_ptr[choice - 1])(beg_rec, end_rec) )
		return(1);
	else 
		return(0);
}

/*******************************************************************************
*       attribu_prt                                                            *
*                       This program prints the attribute  file for the        *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long attribu_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	int  tmp_fd;
	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	long rec_no;
	char tmp_buf[1024];
	ATTRIBUTES tmp_attr;
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 1 )
		rec0 = 1;
	if( !recn || recn >= job_stats[13] )
		recn = job_stats[13] - 1;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create attribute tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open attribute file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "\t\tATTRIBUTE raw print\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\tRECORD #\t    NEXT\
ATTRIBUTE RECORD #\tATTRIBUTE DESCRIPTION\n\n\n");
	writea( tmp_buf, tmp_fd );

/**  Read the specified number of records  **/
	for( rec_no = rec0; rec_no <= recn; rec_no++ )
	{	if( get_attr( &tmp_attr, rec_no) )
		{	append_msg("  Cannot get attribute record");
			closef(tmp_fd);
			return(1);
		}
		sprintf(tmp_buf, "\t%7d\t%10d\t%32s\n", rec_no, 
							tmp_attr.next, tmp_attr.string );
		writea(tmp_buf, tmp_fd);
	}

	closef(tmp_fd);
	if( disp_on_crt )
		more( tmp_name );
	else
		print_file( tmp_name );
	return(0);
}

/*******************************************************************************
*       coord_l_prt                                                            *
*                       This program prints the coord_list file for the        *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long coord_l_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	int  tmp_fd;
	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	long rec_no;
	char tmp_buf[1024];
	COORD_LIST tmp_coord;
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 0 )
		rec0 = 0;
	if( !recn || recn >= job_stats[11] )
		recn = job_stats[11] - 1;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create coordlist tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open coordlist tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "COORD_LIST raw print\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "RECORD #    X       Y\n");
	writea( tmp_buf, tmp_fd );

/**  Read the specified number of records  **/
	for( rec_no = rec0; rec_no <= recn; rec_no++ )
	{	if( get_coordlst( &tmp_coord, rec_no) )
		{	append_msg("  Cannot read coordlist file.");
			closef(tmp_fd);
			return(1);
		}
		sprintf(tmp_buf,"%7d%10.2f%10.2f\n",
							rec_no,tmp_coord.x,tmp_coord.y );
	    writea( tmp_buf, tmp_fd );
	}

/**  Free up malloc'ed space  **/
	closef(tmp_fd);

	if( disp_on_crt )
		more( tmp_name );
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       diary_prt                                                              *
*                       This program prints the diary   file of the current    *
*                       map in a semi-formatted report.  The entire file is    *
*                       printed.                                               *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long diary_prt()
{
/***  Locals   ***/

	char diary_path[512];

/** Print the entire file  **/

	sprintf(diary_path, "%s/diary", temp_map_path );
	if( disp_on_crt )
		more( diary_path );
	else
		print_file( diary_path);

	return(0);
}

/*******************************************************************************
*       errors_prt                                                             *
*                       This program prints the  errors file of the current    *
*                       map in a formatted report.                             *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long errors_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	int tmp_fd;
	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	long rec_no;
	char tmp_buf[1024];
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 0 )
		rec0 = 0;
	if( !recn || recn >= job_stats[14] )
		recn = job_stats[14] - 1;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create errors tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open errors file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "\t\tERRORS raw print\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\n\n\t\tR\tX\tY\tE\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\t\tE\t \t \tR\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\t\tC\tC\tC\tR\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\t\tO\tO\tO\tO\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\t\tR\tO\tO\tR\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\t\tD\tR\tR\t \n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\t\t \tD\tD\tC\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\t\t \t \t \tO\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\t\t \t \t \tD\n");
	writea( tmp_buf, tmp_fd );
	sprintf(tmp_buf, "\t\t \t \t \tE\n\n\n");
	writea( tmp_buf, tmp_fd );

/**  Read the specified number of records and write to tmp file **/
	for( rec_no = rec0; rec_no <= recn; rec_no++ )
	{	if( get_errors( tmp_buf, rec_no) )
		{	append_msg("  Cannot read errors file.");
			closef(tmp_fd);
			return(1);
		}
		writea( tmp_buf, tmp_fd );
	}

	closef(tmp_fd);

	if( disp_on_crt )
		more( tmp_name );
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       header_prt                                                             *
*                       This program prints the header  file of the current    *
*                       map in a semi-formatted report.  The entire file is    *
*                       printed.                                               *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long header_prt()
{
/***  Locals   ***/

	char head_path[512];

/** Print the entire file  **/

	sprintf(head_path, "%s/header", temp_map_path );
	if( disp_on_crt )
		more( head_path );
	else
		print_file( head_path);
	return(0);

}

/*******************************************************************************
*       history_prt                                                            *
*                       This program prints the history file of the current    *
*                       map in a semi-formatted report.                        *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long history_prt()
{
/***  Locals   ***/
	long i;
	int  tmp_fd;
	char tmp_name[L_tmpnam];
	char string[256];
	extern int errno;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create history tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open history file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Check the history strings.  If null, do not write to temp file  **/
	for( i = 0; i < 512; i++)
	{	if( strlen(history[i]) )
		{	sprintf(string,"%s\n",history[i]);
			writea( string, tmp_fd);
		}
	}

	closef(tmp_fd);

/**  Print the entire history file  **/

	if( disp_on_crt )
		more( tmp_name );
	else
		print_file( tmp_name);
	return(0);
}

/*******************************************************************************
*       index_a_prt                                                            *
*                       This program prints the index array file for the       *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long index_a_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	long i, j;
	long rows, cols;
	int  tmp_fd;
	char tmp_buf[1024];
	char tmp_name[L_tmpnam];
	extern int errno;

/***  Calculate the number of rows and columns in index_array ***/
	rows = (job_stats[2] >> 5);
	cols = (job_stats[1] >> 5);

	if( rec0 < 0 )
		rec0 = 0;
	if( recn > rows )
		recn = rows;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create index_array tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open index_array file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "INDEX ARRAY raw print\n\n\n");
	writea(tmp_buf, tmp_fd);

/**  Read in the index array elements and write them to temp file **/
	sprintf(tmp_buf,"  col x0  col x1  col x2  col x3  col x4  col x5  col x6  col x7  col x8  col x9\n");
	writea(tmp_buf, tmp_fd);
	for(i = recn; i>= rec0 ; i--)
	{  sprintf( tmp_buf, "row [%d]:\n", i);
		writea(tmp_buf, tmp_fd);
		for(j = 0; j <= cols; j++)
		{	sprintf(tmp_buf,"%8d", index_array[i][j] );
			writea(tmp_buf, tmp_fd);
			if((i % 10) == 9)          /**** add linefeed every 80 char ***/
			{	sprintf(tmp_buf,"\n");
				writea(tmp_buf, tmp_fd);
			}
		}
		if(i % 10)                   /** add ending linefeed if necessary **/
		{	sprintf(tmp_buf,"\n");
			writea(tmp_buf, tmp_fd);
		}
	}

	closef(tmp_fd);

	if( disp_on_crt )
		more( tmp_name);
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       index_l_prt                                                            *
*                       This program prints the index list file for the        *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long index_l_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	int  tmp_fd;
	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	long rec_no;
	char tmp_buf[1024];
	INDEX_LIST tmp_indexl;
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 1 )
		rec0 = 1;
	if( !recn || recn >= job_stats[15] )
		recn = job_stats[15] - 1;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create index_list tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open index_list file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "INDEX_LIST raw print\n\n");
	writea(tmp_buf, tmp_fd);
	sprintf(tmp_buf,"       rec      type       obj        v1        v2      next\n");
	writea(tmp_buf, tmp_fd);

/**  Read the specified number of records  **/
	for( rec_no = rec0; rec_no <= recn; rec_no++ )
	{	if( get_indexl( &tmp_indexl, rec_no) )
			return(1);
		sprintf(tmp_buf, "%10d%10d%10d%10d%10d%10d\n",
		rec_no, tmp_indexl.type, tmp_indexl.obj_rec, tmp_indexl.v1, tmp_indexl.v2,
		tmp_indexl.next);
		writea(tmp_buf, tmp_fd);
	}

	closef(tmp_fd);
	if( disp_on_crt )
		more( tmp_name);
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       job_sta_prt                                                            *
*                       This program prints the  job_stats file for the        *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long job_sta_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	long i;
	char path[512];
	int tmp_fd;
	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	char tmp_buf[1024];
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 0 )
		rec0 = 0;
	if( !recn || recn >= 100 )
		recn = 99;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create job_stats tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open job_stats file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "JOB STATS raw print\n");
	writea(tmp_buf, tmp_fd);
	sprintf(tmp_buf, "\nRECORD\t\t\t\tVALUE\n\n\n");
	writea(tmp_buf, tmp_fd);

/**  Read in the array elements from job_stats  **/
	sprintf(path, "%s/job_stats", temp_map_path);
	for(i = rec0; i <= recn; i++)
	{	sprintf(tmp_buf, "job_stats[%d]\t\t%d\n", i, job_stats[i] );
		writea(tmp_buf, tmp_fd);
	}

	closef(tmp_fd);
	if( disp_on_crt )
		more( tmp_name);
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       objects_prt                                                            *
*                       This program prints the  objects   file for the        *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long objects_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	int tmp_fd;
	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	long rec_no;
	char tmp_buf[1024];
	OBJECTS tmp_obj;
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 0 )
		rec0 = 0;
	if( !recn || recn >= job_stats[10] )
		recn = job_stats[10] -1;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create objects tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open objects file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}


/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "\t\tOBJECTS raw print\n\n\n");
	writea(tmp_buf, tmp_fd);
	sprintf(tmp_buf, "REC#   LONG 0   LONG 2   LONG 4   LONG 6   LONG 8   LONG 10   FLOAT 0    FLOAT 2\n");
	writea(tmp_buf, tmp_fd);
	 sprintf(tmp_buf, "        LONG 1   LONG 3   LONG 5   LONG 7   LONG 9   LONG 11  FLOAT 1    FLOAT 3\n");
	writea(tmp_buf, tmp_fd);

/**  Read the specified number of records  **/
	for( rec_no = rec0; rec_no <= recn; rec_no++ )
	{	if( get_objects( &tmp_obj, rec_no) )
		{	strcpy(tmp_str, "  Cannot read objects file.");
			message(tmp_str);
			return(1);
		}
		sprintf(tmp_buf, "%7d%3d %7d %7u %7d %7d%7d%10.2f%10.2f\n", 
			rec_no,tmp_obj.i12[0], tmp_obj.i12[2],(unsigned long)tmp_obj.i12[4],
				tmp_obj.i12[6], tmp_obj.i12[8], tmp_obj.i12[10], 
					tmp_obj.f4[0], tmp_obj.f4[2] );
		writea(tmp_buf, tmp_fd);
		
		sprintf(tmp_buf, "      %7d %7d %7d%7d%7d%7d%10.2f%10.2f\n\n", 
			tmp_obj.i12[1], tmp_obj.i12[3], tmp_obj.i12[5], tmp_obj.i12[7], 
				tmp_obj.i12[9], tmp_obj.i12[11],tmp_obj.f4[1], tmp_obj.f4[3] );
		writea(tmp_buf, tmp_fd);
	}

	closef(tmp_fd);
	if( disp_on_crt )
		more( tmp_name);
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       registr_prt                                                            *
*                       This program prints the  register  file for the        *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long registr_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	long i;
	char rpath[512];
	int  tmp_fd;
	char tmp_buf[1024];
	char tmp_name[L_tmpnam];
	extern int errno;
	static char reg_type[30][16] = { "raster (pix)","geodetic    ",
		"utm (meters)","st pln (ft) ","digitizer 1 ","digitizer 2 ",
		"map inches  ","last dig    ","temp        ","temp        ",
		"user 10     ","user 11     ","user 12     ","user 13     ",
		"user 14     ","user 15     ","user 16     ","user 17     ",
		"user 18     ","user 19     "  };
	double  s[4];
	long j, d[4], m[4];

/*  Get register info from disk file  */
/***  Check parms for accuracy  ***/

	if( rec0 < 0 )
		rec0 = 0;
	if( !recn || recn >= 29 )
		recn = 28;		/* 29 records are 0 - 28! */

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create register tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open temporary file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Write headers to temp file  **/
	/* Calculate the number of feet per pixel */
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "REGISTRATION raw print\n\n");
	writea(tmp_buf, tmp_fd);
	get_units();		/* set up the unit_factor global array */
	if( unit_factor[2][0] )
	{	sprintf(tmp_buf, "- 1 pixel  =  %.3f feet -\n\n", unit_factor[2][0]);
		writea(tmp_buf, tmp_fd);
	}
	sprintf( tmp_buf, "REGISTRATION TYPE\t CORNER (x, y) COORDINATE PAIRS\n");
	writea(tmp_buf, tmp_fd);
	sprintf( tmp_buf, "                       SW              NW              NE              SE\n\n");
	writea(tmp_buf, tmp_fd);

/**  Read in the index array elements and write them to temp file **/
	for(i = rec0; i <= recn; i++)
	{		/**** display geodetics different ***/
		if(i == 1)
		{	for(j=0; j<4; j++)
			{	dms2(corner[i][j][0],&d[j],&m[j],&s[j]);
				if(s[j] > 59.9995)
				{	s[j] = 0.0;
					m[j]++;
				}
				if(m[j] == 60)
				{	m[j] = 0;
					d[j]++;
				}
			}
			sprintf(tmp_buf, "%s   %3d/%2d/%6.3fW  %3d/%2d/%6.3fW  %3d/%2d/%6.3fW  %3d/%2d/%6.3fW\n",reg_type[i],d[0],m[0],s[0],d[1],m[1],s[1],d[2],m[2],s[2],d[3],m[3],s[3]);
			writea(tmp_buf, tmp_fd);

			for(j=0; j<4; j++)
			{	dms2(corner[i][j][1],&d[j],&m[j],&s[j]);
				if(s[j] > 59.9999)
				{	s[j] = 0.0;
					m[j]++;
				}
				if(m[j] == 60)
				{	m[j] = 0;
					d[j]++;
				}
			}
			sprintf(tmp_buf, "               %3d/%2d/%6.3fN  %3d/%2d/%6.3fN  %3d/%2d/%6.3fN  %3d/%2d/%6.3fN\n\n",d[0],m[0],s[0],d[1],m[1],s[1],d[2],m[2],s[2],d[3],m[3],s[3]);
			writea(tmp_buf, tmp_fd);
		}
		else if((i < 4) && (i>1))
		{	sprintf(tmp_buf, "%s %15.3fE%15.3fE%15.3fE%15.3fE\n",reg_type[i],
			corner[i][0][0], corner[i][1][0], corner[i][2][0], corner[i][3][0] );
			writea(tmp_buf, tmp_fd);
			sprintf(tmp_buf, "             %15.3fN%15.3fN%15.3fN%15.3fN\n\n",
			corner[i][0][1], corner[i][1][1], corner[i][2][1], corner[i][3][1] );
			writea(tmp_buf, tmp_fd);
		}
		else
		{	sprintf(tmp_buf, "%s %15.3fx%15.3fx%15.3fx%15.3fx\n",reg_type[i],
			corner[i][0][0], corner[i][1][0], corner[i][2][0], corner[i][3][0] );
			writea(tmp_buf, tmp_fd);
			sprintf(tmp_buf, "             %15.3fy%15.3fy%15.3fy%15.3fy\n\n",
			corner[i][0][1], corner[i][1][1], corner[i][2][1], corner[i][3][1] );
			writea(tmp_buf, tmp_fd);
		}
	}

	closef( tmp_fd );
	if( disp_on_crt )
		more( tmp_name);
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       seg_lis_prt                                                            *
*                       This program prints the seg_list   file for the        *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long seg_lis_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/

	long i;
	long seg_list;
	int  tmp_fd;
	char tmp_buf[1024];
	char tmp_name[L_tmpnam];
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 0 )
		rec0 = 0;
	if( !recn || recn >= job_stats[12] )
		recn = job_stats[12] - 1;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create seg_list tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open seg_list file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "    SEG_LIST raw print\n\n");
	writea(tmp_buf, tmp_fd);
	sprintf(tmp_buf, "RECORD #    SEGMENT\n\n");
	writea(tmp_buf, tmp_fd);

/**  Read in the index array elements and write them to temp file **/

	for(i = rec0; i <= recn; i++)
	{	if( get_seglist( &seg_list, i))
		{	append_msg("  Cannot read seg_list file.");
			closef(tmp_fd);
			return(1);
		}
		sprintf(tmp_buf,"%10d    %+10d\n", i, seg_list );
		writea(tmp_buf, tmp_fd);
	}

	closef(tmp_fd);
	if( disp_on_crt )
		more( tmp_name);
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       v_attri_prt                                                            *
*                       This program prints the v_attribute file for the       *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long v_attri_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	int  tmp_fd;
	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	long rec_no;
	char tmp_buf[1024];
	ATTRIBUTES tmp_vattr;
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 1 )
		rec0 = 1;
	if( !recn || recn >= job_stats[16] )
		recn = job_stats[16] - 1;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create v_attribute tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open v_attribute file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "\t\tV_ATTRIBUTE raw print\n");
	writea(tmp_buf, tmp_fd);
	sprintf(tmp_buf, "\tRECORD #\t    NEXT\
ATTRIBUTE RECORD #\tATTRIBUTE DESCRIPTION\n\n\n");
	writea(tmp_buf, tmp_fd);

/**  Read the specified number of records  **/
	for( rec_no = rec0; rec_no <= recn; rec_no++ )
	{	if( get_v_attr( &tmp_vattr, rec_no) )
		{	append_msg("  Cannot read v_attributes file.");
			closef(tmp_fd);
			return(1);
		}
		sprintf(tmp_buf, "\t%7d\t%10d\t%32s\n", rec_no, 
					tmp_vattr.next, tmp_vattr.string );
		writea(tmp_buf, tmp_fd);
	}

	closef(tmp_fd);
	if( disp_on_crt )
		more( tmp_name);
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       v_coord_prt                                                            *
*                       This program prints the v_coordlist file for the       *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long v_coord_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	int  tmp_fd;
	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	long rec_no;
	char tmp_buf[1024];
	COORD_LIST tmp_vcoord;
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 0 )
		rec0 = 0;
	if( !recn || recn >= job_stats[18] )
		recn = job_stats[18] - 1;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create v_coordlist tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open v_coord_list file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "\t\tCOORD_LIST raw print\n");
	writea(tmp_buf, tmp_fd);
	sprintf(tmp_buf, "\tRECORD #\t    X COORDINATE\
\t\tY COORDINATE\n\n\n");
	writea(tmp_buf, tmp_fd);

/**  Read the specified number of records  **/
	for( rec_no = rec0; rec_no <= recn; rec_no++ )
	{	if( get_v_coord( &tmp_vcoord, rec_no) )
		{	append_msg("  Cannot read v_coordlist file.");
			closef(tmp_fd);
			return(1);
		}
		sprintf(tmp_buf, "\t%7d\t%9.6f\t\t%9.6f\n", rec_no, 
					tmp_vcoord.x, tmp_vcoord.y );
		writea(tmp_buf, tmp_fd);
	}

	closef(tmp_fd);
	if( disp_on_crt )
		more( tmp_name);
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       v_objec_prt                                                            *
*                       This program prints the v_objects   file for the       *
*                       current map in a formatted report.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long v_objec_prt(rec0, recn)
	long rec0, recn;
{
/***  Locals   ***/
	int tmp_fd;
	char tmp_name[L_tmpnam];		/*  Holds temp file name  */
	long rec_no;
	char tmp_buf[1024];
	V_OBJECTS tmp_obj;
	extern int errno;

/***  Ensure parms are valid  ***/
	if( rec0 < 0 )
		rec0 = 0;
	if( !recn || recn >= job_stats[17] )
		recn = job_stats[17] - 1;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create v_objects tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open v_objects file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Write headers to temp file  **/
	writea(std_header(tmp_buf),tmp_fd);
	sprintf(tmp_buf, "\t\tV_OBJECTS raw print\n\n");
	writea(tmp_buf, tmp_fd);
	sprintf(tmp_buf, 
	"REC#       LONG 1       LONG 2       LONG 3       LONG 4 \n\n\n");
	writea(tmp_buf, tmp_fd);

/**  Read the specified number of records  **/
	for( rec_no = rec0; rec_no <= recn; rec_no++ )
	{	if( get_vobjects( &tmp_obj, rec_no) )
		{	append_msg("  Cannot read v_objects file.");
			closef(tmp_fd);
			return(1);
		}
		sprintf(tmp_buf, "%d %10d %10d %10d %10d\n", rec_no, 
			tmp_obj.type, tmp_obj.v1, tmp_obj.v2, tmp_obj.atr);
		writea(tmp_buf, tmp_fd);
	}
		
	closef(tmp_fd);
	if( disp_on_crt )
		more( tmp_name);
	else
		print_file( tmp_name );

	return(0);
}

/*******************************************************************************
*       cmd_line_hist                                                          *
*                       This program prints the last 50 command lines          *
*                       displayed on the command line portion of the screen    *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long cmd_line_hist()
{
/***  Locals   ***/
	long i,j;
	int  tmp_fd;
	char tmp_name[L_tmpnam];
	char string[256];
	extern int errno;

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create cmd_line_hist tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open cmd_line_hist file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Check the cmd_history strings.  If null, do not write to temp file  **/
	for( i = 0, j=(t4_indx + 1) & 127; i < 128; i++, j = (j+1) & 127)
	{	if( strlen(text_buf[j]) )
		{	sprintf(string,"%s\n",text_buf[j]);
			writea( string, tmp_fd);
		}
	}
	closef(tmp_fd);

/**  Print the entire cmd_history file  **/

	if( disp_on_crt )
		more( tmp_name );
	else
		print_file( tmp_name);
	return(0);
}


/*******************************************************************************
*       open_files                                                             *
*                       This program a list of all files currently open.       *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long open_files()
{
/***  Locals   ***/
	long i;
	long count;
	int  tmp_fd;
	char tmp_name[L_tmpnam];
	char string[256];
	char filenames[60][STRLIST];
	int fileptrs[60];

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create cmd_line_hist tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open cmd_line_hist file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

/**  Get list of currently open files **/
	count = openfiles_list(filenames,fileptrs);
/**  write list to file **/
	if(count)
	{	for( i = 0; i<count; i++)
		{	sprintf(string,"fd=%d  %s\n",fileptrs[i],filenames[i]);
			writea( string, tmp_fd);
		}
	}
	else
	{	sprintf(string,"There are no files currently open.");
		writea( string, tmp_fd);
	}
	closef(tmp_fd);

/**  Print the entire open_files file  **/

	if( disp_on_crt )
		more( tmp_name );
	else
		print_file( tmp_name);
	return(0);
}


/*******************************************************************************
*       malloc_state                                                           *
*                       This program a list of all ptrs & sizes of malloc'd    *
*                       space.                                                 *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*******************************************************************************/

long malloc_state()
{
/***  Locals   ***/
	long i;
	long count;
	long size;
	int  tmp_fd;
	char tmp_name[L_tmpnam];
	char string[256];

 /**  Create and open temp file  **/
	tmpnam( tmp_name );
	if((tmp_fd = creatf( tmp_name )) == -1) 
	{	sprintf(tmp_str, "  Cannot create cmd_line_hist tempfile.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}
	if((tmp_fd = openf( tmp_name, WRITE )) == -1) 
	{	sprintf(tmp_str, "  Cannot open cmd_line_hist file.  Errno is %d.", errno);
		append_msg( tmp_str );
		return(1);
	}

		/**  Write the entire malloc_state file ***/
	count = 0;
	for(i=0; i<100; i++)
	{	if(mcontrol[i][0] != (char *) 0)
		{	size = mcontrol[i][1] - mcontrol[i][0];
			sprintf(string,"%2d.  mcontrol pointer = %10d,  size = %10d\n",count,mcontrol[i][0],size);
			writea( string, tmp_fd);
			count++;
		}
	}

	closef(tmp_fd);


	if( disp_on_crt )
		more( tmp_name );
	else
		print_file( tmp_name);
	return(0);
}
