/*******************************************************************************
*	redisp_map	(see manual, Command Ref., User Function)	<fc>   *
*			Recalculates the map resolution raster for display     *
*			and displays that resolution on the screen.            *
*                                                                              *
*			Modifies the displayed graphics screen and cursor      *
*			position and last cursor position variables in the     *
*			input.h include variables.                             *
*                                                                              *
*			Returns zero upon normal end-of-job (EOJ).             *
*******************************************************************************/

#include "../../incl/globals.h" 

long redisp_map()
{
/***  Locals   ***/
	long *mptr, *endptr;
	long max, row, word;
	long max_word;
	long width;
	long mrx, mry, mrchar;
	double sfactor;
	long bit;
	long mrbit;
	long on_bits, bits;
	long background, increment;

		/************** check for map *******************************/
	if(!job_stats[0])
	{	message("You must Create or Acquire a map befor redisplaying the raster.");
		return(1);
	}

/***  Let user know all is well ***/
	append_msg("...regenerating mini_raster.");

/***  check raster to see if it has '0' or '1' background ***/
	mptr = (long *) lraster[0];
	endptr = (long *) lraster[job_stats[2]];
	on_bits = 0;
	bits = 0;
	increment = 1 + (1 + job_stats[2] / 100) * ((job_stats[1] + 1) >> 5);
	for(; mptr < endptr; mptr += increment, bits += 32)
	{	if(*mptr)
		{	for(bit = 0; bit < 32; bit++)
				on_bits += ((*mptr) >> bit)  & 1;
		}
	}

	if((on_bits * 2) > bits)
		background = 1;
	else
		background = 0;


/***  Null out the mini_raster.  400 * 400 / 32   ***/
	mptr = (long *)(mini_raster[0]);
	endptr = mptr + 5000;
	if(background)
	{	for( ; mptr < endptr ; mptr++)
			*mptr = -1;
	}
	else
	{	for( ; mptr < endptr ; mptr++)
			*mptr = 0;
	}


/***  Calculate shrink factor   (raster -> mini_raster)  ***/
	max = (job_stats[1] > job_stats[2])  ? job_stats[1]+1 : job_stats[2]+1;

	if( max >= 400 )
		sfactor = 400.0 / max;
	else			/*  load raster directly into mini_ras  */
	{
		width = job_stats[1] >> 3;
		bitmap(width,job_stats[2]+1,width,50,craster[0],mini_raster[0]);
		return(0);
	}
		
/***  Compute the mini_raster  ***/
	
	max_word = job_stats[1] >> 5;
	if(background == 0)
	{	for( row = 0;  row <= job_stats[2];   row++ ) 
		{	mry = row * sfactor;
	    	for( word = 0; word <= max_word;  word++ )
			{	if( lraster[row][word] )	
		    	{	for( bit = 31; bit >= 0; bit-- )
					{	if( lraster[row][word] & (1 << bit) )
						{	mrx = sfactor * ((word << 5) | (7 & ~bit) | (bit & 24));
			    			mrchar = mrx >> 3;	/*  Convert x to chars  */
			    			mrbit = 7 & ~mrx;
							mini_raster[mry][mrchar] |= (1 << mrbit);
						}
					}
				}
			}
		}
	}
	else
	{	for( row = 0;  row <= job_stats[2];   row++ ) 
		{	mry = row * sfactor;
	    	for( word = 0; word <= max_word;  word++ )
			{	if( lraster[row][word] != -1)	
		    	{	for( bit = 31; bit >= 0; bit-- )
					{	if( (lraster[row][word] & (1 << bit)) == 0)
						{	mrx = sfactor * ((word << 5) | (7 & ~bit) | (bit & 24));
			    			mrchar = mrx >> 3;	/*  Convert x to chars  */
			    			mrbit = 7 & ~mrx;
							mini_raster[mry][mrchar] &= (255 ^ (1 << mrbit));
						}
					}
				}
			}
		}
	}


/***  Whew!  Redisplay the graphics, then all done.  ***/

	dispgraphics(200, 200, 0);
	append_msg(". . . done");

/***  Let the user know if this is a negative image ***/
	if(background)
	{	message("This seems to be a NEGATIVE image.");
		message("I recommend that you use the   negative_img   command.");
		sleep(2);
	}

	return(0);
}
