/*******************************************************************************
*    reduce_res      (see manual, Command Ref., User Functions)     <fc>       *
*           This command reduces the resolution of the raster by lumping       *
*           a n X n block of pixels into a single pixel.  These are            *
*           thresholded by the parameter t.                                    *
*                                                                              *
*           parameters:   long n;       linear reduction factor.               *
*                         long t;       threshold                              *
*                                                                              *
*           Globals Modified:  nearly all map data & globals.                  *
*                                                                              *
*           Value returned:   0=ok, 1=error                                    *
*           J.Dabritz  1/23/90                                                 *
*******************************************************************************/
#include "../../incl/globals.h"
#include "../../incl/in.h"

long reduce_res( n, t ) 
   long n,t;
{
   ERRORS error;
   OBJECTS object;
   COORD_LIST coordlst;
   V_OBJECTS vobjects;

   char string[STRLIST];

   double x,y;
   double r,r2;

   int  fd;

   long h, w;
   long i;
   long b,j,ix;
   long size;
   long *counter;
   long wwidth, new_wwidth;
   long new_row;
   long raster_size;


      /************** check for map **********************************/
   if(!job_stats[0])
   {  message("You must CREATE or ACQUIRE a map befor using the reduce_res command.");
      return(1);
   }

		/************* check for VIEW ONLY *****************************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}

   message("Checking parameters & setting up variables.");
      /************** check input parameters *************************/
   if(n < 2)
   {  message("Illegal reduction factor.  Reduction factor must be 2 or greater.");
      return(1);
   }
   if((t < 1)  ||  (t > n * n))
   {  message("Illegal threshold.  it must be >= 0 and <= reduction factor squared.");
      return(1);
   }

      /************** allocate counter for 1 new raster row **********/
   size = ((job_stats[1] + 1) / n + 1);
   counter = (long *) calloc_ltp(size,4);
   if(counter == (long *) 0)
   {  message("Cannot allocate RAM for reduce_res.  aborting.");
      return(1);
   }
   
      /************** reduce rasterb, if any *************************/
   if(israsterb())
   {     /************** update temporary raster file *******************/
      if(changes())
      {  message("updating the temporary raster file");
         disprstr_put();
      }

         /******* get rasterb **********/
      message("reading the braster (original scan) file.");
      disprstr_get(1,0);

      size = ((job_stats[1] + 1) / n + 1);
      new_wwidth = (size + 31) / 32;
      wwidth = job_stats[1] >> 5;
      new_row = 0;
      for(i=0; i<=job_stats[2]; i++)
      {  for(j=0; j<=wwidth; j++)
         {  if(lraster[i][j])
            {  for(b=0; b<32; b++)
               {  if(lraster[i][j] & (1<<b))
                  {  ix = (j << 5) | (7 & ~b) | (b & 24);
                     counter[ix/n]++;                        
                  }
               }
            }
         }
         if(((i % n) == (n - 1))  ||  (i == job_stats[2]))
         {  if(!(new_row % 100))
            {  sprintf(string,"computing new b raster, row %d.",new_row);
               message(string);
            }
   
               /******** zero out old raster row ********************/
            for(j=0; j<new_wwidth; j++)
               lraster[new_row][j] = 0;
   
               /******** read counters & set new raster row *********/
               /******** zero out counters **************************/
            for(ix=0; ix<size; ix++)
            {  if(counter[ix] >= t)
               {  w = ix >> 5;
                  b = (7 & ~ix) | (ix & 24);
                  lraster[new_row][w] |= (1 << b);
               }
               counter[ix] = 0;
            }
               /******** increment new row **************************/
            new_row++;
         }
      }
   
      message("Writing new b raster file.");
   
         /************** store new sized raster *************************/
      sprintf(string, "%s/rasterb", temp_map_path);
      if((fd = openf(string,WRITE | O_TRUNC)) <= 0)
      {  message("!-> Cannot open raster temp file.  Aborting.");
         map_name[0] = job_stats[0] = 0;
         free_ltp((char *) counter);
         free_malloc();
         return(1);
      }
      size = new_wwidth * 4;
      for(i=0; i<new_row; i++)
      {  if( writeb((char *) lraster[i], size , 1, fd) == -1)
         {  message("!-> Cannot write all raster recs to temp file.  Aborting.");
            closef(fd);
            map_name[0] = job_stats[0] = 0;
            free_ltp((char *) counter);
            free_malloc();
            return(1);
         }
      }
   
      /***  Close file  ***/
      closef(fd);

         /******* change job_stats [76] for braster *******/
      job_stats[76] = (new_wwidth) * 32 * new_row / 8;

         /******* get raster **********/
      message("reading the main raser file."); 
      disprstr_get(0,0);
   }

      /************** reduce raster **********************************/
   size = ((job_stats[1] + 1) / n + 1);
   new_wwidth = (size + 31) / 32;
   wwidth = job_stats[1] >> 5;
   new_row = 0;
   for(i=0; i<=job_stats[2]; i++)
   {  for(j=0; j<=wwidth; j++)
      {  if(lraster[i][j])
         {  for(b=0; b<32; b++)
            {  if(lraster[i][j] & (1<<b))
               {  ix = (j << 5) | (7 & ~b) | (b & 24);
                  counter[ix/n]++;                        
               }
            }
         }
      }
      if(((i % n) == (n - 1))  ||  (i == job_stats[2]))
      {  if(!(new_row % 100))
         {  sprintf(string,"computing new raster, row %d.",new_row);
            message(string);
         }

            /******** zero out old raster row ********************/
         for(j=0; j<new_wwidth; j++)
            lraster[new_row][j] = 0;

            /******** read counters & set new raster row *********/
            /******** zero out counters **************************/
         for(ix=0; ix<size; ix++)
         {  if(counter[ix] >= t)
            {  w = ix >> 5;
               b = (7 & ~ix) | (ix & 24);
               lraster[new_row][w] |= (1 << b);
            }
            counter[ix] = 0;
         }
            /******** increment new row **************************/
         new_row++;
      }
   }

   message("Writing new raster files.");

      /************** store new sized raster *************************/
   sprintf(string, "%s/raster", temp_map_path);
   if((fd = openf(string,WRITE | O_TRUNC)) <= 0)
   {  message("!-> Cannot open raster temp file.  Aborting.");
      map_name[0] = job_stats[0] = 0;
      free_ltp((char *) counter);
      free_malloc();
      return(1);
   }
   size = new_wwidth * 4;
   for(i=0; i<new_row; i++)
   {  if( writeb((char *) lraster[i], size , 1, fd) == -1)
      {  message("!-> Cannot write all raster recs to temp file.  Aborting.");
         closef(fd);
         map_name[0] = job_stats[0] = 0;
         free_ltp((char *) counter);
         free_malloc();
         return(1);
      }
   }

   /***  Close file  ***/
   closef(fd);

   /***  Free calloc'ed space ***/
   free_ltp((char *) counter);

      /*********** set up new raster size ****************************/
   job_stats[1] = (new_wwidth) * 32 - 1;
   job_stats[2] = new_row - 1;

      /*********** re-size raster  ***********************************/
      /***********  calculate sizes **************/
   h = job_stats[2] + 1;             /* height of raster in rows  */
   w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */
   raster_size = h*w;                /* size of raster in bytes   */
   
   size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster/r */

      /********  calculate & set global pointers to raster ****************/
   rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
   rasterm.recn = rasterm.rec0 + raster_size;
   craster = (unsigned char **)(rasterm.malloc);
   lraster = (unsigned long **) (rasterm.malloc + 4*h);
   for(i=0; i<h; i++)
   {  craster[i] = (unsigned char *)rasterm.rec0 + w*i;
      lraster[i] = (unsigned long *) (craster[i]);
   }

      /***********  read raster from file *********************/
   sprintf(string,"%s/raster",temp_map_path);
   fd = openf(string,READ);
   if(0 > fd)
   {  message("  <<  cannot open raster map file.");
      remove_temp();
      return(0);
   }
   if(0 >= readb(rasterm.rec0,raster_size,1,fd))
   {  message("  <<  cannot read raster map file.");
      closef(fd);
      remove_temp();
      return(0);
   }
   closef(fd);

      /*  set up graphics & display     */
   for(i=0; i<4; i++)
   {  view[i].lower_leftx = 0;
      view[i].lower_lefty = 0;
   }
   view[0].zoom_factor = mind(2,  (double) 1.0, 
         (double) (400.0/(maxl(2,job_stats[1]+1,job_stats[2]+1))));
   view[1].zoom_factor = 0.25;
   view[2].zoom_factor = 1.0;
   view[3].zoom_factor = 8.0;


      /*** calc  new sized index_array file     ***/ 
   h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
   w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
   size = (h*w + 4*h);               /* size to malloc            */

      /********  calculate & set global pointers to index_array ***********/
   index_arraym.rec0 = index_arraym.malloc;
   index_arraym.recn = (char *)(index_arraym.rec0 + h*w);
   for(i=0; i<h; i++)
      index_array[i] = (long *) (index_arraym.malloc + w*i);

      /******** zero out index array *******/
   for(i=0; i<h; i++)
   {  for(j=0; j<w; j++)
         index_array[i][j] = 0;
   }

      /*********** re-build all vector files & re-index map **********/
      /*********** convert all vector data . . . *******************/
      /*********** convert raster registration *********************/
   r = 1 / (double) n;
   r2 = r * r;
   for(i=0; i<4; i++)
   {  corner[0][i][0] *= r;
      corner[0][i][1] *= r;
   }

      /*********** convert coord_list file *************************/
   for(i=0; i<job_stats[11]; i++)
   {     /******** inform user *************/
      if(!(i % 1000))
      {  sprintf(string,"Updating coordinates in the Coord_list file, rec %d (of %d)",i,job_stats[11]);
         message(string);
      }
         /******** get record **************/
      if(get_coordlst(&coordlst,i))
         return(1);
         /******** transform coords *******/
      coordlst.x *= r;
      coordlst.y *= r;

         /******** put record **************/
      if(put_coordlst(&coordlst,i))
         return(1);
   }

      /*********** convert errors file *****************************/
   for(i=0; i<job_stats[14]; i++)
   {     /******** inform user *************/
      if(!(i % 500))
      {  sprintf(string,"Updating coordinates in the Errors file, rec %d",i);
         message(string);
      }
         /******** get record **************/
      if(get_errors(&error,i))
         return(1);
         /******** transform coords *******/
      error.x *= r;
      error.y *= r;

         /******** put record **************/
      if(put_errors(&error,i))
         return(1);
   }

      /*********** convert v_objects *******************************/
   for(i=0; i<job_stats[17]; i++)
   {     /******** inform user *************/
      if(!(i % 250))
      {  sprintf(string,"Updating coordinates in the V_objects file, rec %d",i);
         message(string);
      }
         /******** get record **************/
      if(get_vobjects(&vobjects,i))
         return(1);
         /******** check for map node data ********/
      if(vobjects.type == 00)
      {  x = vobjects.v1;
         y = vobjects.v2;
            /******** transform coords *******/
         x *= r;
         y *= r;
         vobjects.v1 = x + 0.5;
         vobjects.v2 = y + 0.5;
            /******** put record **************/
         if(put_vobjects(&vobjects,i))
            return(1);
      }
   }

      /*********** convert v_coordlist *****************************/
   for(i=0; i<job_stats[18]; i++)
   {     /******** inform user *************/
      if(!(i % 5000))
      {  sprintf(string,"Updating coordinates in the V_coordlist file, rec %d",i);
         message(string);
      }
         /******** get record **************/
      if(get_v_coord(&coordlst,i))
         return(1);
         /******** transform coords *******/
      coordlst.x *= r;
      coordlst.y *= r;
         /******** put record **************/
      if(put_v_coord(&coordlst,i))
         return(1);
   }

      /*********** convert objects file ****************************/
      /******( includes length & perimeter ) ***********************/
   for(i=0; i<job_stats[10]; i++)
   {     /******** inform user *************/
   if(!(i % 250))
   {  sprintf(string,"Updating coordinates in the Objects file, rec %d",i);
      message(string);
   }
         /******** get record **************/
      if(get_objects(&object,i))
         return(1);
         /******** transform coords *******/
      object.f4[0] *= r;
      object.f4[1] *= r;
      object.f4[2] *= r;
      object.f4[3] *= r2;
         /******** put record **************/
      if(put_objects(&object,i))
         return(1);
   }
   
      /*********** re-set node_lookup array ************************/
	if((node_lookup_ptr != (char *) 0)  && (job_stats[10]))
	{	for(i=0; i<=job_stats[2]; i++)
			node_lookup[i] = 0;
	}

      /*********** re-index map objects ****************************/
	job_stats[15] = 0;
   if(job_stats[10])
   {  if(index_map())
         return(1);
   }

      /*********** re-index vector objects *************************/
   if(job_stats[17])
   {  if(index_vobj(0))
         return(1);
   }

      /******* reacquire map *********/
	if(reacquire2())
		return(1);
            
   /*** Write diary record ***/
   sprintf(string, "reduced resolution by a factor of %d, threshold %d.",n,t);
   put_diary(string);
	message(string);

   /*** All done.  Return ***/
   return(0);
}
