/*******************************************************************************
*    reflect         (see manual, Command Ref., User Functions)     <fc>       *
*           Description:  Reflects raster map top-to-bottom, unless flag is    *
*           set to reflect right-to-left.                                      *
*           Parameter:  Reflection direction switch.  If nonzero, reflect on   *
*                       a horizontal plane, else reflect on vertical.          *
*           Globals Modified:  craster is manipulated.                         *
*                                                                              *
*           Returns zero upon normal end-of-job (EOJ), else returns -1.        *
*           C. Jonson  1/5/90                                                  *
*******************************************************************************/
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

long reflect( horiz )
	long horiz;
{
	int  fd;
	long h, w;
	long i;
	long numrows;
	long size;
	unsigned char rindx[256];
	unsigned char tmp;
	unsigned char *top, *end;
	char string[STRLIST];
	long create_braster;
	
		/************ check for map ************************************/
	if(!job_stats[0])
	{	message("You must CREATE or ACQUIRE a map befor using the reflect command.");
		return(-1);
	}

		/************* check for VIEW ONLY *****************************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(-1);
	}

		/************* check for braster *******************************/
	if(israsterb())
		create_braster = 1;
	else
		create_braster = 0;

/*** Set up variables to manipulate the raster ***/
	numrows = job_stats[2];
	reflect_indx(rindx);
	
/** 0 out obj file "first type" rec locators **/
	for(i=3; i<=7; i++)			
		job_stats[i] = 0;

	for(i=10; i<=18; i++)		/** 0 out rec counts for map files **/
	{	if((i==13)  ||  (i==15)  ||  (i==16))
			job_stats[i] = (job_stats[i])  ?    1 : 0;
		else
			job_stats[i] = 0;
	}
	job_stats[49] = 0;			/** 0 "vertex moved" flag **/
	job_stats[75] = 0;			/** 0 "next error" number **/
	if(job_stats[40] < job_stats[45])
	{	job_stats[45]++;
		job_stats[40] = job_stats[45];
	}

/*** If horiz is nonzero, reflect on horizontal plane, else vertical ***/
	if( horiz )
	{	for( i = 0; i < numrows; i++ )
		{	top = craster[i];				   /** Point to first byte in row */
			end = top + raster_bincrement - 1; /** Point to last byte in row **/
			for( ;end > top; end--, top++) 
			{	tmp = rindx[*end];
				*end = rindx[*top];
				*top = tmp;
			}
		}
	}
	else			 /*** Loop through craster and swap rows ***/
	{	top = craster[0];			/** Point to first row of raster **/
		end = craster[numrows];		/** Point to last row of raster **/
		for( ; end > top; end -= raster_bincrement, top += raster_bincrement )
		{	for( i = 0; i < raster_bincrement; i++ )
			{	tmp = *(end + i);
				*(end + i) = *(top + i);
				*(top + i) = tmp;
			}
		}
	}

	/***  Write new raster to temp file  *****/
	sprintf(string, "%s/raster", temp_map_path);
	if((fd = openf(string,WRITE | O_TRUNC)) <= 0)
	{	message("!-> Cannot open raster temp file.  Aborting.");
		map_name[0] = job_stats[0] = 0;
		return(1);
	}
	size = raster_bincrement * (job_stats[2] + 1);
	if( writeb(rasterm.rec0, size , 1, fd) == -1)
	{	message("!-> Cannot write all raster recs to temp file.  Bye.");
		closef(fd);
		map_name[0] = job_stats[0] = 0;
		free_malloc();
		return(1);
	}

	/***  Close file  ***/
	closef(fd);

	/***  Free calloc'ed space for index_array ***/
	free_ltp(index_arraym.malloc);
	index_arraym.malloc = (char *)0;

	/*** Store new sized index_array file     ***/ 
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = h*w;                       /* size to calloc            */

	/*** Allocate memory ***/
	index_arraym.malloc = (char *)calloc_ltp( size, sizeof(char) );
	if(index_arraym.malloc == (char *)0)
	{	message("!->  Cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
		map_name[0] = job_stats[0] = 0;
		free_malloc();
		return(1);
	}

	/***  Write index_array to file ***/
	sprintf(string,"%s/index_array",temp_map_path);
	if( (fd = openf(string,WRITE | O_TRUNC)) == -1)
	{	message("!->  Cannot open index_array map file.");
		map_name[0] = job_stats[0] = 0;
		free_malloc();
		return(0);
	}
	if( writeb(index_arraym.malloc,size,1,fd) == -1 )
	{	message("!->  Cannot write index_array map file.");
		closef(fd);
		map_name[0] = job_stats[0] = 0;
		free_malloc();
		return(0);
	}
	closef(fd);

	/***  Write job_stats to file ***/
	sprintf(string,"%s/job_stats",temp_map_path);
	if(put_jobstats(string))
	{	message("!-> Cannot write job_stats to temp file.  Aborting.");
		map_name[0] = job_stats[0] = 0;
		free_malloc();
		return(1);
	}

	/***  Un-register raster ***/
	for(i=0; i<4; i++)
		corner[0][i][0] = corner[0][i][1] = 0.0;

	/*** Write diary record ***/
	if( horiz )
		strcpy(string, "reflected raster on horizontal axis");
	else
		strcpy(string, "reflected raster on vertical axis");
	put_diary(string);


	/*** Reacquire map files ***/
	if(reacquire())
		return(1);

		/************* re-display if necessary *************************/
	if(!(batch_mode) && resolution_code)
		dispgraphics(0,0,0);

		/************* re-create braster *******************************/
	if(create_braster)
		brstr_create();

	/*** All done.  Return ***/
	return(0);
}
