/*******************************************************************************
*	register_ltp    (see manual, Command Ref., User Function)	        <fc>   *
*		Stores registration data for the map and for 'stick- up'               *
*		menus.  The coord_pairs parameter is an array of double[4][2]          *
*		to hold the x and y values of each corner to be registered.            *
*		The cnum parm determines what corners are set, 0 being SW to           *
*		4 being SE in clockwise fashion.  The num_to_reg parm will             *
*		control the number of corners registered, thus one to four             *
*		corners may be registered beginning from the cnum corner.              *
*		The action parm determines if printing or display is to occur.         *
*                                                                              *
*		Globals Modified:     corner                                           *
*		Returns 0 if all OK, else returns 1.                                   *
*		C. Jonson      6/89         (:set tabstop=4)                           *
*******************************************************************************/
#include "../../incl/globals.h"

long register_ltp( ctype, cnum, coord_pairs, num_to_reg, action )
	long ctype, cnum;				/*** registration type, corner num ***/
	double coord_pairs[][2];		/*** coords for corners       ***/
	long num_to_reg;                /*** number of corners to reg ***/
	char action;					/*** 'd' is display, 'p' is print ***/

{
	long cnr_indx;			/*** Index for the corner array      ***/
	long i, j;				/*** Plain old boring for loop index ***/
	long sph_code;			/*** Used as '2' for Clarke 1866 ***/

/***  Check the action parm for display or print ***/
	if( action == 'd' )
	{	if( raw_print(11, 0, 10, 1) )
			return(0);
		sprintf(raw_buf, "register -d" );
		return(1);
	}
	if( action == 'p' )
	{	if( raw_print(11, 0, 10, 0) )
			return(0);
		else
		{	sprintf(raw_buf, "register -p");
			return(1);
		}
	}

/***  Register the coords into "corner" global ***/
	switch( ctype )
	{
	case 0:		/*** Raster Pixels ***/
		for(j = 0; j < num_to_reg; j++ )
		{	cnr_indx = (cnum + j) % 4;     /* keep indx in range of 0 - 4 */
			corner[0][cnr_indx][0] = coord_pairs[cnr_indx][0];
			corner[0][cnr_indx][1] = coord_pairs[cnr_indx][1];
		}
			/*** add planar frame if raster completely registered */
		if(registered(0))
		{	add_plnframe();
			/*** add geodetic frame if geodetics also registered **/
			if(registered(1))
				add_geoframe();
		}
		break;
	case 1:		/*** Geodetics ***/
		for(j = 0; j < num_to_reg; j++ )
		{	cnr_indx = (cnum + j) % 4;     /* keep indx in range of 0 - 4 */
			corner[1][cnr_indx][0] = coord_pairs[cnr_indx][0];
			corner[1][cnr_indx][1] = coord_pairs[cnr_indx][1];
		}
		if( registered(1) )
		{   for( i = 0; i < 4; i++ )
		   {	geo_to_stpln( corner[1][i][0], corner[1][i][1],
		    	&corner[3][i][0],&corner[3][i][1] );

	    		geo_to_utm( corner[1][i][0], corner[1][i][1], 
		    	&corner[2][i][0], &corner[2][i][1] );
	    	}

				/*** set map inch registration ***/
	    	set_mapin(2);

				/*** set raster registration if absent ***/
			if(!registered(0))
				reg_raster(2);
		
			/*** add geodetic frame if raster also registered **/
			if(registered(0))
				add_geoframe();
		}
		break;
	case 2:		/*** UTM ***/
/***  Coordinates are Easting and Northing  ***/
		for(j = 0; j < num_to_reg; j++ )
		{	cnr_indx = (cnum + j) % 4;     /* keep indx in range of 0 - 4 */
			corner[2][cnr_indx][0] = coord_pairs[cnr_indx][0];
			corner[2][cnr_indx][1] = coord_pairs[cnr_indx][1];

			sph_code = 2;		/***  Hardcoded for Clarke 1866  ***/
			if(utm_to_geo(sph_code, 
			              &coord_pairs[cnr_indx][0], &coord_pairs[cnr_indx][1], 
			              &corner[1][cnr_indx][0], &corner[1][cnr_indx][1]))
				message("Cannot register geodetics from UTM.");
			else
				geo_to_stpln(corner[1][cnr_indx][0], corner[1][cnr_indx][1], 
					&corner[3][cnr_indx][0], &corner[3][cnr_indx][1]);
		}

		if(registered(2))
		{	set_mapin(2);

				/*** set raster registration if absent ***/
			if(!registered(0))
				reg_raster(2);
		
			/*** add geodetic frame if raster also registered **/
			if(registered(0) && registered(1))
				add_geoframe();
		}
		break;
	case 3:		/*** State Plane ***/
		for(j = 0; j < num_to_reg; j++ )
		{	cnr_indx = (cnum + j) % 4;     /* keep indx in range of 0 - 4 */
			corner[3][cnr_indx][0] = coord_pairs[cnr_indx][0];
			corner[3][cnr_indx][1] = coord_pairs[cnr_indx][1];

				/*** Base following calls on cnum of coord_pairs ***/
			if(stpln_to_geo(coord_pairs[cnum][0], coord_pairs[cnum][1], 
					&corner[1][cnum][0], &corner[1][cnum][1]))
				message("Cannot register geodetics from state plane.");
			else
				geo_to_utm(corner[1][cnum][0], corner[1][cnum][1], 
					&corner[2][cnum][0], &corner[2][cnum][1]);
		}

		if(registered(3))
		{	set_mapin(3);

				/*** set raster registration if absent ***/
			if(!registered(0))
				reg_raster(3);
		
			/*** add geodetic frame if raster also registered **/
			if(registered(0) && registered(1))
				add_geoframe();
		}
		break;
	case 4:     /*** digitizer 1 ***/
	case 5:     /*** digitizer 2 ***/
		for(j = 0; j < num_to_reg; j++ )
		{	cnr_indx = (cnum + j) % 4;     /* keep indx in range of 0 - 4 */
			corner[ctype][cnr_indx][0] = coord_pairs[cnr_indx][0];
			corner[ctype][cnr_indx][1] = coord_pairs[cnr_indx][1];
		}
		if(registered(ctype))
			set_mapin(ctype);
		break;
	case 6:		/*** Map Inches ***/
		for(j = 0; j < num_to_reg; j++ )
		{	cnr_indx = (cnum + j) % 4;     /* keep indx in range of 0 - 4 */
			corner[6][cnr_indx][0] = coord_pairs[cnr_indx][0];
			corner[6][cnr_indx][1] = coord_pairs[cnr_indx][1];
		}
		break;
	default:		/*** everything else ***/
		for(j = 0; j < num_to_reg; j++ )
		{	cnr_indx = (cnum + j) % 4;     /* keep indx in range of 0 - 4 */
			corner[ctype][cnr_indx][0] = coord_pairs[cnr_indx][0];
			corner[ctype][cnr_indx][1] = coord_pairs[cnr_indx][1];
		}
		break;
	}
	
/***  Return ***/
	return(0);
}
