/*******************************************************************************
*      rotate       (see manual, Command Ref., User Functions)     <fc>        *
*           Description:  Rotates the raster 90 degrees or in increments of    *
*           90 degrees (+ or -).  The default is 90 degrees.                   *
*           Parameters:  Rotation degrees.                                     *
*           Globals Modified:  craster, lraster, raster_bincrement,            *
*                              raster_wincrement                               *
*                                                                              *
*           Returns zero upon normal end-of-job (EOJ).                         *
*           C. Jonson   1/8/90                                                 *
*******************************************************************************/
/* Modified 7/92 by DBS@CWU for big endian support */

#include "../../incl/globals.h"
#include "../../incl/in.h"

#ifdef BIG_ENDIAN
#include "../../incl/big_end.h"
#endif

long rotate( deg )
	long deg;
{
	int fd;
	long i, j, k;
	long new_width, new_height;
	long old_bincrement, old_rows;
	long numrows;
	long shift;
	long h,n,w;
	long size;
	long raster_size;
	unsigned char *old_row;
	unsigned char *top, *end, *new, *new0;
	unsigned char tmp;
	unsigned char rindx[256];
	char string[256];
	long create_braster;

		/********** check for map loaded *****************************/
	if(!job_stats[0])
	{	message("You must CREATE or ACQUIRE a map befor using the rotate command.");
		return(1);
	}

		/************* check for VIEW ONLY *****************************/
	if(job_stats[78])
	{	message("You CANNOT do ANYTHING to the VIEW ONLY ORIGINAL SCAN.");
		return(1);
	}

		/************* check for braster *******************************/
	if(israsterb())
		create_braster = 1;
	else
		create_braster = 0;

/*** Save the number of rows in the current raster ***/
	old_rows = job_stats[2];
	
/*** If deg is (+-)180, flip and invert the raster ***/
/*** Loop through craster and swap rows ***/
	if( deg == 180 || deg == -180 )
	{	numrows = job_stats[2];
		reflect_indx(rindx);
		top = craster[0];			/** Point to first row of raster **/
		/** Point to end of last row of raster **/
		end = top + (raster_bincrement * numrows) + raster_bincrement - 1;		

		/** Keep swapin and flipin bytes until pointers meet **/
		for(; end > top; top++, end--)
		{	tmp = rindx[*end];
			*end = rindx[*top];
			*top = tmp;
		}

		/** 0 out obj file "first type" rec locators **/
		for(i=3; i<=7; i++)			
			job_stats[i] = 0;

		for(i=10; i<=18; i++)		/** 0 out rec counts for map files **/
		{	if((i==13)  ||  (i==15)  ||  (i==16))
				job_stats[i] = (job_stats[i])  ?    1 : 0;
			else
				job_stats[i] = 0;
		}
		job_stats[49] = 0;			/** 0 "vertex moved" flag **/
		job_stats[75] = 0;			/** 0 "next error" number **/
		if(job_stats[40] < job_stats[45])
		{	job_stats[45]++;
			job_stats[40] = job_stats[45];
		}

	}
	else		/** deg is (+-) 90 or (+-)270 **/
	{	/***  write existing raster to temp file  *****/
		sprintf(string, "%s/raster", temp_map_path);
		if((fd = openf(string,WRITE | O_TRUNC)) <= 0)
		{	message("!-> Cannot open raster temp file.  Aborting.");
			map_name[0] = job_stats[0] = 0;
			return(1);
		}

		/***  write all old craster records to temp file  ***/
		size = raster_bincrement * (old_rows + 1);
#ifdef BIG_ENDIAN
		byte_swap(rasterm.rec0, size, SWAP_LONG);
#endif
		if(0 > writeb(rasterm.rec0, size , 1, fd))
		{	message("!-> Cannot write all raster recs to temp file.  Bye.");
			closef(fd);
			map_name[0] = job_stats[0] = 0;
			return(1);
		}
	
		/***  close file  ***/
		closef(fd);
	
		/***  free malloc space and zero malloc pointer ***/
		free_ltp(rasterm.malloc);
		rasterm.malloc = (char *)0;

		/*** Calculate size of new raster and allocate memory for it ***/
		/*** Notice that the raster heigth and width are being swapped. ***/
		new_width = job_stats[2] + 1;
		new_height = job_stats[1] + 1;

		/*** Ensure width is integral multiple of 32 ***/
		if( new_width % 32 )
		{	n = new_width / 32 + 1;
			new_width = n * 32;
		}

		/***  Calculate sizes ***/
		h = new_height;                   /* height of raster in rows  */
		w = new_width / 8;                /* width of raster in bytes  */
		raster_size = h*w;                /* size of raster in bytes   */
		size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster*/

		/***  Allocate memory ***/
		if( (rasterm.malloc = (char *)calloc_ltp(size, sizeof(char)) ) 
						== (char *)0)
		{	message("!-> cannot get enough RAM for raster. Raster will NOT be \
rotated.");
			map_name[0] = job_stats[0] = 0;
			return(1);
		}

		/*** Reset job_stats ***/
		job_stats[1] = new_width - 1;
		job_stats[2] = new_height - 1;

		/** 0 out obj file "first type" rec locators **/
		for(i=3; i<=7; i++)			
			job_stats[i] = 0;

		for(i=10; i<=18; i++)		/** 0 out rec counts for map files **/
		{	if((i==13)  ||  (i==15)  ||  (i==16))
				job_stats[i] = (job_stats[i])  ?    1 : 0;
			else
				job_stats[i] = 0;
		}
		job_stats[49] = 0;			/** 0 "vertex moved" flag **/
		job_stats[75] = 0;			/** 0 "next error" number **/
		if(job_stats[40] < job_stats[45])
		{	job_stats[45]++;
			job_stats[40] = job_stats[45];
		}

		/***  Save raster byte increment ***/
		old_bincrement = raster_bincrement;

		/***  Set the raster_wincrement and raster_bincrement globals  ***/
		raster_wincrement = (job_stats[1] + 1) / 32;
		raster_bincrement = (job_stats[1] + 1) / 8;

		/***  Calculate & set global pointers to raster       ***/
		rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
		rasterm.recn = rasterm.rec0 + raster_size;
		craster = (unsigned char **)(rasterm.malloc);
		lraster = (unsigned long **)(rasterm.malloc + 4*h);
		for(i=0; i<h; i++)
		{	craster[i] = (unsigned char *)rasterm.rec0 + w*i;
			lraster[i] = (unsigned long *)(craster[i]);
		}

		/*** Allocate space for one row of the old raster ***/
		if((old_row = (unsigned char *)calloc_ltp(old_bincrement, sizeof(char)))
							 == (unsigned char *)0 )
		{	message("!-> Cannot allocate memory for old raster row.");
			map_name[0] = job_stats[0] = 0;
			free_ltp(rasterm.malloc);
			rasterm.malloc = (char *)0;
			return(1);
		}

		/*** Open old raster file ***/
		sprintf(string, "%s/raster", temp_map_path);
		if((fd = openf(string,READ)) == -1)
		{	message("!-> Cannot open raster temp file.  Aborting.");
			map_name[0] = job_stats[0] = 0;
			free_ltp(rasterm.malloc);
			rasterm.malloc = (char *)0;
			return(1);
		}

		/***   Rotate raster         ***/
		if( deg == 90 || deg == -270 )
		{	/*** Point to end of row 1 ***/
			new0 = craster[job_stats[2]];

			for(i=shift=0; (readb(old_row,sizeof(char),old_bincrement,fd) &&
						i <= old_rows); i++)
			{	new = new0;
				for(j = 0; j < old_bincrement; j++)
				{	if( *(old_row + j) )
					{	for( k = 7; k >= 0; k-- )
						{	if( *(old_row + j) & (1 << k) )
								*new |= (1 << (7 - shift));
							new -= raster_bincrement;
						}
					}
					else
						new -= (8 * raster_bincrement);
				}
				shift++;
				shift %= 8;
				if( !shift )
					new0++;
			}
		}
		else
		{	/*** Point to start of first row in new raster ***/
			new0 = craster[0] + raster_bincrement - 1;
			for(i=shift=0; (readb(old_row, sizeof(char), old_bincrement, fd) &&
					i <= old_rows); i++ )
			{	new = new0;
				for(j = 0; j < old_bincrement; j++)
				{	if( *(old_row + j) )
					{	for( k = 7; k >= 0; k-- )
						{	if( *(old_row + j) & (1 << k) )
								*new |= (1 << shift);
							new += raster_bincrement;
						}
					}
					else
						new += (8 * raster_bincrement);
				}
				shift++;
				shift %= 8;
				if( !shift )
					new0--;
			}
		}

		/*** Close temp raster file ***/
		closef(fd);

#ifdef BIG_ENDIAN
		byte_swap(rasterm.rec0, raster_bincrement*(job_stats[2]+1), SWAP_LONG);
#endif
		/*** Free up calloced space for old raster row  ***/
		free_ltp((char *) old_row);
	}

	/***  Write new raster to temp file  *****/
	sprintf(string, "%s/raster", temp_map_path);
	if((fd = openf(string,WRITE | O_TRUNC)) <= 0)
	{	message("!-> Cannot open raster temp file.  Aborting.");
		map_name[0] = job_stats[0] = 0;
		return(1);
	}

	/***  Write all craster records to temp file  ***/
	size = raster_bincrement * (job_stats[2] + 1);
	if( writeb(rasterm.rec0, size , 1, fd) == -1)
	{	message("!-> Cannot write all raster recs to temp file.  Bye.");
		closef(fd);
		map_name[0] = job_stats[0] = 0;
		free_ltp(rasterm.malloc);
		rasterm.malloc = (char *)0;
		return(1);
	}

	/***  Close file  ***/
	closef(fd);

	/***  Free calloc'ed space for index_array ***/
	free_ltp(index_arraym.malloc);
	index_arraym.malloc = (char *)0;

	/*** Store new sized index_array file     ***/ 
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = h*w;                       /* size to calloc            */

	/*** Allocate memory ***/
	index_arraym.malloc = (char *)calloc_ltp( size, sizeof(char) );
	if(index_arraym.malloc == (char *)0)
	{	message("!->  Cannot get enough RAM for index_array.  Re-ACQUIRE map & start over.");
		map_name[0] = job_stats[0] = 0;
		return(1);
	}

	/***  Write index_array to file ***/
	sprintf(string,"%s/index_array",temp_map_path);
	if( (fd = openf(string,WRITE | O_TRUNC)) == -1)
	{	message("!->  Cannot open index_array map file.");
		map_name[0] = job_stats[0] = 0;
		free_ltp(index_arraym.malloc);
		index_arraym.malloc = (char *)0;
		return(0);
	}
	if( writeb(index_arraym.malloc,size,1,fd) == -1 )
	{	message("!->  Cannot write index_array map file.");
		closef(fd);
		map_name[0] = job_stats[0] = 0;
		free_ltp(index_arraym.malloc);
		index_arraym.malloc = (char *)0;
		return(0);
	}
	closef(fd);

	/***  Write job_stats to file ***/
	sprintf(string,"%s/job_stats",temp_map_path);
	if(put_jobstats(string))
	{	message("!-> Cannot write job_stats to temp file.  Aborting.");
		map_name[0] = job_stats[0] = 0;
		return(1);
	}

	/***  Un-register raster ***/
	for(i=0; i<4; i++)
		corner[0][i][0] = corner[0][i][1] = 0.0;

	/*** Write diary record ***/
	sprintf(string, "rotated raster %d degrees", deg);
	put_diary(string);

	/*** Reacquire map files ***/
	if(reacquire2())
		return(1);

		/******** re-build map display *****/
	if(redisp_map())
		return(1);
				
		/************* re-create braster *******************************/
	if(create_braster)
		brstr_create();

	/*** All done.  Return ***/
	return(0);
}
