/*******************************************************************************
*     set_intrsec     (see manual, Command Ref., User Functions)      <fc>     *
*                                                                              *
*         Creates a set of objects by entering all the objects in set s0       *
*         which are not in set s1.                                             *
*                                                                              *
*         Parameters:    long s0;        set to be intersected                 *
*                        long s1;        set to be intersected                 *
*                        long s2;        new set created                       *
*                        char descr[];   set name/description                  *
*                                                                              *
*         globals changed:  objects file, level_names[] global                 *
*                                                                              *
*         Returns 0 upon normal execution, 1 otherwise                         *
*             J.Dabritz  1/17/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"


long set_intrsec(s0,s1,s2,descr)
	long s0,s1,s2;
	char descr[];
{	
	OBJECTS object;

	char string[256];

	long i;
	long mask0, mask1, mask2;
	long no_chosen;
	long x,y;
	
		/*********** check for map *************************************/
	if(!job_stats[0])
	{	message("You must Create or Acquire a map befor creating sets of objects.");
		return(1);
	}

		/*********** check input parameters ****************************/
	if(s2<1  ||  s2>24)
	{	message("The set to be created must be in the range 1-24.");
		return(1);
	}
	if(s0<0  ||  s0>31)
	{	message("The set to intrsec must be in the range 0-32.");
		return(1);
	}
	if(s1<0  ||  s1>31)
	{	message("The set to intrsec must be in the range 0-32.");
		return(1);
	}
	if(descr <= (char *) 0)
	{	message("Illegal string pointer.  set_intrsec aboring.");
		return(1);
	}
	descr[79] = (char) 0;


		/*********** calc set mask *************************************/
	mask0 = 1 << s0;
	mask1 = 1 << s1;
	mask2 = 1 << s2;


		/*********** check all objects for inclusion in set ************/
	no_chosen = 0;
	for(i=0; i<job_stats[10]; i++)
	{	
			/*********** get object ************************************/
		if(get_objects(&object,i))
		{	message("set_intrsec abort location #1.");
			sleep(3);
			break;
		}

			/******* mark set inclusion or exclusion for object ********/
		if((object.i12[4] & mask0)  && (object.i12[4] & mask1))
		{	object.i12[4] |= mask2;      /** raise flag for set **/
			no_chosen++;
		}
		else
			object.i12[4] &= ~mask2;     /** lower flag for set **/

			/******* store object **************************************/
		if(put_objects(&object,i))
		{	message("set_intrsec abort location #2.");
			sleep(3);
			break;
		}

			/*********** inform user ***********************************/
		if(!(i % 100))
		{	sprintf(string,"set_intrsec, checking object %d,  objects chosen = %d.",i,no_chosen);
			message(string);
		}
	}

		/*********** set level_name to set descriptor ******************/
	trim(descr);
	if(! (*descr))
		sprintf(descr,"   [ unlabeled set ]");
	sprintf(level_names[s2],"%s",descr);

		/*********** change display to highlight created set only ******/
	disp_hglight = (1 << s2);
	disp_centroid = 1;

		/*********** re-display map ************************************/
	x = (rwindowxmin + rwindowxmax) / 2;
	y = (rwindowymin + rwindowymax) / 2;
	if(dispgraphics(x,y,resolution_code))
		return(1);

		/*********** inform user about no_obj, len/perim, area, label ***/
	report_1set(s2);

		/*********** return ********************************************/
	return(0);
}
