/*******************************************************************************
*     set_report      (see manual, Command Ref., User Functions)      <fc>     *
*                                                                              *
*         Generates a report to the designated printer based                   *
*         on the report parameters received.                                   *
*         Parameters:                                                          *
*            long outdev   Display device type: 0=screen, 1=printer, 2=file    *
*            char fname    Name of file report writes to.                      *
*                                                                              *
*         Returns 0 upon normal execution, else                                *
*         returns -1 upon error.                                               *
*             J.Dabritz  1/10/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>


long set_report(  outdev, fname )
	long outdev;
	char *fname;
{
	OBJECTS object;
	char tmp_name[L_tmpnam];
	char actual[256];
	char string[256];
	char setnames[32][STRLIST];

	int fd;

	long b,i;
	long element_count[32];
	long subjects;

	float length[32], area[32];


		/*********** check for map *************************/
	if(!job_stats[1])
	{	message("Cannot write a set_criteria report unless a map is acquired.");
		return(1);
	}

		/*********** check output device code **************/
	if((outdev < 0)  ||  (outdev > 2))
	{	message("Output device code must be in the range  0 - 2.");
		return(1);
	}

		/*********** check file name************************/
	message("setting up output files and variables.");
	if(outdev == 2)
	{	if(setup_ofile(fname,actual))
		{	exmossabort(11);
			return(1);
		}
	}
	else
	{	tmpnam( tmp_name);
		sprintf(actual,"%s",tmp_name);
	}
		
		/*********** create file ***************************/
	if( creatf( actual ) == -1 )
	{	message("set_report: Cannot create temp file. Bye.");
		return(1);
	}
	
	if( (fd = openf( actual, RDWR)) == -1 )
	{	message("set_report: Cannot open temp file. Bye.");
		return(1);
	}
	
		/*********** write to file *************************/
		/***** write header *********/
	if(writea("Set_report \n",fd))
	{	message("set_report: Cannot write to temp file. Bye.");
		closef(fd);
		return(1);
	}
	std_header(string);
	if(writea(string, fd))
	{	message("set_report: Cannot write to temp file. Bye.");
		closef(fd);
		return(1);
	}
	if(writea("\n",fd))
	{	message("set_report: Cannot write to temp file. Bye.");
		closef(fd);
		return(1);
	}

		/***** inform user *************************************/
	message("Checking sets and counting objects.");

		/***** initialize element_count ************************/
	for(i=0; i<32; i++)
	{	element_count[i] = 0;
		length[i] = 0.0;
		area[i] = 0.0;
		strcpy(setnames[i],level_names[i]);
		setnames[i][26] = (char) 0;
	}

		/***** count the number of elements in each set ********/
	for(i=0; i<job_stats[10]; i++)
	{		/***** get object record ***************************/
		if(get_objects(&object,i))
		{	closef(fd);
			return(1);
		}
			/***** add to sets *********************************/
		if(object.i12[4])
		{	for(b=0; b<32; b++)
			{	if(object.i12[4] & (1 << b))
				{	element_count[b]++;
					length[b] += object.f4[2];
					area[b] += object.f4[3];
				}
			}
		}
	}

		/***** convert to mi/acres *******/
	get_units();
	if((unit_factor[3][0] != (float) 0.0)  &&  (unit_factor[2][1] != (float) 0.0))
	{	for(i=0; i<32; i++)
		{	length[i] *= unit_factor[3][0];
			area[i] *= unit_factor[2][1];
		}
	}
	
		/***** 0.  write report  *********/
	sprintf(string,"set | members | subjects | len/perim  |    area      | set name\n\n");
	if(writea(string, fd))
	{	message("set_report: Cannot write to temp file. Bye.");
		closef(fd);
		return(1);
	}
	for(i=0; i<32; i++)
	{		/*** count subjects ****/
		if(element_count[i])
			subjects = count_subj(i);
		else
			subjects = 0;
		if((unit_factor[3][0] != (float) 0.0)  &&  (unit_factor[2][1] != (float) 0.0))
			sprintf(string," %2d    %5d     %5d    %6.2f mi. %9.1f ac. %26s\n",i,element_count[i],subjects,length[i],area[i],setnames[i]);
		else
			sprintf(string," %2d    %5d     %5d    %9.0f px. %9.1f px. %26s\n",i,element_count[i],subjects,length[i],area[i],setnames[i]);
		if(writea(string, fd))
		{	message("set_report: Cannot write to temp file. Bye.");
			closef(fd);
			return(1);
		}
		if((i % 5) == 4)
		{	if(writea("\n", fd))
			{	message("set_report: Cannot write to temp file. Bye.");
				closef(fd);
				return(1);
			}
		}
	}


		/*********** inform user ***************************/
	append_msg("   done.");


		/*********** close file ****************************/
	if( closef(fd))
	{	message("set_report: Cannot open temp file. Bye.");
		return(1);
	}


		/*********** print or display file if necess *******/
	switch(outdev)
	{
	case 0:				/* Display on CRT */
		more(actual);
		break;
	case 1:				/* Send to printer1 */
		print_file(actual);
		break;
	case 2:				/* send to file     */
		break;
	default:			/* SHOULD NEVER HAPPEN */
		message("set_report: Darned serious problem occurred. Good luck.");
		return(1);
	}

		/*********** return ********************************/
	return(0);
}
