/*******************************************************************************
*	start_ltp	(see manual, Command Ref., User Function)	<fc>   *
*			Start a new execution of the current graphics input    *
*			mode at the current x,y position.  Does not connect the*
*			drawn line/erasure/seperation etc. to the last point,  *
*			but restarts it at the current position.               *
*                                                                              *
*			Sets first_point global to 0,                          *
*			Updates last_raster_x,y globals.,                      *
*		    Updates anchorx,y globals.                                 *
*		                                                               *
*		    Returns 1 on error, 0 on ok.                               *
*		                                                               *
*		    J.Dabritz  6/26/89                                         *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long start_ltp(x,y)
	long x,y;  /*  raster coordinates at which to start     */
{
	long word, bit;
	long on;
	long x2,y2;


		/******************* check for map loaded **********************/
	if(!job_stats[0])
		return(1);

		/***************** make sure x & y are on raster ***************/
	if(x < 0)
		x = 0;
	else if(x > job_stats[1])
		x = job_stats[1];

	if(y < 0)
		y = 0;
	else if(y > job_stats[2])
		y = job_stats[2];

		/***************** take action according to mode *************/
	cursor_mode = 0;                /**  cross hair, default **/
	switch(input_mode)  {
 	case 0:          /* none                    */
		break;
 	case 1:          /* attribute contours      */
		cursor_mode = 1;   /* rubber_line  */
		if(!first_point)       /* starting new profile, if not first profile */
			context_elev = 1;  /*  then get starting elevation from context. */
		break;
 	case 2:          /* attribute digitize      */
		return(continue_ltp(x,y));
 	case 3:          /* connect                 */
	case 25:         /* recall connect          */
		cursor_mode = 1;   /* rubber_line  */
		break;
 	case 4:          /* draw point              */
		set_undo(4,0,0,0,0,x,y);
		pixel(x,y,1);
		break;
	case 18:         /* recall point            */
		return(continue_ltp(x,y));
 	case 5:          /* draw line               */
	case 19:         /* recall line             */
		cursor_mode = 1;   /* rubber_line  */
		break;
 	case 6:          /* draw fat-line           */
	case 20:         /* recall fat-line         */
		cursor_mode = 1;   /* rubber_line  */
		break;
 	case 7:          /* draw rectangle          */
	case 21:         /* recall rectange bdr     */
	case 22:         /* recall rectangle area   */
		cursor_mode = 2;   /* rubber_box   */
		break;
 	case 8:          /* erase point             */
		set_undo(8,0,0,0,0,x,y);
		pixel(x,y,0);
		break;
 	case 9:          /* erase line              */
		cursor_mode = 1;   /* rubber_line  */
		break;
	case 10:          /* erase fat-line         */
		cursor_mode = 1;   /* rubber_line  */
		break;
	case 11:          /* erase rectangle        */
		cursor_mode = 2;   /* rubber_box   */
		break;
	case 12:          /* erase segment          */
		return(continue_ltp(x,y));
	case 13:          /* query                  */
		return(continue_ltp(x,y));
	case 14:          /* separate               */
	case 24:          /* recall separate        */
		cursor_mode = 1;   /* rubber_line  */
		pixel(x,y,0);
		if(input_mode == 24)
			recall_mode(18,0,0,0,0,x,y);
		else
			set_undo(8,0,0,0,0,x,y);
		break;
	case 15:          /* wink                   */
		word = x >> 5;
		bit  = (7 & ~x) | (x & 24);
		on = (lraster[y][word] & (1 << bit)) ?  0 : 1;
		pixel(x,y,on);
		set_undo(15,0,0,0,0,x,y);
		break;
	case 16:          /* draw wedge             */
	case 17:          /* erase wedge            */
	case 23:          /* recall wedge           */
		cursor_mode = 3;    /*  rubber wedge (triangle) */
		break;
	case 26:          /* bridge_gap             */
		return(continue_ltp(x,y));
	case 27:          /* seg_break              */
		if(seg_breaker(0,x,y))  /** finds break point & sets anchors **/
			return(1);            /** first point not found **/

		cursor_mode = 4;   /* cross hair + highlighted segment portion **/
		cursor(x+1,y);
		cursor(x,y);
		break;
	case 28:            /* draw spur **/
		if(!add_spur(x,y,&x2,&y2))
			set_undo(5,0,0,x2,y2,x,y);
		break;
	case 29:            /* atrbt_line */
		if(line_atrbt(x,y,search_radius,1)) /* find first point, set anchorx,y */
			return(1);
		cursor_mode = 5;   /* box & cross hair */
		cursor(x+1,y);
		cursor(x,y);
		break;
	case 30:             /* erase_obj */
		erase_xyobj((float) x,(float) y,(float) search_radius,atrbt_otype);
		cursor_mode = 0;
		break;
	case 31:             /* vertex_move */
		first_point = 1;
		if(move_vertex(x,y))  /** find vertex to move & set anchorx,y ***/
			return(1);
		cursor_mode = 6;  /* small cross & large cross */
		break;
	case 32:             /* level_icopy */
		first_point = 1;
		if(icopy(x,y))  /** find vertex to move & set anchorx,y ***/
			return(1);
		break;
	case 33:             /* set_dig  */
		if(dig_set(x,y))
			return(1);
		break;
	case 34:             /* atrbt_copy */
		if(find_atrbt((float) x,(float) y,(float) search_radius))
			return(1);
		break;
	case 35:             /* atrbt_list */
		if(next_list_atr())
			return(1);
		break;
	}

		/***************** update anchorx,y **************************/
		/**** NOTE for mode 27 (seg_break) anchors are set in seg_breaker.c ***/
	switch(input_mode)  {
 	case 1:          /* attribute contours      */
 	case 3:          /* connect                 */
 	case 5:          /* draw line               */
 	case 6:          /* draw fat-line           */
	case 7:          /*  draw rectangle         */
 	case 9:          /* erase line              */
	case 10:         /* erase fat-line          */
	case 11:         /*  erase rectangle        */
	case 14:         /* separate                */
	case 19:         /* recall line             */
	case 20:         /* recall fat-line         */
	case 21:         /* recall rectangle bdr    */
	case 22:         /* recall rectangle area   */
	case 24:         /* recall separate         */
	case 25:         /* recall connect          */
		anchorx = x;
		anchory = y;
		break;
	case 16:         /* draw wedge              */
	case 17:         /* erase wedge             */
	case 23:         /* recall wedge            */
		anchorx = anchor2x = x;
		anchory = anchor2y = y;
		break;
	}
	
		/***************** set first_point global ********************/
	first_point = 0;

		/***************** update last_raster_x,y ********************/
	last_raster_x = x;
	last_raster_y = y;

		/***************** return ************************************/
	return(0);
}
