/*******************************************************************************
*    status_ltp       (see manual, Command Ref., User Functions)      <fc>     *
*       Description:  View the status of the active map.                       *
*       Parameters:  Print flag boolean.  If true, send status to printer.     *
*                    If false, display status on screen using 'more'.          *
*                                                                              *
*       Returns 0 upon normal end-of-job (EOJ), else returns -1.               *
*******************************************************************************/
#include "../../incl/globals.h"
#include "../../incl/in.h"

long status_ltp( pr_switch )
	long pr_switch;
{
	int  fd;
	OBJ_CNT ocnt;
	char rec[STRLIST];
	char tfile[L_tmpnam];
	char tmsg[256];

/*** Create file to dump formatted string to ***/
	if( tmpnam(tfile) == (char *)NULL )
	{	message("STATUS_LTP: Cannot generate temp file name.");
		return(-1);
	}

	if( creatf( tfile) == -1 )
	{	sprintf(tmsg,"STATUS_LTP: Cannot create %s.", tfile);
		message(tmsg);
		return(-1);
	}

/*** Open the file ***/
	if( (fd = openf(tfile, RDWR)) == -1)
	{	sprintf(tmsg, "STATUS_LTP: Cannot open %s.", tfile);
		message(tmsg);
		return(-1);
	}

/*** Get number of objects in job stats file ***/
	if( obj_count(&ocnt) )
	{	message("STATUS_LTP:  Cannot count objects file.");
		return(-1);
	}

/*** Go through job stats array and write file too ***/
	sprintf(rec, "The STATUS of the map %s is as follows:\n\n", map_name);
	writea(rec, fd);

	sprintf(rec, "FILE VERSION number........................... %.2f\n", 
			(float)((float)job_stats[0]/100.0));
	writea(rec, fd);
	sprintf(rec, "RASTER HEIGHT................................. %d\n", 
			job_stats[2] + 1);
	writea(rec, fd);
	sprintf(rec, "RASTER WIDTH.................................. %d\n", 
			job_stats[1] + 1);
	writea(rec, fd);
	sprintf(rec, "Number of NODES............................... %d\n", 
			ocnt.nodes);
	writea(rec, fd);
	sprintf(rec, "Number of SEGMENTS............................ %d\n", 
			ocnt.segments);
	writea(rec, fd);
	sprintf(rec, "Number of INTERIOR POLYGONS................... %d\n",
			ocnt.int_polys);
	writea(rec, fd);
	sprintf(rec, "Number of records in OBJECTS file............. %d\n", 
			job_stats[10]);
	writea(rec, fd);
	sprintf(rec, "Number of records in COORD_LIST file.......... %d\n", 
			job_stats[11]);
	writea(rec, fd);
	sprintf(rec, "Number of records in SEG_LIST file............ %d\n", 
			job_stats[12]);
	writea(rec, fd);
	sprintf(rec, "Number of records in ATTRIBUTES file.......... %d\n", 
			job_stats[13] - 1);
	writea(rec, fd);
	sprintf(rec, "Number of records in ERRORS file.............. %d\n", 
			job_stats[14]);
	writea(rec, fd);
	sprintf(rec, "Number of records in INDEX_LIST file.......... %d\n", 
			job_stats[15] - 1);
	writea(rec, fd);
	sprintf(rec, "Number of records in V_ATTRIBUTES file........ %d\n", 
			job_stats[16] - 1);
	writea(rec, fd);
	sprintf(rec, "Number of records in V_OBJECTS file........... %d\n", 
			job_stats[17]);
	writea(rec, fd);
	sprintf(rec, "Number of records in V_COORD_LIST file........ %d\n", 
			job_stats[18]);
	writea(rec, fd);

/*** Check job_stats to determine type of data thats been imported ***/
	if( job_stats[20] )
	{	sprintf(rec, "RSTR type data has been imported.\n");
		writea(rec, fd);
	}
	if( job_stats[21] )
	{	sprintf(rec, "RLE1 type data has been imported.\n");
		writea(rec, fd);
	}
	if( job_stats[22] )
	{	sprintf(rec, "RDEF2 type data has been imported.\n");
		writea(rec, fd);
	}
	if( job_stats[23] )
	{	sprintf(rec, "ASN type data has been imported.\n");
		writea(rec, fd);
	}
	if( job_stats[24] )
	{	sprintf(rec, "RAS type data has been imported.\n");
		writea(rec, fd);
	}
	if( job_stats[25] )
	{	sprintf(rec, "OPTR9 type data has been imported.\n");
		writea(rec, fd);
	}
	if( job_stats[30] )
	{	sprintf(rec, "DLG3 type data has been imported.\n");
		writea(rec, fd);
	}
	if( job_stats[31] )
	{	sprintf(rec, "MOSS type data has been imported.\n");
		writea(rec, fd);
	}
	if( job_stats[32] )
	{	sprintf(rec, "DIGIT_ASC type data has been imported.\n");
		writea(rec, fd);
	}
	if( job_stats[33] )
	{	sprintf(rec, "DIGIT_ATT type data has been imported.\n");
		writea(rec, fd);
	}

/*** See whats been done to this map ***/
	if( job_stats[40] )
	{	sprintf(rec, "Map has been EDITED................Reference #  %d\n",
				job_stats[40]);
		writea(rec, fd);
	}
	if( job_stats[41] )
	{	sprintf(rec, "Map has been ASSEMBLED.............Reference #  %d\n",
				job_stats[41]);
		writea(rec, fd);
	}
	if( job_stats[42] )
	{	sprintf(rec, "Map has been PLOTTED...............Reference #  %d\n", 
				job_stats[42]);
		writea(rec, fd);
	}
	if( job_stats[43] )
	{	sprintf(rec, "Map has had REPORTS generated......Reference #  %d\n", 
				job_stats[43]);
		writea(rec, fd);
	}
	if( job_stats[44] )
	{	sprintf(rec, "Map has been ATTRIBUTED............Reference #  %d\n",
				job_stats[44]);
		writea(rec, fd);
	}

	next_step(rec);
	strcat(rec, "\n");
	writea(rec, fd);

/*** What type of data has been exported ***/
	if( job_stats[50] )
	{	sprintf(rec, "DLG3 type data has been exported.\n");
		writea(rec, fd);
	}
	if( job_stats[51] )
	{	sprintf(rec, "DEM type data has been exported.\n");
		writea(rec, fd);
	}
	if( job_stats[52] )
	{	sprintf(rec, "MOSS type data has been exported.\n");
		writea(rec, fd);
	}
	if( job_stats[53] )
	{	sprintf(rec, "RDEF2 type data has been exported.\n");
		writea(rec, fd);
	}
	if( job_stats[54] )
	{	sprintf(rec, "DIGIT type data has been exported.\n");
		writea(rec, fd);
	}

/*** Close out the file ***/
	closef(fd);

/*** Is file to be displayed or printed.  Only pr_switch knows 4 sure! ***/
	if( pr_switch )
	{	if( print_file(tfile) )
		{	message("Cannot print status.");
			return(-1);
		}
	}
	else if( more(tfile) )
	{	message("Cannot display status.");
		return(-1);
	}

	return(0);
}
