/*******************************************************************************
*       undo           (see manual, Command Ref., User Function)        <fc>   *
*                                                                              *
*    Purpose:    reset the last interactive raster change to the last stored   *
*       raster configuration (changed pixels only).                            *
*                                                                              *
*    Arguments:    none                                                        *
*        (prior modes fixed:                                                   *
*               prior mode            mode used to fix                         *
*                  1, atrbt_contur           1, atrbt_contur                   *
*                  3, connect               25, recall connect                 *
*                  4, draw point            18, recall point                   *
*                  5, draw line             19, recall line                    *
*                  6, draw fat line         20, recall fat line                *
*                  7, draw rectangle        21, recall brectangle              *
*                  8, erase point           18, recall point                   *
*                  9, erase line            19, recall line                    *
*                 10, erase fat-line        20, recall fat-line                *
*                 11, erase rectangle       22, recall arectangle              *
*                 14, separate              24, recall separate                *
*                 15, wink                  18, recall point                   *
*                 16, draw wedge            23, recall wedge                   *
*                 17, erase wedge           23, recall wedge                   *
*                 29, atrbt_line            29, atrbt_line                     *
*                                                                              *
*    Returned Values: (long) 0 = ok,  1=failure                                *
*                                                                              *
*    Global Variables Modified: Updates:  undo_array, undo_index               *
*                      j.dabritz  1/9/90                                       *
*******************************************************************************/

#include "../../incl/globals.h"

long undo()
{	
	long i;
	long x,y,x1,y1,x2,y2;
	long zx1,zx2,zx3;
	long mode,current_mode;
	long save_mp, save_pp, save_mode, save_braster;

	long dx,dy;
	long ymin,ymax;
	long hit;

	double slope;

		/************** check for map loaded ***************************/
	if(!job_stats[0])
	{	message("You must CREATE or ACQUIRE a map for the 'undo' command to have meaning.");
		return(1);
	}

		/************** save current mode ******************************/
	current_mode = input_mode;

		/************** set x & y variables for easy reading ***********/
	mode = undo_array[undo_index].mode;
	x = undo_array[undo_index].x;
	y = undo_array[undo_index].y;
	x1 = undo_array[undo_index].x1;
	y1 = undo_array[undo_index].y1;
	x2 = undo_array[undo_index].x2;
	y2 = undo_array[undo_index].y2;

		/************** undo previous change ***************************/
	switch(mode)        {
	case 1:        /* atrbt_contur            */
		atrbt_profil(x1,y1,x,y,0);
		break;
 	case 3:        /* connect                 */
		dx = x - x1;
		dx = (dx > -dx)  ?  dx : -dx;
		dy = y - y1;
		dy = (dy > -dy)  ?  dy : -dy;
		if(dy > dx)
		{	dy = 0;
			dx = 1;
		}
		else
		{	dy = 1;
			dx = 0;
		}
			/** erase diagonal lines on both edges **/
		line_diag(x1+dx,y1+dy,x+dx,y+dy,0);
		line_diag(x1-dx,y1-dy,x-dx,y-dy,0);
			/** erase diagonal line separating edges **/
		line_diag(x1,y1,x,y,0);

			/** fill in the erased line corners with 'off' pixels  **/
		input_mode = 3;
		fill_concnrs(x1,y1,x,y);

			/** for recall mode, fill in old bits **/
		input_mode = 25;
		save_braster = recall_rset(0);
		recall_mode(25,0,0,x1,y1,x,y);
		recall_rset(save_braster);
		break;
	case 4: case 8: case 15:       /* recall point  */
		pixel(x,y,0);
		save_braster = recall_rset(0);
		recall_mode(18,0,0,0,0,x,y);
		recall_rset(save_braster);
		break;
	case 5: case 9:        /* recall line             */
		line_solid(x1,y1,x,y,0);
		save_braster = recall_rset(0);
		recall_mode(19,0,0,x1,y1,x,y);
		recall_rset(save_braster);
		break;
 	case 6:          /* draw fat-line           */
 	case 10:         /* erase fat-line          */
		dx = x - x1;
		dx = (dx > -dx)  ?  dx : -dx;
		dy = y - y1;
		dy = (dy > -dy)  ?  dy : -dy;
		if(dy > dx)
		{	dy = 0;
			dx = 1;
		}
		else
		{	dy = 1;
			dx = 0;
		}
		for(i= -1; i<2; i++)
			line_solid(x1+i*dx,y1+i*dy,x+i*dx,y+i*dy,0);

		save_braster = recall_rset(0);
		recall_mode(20,0,0,x1,y1,x,y);
		recall_rset(save_braster);
		break;
 	case 7:          /* draw rectangle          */
		line_diag(x1,y1,x1,y,0);
		line_diag(x1,y,x,y,0);
		line_diag(x,y,x,y1,0);
		line_diag(x,y1,x1,y1,0);
		save_braster = recall_rset(0);
		recall_mode(21,0,0,x1,y1,x,y);
		recall_rset(save_braster);
		break;
	case 11:          /* erase rectangle        */
		erase_rect(x1,y1,x,y);
		save_braster = recall_rset(0);
		recall_mode(22,0,0,x1,y1,x,y);
		recall_rset(save_braster);
		break;
	case 14:          /* separate               */
		dx = x - x1;
		dx = (dx > -dx)  ?  dx : -dx;
		dy = y - y1;
		dy = (dy > -dy)  ?  dy : -dy;
		if(dy > dx)
		{	dy = 0;
			dx = 1;
		}
		else
		{	dy = 1;
			dx = 0;
		}
			/** draw solid lines on both edges **/
		line_solid(x1+dx,y1+dy,x+dx,y+dy,0);
		line_solid(x1-dx,y1-dy,x-dx,y-dy,0);
			/** erase solid line separating edges **/
		line_solid(x1,y1,x,y,0);
				
		save_braster = recall_rset(0);
		recall_mode(24,0,0,x1,y1,x,y);
		recall_rset(save_braster);
		break;
	case 16:          /*    draw wedge      */
	case 17:          /*    erase wedge      */
		line_diag(x1,y1,x,y,0);
		line_diag(x2,y2,x,y,0);
		line_diag(x2,y2,x1,y1,0);
			/* find y limits of triangle */
		ymin = ymax = y;
		if(y1 > ymax)
			ymax = y1;
		if(y2 > ymax)
			ymax = y2;

		if(y1 < ymin)
			ymin = y1;
		if(y2 < ymin)
			ymin = y2;


		/** erase horizontal lines within triangle from bottom to top */
		for(ymin++;  ymin < ymax; ymin++)
		{		/** find intersection of row with triangle sides **/
			hit = 0;
				/** side anchor to anchor2 **/
			if(((ymin <= y1) && (ymin >= y2))   ||
			   ((ymin <= y2) && (ymin >= y1)))
			{	dx = x2 - x1;
				dy = y2 - y1;
				if(dy)
				{	slope = dx / (double) dy;
					zx1 = zx2 = x1 + (ymin - y1) * slope + 0.5;
					hit = 1;
				}
			}
				/** side anchor to x,y     **/
			if(((ymin <= y1) && (ymin >= y))   ||
			   ((ymin <= y) && (ymin >= y1)))
			{	dx = x - x1;
				dy = y - y1;
				if(dy)
				{	slope = dx / (double) dy;
					zx3 = x1 + (ymin - y1)* slope + 0.5;
					if(!hit)
					{	zx1 = zx2 = zx3;
						hit = 1;
					}
					else
					{	zx1 = (zx3 < zx1) ?   zx3 : zx1;
						zx2 = (zx3 > zx2) ?   zx3 : zx2;
					}
				}
			}
				/** side anchor2 to x,y    **/
			if(((ymin <= y2) && (ymin >= y))   ||
			   ((ymin <= y) && (ymin >= y2)))
			{	dx = x - x2;
				dy = y - y2;
				if(dy)
				{	slope = dx / (double) dy;
					zx3 = x2 + (ymin - y2)* slope + 0.5;
					if(!hit)
					{	zx1 = zx2 = zx3;
						hit = 1;
					}
					else
					{	zx1 = (zx3 < zx1) ?   zx3 : zx1;
						zx2 = (zx3 > zx2) ?   zx3 : zx2;
					}
				}
			}
				/** erase row between intersects **/
			if(hit)
			{	for(zx3=zx1; zx3 <= zx2; zx3++)
					pixel(zx3,ymin,0);
			}
		}
			/** for recall mode, fill in old bits **/
		save_braster = recall_rset(0);
		recall_mode(23,x2,y2,x1,y1,x,y);
		recall_rset(save_braster);
		break;
	case 29:        /* atrbt_line            */
			/**** save attribure & mile_post & plus_plus & atrbt_action ***/
		save_mp = mile_post;
		save_pp = plus_plus;
		save_mode = atrbt_action;
			/**** reset atrbt action *****/
		atrbt_action = 7;       /*** remove all attributes ***/
			/**** undo all attributes on line ****/
		if(line_atrbt(x1,y1,search_radius,1)) /* find first point,set anchorx,y */
			return(1);
		if(line_atrbt(x,y,search_radius,0))   /* remove attributes */
			return(1);
				/*** reset first point & cursor type ***/
		cursor(x+1,y);
		cursor(x,y);
			/**** reset attribute etc. ***/
		mile_post = save_mp;
		plus_plus = save_pp;
		atrbt_action = save_mode;
			/**** set input mode to clear ***/ 
		change_mode(0,0);
			/**** done ****/
		break;
	default:
		return(1);
	}

		/************** reset undo_array item **************************/
	undo_array[undo_index].mode = 0;

		/************** decrement undo_index ***************************/
	undo_index = (undo_index + 31) & 31;

		/************** reset current mode *****************************/
	input_mode = current_mode;

		/************** return *****************************************/
	return(0);
}
