/*******************************************************************************
*	backupM		(see manual, Command Ref., Function Manager)	<fm>   *
*                                                                              *
*	            	Either calls the backup function or writes the         *
*			backup command to a macro file.                        *
*                                                                              *
*                    	Calls parser to extract  mapdir from  parms,           *
*			optionally passed from function junction (fj),         *
*			and passes that name to the backup function for        *
*			processing.  If no  mapdir argument was found,         *
*			backupM assumes that the action is to be performed     *
*			upon the current, active map, and sets  mapdir equal   *
*			to that name (found in files.h).  If an invalid        *
*			argument is received from fj, backupM will prompt      *
*			the user for a correct name.                           *
*                                                                              *
*			Returns zero upon normal execution.                    *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#include <sys/stat.h>
#include <errno.h>
#include <ctype.h>
 
char * name_is_valid();		/*** Validates map name to backup   ***/
long   cmd_is_valid();		/*** Validates backup device choice ***/

long backupM(parms)
char *parms;
{
	long bkcmd;						/*** config backup cmd choice    ***/
	long num_parsed;				/*** number of items in parms    ***/
	char parsed_str[2][STRLIST];	/*** holds parsed strings        ***/
	char tmp_cmd[512];				/*** string passed to backup_ltp ***/
	char tmp_msg[STRLIST];
	char tmp_path[STRLIST];

/***  Check for parameters.  If present, they may include the mapname     ***/
/***  and/or the backup commands to use.  If no parameters are present,   ***/
/***  the current map (identified by global 'map_path') is back up to     ***/
/***  the device listed in config.bkup_path by the tar command.           ***/

	if( strlen(trim(parms)) )
	{	
		switch( num_parsed = parse(parms, delim_buf, parsed_str, 2L) )
		{
/*------------------------------------------------------------------------*/
		case 1:			/*** Check for name or digit ***/
			if( isdigit(parsed_str[0][0]) )
			{   if( bkcmd = cmd_is_valid(parsed_str[0]) )
			    {	if( bkcmd == 1 )
				    sprintf(tmp_cmd, "%s %s %s", config.bkup11,
					map_path, config.bkup12);
				else
				    sprintf(tmp_cmd, "%s %s %s", config.bkup21,
					map_path, config.bkup22);
			    }
			}	/*** Else parm is a name. Does it exist? ***/
			else if( name_is_valid(parsed_str[0]) )
			    sprintf(tmp_cmd,"%s %s/%s", config.bkup_path,
					maps_path, parsed_str[0] );
			else
			    return(0);
			break;
/*------------------------------------------------------------------------*/
		case 2:	/*** Name and command specified. Validate both. ***/
			if( isalpha(parsed_str[0][0]) )
			{   if( name_is_valid(parsed_str[0]) )
			    {	if( bkcmd = cmd_is_valid(parsed_str[1]) )
			    	{   if( bkcmd == 1 )
				      sprintf(tmp_cmd,"%s %s %s",config.bkup11,
					 parsed_str[0], config.bkup12);
				    else
				      sprintf(tmp_cmd,"%s %s %s", config.bkup21,
					 parsed_str[0], config.bkup22);
				}
			    	else		/*** invalid device ***/
				{   message("Device must be either 1 or 2.");
				    return(0);
				}
			    }
			    else		/*** Bad mapname  ***/
				return(0);
			}
			else 
			{   if( name_is_valid(parsed_str[1]) )
			    {	if( bkcmd = cmd_is_valid(parsed_str[0]) )
			    	{   if( bkcmd == 1 )
				      sprintf(tmp_cmd,"%s %s %s",config.bkup11,
					 parsed_str[1], config.bkup12);
				    else
				      sprintf(tmp_cmd,"%s %s %s", config.bkup21,
					 parsed_str[1], config.bkup22);
				}
			    	else		/*** invalid device ***/
				{   message("Device must be either 1 or 2.");
				    return(0);
				}
			    }
			    else		/*** Bad mapname  ***/
				return(0);
			}
			break;
/*------------------------------------------------------------------------*/
		default:
			message("Usage: backup [mapname] [1 | 2] ");
			return(0);
			break;
		}
	}
	else		 /*** Use current map ***/
		sprintf(tmp_cmd, "tar cvf %s %s", config.bkup_path, map_path);

	if( !collecting_macro )
		if( backup_ltp( tmp_cmd ) )
		{	message("Unable to perform backup. Request cancelled.");
			return(0);
		}

	sprintf(raw_buf, "backup %s", parms);
	return(1);
}
/*****************************************************************************
*     cmd_is_valid                                                           *
*                Checks command number for value of 1 or 2.  If invalid      *
*                value,  prompt user to enter command value or to quit.      *
*                Returns 1 or 2 if valid cmd, else returns 0.                *
*****************************************************************************/
long cmd_is_valid(cmd)
	char *cmd;
{
	long cmd_num;
	char resp[STRLIST];

	cmd_num = strtol(cmd, (char **)NULL, 10);
	for( ;; )
	{	if( !((cmd_num == 1) ? 1 : (cmd_num == 2) ? 2 : 0))
		{	message("Invalid choice.  Use 1 or 2 for valid commands [RETURN to quit].");
			get_string(resp);
			if( strlen( trim(resp) ) == 0 )
			{	message("Backup terminated by user.");
				return(0);
			}
			cmd_num = strtol(resp, (char **)NULL, 10);
		}
		else
			return(cmd_num);
	}
}

/*****************************************************************************
*     name_is_valid                                                          *
*                Checks is passed name exists as an LTPlus map directory.    *
*                Returns valid string, else returns a NULL string.           *
*****************************************************************************/
char * name_is_valid(name)
	char *name;
{
	extern int errno;
	char tmp_path[STRLIST];
	char tmp_msg[STRLIST];
	struct stat st_buf;

	sprintf(tmp_path, "%s/%s", maps_path, name);
	if( stat(tmp_path, &st_buf) == -1 )
	{	if( errno == ENOENT )
			sprintf(tmp_msg, "%s does not exist. ", name);
		else if( errno == ENOTDIR )
			sprintf(tmp_msg, "%s is not a directory. ", name);
		else if( errno == EACCES )
			sprintf(tmp_msg, "Access denied to %s. ",name);
		else
			sprintf(tmp_msg, "Problem accessing %s. ", name);
		message(tmp_msg);
		append_msg(" Backup not performed.");
		tmp_path[0] = (char)NULL;
	}
	else		/*** Use path that exists  ***/
		strcpy(name, tmp_path);
	
	return(tmp_path);
}
