/*******************************************************************************
*	ckeys_dispM	(see manual, Command Ref., Function Manager)	<fm>   *
*			Description:  Parses string parameter and checks       *
*			arguments for their presence and validity.             *
*			Parameters:  A string containing the function name     *
*			and its arguments.  The following values are           *
*			searched for:                                          *
*			arg1:  print flag (-p: print curdor key definitions)   *
*			arg2:  cursor function file name                       *
*			Globals Modified:   none                               *
*                                                                              *
*			Returns 1 upon normal execution, else returns 0.       *
*			    C. Jonson  9/22/89                                 *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for multi user support */

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#include <sys/stat.h>
 
long ckeys_dispM(parms)
	char *parms;
{
	long   parse_num;
	long   print_ckeys;
	char   ckeys_path[256];
	char   tmp_str[256];
	char   ckeys_name[STRLIST];
	char   parsed[2][STRLIST];
	struct stat buf;

/*** See how many parameters are in parm string.                     ***/
	parse_num = parse( parms, delim_buf, parsed, 2L );

/*** If incorrect number of parms, null parsed strngs to force prompting ***/
	switch(parse_num)
	{
		case 1:
#ifdef MULTIUSER
			sprintf(ckeys_path, "%s/.ltp/cursor_keys/%s", home_path, parsed[0]);
			if(!file_exists(ckeys_path,""))
				sprintf(ckeys_path, "%s/cursor_keys/%s", ltp_path, parsed[0]);
#else
			sprintf(ckeys_path, "%s/cursor_keys/%s", ltp_path, parsed[0]);
#endif
			print_ckeys = FALSE;
			break;
		case 2:
			if( !strncmp(parsed[0], "-p", 2) )
			{	print_ckeys = TRUE;
#ifdef MULTIUSER
				sprintf(ckeys_path, "%s/.ltp/cursor_keys/%s", home_path, parsed[1]);
				if(!file_exists(ckeys_path,""))
					sprintf(ckeys_path, "%s/cursor_keys/%s", ltp_path, parsed[1]);
#else
				sprintf(ckeys_path, "%s/cursor_keys/%s", ltp_path, parsed[1]);
#endif
			}
			else if( !strncmp(parsed[1], "-p", 2) )
			{	print_ckeys = TRUE;
#ifdef MULTIUSER
				sprintf(ckeys_path, "%s/.ltp/cursor_keys/%s", home_path, parsed[0]);
				if(!file_exists(ckeys_path,""))
					sprintf(ckeys_path, "%s/cursor_keys/%s", ltp_path, parsed[0]);
#else
				sprintf(ckeys_path, "%s/cursor_keys/%s", ltp_path, parsed[0]);
#endif
			}
			else
			{	print_ckeys = FALSE;
				ckeys_path[0] = 0;		/* NULL out the ckeys_path */
			}
			break;
		default:		/* ask if file is to be printed. (dflt = no) */
			if( answerisyes(264, 0) )
				print_ckeys = TRUE;
			else
				print_ckeys = FALSE;
#ifdef MULTIUSER
			sprintf(ckeys_path, "%s/.ltp/cursor_keys", home_path);
			if(file_exists(ckeys_path,"")) {
			    /*** Show user list of cursor key file to choose from ***/
			    if( !get_fnm(ckeys_path, ckeys_name) ) {
				   /* get_fnm 0 = none chosen */
			    	sprintf(ckeys_path, "%s/cursor_keys", ltp_path);
				if( !get_fnm(ckeys_path, ckeys_name) ) {
				    message("none chosen. ckeys_disp terminated.  bye.");
				    return(0);
				}
				else
				    sprintf(ckeys_path, "%s/cursor_keys/%s", ltp_path, ckeys_name);
			    }
			    else
			        sprintf(ckeys_path, "%s/.ltp/cursor_keys/%s", home_path, ckeys_name);
			}
			else {
			   sprintf(ckeys_path, "%s/cursor_keys", ltp_path);
			   if( !get_fnm(ckeys_path, ckeys_name) ) {
				message("none chosen. ckeys_disp terminated.  bye.");
				return(0);
			   }
			   else
			      sprintf(ckeys_path, "%s/cursor_keys/%s", ltp_path, ckeys_name);
			}
#else
			sprintf(ckeys_path, "%s/cursor_keys", ltp_path);
			/*** Show user list of cursor key file to choose from ***/
			if( !get_fnm(ckeys_path, ckeys_name) )	/* 0 = none chosen */
			{	message("none chosen. ckeys_disp terminated.  bye.");
				return(0);
			}
			sprintf(ckeys_path, "%s/cursor_keys/%s", ltp_path, ckeys_name);
#endif
 
			break;
	}

/*** See if the file exists.  If not, get another  ***/
	if( ckeys_path[0] != '/' )
	{	sprintf(tmp_str, "%s/cursor_keys/%s", ltp_path, ckeys_path);
		strcpy(ckeys_path, tmp_str);
	}
	while( stat(ckeys_path, &buf) == -1 )
	{	sprintf(tmp_str, "%s does not exist.  Check for correct path.",
			ckeys_path );
		message(tmp_str);
		if(!valid_string(ckeys_path, "Enter cursor_keys filename. [q = QUIT]"))
		{	message("User terminated ckey_disp.  Bye.");
			return(0);
		}
		if( ckeys_path[0] != '/' ) {	
		   sprintf(tmp_str, "%s/.ltp/cursor_keys/%s", home_path, ckeys_path);
		   if (!file_exists(tmp_str, ""))
		    
			sprintf(tmp_str, "%s/cursor_keys/%s", ltp_path, ckeys_path);
		   strcpy(ckeys_path, tmp_str);
		}
	}

/*** Pass all parms to ckey_disp and, unless collecting macro ***/
	if( !collecting_macro )
		if(ckeys_disp(print_ckeys, ckeys_path))
		{	raw_buf[0] = '\0';       /* null out raw buf */
			return(0);
		}
	if( print_ckeys )
		sprintf(raw_buf, "ckeys_disp -p %s",  ckeys_path);
	else
		sprintf(raw_buf, "ckeys_disp %s",  ckeys_path);

	return(1);
}
