/*******************************************************************************
*	createM		(see manual, Command Ref., Function Manager)	<fm>   *
*                                                                              *
*	        Creates a new map file in the default maps directory.       *
*	        First, the existing map is checked for changes.  If there   *
*	        changes, the user is asked if s/he wants them stored.       *
*			Next, the input parameter is taken as the map name if it is  *
*           1-14 characters and doesn't already exist.  if the parameter       *
*           is 0 characters, or >14 chanacters, or already exists, then the  *
*			user is asked for a new map name, and it is given the  *
*			same tests.                                            *
*                                                                              *
*			Returns 1 if successful, 0 if not.                     *
*           Craig Jonson, John Dabritz 6/1/89                                  *
*******************************************************************************/
 
#include <string.h> 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#define LISTLENGTH 1024
 
 
long createM(parameters)
char *parameters;
{
	char list[LISTLENGTH][STRLIST];
	char name[STRLIST];
	char path[512];
	long i;
	long no_items;
	long ok;


	/********************* check existing map and save if wanted ********/
	if(!collecting_macro)
	{	path[0] = (char) 0;
		if(changes())
		{	if(!batch_mode)
			{	if(answerisyes(210,1))
					save(path);
			}
		}
	}

	/********************* get map name to be created *******************/
	raw_buf[0] = 0;                                     /*  set to null   */
	no_items = dir_list(maps_path,list[0],LISTLENGTH);  /*  get list of maps */
	if(batch_mode)
	{	strcpy(name,parameters);
		trim(name);
		name[14] = 0;
		if(!name[0])
			return(0);
		/***************** check its length          ********************/
		if(strlen(name) > 14)
			return(0);
		/***************** check to see if it exists ********************/
		else 
		{	for(i=0, ok=1; (i<no_items) && (ok) ; i++)
			{	if(!strcmp(name,list[i]))
					return(0);
			}
		}
	}
	else for(ok=0; !ok ;)
	{
		{	if(trim(parameters)[0])  /* parameter input, use it  */
				strcpy(name,parameters);
			else                             /* no parameter, get input   */
			{	message(prompt[81]);
				trim(get_string(name));
				if(name[0] == 0)           /* user returned a blank string */
				{	append_msg("  ** aborted **");
					return(0);
				}
			}
			/***************** check its length          ********************/
			if(strlen(name) > 14)
			{	append_msg("   << string is longer than 14 characters");
				ok = 0;
				parameters[0] = 0;       /*  set to null */
			}
			/***************** check to see if it exists ********************/
			else 
			{	for(i=0, ok=1; (i<no_items) && (ok) ; i++)
				{	if(!strcmp(name,list[i]))
					{	append_msg("   << duplicate map name.");
						ok = 0;
						parameters[0] = 0;       /*  set to null */
					}
				}
			}
		}
	}
			

	/********************* remove current map ***************************/
	remove_temp();
	


	/********************* null out header information ******************/
	if(!collecting_macro)
	{	header.comments[0] = 0;
		header.compress[0] = 0;
		header.contour_interval[0] = 0;
		header.datum[0] = 0;
		header.dictionary[0] = 0;
		header.edgematch[0] = 0;
		header.edge_e[0] = 0;
		header.edge_n[0] = 0;
		header.edge_s[0] = 0;
		header.edge_w[0] = 0;
		header.layer_name[0] = 0;
		header.map_ht[0] = 0;
		header.map_wd[0] = 0;
		header.maptype[0] = 0;
		header.menu_set[0] = 0;
		header.quad_unit[0] = 0;
		header.revision_yr[0] = 0;
		header.scale[0] = 0;
		header.search_rad[0] = 0;
		header.source_yr[0] = 0;
		header.state[0] = 0;
		header.stplzn[0] = 0;
		header.udrn[0] = 0;
		header.utmln[0] = 0;
		header.utmlt[0] = 0;
	}
	

	/********************* get default job_stats data *******************/
	if(!collecting_macro)
	{	for(i=3; i<100; i++)     /*  initialize to 0  */
			job_stats[i] = 0;
	
		job_stats[0] = (long) (ltp_version * 100); /*  file version number */
	
				/****  raster width   ****/
		if(sscanf(config.rasterw,"%ld",&(job_stats[1])) == EOF)
			job_stats[1] = 1023;          /*   max raster x value  (width-1)  */
		else
			job_stats[1] = job_stats[1] - 1;
		if(job_stats[1] <= 0)             /*   check for negative width  */
			job_stats[1] = 1023;          /*   max raster x value  (width-1) */
		job_stats[1] = ((((job_stats[1]-1)>>5)+1)<<5)-1; /*round up to mult 32*/
	
				/****  raster height   ****/
		if(sscanf(config.rasterh,"%ld",&(job_stats[2])) == EOF)
			job_stats[2] = 1023;          /*   max raster y value (height-1) */
		else
			job_stats[2] = job_stats[2] - 1;
		if(job_stats[2] <= 0)             /*   check for negative height  */
			job_stats[2] = 1023;          /*   max raster y value (height-1) */
	}
	
	
	/********************* create map file           ********************/
	if(!collecting_macro)
	{	if(!create(name))
			return(0);
	}


	/********************* set command string        ********************/
	sprintf(raw_buf,"create %s",name);    /* success, set command string */


	/********************* return                    ********************/
    return(1);
}
