/*******************************************************************************
*      drawM         (see manual, Command Ref., Function Manager)       <fm>   *
*                     Function manager for the draw function.                  *
*                                                                              *
*                     Queries for the draw mode if not present in string:      *
*                                   4 = draw fat lines                         *
*                                   5 = draw lines                             *
*                                   6 = draw points                            *
*                                   7 = draw rectangles                        *
*                                  16 = draw wedge                             *
*                                  28 = draw spur (3 pixels)                   *
*                     Queries for snap mode only if query is made for draw     *
*                     mode.                                                    *
*                                                                              *
*                     Calls function or writes command to macro being built    *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
 
drawM(parameters)
char *parameters;
{
	char parsed[3][STRLIST];
	char snaps[STRLIST];
	char answer[STRLIST];

	long mode;
	long snap;
	long no_params;
	long i;

		/************** break up parameters ***************************/
	no_params = parse(lowerc(trim(parameters)),delim_buf,parsed,3);
	for(i=0; i < no_params; i++)
		trim(lowerc(parsed[i]));

		/************** check first parameter for '-s' ****************/
	snap = 0;
	if(no_params)
	{	if(!strcmp(parsed[0],"-s"))
			snap = 1;
	}

		/************** check first/next parameter for draw type ******/
	mode = 0;
	if(no_params > snap)
	{	switch(*parsed[snap])   {
		case 'p':            /* draw point mode      */
			mode = 4;
			break;
		case 'l':            /* draw line mode      */
			mode = 5;
			break;
		case 'f':            /* draw fat_line mode      */
			mode = 6;
			break;
		case 'r':            /* draw rectangle mode      */
			mode = 7;
			break;
		case 'w':            /* draw wedge mode          */
			mode = 16;
			break;
		case 's':            /* draw spur                */
			mode = 28;
			snap = 1;
			break;
		}
	}

		/************** query for draw type (& snap) if not specified */
	if(!batch_mode && !mode)
	{	message(prompt[224]);
		get_string(answer);
		switch(*lowerc(trim(answer)))  {
		case 'n':            /* none  */
			return(0);
		case 'p':            /* draw point mode      */
			mode = 4;
			break;
		case 'f':            /* draw fat_line mode      */
			mode = 6;
			break;
		case 'r':            /* draw rectangle mode      */
			mode = 7;
			break;
		case 'w':            /* draw wedge mode          */
			mode = 16;
			break;
		case 's':            /* draw spur mode           */
			mode = 28;
			break;
		default:            /* draw line mode      */
			mode = 5;
			break;
		}
			/*    see if user wants snap mode    */
		if(mode == 28)
			snap = 1;
		else
			snap = answerisyes(140,0);
	}

		/************** execute input mode change *********************/
	if(!collecting_macro)
	{	if(draw(mode,snap))
			return(0);
	}

		/************** build string **********************************/
	*snaps = 0;
	if(snap)
		sprintf(snaps," -s");
	switch(mode)  {
	case 4:            /* draw point mode      */
		sprintf(raw_buf,"draw%s point",snaps);
		break;
	case 5:            /* draw line mode      */
		sprintf(raw_buf,"draw%s line",snaps);
		break;
	case 6:            /* draw fat_line mode      */
		sprintf(raw_buf,"draw%s fat_line",snaps);
		break;
	case 7:            /* draw rectangle mode      */
		sprintf(raw_buf,"draw%s rectangle",snaps);
		break;
	case 16:            /* draw wedge mode          */
		sprintf(raw_buf,"draw%s wedge",snaps);
		break;
	case 28:            /* draw spur mode          */
		sprintf(raw_buf,"draw%s spur",snaps);
		break;
	}
	
		/************** return ****************************************/
    return(1);
}
