/*******************************************************************************
*	eraseM		(see manual, Command Ref., Function Manager)	<fm>   *
*			Function manager for the erase function.               *
*                                                                              *
*			Queries for the erase mode if not present in string:   *
*	                            	8 = erase fat lines                     *
*					9 = erase lines                         *
*					10 = erase points                        *
*					11 = erase rectangles                    *
*			Queries for snap mode only if querry is made for erase *
*			mode. 						       *
*                                                                              *
*			Calls function or writes command to macro being built  *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
 
eraseM(parameters)
char *parameters;
{
	char parsed[3][STRLIST];
	char snaps[STRLIST];
	char answer[STRLIST];

	long mode;
	long snap;
	long no_params;
	long i;

		/************** break up parameters ***************************/
	no_params = parse(lowerc(trim(parameters)),delim_buf,parsed,3);
	for(i=0; i < no_params; i++)
		trim(lowerc(parsed[i]));

		/************** check first parameter for '-s' ****************/
	snap = 0;
	if(no_params)
	{	if(!strcmp(parsed[0],"-s"))
			snap = 1;
	}

		/************** check first/next parameter for erase type ******/
	mode = 0;
	if(no_params > snap)
	{	switch(*parsed[snap])   {
		case 'p':            /* erase point mode      */
			mode = 8;
			break;
		case 'l':            /* erase line mode      */
			mode = 9;
			break;
		case 'f':            /* erase fat_line mode      */
			mode = 10;
			break;
		case 'r':            /* erase rectangle mode      */
			mode = 11;
			break;
		case 'w':            /* erase wedge mode      */
			mode = 17;
			break;
		}
	}

		/************** query for erase type (& snap) if not specified */
	if(!batch_mode && !mode)
	{	message(prompt[225]);
		get_string(answer);
		switch(*lowerc(trim(answer)))  {
		case 'n':            /* none  */
			return(0);
		case 'p':            /* erase point mode      */
			mode = 8;
			break;
		case 'f':            /* erase fat_line mode      */
			mode = 10;
			break;
		case 'r':            /* erase rectangle mode      */
			mode = 11;
			break;
		case 'w':            /* erase wedge mode      */
			mode = 17;
			break;
		default:            /* erase line mode      */
			mode = 9;
			break;
		}
			/*    see if user wants snap mode    */
		snap = answerisyes(140,0);
	}

		/************** execute input mode change *********************/
	if(!collecting_macro)
	{	if(erase(mode,snap))
			return(0);
	}

		/************** build string **********************************/
	*snaps = 0;
	if(snap)
		sprintf(snaps," -s");
	switch(mode)  {
	case 8:            /* erase point mode      */
		sprintf(raw_buf,"erase%s point",snaps);
		break;
	case 9:            /* erase line mode      */
		sprintf(raw_buf,"erase%s line",snaps);
		break;
	case 10:            /* erase fat_line mode      */
		sprintf(raw_buf,"erase%s fat_line",snaps);
		break;
	case 11:            /* erase rectangle mode      */
		sprintf(raw_buf,"erase%s rectangle",snaps);
		break;
	case 17:            /* erase wedge mode      */
		sprintf(raw_buf,"erase%s wedge",snaps);
		break;
	}
	
		/************** return ****************************************/
    return(1);
}
