/*******************************************************************************
*	exportM		(see manual, Command Ref., Function Manager)	<fm>   *
*                                                                              *
*			Either calls the appropriate export function or        *
*			writes the export command to a macro file.             *
*                                                                              *
*	           	Parses  parameters  to determine which export function to   *
*			call, based upon the format type ( ftype) value.       *
*			The function name is composed of the prefix "ex"       *
*			concatenated with the  ftype value.  The export_fm     *
*			function checks functions.h to insure that the         *
*			specified function exists and reports an error if      *
*			it is not valid.  It also checks each type, prior      *
*			to any call, to determine that the proper number       *
*			of arguments have been parsed from  params.  Any       *
*			missing arguments result in a call to prompts to       *
*			query the user for the missing information.  The       *
*			exportM function must also find (during parsing)       *
*			an output file name ( ename) or the the user will      *
*			be prompted for one.                                   *
*                                                                             *
*                       Returns 1 upon normal execution, 0 on failure          *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#include <string.h>
 
 
long exportM(parameters)
char *parameters;
{
	char parsed[20][STRLIST];
	char fname[256];
	char str[STRLIST];
	static long fmt_indx[20] = { 0, 1, 2, 3, 11, 9, 4, 5, 6, 7, 8, 10, 12, 13 };
	static char formats[20][STRLIST] = {
	   	/*  0 */  	" - none, lemme out, please?",
		/*  1 */	"cell   (a generic cell format)", 
		/*  2 */	"cff    (cartographic feature file, [USFS, GSC])",
		/*  3 */	"dem    (digital elevation model, [USGS])",
		/* 11 */ "dig    (vector & attribute data for ERDAS input)",
		/*  9 */ "digit  (vector & attribute data for GRASS GIS [SCS])",
		/*  4 */ "dlg3   (digital line graph, level 3, optional [USGS])",
		/*  5 */	"dxf    (as used by many cad & other programs [Autodesk])",
		/*  6 */	"moss   (MOSS import/export format)",
		/*  7 */	"rdef2  (Raster Data Exchange Format [USFS, DWRIS/LIDES/LTPlus])",
		/*  8 */	"    rstr (a generic bit-mapped raster format) - planned",
		/* 10 */ "scsgef (SCS Geographics Exchange Format [SCS])",
		/* 12 */ "snap   (USDA-FS PC-Snap software format)",
		/* 13 */ "xy     (generic/user specified xy vector format)"
										};
	static char head_types[5][STRLIST] = {
		" - I'm lost, please, oh please let me out.",
		"NO HEADER lines for objects.",
		"header, attribute IN HEADER line for each object.",
		"header, attribute in vertex line(s) for each object.",
		"HEADER LINES ONLY, with object centroid location."  };

	long promptem;
	long no_params;
	long fmt;        /** format codes (as above)  **/
	long a, at, c, g, r, s, t;
	long e1, e2, st, f;
	long otype;
	long feature_flag;
	long i;
	long w,h;
	long cd,dl,dp,ht,i1,i2,a1,a2,v1,v2,x1,x2,y1,y2; 

	double wd,hd;


		/************** get format type     ***************************/
	no_params = parse(trim(parameters),delim_buf,parsed,2);

		/************** initialize promptem flag and format type ******/
	fmt = 0;       /* none */
	promptem = 0;  /* set to 1 if & when prompting is necessary */

		/************** get format type *******************************/
	if(no_params == 0)
		promptem = 1;
	else
	{	lowerc(trim(parsed[0]));
		if(!strcmp("cff",parsed[0]))
			fmt = 2;
		else if(!strcmp("dem",parsed[0]))
			fmt = 3;
		else if(!strcmp("dig",parsed[0]))
			fmt = 11;
		else if(!strcmp("digit",parsed[0]))
			fmt = 9;
		else if(!strcmp("dlg3",parsed[0]))
			fmt = 4;
		else if(!strcmp("dxf",parsed[0]))
			fmt = 5;
		else if(!strcmp("moss",parsed[0]))
			fmt = 6;
		else if(!strcmp("rdef2",parsed[0]))
			fmt = 7;
		else if(!strcmp("scsgef",parsed[0]))
			fmt = 10;
		else if(!strcmp("cell",parsed[0]))
			fmt = 1;
		else if(!strcmp("snap",parsed[0]))
			fmt = 12;
		else if(!strcmp("xy",parsed[0]))
			fmt = 13;
		else
			promptem = 1;
	}

	if(!fmt  ||  promptem)
	{	if(batch_mode)
			return(0);
		fmt = fmt_indx[list_choice(formats[0],14,66)];
		if(!fmt)
			return(0);
	}

		/************** break into different sequences depending of format */
	switch(fmt)   {
	case 1: /************** cell format (generic cell format) *************/
			/****** parse string for 7 parameters *************************/
		no_params = parse(trim(parameters),delim_buf,parsed,8);

			/****** check for parameter 1, file name (fname) **************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);

			/****** check for parameter 2, number of characters/cell ******/
		if(get_long(parsed[2],&t,&promptem,349,1,30))
			return(0);
	
			/****** check for parameter 3, coordinate type code ***********/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);
	
			/****** check for parameter 4, cell width *********************/
		if(get_double(parsed[4],&wd,&promptem,68,(double) 0.0,(double) 999999999999.0))
			return(0);

			/****** check for parameter 5, cell height ********************/
		if(get_double(parsed[5],&hd,&promptem,69,(double) 0.0,(double) 999999999999.0))
			return(0);

			/****** check for parameter 6, characters to use for polys w/o atrbt */
		sprintf(str,"%d",parsed[6]);
		if(promptem)
		{	message(prompt[348]);
			get_string(str);
		}
			/** if str is <t char long, fill with blanks **/
		str[t] = (char) 0;
		for(i=strlen(str); i<t; i++)
			str[i] = (char) 32;

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(excell(fname,t,c,wd,hd,str))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export  cell  %s  %d  %d  %f  %f  %s",fname,t,c,wd,hd,str);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/
	case 2: /************** cff format (cartographic feature file) ********/
			/****** parse string for 3 parameters *************************/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name (fname) **************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);

			/****** check for parameter 2, coordinate type code ***********/
		if(get_long(parsed[2],&c,&promptem,50,0,19))
			return(0);
	
			/****** check for parameter 3, set to be exported *************/
		s = get_set(parsed[3],&promptem,0,31,202);
		if(s == -1)
			return(0);

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(excff(fname,c,s))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export  cff  %s  %d  %d",fname,c,s);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/
	case 3: /************** dem format (digital elevation model) **********/
			/****** parse string for 10 parameters *************************/
		no_params = parse(trim(parameters),delim_buf,parsed,10);

			/****** check for parameter 1, file name (fname) **************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);


			/****** check if user wants a standard DEM from feet contour **/
		e2 = 0;
		if((no_params < 9)  &&  (!batch_mode))
		{	if(answerisyes(242,1))
			{	c = 2;     /* utm grid                 */
				w = 30.0;  /* 30 meter grid wd.        */
				h = 30.0;  /* 30 meger grid ht.        */
				g = 2;     /* utm clipping             */
				e1= 1;     /* map atrbts in ft         */
				e2= 2;     /* dem elev in meters       */
			}
		}

		if(!e2)
		{		/****** check for paramerer 2, grid coord type code (c) *******/
			if(get_long(parsed[2],&c,&promptem,50,0,29))
				return(0);
	
				/****** check for parameter 3, cell width (w) *****************/
			if(get_long(parsed[3],&w,&promptem,68,0,99999999))
				return(0);
	
				/****** check for parameter 4, cell height (h) ****************/
			if(get_long(parsed[4],&h,&promptem,69,0,99999999))
				return(0);
	
				/****** check for parameter 5, clipping coord type code (g) ***/
			if(get_long(parsed[5],&g,&promptem,239,0,29))
				return(0);
	
				/****** check for parameter 6, map attribute unit code (e1) ***/
			if(get_long(parsed[6],&e1,&promptem,243,1,2))
				return(0);
	
				/****** check for parameter 7, dem elevation unit code (e2) ***/
			if(get_long(parsed[7],&e2,&promptem,244,1,2))
				return(0);
		}
	
			/****** check for parameter 8, smoothing type (st) ************/
		if(get_long(parsed[8],&st,&promptem,245,-200000,200000))
			return(0);
	
			/****** check for parameter 9, filter (smoothing) passes (f) **/
		if(st)
		{	if(get_long(parsed[9],&f,&promptem,315,0,3))
				return(0);
		}
		else
			st = 0;
	
			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(exdem(fname,c,w,h,g,e1,e2,st,f))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export dem %s %d %d %d %d %d %d %d %d",
		 fname,c,w,h,g,e1,e2,st,f);

			/****** return ************************************************/
		break;
	
		/*-----------------------------------------------------------------*/

	case 4: /************** dlg3 format (usgs digital line graph) *********/
			/****** parse string for 5 parameters (0 = fmt, get 1-4) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,6);

			/****** check for parameter 1, file name **********************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);

			/****** check for parameter 2, attribute code *****************/
		if(get_long(parsed[2],&a,&promptem,253,1,3))
			return(0);

			/****** check for parameter 3, attribute type code ************/
		if(a & 2)
		{	if(get_long(parsed[3],&at,&promptem,241,1,2))
				return(0);
		}
		else
			at = 1;

			/****** check for parameter 4, coord type code (utm/stpln) ****/
		if(get_long(parsed[4],&c,&promptem,238,2,3))
			return(0);

			/****** check for parameter 5, set to be exported *************/
		s = get_set(parsed[5],&promptem,0,31,202);
		if(s == -1)
			return(0);

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(exdlg3(fname,a,at,c,s))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export  dlg3  %s  %d  %d  %d  %d",fname,a,at,c,s);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 5: /************** dxf format ************************************/
			/****** parse string for 5 parameters (0 = fmt, get 1-4) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);

			/****** check for parameter 2, attribute code *****************/
		if(get_long(parsed[2],&a,&promptem,359,1,4))
			return(0);

			/****** check for parameter 3, coordinate type code ***********/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 4, set to be exported *************/
		s = get_set(parsed[4],&promptem,0,31,202);
		if(s == -1)
			return(0);

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(exdxf(fname,a,c,s))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export  dxf  %s  %d  %d  %d",fname,a,c,s);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 6: /************** moss import/export format *********************/
			/****** parse string for 6 parameters (0 = fmt, get 1-5) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,6);

			/****** check for parameter 1, file name **********************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);

			/****** check for parameter 2, attribute code *****************/
		if(get_long(parsed[2],&a,&promptem,253,1,3))
			return(0);

			/****** check for parameter 3, coordinate type code ***********/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 4, object type code ***************/
		if(get_long(parsed[4],&otype,&promptem,53,0,2))
			return(0);

			/****** check for parameter 5, set to be exported *************/
		s = get_set(parsed[5],&promptem,0,31,202);
		if(s == -1)
			return(0);

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(exmoss(fname,a,c,otype,s))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export  moss  %s  %d  %d  %d  %d",fname,a,c,otype,s);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 7: /************** rdef2 format **********************************/
			/****** parse string for 4 parameters (0 = fmt, get 1-3) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);

			/****** check for parameter 2, attribute code *****************/
		if(get_long(parsed[2],&a,&promptem,75,1,3))
			return(0);

			/****** check for parameter 3, registration code **************/
		if(get_long(parsed[3],&r,&promptem,254,0,1))
			return(0);

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(exrdef2(fname,a,r))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export  rdef2  %s  %d  %d",fname,a,r);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 9: /************** digit (GRASS import format) *******************/
			/****** parse string for 5 parameters (0 = fmt, get 1-4) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);
		fname[8] = 0;  /* trim file name string to 8 characters for extension */

			/****** check for parameter 2, attribute code *****************/
		if(get_long(parsed[2],&a,&promptem,253,1,3))
			return(0);

			/****** check for parameter 3, coordinate type code ***********/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 4, set to be exported *************/
		s = get_set(parsed[4],&promptem,0,31,202);
		if(s == -1)
			return(0);

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(exdigit(fname,a,c,s))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export  digit  %s  %d  %d  %d",fname,a,c,s);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 10: /************** scsgef format  ********************************/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, filename ***********************/
		if(get_fname(parsed[1], fname, &promptem, 109))
			return(0);
		fname[10] = 0;  /* trim file name str to 10 characters for extension */

			/****** check for parameter 2, feature file flag  *************/
		feature_flag = FALSE;
		if( !strncmp(parsed[2], "-f", 2) )
			feature_flag = TRUE;
		else if( !batch_mode )
		{	message("Would you like to create the optional Feature File?");
			if(answerisyes(0, 1) )
				feature_flag = TRUE;
		}


			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if( exscsgef(fname, feature_flag) )
				return(0);
		}

			/****** write command line ************************************/
		if( feature_flag )
			sprintf(raw_buf,"export  scsgef  %s  -f", fname);
		else
			sprintf(raw_buf,"export  scsgef  %s", fname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 11: /************** dig format ************************************/
			/****** parse string for 5 parameters (0 = fmt, get 1-4) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);

			/****** check for parameter 2, attribute code *****************/
		if(get_long(parsed[2],&a,&promptem,253,1,3))
			return(0);

			/****** check for parameter 3, coordinate type code ***********/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 4, set to be exported *************/
		s = get_set(parsed[4],&promptem,0,31,202);
		if(s == -1)
			return(0);

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(exdig(fname,a,c,s))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export  dig  %s  %d  %d  %d",fname,a,c,s);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 12: /************** snap   (USDA-FS PC-Snap software format *****/
			/****** parse string for 6 parameters (0 = fmt, get 1-4) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,6);

			/****** check for parameter 1, file name **********************/
		if(get_fname(parsed[1],fname,&promptem,350))
			return(0);

			/****** check for parameter 2, object type code ***************/
		if(get_long(parsed[2],&otype,&promptem,351,1,2))
			return(0);

			/****** check for parameter 3, coordinate type code ***********/
		if(get_long(parsed[3],&c,&promptem,352,2,3))
			return(0);

			/****** check for parameter 4, set to be exported *************/
		s = get_set(parsed[4],&promptem,0,31,202);
		if(s == -1)
			return(0);

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(exsnap(fname,otype,c,s))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"export  snap  %s  %d  %d  %d",fname,otype,c,s);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 13: /************** xy  generic/user defined  *********************/
			/****** parse string for 18 parameters (0 = fmt, get 1-17) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,18);

			/****** check for parameter 1, file name **********************/
		if(get_fname(parsed[1],fname,&promptem,109))
			return(0);

			/****** check for parameter 2, coordinate type code ***********/
		if(get_long(parsed[2],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 3, set to be exported *************/
		s = get_set(parsed[3],&promptem,0,31,202);
		if(s == -1)
			return(0);

			/****** check for parameter 4, cd, coord display type code ****/
		cd = 1;
		dl = 32;
		if(c == 1)
		{	if(get_long(parsed[4],&cd,&promptem,373,1,3))
				return(0);
		
				/****** check for parameter 5, dl, delimeter value ************/
			if(get_long(parsed[5],&dl,&promptem,374,32,127))
				return(0);
		}
			
			/****** check for parameter 6, dp, number of decimal places (x,y) ***/
		dp = 2;
		if(get_long(parsed[6],&dp,&promptem,375,0,16))
			return(0);
		
			/****** check for parameter 7, ht, header type code ***********/
		ht = 2;
		if(get_long_lst(parsed[7],&ht,&promptem,376,1,4,head_types,5))
			return(0);

			/****** check for parameter 8, i1, item number begin column ***/
		i1 = 0;
		if(get_long(parsed[8],&i1,&promptem,377,0,256))
			return(0);
	
			/****** check for parameter 9, i2, item number end column *****/
		if(i1 > 0)
		{	if(get_long(parsed[9],&i2,&promptem,378,0,256))
				return(0);
		}
		else
			i2 = 0;

			/****** check for parameter 10, a1, attribute begin column *****/
		a1 = 0;
		if(get_long(parsed[10],&a1,&promptem,379,0,256))
			return(0);
	
			/****** check for parameter 11, a2, attribute end column ******/
		if(a1 > 0)
		{	if(get_long(parsed[11],&a2,&promptem,380,0,256))
				return(0);
		}
		else
			a2 = 0;

			/****** check for parameter 12, v1, attribute begin column *****/
		v1 = 0;
		if(get_long(parsed[12],&v1,&promptem,385,0,256))
			return(0);
	
			/****** check for parameter 13, v2, attribute end column ******/
		if(v1 > 0)
		{	if(get_long(parsed[13],&v2,&promptem,386,0,256))
				return(0);
		}
		else
			v2 = 0;

			/****** check for parameter 14, x1, x coord begin column ******/
		x1 = 0;
		if(get_long(parsed[14],&x1,&promptem,381,0,256))
			return(0);
	
			/****** check for parameter 15, x2, x coord end column ********/
		if(x1 > 0)
		{	if(get_long(parsed[15],&x2,&promptem,382,0,256))
				return(0);
		}
		else
			x2 = 0;

			/****** check for parameter 16, y1, y coord begin column ******/
		y1 = 0;
		if(get_long(parsed[16],&y1,&promptem,383,0,256))
			return(0);
	
			/****** check for parameter 17, y2, y coord end column ********/
		if(y1 > 0)
		{	if(get_long(parsed[17],&y2,&promptem,384,0,256))
				return(0);
		}
		else
			y2 = 0;

			/****** call export program ***********************************/
		if(!collecting_macro)
		{	if(exxy(fname,c,s,cd,dl,dp,ht,i1,i2,a1,a2,v1,v2,x1,x2,y1,y2))
				return(0);
		}

			/****** write command line ************************************/
			/*                 0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 151617*/
	sprintf(raw_buf,"export xy %s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
		                     fname, c, s,cd,dl,dp,ht,i1,i2,a1,a2,v1,v2,x1,x2,y1,y2);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	default:/************** bad format type *******************************/
			/****** ----- template ------- ********************************/
			/****** check for parameter X, whatever (w) *******************/
			/****** check for parameter X, whatever (w) *******************/
			/****** check for parameter X, whatever (w) *******************/
			/****** check for parameter X, whatever (w) *******************/
			/****** call export program ***********************************/
			/****** write command line ************************************/
			/****** return ************************************************/
			/****** *******************************************************/
			/****** *******************************************************/
		message("format not yet available.");
		return(0);

		/*-----------------------------------------------------------------*/

			/************** ***********************************************/

		/*-----------------------------------------------------------------*/

			/************** ***********************************************/

		/*-----------------------------------------------------------------*/

			/************** ***********************************************/

		/*-----------------------------------------------------------------*/
	}
	
		/************** return ****************************************/
    return(1);
}
