/*******************************************************************************
*	ginputM		(see manual, Command Ref., Function Manager)	<fm>   *
*	 		Checks for device parameter.  Queries user for device  *
*			type if necessary.                                     *
*                                                                              *
*			Either calls ginput() or writes ginput command         *
*			line to the macro being built.                         *
*                                                                              *
*			Returns zero upon normal end-of-job (EOJ).             *
*******************************************************************************/
 
#include "../../incl/globals.h"
 
long check_dev();

long ginputM(parms)
char *parms;
{
/*  Locals  */
	char dev[32];
	long abs_mode = 0;
	long choice;
	long i;
	long no_params;
	long debug;

	char parsed_str[2][STRLIST];
	static char dev_list[][STRLIST] = {"- none -", "dig1", "dig2", "mouse" };

/***  Check parms  ***/
	choice = 0;
	debug = 0;
	no_params =  parse( parms, delim_buf, parsed_str, 2L); 
	no_params = (no_params < 3) ?  no_params : 2;
	for(i=0; i < no_params; i++)
	{	trim(parsed_str[i]);
		if(*parsed_str[i] == 'm')
			choice = 3;                    /* mouse  */
		else if(strstr(parsed_str[i],"1"))
			choice = 1;                    /* dig1   */
		else if(strstr(parsed_str[i],"2"))
			choice = 2;                    /* dig2   */
		else if(strstr(parsed_str[i],"-d"))
			debug = 1;                   /* dig debug */
		else if(strstr(parsed_str[i],"-a"))
			abs_mode = 1;
	}

		/*************** prompt if necessary *************************/
	if(!batch_mode)
	{	if(!choice)
		{	choice = list_choice(dev_list, 4, 223);
			if(choice  &&  (choice != 3)  &&  (!abs_mode))
				abs_mode = answerisyes( 221 ,2);
		}
	}

		/*************** check input *********************************/
	if((choice <1) || (choice > 4))
		return(0);

	if( abs_mode )
	{	
					 /***  is map loaded? ***/
		if(!job_stats[0]  &&  !collecting_macro)
		{	append_msg("  Map not loaded.  Acquire map first.");
			return(0);
		}
		
			/*** check for absolute mouse input (not allowed.)    **/
		if(choice == 3)
		{	message("You canot have mouse input in absolute coordinates.");
			return(0);
		}

			/*** check for both digitizer & raster registration (required) **/
		if(!registered(0)  &&  !collecting_macro)
		{	message("You must register the raster before absolute mode digitizer input.");
			return(0);
		}
		if(!registered(3+choice)  &&  !collecting_macro)
		{	message("You must register the digitizer before absolute mode digitizer input.");
			return(0);
		}
	}

	if(!collecting_macro)
	{	if( ginput( choice, abs_mode, debug ) )
			return(0);
	}

	switch(choice)  {
	case 1:             /*  dig1  */
	case 2:             /*  dig2  */
		if(abs_mode)
			sprintf(raw_buf,"ginput -a dig%d",choice);
		else
			sprintf(raw_buf,"ginput dig%d",choice);
		break;
	case 3:
		sprintf(raw_buf,"ginput mouse");
		break;
	}
	if(debug)
		strcat(raw_buf," -d");
	
	return(1);
}			/** End of ginputM()  **/
