/*******************************************************************************
*	gotoM 	    (see manual, Command Ref., Function Manager)	<fm>           *
*			Function manager for the log function.                             *
*			Queries for user input if necessary.                               *
*                                                                              *
*			Calls function or writes command to macro being built              *
*                                                                              *
*           Returns 1 on successful completion, 0 on failure.                  *
*           J.Dabritz 8/19/89                                                  *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
long gotoM(parameters)
char *parameters;
{
	char parsed[3][STRLIST];
	char string[STRLIST];

	long i;
	long gtype;
	long no_params;

	double arg1, arg2;


		/************** break up parameters ***************************/
	no_params = parse(lowerc(trim(parameters)),delim_buf,parsed,3);
	for(i=0; i < no_params; i++)
		trim(lowerc(parsed[i]));

		/************** check first parameter (gtype)  ****************/
	gtype = -1;
	arg1 = -3.0;
	arg2 = -3.0;
	if(no_params)
	{	switch(*parsed[0])    {
		case 'r':                 /**** object record no ****/
			gtype = 0;
			break;
		case 's':                 /**** object serial no  ****/
			gtype = 1;
			break;
		case 'c':                 /**** coordinate location ****/
			gtype = 2;
			break;
		case 'l':                 /**** logged location ****/
			gtype = 3;
			break;
		default:                  /**** bad entry ****/
			break;
		}
	}

	if(no_params >= 2)
		sscanf(parsed[1],"%lf",&arg1);
	if(no_params >= 3)
		sscanf(parsed[2],"%lf",&arg2);


		/************* query for first parameter if necessary *******/
	if(gtype == -1)
	{	if(batch_mode)
			return(0);
		message(prompt[54]);
		get_string(string);
		trim(lowerc(string));
		switch(*string)    {
		case 'r':                 /**** object record no ****/
			gtype = 0;
			break;
		case 's':                 /**** object serial no  ****/
			gtype = 1;
			break;
		case 'c':                 /**** coordinate location ****/
			gtype = 2;
			break;
		case 'l':                 /**** logged location ****/
			gtype = 3;
			break;
		default:                  /**** bad entry ****/
			return(0);
		}
	}

		/*** check remaining needed parameters ***/
	switch(gtype)    {
	case  0 :                 /**** object record no ****/
		if((no_params < 2)  ||  (arg1 < 0.0))
		{	if(batch_mode)
				return(1);
			message(prompt[56]);
			trim(get_string(string));
			if((*string >= '0') && (*string <= '9'))
			{	if(!sscanf(string,"%lf",&arg1))
					return(0);
			}
			else
				return(0);
		}
		sprintf(string,"goto record %.0f",arg1);
		break;
	case  1 :                 /**** object serial no  ****/
		if((no_params < 2)  ||  (arg1 < 0.0))
		{	if(batch_mode)
				return(1);
			message(prompt[57]);
			trim(get_string(string));
			if((*string >= '0') && (*string <= '9'))
			{	if(!sscanf(string,"%lf",&arg1))
					return(0);
			}
			else
				return(0);
		}
		sprintf(string,"goto serial %.0f",arg1);
		break;
	case  2 :                 /**** coordinate location ****/
		if(no_params < 3)
		{	if(batch_mode)
				return(1);
			message(prompt[236]);
			trim(get_string(string));
			if(((*string >= '0') && (*string <= '9')) || (*string == '-'))
			{	if(2 > sscanf(string,"%lf %lf",&arg1,&arg2))
					return(0);
			}
			else
				return(0);
		}
		sprintf(string,"goto coordinate %f %f",arg1,arg2);
		break;
	case  3 :                 /**** logged location ****/
		if(no_params >= 2)
		{	if(*parsed[1] == '+')
				arg1 = -1.0;
			else if(*parsed[1] == '-')
				arg1 = -2.0;
			else if((*parsed[1] >= '0')  &&  (*parsed[1] <= '9'))
				sscanf(parsed[1],"%lf",&arg1);
		}

		if(arg1 < -2.0)                        
		{	if(batch_mode)
				return(1);
			message(prompt[237]);
			trim(get_string(string));
			if(*string == '+')
				arg1 = -1.0;
			else if(*string == '-')
				arg1 = -2.0;
			else if((*string >= '0')  &&  (*string <= '9'))
				sscanf(string ,"%lf",&arg1);
			if(arg1 < -2.0)
				return(0);
		}
		if((arg1 < 0.0)  && (arg1 > -1.5))
			sprintf(string,"goto log +");
		else if(arg1 < -1.5)
			sprintf(string,"goto log -");
		else
			sprintf(string,"goto log %.0f",arg1);
		break;
	}
		

		/************** execute input mode change *********************/
	if(!collecting_macro)
	{	if(goto_ltp(gtype,arg1,arg2))
			return(0);
	}

		/************** return ****************************************/
	sprintf(raw_buf,"%s",string);
    return(1);
}
