/*******************************************************************************
*	graphicsM	(see manual, Command Ref., Function Manager)	<fm>   *
*			Checks for (- optional) parameters.  Queries for all   *
*			if none are present.                                   *
*                                                                              *
*			Either calls graphics() or writes graphics command     *
*			line to the macro being built.                         *
*                                                                              *
*			Returns one  upon normal end-of-job (EOJ).             *
*           J.Dabritz      8/13/89                                             *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
 
long graphicsM(parameters)
char *parameters;
{
	char parsed[25][STRLIST];
	char string[256], str2[256];
	char *ptr;
	char str[256];

	long raster_disp;               /*** flag, 1= display raster  **/
	long centroid_disp;             /*** flag, 1= display poly centroids **/
	long full_width;                /*** flag, 0=no, 1=yes **/
	long normal_v;        /*  bit flags for levels  0-31 display  */
	long normal_i1;       /*  bit flags for levels 32-63 display  */
	long normal_i2;       /*  bit flags for levels 64 95 display  */
	long highlight_v;     /*  bit flags for levels  0-31 display  */
	long highlight_i1;     /*  bit flags for levels 32-63 display  */
	long highlight_i2;     /*  bit flags for levels 64-95 display  */
	long no_params;
	long i,n;

		/*************** break parameters string into individual parameters **/
	no_params = parse(lowerc(trim(parameters)),delim_buf,parsed,25);

		/*************** set default values **************************/
	normal_v =  disp_normal;               /*  default values */
	normal_i1 = disp_i1normal;
	normal_i2 = disp_i2normal;
	highlight_v = disp_hglight;            /*  default values */
	highlight_i1  = disp_i1hglight;
	highlight_i2  = disp_i2hglight;
	full_width = full_width_disp;             /*  default values */
	raster_disp = disp_raster;                  /*  default values */
	centroid_disp = disp_centroid;             /* default values */

		/*************** interpret parameters & build command string **/
	sprintf(string,"graphics ");
	if(no_params)
	{	for(i=0; i<no_params; i++)
		{	trim(parsed[i]);
			ptr = parsed[i];
			if((*ptr == '-')  ||  (*ptr == '+'))
			{	if(*ptr == '+')
				{	ptr++;
					switch(*ptr)   {
					case 'c':         /** display poly centroids **/
						centroid_disp = 1;
						strcat(string,"+c ");
						break;
					case 'r':         /** suppress the raster **/
						raster_disp = 1;
						strcat(string,"+r ");
						break;
					case 'f':         /** use full screen width flag **/
						full_width = 1;
						strcat(string,"+f ");
						break;
					case 'n':         /** normal vector display flag **/
							/*** get item number ***/
						n = -1;
						sscanf((char *) (ptr + 1),"%ld", &n);
						if(!batch_mode && (n == -1))
							n = choose_level(0,95,248);

						if((n >= 0) && (n<=31))
							normal_v |= (1<<n);
						else if(n<64)
							normal_i1 |= (1 << (n-32));
						else if(n<96)
							normal_i2 |= (1 << (n-64));

						if((n>=0) && (n<=95))
						{	sprintf(str,"+n%d ",n);
							strcat(string,str);
						}
						break;
					case 'h':         /** highlight vector disp flag **/
							/*** get item number ***/
						n = -1;
						sscanf((char *) (ptr + 1),"%ld", &n);
						if(!batch_mode && (n == -1))
							n = choose_level(0,95,248);
						if((n >= 0) && (n<=31))
							highlight_v |= (1<<n);
						else if(n<64)
							highlight_i1 |= (1 << (n-32));
						else if(n<96)
							highlight_i2 |= (1 << (n-64));
						if((n>=0) && (n<=95))
						{	
							sprintf(str,"+h%d ",n);
							strcat(string,str);
						}
						break;
					}
				}
				else if(*ptr == '-')
				{	ptr++;
					switch(*ptr)   {
					case 'c':         /** display poly centroids **/
						centroid_disp = 0;
						strcat(string,"-c ");
						break;
					case 'r':         /** suppress the raster **/
						raster_disp = 0;
						strcat(string,"-r ");
						break;
					case 'f':         /** use full screen width flag **/
						full_width = 0;
						strcat(string,"-f ");
						break;
					case 'n':         /** normal vector display flag **/
							/*** get item number ***/
						n = -1;
						sscanf((char *) (ptr + 1),"%ld", &n);
						if(!batch_mode && (n == -1))
							n = choose_level(0,95,248);
						if((n >= 0) && (n<=31))
							normal_v &= ~(1<<n);
						else if(n<64)
							normal_i1 &= ~(1 << (n-32));
						else if(n<96)
							normal_i2 &= ~(1 << (n-64));
						if((n>=0) && (n<=95))
						{	
							sprintf(str,"-n%d ",n);
							strcat(string,str);
						}
						break;
					case 'h':         /** highlight vector disp flag **/
							/*** get item number ***/
						n = -1;
						sscanf((char *) (ptr + 1),"%ld", &n);
						if(!batch_mode && (n == -1))
							n = choose_level(0,95,248);
						if((n >= 0) && (n<=31))
							highlight_v &= ~(1<<n);
						else if(n<64)
							highlight_i1 &= ~(1 << (n-32));
						else if(n<96)
							highlight_i2 &= ~(1 << (n-64));
						if((n>=0) && (n<=95))
						{	
							sprintf(str,"-h%d ",n);
							strcat(string,str);
						}
						break;
					}
				}
			}
		}
	}
	else
	{	normal_v = 0;
		normal_i1 = 0;
		normal_i2 = 0;
		highlight_v = 0;
		highlight_i1 = 0;
		highlight_i2 = 0;
		full_width = 0;
		raster_disp = 1;
		centroid_disp = 0;
	}

		/*************** call graphics function ***********************/
	if(!collecting_macro)
	{	
	 	if(graphics(full_width,raster_disp,normal_v,normal_i1,normal_i2,highlight_v,highlight_i1,highlight_i2,centroid_disp))
			return(0);
	}

	string[95] = 0;   /** clip, just in case **/
	sprintf(raw_buf,"%s",string);

		/*************** return ***************************************/
	return(1);
}
