/*******************************************************************************
*	headerM		(see manual, Command Ref., Function Manager)	<fm>   *
*			Description:  Parses string parameter and checks       *
*			arguments for their presence and validity.             *
*			Parameters:  A string containing the function name     *
*			and its arguments.  The following values are           *
*			searched for:                                          *
*			arg1:  print flag (-p=print header on default printer) *
*			arg2:  array of character strings containing item-actio*
*                                                                              *
*			Returns zero upon normal end-of-job (EOJ).             *
*******************************************************************************/
 
#include <string.h>
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
 
long headerM(parameters)
	char *parameters;
{	
	char print_header[STRLIST];
	char string[STRLIST];
	char part1[STRLIST], part2[STRLIST];
	char list[50][STRLIST];
	long print_flag;
	long ret_code;
	long i,n;


	*part1 = 0;
	*part2 = 0;

		/*************** if parameters, check for -p & item=string ******/
	if(*trim(parameters))
	{         /*   first check for -p  */
		if(strstr(parameters,"-p") == parameters)
		{	sprintf(print_header,"-p");
			parameters[0] = ' ';
			parameters[1] = ' ';
			trim(parameters);
		}
		else
			*print_header = 0;

			/*   next check for item=string   (must have part1 ) */
		if(strsplit(parameters,'=',part1,part2) )
		{	if(!*trim(part1) )
			{	*part1 = 0;
				*part2 = 0;
			}
			else
				trim(part2);
		}
		else
		{	*part1 = 0;
			*part2 = 0;
		}
	}
	else
	{	*print_header = 0;                      /*  set to null   */
		*part1 = 0;                             /*  set to null   */
		*part2 = 0;                             /*  set to null   */
	}

		/*************** else querry for print, item=string *************/
	if(!batch_mode)
	{	if(!*print_header  && !*part1)
		{
				/*   querry for item    */
			sprintf(list[0]," - none -");
			sprintf(list[1],"comments = %s", header.comments);
			sprintf(list[2],"compress = %s" , header.compress);
			sprintf(list[3],"contour_interval = %s" , header.contour_interval);
			sprintf(list[4],"datum = %s", header.datum);
			sprintf(list[5],"dictionary = %s" , header.dictionary);
			sprintf(list[6],"edgematch = %s" , header.edgematch);
			sprintf(list[7],"edge_e = %s" , header.edge_e);
			sprintf(list[8],"edge_n = %s" , header.edge_n);
			sprintf(list[9],"edge_s = %s" , header.edge_s);
			sprintf(list[10],"edge_w = %s", header.edge_w);
			sprintf(list[11],"head1 = %s", header.head1);
			sprintf(list[12],"head2 = %s", header.head2);
			sprintf(list[13],"layer_name = %s" , header.layer_name);
			sprintf(list[14],"map_ht = %s" , header.map_ht);
			sprintf(list[15],"map_wd = %s" , header.map_wd);
			sprintf(list[16],"maptype = %s" , header.maptype);
			sprintf(list[17],"menu_set = %s" , header.menu_set);
			sprintf(list[18],"org = %s" , header.org);
			sprintf(list[19],"quad_unit = %s", header.quad_unit);
			sprintf(list[20],"revision_yr = %s" , header.revision_yr);
			sprintf(list[21],"scale = %s" , header.scale);
			sprintf(list[22],"search_rad = %s" , header.search_rad);
			sprintf(list[23],"source_yr = %s" , header.source_yr);
			sprintf(list[24],"state = %s" , header.state);
			sprintf(list[25],"stplzn = %s", header.stplzn);
			sprintf(list[26],"template1 = %s", header.template1);
			sprintf(list[27],"template2 = %s", header.template2);
			sprintf(list[28],"template3 = %s", header.template3);
			sprintf(list[29],"udrn = %s" , header.udrn);
			sprintf(list[30],"utmln = %s" , header.utmln);
			sprintf(list[31],"utmlt = %s", header.utmlt);


				/*   get users choice    */
			n = list_choice(list[0],32,217);
			
					/*   if item, then querry for string  */
			if(n)
			{	message("old item:  ");
				append_msg(list[n]);
				message(prompt[218]);
				trim(get_string(string));
					/*   set part1 & part2  strings      */
				strcpy(part2,string);                /* set part2  */
				for(i=0; list[n][i] != '=' ; i++)    /* set part1  */
					part1[i] = list[n][i];
				part1[i] = 0;
				trim(part1);
			}
			else
			{	*part1 = 0;
				*part2 = 0;
			}
		}
	}

		/*************** check to see if there is anything to do ********/
	if(!*print_header && !*part1)
	{	message(" no print flag or header item.  No action taken.");
		return(0);
	}

		/*************** execute change & print *************************/
	if(!collecting_macro)
	{	if(*print_header)
			print_flag = 1;
		else
			print_flag = 0;
		ret_code = header_ltp(print_flag,part1,part2);
	}

		/*************** set command string & return ********************/
	if(*part1)
		sprintf(raw_buf,"header %s %s = %s",print_header,part1,part2);
	else
		sprintf(raw_buf,"header %s",print_header);

    return(ret_code);
}
