/*******************************************************************************
*	importM		(see manual, Command Ref., Function Manager)	<fm>   *
*                                                                              *
*			Either calls the appropriate import function or        *
*			writes the import command to a macro file.             *
*                                                                              *
*           Parses the input string for necessary parameters.  queries         *
*           the user for necessary information.  calls the function            *
*           for the appropriate import file format type.                       *
*                                                                              *
*           Returns  1 on successful completion, 0 on abort/failure.           *
*                                                                              *
*           J.Dabritz   9/12/89                                                *
*******************************************************************************/
 
#include <string.h>
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
long enter_lname();
 
long importM(parameters)
	char *parameters;
{
	char parsed[20][STRLIST];
	char fname[256];
	char lname[256];
	char head_type[7][STRLIST];
	static long fmt_indx[25] = {0,11,1,2,18,12,13,3,4,16,14,5,6,8,7,17,10,9,15,19,20,0,0,0,0};
	static char formats[25][STRLIST] = {
		/*  0 */	" - none, lemme out, please?",
		/* 11 */	"asn         (a strip-run length encoded format (raster data), USFS GRAMS output)",
		/*  1 */	"cell        (a generic cell format, R2map/MtHmap etc.) ", 
		/*  2 */	"cff         (cartographic feature file (vector data), [USFS, GSC])",
		/* 18 */ "dem         (a Digital Elevation Model, USGS)",
		/* 12 */ "digit.ascii (a vector format used by GRASS)",
		/* 13 */ "digit.att   (an attribute format used by GRASS)",
		/*  3 */ "dlg3        (digital line graph (vector format), level 3, optional [USGS])",
		/*  4 */	"dxf         ((vector format), as used by Cad programs [Autodesk/Autocad])",
		/* 16 */ "gis         (ERDAS raster(cell) output file",
      /* 14 */ "map         (another LTPlus map (vector data))",
		/*  5 */	"moss        (MOSS import/export format (vector data))",
		/*  6 */ "optr9       (a raster format output by optronics)",
		/*  8 */	"ras         (a raster format output by HI plotters)",
		/*  7 */	"rdef2       (Raster Data Exchange Format [USFS, DWRIS/LIDES/LTPlus]",
		/* 17 */ "rlc         (a raster format output by Ideal scanners)",
		/* 10 */ "rle1        (a standard Run Length Encoded (raster) format.)",
		/*  9 */	"rstr        (a generic bit-mapped raster format)",

		/* 15 */ "scsgef      (Soil Conservation Service Geographics Exchange Format (vector data)",
		/* 19 */ "tan         (A Scan format output by the Tangent Scanner.)",
      /* 20 */ "xy          (Any user specified [generic] xy vector format)",
               ""  /** must have null last item **/
										};

	long promptem;
	long no_params;
	long fmt;          
	long ftype;          
	long level;
	long otype;
	long a,b,c,j,n,r,t;
	long s,ac;
	long i;
	long format_count;
	long ht, in, v, at, x1, x2, x3, y1, y2, y3;

	struct geftype
	{	long p_num;
		char type;
	}	g_type[4];


		/****** count formats ****/
	for(format_count=0; formats[format_count][0]; format_count++);

		/************** get format type     ***************************/
	no_params = parse(trim(parameters),delim_buf,parsed,2);

		/************** initialize promptem flag and format type ******/
	fmt = 0;       /* none */
	promptem = 0;  /* set to 1 if & when prompting is necessary */

		/************** get format type *******************************/
	if(no_params == 0)
		promptem = 1;
	else
	{	lowerc(trim(parsed[0]));
		if(!strcmp("asn",parsed[0]))
			fmt = 11;
		else if(!strcmp("cell",parsed[0]))
			fmt = 1;
		else if(!strcmp("cff",parsed[0]))
			fmt = 2;
		else if(!strcmp("dem",parsed[0]))
			fmt = 18;
		else if(!strcmp("digit.ascii",parsed[0]))
			fmt = 12;
		else if(!strcmp("digit.att",parsed[0]))
			fmt = 13;
		else if(!strcmp("dlg3",parsed[0]))
			fmt = 3;
		else if(!strcmp("dxf",parsed[0]))
			fmt = 4;
		else if(!strcmp("gis",parsed[0]))
			fmt = 16;
		else if(!strcmp("map",parsed[0]))
			fmt = 14;
		else if(!strcmp("moss",parsed[0]))
			fmt = 5;
		else if(!strcmp("optr9",parsed[0]))
			fmt = 6;
		else if(!strcmp("ras",parsed[0]))
			fmt = 8;
		else if(!strcmp("rdef",parsed[0]) ||
		 (!strcmp("rdef2",parsed[0])))
			fmt = 7;
		else if(!strcmp("rlc",parsed[0]))
			fmt = 17;
		else if(!strcmp("rle1",parsed[0]))
			fmt = 10;
		else if(!strcmp("rstr",parsed[0]) ||
		 (!strcmp("rast",parsed[0])))
			fmt = 9;
		else if(!strcmp("scsgef",parsed[0]))
			fmt = 15;
		else if(!strcmp("tan",parsed[0]))
			fmt = 19;
		else if(!strcmp("xy",parsed[0]))
			fmt = 20;
		else
			promptem = 1;
	}

	if(!fmt  ||  promptem)
	{	if(batch_mode)
			return(0);
		fmt = fmt_indx[list_choice(formats[0],format_count,65)];
		if(!fmt)
			return(0);
	}

		/************** break into different sequences depending of format */
	switch(fmt)   {
	case 1: /*******  cell format (R2map/MtHmap)  ***********************/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** get parameter n, number of ascii characters/cell ******/
		if(get_long(parsed[2],&n,&promptem,67,0,30))
			return(0);

			/****** get parameter a, does the user want attributes too? ***/
		if(get_long(parsed[3],&a,&promptem,253,1,3))
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impcell(fname,n,a))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  cell  %s  %d  %d",fname,n,a);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 2: /************** cff format (Cartographic feature file format) */
			/****** parse string for 5 parameters (0 = fmt, get 1-4) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, import level number ************/
		level = 0;
		if(no_params >= 4)
			sscanf(parsed[2],"%d",&level);
		if((level < 35)  ||  (level > 95))
		{	if(!batch_mode)
				level = choose_level(35,95,246);
			else
				return(0);
		}

			/****** check for parameter 3, coordinate type  ***************/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 4, level name/description *********/
		if(enter_lname(parameters,lname,no_params,4))
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impcff(fname,level,c,lname))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  cff  %s  %d  %d  %s",fname,level,c,lname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 3: /************** dlg3 format (usgs digital line graph) *********/
			/****** parse string for 4 parameters (0 = fmt, get 1-3) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, import level number ************/
		level = 0;
		if(no_params >= 4)
			sscanf(parsed[2],"%d",&level);
		if((level < 35)  ||  (level > 95))
		{	if(!batch_mode)
				level = choose_level(35,95,246);
			else
				return(0);
		}

			/****** check for parameter 3, level name/description *********/
		if(enter_lname(parameters,lname,no_params,3))
			return(0);


			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impdlg3(fname,lname,level))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  dlg3  %s  %d  %s",fname,level,lname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 4: /************** dxf format (Autocad) *************************/
			/****** parse string for 5 parameters (0 = fmt, get 1-4) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, import level number ************/
		level = 0;
		if(no_params >= 4)
			sscanf(parsed[2],"%d",&level);
		if((level < 35)  ||  (level > 95))
		{	if(!batch_mode)
				level = choose_level(35,95,332);
			else
				return(0);
		}

			/****** check for parameter 3, coordinate type  ***************/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 4, level name/description *********/
		if(enter_lname(parameters,lname,no_params,4))
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impdxf(fname,level,c,lname))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  dxf  %s  %d  %d  %s",fname,level,c,lname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 5: /************** moss format (MOSS import/export format) *******/
			/****** parse string for 6 parameters (0 = fmt, get 1-5) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,6);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, import level number ************/
		level = 0;
		if(no_params >= 4)
			sscanf(parsed[2],"%d",&level);
		if((level < 35)  ||  (level > 95))
		{	if(!batch_mode)
				level = choose_level(35,95,246);
			else
				return(0);
		}

			/****** check for parameter 3, coordinate type  ***************/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 4, otype (object type)  ***********/
		otype = -1;
		if(get_long(parsed[4],&otype,&promptem,53,0,2))
			return(0);

			/****** check for parameter 5, level name/description *********/
		if(enter_lname(parameters,lname,no_params,5))
			return(0);


			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impmoss(fname,level,c,otype,lname))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  moss  %s  %d  %d  %d  %s",fname,level,c,
															otype,lname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 6: /************** optr9 (optronics rle raster format 9 ) *********/
			/****** parse string for 2 parameters (0 = fmt, get 1) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,3);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impoptr9(fname))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  optr9  %s",fname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 7: /************** rdef2 (Raster Data Echange Format, version 2) */
			/****** parse string for 3 parameters (0 = fmt, get 1-2) ****/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, a (attributes code) ************/
		if(get_long(parsed[2],&a,&promptem,253,1,3))
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(imprdef2(fname,a))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  rdef2  %s  %d",fname,a);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 8: /************** ras  (HI plotter/scanner output format )   ****/
			/****** parse string for 4 parameters (0 = fmt, get 1-3) ****/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2,   (reduction factor, 1-6) ******/
		if(get_long(parsed[2],&r,&promptem,255,1,6))
			return(0);

			/****** check for parameter 3,   (threshold            ) ******/
		if(r == 1)
			t=1;
		else 
		{	t = 0;
			if(get_long(parsed[3],&t,&promptem,256,0,r*r))
				return(0);
			if(!t)
				t = (r * 3) / 4;
		}

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impras(fname,r,t))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  ras  %s  %d  %d",fname,r,t);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 9: /************** raster format (generic bitmap) ****************/
			/****** parse string for 4 parameters (0 = fmt, get 1-3) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, b (bits used code) *************/
		if(get_long(parsed[2],&b,&promptem,76,0,255))
			return(0);
		if(!b)
			return(0);

			/****** check for parameter 3, j (ascii/scan ratio) ***********/
		if(get_long(parsed[3],&j,&promptem,79,0,255))
			return(0);
		if(!j)
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(imprstr(fname,b,j))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  rstr  %s  %d  %d",fname,b,j);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 10: /************** rle1 (run length encoded, raster format) ******/
			/****** parse string for 2 parameters (0 = fmt, get 1) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,3);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(imprle1(fname))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  rle1  %s",fname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 11: /************* asn (strip-rle (USFS GRAMS output)) ***********/
			/****** parse string for 2 parameters (0 = fmt, get 1) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impasn(fname))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  asn  %s",fname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 12: /************** digit.ascii (GRASS import format, vect) *******/
			/****** parse string for 5 parameters (0 = fmt, get 1-4) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, import level number ************/
		level = 0;
		if(no_params >= 4)
			sscanf(parsed[2],"%d",&level);
		if((level < 35)  ||  (level > 95))
		{	if(!batch_mode)
				level = choose_level(35,95,246);
			else
				return(0);
		}

			/****** check for parameter 3, coordinate type  ***************/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 4, level name/description *********/
		if(enter_lname(parameters,lname,no_params,4))
			return(0);


			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impdigit_asc(fname,level,c,lname))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  digit.ascii  %s  %d  %d  %s",fname,level,c,lname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 13: /************** digit.att (GRASS import format, attributes) */
			/****** parse string for 5 parameters (0 = fmt, get 1-4) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,5);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, coordinate type  ***************/
		if(get_long(parsed[2],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 3, set number  ********************/
		s = -1;
		if(no_params >= 4)
			sscanf(parsed[3],"ld",&s);

		if(s == -1)
			s = get_set(parsed[3],&promptem,0,31,202);

		if(s<0  ||  s>24)
			return(0);

			/****** check for parameter 4, attribute code *****************/
		ac = -1;
		if(no_params >= 5)
			sscanf(parsed[4],"ld",&ac);

		if(!batch_mode  &&  (ac < 0   ||   ac > atrbt_action_limit))
			ac = list_choice(atrbt_actions[0],atrbt_action_limit+1,3);

		if(ac < 0   ||   ac > atrbt_action_limit)
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impdigit_att(fname,c,s,ac))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  digit.att  %s  %d  %d  %d",fname,c,s,ac);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 14: /************** LTPlus map format (internal LTPlus format) ****/
			/****** parse string for 4 parameters (0 = fmt, get 1-3) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name **********************/
		*fname = (char) 0;
		if(no_params > 1)
			sprintf(fname,"%s",parsed[1]);
		else
		{	if(!batch_mode)
				choose_map(fname);
			promptem = 1;
		}
		trim(fname);
		if(!(*fname))
			return(0);

			/****** check for parameter 2, import level number ************/
		level = 0;
		if(no_params >= 3)
			sscanf(parsed[2],"%d",&level);
		if((level < 35)  ||  (level > 95))
		{	if(!batch_mode)
				level = choose_level(35,95,246);
			else
				return(0);
		}

			/****** check for parameter 3, level name/description *********/
		if(enter_lname(parameters,lname,no_params,3))
			return(0);


			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impmap(fname,level,lname))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  map  %s  %d  %s",fname,level,lname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 15: /*******  SCSGEF import format                       ********/
	/*** Initialize the 'type' structure element  ***/
		g_type[0].type = 'h';
		g_type[0].p_num = 311;
		g_type[1].type = 'l';
		g_type[0].p_num = 312;
		g_type[2].type = 'p';
		g_type[0].p_num = 313;
		g_type[3].type = 'f';
		g_type[0].p_num = 314;
		
	/*** Parse input.  4 parms are req'd, extras are treated as level descrip */
		no_params = parse(trim(parameters), delim_buf, parsed, 10);
		parsed[10][0] = CNULL;

	/****** check for '-' to indicate type of file to import  ********/
		ftype = 0;
		for( i = 1; i < no_params; i++)		/* 1 used to avoid 'scsgef' parm */
		{	if( parsed[i][0] == '-' )
			{	for( j = 0; j < 4; j++ )
				{	if( parsed[i][1] == g_type[j].type )
					{	ftype = j + 1;
						if(get_dirfname(parsed[i + 1],fname,config.import_path,&promptem))
							return(0);
						break;
					}
				}
			}
		}

	/*** No file name flags were found...prompt user for scsgef file names ***/
		if( !ftype )
		{	if( get_gefname( fname, config.import_path, &ftype ) )
			{	sprintf(raw_buf, "No files chosen, import scsgef terminating.");
				return(0);
			}
		}
	
	/****** check parameter 4, import level number ************/
		level = 0;
		if(no_params > 3)
			sscanf(parsed[3], "%d", &level);

		if((level < 35)  ||  (level > 95))
		{	if(!batch_mode)
				level = choose_level(35, 95, 246);
			else
				return(0);
		}

	/****** check for parameter 5, level name/description *********/
		lname[0] = 0;
		if(no_params > 4)
		{	trim(parsed[4]);
			if(parsed[4][0] != 0)
				sprintf(lname,"%s %s %s %s %s %s", parsed[4], parsed[5],
							parsed[6], parsed[7], parsed[8], parsed[9] );
		}

		if(!batch_mode  &&  (lname[0] == 0))
		{	message(prompt[247]);
			get_string(lname);
			trim(lname);
		}	

	/*** Call import function, then write command line to raw_buf ***/
		switch( ftype )
		{
		case 1:
			sprintf(raw_buf, "Import SCSGEF header not available.");
			break;
		case 2:
			if(!collecting_macro)
				if(impgef_lin(fname, level, lname))
					return(0);
			sprintf(raw_buf,"import scsgef -l %s %d %s",fname,level,lname);
			break;
		case 3:
			if(!collecting_macro)
				if(impgef_pnt(fname, level, lname))
					return(0);
			sprintf(raw_buf,"import scsgef -p %s %d %s",fname,level,lname);
			break;
		case 4:
			if(!collecting_macro)
				if(impgef_fea(fname, level, lname))
					return(0);
			sprintf(raw_buf,"import scsgef -f %s %d %s",fname,level,lname);
			break;
		default:
			sprintf(raw_buf, "Import SCSGEF invalid type!  RTFM!");
			break;
		}

		/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 16: /*******  ERDAS  .GIS format                         ********/
			/****** parse string for 3 parameters (0 = fmt, get 2  ) ****/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, a (attributes code) ************/
		if(get_long(parsed[2],&a,&promptem,75,1,3))
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impgis(fname,a))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  gis  %s  %d",fname,a);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 17: /************** rlc  (Ideal scanner output format) *****/
			/****** parse string for 4 parameters (0 = fmt, get 1-3) ****/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2,   (reduction factor, 1-10) ******/
		if(get_long(parsed[2],&r,&promptem,331,1,10))
			return(0);

			/****** check for parameter 3,   (threshold            ) ******/
		if(r == 1)
			t=1;
		else 
		{	t = 0;
			if(get_long(parsed[3],&t,&promptem,256,0,r*r))
				return(0);
			if(!t)
				t = (r * 3) / 4;
		}

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(imprlc(fname,r,t))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  rlc  %s  %d  %d",fname,r,t);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 18: /************** dem   (usgs)                       *****/
			/****** parse string for 4 parameters (0 = fmt, get 1,2,3  ) ****/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, import level number ************/
		level = 0;
		if(no_params >= 3)
			sscanf(parsed[2],"%d",&level);
		if((level < 35)  ||  (level > 95))
		{	if(!batch_mode)
				level = choose_level(35,95,246);
			else
				return(0);
		}

			/****** check for parameter 3, level name/description *********/
		if(enter_lname(parameters,lname,no_params,3))
			return(0);

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impdem(fname,level,lname))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  dem  %s  %d  %s",fname,level,lname);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/

	case 19: /************** tan  (Tangent scanner output format) ***/
			/****** parse string for 4 parameters (0 = fmt, get 1-3) ****/
		no_params = parse(trim(parameters),delim_buf,parsed,4);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2,   (reduction factor, 1-10) ******/
		if(get_long(parsed[2],&r,&promptem,331,1,10))
			return(0);

			/****** check for parameter 3,   (threshold            ) ******/
		if(r == 1)
			t=1;
		else 
		{	t = 0;
			if(get_long(parsed[3],&t,&promptem,256,0,r*r))
				return(0);
			if(!t)
				t = (r * 3) / 4;
		}

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(imptan(fname,r,t))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import  tan  %s  %d  %d",fname,r,t);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/
	case 20:/********* xy fmt(Any user specified [generic] xy vector format) **/
			/*** init header type list ***/
		sprintf(head_type[0],"exit from this infernal command.");
		sprintf(head_type[1],"NO Header, only vertex lines.");
		sprintf(head_type[2],"Header ONLY with centroid locations.");
		sprintf(head_type[3],"Header with attribute & object number (if any) in header.");
		sprintf(head_type[4],"Header with attribute, but object number on vertex line(s).");
		sprintf(head_type[5],"Header with object number, but attribute on vertex line(s).");
		sprintf(head_type[6],"Header, but attribute & object number on vertex line(s).");

			/****** parse string for 15 parameters (0 = fmt, get 1-14) ******/
		no_params = parse(trim(parameters),delim_buf,parsed,15);

			/****** check for parameter 1, file name **********************/
		if(get_dirfname(parsed[1],fname,config.import_path,&promptem))
			return(0);

			/****** check for parameter 2, import level number ************/
		level = 0;
		if(no_params >= 4)
			sscanf(parsed[2],"%d",&level);
		if((level < 35)  ||  (level > 95))
		{	if(!batch_mode)
				level = choose_level(35,95,246);
			else
				return(0);
		}

			/****** check for parameter 3, coordinate type  ***************/
		if(get_long(parsed[3],&c,&promptem,50,0,19))
			return(0);

			/****** check for parameter 4, otype (object type)  ***********/
		otype = -1;
		if(get_long(parsed[4],&otype,&promptem,396,0,3))
			return(0);

			/****** check for parameter 5, ht: header type ****************/
		if(get_long_lst(parsed[5],&ht,&promptem,376,1,6,head_type[0],7))
			return(0);

			/****** check for parameter 6, in; item no. of object number **/
		if(get_long(parsed[6],&in,&promptem,387,0,20))
			return(0);

			/****** check for parameter 7, v:  item no. of vertex count ***/
		if(get_long(parsed[7],&v,&promptem,388,0,20))
			return(0);

			/****** check for parameter 8, at: item no. of attribute ******/
		if(get_long(parsed[8],&at,&promptem,389,0,20))
			return(0);

			/****** check for parameter 9, x1: item no. of x units ********/
		if(get_long(parsed[9],&x1,&promptem,390,1,20))
			return(0);

		x2 = 0;
		x3 = 0;
		if(c == 1)
		{		/****** check for parameter 10, x2: item no. of 1/60 x units ***/
			if(get_long(parsed[10],&x2,&promptem,391,0,20))
				return(0);

			if(x2)
			{		/****** check for parameter 11, x3: item no. of 1/3600 x units */
				if(get_long(parsed[10],&x3,&promptem,392,0,20))
					return(0);
			}
		}

			/****** check for parameter 12, y1: item no. of y units *******/
		if(get_long(parsed[12],&y1,&promptem,393,1,20))
			return(0);

		y2 = 0;
		y3 = 0;
		if(c == 1)
		{		/****** check for parameter 13, y2: item no. of 1/60 y units **/
			if(get_long(parsed[13],&y2,&promptem,394,0,20))
				return(0);

			if(y2)
			{		/****** check for parameter 14, y3: item no. of 1/3600 y units */
				if(get_long(parsed[14],&y3,&promptem,395,0,20))
					return(0);
			}
		}

			/****** call import program ***********************************/
		if(!collecting_macro)
		{	if(impxy(fname,level,c,otype,ht,in,v,at,x1,x2,x3,y1,y2,y3))
				return(0);
		}

			/****** write command line ************************************/
		sprintf(raw_buf,"import xy %s %d %d %d %d %d %d %d %d %d %d %d %d %d",fname,level,c,otype,ht,in,v,at,x1,x2,x3,y1,y2,y3);

			/****** return ************************************************/
		break;

		/*-----------------------------------------------------------------*/


	default:/************** bad format type *******************************/
			/****** ----- template ------- ********************************/
			/****** check for parameter X, whatever (w) *******************/
			/****** check for parameter X, whatever (w) *******************/
			/****** check for parameter X, whatever (w) *******************/
			/****** check for parameter X, whatever (w) *******************/
			/****** call import program ***********************************/
			/****** write command line ************************************/
			/****** return ************************************************/
			/****** *******************************************************/
			/****** *******************************************************/
		message("format not yet available.");
		return(0);

		/*-----------------------------------------------------------------*/

			/************** ***********************************************/

		/*-----------------------------------------------------------------*/

			/************** ***********************************************/

		/*-----------------------------------------------------------------*/

			/************** ***********************************************/

		/*-----------------------------------------------------------------*/
	}
	
		/************** return ****************************************/
    return(1);
}


/*******************************************************************************
*   enter_lname                                                                *
*       gets level_name from the string or prompts user                        *
*                                                                              *
*       Parameters: char string[];  input parameters                           *
*                   char lname[];   string in which to store level name        *
*                   long no_params; total params                               *
*                   long pnum;      parameter number of level name (0 based)   *
*                                                                              *
*       Value returned:  0 = level name returned, 1 = level name not retured   *
*       J.Dabritz   2/12/90                                                    *
*******************************************************************************/

long enter_lname(string,lname,no_params,pnum)
char string[], lname[];
long no_params,pnum;
{
	char *ptr;
	long n;


	*lname = (char) 0;
	n = 0;
	if(no_params > pnum)
	{	for(ptr=string; *ptr && (n<pnum); ptr++)
		{	if((*ptr == (char) 32) && (*(ptr + 1) != (char) 32))
				n++;
		}
	}
	if(n == pnum)
	{	strcpy(lname,ptr);
		trim(lname);
	}
	if(*ptr == (char) 0)
	{	if(batch_mode)
			return(1);
		message(prompt[247]);
		get_string(lname);
		trim(lname);
	}	

	return(0);
}

