/*******************************************************************************
*	logM		(see manual, Command Ref., Function Manager)	<fm>           *
*			Function manager for the log function.                             *
*			Queries for user input if necessary.                               *
*                                                                              *
*			Calls function or writes command to macro being built              *
*                                                                              *
*           Returns 1 on successful completion, 0 on failure.                  *
*           J.Dabritz 8/19/89                                                  *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
long logM(parameters)
char *parameters;
{
	char parsed[2][STRLIST];
	char string[STRLIST];
	char list[40][STRLIST];

	long ltype, i;
	long no_params;


		/************** break up parameters ***************************/
	no_params = parse(lowerc(trim(parameters)),delim_buf,parsed,2);
	for(i=0; i < no_params; i++)
		trim(lowerc(parsed[i]));

		/************** check first parameter (ltype)  ****************/
	ltype = -1;
	if(no_params)
	{	switch(*parsed[0])    {
		case 'c':                 /**** clear  ****/
			ltype = 32;
			break;
		case 'p':                 /**** point  ****/
			ltype = 33;
			break;
		case 'h':                 /**** hole   ****/
			ltype = 36;
			break;
		case 'j':                 /**** junctions ***/
			ltype = 34;
			break;
		case 's':                 /**** spur-end or set ****/
			if(*(parsed[0] + 1) == 'e')       /**** set ****/
				ltype = 35;
			else if(*(parsed[0] + 1) == 'p')  /**** spur ****/
				ltype = 37;
			break;
		default:                  /**** bad entry ****/
			break;
		}
	}


		/************* query for first parameter if necessary *******/
	if(ltype == -1)
	{	if(batch_mode)
			return(0);
		strcpy(list[0]," - none -  ");
		strcpy(list[1]," - clear log of all records");
		strcpy(list[2],"isolated pixels (points)");
		strcpy(list[3],"junctions ");
		strcpy(list[4],"a set of objects");
		strcpy(list[5],"single pixel holes");
		strcpy(list[6],"spur endpoints");
		ltype = list_choice(list[0],7,55);
		if(!ltype)
			return(0);
		ltype += 31;
	}



	if(ltype == 35)
	{		/**** check second parameter for set number **/
		if(no_params >= 2)
			sscanf(parsed[1],"%d",&ltype);

			/** prompt user for set number **/
		if((ltype < 0)  ||  (ltype > 31))
		{	if(batch_mode)
				return(0);
			ltype = list_choice(level_names[0],32,235);
		}
	}
		

		/************** execute input mode change *********************/
	if(!collecting_macro)
	{	if(log_ltp(ltype))
			return(0);
	}

		/************** build string **********************************/
	if((ltype  >= 0)  &&  (ltype < 32))
		sprintf(raw_buf,"log set %d",ltype);
	else
	{	switch(ltype)  {
		case 32:             /*** clear ****/
			sprintf(raw_buf,"log clear");
			break;
		case 36:            /*** holes ****/
			sprintf(raw_buf,"log holes");
			break;
		case 37:            /*** spurs ****/
			sprintf(raw_buf,"log spurs");
			break;
		case 33:            /*** points ****/
			sprintf(raw_buf,"log points");
			break;
		case 34:            /*** junctions ****/
			sprintf(raw_buf,"log junctions");
			break;
		}
	}
	
		/************** return ****************************************/
    return(1);
}
