/*******************************************************************************
*	macro_histM	(see manual, Command Ref., Function Manager)	<fm>   *
*			Description:  Parses string parameter and checks       *
*			arguments for their presence and validity.             *
*			Parameters:  A string containing the function name     *
*			and its arguments.  The following values are           *
*			searched for:                                          *
*			arg1:  Name of macro to create.                        *
*			arg2:  Beginning record within history file.           *
*			arg3:  Ending record within history file.              *
*			arg4:  Description of macro for help function.         *
*                                                                              *
*			Returns one  upon normal end-of-job (EOJ).             *
*        J.Dabritz  2/2/90                                                     *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for X11 support */
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#include <string.h>
#ifdef LTP_X11
#include "../../incl/x11.h"
#endif
 
long history_lims();
 
long macro_histM(parameters)
char *parameters;
{
	long no_params;
	long promptem;
	char *ptr; 
	char parsed[4][STRLIST];
	char macro_name[STRLIST];
	char descrip[STRLIST];
	long begin_rec;
	long end_rec;


		/*********** parse input string *******************************/
	no_params = parse(trim(parameters),delim_buf,parsed,4);
	promptem = 0;


		/*********** check for parameter 0, macro name ****************/
	macro_name[0] = (char) 0;
	if(no_params == 0)
	{	if(batch_mode)
			return(0);
		message(prompt[301]);
		get_string(macro_name);
		promptem = 1;
	}
	else
		sprintf(macro_name,"%s",parsed[0]);

	trim(macro_name);
	if(!(*macro_name))
		return(0);

		/*********** check for parameters 1 & 2, limits of hist. used */
	if((no_params < 3) || (*parsed[1]<'0') || (*parsed[1]>'9') ||
    (*parsed[2]<'0') || (*parsed[2]>'9'))
	{	if(history_lims(&begin_rec,&end_rec))
			return(0);
	}
	else
	{	get_long(parsed[1],&begin_rec,&promptem,0,0,99999999);
		get_long(parsed[2],&end_rec,&promptem,0,0,99999999);
	}

		/*********** check for parameter 3, macro description *********/
	if((no_params > 3)  &&  !promptem)
	{	for(ptr = parameters; *ptr  &&  (*ptr > ' ') ; ptr++);
		trim(ptr);
		for(; *ptr  &&  (*ptr >  ' ') ; ptr++);
		trim(ptr);
		for(; *ptr  &&  (*ptr >  ' ') ; ptr++);
		trim(ptr);
		sprintf(descrip,"%s",ptr);
	}
	else
	{	if(batch_mode)
			descrip[0] = (char) 0;
		else
		{	message(prompt[302]);
			get_string(descrip);
			trim(descrip);
		}
	}


		/*********** call function ************************************/
	macro_name[14] = (char) 0;
	descrip[40] = (char) 0;
	if(!collecting_macro)
	{	if(macro_hist(macro_name,begin_rec,end_rec,descrip))
		{	*raw_buf = 0;
			return(0);
		}
	}

		/*********** write command string *****************************/
	macro_name[14] = (char) 0;
	descrip[40] = (char) 0;
	sprintf(raw_buf,"macro_hist %s %d %d %s",macro_name,begin_rec,end_rec,descrip);

		/*********** return *******************************************/
	return(1);
}


/*******************************************************************************
*       history_lims    (see manual, Internal Functions)                <in>   *
*           get the beginning and ending records of the history file           *
*           to be in the macro.                                                *
*                                                                              *
*           parameters:  long *begin_rec;   ptr to long, beginning rec in hist *
*                        long *end_rec;     ptr to long, ending rec in hist    *
*                                                                              *
*           Returns 0 if input ok, otherwise returns 1                         *
*               J.Dabritz   2/2/90                                             *
*******************************************************************************/


long history_lims(begin_rec,end_rec)
	long *begin_rec, *end_rec;
{

	char *list_ptr;
	char list[515][STRLIST];
	long no_items;

	long k;
	long i,j,begin,choice,end,next,max_length,items,fit;
	long col_incr,list_col,row,n,col;
	long save_dctype;
	long save_cmode;
	long save_cx, save_cy;

	char string[STRLIST], *ptr;


		/****** check for map acquired *****************/
	if(!job_stats[0])
	{	message("You must Create or Acquire a map befor choosing history items interactively.");
		return(1);
	}

		/*   check to see if grahpics are being used.  */
	if(batch_mode)
		return(1);

		/*    save graphics for redisplay later & clear screen   */
	save_dctype = disp_coord_type;
	disp_coord_type = -1;         /*  no coord display  */
	save_cmode = cursor_mode;
	cursor_mode = 0;        /* cross hair  */
	save_cx = ginfo.scoord[0];
	save_cy = ginfo.scoord[1];
	cursor(-200, -200);            /*  take cursor off screen  */
	for(i=0; i<4; i++)
	{	swplane(i);
#ifndef LTP_X11
		bitmap(BITPLN_WIDTH,415,BITPLN_WIDTH,BITPLN_WIDTH,vga_plane,scratch_pad[i]);
#else
                bitmap4x(WIDTH,415,0,0,xmainwin,0,0,xmainpix);
#endif
		bar_off(0,0,639,414);
	}
	cursor(save_cx,save_cy);       /*  place cursor back on screen  */

		/******** copy history to list **************/
	for(i=next_hist, j=0; j<512; i = (i+1) % 512, j++)
	{	history[i][STRLIST-1] = (char) 0;
		strcpy(list[j],history[i]);
	}

	no_items = 512;
	list_ptr = list[0];
	begin = 512 - 24;

		/*    loop through list displays on screen  */
	for(choice = -1;  choice < 1;  )
	{		/*  check & set begin          */
		if(begin >= no_items)
			begin = 0;

			/*    display prompt                        */
		message(prompt[304 + choice]);

			/*	find first/last item to display & rows & columns   */
		max_length = 0;
		for(next = begin, fit=1;  fit ;  next++)
		{	if(next >= no_items)
				fit = 0;
			else
			{ 	ptr = list_ptr + next*STRLIST;
				max_length = maxl(2, (long) strlen(ptr), max_length);
				items = next - begin + 1;
				if(items > 100)
					fit = 0;
				else if(items > 75)
				{	if(max_length > (20-7))
						fit = 0;
				}
				else if(items > 50)
				{	if(max_length > (26-7))
						fit = 0;
				}
				else if(items > 25)
				{	if(max_length > (40-7))
						fit = 0;
				}
			}

			if(!fit)
				end = next-1;
		}

			/*  erase old text                                     */
		swplane(3);
		bar_off(0,0,639,414);

			/*  display list                                       */
		items = end - begin + 1;
		if(items > 75)
			col_incr = 20;
		else if(items > 50)
			col_incr = 26;
		else if(items > 25)
			col_incr = 40;
		else 
			col_incr = 80;
		for(i=0 ; i<items ; i++)
		{	list_col = i/25;
			col = list_col * col_incr;
			row = i % 25;
			ptr = list_ptr + (begin+i)*STRLIST;
			sprintf(string,"%3d. %s", (begin+i) , ptr);
			disptext(row,col,string);
		}

		/*  display message below list to press return for more (if needed)*/
		if((begin != 0) || (end+1 != no_items))
			disptext(25,0,"Press + or - [RETURN] to view the next/previous portion of the list.");

			/*  get answer  */
		for(;;)
		{	if(km() > 0)                  /******* check for keyboard input */
			{	strcpy(string,raw_buf);
				trim(string);
				ptr = string;
				if((*ptr >= '0') && (*ptr <= '9'))
				{	sscanf(ptr,"%ld",&n);
					if((n >= 0) && (n < no_items))
					{	choice++;
						if(choice)
							*end_rec = n;
						else
							*begin_rec = n;
					}
				}
				else if(*ptr == '+')
					begin = end - 10;
				else if(*ptr == '-')
				{	k = end - begin - 10;
					begin = (k > 10) ?  begin - k : begin - 10;
					if(begin < 0)
						begin = 0;
				}
				break;
			}
			if(gr_dev)
			{	if(gi() >= 0)              /******* check for graphics input */
				{	if(*cursor_keys[ginfo.button])
					{	row = ginfo.scoord[1] / 16;
						col = ginfo.scoord[0] / 8;
						if(row < 25)
						{	col = col / col_incr;
							n = begin + 25 * col + row;
							if((n >= 0) && (n <= end))
							{	choice++;
								if(choice)
									*end_rec = n;
								else
									*begin_rec = n;
								break;
							}
						}
						else
						{	if(col < 9)         /***  + *****/
								begin = end - 10;
							else                 /**** - *****/
							{	k = end - begin - 10;
								begin = (k > 10) ?  begin - k : begin - 10;
								if(begin < 0)
									begin = 0;
							}
							break;
						}
					}
				}
			}
		}
	}

		/*   reset screen     */
	for(i=0; i<4; i++)
	{	swplane(i);
#ifndef LTP_X11
		bitmap(BITPLN_WIDTH,415,BITPLN_WIDTH,BITPLN_WIDTH,scratch_pad[i],vga_plane);
#else
                bitmap4x(WIDTH,415,0,0,xmainpix,0,0,xmainwin);
#endif
	}
	disp_coord_type = save_dctype;
	cursor_mode = save_cmode;
	cursor(ginfo.scoord[0], ginfo.scoord[1]);

		/*   return           */
	return(0);

}
