/*******************************************************************************
*	pers_plotM	(see manual, Command Ref., Function Manager)	<fm>   *
*                                                                              *
*			Either calls the pers_plot function or writes the      *
*			pers_plot command to a macro file.                     *
*                                                                              *
*			Calls parser to extract 5 required arguments from      *
*			rawrem: ltypn (label type code),  d (direction map     *
*			is viewed from), r (depth ratio), e (elevation value   *
*			on the x-axis) and z (elevation ratio).  The function  *
*			pers_plotM checks for any missing arguments and        *
*			calls prompt to query the user for any that are        *
*			missing.  The pers_plot function is assumed to act     *
*			upon the current, active map.                          *
*                                                                              *
*			Returns zero upon normal execution.                    *
*******************************************************************************/
 
#include <string.h>
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
 
long pers_plotM(parameters)
char *parameters;
{
	char parsed[20][STRLIST];
	char string[256];
	char list[10][STRLIST];

	long promptem;
	long no_params;
	long d;

	double esf,yvf,ysf;


		/************** parse parameters    ***************************/
	no_params = parse(trim(parameters),delim_buf,parsed,5);

		/************** initialize promptem ******/
	if(no_params)
		promptem = 0;  /* set to 1 if & when prompting is necessary */
	else
		promptem = 1;

		/************** get parameters ********************************/

		/***** parameter 0, view direction ****************************/
	lowerc(trim(parsed[0]));
	d = 4;
	switch(*parsed[0]) {
	case 's':               /* south     */
	case 'b':               /* bottom    */
		d = 0;
		break;
	case 'w':               /* west      */
	case 'l':               /* left      */
		d = 1;
		break;
	case 'n':               /* north     */
	case 't':               /* top       */
		d = 2;
		break;
	case 'e':               /* east      */
	case 'r':               /* right     */
		d = 3;
		break;
	}
	if(d == 4)
	{	if(batch_mode)
			return(0);
		strcpy(list[0]," none, abort");
		strcpy(list[1],"Bottom");
		strcpy(list[2],"Left");
		strcpy(list[3],"Top");
		strcpy(list[4],"Right");

		promptem = 1;
		d = list_choice(list[0],5,144);

		if(!d)
			return(0);

		d--;
	}

		/***** parameter 1, elevation sensitivity factor **************/
	if(get_double(parsed[1],&esf,&promptem,145,0.01,100.0))
		return(0);

		/***** parameter 2, y vertical factor *************************/
	if(get_double(parsed[2],&yvf,&promptem,146,0.01,10.0))
		return(0);

		/***** parameter 3, y skew factor *****************************/
	if(get_double(parsed[3],&ysf,&promptem,147,-10.0,10.0))
		return(0);


		/****** call import program ***********************************/
	if(!collecting_macro)
	{	if(pers_plot(d,esf,yvf,ysf))
			return(0);
	}

		/****** write command line ************************************/
	switch(d) {
	case 0:
		strcpy(string,"bottom");
		break;
	case 1:
		strcpy(string,"left");
		break;
	case 2:
		strcpy(string,"top");
		break;
	case 3:
		strcpy(string,"right");
		break;
	}
	sprintf(raw_buf,"pers_plot  %s  %.2f  %.2f  %.2f",string,esf,yvf,ysf);
	
		/************** return ****************************************/
    return(1);
}
