/*******************************************************************************
*	plotM		(see manual, Command Ref., Function Manager)	<fm>   *
*                                                                              *
*			Either calls the plot function or writes the           *
*			plot command to a macro file.                          *
*                                                                              *
*			Calls parser to extract two required arguments         *
*			from  rawrem:  dev (plotter device type code), and     *
*			ltypn (label type code).  Two optional arguments       *
*			may also be found in  rawrem:  -s, which requires      *
*			pfile (pen:set assignment file) and sizen (label       *
*			size).  If -s is found set=1, else set=0.  The         *
*			plotM function checks for the required arguments       *
*			and calls prompt to query the user for any that        *
*			are missing.                                           *
*                                                                              *
*			Returns zero upon normal execution.                    *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for multi user support */
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#include <sys/stat.h>
 
long plotM(parms)
char *parms;
{
	long   valid_penfile();
	double strtod();
	double mapin2pu;
	double p_cnr[4][2];
	double p_dsize;
	double x_avg, y_avg;

	long   promptem;
	long   center_orig, i;
	long   pindx = 0;
	long   p_dev, p_ltype;
	long   parse_num;
	long   p_isize;                        /* integer size value */
	long   p_iscale;                       /* integer scale value */
	long   sel_plot;
	long n;
	long choice;

	char   p_afname[STRLIST];
	char   list[100][STRLIST];
	char   parsed[8][STRLIST];
	char   pen_flag[4];
	char   pen_file[256];
	char   string[256];
	char   sizen_str[32];
	char   co_string[32];

/*** If map is not registered, ask user to do so and return.   ***/
	if( !registered(6) )            /** Check for map inches **/
	{	message("Register map before plotting it!  Bye.");
		return(0);
	}

/*** See how many parameters are in parm string.  ***/
	parse_num = parse( parms, delim_buf, parsed, 8L );
	promptem = 0;
	pindx = 0;
	sel_plot = 0;


		/*** Check for -s parse parameter or prompt if no params **/
	if(parse_num > pindx)
	{	if(!strcmp(parsed[0],"-s"))
		{	sel_plot = 1;
			pindx++;
		}
	}
	else
	{	if(batch_mode)
			return(0);
		if(answerisyes(334,0))
			sel_plot = 1;
		promptem = 1;
	}

		/**** Get plot file name if selective plotting ***/
	pen_file[0] = (char) 0;
	if(sel_plot)
	{		/**** check for filename in command string ***/
		if(parse_num <= pindx)
			promptem = 1;
		if(!promptem)
		{	sprintf(pen_file,"%s",parsed[1]);
			pindx++;
		}
		else
		{	if(batch_mode)
				return(0);
#ifdef MULTIUSER
			sprintf(string,"%s/.ltp/pens",home_path);
			n = dir_list(string,list+1,99);
			choice = 0;
			if (n > 0) {
				sprintf(list[0]," - none - (next list)");
				n++;
				choice = list_choice(list,n,335);
			}
			if(!choice) {
				sprintf(string,"%s/pens",ltp_path);
				n = dir_list(string,list+1,99);
				sprintf(list[0]," - none, escape -");
				n++;
				choice = list_choice(list,n,335);
			}
			if(!choice)
				sel_plot = 0;
			else
				sprintf(pen_file,"%s",list[choice]);

#else   /***** original code *****/
			sprintf(string,"%s/pens",ltp_path);
			n = dir_list(string,list+1,99);
			sprintf(list[0]," - none, escape -");
			n++;
			choice = list_choice(list,n,335);
			if(!choice)
				sel_plot = 0;
			else
				sprintf(pen_file,"%s",list[choice]);
#endif
		}
	}



		/*** Check the device type.  (plotter, screen, ascii file, etc ) ***/
	p_dev = 0;
	get_long(parsed[pindx],&p_dev,&promptem,336,-1,3);
	if(p_dev == -1)
		return(0);
	pindx++;


		/*** Check the plot scale ***/
	p_iscale = 0;
	get_long(parsed[pindx],&p_iscale,&promptem,337,-1,200000000);
	if(p_iscale < 0)
		return(0);
	else if(p_iscale == 0)
		p_iscale = map_scale;
	pindx++;


		/*** Check the label type ***/
	p_ltype = 3;
	get_long(parsed[pindx],&p_ltype,&promptem,338,-1,3);
	if(p_ltype == -1)
		return(0);
	pindx++;


		/*** If label desired, check the character size of the label ***/
	if( p_ltype == 0 )
		p_isize = 0;
	else 
	{	p_isize = 100;
		get_long(parsed[pindx],&p_isize,&promptem,339,-1,10000);
		if(p_isize == -1)
			return(0);
		if(p_isize == 0)
			p_ltype = 0;
		pindx++;
	}


		/* If ascii file, get the name ***/
	p_afname[0] = (char) 0;
	if( p_dev == 3 )
	{	if((pindx < parse_num) && (strlen(parsed[pindx])))
		{	strcpy(p_afname,parsed[pindx]);
			pindx++;
		}
		else
		{	if( batch_mode )
				return(0);
			message(prompt[340]);
			string[0] = (char) 0;
			get_string(string);
			trim(string);
			if(!(*string))
				return(0);
			sprintf(p_afname,"%s",string);
		}
	}


		/*** check for center or lower-left origin on plotter ***/
		/*** set default, a center origin (large format plotter) ***/
	center_orig = TRUE; 
		/*** See if its a center or lower left origin plotting device ***/
		/*** if present, overides first default ***/
	if((p_dev == 1 ) || (p_dev == 3))
	{	if( config.plt1_type[0] == 'l' || config.plt1_type[0] == 'L' )
			center_orig = FALSE;
		else
			center_orig = TRUE;
	}
	else if( p_dev == 2 )
	{	if( config.plt2_type[0] == 'l' || config.plt2_type[0] == 'L' )
			center_orig = FALSE;
		else
			center_orig = TRUE;
	}
		/*** Check for -L in parms. If present, its a lower left origin plot ***/
		/*** if present, overides all defaults ***/
	co_string[0] = (char) 0;
	if((promptem == 0) && (parsed[pindx][0]))
	{	if(!batch_mode)
		{	if(strstr(parsed[pindx],"-L") == parsed[pindx])
			{	center_orig = FALSE;
				sprintf(co_string,"-L");
			}
		}
	}

/*** If no open plotter (fd_plot == -1) open device ***/
	if( fd_plot == -1 && (p_dev == 1 || p_dev == 2) )
	{	if( plot_setup(p_dev) )
			return(0);
	}


/*** Set ratio for units conversion ***/
/*	ratio = map_scale / p_iscale;        **/
/*	p_factor = (1200.0 * 1000.0)/(12.0 * 3937.0 * .025);        **/
/*	mapin2pu = ratio * p_factor;        **/

	mapin2pu = map_scale * 4000.0 / p_iscale / 3.937;

/*** Set map inches to plotter units (.025mm = 1pu) for corners ***/

	if( center_orig )
		for( i = 0; i < 4; i++ )
		{	p_cnr[i][0] = corner[6][i][0] * mapin2pu;
			p_cnr[i][1] = corner[6][i][1] * mapin2pu;
		}
	else
		for( i = 0; i < 4; i++ )
		{	p_cnr[i][0] = (corner[6][i][0] - .75) * mapin2pu;
			p_cnr[i][1] = (corner[6][i][1] - .75) * mapin2pu;
		}

/*** Convert mils to cm's.  (if p_isize = 100 it represents .100")   ***/
	p_dsize = (double)p_isize * .00254;           /* 1" = 25.4mm  */
	
/*** Find offsets to center map on plotter with origin at center ***/
	if( center_orig )
	{	x_avg = y_avg = 0.0;
		for( i = 0; i < 4; i++ )
		{	x_avg += p_cnr[i][0];
			y_avg += p_cnr[i][1];
		}
		x_avg /= 4.0;
		y_avg /= 4.0;
	
	/*** Calculate and load limits for plot in corner[8]  ***/
		for(i=0; i<4; i++)
		{	corner[8][i][0] = p_cnr[i][0] - x_avg;
			corner[8][i][1] = p_cnr[i][1] - y_avg;
		}
	}
	else	 /*** Calculate and load limits for lowerleft plot in corner[8] ***/
	{	for(i=0; i<4; i++)
		{	corner[8][i][0] = p_cnr[i][0];
			corner[8][i][1] = p_cnr[i][1];
		}	
	}

/*** Pass all parms to plot_ltp and let it process, unless collecting macro ***/
	if( !collecting_macro )
	{	if(plot_ltp(p_dev, p_ltype, p_dsize, p_iscale, pen_file, p_afname) )
		{	raw_buf[0] = '\0';       /* null out raw buf */
			return(0);
		}
	}
	if(sel_plot)
		strcpy(pen_flag,"-s");
	else
		pen_flag[0] = (char) 0;

	if(p_ltype)
		sprintf(sizen_str,"%d",p_isize);
	else
		sprintf(sizen_str,"");


	sprintf(raw_buf, "plot %s %s %d %d %d %s %s %s", pen_flag, pen_file,
		p_dev, p_iscale, p_ltype, sizen_str, p_afname,co_string);

/*** Set job_stats plot flag ***/
	job_stats[42] = job_stats[45] + 1;

	return(1);
}
