/*******************************************************************************
*	purgeM		(see manual, Command Ref., Function Manager)	<fm>   *
*                                                                              *
*			Either calls the purge function or writes the          *
*			purge command to a macro file.                         *
*                    	Calls parse to extract any parameters from parms.      *
*			If parms are missing, the user will be prompted for    *
*			them.  Else, the parms are checked for validity        *
*			before calling purge().  The user may end purge at     *
*			any time by responding with a zero to list choices,    *
*			or by responding with the letter 'n' or 'N' to the     *
*			final purge prompt.  If a map is being purged, the     *
*			"rm -r ..." shell command is used, else the system     *
*			call unlink is used to purge the desired file.         *
*                                                                              *
*			Globals Modified:                                      *
*			        raw_buf                                        *
*                                                                              *
*			Returns zero upon normal execution, else               *
*			returns 1 indicating no action, else                   *
*			returns -1 upon error.                                 *
*			C. Jonson                                              *
*			  Modified 11/11/89  to use config.maps_path if a      *
*			     map is to be deleted, not ltp_path.path if a      *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#include <sys/stat.h>
#define NDTYPE 10
 
static char dtype_list[][STRLIST] = { "- none -", "map", "maplist", "macro", 
			"help", "menuset", "dictionary", "penfile", "import", "export" };
long purgeM(parms)
char *parms;
{
/*  Locals  */
	long choice;
	char parsed_str[2][STRLIST];	/*  [0] is dtype, [1] is fname  */
	char dir_path[512];
	static char pname[][32] = { "", "maps", "map_lists", "macros", "help",
				"menus", "dictionaries",  "pens", "import", "export" };

	long get_dtype();
	long check_dtype();
	long check_fname();

/*  If in batch mode, simply call purge and then return */

	if( batch_mode )	         /*  True if running in batch mode  */
	{	if( collecting_macro )   /*  True if in macro collection mode */
		{	sprintf( raw_buf, "purge %s %s", 
					parsed_str[0], parsed_str[1] );
			return(1);
		} 
	
		if( purge( parsed_str[0], parsed_str[1], dir_path ) ) 
			return(0);	/*  No action occurred  */
		else 
		{	sprintf( raw_buf, "purge %s %s", 
					parsed_str[0], parsed_str[1] );
			return(1);
		}
	}


/*  Check the number of parameters  */

	switch( parse( parms, delim_buf, parsed_str, 2L ) )
	{	case 0:			/*  No parameters passed  */
			if(!(choice = get_dtype( parsed_str[0])))
			{	strcpy(raw_buf,"** purge request cancelled **");
				return(1);
			}
			if( choice == 1 )          /* Map chosen, use config.maps_path */
				strcpy( dir_path, config.maps_path);
			else
				sprintf( dir_path, "%s/%s", ltp_path, pname[choice] );

			if(!(get_fnm( dir_path, parsed_str[1] )) )
			{	strcpy(raw_buf,"** purge request cancelled **");
				return(1);
			}
			break;
		case 1:			/*  One parm passed.  Assume dtye  */
			if(!(choice = check_dtype(parsed_str[0])))
			{	strcpy(raw_buf,"** purge request cancelled **");
				return(1);
			}

			if( choice == 1 )          /* Map chosen, use config.maps_path */
				strcpy( dir_path, config.maps_path);
			else
				sprintf( dir_path, "%s/%s", ltp_path, pname[choice] );

			if(!(get_fnm( dir_path, parsed_str[1] )) )
			{	strcpy(raw_buf,"** purge request cancelled **");
				return(1);
			}
			break;
		case 2:			/*  Two parms passed.  */       
			if(!(choice = check_dtype( parsed_str[0])))
			{	strcpy(raw_buf,"** purge request cancelled **");
				return(1);
			}

			if( choice == 1 )          /* Map chosen, use config.maps_path */
				strcpy( dir_path, config.maps_path);
			else
				sprintf( dir_path, "%s/%s", ltp_path, pname[choice] );

			if(check_fname( dir_path, parsed_str[1] ))
			{	strcpy(raw_buf,"** purge request cancelled **");
				return(1);
			}
			break;
		default:		/*  Should not happen  */
			message("  purgeM():  You're up the creek now!");
			return(-1);
	}

/*  Valid parms, call purge then return  */

	strcat( dir_path, "/" );
	strcat( dir_path, parsed_str[1] );

	if( collecting_macro )		/*  True if in macro collection mode */
	{	sprintf( raw_buf, "purge %s %s", parsed_str[0], parsed_str[1] );
		return(1);
	} 

	if( purge( parsed_str[0], parsed_str[1], dir_path ) ) 
	{	append_msg("  ** unable to perform purge **");
		return(0);
	}
	else 
	{	sprintf( raw_buf, "purge %s %s", parsed_str[0], parsed_str[1] );
		return(1);
	}
}

/*******************************************************************************
*   check_dtype                                                                *
*   		Checks the dtype the user entered against the list of accepted *
*               dtypes.  If there is a match, the index into the dtype list is *
*               returned.  Else the user is prompted to select a dtype from a  *
*               list of valid types.                                           *
*               Returns the user choice, which will be non-negative long.      *
*******************************************************************************/
long check_dtype( dtype )
	char *dtype;
{
/*  Locals  */
	long i, choice;

	for( i = 0; strcmp( dtype, dtype_list[i]) && i < NDTYPE; i++ );

	if( i == NDTYPE )		/*  If no match, prompt for new dtype */
		choice = get_dtype(dtype);
	else
		choice = i;

	return(choice);
}

/*******************************************************************************
*   get_dtype                                                                  *
*   		Displays list of valid dtypes to the user, and if a choice is  *
*               greater than zero, the dtype parm is loaded with the string.   *
*               Else no action is taken.                                       *
*               Returns the user choice, which will be non-negative long.      *
*******************************************************************************/
long get_dtype( dtype )
	char *dtype;
{
/*  Locals  */
	long choice;

	choice = list_choice( dtype_list, NDTYPE, 214 );
	if( choice )
		strcpy( dtype, dtype_list[choice] );
	
	return(choice);
}

/*******************************************************************************
*   check_fname                                                                *
*   		Checks the fname the user entered against the actual directory *
*               entries.  If the file exists, no further action is taken.  If  *
*               the file does not exist, a message is displayed to that end.   *
*               Returns 0 if file exists, else returns -1.                     *
*******************************************************************************/
long check_fname( dir_path, fname )
	char *dir_path, *fname;
{
/*  Locals  */
	char tmp_path[512];
	struct stat stat_buf;

	sprintf(tmp_path, "%s/%s", dir_path, fname );
	if( stat( tmp_path, &stat_buf ) )
	{
		sprintf(tmp_path, "  ** unable to find %s **", fname );
		append_msg( tmp_path );
		return(-1);
	}
	else 
		return(0);
}
