/*******************************************************************************
*	raw_printM	(see manual, Command Ref., Function Manager)	<fm>   *
*			Description:  Parses string parameter and checks       *
*			arguments for their presence and validity.             *
*			Parameters:  A string containing the function          *
*			arguments.  The following values are searched for:     *
*			arg1:  Optional screen display.  No printing done.
*			arg2:  Type of raw data file			       *
*			arg3:  Optional starting record number 		       *
*			arg4:  Optional ending record number  		       *
*									       *
*			Returns zero upon normal end-of-job (EOJ).             *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#include <sys/stat.h>
#define NFTYPE 19L
#define NUMPARS 4L

long get_ftype();
long check_ftype();
long get_recno();
long not_text_file();
long strtol();

static char ftype_list[][STRLIST] = { "- none -", "attributes", "coord_list", 
			"diary", "errors", "header", "history", "index_array",
			"index_list", "job_stats", "objects", "register", 
			"seg_list", "v_attributes", "v_coordlist", "v_objects",
			"cmd_line_hist","open_files","malloc_state"};
 
long raw_printM(parms)
char *parms;
{
	long choice, num_parsed;
	long beg_rec, end_rec;
	long disp_flag = 0;
	char dir_path[512];
	char *ptr;
	char fname[STRLIST];
	char msg_buf[STRLIST];
	char parsed[NUMPARS][STRLIST];
	char dispf_str[4];
	struct stat stat_buf;

/**  Parse the input string into seperate parms  **/
	num_parsed = parse( parms, delim_buf, parsed, NUMPARS );

/**  If in batch mode, simply call raw_print and then return  **/
	if( batch_mode )
	{	if( collecting_macro )	/*  True if in macro collection mode */
		{
			sprintf(raw_buf, "raw_print, %s", parms );
			return(1);
		}

		if((choice = check_ftype(parsed[0])) == 0 )
		{	append_msg("  ** raw_print cancelled **");
			return(0);
		}
		beg_rec = strtol(parsed[1], (char **)NULL, 10);
		end_rec = strtol(parsed[2], (char **)NULL, 10);
		if( !raw_print(choice, beg_rec, end_rec, 0L) )
		{	sprintf(raw_buf, "raw_print %s", parms );
			return(1);		/*  bad  return  */
		}
		else
			return(0);		/*  good return  */
	}

/*** Check for the display switch and set flag if present ***/
	if( num_parsed == 0 )
		disp_flag = answerisyes(361,1);
	else
	{	if( strstr(parms, "-d") )
			disp_flag = TRUE;
		else
			disp_flag = FALSE;
	}
	if( disp_flag )
		strcpy( dispf_str, "-d");
	else
		dispf_str[0] = 0;

	end_rec = beg_rec = -1;

/*** FALL THROUGH IS EXPECTED.  DO NOT USE BREAKS IN THIS SWITCH!!!!! ***/
	if( disp_flag )
	{	switch( num_parsed )
		{
		case 4:			/* Fall though */
			ptr = 0;
			end_rec = strtol(parsed[3], &ptr, 10);
			if( ptr == parsed[3] )
				end_rec = -1;
		case 3:			/* Fall though */
			ptr = 0;
			beg_rec = strtol(parsed[2], &ptr, 10);
			if( ptr == parsed[2] )
				beg_rec = -1;
		case 2:
			strcpy(fname, parsed[1]);
			break;		/*  This break is OK  */
		case 1:
		default:
			strcpy( fname, "BAD111");
			break;		/*  This break is OK  */
		}
	}
	else
	{	switch( num_parsed )
		{
		case 3:			/* Fall though */
			ptr = 0;
			end_rec = strtol(parsed[2], &ptr, 10);
			if( ptr == parsed[2] )
				end_rec = -1;
		case 2:			/* Fall though */
			ptr = 0;
			beg_rec = strtol(parsed[1], &ptr, 10);
			if( ptr == parsed[1] )
				beg_rec = -1;
		case 1:
			strcpy(fname, parsed[0]);
			break;		/*  This break is OK  */
		case 0:
		default:
			strcpy(fname, "BAD111");
			break;		/*  This break is OK  */
		}
	}

/*** Check beg/end_rec, fname for validity    ***/
	if((choice = check_ftype(fname)) == 0 )
	{	append_msg("  ** raw_print cancelled **");
		return(1);
	}
	sprintf(dir_path, "%s/%s", temp_map_path, ftype_list[choice]);

/*** Does file exist?  ***/
	if(choice < 16)
	{	if( stat( dir_path, &stat_buf ) )
		{	append_msg("  Map file missing");
			return(0);
		}
	
		if( stat_buf.st_size == 0 )
		{	sprintf(msg_buf, "raw_print %s   ** File is empty.  No print will occur.", ftype_list[choice] );
			message(msg_buf);
			return(0);
		}
	}

	if( not_text_file(ftype_list[choice]) )
	{	if( beg_rec == -1 || end_rec == -1 || beg_rec >= end_rec )
		{	if(get_recno(&beg_rec, &end_rec))
			{	if( !batch_mode )
					message("** raw_print cancelled **");
				return(0);
			}
		}
	}
	
/*** Parms checked, call raw_print if not collecting macros  ***/
	if( collecting_macro )	/*  True if in macro collection mode */
	{	sprintf(raw_buf, "raw_print %s %s %ld %ld", dispf_str,
			ftype_list[choice], beg_rec, end_rec );
		return(1);		/* Simulate good print!  */
	}

	if( !raw_print(choice, beg_rec, end_rec, disp_flag) )
	{	sprintf(raw_buf, "raw_print %s %s %ld %ld", dispf_str,
			ftype_list[choice], beg_rec, end_rec );
		return(1);		/*  good  return  */
	}
	else
	{	sprintf(msg_buf, "raw_print %s  Error occurred.", 
			ftype_list[choice] );
		message(msg_buf);
		return(0);		/*  bad return  */
	}
}

/*******************************************************************************
*	get_ftype                                                           
*                                                                              *
*******************************************************************************/

long get_ftype( ftype )
	char *ftype;
{
	long choice;

	choice = list_choice( ftype_list, NFTYPE, 220 );
	if( choice )
		strcpy( ftype, ftype_list[choice] );

	return(choice);
}

/*******************************************************************************
*	check_ftype                                                         
*                                                                              *
*******************************************************************************/

long check_ftype( ftype )
	char *ftype;
{
	long i, choice;

	for(i=0; strncmp(ftype, ftype_list[i], strlen(ftype)) && i<NFTYPE; i++);

	if( i == NFTYPE )
		choice = get_ftype( ftype );
	else
		choice = i;

	return(choice);
}

/*******************************************************************************
*	get_recno                                                             
*               Return 0 if record numbers accepted, else return -1.           *
*******************************************************************************/

long get_recno( first, last )
	long *first;
	long *last;
{
	char resp[32];

	message("Enter the beginning record number or RETURN to cancel.");
	if( !strlen( trim(get_string(resp)) ) )
		return(-1);

	*first = strtol( resp, (char **)NULL, 10 );

	message("Enter the ending record number or RETURN to cancel.");
	if( !strlen(trim(get_string(resp)) ))
		return(-1);

	*last = strtol( resp, (char **)NULL, 10 );

	if( *first >= *last )
	{
		message("Beginning record number must be less than ending record number.");
		if(get_recno(first, last))
			return(-1);
	}

	return(0);
}
/*******************************************************************************
*	not_text_file                                                          *
*		Determines if parameter is name of a text file.                *
*		                                                               *
*		Returns 1 if param is NOT a text file, else                    *
*		returns 0 if param is a text file.                             *
*******************************************************************************/

long not_text_file(ftype)
	char *ftype;
{
	static char text_mfiles[][STRLIST] = {"diary", "header", "history",
			"cmd_line_hist", "open_files","malloc_state" };
	long indx;

	for( indx = 0; strcmp(ftype, text_mfiles[indx]) && indx < 6; indx++ );
	
	if( indx == 6)
		return(1);
	else 
		return(0);
}
