/*******************************************************************************
*   recallM          (see manual, Command Ref., Function Manager)       <fm>   *
*           Function manager for the recall function                           *
*                                                                              *
*         Queries for the draw mode if not present in string:                  *
*                 mode 18 = recall point                                       *
*                 mode 19 = recall point                                       *
*                 mode 20 = recall point                                       *
*                 mode 21 = recall point                                       *
*                 mode 22 = recall point                                       *
*                 mode 23 = recall point                                       *
*                 mode 24 = recall point                                       *
*                 mode 25 = recall point                                       *
*                                                                              *
*         Calls function or writes command to macro being built                *
*           J.Dabritz  1/4/89                                                  *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
 
recallM(parameters)
char *parameters;
{
	char parsed[3][STRLIST];
	char string[STRLIST];
	char *ptr;
	static char modes[9][STRLIST] = {  " - none -  ", 
                                       "recall point",
                                       "recall line",
                                       "recall fat-line",
                                       "recall brectangle  (edge only)",
                                       "recall arectangle  (including center)",
                                       "recall wedge     (walking triangle)",
                                       "recall separate",
                                       "recall connect"  };

	long mode;
	long no_params;
	long promptem;
	long braster;


		/************** parse parameters *******************/
	no_params = parse(trim(parameters),delim_buf,parsed,3);
	promptem = 0;

		/************** check parameter for draw type ******/
	mode = 0;
	if(*parsed[0])
	{	switch(*parsed[0])   {
		case 'p':            /* recall point mode      */
			mode = 18;
			break;
		case 'l':            /* recall line mode      */
			mode = 19;
			break;
		case 'f':            /* recall fat_line mode      */
			mode = 20;
			break;
		case 'a':            /* recall arectangle mode      */
			mode = 22;
			break;
		case 'b':            /* recall brectangle mode      */
			mode = 21;
			break;
		case 'w':            /* recall wedge mode          */
			mode = 23;
			break;
		case 's':            /* recall separate mode          */
			mode = 24;
			break;
		case 'c':            /* recall connect mode          */
			mode = 25;
			break;
		}
	}

		/************** query for draw type (& snap) if not specified */
	if(!batch_mode && !mode)
	{	mode = list_choice(modes,9,268);
		if(mode)
			mode += 17;
		else
			return(0);
		promptem = 1;
	}

		/************** check for raster type to recal from ***********/
	braster = 0;
	if(israsterb())
	{	if((no_params > 1)  ||  (promptem))
		{	if((no_params > 1) && ((parsed[1][0] == 'b')  ||  
			   (parsed[1][0] == 'B')  ||  (parsed[1][0] == '1') ))
				braster = 1;
			else if((no_params > 1) && ((parsed[1][0] == 'a')  ||  
			   (parsed[1][0] == 'A')))
				braster = 0;
			else
			{		/********* if rasterb exists, prompt for which raster *****/
				message(prompt[372]);
				get_string(parameters);
					/**** check for escape ***/
				for(ptr=parameters; *ptr; ptr++)
				{	if(*ptr == (char) 27)
						return(0);
				}
				trim(parameters);
				parameters[1] = (char) 0;
				if(*parameters)
				{	if(parameters[0] == 'a')
						braster = 0;
					else if(parameters[0] == 'b')
						braster = 1;
					else
						return(0);
				}
			}
		}
	}

		/************** execute input mode change *********************/
	if(!collecting_macro)
	{	if(recall_ltp(mode,braster))
			return(0);
	}

		/************** build string **********************************/
	if(braster)
		sprintf(string,"  b");
	else
		sprintf(string,"");

	switch(mode)  {
	case 18:           /* recall point mode     */
		sprintf(raw_buf,"recall point%s",string);
		break;
	case 19:           /* recall line mode     */
		sprintf(raw_buf,"recall line%s",string);
		break;
	case 20:           /* recall fat-line mode     */
		sprintf(raw_buf,"recall fat-line%s",string);
		break;
	case 21:           /* recall brectangle mode     */
		sprintf(raw_buf,"recall brectangle%s",string);
		break;
	case 22:           /* recall arectangle mode     */
		sprintf(raw_buf,"recall arectangle%s",string);
		break;
	case 23:           /* recall wedge mode     */
		sprintf(raw_buf,"recall wedge%s",string);
		break;
	case 24:           /* recall separate mode     */
		sprintf(raw_buf,"recall separate%s",string);
		break;
	case 25:           /* recall connect mode     */
		sprintf(raw_buf,"recall connect%s",string);
		break;
	}
	
		/************** return ****************************************/
    return(1);
}
