/*******************************************************************************
*	renameM		(see manual, Command Ref., Function Manager)	<fm>   *
*                                                                              *
*			Either calls the rename function or writes the         *
*			rename command to a macro file.                        *
*                                                                              *
*                    	Calls parser to extract both source and                *
*			destination file or directory names ( old and          *
*			new) from  rawrem, passed from function                *
*			junction (fj), and passes these items to the           *
*			rename function for processing.  If one or both        *
*			arguments are missing, it calls the prompt             *
*			function to query the user for valid file or directory *
*			names.  If the old file name does not exists or the    *
*			new one does already, renameM will query the user for  *
*			for correct data.                                      *
*                                                                              *
*			Returns zero upon normal execution.                    *
*			C. Jonson                                              *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#include <sys/stat.h>
#include <errno.h>
/***  Function Declarations ***/
long from_name_valid();
long to_name_valid();
long get_from_name();
long get_to_name();
 
long renameM(parms)
char *parms;
{
	char cmd[512];
	char parsed_buf[2][STRLIST];

/***  Parse and trim the parameters.  2 are expected.   ***/
/***  Users are prompted for names until names are valid or user quits. ***/
	if( parse(parms, delim_buf, parsed_buf, 2L) == 2)
	{	lowerc( trim(parsed_buf[0]) );
		if( from_name_valid(parsed_buf[0]))
		{	append_msg(" *** rename terminated ***");
			return(0);
		}
			
		lowerc( trim(parsed_buf[1]) );
		if( to_name_valid(parsed_buf[1]))
		{	append_msg(" *** rename terminated ***");
			return(0);
		}
	}
	else		/*** no parameters received.  Prompt for names ***/
	{	if( get_from_name(parsed_buf[0]) )
		{	append_msg(" *** rename terminated ***");
			return(0);
		}
		if( from_name_valid(parsed_buf[0]) )
		{	append_msg(" *** rename terminated ***");
			return(0);
		}
		
		if( get_to_name(parsed_buf[1]) )
		{	append_msg(" *** rename terminated ***");
			return(0);
		}
		if( to_name_valid(parsed_buf[1]) )
		{	append_msg(" *** rename terminated ***");
			return(0);
		}
	}

/***  Call rename to move the old to the new.  ***/
	if( rename(parsed_buf[0], parsed_buf[1]) )
	{	sprintf(cmd, "--> Cannot rename %s to %s", parsed_buf[0],parsed_buf[1]);
		message(cmd);
		return(0);
	}

/***  Return to fj  ***/
	sprintf(raw_buf, "rename %s %s", parsed_buf[0], parsed_buf[1]);
	return(1);
}

/****************************************************************************
*	get_from_name                                                       *
*                    Returns 0 if name present, else returns 1              *
*                    if the user wants to terminate this process.           *
****************************************************************************/
long get_from_name( name )
	char *name;
{
/*** Get filenames of old file/dir  ***/
	for( ;; )	
	{	message(prompt[266]);
		lowerc( trim( get_string(name) ) );
/*** If user entered a 'q', return 1 signaling a user termination ***/
		if(!strcmp(name, "q"))	
			return(1);

/*** If user entered something, return 0 ***/
		if(strlen(name))
			return(0);
	}
}

/****************************************************************************
*       from_name_valid                                                     *
*                    Ensures name exists and user has read write permission *
*                    for the file.  If no permissions, user is prompted to  *
*                    select another file or to quit.                        *
*                    Returns 0 if name is an existing directory or          *
*                    file, or returns 1 if the user wants to quit.          *
****************************************************************************/
long from_name_valid( name )
	char *name;
{
	extern int errno;
	char cmd[STRLIST];

/*** Ensure file/dir name exists, unless the user wants to quit.  ***/
	for( ;; )
	{	if( file_access(name, 6, 1) )
			if( get_from_name( name ) )
				return(1);
			else
				continue;

		return(0);		/* Filename is OK */
	}
}

/****************************************************************************
*	get_to_name                                                         *
*                    Returns 0 if name present, else returns 1              *
*                    if the user wants to terminate this process.           *
****************************************************************************/
long get_to_name( name )
	char *name;
{
/*** Get filenames of new file/dir  ***/
	for( ;; )	
	{	message(prompt[267]);
		lowerc( trim( get_string(name) ) );

/*** If user entered a 'q', return 1 signaling a user termination ***/
		if(!strcmp(name, "q"))	
			return(1);

/*** If user entered something, return 0 ***/
		if(strlen(name))
			return(0);
	}
}

/****************************************************************************
*       to_name_valid                                                       *
*                    If the file or directory exists, the user is asked if  *
*                    its OK to overwrite existing file or directory.        *
*                    Returns 0 if name begins with ltp_path or maps_path.   *
*                    Returns 1 if user wants to quit.                       *
****************************************************************************/
long to_name_valid( name )
	char *name;
{
	struct stat stat_st;
	extern int errno;
	extern int sys_nerr;
	extern char *sys_errlist[];
	char cmd[STRLIST];

/*** Ensure name begins with contents of 'ltp_path' or 'maps_path'  ***/
	for( ;; )
	{	if( strncmp(ltp_path, name, strlen(ltp_path)) &&
				strncmp(maps_path, name, strlen(maps_path)) )
		{	if( get_to_name(name) )
				return(1);
		}

/*** See if file exists.  Yes? Ask user if overwrite is OK.  ***/
		if(!access(name, 0))		/*** True if exists ***/
		{	sprintf(cmd, "--> %s exists.  Do you wish\ to overwrite it?",name);
			message(cmd);
			if( answerisyes(0, 2) )
				return(0);
			else 
			{	if(get_to_name( name ))
					return(1);
			}
		}
		else
			return(0);
	}
}
