/*******************************************************************************
*	reportM		(see manual, Command Ref., Function Manager)	<fm>   *
*                                                                              *
*			Either calls the report function or writes the         *
*			report command to a macro file.                        *
*                                                                              *
*			Calls parser to extract 3 required arguments           *
*			from  rawrem:  disp_dev (output device type code),     *
*			len (length units code), area (area units code),       *
*			r1 (primary ordering code), r2 (secondary ordering     *
*			code) and setnum (set identifier).  If  prtr=3,        *
*			then a seventh argument,  fname (output ascii file     *
*			name) must also be found.                              *
*                                                                              *
*			Returns 1 upon normal execution else returns 0 if      *
*			user wants to terminate report.                        *
*******************************************************************************/
 
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
#include <errno.h>
#include <string.h>
#define MINARGS 6
#define MAXARGS 7
 
long reportM(parms)
char *parms;
{
	long  strtol();                 /*** Conv't strings to longs ***/
	long  valid_long();             /*** Conv't str, check       ***/
	long  valid_string();           /*** Get string from user    ***/
	long  area,
	      disp_dev,
    	      i,
	      len,
	      num_parsed,
	      parm_indx = 0,
	      setnum,
	      r1,
	      r2,
	      user_quit = FALSE;
	long  pf = 0;
	char  parsed[9][STRLIST],
	      fil_nam[256];
	
/*** See if map has been assembled.  If not, return.  ***/
	if( !job_stats[10] )		/*** True if map not assembled ***/
	{	if( !batch_mode )
			message( errors[31] );
		return(0);
	}

/*** Parse up parm string and check number of args present.  ***/
	num_parsed = parse(parms, delim_buf, parsed, 9L);

/*** If bogus number of parms, load parsed with trash so valid_long() ***/
/*** will be forced to prompt for all arguments.                      ***/
	if( num_parsed < MINARGS || num_parsed > MAXARGS )
	{	for( i = 0; i < MAXARGS; i++ )
			strcpy(parsed[i], "-1");
	}

/*** Assuming users will be users, check all values for proper range ***/
	if( !valid_long(parsed[parm_indx++], &disp_dev, 0L, 3L, 
"Enter display device #. [0=screen 1=printer1 2=printer2 3=asci file q=QUIT]") )
		user_quit = TRUE;
	else if( !valid_long(parsed[parm_indx++], &len, 0L, 6L, prompt[51]) )
		user_quit = TRUE;
	else if( !valid_long(parsed[parm_indx++], &area, 0L, 6L, prompt[52]) )
		user_quit = TRUE;
	else if( !valid_long(parsed[parm_indx++], &r1, 0L, 6L, prompt[370]) )
		user_quit = TRUE;
	else if( !valid_long(parsed[parm_indx++], &r2, 0L, 6L, prompt[371]) )
		user_quit = TRUE;
	setnum = get_set(parsed[parm_indx++], &pf, 0L, 31L, 202);
	if( setnum == -1 )
		return(0);

/*** True if user wants to create file for generated report ***/
	if(disp_dev == 3 && !user_quit)            /*** create ascii file ***/
	{	strcpy( fil_nam, parsed[parm_indx] );
		if( !strcmp(fil_nam, "-1") || !strlen(fil_nam) )
		{	if( batch_mode )
				return(0);
			if(!valid_string(fil_nam,"Enter filename. [q = QUIT]"))
				user_quit = TRUE;
		}
/*** If absolute path, assume user has a clue as to whats going on, else  ***/
/*** use the default export path for a directory to park users info.      ***/
		if( fil_nam[0] != '/' )
			sprintf(fil_nam, "%s/%s", export_path, fil_nam);
	}

	if( user_quit )
	{	if( !batch_mode )
			message("REPORT: User requested quit. Report aborted.");
		return(0); 
	}
	else
	{	if( !collecting_macro )
			report_ltp(disp_dev, len, area, r1,r2, setnum, fil_nam);
		sprintf(raw_buf, "report %d  %d  %d  %d  %d  %d",
			disp_dev, len, area, r1, r2, setnum);
	/*** Set job_stats report flag  ***/
		job_stats[43] = job_stats[45] + 1;

		return(1);
	}
}

/*******************************************************************************
*	valid_long                                                            *
*            Checks for valid range of first parameter based on parms 2 and 3. *
*            If not valid, user is prompted for new value, and once a valid    *
*            value or quit has been received, break from loop and return.      *
*            Parameters:   string - user input to be convt'ed and validated.   *
*                          unit - Pointer to value to be checked/updated.      *
*                          low  - Low value of unit to be checked.             *
*                          high - High value of unit to be checked.            *
*                          msg  - Character string for prompting user.         *
*                      *----->>>  Always give user option to quit with 'q'!!!  *
*            Returns 1 if valid number received, else returns                  *
*            0 if user wants to quit.                                          *
*******************************************************************************/

long valid_long(string, unit, low, high, msg)
	char *string;
	long *unit;
	long low, high;
	char *msg;
{
	char *cptr;
	char resp[256];

	cptr = (char *)NULL;
	*unit = strtol(string, &cptr, 10);
	for( ;; )
	{	if( cptr == string )  /** force non-zero if bad convertion **/
			*unit = -1;
		if( *unit < low || *unit > high )
		{	if( batch_mode )
				return(0);
			message(msg);
			lowerc( trim( get_string(resp) ) );
			if( resp[0] == 'q' )
				return(0);	/** User wants to quit **/
			else
			{	cptr = (char *)NULL;
				*unit = strtol(resp, &cptr, 10);
			}
		}
		else
			break;
	}
	return(1);				/*** Device value OK  ***/
}

/*******************************************************************************
*	valid_string                                                           *
*            Promts user to enter a string for a filename or 'q' to quit.      *
*            Parameters:   string - char array to load filename into.          *
*                          msg    - Character string for prompting user.       *
*            Returns 1 if string was read OK, else returns 0 if user wants     *
*            to quit.                                                          *
*******************************************************************************/
long valid_string( string, msg )
	char *string,
             *msg;
{
/*** Loop until the user enters a string or quits!  ***/
	for( ;; )
	{	message(msg);
		lowerc( trim( get_string(string) ) );
		if( *string == 'q' )
			return(0);	/** User wants to quit **/
		else if( strlen(string) )
			return(1);
	}
}
