/*******************************************************************************
*    rotateM          (see manual, Command Ref., Function Manager)    <fm>     *
*        Displays the job stats file of the current map to the user in an      *
*        informative manner.  No changes are made to the job stats file.       *
*        Parameters:  A string containing the function arguments.  The         *
*                     following values are searched for:                       *
*                     arg1:  Number of degrees to rotate.  The allowable       *
*                            values are 90, 180, 270, or their negatives.      *
*        Globals Modified:  craster.                                           *
*                                                                              *
*        Returns 1 upon normal end-of-job (EOJ), else returns 0.               *
*        C. Jonson   1/5/90                                                    *
*******************************************************************************/
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include "../../incl/fc.h"
 
long rotateM(parms)
	char *parms;
{
	long   deg;
	char   resp[STRLIST];

/*** Ensure map exists first! ***/
	if( !job_stats[0] &&  !collecting_macro)
	{	message("ACQUIRE or CREATE a map before raster can be ROTATED.");
		*raw_buf = CNULL;
		return(0);
	}

/*** Is the parameter a -h (indicating rotate on horizontal plane)?  ***/
	if( strlen(trim(parms)) )
	{	deg = strtol(parms, (char **)NULL, 10);
		while( deg != 90 && deg != 180 && deg != 270 &&
				deg != -90 && deg != -180 && deg != -270)
		{	if( batch_mode )
				return(0);
			message("Valid degrees of rotation are 90, 180, 270, and their \
negatives. Enter degrees.");
			get_string(resp);
			if( abort_cmd() )
				return(0);
			deg = strtol(resp, (char **)NULL, 10);
		}
	}
	else 
		deg = 90;

/*** Pass the print flag to rotate_ltp, unless collecting macro ***/
	if( !collecting_macro )
		if(rotate(deg))
		{	*raw_buf = CNULL;       /* null out raw buf */
			return(0);
		}

	sprintf(raw_buf, "rotate %d", deg);

	return(1);
}
