/*******************************************************************************
*       find_atrbt      (see manual, Internal Functions)                <in>   *
*    Finds objects using the input x,y location & search radius.  Finds        *
*    the first attribute string and sets atrbt_string global.                  *
*                                                                              *
*    Arguments:    float x,y,r:   raster coordinates and search radius.        *
*                                 (search radius used for segs & nodes)        *
*                  long otype:    0 = node,  1 = segment,  2 = polygon         *
*                                                                              *
*    Returned Values: (long) 0 = found object & set attribute string global.   *
*                            1 = didn't find object or didn't find attribute.  *
*                                                                              *
*    Global Variables Modified:  none.                                         *
*                                                                              *
*                      j.dabritz  8/9/89                                       *
*******************************************************************************/

#include "../../incl/globals.h"

long find_atrbt(x,y,r,otype)
float x,y,r;
long otype;
{	
	OBJECTS object;
	ATTRIBUTES atrbt;

	char string[STRLIST];

	long obj_no;


		/********* check for map ****************************************/
	if(!job_stats[0])
	{	message("no current map.");
		return(1);
	}


		/********* check for objects ************************************/
	if(!job_stats[10])
	{	message("Map has not been assembled.  There are no objects to assign attributes.");
		return(1);
	}

		/********* find object ******************************************/
	switch(otype)     {
	case 0:           /** nodes ***/
		obj_no = find_vnode(x,y,r,0);
		break;
	case 1:           /** segments  */
		obj_no = find_vseg(x,y,r,0);
		break;
	case 2:           /** polygons  */
		obj_no = find_vpoly(x,y,0);
		break;
	default:
		return(1);
	}

		/********* read attribute & set global **************************/
	if(obj_no >= 0)
	{		/**** get object ****/
		if(get_objects(&object,obj_no))
		{	message("find_atrbt ERROR #1.  Attribute string NOT set.");
			return(1);
		}
			/**** set attribute string global ***/
		if(object.i12[2] == 0)
		{	sprintf(string,"Object found, but has NO attribute.  current atrbt = %s",atrbt_string);
			message(string);
			return(1);
		}
		if(get_attr(&atrbt,object.i12[2]))
		{	message("find_atrbt ERROR #2.  Attribute string NOT set.");
			return(1);
		}
		sprintf(atrbt_string,"%s",atrbt.string);
		atrbt_string[30] = (char) 0;
		sprintf(string,"New Attribute String:  %s",atrbt_string);
		message(string);
	}
	else
	{	message("No object found.");
		return(1);
	}

		/********* return ***********************************************/
	return(0);
}
