/*******************************************************************************
*       add_geoframe    (see manual, Internal Functions)                <in>   *
*                       This program draws a geodetic fram on import level 32. *
*                                                                              *
*                       The index_list, index_array, v_objects, v_coordlist,   *
*                       v_attributes files are changed and the conters for     *
*                       them in the job_stats file.                            *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  10/6/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */

int   closef();         /*  (int stream)                                    */
int openf();          /*  (char *path, char *type)                      */
char *reada();          /*  ( char *string, int bytes, int stream )         */
char *trim();           /*  (char *string)                                    */


long add_geoframe()
{
	V_OBJECTS v_object;
	COORD_LIST coord_list;

	long i,j;

	double x,y;
	double x0,y0;
	double dx,dy;


		/******** check for map & registration **********************/
	if(!job_stats[0]   ||   !registered(1) ||  !registered(0))
		return(1);

	for(i=0; i<4; i++)
	{	if((corner[0][i][0] == 0.0)  || (corner[1][i][0] == 0.0)  )
			return(1);
	}

		/******** delete old frame **********************************/
	level_clear(32,32);

		/******** add all 4 sides to level 32 ***********************/
	v_object.type = 321;
	v_object.atr = 0;
	v_object.v1 = job_stats[18];


	for(i=0; i<4; i++)
	{		/*** get starting coordinates for side.  **/
		x = x0 = corner[1][i][0];
		y = y0 = corner[1][i][1];

			/*** add 100 points along the side ***/
		dx = (corner[1][(i + 1) % 4][0] - corner[1][i][0]) / 100.0;
		dy = (corner[1][(i + 1) % 4][1] - corner[1][i][1]) / 100.0;

		for(j=0; j<100; j++)
		{	
			transform(&x,&y,1,0);
			coord_list.x = x;
			coord_list.y = y;
			if(put_v_coord(&coord_list,job_stats[18]))
			{	message("Unable to store verticies for geodetic frame.");
				return(1);
			}
			x0 += dx;
			y0 += dy;
			x = x0;
			y = y0;
		}
	}
		/*** add final corner to v_coordlist file ***/
	x = corner[1][0][0];
	y = corner[1][0][1];
	transform(&x,&y,1,0);
	coord_list.x = x;
	coord_list.y = y;
	if(put_v_coord(&coord_list,job_stats[18]))
	{	message("Unable to store verticies for geodetic frame.");
		return(1);
	}


		/******** store v_object record *****************************/
	v_object.v2 = job_stats[18] - 1;
	if(put_vobjects(&v_object,job_stats[17]))
	{	message("Unable to store objects for geodetic frame.");
		return(1);
	}


		/******** index v_objects ***********************************/
	if(index_vobj(job_stats[17] - 1))
	{
		message("Add_geoframe:  Index_vobj failed.");
		return(1);
	}
	

		/******** set level name ************************************/
	sprintf(level_names[32],"geodetic map frame");


		/******** return ********************************************/

	return(0);
}
