/*******************************************************************************
*       add_plnframe    (see manual, Internal Functions)                <in>   *
*                       This program draws a planar frame on import level 33,  *
*                       and the raster corners (as points) on level 34.        *
*                                                                              *
*                       The index_list, index_array, v_objects, v_coordlist,   *
*                       v_attributes files are changed and the conters for     *
*                       them in the job_stats file.                            *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  11/1/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */

int   closef();         /*  (int stream)                                    */
int openf();          /*  (char *path, char *type)                      */
char *reada();          /*  ( char *string, int bytes, int stream )         */
char *trim();           /*  (char *string)                                    */


long add_plnframe()
{
	V_OBJECTS v_object;
	COORD_LIST coord_list;

	long i,j;
	long index_start;

		/******** check for map & registration **********************/
	if(!job_stats[0]   ||  !registered(0))
		return(1);

	for(i=0; i<4; i++)
	{	if((corner[0][i][0] == 0.0)  || (corner[0][i][1] == 0.0)  )
			return(1);
	}

		/******** delete old frame **********************************/
	level_clear(33,34);

		/******** add all 4 sides to level 33 ***********************/
	index_start = job_stats[17];
	v_object.type = 331;
	v_object.atr = 0;
	v_object.v1 = job_stats[18];

	for(i=0; i<5; i++)
	{		/*** get starting coordinates for side.  **/
		j = i % 4;
		coord_list.x = corner[0][j][0];
		coord_list.y = corner[0][j][1];
		if(put_v_coord(&coord_list,job_stats[18]))
		{	message("Unable to store verticies for planar frame.");
			return(1);
		}
	}

		/******** store v_object record *****************************/
	v_object.v2 = job_stats[18] - 1;
	if(put_vobjects(&v_object,job_stats[17]))
	{	message("Unable to store objects for planar frame.");
		return(1);
	}


		/******** store raster corners in level 34 ******************/
	v_object.type = 340;
	v_object.atr = 0;

	for(i=0; i<4; i++)
	{	j = i % 4;
		coord_list.x = corner[0][j][0];
		coord_list.y = corner[0][j][1];
		if(put_v_coord(&coord_list,job_stats[18]))
		{	message("Unable to store verticies for corner level.");
			return(1);
		}
		v_object.v1 = job_stats[18] - 1;
		v_object.v2 = job_stats[18] - 1;
		if(put_vobjects(&v_object,job_stats[17]))
		{	message("Unable to store objects for corner level.");
			return(1);
		}
	}

		/******** index v_objects ***********************************/
	if(index_vobj(index_start))
		return(1);
	

		/******** set level names ***********************************/
	sprintf(level_names[33],"planar (straight) map frame");
	sprintf(level_names[34],"map corners");


		/******** return ********************************************/
	return(0);
}
