/*******************************************************************************
*       add_spur        (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    adds a 3 pixel spur to the point x,y (to create a node junct).*
*                                                                              *
*    Arguments:    long x,y;      pixel coords to draw spur from               *
*                  long *x,*y;    pointers to longs for other end of spur      *
*                                                                              *
*    Returned Values: (long) 0 = spur drawn, x2 & y2 set to other end          *
*                            1 = no spur drawn,  x & y not on an 'on' pixel.   *
*                                                                              *
*    Global Variables Modified:  none                                          *
*                      j.dabritz  6/25/90                                      *
*******************************************************************************/

#include "../../incl/globals.h"


long add_spur(x,y,x2,y2)
	long x,y;
	long *x2,*y2;
{	
	static long dir_x[8] = { 0, 3, 3, 3, 0, -3, -3, -3 };
	static long dir_y[8] = { 3, 3, 0, -3, -3, -3, 0, 3 };
	long word, bit, b;
	long i;
	long gap[2], start[2];
	long dir, hit;
	long locale;


		/***************** check for map present *********************/
	if(!job_stats[0])
		return(1);

		/***************** check input paramters *********************/
	if((x < 0) || (x > job_stats[1]) || (y < 0) || (y > job_stats[2]))
		return(1);

		/***************** check for an 'on' pixel *******************/
	word = x >> 5;
	bit = (7 & ~x) | (x & 24);
	if(!(lraster[y][word] & (1 << bit)))
	{	message("Not near enough to a line to draw a spur.");
		return(1);
	}

		/***************** get locale ********************************/
	locale = locale_xy(x,y);

		/***************** find direction to draw spur ***************/
	if(!(tlocale[locale] & (1 << 10)))
	{	message("Spur cannot be located at the current position.");
		return(1);
	}

	gap[0] = gap[1] = 0;
	for(i=0, b=0, hit=1; b<8; b++)
	{	if(tlocale[locale] & (1 << b))        /** hit a branch **/
		{	i ^= 1;                          /** toggle betwen 0 & 1 **/
			hit = 1;
		}
		else                                /** maybe open    **/
		{	if(!(locale & (1 << b)))         /** definitely open **/
			{	gap[i]++;                     /** add to gap count **/
				if(hit)
				{	start[i] = b;
					hit = 0;
				}
			}
		}
	}

	if(gap[0] > gap[1])
		dir = start[0] + gap[0] / 2;
	else
		dir = start[1] + gap[1] / 2;

	dir = dir % 8;

		/***************** find other endpoint ***********************/
	*x2 = x + dir_x[dir];
	*y2 = y + dir_y[dir];

		/***************** draw line *********************************/
	line_diag(x,y,*x2,*y2,1);

		/***************** return ************************************/
	return(0);
}
