/*******************************************************************************
*       asmbl_polys     (see manual, Internal Functions)                <in>   *
*                                                                              *
*    assemble all polygons including complete map topology.                    *
*                                                                              *
*    Arguments:    none                                                        *
*                                                                              *
*    Returned Values: (long) 1 = failure                                       *
*                            0 = normal end of job.                            *
*                                                                              *
*    Global Variables Modified:  job_stats[10, 11, 12], objects, and           *
*    seg_list files.                                                           *
*                                                                              *
*                      j.dabritz  7/11/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long *seg_done;           /*  flags for each seg marding traced status:
                                    0=untraced, 1=1 side traced, 2=both sides */

long asmbl_polys()
{ 
  long trace_poly();        /*  program declaration */

  OBJECTS segment;          /*  structure for segments  */
  OBJECTS node;             /*  structure for nodes  */
  OBJECTS poly, poly2;      /*  structures for polygons  */
  long *low_ys;             /*  array of low y values for each segment  */
  long *seg_list;           /*  list of all segments in increas. low_y order*/

  long b,d,i,j,p,s,y;       /*  counters (p=poly, s=segment) & utility vars. */
  long map_border;          /*  flag for map border polygon    */
  long next_y;              /*  y value of next lowest segment(s)  */
  long rec;                 /*  record number  */
  long rt_poly;             /*  record number of polygon to the right  */
  long seg_list_index;      /*  index into the segment list (array el. number)*/
  long size;                /*  number of elements in the seg_list array   */
  long base, breakoff;      /*  used for filling seg_list array    */

  char string[256];

     /***************** check for polygon map type ****************/
  if(!(job_stats[0])  ||  !(map_type & 4))
    return(0);

     /***************** check for segments ************************/
  if(!job_stats[5] && !job_stats[6])
  { message(" there are no segments to assemble.");
    return(0);
  }

     /***************** create & sort seg_order array (sorted bottom to top) */
  if(job_stats[6])
  { if(job_stats[5])
      size = job_stats[4] - job_stats[5] + (job_stats[10] - job_stats[4]) / 2;
    else
      size = (job_stats[10] - job_stats[4]) / 2;
  }
  else
    size = (job_stats[10] - job_stats[5]);
    /*** allocate space for array of segment rec numbers ordered by y pos. */
  seg_list = (long *) malloc_ltp(size * 4);
  if(!(seg_list))
  {  message("not enough ram to continue assembly.");
    return(1);
  }
    /*** allocate space for array of low y values for each segment *********/
  low_ys = (long *) malloc_ltp(size * 4);
  if(!(low_ys))
  {  message("not enough ram to continue assembly.");
    free_ltp((char *) seg_list);
    return(1);
  }
    /**** write ordered list ***/

  message("writting list of segments ordered by y-position . . . ");
    /*** first read all segment records & copy the low_y value to low_ys[] */
  i = (job_stats[5]) ? job_stats[5] : job_stats[6];
  rec = 0;
    /** fill low_ys array with segment low_y values in segment order **/
  for(; (i < job_stats[10]) ; i++)
  {    /*** check for user abort **/
    if(abort_cmd())
      return(1);
      /*** read object record ***/
    if(get_objects(&segment,i))
    {  message("abort, asmbl_polys, location 1");
      return(1);
    }
    if(segment.i12[0] > 7)     /** not a node **/
      low_ys[rec++] = segment.i12[2];
  }

    /*** fill seg_list array with seg rec nos in increasing low_y order ***/
  base = (job_stats[5]) ?  job_stats[5] : job_stats[6];
  if(job_stats[5])
    breakoff = (job_stats[6]) ? (job_stats[4] - job_stats[5]) : job_stats[10];
  else
    breakoff = 0;
  y = 0;
  next_y = job_stats[2];
    /******* fill seg_list array with seg recs in increasing low_y order **/
  for(j=0; j<size; y = next_y, next_y = job_stats[2]) 
  {    /** check for user abort **/
    if(abort_cmd())
      return(1);

    for(i=0; i<size; i++)        /** look through low_ys array **/
    {
      if(low_ys[i] == y)       /** if low_y is on this row, put s in array */
      {  s = (i < breakoff) ?  base + i:  job_stats[6] + 2 * (i - breakoff);
        seg_list[j++] = s;
      }
      else                       /** otherwise check for next low_y  */
      {  if((low_ys[i] > y)  &&  (low_ys[i] < next_y))
          next_y = low_ys[i] ;
      }
    }
  }
  append_msg("done.");
      
  
    /**** set up seg_done array re-using space of low_ys array ***/
  seg_done = low_ys;
  for(i=0; i<size; i++)
    seg_done[i] = 0;

     /** check for user abort **/
  if(abort_cmd())
    return(1);

  message("Tracing all polygons . . . ");
     /***************** trace all polygons ************************/
    /** set begin poly number **/
  job_stats[7] = job_stats[10];
    /** set loop flags **/
  map_border = 1;          /*  flag for map_border polygon  */
  seg_list_index = 0;
  for(;;seg_list_index++)
  {    /***************** trace lowest exterior polygon *************/
      /*---------------------------------------------------------*/
       /** check for user abort **/
    if(abort_cmd())
      return(1);
      /** look in seg_list for next lowest segment ***/
    for(; (seg_list_index < size) ; seg_list_index++)
    {  s = abs(seg_list[seg_list_index]);
      if(job_stats[5])
      {  if(job_stats[6]  &&  (s > job_stats[4]) )
          i = job_stats[4] - job_stats[5] + (s - job_stats[6]) / 2;
        else
          i = s - job_stats[5];
      }
      else
        i = (s - job_stats[6]) / 2;
      if(!seg_done[i])
        break;
    }
    if(seg_list_index >= size)
      break;
    if(get_objects(&segment,seg_list[seg_list_index]))
    {  message("abort, asmbl_polys, location 2");
      return(1);
    }
      /*---------------------------------------------------------*/
      /**** make sure that segment is on the exterior of group **/
    switch(segment.i12[3])  {
    case 0:                      /*** low point at beginning node */
      if(get_objects(&node,segment.i12[9]))
      {  message("abort, asmbl_polys, location 3");
        return(1);
      }
        /** find direction of first branch counter clockwise of direction 2 ***/
      for(d=2; !(node.i12[5] & (1 << d)); d = (d + 7) & 7);
        /** find branch number **/
      for(b=0, d--; d>=0; d--)
      { if(node.i12[5] & (1 << d))
          b++;
      }
        /*** get segment number ***/
      s = node.i12[6 + b];
      break;
    case 1:                      /*** low point with right side down ***/
      s = seg_list[seg_list_index];  /** use segment in forward direction */
      break;
    case 2:                      /*** low point with left side down ***/
      s = -seg_list[seg_list_index]; /** use segment in reverse direction */
      break;
    }
      /*---------------------------------------------------------*/
      /**** find the polygon that it is interior to ***/
    rt_poly = find_polyr(segment.i12[1], segment.i12[2] - 1);
      /*---------------------------------------------------------*/
      /**** trace exterior polygon. **/
    if((p = trace_poly(s,rt_poly, (15 + map_border))) == -1)
		return(1);
      /*---------------------------------------------------------*/
      /*****  link poly into next island list of parent.  ***/
    if(p && !map_border)
    {    /*** get polygon & parent polygon **/
      if(get_objects(&poly,p))               /** exterior poly just traced */
      {  message("abort, asmbl_polys, location 4");
        return(1);
      }
      if(get_objects(&poly2,poly.i12[7]))          /** parent polygon **/
      {  message("abort, asmbl_polys, location 5");
        return(1);
      }
        /** insert poly into front of linked-list **/
      poly.i12[8] = poly2.i12[8];
      poly2.i12[8] = p;
        /** store polygons **/
      if(put_objects(&poly,p))
      {  message("abort, asmbl_polys, location 6");
        return(1);
      }
      if(put_objects(&poly2,poly.i12[7]))
      {  message("abort, asmbl_polys, location 7");
        return(1);
      }
    }
    map_border = 0;         /** reset map_border flag **/
    
       /***************** trace all connected interior polygons *****/
    if(p)  /** polygon has innards ********************************/
    {    /*** read through rest of segments on list, trace those started. */
      for(;;)
        /*---------------------------------------------------------*/
      {
          /** check for user abort **/
        if(abort_cmd())
          return(1);

        if(!((job_stats[10]-job_stats[7]) % 250))
        { sprintf(string,"  assembling, now at polygon %d",job_stats[10]-job_stats[7]);
          message(string);
        }
           /** find next segment not traced **/
        for(i=0; i < size; i++)
        {  if(seg_done[i] == 1)
            break;
        }
        if(i >= size)      /*** no more segments partially traced **/
          break;
      
        s = (i < breakoff) ?  base + i:  job_stats[6] + 2 * (i - breakoff);
        if(get_objects(&segment,s))
        {  message("abort, asmbl_polys, location 8");
          return(1);
        }
        s = (segment.i12[7])  ?  -s : s;
          /*---------------------------------------------------------*/
          /**** trace interior polygon. **/
        if((p = trace_poly(s,0,14)) == -1)
			return(1);
      }
    }
  }

    /***************** free allocated space **********************/
  free_ltp((char *) seg_list);
  free_ltp((char *) low_ys);

  sprintf(string,"all polygons built.  (%d total)",job_stats[10]-job_stats[7]);
  message(string);
    /***************** return ************************************/

  return(0);

}

/*******************************************************************************
*    trace_poly                                                                *
*          This program starts with an input segment number and traces the     *
*    polygon to its right.                                                     *
*                                                                              *
*    paramaters:  long seg       record number of beginning segment, negative  *
*                                sign (-) indicates to use segment in reverse  *
*                 long parent    record number of parant polygon (if any)      *
*                 long type      object type for this polygon (14-16) (see     *
*                                objects file description for .i12[0])         *
*                                                                              *
*    globals & files changed:  job_stats[10], job_stats[12], and the objects   *
*                              and seg_list files.                             *
*                                                                              *
*    returned:   (long) p = poly rec number (0 = nothing there but spurs).     *
*                       ( -1 = abort)                                          *
*                      j.dabritz  7/17/89                                      *
*******************************************************************************/

long trace_poly(seg, parent, type)
  long seg;
  long parent;
  long type;
{
  long trim_spurs();

  OBJECTS node, segment, poly; /*  object structures */
  
  long b;                    /*  branch number of segment into node */
  long i;                    /*  program utility counter */
  long s,n;                  /*  segemnt & node record numbers in objects file*/
  long p;                    /*  polygon record number in objects file */

    /***************** init polygon structure ****************************/
  for(i=0; i<12; i++)
    poly.i12[i] = 0;
  for(i=0; i<4; i++)
    poly.f4[i] = 0.0;

  poly.i12[0] = type;
  if(type == 15)
    poly.i12[7] = parent;
  poly.i12[5] = job_stats[12];

    /***************** trace around polygon until first seg repeated. *****/
  for(s = seg; ;)
  {    /***************** store segment number in seg_list & update poly ****/
    if(put_seg_list(&s,job_stats[12]))
    {  message("abort, asmbl_polys, location 9");
      return(-1);
    }

      /***************** update seg_done array marking segment as traced ***/
    if(job_stats[5])
    {  if(job_stats[6]  &&  (abs(s) > job_stats[4]) )
        i = job_stats[4] - job_stats[5] + (abs(s) - job_stats[6]) / 2;
      else
        i = abs(s) - job_stats[5];
    }
    else
      i = (abs(s) - job_stats[6]) / 2;

    seg_done[i] += 1;
    
      /***************** get segment (object) record ***********************/
    if(get_objects(&segment,abs(s)))
    {  message("abort, asmbl_polys, location 10");
      return(-1);
    }

      /***************** store poly number in segment record ***************/
    if(s > 0)              /** segment traced in forware direction */
      segment.i12[7] = (type == 15) ?   parent : job_stats[10];
    else
      segment.i12[8] = (type == 15) ?   parent : job_stats[10];
    if(put_objects(&segment,abs(s)))
    {  message("abort, asmbl_polys, location 11");
      return(-1);
    }

      /***************** find next node ************************************/
    n = (s > 0) ?   segment.i12[10] : segment.i12[9];

      /***************** add segment length to polygon perimeter ***********/
    poly.f4[2] += segment.f4[2];

      /***************** find segment incomming branch at node *************/
    if(get_objects(&node,n))
    {  message("abort, asmbl_polys, location 12");
      return(-1);
    }

    for(b=1; node.i12[5+b] != -s; b++);

      /***************** find next segment record number *******************/
    s = (b > 1)  ?  node.i12[b+4] :  node.i12[node.i12[0] + 2];
  
      /***************** if next segment is same as beginning seg, exit ****/
    if(s == seg)
      break;
  }

    /***************** set last seg_list record number *******************/
  poly.i12[6] = job_stats[12] - 1;

    /***************** trim all spurs from polygon seg_list **************/
  switch(trim_spurs(&poly)) {      /* returns 0 if nothing left */
  case 1:
    return(0);   /* <<<<<<<<<<<<<<<<<<<< return <<<<<<<<< */
  case -1:
	return(-1);
  }

     /***************** store polygon structure ***************************/
  p = job_stats[10];
  if(put_objects(&poly,p))
  {  message("abort, asmbl_polys, location 13");
    return(-1);
  }

    /***************** return ********************************************/
  return(p);
}



/*******************************************************************************
*    trim_spurs                                                                *
*          This program removes all spurs from the seg_list for 1 polygon.     *
*    This program assumes that seg_list portion for this polygon is at the     *
*    end of the seg_list file.                                                 *
*                                                                              *
*    paramaters:  OBJECTS *poly  pointer to a polygon structure to be trimmed  *
*                                                                              *
*    globals & files changed:  job_stats[12], seg_list file                    *
*                                                                              *
*    returned:   (long) 1 = no segments left in poly, 0=segments left in poly, *
*                      or -1 = abort failure.                                  *
*                      j.dabritz  7/17/89                                      *
*******************************************************************************/

long trim_spurs(poly_ptr)
  OBJECTS *poly_ptr;
{  
  static long zero = 0;  /*  for storing blank records */
  long begin,end;        /*  beginning & ending rec. nos in the seg_list file */
  long rec1,rec2;        /*  current records to check for spur */
  long s1,s2;            /*  segment record values to check for spur */
  long hits;             /*  number of spurs deleted        */
  long new,old;          /*  record numbers used in re-writting seg_list file */
  
  OBJECTS segment;

    /******************* check for poly type (don't trim map border) ***/
  if((*poly_ptr).i12[0] == 16)
    return(0);

    /******************* go through seg list, setting spur recs to 0 ***/
  begin = (*poly_ptr).i12[5];
  end = (*poly_ptr).i12[6];
  rec1 = begin;
  rec2 = begin + 1;
  if(get_seglist(&s1,rec1))
  {  message("abort, asmbl_polys, location 14");
    return(-1);
  }
  hits = 0;
  for(; rec2 <= end; )
  {    /*** get s2, (segment number )   ****/
    if(get_seglist(&s2,rec2))
    {  message("abort, asmbl_polys, location 15");
      return(-1);
    }
    if(s1 == -s2)         /*** a spur, eliminate it ***/
    { if(put_seg_list(&zero,rec1))
      {  message("abort, asmbl_polys, location 16");
        return(-1);
      }
      if(put_seg_list(&zero,rec2))
      {  message("abort, asmbl_polys, location 17");
        return(-1);
      }
        /*** get segment record & remove segment length from poly perimeter */
      if(get_objects(&segment,abs(s1)))
      {  message("abort, asmbl_polys, location 18");
        return(-1);
      }
      (*poly_ptr).f4[2] -= (2.0 * segment.f4[2]);
        /*** increment hits flag **/
      hits++;
        /*** increment rec2 up ***/
      rec2++;
        /*** decrement rec1 back to non-zero **/
      for(rec1--; rec1 >= begin; rec1--)
      { if(get_seglist(&s1,rec1))
        {  message("abort, asmbl_polys, location 19");
          return(-1);
        }
        if(s1)
          break;
      }
      if(rec1 < begin)   /** rec1 prior to polygon, increment rec2 & rec1 **/
      { rec1 = rec2;
        rec2++;
        if(rec2 <= end)
        { if(get_seglist(&s1,rec1))
          {  message("abort, asmbl_polys, location 20");
            return(-1);
          }
        }
      }
    }
    else
    { s1 = s2;
      rec1 = rec2;
      rec2++;
    }
  }

    /******************* re-write list eliminating zeros ***************/
  if(hits);
  { new = begin;
    old = begin;
    for(; old <= end; old++)
    { if(get_seglist(&s1,old))
      {  message("abort, asmbl_polys, location 21");
        return(-1);
      }
      if(s1)
      { if(put_seg_list(&s1,new++))
        {  message("abort, asmbl_polys, location 22");
          return(-1);
        }
      }
    }
      /***************** reset polygon structure pointer to last seg_list rec */
    (*poly_ptr).i12[6] = new - 1;
      /***************** reset job_stats[12] ***************************/
    job_stats[12] = new;
      /******************* return ****************************************/
    if(new == begin)
      return(1);
  }
  return(0);
}
