/*******************************************************************************
*	attmse_input   (see manual, Internal Functions)            	<in>   *
*	    		Checks for ATT busmouse input.  The raw input          *
*			is put into the ginfo global data structure.           *
*                                                                              *
*			Globals Modified:                                      *
*			        ginfo.button                                   *
*			        ginfo.rawcoord[0]                              *
*			        ginfo.rawcoord[1]                              *
*                                                                              *
*			Returns 0.                                             *
*			J.Dabritz  6/21/89, modified code written by C.Jonson  *
*******************************************************************************/

#include <stdio.h>
#include <sys/ioctl.h>
#include "../../incl/globals.h"

static struct {			/* ioctl structure */
    int x;			/* x coordinate    */
    int y;			/* y coordinate    */
    int button;			/* button pressed  */
    int status;			/* status: 0 = disabled, 1 = enabled */
} mouse_data;

static int oldx, oldy;		/* x and y coordinates before move */

long attmse_input()
{
/*  Locals       */

#ifdef SV32
	static int first_time = 1;
	static long debounce;

		/* initialize the mouse and button status */
	if(first_time || (oldx-ginfo.scoord[0]) || (oldy-ginfo.scoord[1]))
	{
			/* put the coordinates in the ioctl structures */
		oldx = mouse_data.x = (int) ginfo.scoord[0];
		oldy = mouse_data.y = (int) ginfo.scoord[1];

		first_time = 0;
		mouse_data.button = 0;
		ioctl(mousfd, MOUSE_INIT, &mouse_data);
		return(0);
	}

		/* read the mouse */
	oldx = mouse_data.x;
	oldy = mouse_data.y;
	mouse_data.status = 0;
	ioctl(mousfd, GET_MOUSE, &mouse_data);
	while (mouse_data.status == 0);

		/*  set raw data in ginfo global structure */
	ginfo.rawcoord[0] = mouse_data.x - oldx;
	ginfo.rawcoord[1] = oldy - mouse_data.y;
	if(mouse_data.button)
	{	if(debounce > 0)
			ginfo.button = 0;
		else
		{	switch(mouse_data.button)  {
			case 1:
				ginfo.button = 'L';
				break;
			case 2:
				ginfo.button = 'M';
				break;
			case 3:
				ginfo.button = 'R';
				break;
			default:
				ginfo.button = 0;
				break;
			}
			debounce = 200;
		}
	}
	else
		ginfo.button = 0;

	debounce--;

#endif

	return(0);
}
