/*******************************************************************************
*       bar_off         (see manual, Internal Functions)                <in>   *

     Purpose:	Erases a rectangular bar within limits x1-x2, y1-y2 inclusive

     Arguments:	long x1,y1,x2,y2;     limits of rectangle

     Returned Values: (long) 0

     Global Variables Modified:   - none - 

*                      j.dabritz  3/7/89 - 5/8/89                              *
*******************************************************************************/
/* Modified 5/92 by DBS@CWU for X11 support */


#include "../../incl/globals.h"

long minl();
long maxl();
long swplane();       /*  (long fd,planenum)                               */

#ifndef LTP_X11   /***** first the VGA code *****/

long bar_off(x1,y1,x2,y2)
long x1,y1,x2,y2;
{ 	char *ptr;		/*  pointer used for copying to bitplanes  */
	char *ptr0, *ptrn;	/*  pointers to beginning & ending of bar  */
		/*   setup bitmasks for bars which start/end within bytes  */
	static char w0[8] = {0, -128, -64, -32, -16, -8, -4, -2};
	static char wn[8] = {127, 63, 31, 15, 7, 3, 1, 0};
	long i;			/*  counter				   */
		/*   masks taken from the list above used for actual copy  */
	char mask0,maskn;
	long increment;          /* pointer increment value                */
	long whole_words;	 /* number of complete bytes accross bar   */
	long minx,maxx,miny,maxy;/* limits of bar within screen area       */


		/*   check to see if grahpics are being used.  */
	if(batch_mode)
		return 0;


	/* Check to see if any of the bar is on screen                     */
	if((x1<0 && x2<0) || (x1>639 && x2>639) || (y1<0 && y2<0) || (y1>479 && y2>479))
		return (long) 0;


	/* Set mins and maxes & make sure that they are on the screen  */
	minx = maxl(2, 0, minl(2, x1, x2));
	maxx = minl(2, 639, maxl(2, x1, x2));
	miny = maxl(2, 0, minl(2, y1, y2));
	maxy = minl(2, 479, maxl(2, y1, y2));


	/*  calculate beginning pointer & ending pointer               */
	ptr0 = vga_plane + (miny * BITPLN_WIDTH) + (minx>>3);
	ptrn = vga_plane + (maxy * BITPLN_WIDTH) + (maxx>>3);
	

	/*  calculate beginning & ending masks                         */
	mask0 = w0[minx & 7];
	maskn = wn[maxx & 7];


	/*  draw the bar on the bit plane                              */
	if(minx>>3 == maxx>>3)    /*  then all in one byte column     */
	{	mask0 = mask0 | maskn;
		for(ptr=ptr0; ptr<=ptrn; ptr = ptr + BITPLN_WIDTH)
			*ptr = *ptr & mask0;
	}
	else                      /*  in more than one byte column   */
	{	whole_words = (maxx>>3) - (minx>>3) - 1;
		increment = BITPLN_WIDTH - whole_words - 2;
		for(ptr=ptr0; ptr<=ptrn; ptr = ptr + increment)
		{	*ptr = *ptr & mask0;
			ptr++;
			for(i=0; i<whole_words; i++, ptr++)
				*ptr = 0;
			*ptr = *ptr & maskn;
			ptr++;
		}
	}

	return (long) 0;
}	

#else   /***** now the X11 code *****/

#include "../../incl/x11.h"

long bar_off(x1,y1,x2,y2)
long x1,y1,x2,y2;
{
  long minx,maxx,miny,maxy;	/* limits of bar within screen area       */


  /*   check to see if grahpics are being used.  */
  if(batch_mode)
    return 0;


  /* Check to see if any of the bar is on screen                     */
  if((x1<0 && x2<0) || (x1>=WIDTH && x2>=WIDTH) || (y1<0 && y2<0) || (y1>=HEIGHT && y2>=HEIGHT))
    return (long) 0;


  /* Set mins and maxes & make sure that they are on the screen  */
  minx = maxl(2, 0, minl(2, x1, x2));
  maxx = minl(2, WIDTH-1, maxl(2, x1, x2));
  miny = maxl(2, 0, minl(2, y1, y2));
  maxy = minl(2, HEIGHT-1, maxl(2, y1, y2));


  /*  draw the bar on the bit plane */
  XFillRectangle(xdisplay,xmainwin, xgcclear, minx, miny, maxx-minx+1, maxy-miny+1);


  return (long) 0;
}	

#endif  /***** end of the X11 code *****/
