
/*  This function swaps bytes within a section of memory to help
    LTPlus work correctly on a Big Endian machine.
       byte_swap(ptr, length, type)
	 ptr - pointer to the memory to be swapped
	 length - the number of bytes to perform the swapping on.
	 type - the type of byte swapping to perform
		possible values: SWAP_LONG, SWAP_SHORT

       return value: 0 if successful, 1 if failed.

    DBS@CWU 5/92
*/
#include "../../incl/big_end.h"


long byte_swap(ptr, length, type)
  unsigned char *ptr;
  long length;
  long type;
{
  unsigned char temp;
  unsigned char *end_ptr;

/* printf("byte_swap(): at beginning\n"); */

  /* check for a match on the length and type of swapping */
  switch (type) {
    case SWAP_LONG:
	if (length % sizeof(long int) != 0) { 
	  printf("byte_swap: mismatched size and type LONG\n");
	  return 1;
	}
	break;
    case SWAP_SHORT:
	if (length % sizeof(short int) != 0) {
	  printf("byte_swap: mismatched size and type SHORT\n");
	  return 1;
	}
	break;
    case SWAP_DOUBLE:
	if (length % sizeof(double) != 0) {
	  printf("byte_swap: mismatched size and type DOUBLE\n");
	  return 1;
	}
	break;
    default:
        printf("byte_swap: undefined type - failing.\n");
	return 1;
	break;
  }


  /* handle each swap case separately to allow efficient algorithms
     for each spaw type.  */
  switch (type) {
    case SWAP_LONG:
	end_ptr = ptr + length;
	while (ptr<end_ptr){
          /* this swap assumes a 4 byte long int!!! */
	  temp = *ptr;      /* swap first byte and last */
	  *ptr = *(ptr+3);
	  *(ptr+3) = temp;
	  temp = *(ptr+1);      /* swap second and third bytes */
	  *(ptr+1) = *(ptr+2);
	  *(ptr+2) = temp;
	  ptr+=4;
	}
	break;
    case SWAP_SHORT:
	break;
    case SWAP_DOUBLE:
	break;
  }

  return 0;
}
