/*******************************************************************************
*       change_mode     (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Set the new input mode for use by 'start', 'continue',        *
*       'increment', and 'decrement'.  Sets cursor type, and displays mode.    *
*                                                                              *
*    Arguments:    long new_mode:    0 = none                                  *
*                                    1 = attribute contours                    *
*                                    2 = attribute digitize                    *
*                                    3 = connect                               *
*                                    4 = draw point                            *
*                                    5 = draw line                             *
*                                    6 = draw fat-line                         *
*                                    7 = draw rectangle                        *
*                                    8 = erase point                           *
*                                    9 = erase line                            *
*                                   10 = erase fat-line                        *
*                                   11 = erase rectangle                       *
*                                   12 = erase segment                         *
*                                   13 = query                                 *
*                                   14 = separate                              *
*                                   15 = wink                                  *
*                                   16 = draw wedge                            *
*                                   17 = erase wedge                           *
*                                   18 = recall point                          *
*                                   19 = recall line                           *
*                                   20 = recall fat-line                       *
*                                   21 = recall rectangle border               *
*                                   22 = recall rectangle area                 *
*                                   23 = recall wedge                          *
*                                   24 = recall separate                       *
*                                   25 = recall connect                        *
*                                   26 = bridge_gap                            *
*                                   27 = seg_break                             *
*                                   28 = draw spur                             *
*                                   29 = atrbt_line                            *
*                                   30 = erase_obj                             *
*                                   31 = vertex_move                           *
*                                   32 = level_icopy                           *
*                                   33 = set_dig                               *
*                                   34 = atrbt_copy                            *
*                                   35 = atrbt_list                            *
*                                                                              *
*                  long snap:  0 = set snap_input to 0, otherwise set to 1.    *
*       NOTE:  see incl/lists.h for a current list of input modes.             *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified: Updates:  input_mode, snap_input               *
*                               Sets:     first_point = 1;                     *
*                                                                              *
*                      j.dabritz  6/26/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"

long change_mode(new_mode,snap)
	long new_mode;
	long snap;     
{	
	long i;
	long ot10, level;


		/***************** check input parameters ********************/
	if((new_mode < 0) || (new_mode > input_mode_limit))
		new_mode = 0;


		/***************** set cursor type ***************************/
	cursor_mode = 0;  /*  cross hair (changes after first point set) */


		/***************** close file if last mode == atrbt_list (35) */
	if(input_mode == 35)
		atrbt_list_close();

		/***************** set input_mode global variable ************/
	input_mode = new_mode;


		/***************** set snap_input global variable ************/
	snap_input = snap;


		/***************** display new mode **************************/
	if(input_mode == 13)                        /* query mode */
	{	ot10 = atrbt_otype % 10;
		level = atrbt_otype / 10;
		if(level == 0)
		{	if(ot10 == 0)
				sprintf(input_modes[input_mode],"query point");
			else if(ot10 == 1)
				sprintf(input_modes[input_mode],"query line");
			else if(ot10 == 2)
				sprintf(input_modes[input_mode],"query poly");
		}
		else
		{	if(ot10 == 0)
				sprintf(input_modes[input_mode],"L%d query pt",level);
			else if(ot10 == 1)
				sprintf(input_modes[input_mode],"L%d query line",level);
			else if(ot10 == 2)
				sprintf(input_modes[input_mode],"L%d query poly",level);
		}
	}
	else if(input_mode == 30)                        /* erase_obj  mode */
	{	if(atrbt_otype == 0)
			sprintf(input_modes[input_mode],"erase_obj pnt");
		else if(atrbt_otype == 1)
			sprintf(input_modes[input_mode],"erase_obj line");
		else if(atrbt_otype == 2)
			sprintf(input_modes[input_mode],"erase_obj poly");
	}
	else if(input_mode == 33)                        /* set_dig **/
	{	if(atrbt_otype == 0)
			sprintf(input_modes[input_mode],"set_dig pt %d",entry_level);
		else if(atrbt_otype == 1)
			sprintf(input_modes[input_mode],"set_dig sg %d",entry_level);
		else if(atrbt_otype == 2)
			sprintf(input_modes[input_mode],"set_dig pl %d",entry_level);
	}


	for(i = 2;  i < 4;  i++)
	{	swplane(i);
		if(windowxmin)
		{	bar_off(0,47,111,32);
			disptext(2,0,input_modes[input_mode]);
		}
		else
		{	bar_off(528,448,639,463);
			disptext(28,66,input_modes[input_mode]);
		}
	}
		


		/***************** display snap on,off ***********************/
	for(i=1; i<4; i++)
	{	swplane(i);
		if(windowxmin)
			bar_off(0,62,111,48);
		else
			bar_off(528,464,479,639);
	}

	if(snap_input)
	{	for(i = 2;  i < 4;  i++)
		{	swplane(i);
			if(windowxmin)
				disptext(3,0,"** SNAP ON **");
			else
				disptext(29,66,"** SNAP ON **");
		}
	}
	else
	{	swplane(1);
		if(windowxmin)
			disptext(3,0,"  snap off");
		else
			disptext(29,66,"  snap off");
	}


		/***************** call cursor if on tile 0 ******************/
	if(!ginfo.tile_num)
		cursor(ginfo.tcoord[0],ginfo.tcoord[1]);


		/***************** set first_point ***************************/
	first_point = 1;

		/***************** return ************************************/
	return(0);
}
