/*******************************************************************************
*       check_llref                                                            *
*           Checks and sets the globals lat_ref & lon_ref, if not already      *
*       initialized by get_header.  These variables specify a (any) location   *
*       within the utm zone to be used in converting coordinates between       *
*       latitude & longitude (geodetics) and universal transverse mercator     *
*       (utm).                                                                 *
*           If the current lat_ref & lon_ref are both 0.0, the they are set    *
*       to the average lat & lon for the corners of the current map.           *
*       Otherwise they are left alone.  The lon_ref is stored as Easting,      *
*       the lat_ref is stored as Northing.                                     *
*                                                                              *
*       Parameters:   none                                                     *
*                                                                              *
*       Globals modified:   (double) lat_ref, lon_ref.                         *
*                                                                              *
*       Returned:  0 if lon and lat ref present, else returns -1.              *
*                                                                              *
*       J.Dabritz    8/8/89                                                    *
*******************************************************************************/
#include <errno.h>
#include <math.h>
#include "../../incl/globals.h"

long check_llref()
{
	extern int errno;
	long i, hits;
	double strtod();
	double test;

/*** Check for lon and lat ref in the header.  If not ***/
/*** present, and geodetics are not registered, return **/
	errno = 0;
	lat_ref = lon_ref = 0.0;
	lon_ref = strtod(header.utmln, (char **)NULL);
	lat_ref = strtod(header.utmlt, (char **)NULL);
	if( errno == ERANGE )
		return(-1);

/*** Check range of ref values.  Convert into Easting  ***/
	for(; lon_ref < -180.0; lon_ref += 360.0);
	for(; lon_ref > 180.0;  lon_ref -= 360.0);
	if(lon_ref > 0.0)
		lon_ref = -lon_ref;

/*** If no lat/lon reference in header, calculate from Geodetics and average **/
	if( lon_ref == 0.0 )
	{	hits = 0;
		for( i = 0; i < 4; i++ )
		{	if(corner[1][i][0] != (double) 0.0)
			{	lon_ref += corner[1][i][0];
				hits++;
			}
		}
		if(hits > 0)
			lon_ref = lon_ref / hits;
		else
			return(-1);
	}

	if( lat_ref == 0.0 )
	{	hits = 0;
		for( i = 0; i < 4; i++ )
		{	if(corner[1][i][1] != (double) 0.0)
			{	lat_ref += corner[1][i][1];
				hits ++;
			}
		}
		if(hits >  0)
			lat_ref = lat_ref / hits;
	}

/*** put lon_ref in range   -180.0 <= lon_ref < 180.0 ***/
	for(; lon_ref < -180.0; lon_ref += 360.0);
	for(; lon_ref >= 180.0 ; lon_ref -= 360.0);

		/**** check for lon_ref on 6 degree boundry ****/
	test = fmod(lon_ref, (double) 6.0);
	if(test == (double) 0.0)
		lon_ref += .0000000001;

/*** put lat_ref in range  -90.0 <= lat_ref <= 90.0  ***/
	for(; lat_ref < -180.0; lat_ref += 360.0);
	for(; lat_ref > 180.0; lat_ref -= 360.0);
	if(lat_ref < -90.0)
		lat_ref = -180.0 - lat_ref;
	if(lat_ref > 90.0)
		lat_ref = 180.0 - lat_ref;

	return(0);
}
