/*******************************************************************************
*	clipd         (see manual, Internal Functions)            	<in>   *
*	    		clips line segments to a window.                       *
*           This program clips doubles, use clipl & clipl for floats & longs   *
*                                                                              *
*			Globals Modified:  none                                *
*                                                                              *
*			Returns 1 if segment is entirely outside window.       *
*			        0 if part or all of segment is in window       *
*              J.dabritz  8/4/89 (crass adaptation of clipf)                   *
*******************************************************************************/

#include "../../incl/globals.h" 

long dclip_code();


long clipd(x1,y1,x2,y2,left,right,bottom,top)
double *x1, *y1, *x2, *y2;        /* endpoints of segment to be clipped.  */
double left, right, bottom, top;   /* limits of window to clip at.         */
{
	double dx, dy;
	long c1, c2;
/* char string[256];

sprintf(string,"clipf, input: %f, %f, %f, %f, %d, %d, %d, %d",*x1,*y1,*x2,*y2,left,right,bottom,top);
message(string);   */

		/**************** switch window limits if necessary *************/
	if(left > right)
	{	c1 = left;
		left = right;
		right = c1;
	}
	if(bottom > top)
	{	c1 = bottom;
		bottom = top;
		top = c1;
	}
	
		/**************** get initial clip codes ************************/
	c1 = dclip_code(*x1, *y1, left, right, bottom, top);
	c2 = dclip_code(*x2, *y2, left, right, bottom, top);
	
		/**************** do initiail test of endpoints *****************/
	if( !(c1 | c2))          /*  both endpoints in window  */
		return(0);
	if( c1 & c2 )            /*  both endpoints off same side of window   */
		return(1);

		/**************** calculate dx and dy ***************************/
	dx = *x2 - *x1;
	dy = *y2 - *y1;

		/**************** clip point1 to left & right window edges ******/
	if( c1 & 3)              /*  point1 is left or right of window      */
	{	if( c1 & 1 )         /*  point1 is left of window               */
		{	*y1 = *y1 + (left - *x1) * dy / dx;
			*x1 = left;
		}
		else                 /*  point1 is right of window              */
		{	*y1 = *y1 + (right - *x1) * dy / dx;
			*x1 = right;
		}
		c1 = dclip_code(*x1, *y1, left, right, bottom, top);
		if( !(c1 | c2))          /*  both endpoints in window  */
			return(0);
		if( c1 & c2 )            /*  both endpoints off same side of window   */
			return(1);
	} 

		/**************** clip point1 to top and bottom edges ***********/
	if( c1 & 12)             /*  point1 is below or above window        */
	{	if( c1 & 4 )         /*  point1 is below window                 */
		{	*x1 = *x1 + (bottom - *y1) * dx / dy;
			*y1 = bottom;
		}
		else                 /*  point1 is above window                 */
		{	*x1 = *x1 + (top - *y1) * dx / dy;
			*y1 = top;
		}
		c1 = dclip_code(*x1, *y1, left, right, bottom, top);
		if( !(c1 | c2))          /*  both endpoints in window  */
			return(0);
		if( c1 & c2 )            /*  both endpoints off same side of window   */
			return(1);
	} 

		/**************** clip point2 to left and right edges ***********/
	if( c2 & 3)              /*  point2 is left or right of window      */
	{	if( c2 & 1 )         /*  point2 is left of window               */
		{	*y2 = *y1 + (left - *x1) * dy / dx;
			*x2 = left;
		}
		else                 /*  point2 is right of window              */
		{	*y2 = *y1 + (right - *x1) * dy / dx;
			*x2 = right;
		}
		c2 = dclip_code(*x2, *y2, left, right, bottom, top);
		if( !(c1 | c2))          /*  both endpoints in window  */
			return(0);
		if( c1 & c2 )            /*  both endpoints off same side of window   */
			return(1);
	} 

		/**************** clip point2 to top and bottom edges ***********/
	if( c2 & 12)             /*  point2 is below or above window        */
	{	if( c2 & 4 )         /*  point2 is below window                 */
		{	*x2 = *x1 + (bottom - *y1) * dx / dy;
			*y2 = bottom;
		}
		else                 /*  point2 is above window                 */
		{	*x2 = *x1 + (top - *y1) * dx / dy;
			*y2 = top;
		}
		c2 = dclip_code(*x1, *y1, left, right, bottom, top);
	}

	if( !(c1 | c2))          /*  both endpoints in window  */
		return(0);
	return(1);

}

/******************************************************************************/
/*        dclip_code                                                          */
/*                                                                            */
/*        returns a clip code according to where a single point lies in       */
/*        relation to a window:                                               */
/*            bit 0:  set if point is to the left of the window               */
/*            bit 1:  set if point is to the right of the window              */
/*            bit 2:  set if point is below window                            */
/*            bit 3:  set if point is above window                            */
/*        No globals or files changed, the code is returned.                  */
/*        J.Dabritz, 8/4/89                                                   */
/******************************************************************************/

long dclip_code(x, y, left, right, bottom, top)
double x, y;
double left, right, bottom, top;
{
	long c;
/* char string[256];   */

	c = 0;
	if(x < left)
		c |= 1;
	else if(x > right)
		c |= 2;
	if(y < bottom)
		c |= 4;
	else if(y > top)
		c |= 8;

/*  sprintf(string,"dclip_code, input: %f, %f, %d, %d, %d, %d, output: %d",x, y, left, right, bottom, top, c);
message(string);
get_string(string);  */

	return(c);
}
	
