/*******************************************************************************
*       common_obj      (see manual, Internal Functions)                <in>   *
*    Searches for a vector object on a range of levels which contains both     *
*    of 2 verticies as an exact match, or on a line segment.                   *
*                                                                              *
*    Arguments:    double x1,y1,x2,y2;      verticies (pixels)                 *
*                  double *v11,*v12,*v21,*v22;  ptrs to longs (vertex numbers) *
*                  long l1,l2;              range of level numbers (32-95)     *
*                                                                              *
*    Returned Values:  1+ = v_object record number of common object            *
*                      0 = none found                                          *
*                      -1 = error.                                             *
*                                                                              *
*    Global Variables Modified:  none.                                         *
*                                                                              *
*                      j.dabritz  4/8/91                                       *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

#define LIMIT 50

long find_plobjects();

long common_obj(x1,y1,x2,y2,v11,v12,v21,v22,l1,l2)
double x1,y1,x2,y2;
long *v11,*v12,*v21,*v22;
long l1,l2;
{	
	long objects1[LIMIT][3],objects2[LIMIT][3];
	long i,j;
	long o1,o2;


		/*********** check for map **************************************/
	if(!job_stats[0])
		return(-1);

		/*********** check input params ******************************/
	if((l1 > l2) || (l1 < 32) || (l2 > 95))
	{	message("common_obj:  error, bad input parameter.");
		return(-1);
	}

		/*********** find objects at first location *****/
	o1 = find_plobjects(x1,y1,l1,l2,LIMIT,objects1);
	if(o1 < 0)
		return(-1);
	if(o1 == 0)
		return(0);

		/*********** find objects at second location ****/
	o2 = find_plobjects(x2,y2,l1,l2,LIMIT,objects2);
	if(o2 < 0)
		return(-1);
	if(o2 == 0)
		return(0);

		/*********** compare lists **********************/
	for(i=0; i<o1; i++)
	{	for(j=0; j<o2; j++)
		{	if(objects1[i][0] == objects2[j][0])
			{	*v11 = objects1[i][1];
				*v12 = objects1[i][2];
				*v21 = objects2[j][1];
				*v22 = objects2[j][2];
				return(objects1[i][0]);
			}
		}
	}

		/*********** return *****************************/
	return(0);      /**** none found */
}

/*******************************************************************************
*    find_plobjects                                                            *
*      This program find plural objects which have verticies or line segments  *
*    at a given location and are part of a given range of levels.              *
*                                                                              *
*    Parameters:  float x,y;  location (pixels coords)                         *
*                 long l1,l2; range of levels (32-95)                          *
*                 long limit; limit of number of obj. to put on list;          *
*                 long *ptr;  pointer to beginning of list (3 elements/item);  *
*                                                                              *
*    Values returned:  n = number of object on list (0 - limit)                *
*                      -1 = error;                                             *
*    JPD  4/8/91                                                               *
*******************************************************************************/

long find_plobjects(x,y,l1,l2,limit,ptr)
double x,y;
long l1,l2,limit;
long ptr[][3];
{
	INDEX_LIST index_list;
	COORD_LIST coord_list;

	double dx1,dy1,dx2,dy2;
	double dx, dy;
	double dist;
	double mat[2][3];

	long x1,y1;
	long increment;
	long level;
	long v,v1,v2;
	long hits;
	long i;


		/*********** init ret_val ************************************/
	hits = 0;

		/*********** set index_array cell             ****************/
	x1 = (x + 0.5);    /* x min (pixels) */
	x1 = x1 >> 5;      /* x min (index_array coords) */
	y1 = (y + 0.5);    /* y min (pixels) */
	y1 = y1 >> 5;      /* y min (index_array coords) */

		/** test for raster coverage *****************************/
	if((x1 < 0)  || (x1 > (job_stats[1] >> 5)))
		return(-1);
	if((y1 < 0)  || (y1 > (job_stats[2] >> 5)))
		return(-1);

		/** limit to range of index_array ***/
	if(x1 < 0) 
		x1 = 0;
	if(y1 < 0)
		y1 = 0;
	if(x1 > (job_stats[1] >> 5))
		x1 = (job_stats[1] >> 5);
	if(y1 > (job_stats[2] >> 5))
		y1 = (job_stats[2] >> 5);
	
		/*********** check for nodes & endpoints *********************/
		/** for each row & col of index array, search objects */
		/** search all indicated objects **/
	index_list.next = index_array[y1][x1];
	for(; index_list.next;)
	{		/** get index_list record **/
		if(get_indexl(&index_list,index_list.next))
			return(-1);

			/*** check level **/
		level = index_list.type / 10;
		if((level >= l1) && (level <= l2))
		{
			v1 = index_list.v1;
			v2 = index_list.v2;
			if(v1 == v2)
				increment = 0;
			else if((v2 - v1) > 0)
				increment = 1;
			else
				increment = -1;

				/*** check all verticies ***/
			if(increment)
			{	for(v = v1 ; ; v += increment)
				{		/*** read vertex record for vertex         ***/
					if(get_v_coord(&coord_list,v))
						return(-1);
					dx2 = coord_list.x;
					dy2 = coord_list.y;

						/*** check for a match ****/       
					if((dx2 == x) && (dy2 == y))
					{		/*** check obj against list ***/
						for(i=0; i<hits; i++)
						{	if(ptr[i][0] == index_list.obj_rec)
								break;
						}
						if(i == hits)	 /**** add to list ****/
						{	ptr[hits][0] = index_list.obj_rec;
							ptr[hits][1] = v;
							ptr[hits][2] = v;
							hits++;
							if(hits == limit)
								return(hits);
						}
					}
					else if(v != v1  && ((dx2 != dx1) || (dy2 != dy1)))
					{		/*** check for line segment intersecting point ***/
						dx = dx2 - dx1;
						dy = dy2 - dy1;
						dist = sqrt(dx * dx  +  dy * dy);
						dx = dx / dist;
						dy = dy / dist;
						mat[0][0] = dx;
						mat[0][1] = dy;
						mat[0][2] = x - dx1;
						mat[1][0] = dy;
						mat[1][1] = -dx;
						mat[1][2] = y - dy1;
						if(!mat_solve(2,3,mat))
						{		/*** allow a fuzzy tolerance of 0.01 pixels ***/
							if(((mat[0][2] >= 0.0) && (mat[0][2] <= dist)) &&
							   ((mat[1][2] >= -0.01) && (mat[1][2] <= 0.01)))
							{	ptr[hits][0] = index_list.obj_rec;
								ptr[hits][1] = v;
								ptr[hits][2] = v - increment;
								hits++;
								if(hits == limit)
									return(hits);
							}
						}
					}
						/*** save last x,y **/
					dx1 = dx2;
					dy1 = dy2;
						/*** check for end of loop ***/
					if(v == v2)
						break;
				}
			}
		}
	}
	
		/*********** return ******************************************/
	return(hits);
}
