/*******************************************************************************
*     corner_rms.c                                                             *
*        This program accepts 2 corner systems as input and returns the rms    *
*    of a translation-rotation-change of scale best fit transformation between *
*    the two systems.                                                          *
*                                                                              *
*    parameters:  long input_code, output_code:  codes for the coordinate      *
*                              systems.  Input is the 'trial' and output is    *
*                              is the 'true' or 'control' coordinate system.   *
*                              The root-mean-squares (rms) returned is in the  *
*                              units of the output_code system.                *
*                             (these are just 2 different first parameters     *
*                              to the corner[][][]  array)                     *
*                                                                              *
*    Globals modified:   none                                                  *
*                                                                              *
*    Returned values:  double  rms:  >= 0.0  indicates normal completion       *
*                                     < 0.0  indicaties failure.               *
*                                                                              *
*    J.Dabritz,  Aug 4, 1989                                                   *
*******************************************************************************/
   

#include "../../incl/globals.h"
#include <math.h>


double corner_rms(input_code,output_code)     
	long input_code, output_code;  /* codes for coordinate systems to compair */
{
	double params[4];
	double a,b,c,d;
	double x,y;
	double x_out, y_out;
	double dx,dy;
	double sum_squares;
	double rms;              /* root of the mean of the sum of the squares */
	long i;

		/************ check input parameters **************************/
	if((input_code < 0) || (input_code > 29) || (output_code < 0) ||
		(output_code > 29))
		return(-1.0);

		/************ get transformation parameters of a best fit *****/
	if(get_params(params, corner[output_code], corner[input_code], 4))
		return(-1.0);

		/************ transform & sum residual squares ****************/
	sum_squares = 0.0;
	a = params[0];
 	b = params[1];
 	c = params[2];
 	d = params[3];
	for(i=0; i<4; i++)
	{	x = corner[input_code][i][0];
		y = corner[input_code][i][1];
		x_out = a * x  -  b * y  +  c;
		y_out = b * x  +  a * y  +  d;
		dx = x_out - corner[output_code][i][0];
		dy = y_out - corner[output_code][i][1];
		sum_squares  +=  dx * dx  +  dy * dy;
	}

		/************ calcualte square root of the mean of the squares */
	rms = sqrt(sum_squares / 4.0);

		/************ return ******************************************/
	return(rms);
}
