/*******************************************************************************
*     count_subj      (see manual, Command Ref., Internal Functions)  <in>     *
*                                                                              *
*         Counts the number of different attributes in a given set.            *
*         Parameters:                                                          *
*            long set_no;  Number of set for which to count attributes         *
*                                                                              *
*         Returns >=0 upon normal execution, else                              *
*         returns -1 upon error.                                               *
*             J.Dabritz  1/10/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <string.h>

char *calloc_ltp();


long count_subj( set_no)
	long set_no;
{
	OBJECTS object;
	ATTRIBUTES atrbt;
	char *mptr;
	char **attribute;

	long i,j;
	long count;
	long size;


		/*********** check for map *************************/
	if(!job_stats[1])
	{	message("count_subj, map not acquired,  abort.");
		return(-1);
	}

		/***** check for objects *******************************/
	if(!job_stats[10])
	{	message("count_subj, no objects, abort.");
		return(-1);
	}

		/***** check for legal set number **********************/
	if((set_no < 0) || (set_no > 31))
	{	message("count_subj, illegal set number, abort.");
		return(-1);
	}

		/***** malloc space for attribute list *****************/
	size = 36 * job_stats[10];
	mptr = calloc_ltp(size,1);
	if(mptr == (char *) 0)
	{	message("count_subj, cannot allocate needed RAM, abort.");
		return(-1);
	}
	attribute = (char **) mptr;
	for(i=0; i<job_stats[10]; i++)
		attribute[i] = mptr + 4*job_stats[10] + 32*i;

		/***** inform user *************************************/
	message("Checking attributes and counting subjects.");

		/***** check attributes for input set. *****************/
	count = 0;
	for(j=0; j<job_stats[10]; j++)
	{		/***** get object record ***************************/
		if(get_objects(&object,j))
		{	free_ltp(mptr);
			return(-1);
		}
			/***** check for set *******************************/
		if(object.i12[4] & (1 << set_no))
		{		/***** check for attribute ****/
			if(object.i12[2])
			{		/******* get string *********/
				if(get_attr(&atrbt,object.i12[2]))
				{	free_ltp(mptr);
					return(-1);
				}
					/******* check string *******/
				for(i=0; i<count; i++)
				{	if(!strcmp(attribute[i],atrbt.string))
						break;
				}
					/******* add new attribute if not yet found *****/
				if(i == count)
				{	strcpy(attribute[i],atrbt.string);
					count++;
				}
			}
		}
	}

		/*********** inform user ***************************/
	append_msg("   done.");

		/*********** free malloc space *********************/
	free_ltp(mptr);

		/*********** return count **************************/
	return(count);
}
