/*******************************************************************************
*       cursor          (see manual, Internal Functions)                <in>   *

     Purpose:	Displays a cursor - cross-hairs

     Arguments:	long x,y - coordinate pair (screen coordinates )

     Returned Values:  (long) 0

     Global Variables Modified:	 - none -
	
*                      j.dabritz  3/7/89 - 5/8/89                              *
*******************************************************************************/
/* Modified 7.92 by DBS@CWU for X11 support */


#include "../../incl/globals.h"
#include <math.h>

#ifdef LTP_X11
#include "../../incl/x11.h"
#endif

long bar_off();
long bar_on();
long box_off();
long box_on();
long line_off();
long line_on();
long swplane();


long cursor(x,y)
long x,y;
{
	static long oldx, oldy;
	static long lastanchorx, lastanchory;
	static long lastanchor2x, lastanchor2y;
	static long last_mode;
	static long disp_cursor_mode;
	static long coord_count;

	long deg, min;
	double sec;

	char string1[STRLIST], string2[STRLIST];

	double dispx, dispy;
	

	/************* check to see if there are any changes */
	if((oldx==x) && (oldy==y) && (last_mode==disp_cursor_mode))
	{	if(disp_coord_type >= 0)
		{	if(++coord_count == 4)
			{	if(windowxmin)
				{		/************* display coordinates ******************/
						/** calc display coords **/
					dispx = ginfo.tcoord[0];
					dispy = ginfo.tcoord[1];
						/* if on graphics tile, convert coords to disp coords */
					if((disp_coord_type > 0)  &&  (ginfo.tile_num == 0))
					{	transform(&dispx,&dispy,0,disp_coord_type);
						switch(disp_coord_type)  {
						case 1:			/* geodetics */
							dispx = fabs(dispx);
							dms2(dispx,&deg,&min,&sec);
							sprintf(string1,"%3d/%02d/%05.2f",deg,min,sec);
							dispy = fabs(dispy);
							dms2(dispy,&deg,&min,&sec);
							sprintf(string2,"%3d/%02d/%05.2f",deg,min,sec);
							break;
						case 6:			/* map inches  */
							sprintf(string1,"%10.3f",dispx);
							sprintf(string2,"%10.3f",dispy);
							break;
						default:        /* utm, state plane, dig,  user*/
							sprintf(string1,"%10.0f",dispx);
							sprintf(string2,"%10.0f",dispy);
							break;
						}
					}
					else                /* pixels or non-graphics tile coords */
					{	sprintf(string1,"%10.0f",dispx);
						sprintf(string2,"%10.0f",dispy);
					}

						/** print coords on screen **/
					swplane(0);
					bar_off(0,64,111,95);
					disptext(4,2,string1);
					disptext(5,2,string2);
				}
			}
		}
		return (long) 0;
	}
	coord_count = 0;


	/************* switch to the cursor plane (0)    */
	swplane(0);

	/************* Remove old cursor                 */
	switch(last_mode) {
	case 0: 	/*   15 pixel cross    */
		bar_off(oldx-7,oldy,oldx+7,oldy);
		bar_off(oldx,oldy-7,oldx,oldy+7);
		break;
	case 1:		/*   rubberline        */
		line_off(lastanchorx,lastanchory,oldx,oldy);
		break;
	case 2: 	/*   rubberbox         */
		box_off(lastanchorx,lastanchory,oldx,oldy);
		break;
	case 3:     /*   rubber wedge      */
		line_off(lastanchorx,lastanchory,oldx,oldy);
		line_off(lastanchor2x,lastanchor2y,oldx,oldy);
		line_off(lastanchor2x,lastanchor2y,lastanchorx,lastanchory);
	case 4:     /*  15 pixel cross + highlighted line segment */
		bar_off(oldx-7,oldy,oldx+7,oldy);
		bar_off(oldx,oldy-7,oldx,oldy+7);
		line_off(lastanchor2x,lastanchor2y,lastanchorx,lastanchory);
		break;
	case 5:     /* 5 pix solid   + 15 pixel cross (not fixed) */
		bar_off(oldx-7,oldy,oldx+7,oldy);
		bar_off(oldx,oldy-7,oldx,oldy+7);
		bar_off(lastanchorx-2,lastanchory-2,lastanchorx+2,lastanchory+2);
		break;
	case 6:     /* 7 pixel cross + 15 pixel cross  */
		bar_off(lastanchorx-3,lastanchory,lastanchorx+3,lastanchory);
		bar_off(lastanchorx,lastanchory-3,lastanchorx,lastanchory+3);
		bar_off(oldx-7,oldy,oldx+7,oldy);
		bar_off(oldx,oldy-7,oldx,oldy+7);
		break;
	}

	/************* reset last position               */
	oldx = x;
	oldy = y;

	/************* check to see if cursor is within graphics window  */
	if((x<windowxmin) || (x>windowxmax) || (y<windowymin) || (y>windowymax))
	{	disp_cursor_mode = 0;
		if((gr_dev == 1)  ||  (gr_dev == 3))  /* absolute input mode */
		{	last_mode = disp_cursor_mode;     /* no cursor if off graphics */
			return(0);                        /* window */
		}
	}
	else
		disp_cursor_mode = cursor_mode;
	last_mode = disp_cursor_mode;
		

	/************* Place new cursor **********************/
	switch(last_mode) {
	case 0: 	/*   15 pixel cross    */
		bar_on(x-7,y,x+7,y);
		bar_on(x,y-7,x,y+7);
		break;
	case 1:		/*   rubberline        */
		lastanchorx = anchorx;   /*  convert raster to screen coords */
		lastanchory = anchory;
		rstr_to_scrn(&lastanchorx, &lastanchory);
		line_on(lastanchorx, lastanchory, x, y);
		break;
	case 2: 	/*   rubberbox         */
		lastanchorx = anchorx;   /*  convert raster to screen coords */
		lastanchory = anchory;
		rstr_to_scrn(&lastanchorx, &lastanchory);
		box_on(lastanchorx, lastanchory, x, y);
		break;
	case 3:     /*   rubber wedge (triangle) */
		lastanchorx = anchorx;   /*  convert raster to screen coords */
		lastanchory = anchory;
		rstr_to_scrn(&lastanchorx, &lastanchory);
		line_on(lastanchorx, lastanchory, x, y);
		lastanchor2x = anchor2x;   /*  convert raster to screen coords */
		lastanchor2y = anchor2y;
		rstr_to_scrn(&lastanchor2x, &lastanchor2y);
		line_on(lastanchor2x, lastanchor2y, x, y);
		line_on(lastanchor2x, lastanchor2y, lastanchorx, lastanchory);
		break;
	case 4:     /*  15 pixel cross + highlighted line segment */
		bar_on(x-7,y,x+7,y);
		bar_on(x,y-7,x,y+7);
		lastanchor2x = anchor2x;   /*  convert raster to screen coords */
		lastanchor2y = anchor2y;
		rstr_to_scrn(&lastanchor2x, &lastanchor2y);
		lastanchorx = anchorx;   /*  convert raster to screen coords */
		lastanchory = anchory;
		rstr_to_scrn(&lastanchorx, &lastanchory);
		line_on(lastanchor2x, lastanchor2y, lastanchorx, lastanchory);
		break;
	case 5:     /* 7 pixel box (fixed) + 15 pixel cross (not fixed) */
		bar_on(x-7,y,x+7,y);
		bar_on(x,y-7,x,y+7);
		lastanchorx = anchorx;   /*  convert raster to screen coords */
		lastanchory = anchory;
		rstr_to_scrn(&lastanchorx, &lastanchory);
		bar_on(lastanchorx-2,lastanchory-2,lastanchorx+2,lastanchory+2);
		break;
	case 6:
		bar_on(x-7,y,x+7,y);
		bar_on(x,y-7,x,y+7);
		lastanchorx = anchorx;   /*  convert raster to screen coords */
		lastanchory = anchory;
		rstr_to_scrn(&lastanchorx, &lastanchory);
		bar_on(lastanchorx-3,lastanchory,lastanchorx+3,lastanchory);
		bar_on(lastanchorx,lastanchory-3,lastanchorx,lastanchory+3);
		break;
	}

/*#ifdef LTP_X11
	XFlush(xdisplay);
#endif
*/
	return (long) 0;
}
