/*******************************************************************************
*       default_sets    (see manual, Internal Functions)                <in>   *
*                                                                              *
*            This routine updates the sets (object.i12[4]) variable for        *
*       all objects.                                                           *
*                                                                              *
*        parameters:  none                                                     *
*                                                                              *
*       globals/files changed:  (file)  objects                                *
*                                                                              *
*       returns:   (long) 0 = successful,  1 = failure                         *
*                                                                              *
*                      j.dabritz  1/10/90                                      *
*******************************************************************************/

#include "../../incl/globals.h"


long default_sets()
{
	OBJECTS object;
	ATTRIBUTES attributes, atrbt2;

	char string[256];
							/** bit masks **/
	static unsigned long set_mask =     ((1 << 25) -1) & ~1;
	static unsigned long attribute =    (1 << 25);
	static unsigned long no_attribute = (1 << 26);
	static unsigned long attribute2 =   (1 << 27);
	static unsigned long no_attribute2 =(1 << 28);
	static unsigned long diff_atrbts =  (1 << 29);
	static unsigned long bad_segment =  (1 << 30);
	static unsigned long universe =     (1 << 31);
	long flags;
	long i;


		/************** check for map ******************************/
	if(!job_stats[1])
		return(1);

		/************** check all objects **************************/
	for(i=0; i<job_stats[10]; i++)
	{		/************** get object *****************************/
		if(get_objects(&object,i))
		{	message("update_sets aborting, location 1.");
			return(1);
		}

			/******** inform user ***************/
		if(!(i % 250))
		{	sprintf(string,"setting automatic sets, object %d.",i);
			message(string);
		}

			/******* set flags 0, 25-30 according to criteria ******/
		flags = universe;   /** default, all objects in universal set **/

			/** first, set bit 0 if objects is same type as map_type */
		if(object.i12[0] < 8)     /** node **/
		{	if(map_type & 1)
				flags |= 1;
		}
		else if(object.i12[0] < 14)     /** segment **/
		{	if(map_type & 2)
				flags |= 1;
		}
		else                            /** polygon **/
		{	if((map_type & 4) && (object.i12[0] == 14))
				flags |= 1;
		}
	
	
			/********** next set bits 1-24 the same as the objects set flags **/
		flags |= object.i12[4] & set_mask;
	
	
			/** next, set bit flag 25 or 26 according to first attribute **/
		if(object.i12[2])        /** object has attribute **/
		{		/** turn bit 25 on & 26 off **/
			flags |= attribute;
		}
		else
		{		/** turn bit 25 off & 26 on **/
			if(flags & 1)
				flags |= no_attribute;
		}
	
	
			/** set flag 29 'on' as a default **/
		if(flags & 1)
			flags |= diff_atrbts;
	
			/** next, set bit flag 27 or 28 according to second attribute **/
		if(object.i12[2])        /** object has attribute **/
		{	if(get_attr(&attributes,object.i12[2]))
			{	message("update_sets aborting, location 2.");
				return(1);
			}
			else if(attributes.next)
			{		/** turn bit 27 on & 28 off **/
				flags |= attribute2;
					/**** if it has 2 identical attributes turn flag 29 off ***/
				if(get_attr(&atrbt2,attributes.next))
				{	message("update_sets aborting, location 3.");
					return(1);
				}
				if(!strcmp(attributes.string,atrbt2.string))
					flags &= ~diff_atrbts;
			}
			else
			{		/** turn bit 27 off & 28 on **/
				if(flags & 1)
					flags |= no_attribute2;
			}
		}
		else
		{		/** turn bit 27 off & 28 on **/
			if(flags & 1)
				flags |= no_attribute2;
		}
	
			/** now set bit 30 for segments with same poly on both sides */
		if(job_stats[7]  &&  (object.i12[0] >= 8)  &&  (object.i12[0] <= 10)  &&
	   	(object.i12[7] == object.i12[8]))
			flags |= bad_segment;
		
	
			/*********** now, check this (flags) agaist object.i12[4] ***/
		if(flags != object.i12[4])   /* if true, then there have been changes */
		{		/************** store object ***************************/
			object.i12[4] = flags;
			if(put_objects(&object,i))
			{	message("update_sets aborting, location 4.");
				return(1);
			}
		}
	}

		/************** reset the last_map_type variable ***********/
	job_stats[9] = map_type;


		/************** return *************************************/
	return(0);
}
