/*******************************************************************************
*	dig_input      (see manual, Internal Functions)            	<in>   *
*	    		Checks for digitizer input.  If available, the raw     *
*			is cooked into x and y coordinates and button press.   *
*			If a button was pressed, the tm() is called for        *
*			further processing, otherwise, the cursor is displayed *
*			upon bitplane 0 and the reading continues.             *
*			Note that low level I/O is performed using standard    *
*			UNIX/XENIX system calls, not LTPlus calls.  This is    *
*			done so ioctl calls may be used.                       *
*                                                                              *
*			Globals Modified:                                      *
*			        ginfo                                      *
*                                                                              *
*			Returns value greater than zero if there is a string   *
*			in raw_buf to process, otherwise 0.                    *
*******************************************************************************/

#include <string.h>
#include "../../incl/globals.h"

char *trim();


long dig_input()
{	/*  Automatics  */

	static long last_raw_x, last_raw_y; /*  hold raw digitizer input       */
	static double screenx, screeny;   /*  hold raw screen coords as doubles */

	static char string[512];          /*  for input & input manipulation */
	char  string2[512];               /*  for input & input manipulation */
	static char *ptr;                 /*  for reading input from dig     */
	static char *last_ptr;
	char *ptr2;                       /*  for input manipulation         */
	static char last_button;          /*  for button debouncing          */
	static long button_count;         /*  for button debouncing          */

	long ascii_format;                /*  flag indicating input format   */
	long ok;                          /*  checking flag                  */
	long i, j;                        /*  counter                        */
	long length;                      /*  string length                  */
	long dx,dy;                       /*  digitizer movement             */
	unsigned long x, y;               /*  temp storage to calc bin coords*/
	long pass;

	double doublex, doubley;          /*  doubles for function arguments */

		/************* initialize pointer *******/
	if(!ptr)
		last_ptr = ptr = string;
	if( diginfo.format[0] == 'b' )
		ascii_format = 0;
	else 
		ascii_format = 1;

  if( !ascii_format )             /** BINARY INPUT FORMAT **/
  {	/************** read strings until length & markers check out ***/
	string[511] = 0;
	string[0] = *last_ptr;
	for(ok=pass=0; !ok && pass<5; pass++)
	{	length = 0;
		for(; readb(ptr,1,1,fd_ginput); )
		{	if( *ptr & diginfo.sync_mask )        /*  start of input string */
			{	if( ptr == string )               /* first sync byte    */
					ptr++;        
				else                              /* second sync byte hit */
				{	length = ptr - string;
/*					string[0] = *ptr;     */
last_ptr = ptr;
					ptr = string;    /*  reset ptr for next string */
					ptr++;			/* look at next char  */
					break;
				}
			}
			else
			{	if(ptr == string)
					continue;            /* keep reading until sync found */
				else
					ptr++;
			}
		}
		if(length)
		{	ok =  1;   /*  default after the read.  */
				/*   check string length (if specified)  */
			if((diginfo.length)  &&  (length != diginfo.length))
			{	ok = 0;
				ptr = string;        /*  ignore any previous sync byte */
			}
		}
		else
			return(0);
	}
  }
  else
  {		/** ASCII MODE **/
	string[511] = 0;
	for(ok=0, pass=0; !ok && pass<5; pass++)
	{	length = 0;
		for(; readb(ptr,1,1,fd_ginput); )
		{	*ptr &= 127;                        /** trim off parity, if any **/
			if((*ptr == 10)  ||  (*ptr == 13))    /*  end of input string */
			{	if(ptr != string)
				{	*ptr = 0;        /*  append a null    */
					length = ptr - string;
					ptr = string;    /*  reset ptr for next string */
					break;
				}
			}
			else
				ptr++;
		}
		if(length)
		{	ok =  1;   /*  default after the read.  */
				/*   check string length (if specified)  */
			if((diginfo.length)  &&  (strlen(string) != diginfo.length))
				ok = 0;
				/*   check sync markers (if specified)   */
			else 
			{	for(i=0; ok  && i<10; i++)
				{	if(diginfo.sync_char[i])
					{	if(diginfo.sync_char[i] != string[diginfo.sync_col[i]])
							ok=0;
					}
				}
			}
		}
		else
			return(0);
	
	}
  }
	if(pass >= 5)
		return(0);

		/************** set button value ********************************/
  if( !ascii_format )
  {
	ginfo.button = string[diginfo.button] & diginfo.button_mask;
	/**  if(ginfo.button == 32)  
		ginfo.button = 0;  **/
  }
  else
  {
	ginfo.button = string[diginfo.button];
	/**  if(ginfo.button == 32) 
		ginfo.button = 0;  **/
  }

		/************** debounce button *********************************/
	if((ginfo.button == last_button) && (button_count < diginfo.debounce) )
	{	ginfo.button = 0;
		button_count++;
	}
	else
	{	last_button = ginfo.button;
		button_count = 0;
	}


		/************** set x & y raw values ****************************/
  if( !ascii_format )
  {
	if( diginfo.x_begin > diginfo.x_end )
	{	x = y = 0;
		/*** Extract the hi_col bits first ***/
		if( diginfo.xmask[0] )
		{	i = diginfo.hi_x_col;
			x |= (string[i] & diginfo.xmask[0]);
			x >>= diginfo.xmask_bits[0];	/* x is unsigned, 0's filled in */
		}
		if( diginfo.ymask[0] )
		{	i = diginfo.hi_y_col;
			y |= (string[i] & diginfo.ymask[0]);
			y >>= diginfo.ymask_bits[0];	/* y is unsigned, 0's filled in */
		}
		
		for( i = diginfo.x_begin, j = 1; i >= diginfo.x_end; i--, j++ )
		{	x <<= (diginfo.xmask_bits[j]);
			x |= (string[i] & diginfo.xmask[j]);
		}
		for( i = diginfo.y_begin, j = 1; i >= diginfo.y_end; i--, j++ )
		{	y <<= diginfo.ymask_bits[j];
			y |= (string[i] & diginfo.ymask[j]);
		}
	}
	else
	{	x = y = 0;
		/*** Extract the hi_col bits first ***/
		if( diginfo.xmask[0] )
		{	i = diginfo.hi_x_col;
			x |= (string[i] & diginfo.xmask[0]);
			x >>= diginfo.xmask_bits[0];	/* x is unsigned, 0's filled in */
		}
		if( diginfo.ymask[0] )
		{	i = diginfo.hi_y_col;
			y |= (string[i] & diginfo.ymask[0]);
			y >>= diginfo.ymask_bits[0];	/* y is unsigned, 0's filled in */
		}
		
		for( i = diginfo.x_begin, j = 1; i <= diginfo.x_end; i++, j++ )
		{	x <<= diginfo.xmask_bits[j];
			x |= (string[i] & diginfo.xmask[j]);
		}
		for( i = diginfo.y_begin, j = 1; i <= diginfo.y_end; i++, j++ )
		{	y <<= diginfo.ymask_bits[j];
			y |= (string[i] & diginfo.ymask[j]);
		}
	}
	ginfo.rawcoord[0] = x;
	ginfo.rawcoord[1] = y;
  }
  else
  {
	strcpy(string2,string);              /*  copy string  */
	ptr2=string2 + diginfo.x_begin;      /*  set ptr to point of interest */
	string2[diginfo.x_end + 1] = 0;      /*  set null beyond pt of intrst */
	if(!sscanf(ptr2,"%ld",&i))           /*  load into variable      */
		return(0);
	ginfo.rawcoord[0] = i;
	strcpy(string2,string);              /*  copy string  */
	ptr2=string2 + diginfo.y_begin;      /*  set ptr to point of interest */
	string2[diginfo.y_end + 1] = 0;      /*  set null beyong pt of intrst */
	if(!sscanf(ptr2,"%ld",&i))           /*  load into variable      */
		return(0);
	ginfo.rawcoord[1] = i;
  }

		/************** calc x & y screen and raster values *************/
			/******* if stickup menu invoked ******/
	if(*cursor_keys[(long)ginfo.button])
	{	strncpy(string2,trim(cursor_keys[(long)ginfo.button]),7);
		string2[7] = 0;
		length = strlen(string2);
	}
	else
	{	*string2 = 0;          /* null */
		length = 0;
	}

	if((length > 2)  &&  !strncmp("stickup",string2,length))
	{		/*  convert to stickup menu coords (if any)  */
		switch(gr_dev)  {
		case DIG1ABS:
		case DIG1REL:
			if(stickup1reg[2][0] != 0)
			{		/*  load stickup reg (digitizer) corners into corner array*/
				for(i=0; i<4; i++)
				{	corner[7][i][0] = stickup1reg[i][0];
					corner[7][i][1] = stickup1reg[i][1];
				}
					/*  load stick reg (stickup) corners into corner array */
				corner[8][0][0] = 1.0;
				corner[8][0][1] = stickup1_rows + 1.0;
				corner[8][1][0] = 1.0;
				corner[8][1][1] = 1.0;
				corner[8][2][0] = stickup1_cols + 1.0;
				corner[8][2][1] = 1.0;
				corner[8][3][0] = stickup1_cols + 1.0;
				corner[8][3][1] = stickup1_rows + 1.0;
					/*  load raw coords into doubles for transform     */
				doublex = ginfo.rawcoord[0];
				doubley = ginfo.rawcoord[1];
					/*  transform raw dig coords into stickup menu coords */
				transform(&doublex,&doubley,7,8);
				ginfo.tcoord[0] = doublex + 0.5;
				ginfo.tcoord[1] = doubley + 0.5;
					/*  check tile coords    */
				if((ginfo.tcoord[0] < 1))
					return(0);
				if((ginfo.tcoord[0] >= stickup1_cols))
					return(0);
				if((ginfo.tcoord[1] < 1))
					return(0);
				if((ginfo.tcoord[1] >= stickup1_rows))
					return(0);
					/*  calc stickup string for fj()  */
				sprintf(raw_buf,"stickup %d %d",ginfo.tcoord[1],ginfo.tcoord[0]);
					/*  set tile number               */
				ginfo.tile_num = 11;
					/*  return                        */
				return(1);
			}
		case DIG2ABS:
		case DIG2REL:
			if(stickup2reg[2][0] != 0)
			{		/*  load stickup reg (digitizer) corners into corner array*/
				for(i=0; i<4; i++)
				{	corner[7][i][0] = stickup2reg[i][0];
					corner[7][i][1] = stickup2reg[i][1];
				}
					/*  load stick reg (stickup) corners into corner array */
				corner[8][0][0] = 1.0;
				corner[8][0][1] = stickup2_rows + 1.0;
				corner[8][1][0] = 1.0;
				corner[8][1][1] = 1.0;
				corner[8][2][0] = stickup2_cols + 1.0;
				corner[8][2][1] = 1.0;
				corner[8][3][0] = stickup2_cols + 1.0;
				corner[8][3][1] = stickup2_rows + 1.0;
					/*  load tile coord variables with raw dig coords  */
				doublex = ginfo.rawcoord[0];
				doubley = ginfo.rawcoord[1];
					/*  transform raw dig coords into stickup menu coords */
				transform(&doublex,&doubley,7,8);
				ginfo.tcoord[0] = doublex + 0.5;
				ginfo.tcoord[1] = doubley + 0.5;
					/*  check tile coords    */
				if((ginfo.tcoord[0] < 1))
					return(0);
				if((ginfo.tcoord[0] >= stickup2_cols))
					return(0);
				if((ginfo.tcoord[1] < 1))
					return(0);
				if((ginfo.tcoord[1] >= stickup2_rows))
					return(0);
					/*  calc stickup string for fj()  */
				sprintf(raw_buf,"stickup %d %d",ginfo.tcoord[1],ginfo.tcoord[0]);
					/*  set tile number               */
				ginfo.tile_num = 11;
					/*  return                        */
				return(1);
			}
		default:
			return(0);
		}
	}


		/*********** stickup menu not invoked ***********************/
		/*********** find tile tile number & tile coords ************/
	switch(gr_dev) {
	case DIG1ABS:
	case DIG2ABS:      /*  absolute map input, tile 0  */
			/*  calc tile (raster) coords  */
		doublex = ginfo.rawcoord[0];
		doubley = ginfo.rawcoord[1];
		if(gr_dev == DIG1ABS)
			transform(&doublex,&doubley,4,0);
		else
			transform(&doublex,&doubley,5,0);
		ginfo.tcoord[0] = doublex + 0.5;
		ginfo.tcoord[1] = doubley + 0.5;

			/*  calc screen coords         */
		ginfo.scoord[0] = ginfo.tcoord[0];
		ginfo.scoord[1] = ginfo.tcoord[1];
		rstr_to_scrn(&ginfo.scoord[0], &ginfo.scoord[1]);
			/*  set tile number (0)        */
		ginfo.tile_num = 0;
		break;
	case DIG1REL:
	case DIG2REL:
			/*   calc screen coords */
			/*   calc raw movement  */
		dx = ginfo.rawcoord[0] - last_raw_x ;
		last_raw_x = ginfo.rawcoord[0];
			/*   reset float screenx if a new view has happened  */
		screenx = (ginfo.scoord[0] == (long) screenx) ? screenx:ginfo.scoord[0];
			/*   calc new float screen position  */
		screenx = screenx + 100.0 * dx / diginfo.units_per_inch;
		screenx = (0.0 > screenx) ?  0.0 : screenx;
		screenx = (639.0 < screenx) ?  639.0 : screenx;
			/*   set global integer    */
		ginfo.scoord[0] = screenx;

			/*   calc raw movement  */
		dy = ginfo.rawcoord[1] - last_raw_y;
		last_raw_y = ginfo.rawcoord[1];
			/*   reset float screenx if a new view has happened  */
		screeny = (ginfo.scoord[1] == (long) screeny) ? screeny:ginfo.scoord[1];
			/*   calc new float screen position  */
		screeny = screeny - 100.0 * dy / diginfo.units_per_inch;
		screeny = (0.0 > screeny) ?  0.0 : screeny;
		screeny = (415.0 < screeny) ?  415.0 : screeny;
			/*   set global integer    */
		ginfo.scoord[1] = screeny;
		
			/************** find which tile we are on ***********************/
		if((ginfo.scoord[0] >= windowxmin)  &&  (ginfo.scoord[0] <= windowxmax)
		&& (ginfo.scoord[1] >= windowymin)  &&  (ginfo.scoord[1] <= windowymax))
			ginfo.tile_num = 0;   /*  on the graphics tile  */
		else if((ginfo.scoord[0] >= windowxmin)  &&  
			 	(ginfo.scoord[0] < tcoords[8].x[1])  &&
	   		 	(ginfo.scoord[1] >= tcoords[8].y[0])  &&  
			 	(ginfo.scoord[1] < tcoords[8].y[1]) )
			ginfo.tile_num = 8;
		else
		{	for(i=0; i<12; i++)
			{	if((ginfo.scoord[0] >= tcoords[i].x[0])  &&  
			 	(ginfo.scoord[0] < tcoords[i].x[1])  &&
	   		 	(ginfo.scoord[1] >= tcoords[i].y[0])  &&  
			 	(ginfo.scoord[1] < tcoords[i].y[1]) )
					break;
			}
			ginfo.tile_num = i;    /*  note:  12 is default tile */
		}
	
			/************** calculate tile coords ***************************/
		switch( ginfo.tile_num )   {
		case 0:         /*   raster graphics tile      */
			ginfo.tcoord[0] = ginfo.scoord[0];
			ginfo.tcoord[1] = ginfo.scoord[1];
			scrn_to_rstr(&ginfo.tcoord[0], &ginfo.tcoord[1]);
			break;
		case 3:         /*   permanent functions tile  */
		case 5:         /*   menu names tile           */
		case 6:         /*   functions tile            */
				/*   calc row number from the top   */
			i = ginfo.tile_num;
			ginfo.tcoord[1] = (ginfo.scoord[1] - tcoords[i].y[0] ) / 16.0;
			ginfo.tcoord[0] = ginfo.tcoord[1];
			break;
		case 8:         /*   horizontal position tile  */
			if(ginfo.scoord[0] == horz_marker)
			{	ginfo.tcoord[0] = 0;
				ginfo.tcoord[1] = 0;
			}
			else if(ginfo.scoord[0] > horz_marker)
			{	ginfo.tcoord[0] = 1;
				ginfo.tcoord[1] = 1;
			}
			else
			{	ginfo.tcoord[0] = -1;
				ginfo.tcoord[1] = -1;
			}
			break;
		case 9:         /*   vertical position tile    */
			if(ginfo.scoord[1] == vert_marker)
			{	ginfo.tcoord[0] = 0;
				ginfo.tcoord[1] = 0;
			}
			else if(ginfo.scoord[1] < vert_marker)
			{	ginfo.tcoord[0] = 1;
				ginfo.tcoord[1] = 1;
			}
			else
			{	ginfo.tcoord[0] = -1;
				ginfo.tcoord[1] = -1;
			}
			break;
		case 10:        /*   compress/uncompress graphics tile  */
			ginfo.tcoord[0] = 1;
			ginfo.tcoord[1] = 1;
			break;
		default:        /*   any other tile, don't care.  */
			ginfo.tcoord[0] = 0;
			ginfo.tcoord[1] = 0;
			break;
		}
		break;
	default:
		return(0);
	}

		/***************** display cursor *****************************/
	cursor(ginfo.scoord[0], ginfo.scoord[1]);

		/***************** if button press, set string ****************/
	*raw_buf = 0;                     /*  set string to null  */
	if(*cursor_keys[ginfo.button])
	{	switch(ginfo.tile_num)   {
		case 3:         /*   permanent functions tile  */
			strncpy(raw_buf,perm_item[ginfo.tcoord[1]],95);
			raw_buf[95] = 0;
			trim(raw_buf);
			break;
		case 5:         /*   menu names tile           */
			strncpy(raw_buf,menu_name[ginfo.tcoord[1]],95);
			raw_buf[95] = 0;
			trim(raw_buf);
			break;
		case 6:         /*   current menu functions tile   */
			strncpy(raw_buf,menu_item[current_menu][ginfo.tcoord[1]],95);
			raw_buf[95] = 0;
			trim(raw_buf);
			break;
		case 8:         /*   horizontal position tile  */
			if(ginfo.tcoord[0] < 0.0)
				strcpy(raw_buf,"step left");
			else if(ginfo.tcoord[0] > 0.0)
				strcpy(raw_buf,"step right");
			else
				raw_buf[0] = 0;
			break;
		case 9:         /*   vertical position tile    */
			if(ginfo.tcoord[1] < 0.0)
				strcpy(raw_buf,"step down");
			else if(ginfo.tcoord[1] > 0.0)
				strcpy(raw_buf,"step up");
			else
				raw_buf[0] = 0;
			break;
		case 10:        /*   compress/uncompress graphics tile  */
			if(windowxmin)      /*  graphics window is currently 'small' */
				strcpy(raw_buf,"graphics +f");  /*  change to large */
			else                /*  graphics window is currently 'large' */
				strcpy(raw_buf,"graphics -f");  /*  change to small */
			break;
		default:        /*   process button through ckeys array */
			strncpy(raw_buf,cursor_keys[(unsigned char) ginfo.button],95);
			raw_buf[95] = 0;
			trim(raw_buf);
			break;
		}
	}
	else
		ginfo.button = 0;
	
		/****************** return flag dependent on string input **/
	if(raw_buf[0])
		return(1);
	else
		return(0);

}
