/*******************************************************************************
*   dir_list     (see manual, Internal Functions)            	  <in>         *
*                                                                              *
*     Reads a specified directory and loads the names of the active inodes     *
*     into a passed array.  The second parameter is the base address of a      *
*     two dimensional array that is [lcount][STRLIST] in size.                 *
*     For XENIX systems, DOS directories may be searched too.                  *
*                                                                              *
*     Returns the number of items on the list if successful,                   *
*     else returns -1 in case of error.                                        *
*******************************************************************************/
/* Modified 5/92 by DBS@CWU to work with ULTRIX and SUNOS */

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <sys/dir.h>
#include <string.h>
#include "../../incl/in.h"  



#if (SCO_UNIX || XENIX)    /***** use this code for SCO systems *****/

long dir_list( dir_path, list, lcount )
	char *dir_path;
	char *list;
	long lcount;
{
/*  Automatics   */
	extern int errno;
	unsigned int indx = 0;
	int str_comp();                 /*  Compare function declaration  */ 
	long no_items;			/*  returned, number of items on list */
	long i;
	int fpd;
	int fd_tmp;
	char cmd[256];
	char msg[256];
	char t_name[L_tmpnam];
	struct direct st_dir;           /*  Directory structure  */

#ifndef SV32
/*** Check for a colon in the name.  If present, search that DOS directory ***/
/*** If no colon, simply process a UNIX/XENIX directory                    ***/

	if( strchr(dir_path, ':') != (char *)NULL )
	{	if( tmpnam(t_name) == (char *)NULL )
		{	message("Cannot generate a temp file name");
			return(-1);
		}
			
		if( creatf(t_name) == -1)
		{	sprintf(msg,"Cannot create %s", t_name);
			message(msg);
			return(-1);
		}

		sprintf(cmd, "dosls %s > %s", dir_path, t_name);
		if( system_ltp(cmd) == -1 )
		{	sprintf(msg,"Cannot perform--> %s     UNIX errno is %d", cmd, errno);
			message(msg);
			return(-1);
		}
		
	/*** Open the temp file ***/
		if( (fd_tmp = openf(t_name, READ)) == -1 )
		{	sprintf(msg, "Cannot open %s.  UNIX errno is %d\n", t_name, errno);
			message(msg);
			return(-1);
		}

	/*** Read the temp file and copy the names into the list ***/
		for(i = 0; i < lcount; i++)
		{	if( reada(list, (STRLIST - 1), fd_tmp) == (char *)NULL )
				break;
			list[strlen(list) - 1] = (char)NULL;
			list += STRLIST;
		}

	/*** All set.  Return to caller ***/
		closef(fd_tmp);
		if( i )
			return(i);
		else 
			return(0);
	}

#endif

/***  Open the directory  ***/
	if((fpd = openf( dir_path, READ ))  == -1)
	{	message("can't open path:   ");
		append_msg(dir_path);
		return(-1);
	}

/***  Load names from dir file if inode is not null  ***/
	while( readb( (char *)&st_dir, sizeof(st_dir), 1, fpd ) && (indx < lcount) )
	{	if( st_dir.d_ino && st_dir.d_name[0] != '.' )
		{
			/** if( tmp_ptr = strrchr( st_dir.d_name, '.' ) )
				*tmp_ptr = (char)NULL; ***/
			strncpy((list + (indx++ * STRLIST)), st_dir.d_name,
						sizeof(st_dir.d_name) );
				/*** truncate at 14 characters */
			*(list + (indx * STRLIST) + 14) = (char) 0;
		}
	}



/***  Check for an empty directory  ***/
	if( !indx )
	{	closef(fpd);
		return(0);
	}
		
/***  Sort the array into lexicographical order  ***/
	if( indx > 1 )
		qsort( list, indx, STRLIST, str_comp );

/***  Close directory  ***/
	closef( fpd );


/***  Return to caller  ***/
	no_items = indx;
	return( no_items );

}


#else     /***** use this code for normal UNIX systems *****/


long dir_list( dir_path, list, lcount )
	char *dir_path;
	char *list;
	long lcount;
{
  /*  Automatics   */
  extern int errno;
  unsigned int indx = 0;
  int str_comp();		/*  Compare function declaration  */ 
  long no_items;		/*  returned, number of items on list */
  long i;
  DIR * fpd;
  int fd_tmp;
  char cmd[256];
  char msg[256];
  char t_name[L_tmpnam];
  struct direct *st_dir;	/*  Directory structure  */

  /***  Test for an empty path name and then open the directory  ***/
  if(dir_path[0] == '\0') {
    message("Empty directory path, cannot open.");
    return(0);
  }
  if ((fpd = opendir(dir_path))  == NULL) {
    message("can't open path:   ");
    append_msg(dir_path);
    return(-1);
  }

  /***  Load names from dir file if inode is not null  ***/
  while( ((st_dir = readdir(fpd)) != NULL) && (indx < lcount) ) {
    if( st_dir->d_ino && st_dir->d_name[0] != '.' ) {
      strncpy((list + (indx++ * STRLIST)), st_dir->d_name,
	      sizeof(st_dir->d_name));
      /*** truncate at 14 characters */
      *(list + (indx * STRLIST) + 14) = (char) 0;
    }
  }

  /***  Check for an empty directory  ***/
  if( !indx ) {
    closedir(fpd);
    return(0);
  }
		
  /***  Sort the array into lexicographical order  ***/
  if( indx > 1 )
    qsort( list, indx, STRLIST, str_comp );

  /***  Close directory  ***/
  closedir( fpd );
  
  /***  Return to caller  ***/
  no_items = indx;
  return( no_items );

}

#endif



/******************************************************************
*    str_comp                                                     *
*       Compares characters. Returns an int < 0 if a < b,         *
*       or an int == 0 if a == b, or an int > 0 if a > b.         *
******************************************************************/

int str_comp( a, b )
	char a[],
	     b[];
{
	/** return( a < b? -1: (a == b? 0: 1) ); **/
	return( strcmp(a, b) > 0 ? 1 : (strcmp(a, b) ? -1 : 0) );
}
