/*******************************************************************************
*       dispgraphics    (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Displays the current map's raster the resolution specified.   *
*       Uses the input parameters to display, and sets the view structure      *
*       in graphics.h (globals.h)                                              *
*                                                                              *
*    Arguments:    long center_x,    center of view, x coordinate.             *
*                  long center_y,    center of view, y coordinate.             *
*                  long res_code,    code for display resolution:              *
*                                             0 = map                          *
*                                             1 = 1:4                          *
*                                             2 = 1:1  (full res)              *
*                                             3 = 8:1  (zoom)                  *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified: Updates:  resolution_code                      *
*                                         last_res_code                        *
*                                         view (structure)                     *
*                                                                              *
*                      j.dabritz  6/9/89                                       *
*******************************************************************************/
/* Modified 7/92 by DBS@CWU for X11 and big endian support*/

#include "../../incl/globals.h"
#include <string.h>

#ifdef LTP_X11
#include "../../incl/x11.h"
char *malloc_ltp();
#endif

#ifdef BIG_ENDIAN
#include "../../incl/big_end.h"
#endif

#ifdef LTP_X11
/*
**  Code to re-order bits in bytes in bitmaps if necessary 
**
**  Default action is to query the server for bit ordering, but this
**   doesnt always work.  So user can overide default by setting
**   env variable 'BITSWAP'  to "TRUE" or "FALSE", which in turn
**
**  causes the BitSwap variable to be set to one of three options below.
**
**    -dpg
*/
bitmap_order_bits (xdisplay, to_ptr, size)
    Display *xdisplay;
    char *to_ptr;
    int size;
{
    switch (BitSwap) {
	case BITSWAP_DEFAULT:
	    if (BitmapBitOrder(xdisplay) == LSBFirst)
	       bit_swap(to_ptr,size);
	    break;
	case BITSWAP_TRUE:
	       bit_swap(to_ptr,size);
	    break;
	case BITSWAP_FALSE:
	    break;
    }
}
#endif


long dispgraphics(center_x, center_y, res_code)
	long center_x, center_y;
	long res_code;
{
	unsigned char *from_ptr;
	char *to_ptr;
	char *ptr;
	char string[96];

	long plane;     /*  vga plane number  */
	long window_height;  /*  screen height     */
	long window_width;   /*  screen width      */
	long rwidth, rheight;  /*   raster window height/width */
	long last_rwidth, last_rheight; /* last viewed raster window height/width */
	long max_rbytes, max_sbytes;
	long byte_width, rows;
	long d1, d2;
	long from_incr;
	long xdone, ydone;
	long word, bit;
	long sx, sy;
	long x, y;
	long x1,x2,y1,y2;
	long i;
	long mask, target_bit, target_mask;
#ifdef LTP_X11
	long full_bwidth, extrafirst, extralast;
#endif

		/******************* check for graphics_flag ***************/
	if(!graphics_flag)
		return(0);

		/******************* check for batch_mode ******************/
	if(batch_mode)
		return(0);


		/******************* check for loaded map ******************/
	if(!job_stats[0])
	{	message("there is no map currently loaded.  grahics display not possible");
		return(0);
	}


		/******************* remove old graphics display ***********/
	for(plane=0; plane<4; plane++)
	{	swplane(plane);
		bar_off(windowxmin,windowymin,windowxmax,windowymax);
		logo_on = 0;
	}


		/******************* if map is less than 400 x 400, change map to full*/
	if(!res_code)
	{	if((job_stats[1] < 400) && (job_stats[2] < 400))
			res_code = 2;
	}

		/******************* calculate new lowerleft x,y ***********/
		/* calc ht & wd of screen window   */
	window_width = windowxmax - windowxmin + 1;
	window_height = windowymax - windowymin + 1;

		/* calc ht & wd of new raster window   */
	rwidth = window_width/view[res_code].zoom_factor;
	rheight = window_height/view[res_code].zoom_factor;

		/* calc ht & wd of last viewed raster window   */
	last_rwidth = window_width/view[resolution_code].zoom_factor;
	last_rheight = window_height/view[resolution_code].zoom_factor;

		/* set raster window minimums      */
	switch(res_code) {
	case 0:        /*  1:?   entire map  ( ? !< 1 ) */
		rwindowxmin = 0;
		rwindowymin = 0;
		break;
	case 1:        /*  1:4   block resolution      */
	case 2:        /*  1:1   full resolution     */
	case 3:        /*  8:1   zoom resolution     */
		/** if zooming out, use same window as befor 
			(if new center on window) **/ 
		if((res_code < resolution_code) &&
		 (center_x >= view[res_code].lower_leftx) &&
		  (center_x <= (view[res_code].lower_leftx + rwidth-1)) &&
		   (center_y >= view[res_code].lower_lefty) &&
		    (center_y <= (view[res_code].lower_lefty + rheight-1)) )
		{	rwindowxmin = view[res_code].lower_leftx;
			rwindowymin = view[res_code].lower_lefty;
			break;
		}

		/** if zooming in (but not to zoom resoluton) (if new center on window) 
			go to exact window as last time at this res. **/ 
		if((res_code != 3)  &&  (res_code > resolution_code) &&
		 (center_x >= view[res_code].lower_leftx) &&
		  (center_x <= (view[res_code].lower_leftx + rwidth-1)) &&
		   (center_y >= view[res_code].lower_lefty) &&
		    (center_y <= (view[res_code].lower_lefty + rheight-1)) )
		{	rwindowxmin = view[res_code].lower_leftx;
			rwindowymin = view[res_code].lower_lefty;
			break;
		}

			/***** if zooming in to res_code 3, keep new window within 
				last window if new center is on window **/
		if((res_code > resolution_code) &&
		 (center_x >= view[resolution_code].lower_leftx) &&
		  (center_x <= (view[resolution_code].lower_leftx + last_rwidth-1)) &&
		   (center_y >= view[resolution_code].lower_lefty) &&
		    (center_y <= (view[resolution_code].lower_lefty + last_rheight-1)) )
		{		/*** keep new window within current window ***/
			/* calc initial x minimum of raster window   */
			rwindowxmin = center_x - (window_width/2/view[res_code].zoom_factor)+1;
			if(rwindowxmin < view[resolution_code].lower_leftx)
				rwindowxmin = view[resolution_code].lower_leftx;
			if(rwindowxmin + rwidth  > view[resolution_code].lower_leftx + last_rwidth)
				rwindowxmin = view[resolution_code].lower_leftx + last_rwidth - rwidth;
			rwindowxmin = (0 < rwindowxmin)  ?  rwindowxmin : 0;
			if(res_code != 3)    /*  if not zoom, round to nearest byte  */
				rwindowxmin = ((rwindowxmin + 4) >> 3) << 3;

				/* calc initial y minimum of raster window   */
			rwindowymin = center_y -(window_height/2/view[res_code].zoom_factor)+1;
			if(rwindowymin < view[resolution_code].lower_lefty)
				rwindowymin = view[resolution_code].lower_lefty;
			if(rwindowymin + rheight  > view[resolution_code].lower_lefty + last_rheight)
				rwindowymin = view[resolution_code].lower_lefty + last_rheight - rheight;
			rwindowymin = (0 < rwindowymin)  ?  rwindowymin : 0;
			if(res_code != 3)    /*  if not zoom, round to nearest byte  */
				rwindowymin = ((rwindowymin + 4) >> 3) << 3;
			break;
		}
			/********** otherwise, just keep it on the raster ***/
			/* calc initial x minimum of raster window   */
		rwindowxmin = center_x - (window_width/2/view[res_code].zoom_factor)+1;
		rwindowxmin = (0 < rwindowxmin)  ?  rwindowxmin : 0;
		if(res_code != 3)    /*  if not zoom, round to nearest byte  */
			rwindowxmin = ((rwindowxmin + 4) >> 3) << 3;
			/* calc initial y minimum of raster window   */
		rwindowymin = center_y - (window_height/2/view[res_code].zoom_factor)+1;
		rwindowymin = (0 < rwindowymin)  ?  rwindowymin : 0;
		if(res_code != 3)    /*  if not zoom, round to nearest byte  */
			rwindowymin = ((rwindowymin + 4) >> 3) << 3;
		break;
	}
		/* calc initial x maximum of raster window (may be off of raster)  */
	rwindowxmax = rwindowxmin + rwidth - 1;
		/* calc initial y maximum of raster window (may be off of raster)  */
	rwindowymax = rwindowymin + rheight - 1;

		/* put window entirely on raster if possible  */
		/* set new max at current max or raster max (whichever is less) */
	rwindowxmax = (rwindowxmax > job_stats[1]) ?  job_stats[1] : rwindowxmax;
	rwindowymax = (rwindowymax > job_stats[2]) ?  job_stats[2] : rwindowymax;
		/* set final min keyed off of new max  */
	rwindowxmin = rwindowxmax - rwidth + 1;
	rwindowxmin = (0 > rwindowxmin) ?  0 : rwindowxmin;
	rwindowymin = rwindowymax - rheight + 1;
	rwindowymin = (0 > rwindowymin) ?  0 : rwindowymin;
	if(res_code != 3)    /*  if not zoom, round x to nearest byte  */
	{	rwindowxmin = ((rwindowxmin + 4) >> 3) << 3;
/*		rwindowymin = ((rwindowymin + 4) >> 3) << 3;  */
	}
		/* set final max keyed off of final min  */
	rwindowxmax = rwindowxmin + rwidth -1;
	rwindowymax = rwindowymin + rheight - 1;

		/******************* set resolution codes & view struct ****/
	if(res_code != resolution_code)    /*  if we have changed resolutions */
	{	last_res_code = resolution_code;  /*  update res & last res code */
		resolution_code = res_code;
	}
	view[resolution_code].lower_leftx = rwindowxmin;  /* set view struct  */
	view[resolution_code].lower_lefty = rwindowymin;
	

		/******************* display new horiz & vert position bars */
	/*    erase current bars  */
	swplane(3);

		/*    erase horizontal bar */
	x1 = windowxmin;
	y1 = tcoords[8].y[0] + 1.0;
	x2 = tcoords[8].x[1] - 0.5;
	y2 = tcoords[8].y[1] - 1.5;
	bar_off(x1,y1,x2,y2);      

		/*    draw horizontal bar */
	x1 = windowxmin + (rwindowxmin / (1.0 + job_stats[1])) * window_width;
	x1 = (x1 < windowxmin) ?  windowxmin : x1;
	x2 = windowxmin + (rwindowxmax / (1.0 + job_stats[1])) * window_width;
	x2 = (x2 > windowxmax) ?  windowxmax : x2;
	bar_on(x1,y1,x2,y2);      
	horz_marker = (x1 + x2) / 2;    /* center of bar position  */

		/*    erase vertical bar  */
	x1 = tcoords[9].x[0] + 1.0;
	y1 = tcoords[9].y[0] ;
	x2 = tcoords[9].x[1] - 1.5;
	y2 = tcoords[9].y[1] - 0.5;
	bar_off(x1,y1,x2,y2);     

		/*    draw vertical bar    */
	y1 = windowymax - (rwindowymin / (1.0 + job_stats[2])) * window_height;
	y1 = (y1 < windowymin) ?  windowymin : y1;
	y2 = windowymax - (rwindowymax / (1.0 + job_stats[2])) * window_height;
	y2 = (y2 > windowymax) ?  windowymax : y2;
	bar_on(x1,y1,x2,y2);      
	vert_marker = (y1 + y2) / 2;    /* center of bar position  */

	
		/******************* display raster ************************/
	if(disp_raster)
	{	swplane(1);    /*  use bitplane 1  */
		switch(res_code)   {
		case 0:        /*  entire map on 400 x 400   */
#ifndef LTP_X11      /***** first the VGA code *****/
			to_ptr = vga_plane + BITPLN_WIDTH*windowymax + windowxmin/8;
			bitmap(50,400,50,-BITPLN_WIDTH,mini_raster[0],to_ptr);
#else                /***** now the X11 code *****/
			/* get begin pointer to temp space  */
			if ((to_ptr = malloc_ltp(50*400+1)) == NULL) {
			    printf("Unable to allocate space needed to display the requested graphic.");
			    break;
			}
			bitmap(50,400,50,-50,mini_raster,to_ptr+(50*400));

			bitmap_order_bits (xdisplay, to_ptr, 50*400);

			xtemppix = XCreateBitmapFromData(xdisplay,xmainwin,to_ptr,400,400);
			XCopyPlane(xdisplay,xtemppix,xmainwin,xgccopy,0,0,400,400,windowxmin,windowymin,0x01);
			XFreePixmap(xdisplay, xtemppix);
#endif               /***** end of X11 code *****/
			break;

		case 1:        /*  1:4                       */
#ifndef LTP_X11   /***** first the VGA code *****/
			xdone = (rwindowxmax < job_stats[1]) ? rwindowxmax : job_stats[1];
			ydone = minl(2,rwindowymax,job_stats[2]);         /*  last row  */
			for(x = rwindowxmin; x<= xdone ; x += 4)
			{		/*  find word & mask for checking   */
				word = x >> 5;
				bit = (7 & ~x) | (x & 24);
				mask = 15 << (bit - 3);
				target_bit = 7 & ~((x-rwindowxmin) >> 2);
				target_mask = 1 << target_bit;
				ptr = vga_plane + BITPLN_WIDTH*windowymax + (windowxmin >> 3);
				ptr += (x - rwindowxmin) >> 5;
				for(y = rwindowymin; y<=ydone; y++)
				{	if(lraster[y][word] & mask)
					{	*ptr |= target_mask; 
						y = y + 4 -((y-rwindowymin) & 3) -1;
					}
					if(((y-rwindowymin) & 3) == 3)
						ptr = ptr - BITPLN_WIDTH;
				}
			}
#else             /***** now the X11 code *****/
			xdone = (rwindowxmax < job_stats[1]) ? rwindowxmax : job_stats[1];
			ydone = minl(2,rwindowymax,job_stats[2]); /* last row */
			rows = windowymax - windowymin + 1;
			byte_width = (windowxmax + 1  - windowxmin)/8;
			/* get begin pointer to temp space  */
			if ((to_ptr = malloc_ltp(byte_width*rows+1)) == NULL) {
			    printf("Unable to allocate space needed to display the requested graphic.");
			    break;
			}
			for(ptr=to_ptr; ptr<=to_ptr+byte_width*rows; ptr++)
			    *ptr = 0;
			for(x = rwindowxmin; x<= xdone ; x += 4)
			{		/*  find word & mask for checking   */
				word = x >> 5;
				bit = (7 & ~x) | (x & 24);
				mask = 15 << (bit - 3);
				target_bit = 7 & ~((x-rwindowxmin) >> 2);
				target_mask = 1 << target_bit;
				ptr = to_ptr + byte_width*(rows-1);
				ptr += (x - rwindowxmin) >> 5;
				for(y = rwindowymin; y<=ydone; y++)
				{	if(lraster[y][word] & mask)
					{   *ptr |= target_mask; 
					    y = y + 4 -((y-rwindowymin) & 3) -1;
					}
					if(((y-rwindowymin) & 3) == 3)
						ptr = ptr - byte_width;
				}
			}

			bitmap_order_bits (xdisplay, to_ptr, byte_width*rows);	

			xtemppix = XCreateBitmapFromData(xdisplay,xmainwin,
						    to_ptr,byte_width*8,rows);
			XCopyPlane(xdisplay,xtemppix,xmainwin,xgccopy,0,0,
				byte_width*8,rows,windowxmin,windowymin,0x01);
			XFreePixmap(xdisplay, xtemppix);
			free_ltp(to_ptr);
			
#endif            /***** end of X11 code *****/
			break;

		case 2:        /*  1:1 (full resolution)     */
				/* calculate begin bitmap pointer to raster  */
			from_ptr = craster[rwindowymin] + (rwindowxmin >> 3);
				/* calc width to rt edge of raster (bytes)  */
			max_rbytes = (job_stats[1] + 1 - rwindowxmin) >> 3;
				/* calc width to rt edge of screen (bytes)  */
			max_sbytes = (windowxmax + 1 - windowxmin) >> 3;
				/* choose lesser of widths   */
			byte_width = minl(2,max_rbytes,max_sbytes);
				/* calc distance to top of raster   */
			d1 = job_stats[2] + 1 - rwindowymin;
				/* calc distanct to top of screen   */
			d2 = windowymax + 1 - windowymin;
				/* choose lesser of distances for number of rows to bitmap */
			rows = minl(2,d1,d2);
				/* calc increment on raster (bytes) */
			from_incr = (job_stats[1] + 1) >> 3;
#ifndef LTP_X11      /***** first the VGA code *****/
			/* calculate begin bitmap pointer to screen  */
			to_ptr = vga_plane + BITPLN_WIDTH*windowymax + windowxmin/8;
			/* bitmap to screen  */
			bitmap(byte_width,rows,from_incr,-BITPLN_WIDTH,
				from_ptr,to_ptr);
#else                /***** now the X11 code *****/
			/* adjust the from pointer and width to get whole longs */
			if (byte_width == ((job_stats[1] + 1) >> 3)) {
			    extrafirst = extralast = 0;
			    full_bwidth = byte_width;
			}
			else {
			    extrafirst = (rwindowxmin/8 % 4);
			    extralast = (3 - (rwindowxmax/8) % 4);
			    full_bwidth = byte_width + extrafirst + extralast;
			}
			/* get begin pointer to temp space  */
			if ((to_ptr = malloc_ltp(full_bwidth*rows+1)) == NULL) {
			    printf("Unable to allocate space needed to display the requested graphic.");
			    break;
			}

			/* bitmap to tmp space  */
			bitmap(full_bwidth,rows,from_incr,-full_bwidth,
				from_ptr-extrafirst, to_ptr+(full_bwidth*rows));
#ifdef BIG_ENDIAN
			byte_swap(to_ptr, full_bwidth*rows, SWAP_LONG);
#endif

			bitmap_order_bits (xdisplay, to_ptr, full_bwidth*rows);

			xtemppix = XCreateBitmapFromData(xdisplay,xmainwin,
						      to_ptr,full_bwidth*8,rows);
			XCopyPlane(xdisplay,xtemppix,xmainwin,xgccopy,extrafirst*8,0,
				  byte_width*8,rows,windowxmin,windowymin,0x01);
			XFreePixmap(xdisplay, xtemppix);
			free_ltp(to_ptr);
#endif               /***** end of X11 code *****/
			break;

		case 3:        /*  8:1 (zoomed resolution)   */
				/* check entire raster range of x/y's & plot fat pixels */
			xdone = (rwindowxmax > job_stats[1]) ?  job_stats[1] : rwindowxmax;
			ydone = (rwindowymax > job_stats[2]) ?  job_stats[2] : rwindowymax;
			for(x = rwindowxmin; x <= xdone ; x++)
			{	word = x >> 5;            /*   word number on raster row */
				bit = (7 & ~x) | (x & 24);/*   bit number on word        */
				for(y = rwindowymin; y <= ydone; y++)
				{	if(lraster[y][word] & (1 << bit))
					{	sx = x;
						sy = y;
						if(!rstr_to_scrn(&sx,&sy))
						{	sx &= (~7);
							sy &= (~7);
							bar_on(sx,sy,sx+7,sy+7);
						}
					}
				}
			}
			break;
		}
		
	}


		/******************* blank out area outside raster *********/
	if(rwindowxmax > job_stats[1])  /*  screen displays x area beyond raster */
	{	x = job_stats[1];
		y = 0;
		rstr_to_scrn(&x,&y);
		if(x < windowxmax)
		{	swplane(2);
			bar_on(x+1,windowymin,windowxmax,windowymax);
		}
	}
	
	if(rwindowymax > job_stats[2])  /*  screen displays y area beyond raster */
	{	x = 0;
		y = job_stats[2];
		rstr_to_scrn(&x,&y);
		if(y > windowymin)
		{	swplane(2);
			bar_on(windowxmin,y-1,windowxmax,windowymin);
		}
	}


		/******************* display cursor ************************/
	if(!ginfo.tile_num)
	{	x = ginfo.tcoord[0];
		y = ginfo.tcoord[1];
		rstr_to_scrn(&x,&y);
		ginfo.scoord[0] = x;
		ginfo.scoord[1] = y;
	}
	else
	{	x = ginfo.scoord[0];
		y = ginfo.scoord[1];
	}
	cursor(x+1,y+1);
	cursor(x-1,y-1);
	cursor(x,y);
	cursor(x,y);
	cursor(x,y);
	cursor(x,y);
	cursor(x,y);
	
		/******************* display vectors as needed *************/
	if(disp_hglight  ||  disp_normal  ||
	   disp_i1hglight  ||  disp_i1normal  ||
	   disp_i2hglight  ||  disp_i2normal )
	{	if(dispvectors())
			message("vector display aborted.");
	}

		/******************* highlight resolution displayed ********/
	if(windowxmin)
	{	switch(res_code) {
		case 0:
			sprintf(string,"view map");
			break;
		case 1:
			sprintf(string,"view block");
			break;
		case 2:
			sprintf(string,"view full");
			break;
		case 3:
			sprintf(string,"view zoom");
			break;
		default:
			sprintf(string,"xxxxxxxxxxxxxxxxxxxxx");
			break;
		}
		swplane(3);
		bar_off(200,0,215,95);      /** erase last highlight  ***/
		for(i=0; i<6; i++)          /** find view label (if any) & highlight **/
		{	if(!strcmp(perm_item[i],string))
			{	y1 = i * 16 + 6;
				y2 = y1 + 2;
				disptext(i,25,"<");
				bar_on(203,y1,214,y2);
			}
		}
	}	

		/******************* return ********************************/
    return(0);
}
