/*******************************************************************************
*       display_obj     (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Display or remove 1 map object (segment or node) from the     *
*       current display area using the input action and the display variables  *
*       disp_hglight and disp_normal (see graphics.h)                          *
*                                                                              *
*    Arguments:  long action:   0 = remove from display                        *
*                               1 = display according to disp_hglight/normal   *
*                long object_no;  record number of object to be displayed.     *
*                                 (must be node or segment);                   *
*                                                                              *
*    Returned Values: (long) 0 = display done.                                 *
*                            1 = display aborted.                              *
*                                                                              *
*    Global Variables Modified: none                                           *
*                                                                              *
*                      j.dabritz  5/10/90                                      *
*******************************************************************************/

#include "../../incl/globals.h"

long display_obj(action,object_no)
long action,object_no;
{
   COORD_LIST crdlst;
   INDEX_LIST index_list;
   OBJECTS object;

	char string[STRLIST];

   long cx1,cx2,cy1,cy2;     /** index_array cell limits of display */
   long bitpln;              /** bit_plane number */
   long checkmask;           /** full 28 bit bit-mask **/
   long row, col;            /** row & column of index_array */
   long x1,y1,x2,y2;         /** plot coordinates **/
   long v,v1,v2;             /** record numbers of verticies **/
   long display_type;
   long screen_ht;
	long seg;
	long disp_obj;
	long i;
	long act;

   float fx,fy;               /** fractional raster coordinates **/

      /******************* check action & object parameters ******/
	if(object_no < 0)
		object_no = -object_no;

   if(object_no >= job_stats[10])
   {  sprintf(string,"disply_obj:  illegal object number: %d,  no display done.",object_no);
      message(string);
      return(1);
   }

      /** get object record *********/
   if(get_objects(&object,object_no))
      return(1);

      /** check for polygon type object **/
   if(object.i12[0] > 10)	  /** polygon, plot all segments & islands **/
	{		/** first plot all segments of this polygon **/
		for(i=object.i12[5]; i<=object.i12[6]; i++)
		{	if(get_seglist(&seg,i))
				return(1);
			display_obj(action,seg);
		}
		
			/** next, plot centroids **/
		if((object.i12[0] == 14) && disp_centroid)   /* interior polygons only */
		{	act = action;   /** hold action for later use **/
			if(act)
   			disp_obj = disp_object(object_no);

      		/******************* for bit-planes 2-3 ********************/
         	/**** 2 = normal,  2&3 combined = highlight ***/
   		for(bitpln = 2;  bitpln < 4;  bitpln++)
   		{      /******************* switch bit planes *********************/
	
      		swplane(bitpln);
      		sleep(0);
				if(act)
      		{	if(bitpln == 2)
         			checkmask = 3;
      			else
      			{  if(disp_hglight)
            			checkmask = 2;
         			else
           				act = 0;
      			}
		
         			/** check for display of normal/highlight **/
      			if(!(disp_obj & checkmask))
         			act = 0;
				}

						/** plot primary centroid **/
				fx = object.f4[0];
				fy = object.f4[1];
				if(!rstr_to_scr2(fx,fy,&x1,&y1))
				{	if(act)				/** draw 3 x 3 blot at poly centroid */
					{	line_on(x1+1,y1+1,x1+1,y1-1);
						line_on(x1,y1+1,x1,y1-1);
						line_on(x1-1,y1+1,x1-1,y1-1);
					}
					else
					{	line_off(x1+1,y1+1,x1+1,y1-1);
						line_off(x1,y1+1,x1,y1-1);
						line_off(x1-1,y1+1,x1-1,y1-1);
					}
				}
		
					/**** now do secondary centroids *******/
				if(object.i12[3])
				{	v1 = object.i12[3];
					v2 = object.i12[11];
					for(v = v1; v <= v2 ;v++)
					{	if(get_coordlst(&crdlst,v))
							return(1);
						fx = crdlst.x;
						fy = crdlst.y;
						rstr_to_scr2(fx,fy,&x1,&y1);
						if(act)               /* draw 3x3 blot at poly centroid */
						{	line_on(x1+1,y1+1,x1+1,y1-1);
							line_on(x1,y1+1,x1,y1-1);
							line_on(x1-1,y1+1,x1-1,y1-1);
						}
						else
						{	line_off(x1+1,y1+1,x1+1,y1-1);
							line_off(x1,y1+1,x1,y1-1);
							line_off(x1-1,y1+1,x1-1,y1-1);
						}
					}
				}
			}
		}
		
			/** next, plot all island polygons **/
		if(object.i12[0] != 15)	
		{	for(; object.i12[8]; )
			{	if(display_obj(action,object.i12[8]))   /* display */
					return(1);
				if(get_objects(&object,object.i12[8]))  /* read obj for next ptr */
					return(1);
			}
		}

		return(0);
	}




      /******************* find best method of display ***********/
      /** check for node **/
   if(object.i12[0] < 8)
      display_type = 0;

      /** check length vs screen length **/
   else if((job_stats[1] + job_stats[2]) < 2000)  /* if raster < 2000 pix **/
      display_type = 0;          /** write out entire length of obj **/

   else
   {   screen_ht = rwindowymax - rwindowymin;
		screen_ht = (screen_ht > 0)?    screen_ht + 1  :  1 - screen_ht;
		
      if((screen_ht * 2) >= job_stats[2])  /* if scrn_ht > 1/2 raster ht */
         display_type = 0;          /** write out entire length of obj **/
      else
      {  if(object.f4[2] < 500.0)       /* if object < 500  pixels */
            display_type = 0;          /** write out entire length of obj **/
   
               /* if object lenght < twice screen ht. */
         else if((double)(object.f4[2] * 2.0) < (double) screen_ht)
            display_type = 0;          /** write out entire length of obj **/
   
         else
            display_type = 1;     /** check screen index to display object **/
      }
   }

      /******************* find cell limits of current display ***/
   cx1 = rwindowxmin >> 5;
   cx2 = (rwindowxmax <= job_stats[1]) ?  rwindowxmax : job_stats[1];
   cx2 = cx2 >> 5;
   cy1 = rwindowymin >> 5;
   cy2 = (rwindowymax <= job_stats[2]) ?  rwindowymax : job_stats[2];
   cy2 = cy2 >> 5;

	if(action)
   	disp_obj = disp_object(object_no);

      /******************* for bit-planes 2-3 ********************/
         /**** 2 = normal,  2&3 combined = highlight ***/
   for(bitpln = 2;  bitpln < 4;  bitpln++)
   {      /******************* switch bit planes *********************/

      swplane(bitpln);
      sleep(0);
		if(action)
      {	if(bitpln == 2)
         	checkmask = 3;
      	else
      	{  if(disp_hglight)
            	checkmask = 2;
         	else
           		action = 0;
      	}

         	/** check for display of normal/highlight **/
      	if(!(disp_obj & checkmask))
         	action = 0;
		}

      
         /******************* display according to display_type *****/
      switch(display_type)  {
         /******************* case 0, display entire obj length *****/
         /*******************  & let screen clipping take care of ***/
         /*******************  the display **************************/
      case 0:
         if(object.i12[0] < 8)    /** node **/
         {   /** plot point **/
            fx = object.f4[0];
            fy = object.f4[1];
            if(!rstr_to_scr2(fx,fy,&x1,&y1))
            {   if(!action)
               {   line_off(x1,y1-2,x1,y1+2);  /* for pts.*/
                  line_off(x1-2,y1,x1+2,y1);
               }
               else
               {   line_on(x1,y1-2,x1,y1+2);  /* for pts.*/
                  line_on(x1-2,y1,x1+2,y1);
               }
            }
         }
         else                      /** segment **/
         {      /*** plot list of verticies   ***/
            v1 = object.i12[5];
            v2 = object.i12[6];
            if(get_coordlst(&crdlst,v1))
               return(1);
            fx = crdlst.x;
            fy = crdlst.y;
            rstr_to_scr2(fx,fy,&x2,&y2);
            for(v = v1; v <= v2 ;v++)
            {   if(get_coordlst(&crdlst,v))
                  return(1);
               fx = crdlst.x;
               fy = crdlst.y;
               rstr_to_scr2(fx,fy,&x1,&y1);
               if(!action)
                  line_off(x1,y1,x2,y2);
               else
                  line_on(x1,y1,x2,y2);
               x2 = x1;
               y2 = y1;
            }
         }
         
         break;

         /******************* case 1, check index_array for screen **/
         /*******************  area & draw only parts of object *****/
         /*******************  which cross screen *******************/
      case 1:
            /******************* display vectors of range of cells *****/
         for(row = cy1; row <= cy2; row++)
         {   for(col = cx1; col <= cx2; col++)
            {      
                  /*********** for each cell check all index_list recs  */
               index_list.next = index_array[row][col];
               for(; index_list.next; )
               {      
                     /*** get next indext_list record ****/
                  if(get_indexl(&index_list,index_list.next))
                     return(1);

                     /*********** check index_list object for plotting **/
                  if((index_list.type < 3) && 
                     (index_list.obj_rec == object_no))
                  {   if(index_list.type == 0)    /** node **/
                     {      /** plot point **/
                        fx = object.f4[0];
                        fy = object.f4[1];
                        if(!rstr_to_scr2(fx,fy,&x1,&y1))
                        {   if(!action)
                           {  line_off(x1,y1-2,x1,y1+2);  /* for pts.*/
                              line_off(x1-2,y1,x1+2,y1);
                           }
                           else
                           {  line_on(x1,y1-2,x1,y1+2);  /* for pts.*/
                              line_on(x1-2,y1,x1+2,y1);
                           }
                        }
                     }
                     else if(index_list.type == 1)   /* segment */
                     {      /*** plot list of verticies   ***/
                        v1 = index_list.v1;
                        v2 = index_list.v2;
                        if(get_coordlst(&crdlst,v1))
                           return(1);
                        fx = crdlst.x;
                        fy = crdlst.y;
                        rstr_to_scr2(fx,fy,&x2,&y2);
                        for(v = v1; v <= v2 ;v++)
                        {   if(get_coordlst(&crdlst,v))
                              return(1);
                           fx = crdlst.x;
                           fy = crdlst.y;
                           rstr_to_scr2(fx,fy,&x1,&y1);
                           if(!action)
                              line_off(x1,y1,x2,y2);
                           else
                              line_on(x1,y1,x2,y2);
                           x2 = x1;
                           y2 = y1;
                        }
                     }   
                  }
               }
            }
         }
         break;
      }
   }
   
      /******************* return ********************************/
   return(0);
}
