/*******************************************************************************
*       disptext        (see manual, Internal Functions)                <in>   *

     Purpose:	Puts text on screen at row and col.  Only that portion of the
		text which falls on the screen will be displayed.  Also will
		remove text that is already displayed (it actually reverses the 
		bit pattern over the letter pixels.)

     Arguments:	int row - starting row
		int col - starting column
		char *string - pointer to string to be displayed

     Returned Values: (long) 0

     Global Variables Modified: - none -

*                      j.dabritz  3/7/89 - 5/8/89                              *
*******************************************************************************/
/* Modified 5/92 by DBS@CWU for X11 support */

#include "../../incl/globals.h"

#ifndef LTP_X11         /***** First the VGA code *****/

#include "malloc.h"
#include <sys/ioctl.h>
#include <memory.h>


long disptext(row,col,string) 
long row,col;
unsigned char *string;
{
	char *pn, *p, *s;
	long i,j,c;
	long length;


		/*   check to see if grahpics are being used.  */
	if(batch_mode)
		return 0;


		/*  check row and col & reject those which aren't on scrn */
	if((row <0) || (row >29) || (col >79))
		return (long) 0;


	if(batch_mode || !graphics_flag)
		printf("\ndisptext:  %d  %d  %s\n",row,col,string);
	else
	{	/*  calculate p0 & pn pointers for 1 pixel row on screen. */
		length = strlen(string);
			/*   p is pointer to screen location at start of text     */
		p = vga_plane + (row * BITPLN_WIDTH * 16) + col;
			/*   for each column to be printed      */
		for(i=0, c=col; (i<length) && (c<80) ; c++ , i++, p++)
		{		/*     print only if column is on screen    */
			if((c >=0) && (c < BITPLN_WIDTH))
			{		/*    s is pointer to binary pattern for text*/
				s = (char *)textchar + ((string[i]) << 4);
					/*    pn is pointer moved through bit pattern */
				pn = p;
					/*    copy each line of bit pattern to screen */
				for(j=0; j<16; j++, pn += BITPLN_WIDTH, s++)
					*pn = *pn ^ *s;
			}
		}
	}
	
	return (long) 0;
}

#else        /***** now the X11 code *****/

#include "../../incl/x11.h"

long disptext(row,col,string) 
     long row,col;
     unsigned char *string;
{
  int length, x, y;

  /*   check to see if grahpics are being used.  */
  if(batch_mode)
    return 0;

  /*  check row and col & reject those which aren't on scrn */
  if((row <0) || (row >29) || (row<0) || (col >79))
    return (long) 0;


  if(batch_mode || !graphics_flag)
    printf("\ndisptext:  %d  %d  %s\n",row,col,string);
  else {
    length = strlen(string);
    x = col * 8;
    y = row * 16 + 10;
    XDrawString(xdisplay, xmainwin, xgcxor, x, y, string, length);
  }
  return (long) 0;

}

#endif     /***** end of X11 code *****/

