/*******************************************************************************
*       dispvectors     (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Displays the current map's vectors according to the globals   *
*       disp_hglight and disp_normal (see graphics.h)                          *
*                                                                              *
*    Arguments:    none                                                        *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified: none                                           *
*                                                                              *
*                      j.dabritz  8/13/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"

long dispvectors()
{
	COORD_LIST crdlst;
	INDEX_LIST index_list;
	OBJECTS object;

	long cx1,cx2,cy1,cy2;     /** index_array cell limits of display */
	long bitpln;              /** bit_plane number */
	long checkmask;           /** full 28 bit bit-mask **/
	long row, col;            /** row & column of index_array */
	long x1,y1,x2,y2;         /** plot coordinates **/
	long v,v1,v2;             /** record numbers of verticies **/
	long n, dsp;

	float fx,fy;               /** fractional raster coordinates **/


		/******************* find cell limits of current display ***/
	cx1 = rwindowxmin >> 5;
	cx2 = (rwindowxmax <= job_stats[1]) ?  rwindowxmax : job_stats[1];
	cx2 = cx2 >> 5;
	cy1 = rwindowymin >> 5;
	cy2 = (rwindowymax <= job_stats[2]) ?  rwindowymax : job_stats[2];
	cy2 = cy2 >> 5;

		/******************* for bit-planes 2-3 ********************/
			/**** 2 = normal,  2&3 combined = highlight ***/
	for(bitpln = 2;  bitpln < 4;  bitpln++)
	{		/******************* switch bit planes *********************/

		swplane(bitpln);
		sleep(0);
		if(bitpln == 2)
			checkmask = 3;
		else
		{	if(disp_hglight || disp_i1hglight || disp_i2hglight)
				checkmask = 2;
			else
				break;
		}

			/******************* display vectors of range of cells *****/
		for(row = cy1; row <= cy2; row++)
		{	if(abort_cmd())
				break;
			swplane(bitpln);
			for(col = cx1; col <= cx2; col++)
			{		
					/*********** for each cell check all index_list recs  */
				index_list.next = index_array[row][col];
				for(; index_list.next; )
				{		
						/*** get next indext_list record ****/
					if(get_indexl(&index_list,index_list.next))
						return(1);

						/*********** check index_list object for plotting **/
					if(index_list.type < 3)
					{	if(disp_object(index_list.obj_rec)  & checkmask)
						{	if(index_list.type == 0)    /** node **/
							{		/** get object record **/
								if(get_objects(&object,index_list.obj_rec))
									return(1);
									/** plot point **/
								fx = object.f4[0];
								fy = object.f4[1];
/*								fx = index_list.v1; 										*/
/*								fy = index_list.v2;                             */
								if(!rstr_to_scr2(fx,fy,&x1,&y1))
								{	line_on(x1,y1-2,x1,y1+2);  /* for pts.*/
									line_on(x1-2,y1,x1+2,y1);
								}
							}
							else if(index_list.type == 1)   /* segment */
							{		/*** plot list of verticies   ***/
								if(get_coordlst(&crdlst,index_list.v1))
									return(1);
								fx = crdlst.x;
								fy = crdlst.y;
								rstr_to_scr2(fx,fy,&x2,&y2);
								v1 = index_list.v1 + 1;
								v2 = index_list.v2;
								for(v = v1; v <= v2 ;v++)
								{	if(get_coordlst(&crdlst,v))
										return(1);
									fx = crdlst.x;
									fy = crdlst.y;
									rstr_to_scr2(fx,fy,&x1,&y1);
									line_on(x1,y1,x2,y2);
									x2 = x1;
									y2 = y1;
								}
	
							}	
							else if(index_list.type == 2)   /* polygon */
							{		/*** plot centroid blots      ***/
								if(disp_centroid)
								{	fx = index_list.v1;
									fy = index_list.v2;
									rstr_to_scr2(fx,fy,&x1,&y1); /* plot centroid blot */
									{	line_on(x1+1,y1+1,x1+1,y1-1);
										line_on(x1,y1+1,x1,y1-1);
										line_on(x1-1,y1+1,x1-1,y1-1);
									}
								}
							}	
						}
					}
					else if(index_list.type > 319)  /** v_object **/
					{	dsp = 0;
						if(index_list.type < 640)
						{	n = (index_list.type / 10) - 32;
							if(disp_i1hglight & (1 << n))
								dsp = 3;
							else if(disp_i1normal & (1 << n))
								dsp = 1;
						}
						else if(index_list.type < 960)
						{	n = (index_list.type / 10) - 64;
							if(disp_i2hglight & (1 << n))
								dsp = 3;
							else if(disp_i2normal & (1 << n))
								dsp = 1;
						}
						if(dsp & checkmask)
						{		/*** plot list of verticies   ***/
							if(index_list.v1 <= index_list.v2)
							{	v1 = index_list.v1;
								v2 = index_list.v2;
							}
							else
							{	v1 = index_list.v2;
								v2 = index_list.v1;
							}

							if(get_v_coord(&crdlst,v1))
								return(1);

							fx = crdlst.x;
							fy = crdlst.y;
							rstr_to_scr2(fx,fy,&x1,&y1);

							if((index_list.type % 10) == 0)      /*  pt */
							{	line_on(x1,y1-2,x1,y1+2);
								line_on(x1-2,y1,x1+2,y1);
							}
							else                        /* seg or poly */
							{	
								for(v=v1+1; v<=v2; v++)
								{	if(get_v_coord(&crdlst,v))
										return(1);
									fx = crdlst.x;
									fy = crdlst.y;
									rstr_to_scr2(fx,fy,&x2,&y2);
									line_on(x1,y1,x2,y2);
									x1 = x2;
									y1 = y2;
								}
							}
						}
					}
				}
			}
		}
	}

		/******************* return ********************************/
	return(0);
}


