/*******************************************************************************
*       erase_mapobj    (see manual, Internal Functions)                <in>   *
*    Erases the raster in a fatline under one vector object.                   *
*                                                                              *
*    Arguments:    long obj_no;   map object record number                     *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified:  none.                                         *
*                                                                              *
*                      j.dabritz  7/15/90                                      *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long erase_sobj();



long erase_mapobj(obj_no)
long obj_no;
{	
	OBJECTS object,poly;

	char string[STRLIST];

	long x1,y1;
	long next;
	long listrec;
	long s;
	long save_mode;


		/********* check for map ****************************************/
	if(!job_stats[0])
	{	message("no current map.");
		return(1);
	}


	if(obj_no < 0)
		message(" Illegal object number.  No action taken.");


		/********* get object record ************************************/
	if(get_objects(&object,obj_no))
		return(1);

		/********* erase a fatline under object *************************/
	if(object.i12[0] < 8)    /* node         */
	{	save_mode = input_mode;
		input_mode = 11;         /* erase rectangle*/
		x1 = (long) (object.f4[0] + 0.5);
		y1 = (long) (object.f4[1] + 0.5);
		start_ltp(x1 - 1, y1 - 1);
		continue_ltp(x1 + 1, y1 + 1);
		input_mode = save_mode;
	}
	else if(object.i12[0] < 14)  /* segment  */
		erase_sobj(obj_no);
	else                     /* polygon      */
	{		/*** read all polygons & island recs ****/
		next = obj_no;
		for(next=obj_no; next; next = poly.i12[8])
		{		/**** read polygon record ***/
			if(get_objects(&poly,next))
				break;
			for(listrec=poly.i12[5]; listrec<=poly.i12[6]; listrec++)
			{	if(get_seglist(&s,listrec))
					break;
				s = (s < 0) ? -s: s;
				erase_sobj(s);
			}
		}
	}

		/********* return ***********************************************/
	return(0);
}


/*******************************************************************************
*    erase_sobj                                                                *
*       erase one segment object                                               *
*******************************************************************************/


long erase_sobj(obj_no)
long obj_no;
{
	OBJECTS object;
	COORD_LIST coordlist;

	long save_mode;
	long v;
	long ix,iy;

	float x1,y1;
	float dx,dy;
	double len;
			
		/******* get segment object record ********************/
	if(get_objects(&object,obj_no))
		return(1);

		/******* check object type ****************************/
	if((object.i12[0] < 8) || (object.i12[0] > 10))
		return(1);

		/**** erase between all verticies ****/
	if(get_coordlst(&coordlist,object.i12[5]))
		return(1);
	x1 = coordlist.x;
	y1 = coordlist.y;

		/******* change input mode ****************************/
	save_mode = input_mode;
	input_mode = 10;           /* erase_fatline  */

	for(v=object.i12[5] + 1; v<=object.i12[6]; v++)
	{	if(get_coordlst(&coordlist,v))
			break;
		if(v==object.i12[5]+1)        /* first vertex pair */
		{	dx = coordlist.x - x1;
			dy = coordlist.y - y1;
			len = sqrt((double) (dx*dx + dy*dy));
			if(len > 0.0)
			{	x1 += (dx / len);
				y1 += (dy / len);
			}
			ix = x1 + 0.5;
			iy = y1 + 0.5;
			start_ltp(ix,iy);
		}

		if(v==object.i12[6])     /* last vertex pair */
		{	dx = x1 - coordlist.x;
			dy = y1 - coordlist.y;
			len = sqrt((double) (dx*dx + dy*dy));
			if(len > 0.0)
			{	coordlist.x += (dx / len);
				coordlist.y += (dy / len);
			}
		}
		
		ix = coordlist.x + 0.5;
		iy = coordlist.y + 0.5;
		
		continue_ltp(ix,iy);

		x1 = coordlist.x;
		y1 = coordlist.y;
	}

		/********* reset input mode ********************************/
	input_mode = save_mode;

		/********* return ******************************************/
	return(0);
}
