/*******************************************************************************
*       erase_rect      (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Turn all pixels 'off' which are in the input raster window.   *
*    Update the mini-raster, and the current display.                          *
*                                                                              *
*    Arguments:    long x1,y1,x2,yx:   raster coordinates of window.           *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified: Updates:  lraster (craster), mini-raster;      *
*                                                                              *
*                      j.dabritz  6/28/89                                      *
*******************************************************************************/
/* Modified 7/92 by DBS@CWU for big endian support */

#include "../../incl/globals.h"
#ifdef BIG_ENDIAN
#include "../../incl/big_end.h"
#endif

long erase_rect(x1, y1, x2, y2)
	long x1, y1, x2, y2;
{	
	long xmin,xmax,ymin,ymax;     /* limits of raster rectangle */
	long xmmin,xmmax,ymmin,ymmax; /* limits of mini_raster rectangle */
	long word0, word1;            /* limits of lraster words involved */
	long byte0, byte1;            /* limits of craster bytes involved */
	long bit0, bit1;              /* bit numbers on byte0 & byte1     */
	long byte_width, word_width;  /* width of raster in bytes and words */
	long b, w;                    /* byte and word, within loops        */
	long divisor;                 /* for conversion of coords to mini-raster */
	unsigned long *wptr, *end_wptr; /* incremental word ptr and stop ptr  */

	char byte_mask0, byte_mask1;  /* mask for turning off pixels within a byte*/
	unsigned char *ptr, *end_ptr; /* incremental byte ptr and stop ptr   */


		/***************** check for map present *********************/
	if(!job_stats[0])
		return(0);


		/***************** check input paramters *********************/
	if(((x1<0) && (x2<0)) || ((x1>job_stats[1]) && (x2>job_stats[1])) ||
	   ((y1<0) && (y2<0)) || ((y1>job_stats[2]) && (y2>job_stats[2])))
		return(0);


		/***************** erase window on raster ********************/
#ifdef BIG_ENDIAN  /***** swap the bytes before and after opeartion *****/
	/* This is a HACK.  It works, but is very inefficient.  I will find
	   a better solution if time permits.  DBS@CWU */
	byte_swap(rasterm.rec0, raster_bincrement * (job_stats[2]+1), SWAP_LONG);
#endif
		/*  find min and max x & y */
	if(x1 < x2)
	{	xmin = x1;
		xmax = x2;
	}
	else
	{	xmin = x2;
		xmax = x1;
	}
	xmin = (0 > xmin) ?  0 : xmin;          /*   place within raster   */
	xmax = (job_stats[1] < xmax) ?  job_stats[1] : xmax;

	if(y1 < y2)
	{	ymin = y1;
		ymax = y2;
	}
	else
	{	ymin = y2;
		ymax = y1;
	}
	xmin = (0 > xmin) ?  0 : xmin;          /*   place within raster   */
	xmax = (job_stats[1] < xmax) ?  job_stats[1] : xmax;

		/*  find first (0) and last (1) word, byte, & bit   */
	word0 = xmin >> 5;
	byte0 = xmin >> 3;
	bit0 = (7 & ~xmin);
	byte_mask0 = (-1) << (bit0+1);     /* mask for first byte in each row */
	word1 = xmax >> 5;
	byte1 = xmax >> 3;
	bit1 = (7 & ~xmax);
	byte_mask1 = (255) >> (7-bit1+1);     /* mask for last byte in each row  */
	byte_width = (job_stats[1] + 1) >> 3; /* width of raster row in bytes    */
	end_ptr = craster[ymax] + byte1 + 1; /* ptr just beyond last byte of rect */
		/*  check if first word = last word   & turn off pixels  */
	if(word0 == word1)
	{	if(byte0 == byte1)
		{	byte_mask0 |= byte_mask1;
			for(ptr = craster[ymin] + byte0; ptr < end_ptr; ptr += byte_width)
				*ptr &= byte_mask0;
		}
		else
		{		/*  do first byte  */
			for(ptr = craster[ymin] + byte0; ptr < end_ptr; ptr += byte_width)
				*ptr &= byte_mask0;
				/*  do middle bytes (if any)  */
			for(b = byte0 + 1;  b < byte1; b++)
			{	for(ptr = craster[ymin] + b; ptr < end_ptr; ptr += byte_width)
					*ptr = 0;
			}
				/*  do last byte  */
			for(ptr = craster[ymin] + byte1; ptr < end_ptr; ptr += byte_width)
				*ptr &= byte_mask1;
		}
	}
	else
	{		/*  do first byte  */
		for(ptr = craster[ymin] + byte0; ptr < end_ptr; ptr += byte_width)
			*ptr &= byte_mask0;
			/*  do middle bytes of first word (if any)  */
		for(b = byte0 + 1; (b>>2) == word0 ; b++)
		{	for(ptr = craster[ymin] + b; ptr < end_ptr; ptr += byte_width)
				*ptr = 0;
		}
			/*  do middle words (if any)              */
		word_width = (job_stats[1] + 1) >> 5;
		end_wptr = lraster[ymax] + word1 + 1;
		for(w = word0 + 1; w < word1; w++)
		{	for(wptr = lraster[ymin] + w; wptr < end_wptr; wptr += word_width)
				*wptr = 0;
		}
			/*  do middle bytes of last word (if any) */
		for(b = byte0 + 1; b < byte1; b++)
		{	for(ptr = craster[ymin] + b; ptr < end_ptr; ptr += byte_width)
				*ptr = 0;
		}
			/*  do last byte  */
		for(ptr = craster[ymin] + byte1; ptr < end_ptr; ptr += byte_width)
			*ptr &= byte_mask1;
	}
#ifdef BIG_ENDIAN  /***** swap the bytes before and after opeartion *****/
	/* This is a HACK.  It works, but is very inefficient.  I will find
	   a better solution if time permits.  DBS@CWU */
	byte_swap(rasterm.rec0, raster_bincrement * (job_stats[2]+1), SWAP_LONG);
#endif


		/***************** update mini-raster ************************/
	divisor = (job_stats[1] > job_stats[2]) ? (job_stats[1]+1):(job_stats[2]+1);
	xmmin = (xmin * 400) / divisor;
	xmmax = (xmax * 400) / divisor;
	ymmin = (ymin * 400) / divisor;
	ymmax = (ymax * 400) / divisor;
		/*  find first (0) and last (1)  byte, & bit   */
	byte0 = xmmin >> 3;
	bit0 = (7 & ~xmmin);
	byte_mask0 = (-1) << (bit0+1);     /* mask for first byte in each row */
	byte1 = xmmax >> 3;
	bit1 = (7 & ~xmmax);
	byte_mask1 = (255) >> (7-bit1+1);     /* mask for last byte in each row  */
	byte_width = 50;                   /* width of mini-raster row in bytes */
	end_ptr = mini_raster[ymmax] + byte1 + 1; /*ptr just past last byte of chg*/
	if(byte0 == byte1)
	{	byte_mask0 |= byte_mask1;
		for(ptr = mini_raster[ymmin] + byte0; ptr < end_ptr; ptr += byte_width)
			*ptr &= byte_mask0;
	}
	else
	{		/*  do first byte  */
		for(ptr = mini_raster[ymmin] + byte0; ptr < end_ptr; ptr += byte_width)
			*ptr &= byte_mask0;
			/*  do middle bytes (if any)  */
		for(b = byte0 + 1;  b < byte1; b++)
		{	for(ptr = mini_raster[ymmin] + b; ptr < end_ptr; ptr += byte_width)
				*ptr = 0;
		}
			/*  do last byte  */
		for(ptr = mini_raster[ymmin] + byte1; ptr < end_ptr; ptr += byte_width)
			*ptr &= byte_mask1;
	}


		/***************** update display ****************************/
	if(!batch_mode)
	{		/*  convert rectangle to screen coords  */
		rstr_to_scrn(&xmin,&ymin);
		rstr_to_scrn(&xmax,&ymax);
			/* exchange ymin & ymax 'cause they are reversed on screen */
		w = ymin;
		ymin = ymax;
		ymax = w;

		if( (xmin > windowxmax) || (xmax < windowxmin) ||
			(ymin > windowymax) || (ymax < windowymin) );
				/*   do not try to display  */
		else
		{		/*   put rectangle on window  */
			xmin = (xmin >= windowxmin) ?  xmin : windowxmin;
			xmax = (xmax <= windowxmax) ?  xmax : windowxmax;
			ymin = (ymin >= windowymin) ?  ymin : windowymin;
			ymax = (ymax <= windowymax) ?  ymax : windowymax;
				
				/*  if resolution_code = 3, move limits out to edge of fat pix*/
			if(resolution_code == 3)
			{	xmin = xmin & ~7;
				ymin = ymin & ~7;
				xmax = (xmax & ~7) + 7;
				ymax = (ymax & ~7) + 7;
			}
				
				/*  switch to raster plane  */
			swplane(1);               /*  switch to raster plane  */
	
				/*  erase rectangle on screen   */
			bar_off(xmin,ymin,xmax,ymax);
		}
	}
	
		/***************** update edit flag **************************/
	if((job_stats[40] < job_stats[50]) || !job_stats[45])
	{	job_stats[45]++;
		job_stats[40] = job_stats[45];
	}

		/***************** return ************************************/
	return(0);
}
