/*******************************************************************************
*  erase_segment  (see manual, Command Ref., Internal Functions   <in>         *
*     This function looks at the position x,y (raster coords).  If the pixel   *
*    is 'on' then the segment containing it is erased.  This means that the    *
*    segment as on the raseter (not necessarily as assembled) is erased.       *
*    no assembly is required, and only nodes on the raster will end the        *
*    erasure.                                                                  *
*                                                                              *
*    Parameters:   long x,y    position on the raster to begin erasing.        *
*                  long *xmin,*ymin  minimum   value erased                    *
*                  long *xmax,*ymax  maximum   value erased                    *
*    Files/globals changed: raster                                             *
*    Values returned:  0 = segment erased,  1 = no erasure                     *
*    J.Dabritz  2/5/90                                                         *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"


long erase_segment(x,y,xmin,ymin,xmax,ymax)
	long x,y;
	long *xmin,*ymin,*xmax,*ymax;
{
	static long dirx[] = {0, 1, 1, 1, 0,-1,-1,-1};
	static long diry[] = {1, 1, 0,-1,-1,-1, 0, 1};

	long bit,word,on;
	long exit_mask;
	long i,j;
	long x0,y0;
	long x1,y1;
	long x2,y2;
	long done;
	long locale;

		/************ check for map *********************************/
	if(!job_stats[0])
		return(1);

		/************ check if x,y on raster ************************/
	if((x<0) || (x > job_stats[1]) || (y < 0) || (y > job_stats[2]))
		return(1);

		/************ check x,y for 'on' ****************************/
	word = x >> 5;
	bit  = (7 & ~x) | (x & 24);
	on = (lraster[y][word] & (1 << bit)) ?  1 : 0;
	if(!on)
		return(1);

		/************ initialize x,y min,max ************************/
	*xmin = x;
	*xmax = x;
	*ymin = y;
	*ymax = y;

		/************ erase in both directions  *********************/
	exit_mask = (1 << 12) | (1 << 11);

			/** erase in first direction **/
	i=0;
	x0 = x;
	y0 = y;
	/*** look at adjacent pixels ***/
	locale = locale_xy(x0,y0);
		/*** check for node          ***/
	if(tlocale[locale] & exit_mask)
		return(0);
	i = 0;
	done = 0;
	for(;;)
	{		/*** find next position      ***/
		i = (i + 6) % 8;    /** look cw from last pixel **/
		for(j=0; (j<8) && (!(tlocale[locale] & (1<<i))) ; j++,  i=(i+1) % 8);
		if(j==8)
			done = 1;
		else
		{	x1 = x0 + dirx[i];
			y1 = y0 + diry[i];
		}

			/*** look at pixels adjacent to next position ***/
		if(!done)
			locale = locale_xy(x1,y1);

			/*** erase current pixel     ***/
	  	pixel(x0,y0,0);

			/*** exit if next position is a node **/
		if(done)
			break;
		if(tlocale[locale] & exit_mask)
			break;

			/*** move to next position   ***/
		x0 = x1;
		y0 = y1;

			/*** reset min & max         ***/
		if(x0 < *xmin)
			*xmin = x0;
		if(y0 < *ymin)
			*ymin = y0;
		if(x0 > *xmax)
			*xmax = x0;
		if(y0 > *ymax)
			*ymax = y0;
	}
			/** save last endpoint **/
	if((x1 != x0)  ||  (y1 != y0))
	{	x2 = x1;
		y2 = y1;
	}
	else
	{	x2 = -1;
		y2 = -1;
	}

			/** erase in second direction **/
	i=0;
	x0 = x;
	y0 = y;
		/*** re-draw beginning pixel ***/
	pixel(x0,y0,1);

		/*** look at adjacent pixels ***/
	locale = locale_xy(x0,y0);
	i = 3;
	done = 0;
	for(;;)
	{		/*** find next position      ***/
		i = (i + 2) % 8;    /** look cw from last pixel **/
		for(j=0; (j<8) && (!(tlocale[locale] & (1<<i))) ; j++,  i=(i+7) % 8);
		if(j==8)
			done = 1;
		else
		{	x1 = x0 + dirx[i];
			y1 = y0 + diry[i];
		}

			/*** look at pixels adjacent to next position ***/
		if(!done)
			locale = locale_xy(x1,y1);

			/*** erase current pixel     ***/
	  	pixel(x0,y0,0);

			/*** exit if next position is a node **/
		if(done)
			break;
		if(tlocale[locale] & exit_mask)
			break;

			/*** move to next position   ***/
		x0 = x1;
		y0 = y1;
		if((x0 == x2)  &&  (y0 == y2))
			break;

			/*** reset min & max         ***/
		if(x0 < *xmin)
			*xmin = x0;
		if(y0 < *ymin)
			*ymin = y0;
		if(x0 > *xmax)
			*xmax = x0;
		if(y0 > *ymax)
			*ymax = y0;
	}
		/************ return ****************************************/
	return(0);
}
