/*******************************************************************************
*       erase_xyobj     (see manual, Internal Functions)                <in>   *
*    Finds objects using the input x,y location & search radius.  Erases       *
*    a 3 pixel with path on raster under vector object.                        *
*                                                                              *
*    Arguments:    float x,y,r:   raster coordinates and search radius.        *
*                                 (search radius used for segs & nodes)        *
*                  long otype:    0 = node,  1 = segment,  2 = polygon         *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified:  none.                                         *
*                                                                              *
*                      j.dabritz  7/15/90                                      *
*******************************************************************************/

#include "../../incl/globals.h"

long erase_xyobj(x,y,r,otype)
float x,y,r;
long otype;
{	
	long obj_no;


		/********* check for map ****************************************/
	if(!job_stats[0])
	{	message("no current map.");
		return(1);
	}

		/********* check for objects ************************************/
	if((otype <0) || (otype > 2))
	{	message("Illegal object type.  erase_xyobj  aborted.");
		return(1);
	}

		/********* find object ******************************************/
	switch(otype)     {
	case 0:           /** nodes ***/
		obj_no = find_vnode(x,y,r,0);
		break;
	case 1:           /** segments  */
		obj_no = find_vseg(x,y,r,0);
		break;
	case 2:           /** polygons  */
		obj_no = find_vpoly(x,y,0);
		break;
	default:
		message("Illegal object type.  Query not performed.");
		return(0);
	}

	if(obj_no < 0)
	{	message(" No vector object found within search radius.  No action taken.");
		return(0);
	}

		/****** erase object from map raster ********************/
	erase_mapobj(obj_no);
	

		/********* return ***********************************************/
	return(0);
}
