/*******************************************************************************
*       fill_concnrs    (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Erase the corners left un-erased by the line_diag function    *
*       when executing the continue command while in the connect mode.         *
*       UNLESS in mode 25 (recall connect), then recall these bits.            *
*                                                                              *
*    Arguments:    long x1,y1,x2,t2  line segment endpoints, (raster coords)   *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified:  none                                          *
*                                                                              *
*                      j.dabritz  8/13/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>


long fill_concnrs(x1,y1,x2,y2)
	long x1,y1,x2,y2;
{	
	long dx,dy;
	long e,e1,e2;
	long i;
	long on;

		/********* set 'on' variable *******************************/
	if(input_mode == 25)       /** recall connect **/
		on = 2;
	else
		on = 0;

		/*********   clip line to raster ***************************/
	if(clipl(&x1,&y1,&x2,&y2,0,job_stats[1],0,job_stats[2]))
		return(0);

		/*********   calc dx and switch points to make it non-negative.   */
	dx = x2 - x1;
	if(dx < 0) 
	{	i = x1;
		x1 = x2;
		x2 = i;
		i = y1;
		y1 = y2;
		y2 = i;
		dx = x2 - x1;
	}
		/*********   calc cy  */
	dy = y2 - y1;


		/*********   divide situation up into cases ****************/
	if(dy >= 0)              /* line has positive slope */
	{	if(dx > dy)          /* slope is < 1  */
		{	e  = (dy * 2) - dx;
			e1 = (dy - dx) * 2;
			e2 = dy * 2;
			for(i = -1; i != dx; i++, x1++)
			{	
				if(e > 0)
				{	if((i+1) != dx)  /* fill in corners of a 3 pixel wide line*/
					{	pixel(x1+1, y1-1, on);
						pixel(x1, y1+2,on);
					}
					y1++;
					e += e1;
				}
				else
					e += e2;
			}
		}
		else                  /* slope is > 1    */
		{	e  = (dx * 2) - dy;
			e1 = (dx - dy) * 2;
			e2 = dx * 2;
			for(i = -1; i != dy; i++, y1++)
			{	
				if(e > 0)
				{	if((i+1) != dy)         /* add square jump to diag. line */
					{	pixel(x1-1, y1+1, on);
						pixel(x1+2, y1, on);
					}
					x1++;
					e += e1;
				}
				else
					e += e2;
			}
		}
	}
	else                     /* slope is negative  */
	{	if(dx > -dy)          /* slope is > -1  */
		{	e  = ((-dy) * 2) - dx;
			e1 = ((-dy) - dx) * 2;
			e2 = (-dy) * 2;
			for(i = -1; i != dx; i++, x1++)
			{	
				if(e > 0)
				{	if((i+1) != dx)         /* add square jump to diag. line */
					{	pixel(x1+1, y1+1, on);
						pixel(x1, y1-2, on);
					}
					y1--;
					e += e1;
				}
				else
					e += e2;
			}
		}
		else                  /* slope is < -1   */
		{	e  = (dx * 2) + dy;
			e1 = (dx + dy) * 2;
			e2 = dx * 2;
			for(i = 1; i != dy; i--, y1--)
			{	
				if(e > 0)
				{	if((i-1) != dy)         /* add square jump to diag. line */
					{	pixel(x1-1, y1-1, on);
						pixel(x1+2, y1, on);
					}
					x1++;
					e += e1;
				}
				else
					e += e2;
			}
		}
	}

	return(0);
}
