/*******************************************************************************
*       find_bit        (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    searches a square area around raster location x,y up to a     *
*    'radius' r.  Returns 1 if an 'on' bit is located & sets x & y to that     *
*    location.                                                                 *
*                                                                              *
*    Arguments:    long *x,*y:   raster coordinates of center of search square *
*                  long r:       search limit (half of a side of max square)   *
*                                                                              *
*    Returned Values: (long) 1 = bit found, x & y set to that location.        *
*                            0 = bit not found.                                *
*                                                                              *
*    Global Variables Modified:  none                                          *
*                                                                              *
*                      j.dabritz  6/28/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"

long check_snap();

long find_bit(x_center, y_center, radius)
	long *x_center, *y_center;
	long radius;
{	
	long x,y;
	long r,d;
	long found;    /*  flag   */


		/***************** check for map present *********************/
	if(!job_stats[0])
		return(0);

		/***************** check input paramters *********************/
	if(radius < 0)
		return(0);


		/***************** look for 'on' pixel from center out *******/
	found = 0;
	for(r=0; r <= radius; r++)
	{	for(d=0; d <= r; d++)   /*  check 8 positions at a time */
		{	x = (*x_center) + r;          /*  check right side, upper */
			y = (*y_center) + d;
			if(found = check_snap(x,y))
				break;
			x = (*x_center) - r;          /*  check left side, lower */
			y = (*y_center) - d;
			if(found = check_snap(x,y))
				break;
			x = (*x_center) - d;          /*  check top side, left */
			y = (*y_center) + r;
			if(found = check_snap(x,y))
				break;
			x = (*x_center) + d;          /*  check bottom side, right */
			y = (*y_center) - r;
			if(found = check_snap(x,y))
				break;
				/* check next 4 only if different from first 4 */
			if(d)
			{	x = (*x_center) + r;          /*  check right side, lower */
				y = (*y_center) - d;
				if(found = check_snap(x,y))
					break;
				x = (*x_center) - r;          /*  check left side, upper */
				y = (*y_center) + d;
				if(found = check_snap(x,y))
					break;
				x = (*x_center) + d;          /*  check top side, right */
				y = (*y_center) + r;
				if(found = check_snap(x,y))
					break;
				x = (*x_center) - d;          /*  check bottom side, left */
				y = (*y_center) - r;
				if(found = check_snap(x,y))
					break;
			}
		}
			/*  if found, set x_center, y_center & exit  */
		if(found)
		{	*x_center = x;
			*y_center = y;
			break;
		}
	}

		/***************** return ************************************/
	return(found);
}

/*******************************************************************************
*       check_snap      (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    checks a single bit & returns 1 if it is on.                  *
*                                                                              *
*    Arguments:    long x,y:   raster coordinates of bit posision              *
*                                                                              *
*    Returned Values: (long) non-zero = bit found                              *
*                            0 = bit not found.                                *
*                                                                              *
*    Global Variables Modified:  none                                          *
*                                                                              *
*                      j.dabritz  6/28/89                                      *
*******************************************************************************/


long check_snap(x, y)
	long x,y;
{	
	long word, bit;

		/**************** check input *************************************/
	if((x<0) || (x>job_stats[1]) || (y<0) || (y>job_stats[2]) )
		return(0);

		/**************** calculate word and bit **************************/
	word = x >> 5;
	bit = (7 & ~x) | (x & 24);

		/**************** check bit & return ******************************/
	return(lraster[y][word] & (1 << bit));
}

/******************************************************************************/
