/*******************************************************************************
*       find_nodes      (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Locates all 0, 1, 3, 4 branch nodes on the raster and stores them as      *
*    objects.                                                                  *
*                                                                              *
*    Arguments:    none                                                        *
*                                                                              *
*    Returned Values: (long) 1 = failure                                       *
*                            0 = normal end of job.                            *
*                                                                              *
*    Global Variables Modified:  the job_stats[10] variable may be changed     *
*    as a result of this program, but is actually changed by the put_objects   *
*    program, as are the objects records in the temp file.  job_stats[3].      *
*                                                                              *
*                      j.dabritz  7/11/89                                      *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for Ultrix support */

#include "../../incl/globals.h"

long find_nodes()
{	
	long j;
	long node_count;
	long bit;
	long key;
	long x,y;
#ifndef ULTRIX
	long *end_ptr, *ptr;
#else
	unsigned long *end_ptr, *ptr;
#endif

	OBJECTS node;

	char string[256];

		/***************** check for map present *********************/
	if(!job_stats[0])
	{	message("no map present, aborting");
		return(1);
	}


		/***************** init node object structure ****************/
	for(j=0; j<12; j++)
		node.i12[j] = 0;
	for(j=0; j<4; j++)
		node.f4[j] = 0.0;

		/***************** zero out node_lookup global ***************/
	for(j=0; j<=job_stats[2]; j++)
		node_lookup[j] = 0;

		/***************** look for nodes over entire map ************/
		/*   inform user  */
	message("finding all 0, 1, 3, & 4 branch nodes. . .");
		/*   examine raster by word, bottom to top   */
#ifndef ULTRIX
	(unsigned long *)end_ptr = lraster[job_stats[2]] + raster_wincrement;
#else
	end_ptr = lraster[job_stats[2]] + raster_wincrement;
#endif
	node_count = 0;
#ifndef ULTRIX
	for((unsigned long *)ptr = lraster[0]; ptr < end_ptr; ptr++)
#else
	for(ptr = lraster[0]; ptr < end_ptr; ptr++)
#endif
	{	if(*ptr)               /*  word has at least 1 'on' bit   */
		{		/*** look at all bits, 0-31  */
			for(bit = 0; bit < 32; bit++)
			{		/*** if bit is 'on', get locale variable & check for node */
				if(*ptr & (1<<bit))
				{	key = tlocale[locale_ptr(ptr,bit)];
					if(key & (1 << 13))         /*  hit a node   */
					{		/*  set obje type code (using number of branches */
						node.i12[0] = 3 + ((key >> 29) & 7);
							/*  set branch type   */
						node.i12[5] = key & 255;
							/*  set node centroid (node location) */
						x = ((((unsigned long *)ptr - lraster[0]) % raster_wincrement) << 5) | 
					 	 (7 & ~bit) | (bit & 24);
						y = ((unsigned long *)ptr - lraster[0]) / raster_wincrement;
						node.f4[0] = x;
						node.f4[1] = y;

							/*  update node lookup global ***/
						node_lookup[y] = node_count;

							/*  store node  */
						if(put_objects(&node,node_count++))
						{	message("abort, find_nodes, location 1");
							return(1);
						}
						if(!(node_count % 250))
						{	sprintf(string,"stored node %d  ",node_count);
							message(string);
						}
					}
				}
			}
		}
	}


		/***************** return ************************************/
		/*   inform user  */
	sprintf(string,"   done, %d nodes stored.",node_count);
	message(string);

	return(0);

}
