/*******************************************************************************
*       find_polyr      (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Given raster coordinates, finds the polygon in which they lie using       *
*    only raster and object information (no vector information).               *
*                                                                              *
*    Arguments:    x, y   raster coordinates.                                  *
*                                                                              *
*    Returned Values: (long) p = polygon (object) record number                *
*                            0 = no polygon found                              *
*                                                                              *
*    Global Variables Modified:  none                                          *
*                                                                              *
*                      j.dabritz  7/17/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"

long find_polyr(x,y)
long x,y;
{  
  OBJECTS node;                 /*  object structure for node  */
  OBJECTS segment;              /*  object structure for segment */

  long *wrd_ptr;                /*  long pointer for examining raster */
  long bit;                     /*  bit number (in word) */
  long b;                       /*  node branch numbers  */
  long n;                       /*  node numbers         */
  long d, d2;                   /*  vector direction (see above)  */
  long s;                       /*  segment record number */
  long p;                       /*  polygon record number */
  long found;                   /*  flag for node found */
  long key;                     /*  tlocale element (see tlocale global) */
  long mask;                    /*  mask for checking raster words */
  long begin_x, begin_y;        /*  raster coordinates         */
  long low_yx, low_y;           /*  x & y of lowest (y) of segment (temp use) */
  long low_d;                   /*  forward direction at low point  */
  long end;                     /*  loop control counter variable */
  static long movex[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
  static long movey[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };


    /***************** check for map present *********************/
  if(!job_stats[0])
    return(0);

    /***************** check for nodes ***************************/
  if(!job_stats[10])
    return(0);

    /***************** drop down until a line is hit *************/
  wrd_ptr = (long *) (lraster[0] + raster_wincrement*y + (x >> 5));
  bit = (7 & ~x) | (x & 24);
  mask = (1 << bit);
  for(; y>=0 ; y--, wrd_ptr -= raster_wincrement)
  { if(*wrd_ptr & mask)                /*  hit an 'on' bit  */
    { if(locale_ptr(wrd_ptr,bit))      /*  ignores isolated 'on' pixels */
        break;
    }
  }


    /***************** trace line until a node is hit ************/
  if(y<0)            /*  map border polygon  */
    return(job_stats[7]);

  key = tlocale[locale_xy(x,y)];         /*  get locale key */
  if(key & (1 << 13))                    /*  landed on a node */
  {    /****** get highest  branch as incomming direction   */
    for(d=7; !(key & (1 << d)) ; d--);
    d = (d + 4) & 7;     /*   reverse direction as if segment had been traced */
  }
  else                                   /*  landed not on a node  */
  { begin_x = x;
    begin_y = y;
    low_yx = x;
    low_y  = y;
    low_d  = 3;
    for(d=0; !(key & (1 << d)) ; d++);     /*  find beginning direction  */
    for(;;)
    {    /********** move ahead 1 pixel **********/
      x += movex[d];
      y += movey[d];
        /********** check/reset low_y, low_yx ***/
      if((y < low_y)  ||  ((y == low_y) && (x < low_yx)))
      { low_y = y;
        low_yx = x;
        low_d = d;
      }
        /********** get locale key variable *****/
      key = tlocale[locale_xy(x,y)];
        /********** check for node **************/
      if(key & (1<<13))
        break;
        /********** check for closed loop (simple island) ******/
      if((x == begin_x)  &&  (y == begin_y))
        break;
        /********** find next direction *********/
      for(d = (d+5) & 7; !(key & (1<<d)); d = (d+1) & 7);
    }
  }

    /***************** find node record number *******************/
  found = 0;
  p = 0;           /*  default polygon number  (= none) */
  if(key & (1 << 13))              /*  a 1,3,or 4 branch node  */
	{
    n = node_lookup[y];
    for(; n>=0; n--)
    { get_objects(&node,n);
      if(node.f4[1] < (float) y)
        break;
      if(node.f4[0] == (float) x)
      { found = 1;
        break;
      }
    }
  }
  else                               /* a 2-branch (island) node  */
	{
    n = job_stats[4];
    d = low_d;
    end = (job_stats[7]) ?  job_stats[7] : job_stats[10];
    for(; n < end; n += 2)
    { get_objects(&node,n);
      if(node.f4[1] > (float) low_y)
        break;
      if(node.f4[0] == (float) low_yx)
      {	if(node.f4[1] == (float) low_y)
      	{ found = 1;
        	  break;
			}
      }
    }
  }

    /***************** find segment traced **********/
  if(found)
  { d = (d + 4) & 7;       /*  reverse direction to look out from node */
      /***** find branch number of segment ***/
    for(b=0, d2 = 0; d2 <= d; d2++)
    { if(node.i12[5] & (1<<d2))
        b++;
    }
      /****** find segment number of branch ***/
    s = node.i12[5 + b];
      /***************** find polygon above segment ****************/
    if(s > 0)              /*  segment starts at this node  */
    { get_objects(&segment,s);
      p = segment.i12[7];
    }
    else
    { get_objects(&segment,-s);
      p = segment.i12[8];
    }
  }

    /***************** return polygon (object) record number *****/
  return(p);
}
