/*******************************************************************************
*    find_vertex   (see manual, Command Ref., Internal Functions)    <In>      *
*            This program finds nearest sides & verticies within the           *
*       specified levels, snap radius, & snap flags.  Information              *
*       about the location/object/verticies found is returned.                 *
*                                                                              *
*       parameters:    double x_in raster coordinates of center point.         *
*                      double y_in raster coordinates of center point.         *
*                      double r    search radius in pixels ( > 0.0)            *
*                      long l1,l2  range of levels on which to search (0,32-95)*
*                      long vf     vertex flag: 0=snap to side or vertex       *
*                                  vertex flag: 1=snap preferentially to vertex*
*                                  vertex flag: 2=snap only to vertex.         *
*      (values set)    double *rr  actual distance to side/vertex found.       *
*                      double *rx  actual position of side/vertex found.       *
*                      double *ry  actual position of side/vertex found.       *
*                      long *ro    object/v_object record number found.        *
*                      long *rv1   vertex record number of vertex/side         *
*                      long *rv2   vertex record number of side second vertex  *
*                      long *rl    level on which vertex/side found            *
*                      long *rf    type found flag:0=node                      *
*                                                  1=vertex                    *
*                                                  2=side between verticies    *
*                                                                              *
*       files/globals changed:  none                                           *
*                                                                              *
*       values returned:  long 0=found, 1=not found, -1=error                 *
*                                                                              *
*            J.Dabritz   7/16/90                                               *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long find_vertex(x_in,y_in,r,l1,l2,vf,rr,rx,ry,ro,rv1,rv2,rl,rf)
double x_in, y_in;
double r;
long l1,l2;
long vf;
double *rr;
double *rx, *ry;
long *ro;
long *rv1;
long *rv2;
long *rl;
long *rf;
{
   OBJECTS node;
   INDEX_LIST index_list;
   COORD_LIST coord_list;

   long row, col;
   long x1, y1, x2, y2;
   long i, v;
   long return_val;
   long otype, level;
   long best_otype;

   double x,y;
   double vx, vy;
   double last_vx, last_vy;
   double array[2][3];
   double dx, dy;
   double dist_squared, min_square;
   double r2;
   double r_squared;


      /************* init output object   ************************/
	*ro = 0;

      /************* check for map loaded ************************/
   if(!job_stats[0])
   {   message("You must Create or Acquire a map befor executing the Level_snap command.");
      return(1);
   }

      /************* check level number **************************/
   if(l1 > l2)
   {   i = l1;
      l1 = l2;
      l2 = i;
   }
   if((l1 < 0) || (l1 > 95)  ||  (l2 < 0) || (l2 > 95))
   {   message("find_vertex:  Illegal level number.");
      return(1);
   }


      /************* check radius ********************************/
   if(r < (float) 0.0)
   {   message("find_vertex:  Snap radius must be greater than zero.");
      return(1);
   }

      /************* check vertex flag ***************************/
   if((vf < 0)  ||  (vf > 2))
   {   message("find_vertex:  vertex flag must be in range 0-2.");
      return(1);
   }

      /*** Initialize automatics ***/
   last_vx = last_vy = 0.0;

      /*********** set up output coords *******************************/
   x = x_in;
   y = y_in;
   return_val = 1;

      /*********** find index_array window ****************************/
   r2 = (x - r + 0.5);    /* x min (pixels) */
   x1 = r2;               /* x min (pixels) */
   x1 = x1 >> 5;          /* x min (index_array coords) */
   r2 = (x + r + 0.5);    /* x max (pixels) */
   x2 = r2;               /* x max (pixels) */
   x2 = x2 >> 5;          /* x max (index_array coords) */
   r2 = (y - r + 0.5);    /* y min (pixels) */
   y1 = r2;               /* y min (pixels) */
   y1 = y1 >> 5;          /* y min (index_array coords) */
   r2 = (y + r + 0.5);    /* y max (pixels) */
   y2 = r2;               /* y max (pixels) */
   y2 = y2 >> 5;          /* y max (index_array coords) */
      /** test for raster coverage *****************************/
   if((x2 < 0)  || (x1 > (job_stats[1] >> 5)))
      return(-1);
   if((y2 < 0)  || (y1 > (job_stats[2] >> 5)))
      return(-1);
      /** limit to range of index_array ***/
   if(x1 < 0) 
      x1 = 0;
   if(y1 < 0)
      y1 = 0;
   if(x2 > (job_stats[1] >> 5))
      x2 = (job_stats[1] >> 5);
   if(y2 > (job_stats[2] >> 5))
      y2 = (job_stats[2] >> 5);
   
      /*********** search for segment within r dist **********************/
      /** for each row & col of index array, search objects */
   min_square = (r * r) + .01;       
   r_squared = r * r;
   best_otype = 1;
   for(row = y1; row <= y2; row++)
   {   for(col = x1; col <= x2; col++)
      {      /** search all indicated objects **/
         index_list.next = index_array[row][col];
         for(; index_list.next;)
         {      /** get index_list record **/
            if(get_indexl(&index_list,index_list.next))
               return(-1);
              /*** check for object type == segment ***/
            if(index_list.type > 31)
               level = index_list.type / 10;
            else
               level = 0;

            otype = index_list.type   % 10;

            if((level >= l1)  &&  (level <= l2))
            {      /**** test every vertex & side for distance **/
               i = (index_list.v1 < index_list.v2)  ?  1 : -1;
               for(v = index_list.v1;   ; v += i)
               {      /*** get vertex coords ***/
                  if(level)
                  {   if(get_v_coord(&coord_list,v))
                     {   message("find_vertex, abort, location 0.");
                        return(-1);
                     }
                  }
                  else if(otype == 1)     /** segments **/
                  {   if(get_coordlst(&coord_list,v))
                     {   message("find_vertex, abort, location 1.");
                        return(-1);
                     }
                  }
                  else if(otype == 2)     /** polygon **/
                     break;
                  else                    /** node **/
                  {     /*** get node record ***/
                     if(get_objects(&node,index_list.obj_rec))
                     {   message("find_vertex, abort, location 1.1.");
                        return(-1);
                     }
                     coord_list.x = node.f4[0];
                     coord_list.y = node.f4[1];
                  }

                  vx = coord_list.x;
                  vy = coord_list.y;
            
                     /*** calculate distance to input point **/
                  dx = x - vx;
                  dy = y - vy;
                  dist_squared = (dx * dx) + (dy * dy);
                     
                     /*** check against min dist ***/
                  if(dist_squared <= r_squared)
                  {   if((dist_squared < min_square) ||  (best_otype && vf))
                     {   min_square = dist_squared;
                        *rr = min_square;
                        *rx = vx;
                        *ry = vy;
                        *ro = index_list.obj_rec;
                        *rv2 = v;
								if(v != index_list.v1)
                           *rv1 = v - i;
								else
								{	*rv1 = v;
									if(v != index_list.v2)
										*rv2 = v + i;
								}
                        *rl = level;
                        *rf = otype;
                        return_val = 0;
                        best_otype = 0;
                     }
                  }

                  if(level==0  && otype != 1)
                     break;
               
                     /* if not first point, then get dist to vector **/
                  if((v != index_list.v1) && (vf < 2) && ((vf <1) || (best_otype)))
                  {   dx = vx - last_vx;
                     dy = vy - last_vy;
                     if((dx != 0.0) || (dy != 0.0))
                     {   array[0][0] = dx;
                        array[0][1] = dy;
                        array[0][2] = x - last_vx;
                        array[1][0] = dy;
                        array[1][1] = -dx;
                        array[1][2] = y - last_vy;
                        if(!mat_solve(2,3,array))
                        {      /*** see if pt perp. off side of vector **/
                           if((array[0][2] > 0.0) && (array[0][2] < 1.0))
                           {   dist_squared = array[1][2] * array[1][2] * 
                                          ((dx * dx) + (dy * dy));
                                 /*** check against min dist ***/
                              if(dist_squared < min_square)
                              {   min_square = dist_squared;
                                 return_val = 0;
                                 best_otype = 1;
                                 *rr = min_square;
                                 *rx = last_vx + array[0][2] * dx;
                                 *ry = last_vy + array[0][2] * dy;
                                 *ro = index_list.obj_rec;
                                 *rv1 = v - i;
                                 *rv2 = v;
                                 *rl = level;
                                 *rf = 2;
                              }
                           }
                        }
                     }
                  }
         
                     /*** check to see if verticies done ***/
                  if(v == index_list.v2)
                     break;
   
                     /*** reset last_v coords ***/
                  last_vx = vx;
                  last_vy = vy;
               }
            }
         }
      }
   }

      /*************** return ************************************/
   return(return_val);
}
