/*******************************************************************************
*       find_vseg       (see manual, Internal Functions)                <in>   *
*    Searches the index_array / index_list or objects files for a segment      *
*    within r pixels of the input x,y location which has the input level.      *
*    If any are found, the record number of the closest one is returned.       *
*                                                                              *
*    Arguments:    float x,y,r:   raster coordinates and radius (pixels)       *
*                  long level:    0 = map (objects record returned)            *
*                                 1-24 = map set (objects record returned)     *
*                                 32-95 = import obj (v_objects rec returned)  *
*                                                                              *
*    Returned Values: (long) n = record number of segment, or                  *
*                           -1 = no segment found or error.                    *
*                                                                              *
*    Global Variables Modified:  none.                                         *
*                                                                              *
*                      j.dabritz  8/2/89                                       *
*******************************************************************************/

#include "../../incl/globals.h"

long find_vseg(x,y,r,level)
float x,y,r;
long level;
{	
	INDEX_LIST index_list;
	OBJECTS object;
	COORD_LIST coord_list;

	long go_on;
	long row, col;
	long record_no;
	long x1,y1,x2,y2;
	long i, v;

	double vx, vy;
	double last_vx, last_vy;
	double array[2][3];
	double dx, dy;
	double dist_squared, min_square;


		/*********** check for map **************************************/
	if(!job_stats[0])
		return(-1);

		/*********** check for file type needed *************************/
	if((level >= 0) && (level < 32))    /* objects file record wanted   */
	{	if(job_stats[10] == 0)
			return(-1);
		if(job_stats[49])        /*  map needs to be re-indexed */
		{	if(index_map())      /*  1 = index failed           */
				return(-1);
		}
	}
	else if((level >=32) && (level < 96))  /* v_objects file record wanted */
	{	if(job_stats[15] == 0)
			return(-1);
	}
	else              /* illegal level, abort          */
		return(-1);

		/*********** check r ********************************************/
	if(r < 0.0)
		return(-1);

		/*********** find index_array window ****************************/
	x1 = (x - r + 0.5);    /* x min (pixels) */
	x1 = x1 >> 5;          /* x min (index_array coords) */
	x2 = (x + r + 0.5);    /* x max (pixels) */
	x2 = x2 >> 5;          /* x max (index_array coords) */
	y1 = (y - r + 0.5);    /* y min (pixels) */
	y1 = y1 >> 5;          /* y min (index_array coords) */
	y2 = (y + r + 0.5);    /* y max (pixels) */
	y2 = y2 >> 5;          /* y max (index_array coords) */
		/** test for raster coverage *****************************/
	if((x2 < 0)  || (x1 > (job_stats[1] >> 5)))
		return(-1);
	if((y2 < 0)  || (y1 > (job_stats[2] >> 5)))
		return(-1);
		/** limit to range of index_array ***/
	if(x1 < 0) 
		x1 = 0;
	if(y1 < 0)
		y1 = 0;
	if(x2 > (job_stats[1] >> 5))
		x2 = (job_stats[1] >> 5);
	if(y2 > (job_stats[2] >> 5))
		y2 = (job_stats[2] >> 5);
	
		/*********** search for segment within r dist **********************/
		/** for each row & col of index array, search objects */
	min_square = (r + .01) * 1.01;          /** in case r = 0 or very large **/
	min_square = min_square * min_square;   /** compair squares to save time **/
	for(row = y1; row <= y2; row++)
	{	for(col = x1; col <= x2; col++)
		{		/** search all indicated objects **/
			index_list.next = index_array[row][col];
			for(; index_list.next;)
			{		/** get index_list record **/
				if(get_indexl(&index_list,index_list.next))
					return(-1);
				  /*** check for object type == segment ***/
				go_on = 1;
				if(level < 32)                /* objects file records */
				{	if(index_list.type != 1)
						go_on = 0;
					else if(level > 0)
					{	if(get_objects(&object,index_list.obj_rec))
							go_on = 0;
						else
						{	if(!(object.i12[4] & (1 << level)))
								go_on = 0;
						}
					}
				}
				else            /* v_objects file records */
				{	if(((index_list.type / 10) != level) || ((index_list.type % 10) != 1))
						go_on = 0;
				}

					/**** test every vertex & side for distance **/
				i = (index_list.v1 < index_list.v2)  ?  1 : -1;
				for(v = index_list.v1; go_on ; v += i)
				{		/*** get vertex coords ***/
					if(level < 32)    /* objects file record wanted   */
					{	if(get_coordlst(&coord_list,v))
							return(-1);
					}
					else              /* v_objects file record wanted */
					{	if(get_v_coord(&coord_list,v))
							return(-1);
					}
					vx = coord_list.x;
					vy = coord_list.y;
			
						/*** calculate distance to input point **/
					dx = x - vx;
					dy = y - vy;
					dist_squared = (dx * dx) + (dy * dy);
						
						/*** check against min dist ***/
					if(dist_squared < min_square)
					{	min_square = dist_squared;
						record_no = index_list.obj_rec;
					}
				
						/*** if not first point, then get dist to vector **/
					if(v != index_list.v1)
					{	dx = vx - last_vx;
						dy = vy - last_vy;
						if((dx != 0.0) || (dy != 0.0))
						{	array[0][0] = dx;
							array[0][1] = dy;
							array[0][2] = x - last_vx;
							array[1][0] = dy;
							array[1][1] = -dx;
							array[1][2] = y - last_vy;
							if(!mat_solve(2,3,array))
							{		/*** see if pt perp. off side of vector **/
								if((array[0][2] > 0.0) && (array[0][2] < 1.0))
								{	dist_squared = array[1][2] * array[1][2] * 
                                     ((dx * dx) + (dy * dy));
										/*** check against min dist ***/
									if(dist_squared < min_square)
									{	min_square = dist_squared;
										record_no = index_list.obj_rec;
									}
								}
							}
						}
					}
				
						/*** check to see if verticies done ***/
					if(v == index_list.v2)
						break;

						/*** reset last_v coords ***/
					last_vx = vx;
					last_vy = vy;
				}
			}
		}
	}

		/*********** return closest segment record number or -1 ************/
	if(min_square <= (r * r))
		return(record_no);
	else
		return(-1);

}
