/*******************************************************************************
*	fj      	(see manual, Internal Functions)            	<in>   *
*                                                                              *
*	    		Function junction is the premier command interpreter.  *
*			It will determine if the first element of a raw        *
*			parameter string is a menu, an intrinsic function,     *
*			or a macro, and it will then call the appropriate      *
*			function manager (fnM).                                *
*			Parameters:        none                                *
*			Globals Modified:  raw_buf                             *
*                                                                              *
*			Returns zero upon normal end-of-job (EOJ).             *
*			J. Dabritz   C. Jonson   5/89                          *
*******************************************************************************/
#include <string.h>

#include "../../incl/globals.h"
#include "../../incl/fm.h"
#include "../../incl/in.h"

long fj()
{
/*   Automatics and statics   */

	static char parse_tok[STRLIST];

	long indx = 0;
	long len;
	long qflag;

	char parameters[256];
	char *param;
	char string[256];
	char *ptr;
	

		/***********   substitute $strings in raw_buf        ***********/
		/*** check for $ character ****/
	if(strstr(raw_buf,"$") != (char *) 0)
	{	strcpy(string,raw_buf);
		dollar_cnvt(string);
		string[STRLIST - 1] = (char) 0;
		strcpy(raw_buf,string);
	}


/*   Parse the first token from raw_buf    */
	trim(raw_buf);
	len = parsen( raw_buf, delim_buf, parse_tok, 0L );
	if( len == 0)
		return(0);
	if(len == strlen(raw_buf))     /*  (no parameters)     */
		param = raw_buf + len ;    /*  set to null  */
	else
		param = raw_buf + len + 1;       /*  set to existing parameters */

	strcpy(parameters,param);            /*  load into a separate, safe place*/

	lowerc(trim(parse_tok));
	if(!(*parse_tok))
		return(0);

		/*   check to see if string is a function  */
	if( (indx = isfunc(parse_tok)) >= 0 )
	{	if(1 == (*fm_ptr[indx])(parameters))	/*  call function mgr */
		{	raw_buf[STRLIST-3] = 0;
			if(macro_level)
			{	sprintf(history[next_hist],"  %s",raw_buf);
				history[next_hist][95] = (char) 0;
			}
			else
				strncpy(history[next_hist],raw_buf,STRLIST-1);
			if(collecting_macro != 1)
				message(history[next_hist]);
			next_hist = (next_hist +1) % 512;
		}
	}
		/*   check to see if string is menu, if so display that menu  */
	else if(ismenu(parse_tok));
		/*   check to see if string is macro, if so execute           */
	else if(ismacro(parse_tok, 0L));
		/*   check to see if it is a partial match to a function      */
	else if( (indx = findfunc(parse_tok,&qflag)) >= 0)
	{	message(uf_name[indx]);
		if(1 == (*fm_ptr[indx])(parameters))	/*  call function mgr */
		{	raw_buf[STRLIST-3] = 0;
			if(macro_level)
			{	sprintf(history[next_hist],"  %s",raw_buf);
				history[next_hist][95] = (char) 0;
			}
			else
				strncpy(history[next_hist],raw_buf,STRLIST-1);
			if(collecting_macro != 1)
				message(history[next_hist]);
			next_hist = (next_hist +1) % 512;
		}
	}
		/*   check to see if it is a partial match to a macro         */
	else if(findmacro(parse_tok, &qflag, 0L));
		/*   otherwise tell user that it couldn't be found.           */
	else
	{	append_msg("      ***  not a command, menu, or macro.");
		raw_buf[0] = 0;
		return (0);
	}


		/*  we must have hit an executable string or we wouldn't have
			gotten past that return just above.         
		check to see if we are in macro collection mode & 
			store the command if necessary.                          */
	if(collecting_macro)
	{	if(collecting_macro < 1)   /** flag for the macro_begin command */
		{	collecting_macro = 1;
			message("  ----------- begin macro collection -----------");
		}
		else
		{	sprintf(string,"%s",raw_buf);
			trim(string);
			if(*string)
			{	ptr = raw_buf + 2;
				trim(ptr);
				sprintf(string,"%s\n",raw_buf);
				if(0 > writea(string,macro_file))
				{	message("error on write to macro file.  macro collection aborted.");
					collecting_macro = 0;
					closef(macro_file);
				}
				else
				{	message(raw_buf);
					append_msg("   <<< added to macro");
				}
			}
		}
	}

		/*   now return                                   */
	raw_buf[0] = 0;
	return(0);
}				/*  End of fj()  */
