/*******************************************************************************
*	fsfree		(see manual, Internal Functions)         	<in>   *
*			Determines the number of free bytes in a filesystem.   *
*			The path is a file in the desired filesystem.          *
*			                                                       *
*			Returns the number of free bytes upon good EOJ, else   *
*			returns -1 upon error.                                 *
*			CEJ 05/30/89 					       *
*******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#ifdef SV32 
#include <sys/statfs.h>
#else
#include <ustat.h>
#include <sys/stat.h>
#endif

long fsfree(path)
	char *path;
{
/*  Local variables  */

#ifdef SV32
	struct statfs fsbuf;
#else
	struct ustat  ubuf;
	struct stat   sbuf;
#endif
	long free_bytes;
	
/*  Get file system status info  */

#ifdef SV32
	if( statfs( path, &fsbuf, sizeof(struct statfs), 0))
		return(-1);
	free_bytes = fsbuf.f_bsize * fsbuf.f_bfree;
#else
	if( stat( path, &sbuf ))
		return(-1);
	if( ustat( sbuf.st_dev, &ubuf ))
		return(-1);
	free_bytes = BUFSIZ * ubuf.f_tfree;
#endif
	
/*  Return the number of free bytes  */

	return(free_bytes);
}
