/*******************************************************************************
*       geo_to_utm      (see manual, Internal Functions)                <in>   *
*                Original program called latlon_utm or coords.                 *
*                Code from Holly Martin...12/10/85                             *
*                Modified by John Dabritz...3/9/87                             *
*                                                                              *
*                Returns 0 if OK, else returns 1.                              *
*                      C.E.Jonson 7/18/89 (EDS)                                *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <math.h>

long geo_to_utm( x, y, xutm, yutm )
	double	x, 	/** East longitude, decimal degrees.  Input.  **/
		y, 	/** North latitude, decimal degrees.  Input.  **/
		*xutm,  /**  E output coord in UTM meters             **/
		*yutm;  /**  N output coord in UTM meters             **/
{
	long	i3;
	double	slat,
		slon,
		a = 6378206.4,
		b = 6356583.8,
		e2 = .0067686568,
		sec = 4.848136811E-6;	   /*  conversion seconds to radians */
	double	an, an2, an3, an4, an5,
		b2,
		cm,
		cosz, cosz2, cosz3, cosz4, cosz5,
		delta, delta2, delta3, diff,
		ep,  eprim2,
		fa, fb, fc, fd, fe,
		s, sa, sb, sr, st, su, sv,
		sin2z, sin4z, sin6z, sin8z, sinz, sinz2,
		tanz, tanz2, tanz4,
		ve,
		wlon_ref,
		zlat, zlon, zlat_abs;

/*** Ensure that lat_ref and lon ref are set **/
	if( check_llref())
		return(1);
	if(lon_ref < 0.0)
		wlon_ref = -lon_ref;			/* both + and - taken as W lon */
	else
		wlon_ref = lon_ref;
	
/*** Ensure latitude is  0 <= y <= 90  ***/
	for( ; y < 0.0; y += 90.0 );
	for( ; y > 90.0; y -= 90.0 );

/*** Ensure longitude is 0 >= x >= (-180)  ***/
	for( ; x > 0.0; x -= 180.0 ); 
	for( ; x < -180; x += 180.0 );
	x = -x;				/** convt East into West value **/

	slat = y * 3600.0,
	slon = x * 3600.0,

/*** Calculate wlon_ref down to 3 degree units ***/
	i3 = wlon_ref/3.0;		/*** Truncation intended  ***/
  
/*** Fix on odd 3 degree angle ***/
	/*** cm = (i3 - (i3 % 2) * 2 + 1) * 3.0; ***/
	cm = (i3 - (i3 % 2) + 1) * 3.0;
	cm *= .017453292;

/*** Compute x, y in meters ***/
	b2 = b * b;
	eprim2 = (a * a - b2)/b2;
	an = (a - b)/(a + b);
	an2 = an * an;
	an3 = an2 * an;
	an4 = an3 * an;
	an5 = an4 * an;

	fa = a * (1.0 - an + 1.2 * (an2 - an3) + 1.2656250 * (an4 - an5));
	fb = 3.0 * (a/2) * (an - an2 + .875 * (an3 - an4)) + .859375 * an5;
	fc = 15.0 * (a/16) * (an2 - an3 + .75 * (an4 - an5));
	fd = 35.0 * (a/48.0) * (an3 - an4 + .6875 * an5);
	fe = 315.0 * (a/512.0) * (an4 - an5);

	zlat = slat * sec;
	zlon = slon * sec;
	zlat_abs = fabs(zlat);

	sin2z = sin(2.0 * zlat_abs);
	sin4z = sin(4.0 * zlat_abs);
	sin6z = sin(6.0 * zlat_abs);
	sin8z = sin(8.0 * zlat_abs);
	sinz = sin(zlat_abs);
	sinz2 = sinz * sinz;

	cosz = cos(zlat_abs);
	cosz2 = cosz * cosz;
	cosz3 = cosz * cosz2;
	cosz4 = cosz2 * cosz2;
	cosz5 = cosz2 * cosz3;

	tanz = sinz/cosz;
	tanz2 = tanz * tanz;
	tanz4 = tanz2 * tanz2;

	diff = cm - zlon;

	if( fabs(diff) - M_PI > 0 )
		diff = fabs(diff)/diff * -1 * 6.2831853 + diff;

	delta = fabs(diff);
	delta2 = delta * delta;	
	delta3 = delta * delta2;
	
	s = .9996 * (fa * fabs(zlat) - fb * sin2z + fc * sin4z - fd *
		sin6z + fe * sin8z);
	ve = a/sqrt( 1.0 - e2 * sinz2);	
	sr = ve / 2.0 * sinz * cosz * .9996;
	st = ve / 24.0 * sinz * cosz3;
	st *= (5.0 - tanz2 + 9.0 * eprim2 * cosz2 + 4.0 * eprim2 * eprim2 *
		cosz4) * .9996;
	su = ve * cosz * .9996;
	sv = ve / 6.0 * cosz3 * (1.0 - tanz2 + eprim2 * cosz2) * .9996;
	sa = delta3 * delta3/720.0 * ve * sinz * cosz5;
	sa *= (61.0 - 58.0 * tanz2 + tanz4 + 270.0 * eprim2 * cosz2 -
		330.0 * eprim2 * sinz2) * .9996;
	sb = delta3 * delta2 * cosz5/120.0;
	sb *= (5.0 - 18.0 * tanz2 + tanz4 + 14.0 * eprim2 * cosz2 - 58.0 *
		eprim2 * sinz2) * .9996;

	ep = su * delta + sv * delta3 + sb;

	*xutm = ( diff >= 0.0 ) ? 500000.0 + ep : 500000.0 - ep;
	*yutm = s + sr * delta2 + st * delta2 * delta2 + sa;
	if( zlat < 0.0 )
		*yutm = 10000000.0 - *yutm;

/*** utm corrections to match army tm 5-541-11 ***/
	*xutm += 0.4 * cos(y * 0.017453292);
	*yutm += 0.0156 * y;
	
	return(0);
}
