/*******************************************************************************
*       get_digfile     (see manual, Internal Functions)                <in>   *
*                       This program reads the specified digitizer file        *
*                       and loads the diginfo structure.                       *
*                                                                              *
*                       Only the diginfo global structure is changed.          *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  6/19/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include "../../incl/in.h"  

long get_digfile(path)
char *path;
{
	int file; 
	int bytes;
	long ascii_format;
	long xmask_count, ymask_count;
	char line[4096];
	char part1[256], part2[256];
	long sync_col_count, start_count, stop_count, sync_char_count;
	long i, j, k;
	char button_mask[10];          /*  binary pattern used to extract    */
                     /*  the button press value.  BINARY FORMAT USE ONLY     */
	char sync_mask[10];        /*  binary pattern to be used to denote the   */
                     /*  start of strings  BINARY FORMAT ONLY                */
	char xmask[10][32];        	   /*  up to 10 binary patterns  to be   */
                     /*  used to extract the binary values for the x coord.  */
                     /*  BINARY FORMAT USE ONLY                              */
	char ymask[10][32];        	   /*  up to 10 binary patterns  to be   */
                     /*  used to extract the binary values for the y coord.  */
                     /*  BINARY FORMAT USE ONLY                              */

/*    open the digitizer information file                 */
	file = openf(path, READ);
	if(file < 0)
	{	message("Cannot open dig file:  ");
		append_msg(path);
		return(1);
	}

/*** Initialize xmask_count to 1 as xmask_count[0] reserved for the ***/
/*** high bits info.  (same true for ymask_count)                   ***/
	xmask_count = ymask_count = 1;

		/*    zero out structure values      */
	for(i=0; i<10; i++)
	{	diginfo.start[i][0] = 0;
		diginfo.stop[i][0] = 0;
		diginfo.sync_char[i] = 0;
		diginfo.sync_col[i] = 0;
		diginfo.xmask[i] = 0;
		diginfo.ymask[i] = 0;
		diginfo.xmask_bits[i] = 0;
		diginfo.ymask_bits[i] = 0;
	}
	diginfo.baud = 9600;   /* default  */
	diginfo.sync_mask = 0;
	diginfo.length = 0;
	diginfo.button = 0;
	diginfo.x_begin = 0;
	diginfo.x_end = 0;
	diginfo.y_begin = 0;
	diginfo.y_end = 0;
	diginfo.a_begin = 0;
	diginfo.a_end = 0;
	diginfo.hi_x_col = 0;
	diginfo.hi_y_col = 0;
	diginfo.button_mask = 0;
	diginfo.digcursor[0] = 0;
	diginfo.units_per_inch = 0.0;

		/*    read the dig info file 1 record at a time and        */
		/*    place in appropriate diginfo structure element     */
	start_count = stop_count = sync_char_count = sync_col_count = 0;
	bytes = 4095;
	for(; reada(line, bytes, file) ;)
	{
		if( *line == '#' )             /** If true, its a comment **/
			continue;
		cnvrt_bslash(line);
		line[strlen(line)-1] = 0;          /* trim off line feed */
		strsplit(line, '=', part1, part2);
		trim(part1);
			/*  interpret string   */
		if(strstr(line, "="))
		{
				/*  convert \n & \nn & \nnn & \\ groups in part 2  */
			cnvrt_bslash(part2);

				/*   load string into structure  */
			if(!(strcmp(part1,"x_begin"))) 
			{	if(!sscanf(part2,"%d",&(diginfo.x_begin)))
				{	message("Bad x_begin string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"x_end"))) 
			{	if( !sscanf(part2,"%d",&(diginfo.x_end)))
				{	message("Bad x_end string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"y_begin"))) 
			{	if( !sscanf(part2,"%d",&(diginfo.y_begin)))
				{	message("Bad y_begin string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"y_end"))) 
			{	if( !sscanf(part2,"%d",&(diginfo.y_end)))
				{	message("Bad y_end string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"digcursor"))) 
			{	if( !sscanf(part2,"%s",diginfo.digcursor))
				{	message("Bad digcursor string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"format"))) 
			{	if( !sscanf(part2,"%s",diginfo.format))
				{	message("Bad format string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"debounce"))) 
			{	if( !sscanf(part2,"%d", &diginfo.debounce))
				{	message("Bad debounce value:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"button"))) 
			{	if( !sscanf(part2,"%d",&(diginfo.button)))
				{	message("Bad button string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"button_mask"))) 
			{	if( !sscanf(part2,"%s", button_mask))
				{	message("Bad button_mask:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"hi_xmask"))) 
			{	if( !sscanf(part2,"%s", xmask[0]))
				{	message("Bad hi_xmask string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"hi_ymask"))) 
			{	if( !sscanf(part2,"%s", ymask[0]))
				{	message("Bad hi_ymask string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strncmp(part1,"xmask", 5))) 
			{	if( !sscanf(part2,"%s", xmask[xmask_count++]))
				{	message("Bad xmask string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strncmp(part1,"ymask", 5))) 
			{	if( !sscanf(part2,"%s", ymask[ymask_count++]))
				{	message("Bad ymask string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"hi_x_col"))) 
			{	if( !sscanf(part2,"%ld", &diginfo.hi_x_col))
				{	message("Bad hi_x_col value: ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"hi_y_col"))) 
			{	if( !sscanf(part2,"%ld", &diginfo.hi_y_col))
				{	message("Bad hi_y_col value: ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"units_per_inch"))) 
			{	if( !sscanf(part2,"%lf",&(diginfo.units_per_inch)))
				{	message("Bad units_per_inch string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"length"))) 
			{	if( !sscanf(part2,"%d",&(diginfo.length)))
				{	message("Bad length string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"baud"))) 
			{	if( !sscanf(trim(part2),"%ld",&(diginfo.baud)))
				{	message("Bad baud string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if((start_count < 10) && !(strcmp(part1,"start"))) 
				strcpy(diginfo.start[start_count++], part2);
			else if((stop_count < 10) && !(strcmp(part1,"stop"))) 
				strcpy(diginfo.stop[stop_count++], part2);
			else if(!(strcmp(part1,"a_begin"))) 
			{	if( !sscanf(part2,"%d",&(diginfo.a_begin)))
				{	message("Bad a_begin string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"a_end"))) 
			{	if( !sscanf(part2,"%d",&(diginfo.a_end)))
				{	message("Bad a_end string:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if((sync_char_count < 10) && !(strcmp(part1,"sync_char"))) 
			{	switch(strlen(part2)) {
				case 0:   /*  empty, default to a blank    */
					diginfo.sync_char[sync_char_count++] = 32;
					break;
				case 1:   /*  take as a literal character  */
					diginfo.sync_char[sync_char_count++] = part2[0];
					break;
				default:  /*  take as ascii decimal value of a character */
					if(!sscanf(part2,"%c",&(diginfo.sync_char[sync_char_count++])))
					{	message("Bad sync_char decimal value:  ");
						append_msg(part2);
						return(1);
					}
					break;
				}
			}
			else if((sync_col_count < 10) && !(strcmp(part1,"sync_col"))) 
			{	if( !sscanf(part2,"%d",&(diginfo.sync_col[sync_col_count++])))
				{	message("Bad sync_col value:  ");
					append_msg(part2);
					return(1);
				}
			}
			else if(!(strcmp(part1,"sync_mask"))) 
			{	if( !sscanf(part2, "%s", sync_mask ))
				{	message("Bad sync_mask value:  ");
					append_msg(part2);
					return(1);
				}
			}
		}
	}

		/*    close file                                         */
	closef(file);

		/*    check for all required input                       */
	if( diginfo.format[0] == 'b' )
		ascii_format = 0;
	else 		/** Else default to ascii format **/
		ascii_format = 1;

	if( !diginfo.debounce )
		diginfo.debounce = 10;

	if( ascii_format )
	{	if(0>diginfo.x_begin  ||  (diginfo.x_begin>diginfo.x_end))
		{	message("No x_begin in dig file, or x_end < x_begin in dig file.");
			return(1);
		}
		if(0>diginfo.y_begin  ||  (diginfo.y_begin>diginfo.y_end))
		{	message("No y_begin in dig file, or y_end < y_begin in dig file.");
			return(1);
		}
	}
	else		/* In Binary format, low order byte may come first */
	{	if( 0 > diginfo.x_begin )
		{	message("No x_begin in dig file.");
			return(1);
		}
		if( 0 > diginfo.y_begin )
		{	message("No y_begin in dig file.");
			return(1);
		}
	}
	if( 0 > diginfo.button)
	{	message("No button in dig file, or button < 0 in dig file.");
		return(1);
	}
	if(0.0 >= diginfo.units_per_inch)
	{	message("No units_per_inch in dig file, or units_per_inch <= 0 in dig file.");
		return(1);
	}
 	

		/*    check for positive baud setting                  */
	if(diginfo.baud <= 0)
	{	message("Baud setting is < 0 in dig file.");
		return(1);
	}

		/*    check for non-overlapping specifications           */
		/*    Binary is not checked for overlapping, as many     */
		/*    fields DO overlap.  User must be careful. (Ha Ha)  */
	if( ascii_format )
	{	if(diginfo.button >= diginfo.x_begin && diginfo.button <= diginfo.x_end)
		{	message("button column overlaps x_begin/end in dig file.");
			return(1);
		}
		if(diginfo.button >= diginfo.y_begin && diginfo.button <= diginfo.y_end)
		{	message("button column overlaps y_begin/end in dig file.");
			return(1);
		}
		if(diginfo.y_begin >=diginfo.x_begin && diginfo.y_begin <=diginfo.x_end)
		{	message("x_begin/end overlaps y_begin/end in dig file.");
			return(1);
		}
		if(diginfo.y_end >= diginfo.x_begin  &&  diginfo.y_end <= diginfo.x_end)
		{	message("x_begin/end overlaps y_begin/end in dig file.");
			return(1);
		}
	}

	if( ascii_format )
	{	for(i=0; i<10; i++)
		{	if(diginfo.sync_char[i])
			{	if(diginfo.sync_col[i] >= diginfo.x_begin  &&  
						diginfo.sync_col[i] <= diginfo.x_end)
				{	message("sync_col column overlaps x_begin/end in dig file.");
					return(1);
				}
				if(diginfo.sync_col[i] >= diginfo.y_begin  &&  
						diginfo.sync_col[i] <= diginfo.y_end)
				{	message("sync_col column overlaps y_begin/end in dig file.");
					return(1);
				}
				if(diginfo.sync_col[i] == diginfo.button)
				{	message("sync_col column overlaps button in dig file.");
					return(1);
				}
			}
			else
				break;
		}
	
			/*    check for matched numbers of sync_markers          */
		if(sync_char_count != sync_col_count)
		{	message("Number of sync_char and sync_col items not equal.");
			return(1);
		}
	}
	else 
	{	/*    check for matched numbers of sync_markers          */
		if(1 != sync_col_count)
		{	message("Sync column missing from dig file. ");
			return(1);
		}
		if( strlen(button_mask) == 0 )
		{	message("No button_mask in dig file. ");
			return(1);
		}
		/*** Convert the masks to decimal values and store in diginfo ***/
		for( i = 0, j = 7; i < 8; i++, j-- )
		{	if( sync_mask[i] == 061 )
				diginfo.sync_mask |= (1 << j);
			if( button_mask[i] == 061)
				diginfo.button_mask |= (1 << j);
			if( xmask[0][i] == 061 )
			{	diginfo.xmask_bits[0]++;
				diginfo.xmask[0] |= (1 << j);
			}
			if( ymask[0][i] == 061 )
			{	diginfo.ymask_bits[0]++;
				diginfo.ymask[0] |= (1 << j);
			}
		}
		/*** diginfo.xmask[0] reserved for hi_x_col. Same true for ymask ***/
		for( k = 1; k < 10; k++ )
		{	for( i = 0, j = 7; i < 8; i++, j-- )
			{	if( xmask[k][i] == 061)
				{	diginfo.xmask_bits[k]++;
					diginfo.xmask[k] |= (1 << j);
				}
				if( ymask[k][i] == 061)
				{	diginfo.ymask_bits[k]++;
					diginfo.ymask[k] |= (1 << j);
				}
			}
		}
	}
	
		/*    check for positive column numbers.               */
	if(diginfo.x_begin < 1)
	{	message("x_begin < 1.");
		return(1);
	}
	if(diginfo.x_end < 1)
	{	message("x_end < 1.");
		return(1);
	}
	if(diginfo.y_begin < 1)
	{	message("y_begin < 1.");
		return(1);
	}
	if(diginfo.y_end < 1)
	{	message("y_end < 1.");
		return(1);
	}
	if(diginfo.button < 1)
	{	message("button < 1.");
		return(1);
	}
	if( ascii_format )
	{	for(i=0; i<10; i++)
			if(diginfo.sync_char[i])
			{	if((diginfo.sync_col[i] < 1))
				{	message("Sync_col is < 1.");
					return(1);
				}
			}
	}
	else
	{	if(diginfo.sync_mask)
		{	if((diginfo.sync_col[0] < 1))
			{	message("Sync_col is < 1.");
				return(1);
			}
		}
	}

		/*    check for column numbers <= length.              */
	if(diginfo.length)
	{	if(diginfo.x_begin > diginfo.length)
		{	message("x_begin > length");
			return(1);
		}
		if(diginfo.x_end > diginfo.length)
		{	message("x_end > length");
			return(1);
		}
		if(diginfo.y_begin > diginfo.length)
		{	message("y_begin > length");
			return(1);
		}
		if(diginfo.y_end > diginfo.length)
		{	message("y_end > length");
			return(1);
		}
		if(diginfo.button > diginfo.length)
		{	message("button > length");
			return(1);
		}
		if(diginfo.hi_x_col > diginfo.length)
		{	message("hi_x_col > length");
			return(1);
		}
		if(diginfo.hi_y_col > diginfo.length)
		{	message("hi_y_col > length");
			return(1);
		}
		if( ascii_format )
		{	for(i=0; i<10; i++)
				if(diginfo.sync_char[i])
				{	if((diginfo.sync_col[i] > diginfo.length))
					{	message("Sync_col is > length.");
						return(1);
					}
				}
		}
		else
		{	if(diginfo.sync_mask)
			{	if((diginfo.sync_col[0] > diginfo.length))
				{	message("Sync_col is > length.");
					return(1);
				}
			}
		}
	}
	
		/*    reset column numbers to 0 referenced from 1 referenced.  */
	diginfo.x_begin = diginfo.x_begin -1;
	diginfo.x_end = diginfo.x_end -1;
	diginfo.y_begin = diginfo.y_begin -1;
	diginfo.y_end = diginfo.y_end -1;
	diginfo.button = diginfo.button - 1;
	if(diginfo.a_begin)
		diginfo.a_begin = diginfo.a_begin -1;
	if(diginfo.a_end)
		diginfo.a_end = diginfo.a_end -1;
	if(diginfo.hi_x_col)
		diginfo.hi_x_col = diginfo.hi_x_col -1;
	if(diginfo.hi_y_col)
		diginfo.hi_y_col = diginfo.hi_y_col -1;
	if( ascii_format )
	{	for(i=0; i<10; i++)
		{	if(diginfo.sync_char[i])
				diginfo.sync_col[i] = diginfo.sync_col[i] -1;
		}
	}
	else
	{	if(diginfo.sync_mask)
			diginfo.sync_col[0] = diginfo.sync_col[0] -1;
	}

#ifdef SVENCODE	/* preprocessed out as of 11/27/89  */
/*** Calculate the number of bits per mask ***/
	for( i = 0; i < 10; i++ )
	{	if( diginfo.xmask[i] )
		{	for( j = 0; j < 9; j++ )
				if( diginfo.xmask[i] == mask_value[j] )
					diginfo.xmask_bits[i] = j;
		}
		else
			break;
	}
	for( i = 0; i < 10; i++ )
	{	if( diginfo.ymask[i] )
		{	for( j = 0; j < 9; j++ )
				if( diginfo.ymask[i] == mask_value[j] )
					diginfo.ymask_bits[i] = j;
		}
		else
			break;
	}
#endif

	return(0);
}
