/*******************************************************************************
*	get_fkeys	(see manual, Internal Functions)         	<in>   *
*			Reads the string mapping table and loads FNKEYS of     *
*			function key strings into global array fkey_vals.      *
*			Function key 1's string is in fkey_vals[0], etc.       *
*			Note that System V needs to have the "base" function   *
*			keys remapped, whereas XENIX does not.                 *
*			                                                       *
*			Globals Modified:                                      *
*			        fkey_vals                                      *
*			                                                       *
*			Returns zero upon normal end-of-job (EOJ), else        *
*			returns -1 upon error.                                 *
*			CEJ 5/23/89      	                               *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for X11 support */

#include "../../incl/globals.h"


#ifndef LTP_X11      /***** first the 386 PC code *****/

#ifndef SV32
#include <sys/keyboard.h>
#endif
#include <errno.h>

long get_fkeys()
{
	long i, j;
	long str_cnt;
	int fd;
	unsigned char repl_char;
#ifdef SV32
	unsigned char *ucptr1;
	strmap_t fbuf, tmp_fbuf;			/*  Holds function strings  */
#else
	unsigned char fbuf[512];
#endif
	unsigned char *cptr, *end_ptr;

/***  Open the console device  ***/
	if( (fd = openf("/dev/console", O_RDWR ))  == -1 )
	{	perror("get_fkeys():  after open");
		return(-1);
	}

/***  Read in the string mapping table  ***/
	if(ioctl( fd, GIO_STRMAP, fbuf) == (char *)-1)
	{	perror("get_fkeys():  after ioctl");
		return(-1);
	}

#ifdef SV32
/***  For AT&T, the base function keys and the Ctrl function keys are the ***/
/***  same, soooo, the Ctrl function keys are modified to be different!   ***/

	memcpy(orig_fkeys, fbuf, sizeof(fbuf));
	end_ptr = fbuf + sizeof(fbuf);

	/*** Make new function key defns for the first 24 function keys! ***/
	for( ucptr1 = tmp_fbuf, repl_char = 'b', i = 0; i < 24; i++ )
	{	sprintf(ucptr1, "\33O%c", repl_char++);
		ucptr1 += 4;
		if( i == 11 )
			repl_char = 'B';
	}
	
	/*** Read past first 24 func key defns in original func key defns ***/
	for( cptr = fbuf, i = 0; i < 24 && cptr < end_ptr;  cptr++)
		if( *cptr == '\0' )	/* found end of one fkey definition */
			i++;

	/*** Copy in the rest of the original function key definitions! ***/
	for( ; cptr <= end_ptr; ucptr1++, cptr++ )
		*ucptr1 = *cptr;

	/*** Copy the tmp_fbuf into the fbuf array ***/
	memcpy(fbuf, tmp_fbuf, sizeof(tmp_fbuf));
	
	/***  Write out the modified string mapping table  ***/
	if(ioctl( fd, PIO_STRMAP, fbuf) == (char *)-1)
	{	perror("get_fkeys():  after ioctl");
		return(-1);
	}
#endif
				
	/***  Store each function string in global array  ***/
	for( i = 0, cptr = fbuf; i < FNKEYS; i++ )
	{	for( j = 0; *cptr != '\0'; j++, cptr++ )
			fkey_vals[i][j] = *cptr;

		fkey_vals[i][j + 1] = *cptr++;		/* NULL term string  */
	}

	/***  Close the console driver  ***/
	closef(fd);
	return(0);
}


#else    /***** now the X11 code *****/

#include "../../incl/x11.h"
#include <X11/keysym.h>

long get_fkeys()
{
  fkey_vals[0] = XK_F1;
  fkey_vals[1] = XK_F2;
  fkey_vals[2] = XK_F3;
  fkey_vals[3] = XK_F4;
  fkey_vals[4] = XK_F5;
  fkey_vals[5] = XK_F6;
  fkey_vals[6] = XK_F7;
  fkey_vals[7] = XK_F8;
  fkey_vals[8] = XK_F9;
  fkey_vals[9] = XK_F10;
  fkey_vals[10] = XK_F11;
  fkey_vals[11] = XK_F12;
}

#endif         /***** end of X11 code *****/
