/*******************************************************************************
*       get_header      (see manual, Internal Functions)                <in>   *
*                       This program reads the header file and loads the       *
*                       information into the header.h common block.            *
*                                                                              *
*       parameters: char* path;    complete path of header file                *
*                   HEADER *head;  pointer to a HEADER structure               *
*                   long setup;    1 = setup header dependent map globals      *
*                                  0 = don't set up map globals                *
*                                                                              *
*                       Only the header.h common block is changed.             *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  3/7/89 - 5/8/89                              *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */

int   closef();         /*  (int stream)                                    */
char *lowerc();         /*  (char *string)                                    */
int   openf();          /*  (char *path, char *type)                      */
char *reada();          /*  ( char *string, int bytes, int stream )         */
char *strstr();         /*  (char *ref, char *test)                           */
char *trim();           /*  (char *string)                                    */
double strtod();


long get_header(path,head,setup)
char *path;
HEADER *head;
long setup;
{
	char line[4096];
	char part1[256], part2[256];
	int file; 
	int bytes;
	long i;

		/*    open the header file                               */
	if((file = openf(path, READ)) == -1)
		return(1);

		/*    zero out header data                               */
	head->comments[0] = 0;
	head->compress[0] = 0;
	head->contour_interval[0] = 0;
	head->datum[0] = 0;
	head->dictionary[0] = 0;
	head->edgematch[0] = 0;
	head->edge_e[0] = 0;
	head->edge_n[0] = 0;
	head->edge_s[0] = 0;
	head->edge_w[0] = 0;
	head->head1[0] = 0;
	head->head2[0] = 0;
	head->layer_name[0] = 0;
	head->map_ht[0] = 0;
	head->map_wd[0] = 0;
	head->maptype[0] = 0;
	head->menu_set[0] = 0;
	head->org[0] = 0;
	head->quad_unit[0] = 0;
	head->revision_yr[0] = 0;
	head->scale[0] = 0;
	head->search_rad[0] = 0;
	head->source_yr[0] = 0;
	head->state[0] = 0;
	head->stplzn[0] = 0;
	head->template1[0] = 0;
	head->template2[0] = 0;
	head->template3[0] = 0;
	head->udrn[0] = 0;
	head->utmln[0] = 0;
	head->utmlt[0] = 0;

		/*    initialize map_type global   */
	if(setup != 0)
		map_type = 0;

		/*    read the header file 1 record at a time and        */
		/*    place in appropriate head-> element.              */
	bytes = 4095;
	for(; reada(line, bytes, file) ;)
	{
		trim(line);
		strsplit(line, '=', part1, part2);
		trim(lowerc(part1));
		trim(part2);
		if(strstr(line, "="))
		{
			if(!(strcmp(part1,"comments"))) 
				strcpy(head->comments, part2);
			else if(!(strcmp(part1,"compress"))) 
				strcpy(head->compress, part2);
			else if(!(strcmp(part1,"contour_interval"))) 
				strcpy(head->contour_interval, part2);
			else if(!(strcmp(part1,"datum"))) 
				strcpy(head->datum, part2);
			else if(!(strcmp(part1,"dictionary"))) 
				strcpy(head->dictionary, part2);
			else if(!(strcmp(part1,"edgematch"))) 
				strcpy(head->edgematch, part2);
			else if(!(strcmp(part1,"edge_e"))) 
				strcpy(head->edge_e, part2);
			else if(!(strcmp(part1,"edge_n"))) 
				strcpy(head->edge_n, part2);
			else if(!(strcmp(part1,"edge_s"))) 
				strcpy(head->edge_s, part2);
			else if(!(strcmp(part1,"edge_w"))) 
				strcpy(head->edge_w, part2);
			else if(!(strcmp(part1,"head1"))) 
				strcpy(head->head1, part2);
			else if(!(strcmp(part1,"head2"))) 
				strcpy(head->head2, part2);
			else if(!(strcmp(part1,"layer_name"))) 
				strcpy(head->layer_name, part2);
			else if(!(strcmp(part1,"map_ht"))) 
				strcpy(head->map_ht, part2);
			else if(!(strcmp(part1,"map_wd"))) 
				strcpy(head->map_wd, part2);
			else if(!(strcmp(part1,"maptype"))) 
			{	strcpy(head->maptype, part2);
				if(strstr(part2,"point") || strstr(part2,"node"))
					map_type |= 1;
				if(strstr(part2,"line") || strstr(part2,"seg"))
					map_type |= 2;
				if(strstr(part2,"poly") || strstr(part2,"area"))
					map_type |= 4;
			}
			else if(!(strcmp(part1,"menu_set"))) 
				strcpy(head->menu_set, part2);
			else if(!(strcmp(part1,"org"))) 
				strcpy(head->org, part2);
			else if(!(strcmp(part1,"quad_unit"))) 
				strcpy(head->quad_unit, part2);
			else if(!(strcmp(part1,"revision_yr"))) 
				strcpy(head->revision_yr, part2);
			else if(!(strcmp(part1,"scale"))) 
				strcpy(head->scale, part2);
			else if(!(strcmp(part1,"search_rad"))) 
				strcpy(head->search_rad, part2);
			else if(!(strcmp(part1,"source_yr"))) 
				strcpy(head->source_yr, part2);
			else if(!(strcmp(part1,"state"))) 
				strcpy(head->state, part2);
			else if(!(strcmp(part1,"stplzn"))) 
				strcpy(head->stplzn, part2);
			else if(!(strcmp(part1,"template1"))) 
				strcpy(head->template1, part2);
			else if(!(strcmp(part1,"template2"))) 
				strcpy(head->template2, part2);
			else if(!(strcmp(part1,"template3"))) 
				strcpy(head->template3, part2);
			else if(!(strcmp(part1,"udrn"))) 
				strcpy(head->udrn, part2);
			else if(!(strcmp(part1,"utmln"))) 
				strcpy(head->utmln, part2);
			else if(!(strcmp(part1,"utmlt"))) 
				strcpy(head->utmlt, part2);
		}
	}
		/*    close file                                         */
	closef(file);

		/*    if setup parameter is zero, then return    */
	if(!setup)
		return(0);

		/************ check map_type global ************************/
	map_type = (map_type) ?  map_type : 4;

		/************ set search radius global *********************/
	search_radius = 0;
	if(!sscanf(head->search_rad,"%ld",&search_radius))
		search_radius = 5;        /*   default  */
	if(!search_radius)
		search_radius = 5;        /*   default  */

		/************ set the stpln_params globals *****************/
		/*  & the stateplane corners in the corner global          */
	if(*head->stplzn)
	{	state_plane_zone = 0;
		if(sscanf(head->stplzn,"%d",&state_plane_zone))
		{
			if(!stpln_data(state_plane_zone))
			{	
				for(i=0; i<4; i++)
				{	corner[3][i][0] = corner[1][i][0];
					corner[3][i][1] = corner[1][i][1];
					if(geo_to_stpln(corner[3][i][0],corner[3][i][1],
						&corner[3][i][0], &corner[3][i][1]) )
					{	corner[3][i][0] = 0.0;
						corner[3][i][1] = 0.0;
					}
				}
			}
			else
				state_plane_zone = 0;      /* none */
		}
	}

		/************ set the lat_ref & lon_ref globals ***/
	check_llref();

		/************  set map scale global ***********/
	map_scale = strtod(head->scale, (char **)NULL);

		/************  set map inch registration if missing ***********/
	if(job_stats[0])     /*** map already loaded ***/
	{	if((map_scale > 0.0)  &&  !registered(6))
		{	if(registered(2))
				set_mapin(2);
			else if(registered(3))
				set_mapin(3);
		}
	}
	
	return(0);
}
