/*******************************************************************************
*       get_penfile     (see manual, Internal Functions)                <in>   *
*            Opens and reads a penfile.                                        *
*                                                                              *
*            Parameters:   char *path;   complete file/path name of penfile    *
*                          long *penmask; long integer array ([8]) of mask     *
*                                         values for 8 pens                    *
*                                                                              *
*                       no global or file data is changed.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns -1 upon failure.                               *
*                      J.Dabritz 7/9/90                                        *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for multi user support */

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <string.h>
#include <sys/stat.h>

char *reada();

long get_penfile(path, penmask)
	char *path;                   /*** pathname of file to get ***/
 	long *penmask;                /*** ptr to 8 penmask longs  ***/
{
	char string[256];
	char *ptr;

	int         fd;

	long pen, set;

	struct stat buf;


/*** 0 out penmasks **************/
	for(pen=0; pen<8;pen++)
		penmask[pen] = 0;

		/*** set path ***/
	trim(path);
	if(*path == '/')
		sprintf(string,"%s",path);
	else {
#ifdef MULTIUSER
		sprintf(string,"%s/.ltp/pens/%s",home_path,path);
		if (!file_exists(path,""))
			sprintf(string,"%s/pens/%s",ltp_path,path);
#else
		sprintf(string,"%s/pens/%s",ltp_path,path);
#endif
	}
		/*** Open the file and read it ***/
	if( (fd = openf(string, RDWR)) == -1 )
		return(-1);

		/*** read file ***/
	set = 0;
	for(; reada(string, 255, fd) != (char *) 0; )
	{	string[256] = (char) 0;                /*** null terminate string ***/
		trim(string);                          /*** trim off blanks ***/
		for(ptr=string; *ptr; ptr++)           /** check all chrs **/
		{	if(*ptr == '#')
				break;
			pen = (*ptr) - '0';                 /*** calculate pen number **/
			if((pen >= 0) && (pen <= 8))       /*** check for legal pen number **/
			{	if((pen >= 1) && (pen <= 8))  
				{	penmask[pen-1] |= (1 << set); /** set penmask **/
				}
				set++;                           /*** increment set number  **/
				if(set >= 32)
					break;
			}
		}
		if(set >= 32)
			break;
	}
	
		/**** close file ***/
	closef(fd);

		/**** return ****/
	return(0);
}
