/*******************************************************************************
*   ginput_setup   (see manual, Internal Functions)             <in>           *
*           Closes existing graphics input device (if any),                    *
*           gets new ginfo structure information (if new device is digitizer), *
*           opens and starts new device.                                       *
*                                                                              *
*           Globals Modified:  fd_ginput, gr_dev, diginfo.                     *
*                                                                              *
*           Returns 0 for success, 1 for failure.                              *
*                                                                              *
*           J.Dabritz, 6/19/89, including code by C.Jonson 5/89.               *
*******************************************************************************/
/* Modified 7/92 by DBS@CWU for X11 support and multi user support*/

#include <sys/ioctl.h>  
#include <string.h>  
#include "../../incl/globals.h"

long mse_setup();
long dig_setup();

ginput_setup( new_dev )
long new_dev;
{
	char path[512];

		/******************* check input parameter *******************/
	switch(new_dev)  {
	case DIG1ABS:
	case DIG1REL:
	case DIG2ABS:
	case DIG2REL:
#ifndef LTP_X11
	case LTSMSE:
	case MSSMSE:
	case ATTBMSE:
#else
	case X11MSE:
#endif
	case 0:
		break;
	default:
		message("Not a legal graphics device");
		return(1);
	}

		/******************* close out old device ********************/
	if(fd_ginput >= 0)
		ginput_reset();

		/******************* start new device ************************/
	switch(new_dev)  {
	case DIG1ABS:
	case DIG1REL:
	case DIG2ABS:
	case DIG2REL:
			/********* get path of diginfo file ********/
		if(new_dev == DIG1ABS  ||  new_dev == DIG1REL)
#ifdef MULTIUSER
			sprintf(path,"%s/.ltp/digitizers/%s",home_path,config.dig1_type);
			if (!file_exists(path,""))
				sprintf(path,"%s/digitizers/%s",ltp_path,config.dig1_type);
#else
			sprintf(path,"%s/digitizers/%s",ltp_path,config.dig1_type);
#endif
		else
#ifdef MULTIUSER
			sprintf(path,"%s/.ltp/digitizers/%s",home_path,config.dig2_type);
			if (!file_exists(path,""))
				sprintf(path,"%s/digitizers/%s",ltp_path,config.dig2_type);
#else
			sprintf(path,"%s/digitizers/%s",ltp_path,config.dig2_type);
#endif

			/********* read diginfo file ***************/
		if(get_digfile(path))
		{	gr_dev = 0;         /* none  */
			return(1);
		}

			/********* get dig device/file path ********/
		if(new_dev == DIG1ABS  ||  new_dev == DIG1REL)
			sprintf(path,"%s",config.dig1_adr);
		else
			sprintf(path,"%s",config.dig2_adr);


			/********* initialize device ***************/
		gr_dev = new_dev;
		if(dig_setup(path))
		{
			gr_dev = 0;      /* none */
			return(1);
		}
		break;

#ifndef LTP_X11
	case LTSMSE:
	case MSSMSE:
	case ATTBMSE:
		if(mse_setup(new_dev))
		{	gr_dev = 0;       /* none */
			return(1);
		}
		break;
#else
	case X11MSE:
		break;
#endif
	case 0:
		fd_ginput = -1;
		gr_dev = 0;
		return(1);
	default:
		gr_dev = 0;
		return(1);
	}

		/******************* set gr_dev global to input code *********/
	gr_dev = new_dev;

		/******************* return **********************************/

	return(0);
}



/*******************************************************************************
*	mse_setup      (see manual, Internal Functions)            	<in>   *
*           Starts output for a serial mouse or the ATT busmouse.              *
*           The original line attributes are not saved as the port is          *
*           port is closed after processing.                                   *
*                                                                              *
*			Globals Modified:                                      *
*			        fd_ginput                                      *
*                                                                              *
*			Returns 0 for success, 1 for failure.                  *
*			C.Johnson, modified by J.Dabritz  6/19/89              *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for X11 support */

long mse_setup(new_dev)
long new_dev;
{
	struct termio termvar;
	char char_buf[4];
	char tmp_msg[STRLIST];
	char ckeypath[256];

#ifndef LTP_X11   /***** this switch statement not needed for X11 support *****/
	switch( new_dev )
	{
	case LTSMSE:		/*  Serial mouse is used       */

			/***** Open the port with mouse attached to it  */
		if( (fd_ginput = openf( config.mouse_adr, O_RDWR | O_NDELAY )) == -1 )
		{	message("Cannot open serial mouse device file.");
			return(1);
		}

			/**** Get and set input attributes of mouse port for serial mouse */
		if( ioctl( fd_ginput, TCGETA, &termvar) == (char *)-1 )
		{	message("Cannot do ioctl #1 on serial mouse file device");
			return(1);
		}
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B1200; 
		if( ioctl( fd_ginput, TCSETAF, &termvar) == (char *)-1 )
		{	message("Cannot do ioctl #2 on serial mouse file device");
			return(1);
		}

			/**** Set mouse mode.  This is for LOGITECH C7 mouse.  */
		char_buf[0] = 0x2a;		/*	Ask for 9600 baud */
		char_buf[1] = 0x71;		/*  Ask for 9600 baud cont.  */
	/*	if( writeb( fd_ginput, char_buf, 2 ) == 0 )	*/
		if( writeb( char_buf, 2, 1, fd_ginput ) == 0 )
		{	message("Cannot do write #1 to logitech serial mouse");
			return(1);
		}

			/**** Allow time for syncronization  */
		sleep_ltp((double) 0.2);

			/****Set input attributes of mouse port (increase baud rate)  */
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B9600;
		termvar.c_lflag = 0;
		if( ioctl( fd_ginput, TCSETAF, &termvar) == (char *)-1 )
		{	message("Cannot do ioctl #3 on serial mouse file device");
			return(1);
		}
		sleep_ltp((double) 0.2);

			/*	char_buf[0] = 0x4f;		/*  Ask for continuous reports */
		char_buf[0] = 0x4f;		/*  Ask for continuous reports */
	/*	if( writeb( fd_ginput, char_buf, 1) == 0 )	*/
		if( writeb( char_buf, 1, 1, fd_ginput ) == 0 )
		{	message("Cannot do write #2 to logitech serial mouse");
			return(1);
		}

		char_buf[0] = 0x42;	 /*  Specify relative bit pad one packed binary */
	/*	if( write( fd_ginput, char_buf, 1) == 0 )	*/
		if( writeb( char_buf, 1, 1, fd_ginput ) == 0 )
		{	message("Cannot do write #3 to logitech serial mouse");
			return(1);
		}

			/**** Allow additional time for the mouse to sync  */
		sleep_ltp((double) 0.2);

			/**** Flush the input buffer prior to any reads  */
		ioctl( fd_ginput, TCFLSH, 2);


		break;
	case ATTBMSE:		/*  AT&T/Logitech bus mouse     */
#ifdef SV32
		mouse_open();
		fd_ginput = mousfd;
		break;
#else
		message("No Busmouse driver on non-ATT machines/code.");
		return(1);
#endif
	case MSSMSE:         /*  microsoft serial mouse      */

			/***** Open the port with mouse attached to it  */
		if( (fd_ginput = openf( config.mouse_adr, O_RDWR | O_NDELAY )) == -1 )
		{	message("Cannot open serial mouse device file.");
			return(1);
		}

			/**** Get and set input attributes of mouse port for serial mouse */
		if( ioctl( fd_ginput, TCGETA, &termvar) == (char *)-1 )
		{	message("Cannot do ioctl #1 on serial mouse file device");
			return(1);
		}
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B1200; 
		if( ioctl( fd_ginput, TCSETAF, &termvar) == (char *)-1 )
		{	message("Cannot do ioctl #2 on serial mouse file device");
			return(1);
		}

			/**** Allow additional time for the mouse to sync  */
		sleep_ltp((double) 0.2);

			/**** Flush the input buffer prior to any reads  */
		ioctl( fd_ginput, TCFLSH, 2);

		break;
	}
#endif /***** above not needed for X11 *****/

	/******* Set up the cursor_keys for the mouse ***********/
#ifdef MULTIUSER
	sprintf(ckeypath,"%s/.ltp/cursor_keys/%s",home_path,config.cursor_file);
	if(!file_exists(ckeypath, ""))
		sprintf(ckeypath, "%s/cursor_keys/%s", ltp_path, config.cursor_file);
#else
	sprintf(ckeypath, "%s/cursor_keys/%s", ltp_path, config.cursor_file);
#endif
	if( get_ckeys(ckeypath))
	{	sprintf(tmp_msg, "Cannot load %s cursor_keys file.  Use ckeys_get.");
		message(tmp_msg);
	}

	return(0);
}



/*******************************************************************************
*	dig_setup      (see manual, Internal Functions)            	<in>   *
*           Starts output for a serial digitizer as specified in the           *
*           digitizer config file.                                             *
*           The original line attributes are not saved as the port is          *
*           port is closed after processing.                                   *
*                                                                              *
*			Globals Modified:                                      *
*			        fd_ginput                                      *
*                                                                              *
*			Returns 0 for success, 1 for failure.                  *
*			                       J.Dabritz  6/19/89              *
*******************************************************************************/

long dig_setup(path)
char *path;
{
	struct termio termvar;
	long i;
	long cursorx, cursory;
	char tmp_msg[STRLIST];
	char key_path[256];

		/***** Open the digitizer file device ***********/
	if( (fd_ginput = openf( path, O_RDWR | O_NDELAY )) == -1 )
	{	sprintf(tmp_msg, "Cannot open %s.  Check the digX_adr field \
in the config file.", path);
		message(tmp_msg);
		sprintf(tmp_msg, "Ensure %s has read write permissions (0666).", path);
		message(tmp_msg);
		return(1);
	}

		/**** Get and set input attributes of mouse port for digitizer */
	if( ioctl( fd_ginput, TCGETA, &termvar) == (char *)-1 )
	{	message("Cannot do ioctl #1 on digitizer file device");
		return(1);
	}
	switch(diginfo.baud) {
	case 1200:
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B1200; 
		break;
	case 1800:
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B1800; 
		break;
	case 2400:
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B2400; 
		break;
	case 4800:
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B4800; 
		break;
	case 9600:
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B9600; 
		break;
	case 19200:
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B19200; 
		break;
	case 38400:
		termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B38400; 
		break;
	}
	if( ioctl( fd_ginput, TCSETAF, &termvar) == (char *)-1 )
	{	message("Cannot do ioctl #2 on digitizer file device");
		return(1);
	}
	sleep_ltp((double) 0.2);

		/**** Set digitizer mode.  */
	for(i=0; i<10; i++)
	{	if( diginfo.start[i][0] )
		{	if( writea( diginfo.start[i], fd_ginput) == -1 )
			{	message("Cannot do write #1 to digitizer");
				return(1);
			}
			sleep_ltp((double) 0.2);
		}
		else
			break;
	}

		/**** Allow additional time for the digitizer to sync  */
	sleep_ltp((double) 0.2);

		/**** Flush the input buffer prior to any reads  */
	ioctl( fd_ginput, TCFLSH, 2);

		/*** Swap in the ckeys file for the digitizer ***/
#ifdef MULTIUSER
	sprintf(key_path, "%s/.ltp/cursor_keys/%s",home_path,diginfo.digcursor);
	if(!file_exists(key_path,""))
		sprintf(key_path, "%s/cursor_keys/%s", ltp_path, diginfo.digcursor);
#else
	sprintf(key_path, "%s/cursor_keys/%s", ltp_path, diginfo.digcursor);
#endif
	if(get_ckeys(key_path))
	{	sprintf(tmp_msg, "Cannot load %s cursor_keys file.  Use ckeys_get.");
		message(tmp_msg);
	}

		/**** save cursor position                                 */
	cursorx = ginfo.scoord[0];
	cursory = ginfo.scoord[1];
		/** do an initial read on the digitizer to update dig_input variables*/
	/***  sleep(1);  ***/
	for(i=0; i<5; i++, sleep_ltp((double) 0.2))
		dig_input();
		/**** update cursor to prior position                      */
	ginfo.scoord[0] = cursorx;
	ginfo.scoord[1] = cursory;
	dig_input();

	return(0);
}
