/*******************************************************************************
*       graphics_on     (see manual, Internal Functions)                <in>   *

     Purpose:
	put the vga card into mode 12 hex             
	set bitplane pointers
	clear screen
	put on basic screen

     Arguments:	- none -

     Returned Values: (long) 0=ok,    -1=failed

     Global Variables Modified:   char *vga_plane   (pointer to bit plane)
		windowxmin, windowxmax, windowymin, windowymax

*                      j.dabritz  3/7/89 - 5/8/89                              *
*******************************************************************************/
/*
    Modification history:
        Sept 1990 by Mandel
            Inserted #include statement to make function work in SCO UNIX.
*/
/* Modified 3/92 by DBS@CWU for X11 support */

#ifdef SCO_UNIX
#include <sys/machdep.h>
#endif

#include "../../incl/globals.h"

#ifndef LTP_X11  /***** first the VGA Code *****/

#include <sys/ioctl.h>
#include <malloc.h>
#define PAGESIZE 4096 

long swplane();      /* (long planenum)                                  */
long bar_on();
long bar_off();
long box_on();
long bitmap();

long graphics_on()
{
	long mem64k = 0x10000;
	long i;      				/*    counter       */
	char *ioctl();

/*   check to see if grahpics are being used.  */
	if(batch_mode)
		return(0);


#ifdef SV32			/*  For AT&T System V 3.2 ver 2.1 or later  */
/* If busmse used, reset before using  */
/*** New code as of 11/30/89. This will set up a graphics_device if present **/
	if( gr_dev )
		ginput_setup(gr_dev);

/*** End of new code ****/

/***************   Commented out as of 11/30/89 ******
	if( gr_dev == ATTBMSE )
		ginput_setup(ATTBMSE);
*****************************************************/

/*** Open the console driver for I/O ***/
	if( (fdvga = open("/dev/console", O_WRONLY)) == -1)
	{	message("\nError opening VGA - Check if installed\n");
		return(-1);
	}

/*** put the vga card into mode 12 hex             ***/
	ioctl(fdvga,SW_VGA640x480E,0);

	/* Map VGA address space to pointer plane */
	/*** Set kdm.vaddr to an address on a 4k boundary within a buffer ***/
	/*** that has been allocated.  Add an extra 4k in case the malloced **/
	/*** area isn't on a 4k boundary.                                  ***/

	kdm.vaddr = (char *)((unsigned long)malloc_ltp(mem64k + PAGESIZE) &
			0xFFFFF000) + PAGESIZE;
	if(kdm.vaddr == (char *) 0)
	{	message("Cannot allocate memory for VGA plane.  Aborting.");
		kb_reset();
		graphics_off();
		return(-1);
	}
	kdm.physaddr = (char *)0xa0000;
	kdm.length = mem64k;
	kdm.ioflg = 1;		/*** enable i/o requests ***/
	ioctl(fdvga, KDMAPDISP, &kdm);
	
	vga_plane = kdm.vaddr;

#else				/*  For XENIX 2.3 or later                 */
	/* put the vga card into mode 12 hex             */
	fdvga = open("/dev/vga",O_WRONLY);
	if (fdvga == -1)
	{	message("\nError opening VGA - Check if installed\n");
		return(-1);
	}
	ioctl(fdvga,SW_VGA12,0);

	/* Map VGA address space to pointer plane */

	vga_plane = ioctl(fdvga,MAPVGA,0);
#endif

	/* Set initial bit plane to use - Needed since above
	IOCTL call does not set the initial address correctly - between 0 and 3*/
	swplane(0);
	
	/* set the initial 16 screen colors              */
	for(i=0; i<16; i++)
		colorvga(i,colors[i]);

	/* reset all bit planes                          */
	for(i=0; i<4; i++)
	{	swplane(i);
		bitmap(BITPLN_WIDTH,480,BITPLN_WIDTH,BITPLN_WIDTH,scratch_pad[i],vga_plane);
	}

	/* set flag for graphics_off */
	graphics_flag = 1;
	
	return(0);
}

/******************************************************************/
#else    /*****  now the X11 code  *****/

#include "../../incl/x11.h"
long bitmap4x();

long graphics_on()
{
  static int initialized = 0;
  int x = 0, y = 0;
  int i;

/* printf("graphics_on(): just inside the function\n"); */
  /*   check to see if graphics are being used.  */
  if(batch_mode)
    return(0);

  if (initialized) {
    /* display the window */
    XMapWindow(xdisplay, xmainwin);

    /* wait for the first expose event */
    while (xevent.type != Expose) XNextEvent(xdisplay, &xevent);

    /* restore graphics to screen   */
    for(i=0; i<4; i++) {
      swplane(i);
      bitmap4x(WIDTH,479,0,0,xmainpix,0,0,xmainwin);
    }

    /* set flag for graphics_off */
    graphics_flag = 1;
  }
  else {
    
    /* connect to server */
    if ((xdisplay=XOpenDisplay(xdisplay_name)) == NULL) {
      fprintf(stderr, "ltplus: cannot connect to X server %s\n",
	      XDisplayName(xdisplay_name));
      return(-1);
    }

/* printf("graphics_on(): calling some macros\n"); */
    xscreen = DefaultScreen(xdisplay);
    xdisplay_width = DisplayWidth(xdisplay, xscreen);
    xdisplay_height = DisplayHeight(xdisplay, xscreen);

/* printf("graphics_on(): calling XCreateSimpleWindow()\n"); */
    xmainwin = XCreateSimpleWindow(xdisplay, RootWindow(xdisplay,xscreen),
				  x, y, xwidth, xheight, xborder_width,
				  WhitePixel(xdisplay,xscreen),
				  BlackPixel(xdisplay,xscreen));

/* printf("graphics_on(): calling XCreateBitmapFromData\n"); */
    xicon_pixmap = XCreateBitmapFromData(xdisplay, xmainwin, icon_bitmap_bits,
					icon_bitmap_width, icon_bitmap_height);

    /* initialize the size hints struct */
    xsize_hints.x = x;
    xsize_hints.y = y;
    xsize_hints.width = xwidth;
    xsize_hints.height = xheight;
    xsize_hints.min_width = WIDTH;
    xsize_hints.min_height = HEIGHT;
    xsize_hints.max_width = WIDTH;
    xsize_hints.max_height = HEIGHT;
    xsize_hints.flags = PPosition | PSize | PMinSize | PMaxSize;

    XSetStandardProperties(xdisplay, xmainwin, xwindow_name, xicon_name,
			   xicon_pixmap, NULL, 0, &xsize_hints);

    XSelectInput(xdisplay, xmainwin, ExposureMask | KeyPressMask |
		 ButtonPressMask | StructureNotifyMask | PointerMotionMask);

    /* Load the font */
    if ((xfont_info = XLoadQueryFont(xdisplay, FONT_NAME)) == NULL) {
      fprintf(stderr, "ltplus: Cannot open %s font\n", FONT_NAME);
      return(-1);
    }

    /* set flag for graphics_off */
    graphics_flag = 1;
	
    /* Setup the colormap */
    xdepth = DisplayPlanes(xdisplay, xscreen);
    xvisual = DefaultVisual(xdisplay, xscreen);
/*    xcolormap = DefaultColormap(xdisplay, xscreen); */
    xcolormap = XCreateColormap(xdisplay, xmainwin, xvisual, AllocAll);
    if (xdepth < 4) {     /* too few planes to display properly! */
      fprintf(stderr, "ltplus: Too few bit planes to display graphics: %d\n", xdepth);
      return(-1);
    }
    else {
      for (i=0; i<NCOLORS; i++) colorvga(i, colors[i]);
      for (i=0; i<NPLANES; i++) xplanemasks[i] = 1<<i;
    }
    XSetWindowColormap(xdisplay, xmainwin, xcolormap);

/* printf("graphics_on(): calling XCreatePixmap\n"); */
    /* setup the pixmap used for saving planes into */
    xmainpix = XCreatePixmap(xdisplay, xmainwin, xwidth, xheight, xdepth);

/* printf("graphics_on(): calling get_GC()\n"); */
    /* setup the GCs */
    get_GC(xmainwin, &xgcclear, &xgcxor, &xgcor, &xgccopy, xfont_info);

    /* XInstallColormap(xdisplay, xcolormap); */
    initialized = 1;

/* printf("graphics_on(): calling XMapWindow\n"); */
    /* display the window */
    XMapWindow(xdisplay, xmainwin);

    /* wait for the first expose event */
    while (xevent.type != Expose) XNextEvent(xdisplay, &xevent);

    /* code added to check env variable BITSWAP 
    ** if this variable is set, it will overide the default action
    ** for deciding bitordering for bitmap draws	-dpg
    */
    {
      char *p, *getenv();;

      if (NULL == (p = getenv ("BITSWAP")))
  	BitSwap = BITSWAP_DEFAULT;
      else
      {
  	if (strlen (p) > 0 && (*p == 'f' || *p == 'F'))
	    BitSwap = BITSWAP_FALSE;
	else
	    BitSwap = BITSWAP_TRUE;
      }
    }
  }


/* printf("graphics_on(): leaving the function\n");  */
  return(0);
}


get_GC(win, gcclear, gcxor, gcor, gccopy, font_info)
     Window win;
     GC *gcclear, *gcxor, *gcor, *gccopy;
     XFontStruct *font_info;
{
  unsigned long on_pixel = -1, off_pixel = 0;
  unsigned long valuemask = (GCForeground | GCBackground | GCLineWidth |
			     GCLineStyle | GCCapStyle | GCJoinStyle |
			     GCFont | GCGraphicsExposures);
  XGCValues values;

  values.foreground = on_pixel;
  values.background = off_pixel;
  values.font = font_info->fid;
  values.line_style = LineSolid;
  values.line_width = (unsigned) 1;
  values.cap_style = CapRound;
  values.join_style = JoinRound;
  values.graphics_exposures = False;

  *gcclear = XCreateGC(xdisplay, win, valuemask, &values);
  *gcxor = XCreateGC(xdisplay, win, valuemask, &values);
  *gcor = XCreateGC(xdisplay, win, valuemask, &values);
  *gccopy = XCreateGC(xdisplay, win, valuemask, &values);

  XSetFunction(xdisplay, *gcclear, GXclear);
  XSetFunction(xdisplay, *gcxor, GXxor);
  XSetFunction(xdisplay, *gcor, GXor);
  XSetFunction(xdisplay, *gccopy, GXcopy);
}

#endif  /***** end of X11 code *****/
