/*******************************************************************************
*    icopy         (see manual, Command Ref., Internal Functions)   <in>       *
*            This program finds an inported object & highlights it if          *
*       first_point == 1.  If not, it copies the object to the raster.         *
*                                                                              *
*       files/globals changed:  raster                                         *
*                                                                              *
*       values returned:  long 0= ok,  1=failure                               *
*                                                                              *
*       parameters:    long x,y    raster coordinates                          *
*                                                                              *
*            J.Dabritz   10/16/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"

long hilite();


long icopy(x,y)
	long x,y;
{
	V_OBJECTS vobj;
	COORD_LIST crdlst;

	double dr,dx,dy;
	double rr,rx,ry;

	static long obj_num;
	long l1,l2;
	long ro,rv1,rv2,rl,rf,vf;
	long v,v1,v2;
	long bitpln;
	long x1,y1,x2,y2;


		/************* check for map loaded ************************/
	if(!job_stats[0])
	{	message("You must Create or Acquire a map befor using the icopy command.");
		return(1);
	}

		/************* check level number **************************/
	if((entry_level < 32) || (entry_level > 95))
	{	message("Illegal level number.  Aborting.");
		return(1);
	}

		/************* check for first point ***********************/
	if(first_point)
	{		/********** remove previously highlighted object ********/
		if((obj_num>0)  && (obj_num < job_stats[17]))
			hilite(obj_num,0);

			/********** find import object **************************/
		dx = (double) x;
		dy = (double) y;
		dr = search_radius;
		l1 = entry_level;
		l2 = entry_level;
		vf = 0;
		if(find_vertex(dx,dy,dr,l1,l2,vf,&rr,&rx,&ry,&ro,&rv1,&rv2,&rl,&rf))
			return(1);


			/********** set object number & vertex ******************/
		obj_num = ro;

			/********** highlight object ****************************/

		if(hilite(obj_num,1))
			return(1);
   }
	else   /****** copy import obj_num to the raster ****************/
	{
			/******* unhighlight object *****************************/
		hilite(obj_num,0);

			/******* get v_obj record ******************************/
		if(get_vobjects(&vobj,obj_num))
		{	message("icopy abort, location 1.");
			return(1);
		}

			/******** set obj_num to 0 *****************************/
		obj_num = 0;

			/*** set loop variables ***/
		if(vobj.v1 <= vobj.v2)
		{	v1 = vobj.v1;
			v2 = vobj.v2;
		}
		else
		{	v1 = vobj.v2;
			v2 = vobj.v1;
		}

      	/******************* for bit-planes 2-3 ********************/
         /**** 2 = normal,  2&3 combined = highlight ***/
   	for(bitpln = 2;  bitpln < 4;  bitpln++)
   	{      /******************* switch bit planes *********************/

      	swplane(bitpln);
      	sleep(0);
		
      	if(get_v_coord(&crdlst,v1))
         	return(1);
      	x2 = crdlst.x + 0.5;
      	y2 = crdlst.y + 0.5;
         line_diag(x2,y2,x2,y2,1);

			for(v=v1;  v<=v2; v++)
      	{      /*** plot list of verticies   ***/
      		if(get_v_coord(&crdlst,v))
               return(1);
            x1 = crdlst.x + 0.5;
            y1 = crdlst.y + 0.5;
            line_diag(x1,y1,x2,y2,1);
            x2 = x1;
            y2 = y1;
         }
		}
	}
		
		/************* return **************************************/

	return(0);
}


/*******************************************************************************
*      hilite_vobj                                                             *
*           highlight/unhighlight one vector (import) object                   *
*******************************************************************************/
long hilite(obj_num,action)
	long obj_num;
{
	V_OBJECTS vobj;
	COORD_LIST crdlst;

	long v,v1,v2;
	long bitpln;
	long x1,y1,x2,y2;
	float fx,fy;


		/*********** get v_obj record ************************/
	if(get_vobjects(&vobj,obj_num))
	{	message("icopy abort, location 2");
		return(1);
	}

		/*** set loop variables ***/
	if(vobj.v1 <= vobj.v2)
	{	v1 = vobj.v1;
		v2 = vobj.v2;
	}
	else
	{	v1 = vobj.v2;
		v2 = vobj.v1;
	}

     	/******************* for bit-planes 2-3 (or just 3 or none)  **/
	bitpln = 2;
	if(!action)
	{	if(entry_level < 64)
		{	if(disp_i1hglight & (1 << (entry_level - 32)))
				return(0);
			if(disp_i1normal  & (1 << (entry_level - 32)))
				bitpln = 3;
		}
			else
		{	if(disp_i2hglight & (1 << (entry_level - 64)))
				return(0);
			if(disp_i2normal  & (1 << (entry_level - 64)))
				bitpln = 3;
		}
	}
  	for(;  bitpln < 4;  bitpln++)
  	{      /******************* switch bit planes *********************/

    	swplane(bitpln);
     	sleep(0);
		
     	if(get_v_coord(&crdlst,v1))
     	return(1);
   	fx = crdlst.x;
      fy = crdlst.y;
      rstr_to_scr2(fx,fy,&x2,&y2);

		for(v=v1;  v<=v2; v++)
     	{      /*** plot list of verticies   ***/
     		if(get_v_coord(&crdlst,v))
            return(1);
         fx = crdlst.x;
         fy = crdlst.y;
         rstr_to_scr2(fx,fy,&x1,&y1);
			if(action)
         	line_on(x1,y1,x2,y2);
			else
         	line_off(x1,y1,x2,y2);
         x2 = x1;
         y2 = y1;
		}
	}

	return(0);
}
