/*******************************************************************************
*       index_map       (see manual, Internal Functions)                <in>   *
*                                                                              *
*        Sets up the index_array & index_list files for the                    *
*    nodes/segments/polygons of the map.  The index_array has a row and column *
*    for every 32 rows and columns in the raster.  0,0 is in the lower         *
*    left corner of each.  The index array has a single index value which      *
*    is -1 or the index_list file record number of the first object which is   *
*    within/accross it.  These index_list records form a linked-list for each  *
*    index_array cell.                                                         *
*                                                                              *
*    Arguments:    none                                                        *
*                                                                              *
*    Returned Values: (long) 0 = ok,  1 = failure.                             *
*                                                                              *
*    Global Variables Modified:  none, the job_stats, index_array, and         *
*    index_list files are changed.                                             *
*                      j.dabritz  7/28/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long index_map()
{
	OBJECTS object;
	COORD_LIST coord_list;
	INDEX_LIST index_list;

	char string[256],string2[256];
	char new_path[256];
	char path[256];

	int new_index_list_fd;
	
	long new_idx_ary_recs;
	long rows, cols;
	long c,r,j,i,o;
	long size;
	long last_rec;
	long long_x, long_y;
	long row, col;
	long ix1, iy1, ix2, iy2;
	long iystep, ixstep;
	long vertex;
	long v,v2;

	double x1,y1,x2,y2;
	double dx,dy;
	double slope;
	double xstep, ystep;
	double xenter, yenter;
	double xexit, yexit;

		/********* check for map ****************************************/
	if(!job_stats[0])
	{	message("abort, index_map, location 1");
		return(1);
	}

		/********* create new file **************************************/
		/* attempt to remove file if it exists */
	message("building new index_list file . . .");
	sprintf(string," %s/index_list2",temp_map_path);
	purgef(string);

		/* create new file */
	sprintf(new_path,"%s/index_list2",temp_map_path);
	if(creatf(new_path) < 0)
	{	message("cannot create temp index_list file.  aborting process.");
		return(1);
	}

		/** check for user abort **/
	if(abort_cmd())
		return(1);

		/********* move v_objects index_list records to new file ********/
	new_index_list_fd = openf(new_path,RDWR);
	if(new_index_list_fd <= 0)
	{	message(" cannot open new index_list file.  aborting.");
		return(1);
	}
	new_idx_ary_recs = 0;
		/** write a blank first record so that real recs start at 1 */
	if(writeb((char *)&index_list,index_listm.rec_size,1,new_index_list_fd) < 0)
	{	message("unable to write into new index_list file.  aborting process.");
		return(1);
	}
	new_idx_ary_recs++;
	rows = (job_stats[2] >> 5);
	cols = (job_stats[1] >> 5);
	for(r = 0; r <= rows; r++)
	{		/** check for user abort **/
		if(abort_cmd())
			return(1);

		for(c = 0; c <= cols; c++)
		{	j = index_array[r][c];
			index_array[r][c] = 0;
	/*		next_rec = 0;                   */
			for(;j;)
			{	if(get_indexl(&index_list,j))
				{	message("abort, index_map, location 3");
					return(1);
				}
				j = index_list.next;
				if(index_list.type > 19)
				{	i = index_array[r][c];
					index_array[r][c] = new_idx_ary_recs++;
					index_list.next = i;
					if(writeb((char *)&index_list,index_listm.rec_size,1
					 ,new_index_list_fd) < 0)
					{	message("unable to write into new index_list file.  aborting process.");
						return(1);
					}
				}
			}
		}
	}
	job_stats[15] = new_idx_ary_recs;
	
		/********* remove old file **************************************/
	if(index_list_fd > 0)
	{	closef(index_list_fd);
		index_list_fd = -1;
	}

	if(index_listm.malloc != (char *) 0)
	{	free_ltp(index_listm.malloc);
		index_listm.malloc = (char *) 0;            
	}

	sprintf(string,"%s/index_list",temp_map_path);
	purgef(string);

		/** check for user abort **/
	if(abort_cmd())
		return(1);

		/********* move new file ****************************************/
	if(new_index_list_fd > 0)
		closef(new_index_list_fd);
	sprintf(string,"%s/index_list2",temp_map_path);
	sprintf(string2,"%s/index_list",temp_map_path);
	if(move_file(string,string2))
	{	message("can't move new index_list file.  aborting.");
		return(1);
	}

		/** check for user abort **/
	if(abort_cmd())
		return(1);
	
		/*    open file        */
		/********* check memory & load new file or set pointer to it ****/
	sprintf(path,"%s/index_list",temp_map_path);
	index_list_fd = openf(path,RDWR);
	if(0 >= index_list_fd)
	{	message("  <<  cannot open index_list map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = index_listm.rec_size * job_stats[15];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(index_listm.malloc = (char *) malloc_ltp(size))
			{   index_listm.rec0 = index_listm.malloc;
				index_listm.recn = index_listm.rec0 + size;
				if(!readb(index_listm.rec0,size,1,index_list_fd))
				{	message("  <<  cannot read index_list map file.");
					return(0);
				}
			}
		}
	}

		/********* put nodes into new file ******************************/
		/********* put segments into new file ***************************/
		/********* put polygon centroids in new file ********************/
	last_rec = job_stats[10];
	for(o = 0; o < last_rec; o++)
	{		/** check for user abort **/
		if(abort_cmd())
			return(1);

		if(get_objects(&object,o))
		{	message("abort, index_map, location 4");
			return(1);
		}
		if(!(o % 250))
		{	sprintf(string,"indexing object %d",o);
			message(string);
		}
		if(object.i12[0] < 8)           /* object is a node */
		{		/***** index a node ********/
				/* find index row and column of node */
			long_x = (object.f4[0] + 0.5);
			long_y = (object.f4[1] + 0.5);
			row = long_y >> 5;
			col = long_x >> 5;
				/* set index_list data */
			index_list.type = 0;
			index_list.obj_rec = o;
			index_list.v1 = long_x;   /*  for nodes, v1 and v2 are x & y */
			index_list.v2 = long_y;
			index_list.next = index_array[row][col];
				/* insert object into linked list */      
			index_array[row][col] = job_stats[15];
			if(put_indexl(&index_list,job_stats[15]))
			{	message("abort, index_map, location 5");
				return(1);
			}
		}
		else if(object.i12[0] < 14)     /* object is a segment */
		{		/***** index a segment ********/
				/* set index_list data */
			index_list.type = 1;
			index_list.obj_rec = o;
			index_list.v1 = object.i12[5];

				/***** set beginning vertex data ***/
			if(get_coordlst(&coord_list,object.i12[5]))
			{	message("abort, index_map, location 6");
				return(1);
			}
			x2 = (coord_list.x + 0.5) * 0.03125; /* fractional cell x */
			y2 = (coord_list.y + 0.5) * 0.03125; /* fractional cell y */

				/***** index all verticies in the segment   */
			for(vertex = object.i12[5]; vertex <= object.i12[6]; vertex++)
			{		/*** get vertex coords **/
					/**** increment pt 1 *******/
				x1 = x2;
				y1 = y2;
				ix1 = x1;                       /* integer cell x    */
				iy1 = y1;                       /* integer cell y    */
				if(get_coordlst(&coord_list,vertex))
				{	message("abort, index_map, location 7");
					return(1);
				}
				x2 = (coord_list.x + 0.5) * 0.03125; /* fractional cell x */
				y2 = (coord_list.y + 0.5) * 0.03125; /* fractional cell y */
				ix2 = x2;                            /* integer cell x    */
				iy2 = y2;                            /* integer cell y    */

					/*** reset ending vertex record pointer **/
				index_list.v2 = vertex;

					/*** if in a new grid cell then store index_list record */
				if((ix1 != ix2) || (iy1 != iy2))
				{	dx = x2 - x1;        /*  change in cell coordinates */
					dy = y2 - y1;
						/*** check for more vertical or horizontal vector */
					if(fabs(dy) > fabs(dx))       /* more vertical than hz */
					{		/* calc y slope */
						slope = dx / dy;

							/* set variables for storing by row */
						if(y2 > y1)
							ystep = 1.0;
						else
							ystep = -1.0;
						xstep = ystep * slope;
						if(xstep > 0.0)
							ixstep = 1;
						else
							ixstep = -1;

							/** set enter and exit variables for first col */
						xenter = x1;
						yenter = y1;
						if(y2 > y1)
							yexit = floor(yenter + ystep);
						else
							yexit = floor(yenter);
						xexit = xenter + (yexit - yenter) * slope;

							/*** store path, one row at a time **/
						for(; (iy1 != iy2) || (ix1 != ix2) ; )
						{		/*** store cells for one row **/
							for(;; ix1 += ixstep)
							{		/** exit befor storing last cell **/
								if((ix1 == ix2) && (iy1 == iy2))  
									break;
								  /** store cell record **/
								index_list.next = index_array[iy1][ix1];
								index_array[iy1][ix1] = job_stats[15];
								if(put_indexl(&index_list,job_stats[15]))
								{	message("abort, index_map, location 8");
									return(1);
								}
									/*** reset first vertex **/
								index_list.v1 = index_list.v2-1;
									/*** check for end of vector **/
								if(ix1 == (long) xexit)
									break;
							}
								/*** calc xenter, xexit for next row **/
							xenter = xexit;
							xexit = xenter + xstep;
							if(iy1 != iy2)
								iy1 += ystep;
						}
					}
					else                          /* more horizontal that vt */
					{		/* calc x slope */
						slope = dy / dx;

							/* set variables for storing by col */
						if(x2 > x1)
							xstep = 1.0;
						else
							xstep = -1.0;
						ystep = xstep * slope;
						if(ystep > 0.0)
							iystep = 1;
						else
							iystep = -1;

							/** set enter and exit variables for first row */
						yenter = y1;
						xenter = x1;
						if(x2 > x1)
							xexit = floor(xenter + xstep);
						else
							xexit = floor(xenter);
						yexit = yenter + (xexit - xenter) * slope;

							/*** store path, one col at a time **/
						for(; (iy1 != iy2) || (ix1 != ix2) ; )
						{		/*** store cells for one col **/
							for(;; iy1 += iystep)
							{		/** exit befor storing last cell **/
								if((iy1 == iy2) && (ix1 == ix2))  
									break;
								  /** store cell record **/
								index_list.next = index_array[iy1][ix1];
								index_array[iy1][ix1] = job_stats[15];
								if(put_indexl(&index_list,job_stats[15]))
								{	message("abort, index_map, location 9");
									return(1);
								}
									/*** reset first vertex **/
								index_list.v1 = index_list.v2-1;
									/*** check for end of vector **/
								if(iy1 == (long) yexit)
									break;
							}
								/*** calc yenter, yexit for next col **/
							yenter = yexit;
							yexit = yenter + ystep;
							if(ix1 != ix2)
								ix1 += xstep;
						}
					}
				}
			}
				/*** store index_list record for final vertex **/
			index_list.next = index_array[iy1][ix1];
			index_array[iy1][ix1] = job_stats[15];
			if(put_indexl(&index_list,job_stats[15]))
			{	message("abort, index_map, location 11");
				return(1);
			}
				/*** reset first vertex **/
			index_list.v1 = index_list.v2;
		}
		else                                 /* object is a polygon */
		{		/***** index all polygon centroids (primary & secondary) ********/
				/* find index row and column of primary centroid */
			long_x = (object.f4[0] + 0.5);
			long_y = (object.f4[1] + 0.5);
			row = long_y >> 5;
			col = long_x >> 5;
				/* set index_list data */
			index_list.type = 2;
			index_list.obj_rec = o;
			index_list.v1 = long_x;   /*  for polys, v1 and v2 are x & y */
			index_list.v2 = long_y;
			index_list.next = index_array[row][col];
				/* insert object into linked list */      
			index_array[row][col] = job_stats[15];
			if(put_indexl(&index_list,job_stats[15]))
			{	message("abort, index_map, location 5");
				return(1);
			}
				/** write index_list records for all secondary centroids **/
			if(object.i12[3])
			{	v2 = object.i12[11];
				for(v=object.i12[3]; v<=v2; v++)
				{	if(get_coordlst(&coord_list,v))
					{	message("abort, index_map, location 10");
						return(1);
					}
					long_x = (coord_list.x + 0.5);
					long_y = (coord_list.y + 0.5);
					row = long_y >> 5;
					col = long_x >> 5;
						/* set index_list data */
					index_list.v1 = long_x;   /*  for polys, v1 and v2 are x & y */
					index_list.v2 = long_y;
					index_list.next = index_array[row][col];
						/* insert object into linked list */      
					index_array[row][col] = job_stats[15];
					if(put_indexl(&index_list,job_stats[15]))
					{	message("abort, index_map, location 11");
						return(1);
					}
				}
			}
		}
	}

		/********* set need-to-index flag to 0 **************************/
	job_stats[49] = 0;


		/********* return ***********************************************/
	message("  done.");
	return(0);
}
