/*******************************************************************************
*       index_obj       (see manual, Internal Functions)                <in>   *
*                                                                              *
*        Indexes 1 map object according to the 'action' parmeter.              *
*                                                                              *
*    Arguments:    long action;  0 = remove obj from index,                    *
*                                1 = index object into index_array/index_list  *
*                  long obj_no;  record number of object to be indexed.        *
*                                                                              *
*    Returned Values: (long) 0 = ok,  1 = failure.                             *
*                                                                              *
*    Global Variables Modified:        the job_stats, index_array, and         *
*    index_list files are changed.                                             *
*                      j.dabritz  5/10/90                                      *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long index_delete();

long index_obj(action,obj_no)
{
   OBJECTS object;
   COORD_LIST coord_list;
   INDEX_LIST index_list;

   
   long long_x, long_y;
   long row, col;
   long ix1, iy1, ix2, iy2;
   long iystep, ixstep;
   long vertex;
   long v,v2;

   double x1,y1,x2,y2;
   double dx,dy;
   double slope;
   double xstep, ystep;
   double xenter, yenter;
   double xexit, yexit;

      /********* check for map ****************************************/
   if(!job_stats[0])
   {   message("abort, index_obj, location 1");
      return(1);
   }

      /********* check for objects ************************************/
   if((obj_no < 0) || (obj_no >= job_stats[10]))
   {   message("index_obj:  illegal object number.");
      return(1);
   }

      /********* put object   into new file ***************************/
   if(get_objects(&object,obj_no))
   {   message("abort, index_obj, location 4");
      return(1);
   }


   if(object.i12[0] < 8)           /* object is a node */
   {      /***** index a node ********/
         /* find index row and column of node */
      long_x = (object.f4[0] + 0.5);
      long_y = (object.f4[1] + 0.5);
      row = long_y >> 5;
      col = long_x >> 5;
         /*** check action ****/
      if(action)       /** add to index_list/index_array **/
      {      /* set index_list data */
         index_list.type = 0;
         index_list.obj_rec = obj_no;
         index_list.v1 = long_x;   /*  for nodes, v1 and v2 are x & y */
         index_list.v2 = long_y;
         index_list.next = index_array[row][col];
            /* insert object into linked list */      
         index_array[row][col] = job_stats[15];
         if(put_indexl(&index_list,job_stats[15]))
         {   message("abort, index_obj, location 5");
            return(1);
         }
      }
      else              /** remove from index_list **/
      {      /** search entire list for record number & delete **/
         if(index_delete(row,col,obj_no))
            return(1);
      }
   }

   else if(object.i12[0] < 14)     /* object is a segment */
   {      /***** index a segment ********/
         /* set index_list data */
      index_list.type = 1;
      index_list.obj_rec = obj_no;
      index_list.v1 = object.i12[5];

         /***** set beginning vertex data ***/
      if(get_coordlst(&coord_list,object.i12[5]))
      {   message("abort, index_obj, location 6");
         return(1);
      }
      x2 = (coord_list.x + 0.5) * 0.03125; /* fractional cell x */
      y2 = (coord_list.y + 0.5) * 0.03125; /* fractional cell y */

         /***** index all verticies in the segment   */
      for(vertex = object.i12[5]; vertex <= object.i12[6]; vertex++)
      {      /*** get vertex coords **/
            /**** increment pt 1 *******/
         x1 = x2;
         y1 = y2;
         ix1 = x1;                       /* integer cell x    */
         iy1 = y1;                       /* integer cell y    */
         if(get_coordlst(&coord_list,vertex))
         {   message("abort, index_obj, location 7");
            return(1);
         }
         x2 = (coord_list.x + 0.5) * 0.03125; /* fractional cell x */
         y2 = (coord_list.y + 0.5) * 0.03125; /* fractional cell y */
         ix2 = x2;                            /* integer cell x    */
         iy2 = y2;                            /* integer cell y    */

            /*** reset ending vertex record pointer **/
         index_list.v2 = vertex;

            /*** if in a new grid cell then store index_list record */
         if((ix1 != ix2) || (iy1 != iy2))
         {   dx = x2 - x1;        /*  change in cell coordinates */
            dy = y2 - y1;
               /*** check for more vertical or horizontal vector */
            if(fabs(dy) > fabs(dx))       /* more vertical than hz */
            {      /* calc y slope */
               slope = dx / dy;

               /* set variables for storing by row */
               if(y2 > y1)
                  ystep = 1.0;
               else
                  ystep = -1.0;
               xstep = ystep * slope;
               if(xstep > 0.0)
                  ixstep = 1;
               else
                  ixstep = -1;

                  /** set enter and exit variables for first col */
               xenter = x1;
               yenter = y1;
               if(y2 > y1)
                  yexit = floor(yenter + ystep);
               else
                  yexit = floor(yenter);
               xexit = xenter + (yexit - yenter) * slope;

                  /*** store path, one row at a time **/
               for(; (iy1 != iy2) || (ix1 != ix2) ; )
               {      /*** store cells for one row **/
                  for(;; ix1 += ixstep)
                  {      /** exit befor storing last cell **/
                     if((ix1 == ix2) && (iy1 == iy2))  
                        break;
                       /** store cell record **/
                     index_list.next = index_array[iy1][ix1];
                     if(action)
                     {   index_array[iy1][ix1] = job_stats[15];
                        if(put_indexl(&index_list,job_stats[15]))
                        {   message("abort, index_obj, location 8");
                           return(1);
                        }
                     }
                     else
                     {      /** search entire list for record number & delete **/
                        if(index_delete(iy1,ix1,obj_no))
                           return(1);
                     }
                        /*** reset first vertex **/
                     index_list.v1 = index_list.v2-1;
                        /*** check for end of vector **/
                     if(ix1 == (long) xexit)
                        break;
                  }
                     /*** calc xenter, xexit for next row **/
                  xenter = xexit;
                  xexit = xenter + xstep;
                  if(iy1 != iy2)
                     iy1 += ystep;
               }
            }
            else                          /* more horizontal that vt */
            {      /* calc x slope */
               slope = dy / dx;

                  /* set variables for storing by col */
               if(x2 > x1)
                  xstep = 1.0;
               else
                  xstep = -1.0;
               ystep = xstep * slope;
               if(ystep > 0.0)
                  iystep = 1;
               else
                  iystep = -1;

                  /** set enter and exit variables for first row */
               yenter = y1;
               xenter = x1;
               if(x2 > x1)
                  xexit = floor(xenter + xstep);
               else
                  xexit = floor(xenter);
               yexit = yenter + (xexit - xenter) * slope;

                  /*** store path, one col at a time **/
               for(; (iy1 != iy2) || (ix1 != ix2) ; )
               {      /*** store cells for one col **/
                  for(;; iy1 += iystep)
                  {      /** exit befor storing last cell **/
                     if((iy1 == iy2) && (ix1 == ix2))  
                        break;
                       /** store cell record **/
                     index_list.next = index_array[iy1][ix1];
                     if(action)
                     {   index_array[iy1][ix1] = job_stats[15];
                        if(put_indexl(&index_list,job_stats[15]))
                        {   message("abort, index_obj, location 9");
                           return(1);
                        }
                     }
                     else
                     {      /** search entire list for record number & delete **/
                        if(index_delete(iy1,ix1,obj_no))
                           return(1);
                     }
                        /*** reset first vertex **/
                     index_list.v1 = index_list.v2-1;
                        /*** check for end of vector **/
                     if(iy1 == (long) yexit)
                        break;
                  }
                     /*** calc yenter, yexit for next col **/
                  yenter = yexit;
                  yexit = yenter + ystep;
                  if(ix1 != ix2)
                     ix1 += xstep;
               }
            }
         }
      }
         /*** store index_list record for final vertex **/
      index_list.next = index_array[iy1][ix1];
      if(action)
      {   index_array[iy1][ix1] = job_stats[15];
         if(put_indexl(&index_list,job_stats[15]))
         {   message("abort, index_obj, location 11");
            return(1);
         }
      }
      else
      {      /** search entire list for record number & delete **/
         if(index_delete(iy1,ix1,obj_no))
            return(1);
      }
         /*** reset first vertex **/
      index_list.v1 = index_list.v2;
   }
   else                                 /* object is a polygon */
   {      /***** index all polygon centroids (primary & secondary) ********/
         /* find index row and column of primary centroid */
      long_x = (object.f4[0] + 0.5);
      long_y = (object.f4[1] + 0.5);
      row = long_y >> 5;
      col = long_x >> 5;
         /* set index_list data */
      index_list.type = 2;
      index_list.obj_rec = obj_no;
      index_list.v1 = long_x;   /*  for polys, v1 and v2 are x & y */
      index_list.v2 = long_y;
      index_list.next = index_array[row][col];
         /* insert object into linked list */      
      if(action)
      {   index_array[row][col] = job_stats[15];
         if(put_indexl(&index_list,job_stats[15]))
         {   message("abort, index_obj, location 5");
            return(1);
         }
      }
      else
      {      /** search entire list for record number & delete **/
         if(index_delete(row,col,obj_no))
            return(1);
      }

         /** write index_list records for all secondary centroids **/
      if(object.i12[3])
      {   v2 = object.i12[11];
         for(v=object.i12[3]; v<=v2; v++)
         {   if(get_coordlst(&coord_list,v))
            {   message("abort, index_obj, location 10");
               return(1);
            }
            long_x = (coord_list.x + 0.5);
            long_y = (coord_list.y + 0.5);
            row = long_y >> 5;
            col = long_x >> 5;
               /* set index_list data */
            index_list.v1 = long_x;   /*  for polys, v1 and v2 are x & y */
            index_list.v2 = long_y;
            index_list.next = index_array[row][col];
            if(action)
            {      /* insert object into linked list */      
               index_array[row][col] = job_stats[15];
               if(put_indexl(&index_list,job_stats[15]))
               {   message("abort, index_obj, location 11");
                  return(1);
               }
            }
            else
            {      /** search entire list for record number & delete **/
               if(index_delete(row,col,obj_no))
                  return(1);
            }
         }
      }
   }

      /********* return ***********************************************/
   return(0);
}

/*******************************************************************************
*       index_delete                                                           *
*             removes all references for one map object from one               *
*         index_array pointer / index_list combination.                        *
*                                                                              *
*     Parameters:    long row,col;  the index_array row and column to search.  *
*                    long obj_no;   the object number to delete from the index.*
*                                                                              *
*     Values returned:    0 = ok,  1 = failure.                                *
*         J.Dabritz     5/10/90                                                *
*******************************************************************************/


long index_delete(row,col,obj_no)
long row,col,obj_no;
{
   INDEX_LIST index_list;

   long begin_list;
   long this_rec, last_rec, next;


      /********** check for 0 index_array pointer ********************/
   if(index_array[row][col] == 0)
      return(0);

      /********** set begin_list flag ********************************/
   begin_list = 1;

      /********** search through entire list for 1 index cell ********/
   index_list.next = index_array[row][col];
   
   this_rec = index_list.next;
   last_rec = 0;
   for(; index_list.next; )
   {      /******* get index_list record **********/
      if(get_indexl(&index_list,index_list.next))
         return(1);

         /******* check for object number/type ***/
      if((index_list.type < 3) && (index_list.obj_rec == obj_no)) /* a hit **/
      {      /******* if beginning of list, reset index_array **/
         if(begin_list)
            index_array[row][col] = index_list.next;
            /******* else reset last pointer ********/
         else
         {   next = index_list.next;
            if(get_indexl(&index_list,last_rec))
               return(1);
            index_list.next = next;
            if(put_indexl(&index_list,last_rec))
               return(1);
         }
      }
      else
      {   begin_list = 0;
         last_rec = this_rec;
      }

      this_rec = index_list.next;
   }
   
      /********** return *********************************************/
   return(0);
}
