/*******************************************************************************
*       initialize      (see manual, Internal Functions)                <in>   *
*		This program sets up the graphics mode and many of     *
*		the global variables for Line Trace Plus.                      *
*                                                                              *
*       Parameters 	long malloc_option;   0 = system malloc                *
*                                         1 = internal memory management       *
*                                                                              *
*       Global variables changed:                                              *
*                                                                              *
*	value returned:	(long) 0=ok,   1=initialization failed, abort          *
*                      j.dabritz  3/7/89 - 5/8/89                              *
*******************************************************************************/

#include <string.h>
#include <malloc.h>
#include "../../incl/globals.h"
#include "../../incl/in.h"
#define LISTCOUNT 256

long initialize(malloc_option)
long malloc_option;
{
	long device;
	long i, j;
	long first_time = 1;
	long temp_num;
	unsigned size;
	char menu_set[256];
	char p_cmd[256];
	char path[512];
	char resp[4];
	char temp_list[LISTCOUNT][STRLIST];

/*-------------    output message telling them where they are.   ------------ */
	if(!batch_mode)
	{ 	printf("\n\n                         Welcome to Line Trace Plus.\n");
		printf("                         Version %5.2f Dabritz/Jonson/Martin, \n\n\n",ltp_version);
	}


/*--------------   load the lookup tables array in tlocale   ----------------*/
	if(!batch_mode)
		printf("Loading lookup tables.");
	if( table_maker() )
	{	printf("\n\n error in initialize:  table_maker failed to execute.\n\n");
		return(1);
	}
	if(!batch_mode)
		printf("   Done.\n");


/*--------------------- load config file variables     -----------------------*/
	
	if(!batch_mode)
		printf("Loading configuration variables.");
	if(get_config())
	{	printf("\n\n error in initialize:  get_config failed to execute.\n\n");
		return(1);
	}
	if(!batch_mode)
		printf("   Done.\n");


/*--------------------- load macro names into globals  -----------------------*/
	if(!batch_mode)
		printf("Loading macro names.");
	file_macros();
	if(!batch_mode)
		printf("   Done.\n");



/*--------------------- allocate block of memory for program use -------------*/
	if(malloc_option)
	{	size = 0;                            /** set default size to malloc **/

						/** try to read size to malloc from config **/ 
		if((*config.malloc >= '0')  &&  (*config.malloc <= '9'))
			sscanf(config.malloc,"%ld",&size);

		if(!size)                /** if all else fails, set default size.  **/
			size = 2000000;                   /** default size is 2 megs **/

		malloc_ptr[0] = (char *) malloc(size);    /** try to malloc space ***/
		if(malloc_ptr[0])
			malloc_ptr[1] = (char *) (malloc_ptr[0] + size);
		else
		{	printf("\n\n error in initialize:  insufficient RAM to allocate [malloc] enough memory.\n");
			printf("You should check for other programs running on the system or reduce the memory\n");
			printf("asked for in the ltp/config/config file [ malloc =     ].  Note:  the default\n"); 
			printf("amount is 2000000  [2 megs].  The value in ltp/config/config should not be more\n");
			printf("than 3 megs less than the total RAM [+ swap space] available.\n");
			return(1);
		}
	}
	else
	{	malloc_ptr[0] = (char *) 0;
		malloc_ptr[1] = (char *) 0;
	}



/*--------------------- initialize malloc rec sizes in global structures -----*/
	rasterm.rec_size = 4;
	objectsm.rec_size = 64;  
	coord_listm.rec_size = 8;
	seg_listm.rec_size = 4;
	attributesm.rec_size = 36;
	errorsm.rec_size = 12;
	index_arraym.rec_size = 4;
	index_listm.rec_size = 20;
	v_attributesm.rec_size = 36;
	v_objectsm.rec_size = 16;
	v_coordlistm.rec_size = 8;



/*--------------------- load the function key string mapping table -----------*/

	if(!batch_mode)
	{
		if( get_fkeys() == -1)
		{ 	perror("initialize():  after get_fkeys()");
			return(-1);
		}
	}

/*--------------------- Check for temp** maps         ------------------------*/
	
	if( !batch_mode ) {
#ifdef MULTIUSER  /***** may need to create maps_path *****/
		if (!file_exists(maps_path,""))
			makedir(maps_path);
#endif
		if( (temp_num = dir_list(maps_path, temp_list, LISTCOUNT)) > -1 )
		{       for( i = j = 0; i < temp_num; i++ )
			{	if(( strncmp(temp_list[i], "temp", 4) == 0)  &&
				   (temp_list[i][4] >= '0')  &&  (temp_list[i][4] <= '9'))

				{	if( first_time )
					{	first_time = 0;
						printf("\nThe following temporary maps exist:\n");
					}
					printf("\t%d.  %s\n", j++, temp_list[i]);
				}
			}

			if( !first_time ) 			/*** temp maps found ***/
			{	printf("%s", prompt[250]);
				fflush(stdin);
				scanf("%c", resp);
				trim(lowerc(resp));
				if( resp[0] != 'n' )
				{	for( i = 0; i < temp_num; i++ )
					{	if( strncmp(temp_list[i], "temp", 4) == 0)
							sprintf(p_cmd, "%s/%s", maps_path, temp_list[i]);
						else
							continue;
						if( purgef(p_cmd) )
							printf("\nUnable to purge temp map: %s\n",p_cmd);
					}
					printf("\nTemporary maps have been purged.\n");
					sleep(1);
				}
			}
		}
	}
			

/*--------------------- start trapping keyboard input ------------------------*/

	if(!batch_mode)
	{	if(kb_setup() == -1)
		{	printf("\n\n error when setting up keyboard input.\n");
			return(-1);
		}
	}

/*--------------------- start screen graphics         ------------------------*/
		/*    set the graphics window limits  */
	windowxmin = 224;
	windowymin = 0;
	windowxmax = 623;
	windowymax = 399;

	
	if(!batch_mode)
	{	if(graphics_on())
		{
#ifdef LTP_X11
		    printf("\nCannot start X11 Window.  aborting.\n");
#else
		    printf("\nCannot start VGA grapihcs mode 12 Hex.  aborting.\n");
#endif
		  return(1);
		}

			/**** set up boxes/tiles of screen display ****/
		scrn_display();

			/**** display input mode ***/
		change_mode(0,0);

			/**** display Line Trace Plus logo   ****/
		displogo();
	}
	

/*--------------------- load the cursor keys file into globals ---------------*/
	if(!batch_mode) {
	  if(*config.cursor_file) {
#ifdef MULTIUSER
	    sprintf(path,"%s/.ltp/cursor_keys/%s",home_path,config.cursor_file);
	    if(file_exists(path,"")) {
		get_ckeys(path);
	    }
	    else {
	      sprintf(path,"%s/cursor_keys/%s",ltp_path,config.cursor_file);
	      get_ckeys(path);
	    }
#else
	    sprintf(path,"%s/cursor_keys/%s",ltp_path,config.cursor_file);
	    get_ckeys(path);
#endif
	  }
	}


/*--------------------- start trapping graphics input ------------------------*/

	if(!batch_mode)
	{
		if( !(strcmp(config.ginput,"dig1")))
			device = DIG1REL;
		else if( !(strcmp(config.ginput,"dig2")))
			device = DIG2REL;
		else if( !(strcmp(config.ginput,"mouse")))
		{	
#ifndef LTP_X11
		        if( !(strcmp(config.mouse_type,"ltsmse")))
				device = LTSMSE;
			else if( !(strcmp(config.mouse_type,"attbmse")))
				device = ATTBMSE;
			else if( !(strcmp(config.mouse_type,"mssmse")))
				device = MSSMSE;
			else 
				device = 0;
#else
		        strcpy(config.mouse_type, "x11mse");  /** force the X11 mouse **/
			device = X11MSE;
#endif
		}
		else
			device = 0;
	
		if(ginput_setup(device))
		{	message("Graphics input failed, use ginput command or keyboard.");
			return(0);
		}
		if(device == LTSMSE)
		{	ginput_reset();
			if(ginput_setup(device))
				message("Graphics input failed, use ginput command or keyboard.");
		}
	}


/*--------------------- place cursor                  ------------------------*/
	if(!batch_mode)
	{	ginfo.scoord[0] = 560;
		ginfo.scoord[1] = 80;
		cursor(560,80);
	}


/*--------------------- load initial default menu set ------------------------*/
	trim(config.menuset);
	if(*config.menuset)
		sprintf(menu_set,config.menuset);
	else
		sprintf(menu_set,"poly_menu");
	if(menuset_get(menu_set))
	{	message("error in initialize:  default menus not accessable.");
		sleep(2);
			/*	turn graphics and keyboard trap off   */
		if(!batch_mode)
		{	graphics_off();
			kb_reset();
			return(1);
		}
	}
		


	return(0);
}
