/*******************************************************************************
*        ismacro      (see manual, Internal Functions)            	<in>   *
*            		Determines if a string is the name of a user macro.    *
*        		If true, then the macro is executed.                   *
*        		                                                       *
*        		Parameters:  char*  name of macro                      *
*        		             long   help flag.                         *
*        		Globals Modified:   none                               *
*                                                                              *
*        		Returns 1 if macro is found ( & executed),             *
*                       or 0 if macro not found,                               *
*                       or -1 on error.                                        *
*                       J. Dabritz    1989                                     *
*******************************************************************************/
#include "../../incl/globals.h"
#include <string.h>
#include <sys/stat.h>
#include "../../incl/in.h"

long ismacro( macro, help_flag )
    char * macro;
	long   help_flag;		/** If true, DON'T execute the macro.  **/
{
	char path[512];
	char string[256];

	struct stat buf;

	int mfile;


		/*   check for 0 string length    */
	trim(macro);
	if(!(*macro))
		return(0);

		/*   check to see if file exists  */
#ifdef MULTIUSER
	sprintf(path,"%s/.ltp/macros/%s",home_path,macro);
	if(stat(path,&buf))     /*  return 0 on any stat error (no file etc) */
	{
	  sprintf(path,"%s/%s",macros_path,macro);
	  if(stat(path,&buf))  /*  return 0 on any stat error (no file etc) */
	    return(0);
	}
#else
	sprintf(path,"%s/%s",macros_path,macro);
	if(stat(path,&buf))     /*  return 0 on any stat error (no file etc) */
		return(0);
#endif

	if( help_flag )		/*  only display the macro!   */
		more(path);
	else 
	{		/******* put macro name on history file **********/
		macro[STRLIST-4] = 0;
		if(macro_level)
			sprintf(history[next_hist],"  #%s",macro);
		else
			sprintf(history[next_hist],"#%s",macro);
		message(history[next_hist]);
		next_hist = (next_hist +1) % 512;
		
			/***********   read file executing lines 1 at a time   */
		macro_level++;        /** increment macro level by 1 **/

		append_msg("     ------- execute macro -----|");
		if((mfile = (int)openf(path,READ)) == -1)
			return(0);
		for(; !(exit_program) && !(exit_macro) && (reada(string,STRLIST,mfile) != (char*) 0);)
		{	string[STRLIST-1] = (char) 0;
			trim(string);             /*   trim string from file  */
			if((*string != '#') && *string)
			{	sprintf(raw_buf,"  %s",string);
				raw_buf[STRLIST-1] = (char) 0;
				message(raw_buf);          /*   print string (command) */
				if(fj() == -1)             /*   execute & test for error    */
				{	sprintf(string,"macro command > %s < failed.",raw_buf);
					pause_ltp(1,string);
				}
				if(!batch_mode)
				{	if(abort_cmd())          /*   check for user abort */
					{	exit_macro = 1;
						message("*** macro aborted ***");
					}
				}
			}
		}

		closef(mfile);

		macro_level--;    /** decrement macro_level by 1 ***********/

		if(macro_level)
			sprintf(history[next_hist],"  # macro executed.");
		else
			sprintf(history[next_hist],"# macro executed.");
		message(history[next_hist]);
		next_hist = (next_hist +1) % 512;

	}

	return(1);
}
