/*******************************************************************************
*	kb_setup      (see manual, Internal Functions)            	<in>   *
*	    		Alters stdin to allow raw character processing.        *
*                                                                              *
*	    		Globals Modified:                                      *
*	    		        kb_origflag                                    *
*	    		        kb_tinfo                                       *
*	    		        kb_tsave                                       *
*                                                                              *
*			Returns zero upon normal end-of-job (EOJ) else         *
*			returns -1 upon abnormal end-of-job.                   *
*******************************************************************************/
/* Modified 5/92 by DBS@CWU for X11 support */

#include <errno.h>
#include <sys/ioctl.h>
#include "../../incl/globals.h"

extern int errno;

#ifndef LTP_X11      /* first the original code */

long kb_setup()
{
/*  Check for batch_mode  */
	if(batch_mode)
		return(0);

/*  Get the current flags for stdin  (ie O_RDONLY, O_NDELAY and such)  */

	if( !(kb_origflag = fcntl( 0, F_GETFL )) )
	{
		perror("kb_setup:  error on fcntl");
		return(-1);
	}

/*  Alter the stdin flags so reads are non-blocking  */

	if( fcntl( 0, F_SETFL, O_NDELAY) )
	{
		perror("kb_setup:  error on fcntl");
		return(-1);
	}

/*  Get current line attributes for stdin  */

	if( ioctl(0, TCGETA, &kb_tinfo) == (char *)-1 )
	{
		perror("kb_setup:  error after ioctl");
		return(-1);
	}
	
	kb_tsave = kb_tinfo;

/*  Set line to raw input mode  */

	kb_tinfo.c_iflag &= ~(INLCR | ICRNL | IUCLC | ISTRIP | IXON | BRKINT);
	kb_tinfo.c_oflag &= ~OPOST;
	kb_tinfo.c_lflag &= ~(ICANON | ISIG | ECHO);
	kb_tinfo.c_cc[4] = 1;		/*  Minimum number of chars to read  */
	kb_tinfo.c_cc[5] = 1;		/*  Tenths of a second  */
	
/*  Alter line with flags as set above  */

	if( ioctl(0, TCSETAF, &kb_tinfo) == (char *)-1 )
	{
		perror("kb_setup:  error after ioctl");
		return(-1);
	}
	
	return(0);
}			/*  end of kb_setup()   */

#else   /* now the much simplified X11 code */

long kb_setup()
{
  return(0);
}

#endif 
