/*******************************************************************************
*	km      	(see manual, Internal Functions)            	<in>   *
*	    		Keyboard manager reads keyboard input one character    *
*			per read.  If character read, will "cook" the          *
*			character in the global kb_buf.                        *
*                                                                              *
*			Globals Modified:                                      *
*			        kb_buf                                         *
*                                                                              *
*			Returns 1 if input is available, else                  *
*			returns 0 if no input is available, else               *
*			returns -1 if an error occurred.                       *
*******************************************************************************/
/* Modified 4/92 by DBS@CWU for X11 support */
#include <stdio.h>
#include <string.h>
#include "../../incl/globals.h"

char *strstr();

long km()
{
	long menu_hit();
	long func_hit();
	long func_key();
	long ret_code;

/*  Mainline  */

#ifdef LTP_X11
	switch( xreadkeystroke(kb_buf + kb_nxtwr) )
#else
	switch( readb( (kb_buf + kb_nxtwr), 1, 1, 0) ) 
#endif
	{
	case 1:
		if( kb_cook() )    /*  True if EOL  */
		{
			strcpy( raw_buf, kb_buf );
			swplane(0);
			bar_off( 0,463, 639, 479 );
			dispchar( 29, 0, '_' );
			ret_code = 1;
		}
		else
		{
			swplane(0);
			dispchar( 29, kb_nxtwr -1,
					*(kb_buf + kb_nxtwr - 1) );
			dispchar( 29, kb_nxtwr, '_' );
			ret_code = 0;
				/*   check for a function key press     *
				*    if valid key, 1 returned, else 0  */
			if( *(kb_buf + kb_nxtwr - 1) == '\033' )
				ret_code = func_key();
		}
		break;
	case 0:
		ret_code = 0;
		break;
	case -1:
		ret_code = -1;
		break;
	}
	
	return( ret_code );

}					/*  End of km()  */

#ifndef LTP_X11     /***** first the 386 PC code *****/
long func_key()
{
	char *key;
	long fk = 0;
	long valid_key = FALSE;
	long i;
	long ret_code;
	char tmp[30];

	/*  Complete reading in the control sequence  */
	while( readb( (kb_buf + kb_nxtwr), 1, 1, 0) )
		kb_nxtwr++;
	
	/*  Find the start of the escape sequence  */
	key = strstr( kb_buf, "\033" );

	/*** Check for escape sequence ***/
	for( i = 0; i < FNKEYS; i++ )
		if( !strncmp(key, fkey_vals[i], strlen(fkey_vals[i]) ))
		{	valid_key = TRUE;
			fk = i;
			break;
		}

	/*  If key pressed is not a valid function key, return 0 */

	if( !valid_key )
		return(0);
		
	if( fk < 19 )		/*  Base function keys  */
	{	ret_code = func_hit(fk);
	}
	else if( fk < 24 )
		;		/*  do nada  */  
	else if( fk < 43 )
	{	fk -= 24;
		ret_code = menu_hit(fk);
	}
	else
	{	message("invalid function key pressed");
		ret_code = 0;
	}				/*  Thats all folks  */

	return(ret_code);

}		/*  end of func_key   */
#else    /***** now the X11 code *****/

#  include "../../incl/x11.h"
#  include <X11/keysym.h>
long func_key()
{
  long fk = 0;
  long valid_key = FALSE;
  long i;
  long ret_code;

  if (xkeysym >= XK_F1 && xkeysym <= XK_F35) {
    /*** Check for function keys ***/
    for( i = 0; i < FNKEYS; i++ )
      if(xkeysym == fkey_vals[i]) {
	valid_key = TRUE;
	fk = i;
	if (xevent.xkey.state & ShiftMask) fk += 12;
	if (xevent.xkey.state & ControlMask) fk += 24;
	break;
      }

    /*  If key pressed is not a valid function key, return 0 */
    if( !valid_key )
      return(0);
		
    if( fk < 19 ) {		/*  Base function keys  */
      ret_code = func_hit(fk);
    }
    else if( fk < 24 )
      ;				/*  do nada  */  
    else if( fk < 43 ) {
      fk -= 24;
      ret_code = menu_hit(fk);
    }
    else {
      message("invalid function key pressed");
      ret_code = 0;
    }				/*  Thats all folks  */

    return(ret_code);
  }				/* end of if xkeysym */
}
#endif   /***** end of the X11 code for func_key() *****/

long func_hit( key )
	long key;
{
	
	strncpy(kb_buf,menu_item[current_menu][key],13);
	strcpy( raw_buf, kb_buf );
	swplane(0);
	bar_off( 0,463, 639, 479 );
	dispchar( 29, 0, '_' );
	kb_nxtwr = 0;   

	return(1);
}

long menu_hit( key )
	long key;
{
	
	strncpy(kb_buf,menu_name[key],13);
	strcpy( raw_buf, kb_buf );
	swplane(0);
	bar_off( 0,463, 639, 479 );
	dispchar( 29, 0, '_' );
	kb_nxtwr = 0;   

	return(1);
}

#ifdef LTP_X11

long xreadkeystroke(buf)
     unsigned char *buf;
{
  char tempbuf[20];
  
  if (XCheckMaskEvent(xdisplay, KeyPressMask, &xevent)) {
    XLookupString(&xevent, tempbuf, 20, &xkeysym, NULL);
    if (xkeysym >= XK_F1 && xkeysym <= XK_R15)
      *buf = '\033';  /* if keysym in above range look for func key */
    else if (xkeysym >= XK_Shift_L && xkeysym <= XK_Hyper_R)
      return(0);  /* in this range pretend we didn't get anything... */
    else if (xkeysym == XK_BackSpace || xkeysym == XK_Delete)
      *buf = '\010';
    else
      *buf = *tempbuf;
    
    return (1);
  } /* end of XCheckMaskEvent */
  return (0);
}

#endif
