/*******************************************************************************
*       line_atrbt      (see manual, Internal Functions)                <in>   *
*    assigns attributes to a series of segments on a shortest path from        *
*    node 1 to node 2.                                                         *
*                                                                              *
*    Arguments:    long x,y;      raster coords a search begin point.          *
*                  long r;        search radius (pixels)                       *
*                  long action_flag;  1=find node1 & set anchor                *
*                                     0=find node2 & set attributes            *
*                                                                              *
*    Returned Values: (long) 0 = ok,    1=failure                              *
*                                                                              *
*    Files modified:    objects, attributes.                                   *
*                                                                              *
*                      j.dabritz  6/26/90                                      *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <math.h>

long line_atrbt_abort();
long find_node_ah();

struct netwk { long node_ah;
         long node_bk;
         long seg_bk;
         long list_ah;
         long list_bk;
         float dist_ah;
         float dist_bk;
      } ;

struct netwk **net;
char *malloc_space;
long list_ptr;
float cap_distance;
long cap_node;


long line_atrbt(x,y,r,action_flag)
long x,y,r,action_flag;
{
   OBJECTS object;

   static long node0, node1;

   float fx,fy;
   float mp0, last_mp;
	float sub_dist;

   long node, no_nodes;
   long i;
   long next_node;

      /************* initialize variables ***************************/
   malloc_space = (char *) 0;

      /************* check x & y ************************************/
   if((x<0) || (x>job_stats[1]) || (y<0) || (y>job_stats[2]))
   {   message("line_atrbt:  x & y must both be within the map raster.  no action taken.");
      line_atrbt_abort(1);
      return(1);
   }

      /************* check r ****************************************/
   if(r < 0)
   {   message("line_atrbt:  search radius in header file must be >= 0.  no action taken.");
      line_atrbt_abort(2);
      return(1);
   }

      /************* check action flag ******************************/
   if((action_flag < 0) || (action_flag > 1))
   {   message("line_atrbt:  action flag must be 0-1 inclusive.  no action taken.");
      line_atrbt_abort(3);
      return(1);
   }

      /************* find nearest node ***************************/
   node = find_vnode((float) x, (float) y, (float) search_radius, 0);
   if(node < 0)
   {   message("No node found within search radius.  no action taken.");
      line_atrbt_abort(4);
      return(1);    /** node not found **/
   }
   
      /************* get object record ***************************/
   if(get_objects(&object,node))
   {   message("line_atrbt:  cannot read node record, no action taken.");
      line_atrbt_abort(5);
      return(1);
   }

      /************* check node type *****************************/
   if(object.i12[0] == 5)        /*** simple island node ***/
   {   message("line_atrbt:  cannot use a loop node for network analysis, no action taken.");
      line_atrbt_abort(6);
      return(1);
   }


      /************* if action_flag = 1, find node1 & set anchorx,y */
   if(action_flag)
   {
         /************* set node0 ***********************************/
      node0 = node;
      
         /************* set anchorx,y *******************************/
      fx = object.f4[0] + 0.5;
      fy = object.f4[1] + 0.5;
      anchorx = (long) fx;
      anchory = (long) fy;
   }
   else /************* otherwise, find node2, analize network & asng atrbts. */
	{
         /************* set cap_distance ****************************/
		cap_distance = 10000000000.0;
		cap_node = node0;

         /************* set node1 ***********************************/
      node1 = node;

         /************* set values for anchor ***********************/
      fx = object.f4[0] + 0.5;
      fy = object.f4[1] + 0.5;

         /************* analize network *****************************/
         /************* malloc space & set network ******************/
      no_nodes = job_stats[10];
      malloc_space = calloc_ltp(32,no_nodes);
		if(malloc_space <= (char *) 0)
		{  message("line_atrbt:  cannot get enough RAM from system, no action taken.");
			malloc_space == (char *) 0;
      	line_atrbt_abort(178);
      	return(1);
		}
      net = (struct netwk **) malloc_space;

         /************* initialize network / structures *************/
      for(i=0; i<no_nodes; i++)
      {   net[i] = (struct netwk *) (malloc_space + 4*no_nodes + i*28);
         net[i]->node_bk = -1;
         net[i]->dist_bk = -1.0;
         net[i]->list_ah = -3;  /** flag for not yet in network **/
         net[i]->list_bk = -1;
      }
         
         /************* set first node ******************************/
      list_ptr = -1;
      net[node1]->node_bk = -1;
      net[node1]->dist_bk = 0.0;
      find_node_ah(node1);
      
         /************* search until second node found **************/
      for(;list_ptr != -1;)
      {      /************* set next_node ****************************/
         next_node = net[list_ptr]->node_ah;

            /************* check to see if done *********************/
         if(next_node == node0)
            break;

            /************* reset node_ah, dist_ah, & list_ah/bk (curr node) **/
            /************* & node_bk, dist_bk, seg_bk (node_ah) **/
            /************* update node_ah ***************************/
         if(find_node_ah(next_node))
         {   line_atrbt_abort(8);
            return(1);
         }
            /************* update current node (list_ptr) ***********/
         if(find_node_ah(list_ptr))
         {   line_atrbt_abort(7);
            return(1);
         }
      }

         /************* check to see if done ************************/
      if(next_node != node0)
      {   message("Found ending node, but you can't get there from here!, no action taken");
         line_atrbt_abort(9);
         return(1);
      }


         /************* assign attriubtes ***************************/
      mp0 = mile_post;
		last_mp = mile_post - .1;
		sub_dist = net[node0]->dist_bk;
		i = node0;
		get_units();
      for(i=node0; i!=node1; i = net[i]->node_bk)
      {   /************* reset mile_post *****************************/
         mile_post = mp0 + (sub_dist - net[i]->dist_bk) * unit_factor[3][0];
         if((mile_post - last_mp) < 0.01)
            mile_post = last_mp + 0.01;

         /************* attribute segment ***************************/
         asgn_atrbt(net[i]->seg_bk,atrbt_action,atrbt_string);

         /************* reset last_mp     ***************************/
         last_mp = mile_post;
      }
		mile_post = mp0 + sub_dist * unit_factor[3][0];

         /************* set node0 ***********************************/
      node0 = node;
      
         /************* set anchor **********************************/
      anchorx = (long) fx;
      anchory = (long) fy;
   }

      /************* free malloc space ******************************/
   if(malloc_space > (char *) 01)
      free_ltp(malloc_space);


      /************* return *****************************************/
   return(0);
}


long line_atrbt_abort(n)
long n;
{
   char string[STRLIST];
   
   sprintf(string,"line_atrbt abort, location %d",n);
   message(string);
   
   if(malloc_space > (char *) 0)
      free_ltp(malloc_space);

   return(0);
}





/*******************************************************************************
*   find_node_ah(node_ptr)                                                     *
*         node_ptr is a record number of a node.                               *
*         For this node, the next nearest, unlisted node (if any) is found.    *
*         Its net[] structure is updated:  node_bk, dist_bk, seg_bk            *
*         The origonal node net[] structure is updated:                        *
*            node_ah, dist_ah, list_ah, list_bk                                *
*******************************************************************************/

long find_node_ah(node_ptr)
long node_ptr;
{
   OBJECTS node0, segment;

   long nearest_seg, nearest_node;
   long i, b, seg, s;
   long node_oposite;
   long reset;
   long last_node;
   long branches;
   long list_ah, list_bk;
   
   float dist_min,dist_ah;
   float check_dist;


      /*********** get object record for node ***********************/
   if(get_objects(&node0,node_ptr))
   {   message("line_atrbt:  cannot read node record, no action taken.");
      line_atrbt_abort(10);
      return(1);
   }

      /*********** remove from list, if in list *********************/
   list_ah = net[node_ptr]->list_ah;
   list_bk = net[node_ptr]->list_bk;
	if(list_ptr == node_ptr)
      list_ptr = list_ah;

   if((list_ah >= 0)  ||  (list_bk >= 0))
   {   if(list_ah >= 0)
         net[list_ah]->list_bk = list_bk;

      if(list_bk >= 0)
         net[list_bk]->list_ah = list_ah;
      else
         list_ptr = list_ah;
	}

   net[node_ptr]->list_ah = -1;
   net[node_ptr]->list_bk = -1;

      /*********** check branches & find nearest unused end node ****/
   dist_min = cap_distance;
   nearest_seg = 0;
   branches = 6 + ((tlocale[node0.i12[5]] >> 29) & 7);
   for(b=6; b<branches; b++)
   {      /******* get next segment number ***/
      s = node0.i12[b];
      seg = (s < 0) ?  -s: s;
      if(!seg)
         break;
            /******** get segment record ********/
      if(get_objects(&segment,seg))
      {   message("line_atrbt:  cannot read node record, no action taken.");
         line_atrbt_abort(11);
         return(1);
      }
         /***** calc dist_ah      *****/
      dist_ah = net[node_ptr]->dist_bk + segment.f4[2];

         /***** see if oposite node is closer than dist_ah, reset nearest **/
      if(dist_ah < dist_min)
      {      /***** find oposite node *****/
         if(s < 0)
            node_oposite = segment.i12[9];
         else
            node_oposite = segment.i12[10];
   
				/* see if oposite is not self */
			if(node_oposite != node_ptr)
			{		/** check to see if node is part of network **/
				switch(net[node_oposite]->list_ah)  {
				case -3:          /** node not yet on network **/
         	   nearest_node = node_oposite;
            	dist_min = dist_ah;
            	nearest_seg = seg;
					break;
				case -2:          /** node in adjacent to network **/
            	if((dist_ah) < net[node_oposite]->dist_bk)
            	{   nearest_node = node_oposite;
               	dist_min = dist_ah;
               	nearest_seg = seg;
            	}
					break;
				}
			}
      }
   }

      /*********** if end node found set node_bk, dist_bk, seg_bk ***/
	reset = -1;
   if(nearest_seg)
   {
      if(net[nearest_node]->seg_bk)  /** node already used, it must be reset */
         reset = net[nearest_node]->node_bk;

         /*** set structure for nearest node **/
      net[nearest_node]->node_bk = node_ptr;
      net[nearest_node]->dist_bk = dist_min;
      net[nearest_node]->seg_bk  = nearest_seg;
		net[nearest_node]->list_ah = -2;  /*** flag for adjacent to network **/

         /*** if nearest node is cap_node, reset cap_dist ***/
		if(nearest_node == cap_node)
			cap_distance = net[nearest_node]->dist_bk;

      /*********** set origonal node node_ah, dist_ah, list_ah, list_bk **/
   	check_dist = net[nearest_node]->dist_bk;
      net[node_ptr]->node_ah = nearest_node;
      net[node_ptr]->dist_ah = check_dist;
      if(list_ptr < 0)
      {   list_ptr = node_ptr;
      }
      else
		{
			last_node = -1;
         for(i=list_ptr; i>= 0; i = net[i]->list_ah)
         {      /*** check for pointer ahead == -1 (end of list) ****/
            if((net[i]->dist_ah) > check_dist)
               break;
            last_node = i;
         }
         if(i < 0)                    /*** add to end of list **/
         {   net[last_node]->list_ah = node_ptr;
            net[node_ptr]->list_bk = last_node;
            net[node_ptr]->list_ah = -1;
         }
         else if (last_node >= 0)     /*** insert into middle of list ***/
         {   net[last_node]->list_ah = node_ptr;
            net[i]->list_bk = node_ptr;
            net[node_ptr]->list_bk = last_node;
            net[node_ptr]->list_ah = i;
         }
         else                         /*** insert into beginning of list ***/
         {   list_ptr = node_ptr;
            net[i]->list_bk = node_ptr;
            net[node_ptr]->list_bk = -1;
            net[node_ptr]->list_ah = i;
         }
      }
   }
   
      /*** reset structure for old node pointing to new nearest_node **/
   if(reset >= 0)
   {   
      if(find_node_ah(reset))
      {  line_atrbt_abort(12);
         return(1);
      }
   }

      /*********** return *******************************************/
   return(0);
}
