/*******************************************************************************
*       line_on         (see manual, Internal Functions)                <in>   *

     Purpose:	Draws a line on the screen between integer points.   

     Arguments:	x1,y1 - first coordinate pair
		x2,y2 - second coordinate pair

     Returned Values:  (long) 0;

     Global Variables Modified:

*                      j.dabritz  3/7/89 - 5/8/89                              *
*******************************************************************************/
/* Modified 3/92 by DBS@CWU for X11 support */

#include "../../incl/globals.h"

#ifndef LTP_X11    /***** First is the original VGA code *****/

#include "malloc.h"
#include <sys/ioctl.h>
#include <memory.h>
#include <math.h>


long line_on(x1,y1,x2,y2)
long x1,x2,y1,y2;
{	
	long dx,dy;
	long e,e1,e2;
	long i;

	static long mask[32] = { 1<<7, 1<<6, 1<<5, 1<<4, 1<<3, 1<<2, 1<<1, 1,
		1<<15, 1<<14, 1<<13, 1<<12, 1<<11, 1<<10, 1<<9, 1<<8,
		1<<23, 1<<22, 1<<21, 1<<20, 1<<19, 1<<18, 1<<17, 1<<16,
		1<<31, 1<<30, 1<<29, 1<<28, 1<<27, 1<<26, 1<<25, 1<<24  };

	long *ptr, *mask_ptr;
	long ptr_incr;


		/*********   clip line to raster portion of graphics screen *******/
	if(clipl(&x1, &y1, &x2, &y2, windowxmin, windowxmax, 
	 windowymin, windowymax))
		return(0);
	

		/*********   calc dx and switch points to make it non-negative.   */
	dx = x2 - x1;
	if(dx < 0) 
	{	i = x1;
		x1 = x2;
		x2 = i;
		i = y1;
		y1 = y2;
		y2 = i;
		dx = x2 - x1;
	}
		/*********   calc dy  */
	dy = y2 - y1;


		/*********   calc ptr & mask to screen byte & mask ****************/
	ptr_incr = (BITPLN_WIDTH >> 2);
	ptr = (long *) vga_plane;
	ptr += y1 * ptr_incr + (x1 >> 5);
	mask_ptr = mask + (x1 & 31);


		/*********   divide situation up into cases ****************/
	if(dy >= 0)              /* line has positive slope */
	{	if(dx > dy)          /* slope is < 1  */
		{	e  = (dy * 2) - dx;
			e1 = (dy - dx) * 2;
			e2 = dy * 2;
			for(i = -1; i != dx; i++)
			{	*ptr |= *(mask_ptr);
				if(e > 0)
				{	y1++;
					ptr += ptr_incr;
					e += e1;
				}
				else
					e += e2;
				x1++;
				if(!(x1 & 31))
				{	ptr++;
					mask_ptr = mask;
				}
				else
					mask_ptr++;
			}
		}
		else                  /* slope is > 1    */
		{	e  = (dx * 2) - dy;
			e1 = (dx - dy) * 2;
			e2 = dx * 2;
			for(i = -1; i != dy; i++, y1++, ptr += ptr_incr)
			{	*ptr |= *(mask_ptr);
				if(e > 0)
				{	x1++;
					if(!(x1 & 31))
					{	ptr++;
						mask_ptr = mask;
					}
					else
						mask_ptr++;
					e += e1;
				}
				else
					e += e2;
			}
		}
	}
	else                     /* slope is negative  */
	{	if(dx > -dy)          /* slope is > -1  */
		{	e  = ((-dy) * 2) - dx;
			e1 = ((-dy) - dx) * 2;
			e2 = (-dy) * 2;
			for(i = -1; i != dx; i++)
			{	*ptr |= *(mask_ptr);
				if(e > 0)
				{	y1--;
					ptr -= ptr_incr;
					e += e1;
				}
				else
					e += e2;
				x1++;
				if(!(x1 & 31))
				{	ptr++;
					mask_ptr = mask;
				}
				else
					mask_ptr++;
			}
		}
		else                  /* slope is < -1   */
		{	e  = (dx * 2) + dy;
			e1 = (dx + dy) * 2;
			e2 = dx * 2;
			for(i = 1; i != dy; i--, y1--, ptr -= ptr_incr)
			{	*ptr |= *(mask_ptr);
				if(e > 0)
				{	x1++;
					if(!(x1 & 31))
					{	ptr++;
						mask_ptr = mask;
					}
					else
						mask_ptr++;
					e += e1;
				}
				else
					e += e2;
			}
		}
	}

	return(0);
}

#else    /***** Now here is the X11 code *****/

#include "../../incl/x11.h"

long line_on(x1,y1,x2,y2)
     long x1,x2,y1,y2;
{	
  /*********   clip line to raster portion of graphics screen *******/
  if(clipl(&x1, &y1, &x2, &y2, windowxmin, windowxmax, 
	   windowymin, windowymax))
    return(0);
  
  XDrawLine(xdisplay, xmainwin, xgcor, x1, y1, x2, y2);
  return(0);
}

#endif   /***** end of X11 code *****/
