/*******************************************************************************
*       line_solid      (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Draw or erase a line on the raster and on the screen          *
*       between specified points.  Uses a square jump between rows.            *
*       See line_diag for diagonal jumps between rows.                         *
*                                                                              *
*    Arguments:    long x1,y1,x2,t2  line segment endpoints, (raster coords)   *
*                  long on           flag:  0 = erase, 1 = draw, 2 = recall    *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified:  none                                          *
*                                                                              *
*                      j.dabritz  6/27/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long horizontal;      /*  flag, 0=vertical, non-zero = horizontal  */

long line_solid(x1,y1,x2,y2,on)
	long x1,y1,x2,y2;
	long on;
{	
	long dx,dy;
	long e,e1,e2;
	long i;


		/*********   clip line to raster ***************************/
	if(clipl(&x1,&y1,&x2,&y2,0,job_stats[1],0,job_stats[2]))
		return(0);

		/*********   calc dx and switch points to make it non-negative.   */
	dx = x2 - x1;
	if(dx < 0) 
	{	i = x1;
		x1 = x2;
		x2 = i;
		i = y1;
		y1 = y2;
		y2 = i;
		dx = x2 - x1;
	}
		/*********   calc cy  */
	dy = y2 - y1;


		/*********   divide situation up into cases ****************/
	if(dy >= 0)              /* line has positive slope */
	{	if(dx > dy)          /* slope is < 1  */
		{	e  = (dy * 2) - dx;
			e1 = (dy - dx) * 2;
			e2 = dy * 2;
			for(i = -1; i != dx; i++, x1++)
			{	pixel(x1,y1,on);
				if(e > 0)
				{	if((i+1) != dx)         /* add square jump to diag. line */
					{	if(e < dy)
							pixel(x1+1, y1, on);
						else
							pixel(x1, y1+1, on);
					}
					y1++;
					e += e1;
				}
				else
					e += e2;
			}
		}
		else                  /* slope is > 1    */
		{	e  = (dx * 2) - dy;
			e1 = (dx - dy) * 2;
			e2 = dx * 2;
			for(i = -1; i != dy; i++, y1++)
			{	pixel(x1,y1,on);
				if(e > 0)
				{	if((i+1) != dy)         /* add square jump to diag. line */
					{	if(e < dx)
							pixel(x1, y1+1, on);
						else
							pixel(x1+1, y1, on);
					}
					x1++;
					e += e1;
				}
				else
					e += e2;
			}
		}
	}
	else                     /* slope is negative  */
	{	if(dx > -dy)          /* slope is > -1  */
		{	e  = ((-dy) * 2) - dx;
			e1 = ((-dy) - dx) * 2;
			e2 = (-dy) * 2;
			for(i = -1; i != dx; i++, x1++)
			{	pixel(x1,y1,on);
				if(e > 0)
				{	if((i+1) != dx)         /* add square jump to diag. line */
					{	if(e < -dy)
							pixel(x1+1, y1, on);
						else
							pixel(x1, y1-1, on);
					}
					y1--;
					e += e1;
				}
				else
					e += e2;
			}
		}
		else                  /* slope is < -1   */
		{	e  = (dx * 2) + dy;
			e1 = (dx + dy) * 2;
			e2 = dx * 2;
			for(i = 1; i != dy; i--, y1--)
			{	pixel(x1,y1,on);
				if(e > 0)
				{	if((i-1) != dy)         /* add square jump to diag. line */
					{	if(e < -dy)
							pixel(x1, y1-1, on);
						else
							pixel(x1+1, y1, on);
					}
					x1++;
					e += e1;
				}
				else
					e += e2;
			}
		}
	}

	return(0);
}
