/*******************************************************************************
*       locale_xy       (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Checks the raster at the input location and returns a         *
*    'locale' variable based on the 8 surrounding bits.  The returned          *
*    variable is a long in which only the lowest 8 bits are used.  These       *
*    based on the surrounding bits as follows ( X marks center ):              *
*                                                                              *
*                                     7 0 1                                    *
*                                     6 X 2                                    *
*                                     5 4 3                                    *
*                                                                              *
*    Arguments:    long x,y;    raster coordinates of position to check.       *
*    NOTE: 1. THERE IS NO PARAMETER CHECKING IN THIS ROUTINE.  THE COORDINATES *
*             ARE ASSUMED TO BE LEGITIMATE.                                    *
*          2. THERE IS NO SPECIAL PROCESSING DUE TO MAP EDGES (SIDES OR        *
*             TOP OR BOTTOM).  IT IS ASSUMED THAT THESE AREAS ARE AVAILABLE    *
*             IN RAM TO CHECK.                                                 *
*          (1 & 2 above are assumed to allow faster processing in this         *
*          intensively used routine.)                                          *
*                                                                              *
*    Returned Values: (long) locale (as defined above) ( 0 - 255 ).            *
*                                                                              *
*    Global Variables Modified:  none                                          *
*                                                                              *
*                      j.dabritz  6/29/89                                      *
*******************************************************************************/
/* Modified 10/92 by DBS@CWU for big endian support */

#include "../../incl/globals.h"

long locale_xy(x, y)
	register long x, y;
{	
	static unsigned long *word_ptr;     /*  pointer to word of raster x,y location   */
	static long bit;           /*  bit number on word of x,y location    */

	/***************** calc the word pointer & bit of x,y location *******/
	word_ptr = lraster[y] + (x >> 5);
/*****
#ifdef BIG_ENDIAN
        bit = (31 & ~x);
#else
*****/
	bit = (7 & ~x) | (x & 24);
/*****
#endif
*****/

	/***************** use locale_ptr() to calc locale variable **********/
	return(locale_ptr(word_ptr,bit));
}
