/*******************************************************************************
*	ltsmse_input   (see manual, Internal Functions)            	<in>   *
*	    		Checks for logitech serial mouse input.  the raw       *
*			input is put into the ginfo global data structure.     *
*			button presses are de-bounced.                         *
*                                                                              *
*			Globals Modified:                                      *
*			        ginfo.button                               *
*			        ginfo.rawcoord[0]                          *
*			        ginfo.rawcoord[1]                          *
*                                                                              *
*			Returns 0 if input is read, 1 if none there.           *
*			J.Dabritz  6/21/89, modified code written by C.Jonson  *
*******************************************************************************/

#include <stdio.h>
#include <sys/ioctl.h>
#include "../../incl/globals.h"

long ltsmse_input()
{
/*  Automatics  */

	static char mse_buf[5];
	static long debounce;
	long i,j,dx,dy;

/*  Mainline  */

		/************** read until a byte is read with bit #6 set. ******/
	ginfo.button = 0;
	ginfo.rawcoord[0] = 0;
	ginfo.rawcoord[1] = 0;
	for(;;)
	{	if( read(fd_ginput, mse_buf, 1))
		{	if(*mse_buf & 64)
				break;
		}
		else
			return(1);
	}


		/************** read next 4 bytes into mse_buf  *****************/
	for(i=1, j=0; i<5 && j<10000; j++)
	{	if( read(fd_ginput, (mse_buf + i), 1))
			i++;
		else
			sleep(0);
	}
	if(j>=10000)
		return(1);


		/************** check bytes 1-4 for bit 6 = 0   *****************/
	for(i=1; i<5 ; i++)
	{	if(mse_buf[i] & 64)
			return(1);
	}


		/************** build ginfo button press ********************/
	if(debounce)
	{	ginfo.button = 0;         /* set to null */
		if(!((*mse_buf) & 28))
			debounce = 0;
	}
	else
	{	if(*mse_buf & 16)
		{	ginfo.button = 'L';
			debounce = 1;
		}
		else if(*mse_buf & 8)
		{	ginfo.button = 'M';
			debounce = 1;
		}
		else if(*mse_buf & 4)
		{	ginfo.button = 'R';
			debounce = 1;
		}
		else
			debounce = 0;
	}


		/************** build ginfo x & y movement ******************/
	dx =  (mse_buf[1] & 63) | ((mse_buf[2] & 63) << 6);
	if(dx & 2048)
		dx = (dx | (~2047));
	dy =  (mse_buf[3] & 63) | ((mse_buf[4] & 63) << 6);
	if(dy & 2048)
		dy = (dy | (~2047));
	ginfo.rawcoord[0] = dx;
	ginfo.rawcoord[1] = dy;


	return(0);
}
