/*******************************************************************************
*       main            (see manual, Internal Functions)                <in>   *
*	main	This is the entry point for LTPlus.                            *
*		Checks (or sets) the working directory.                        *
*		Checks for (and completes if necessary) the Config file.       *
*		Sets up all include files/common blocks                        *
*		Sets up the tiles etc.                                         *
*		Processes optional parameters which are macro names.           *
*		Starts user input.                                             *
*		Pops back to the previous directory upon exit.                 *
*		Returns zero upon normal end-of-job (EOJ).                     *
*                      j.dabritz  3/7/89 - 5/8/89                              *
*******************************************************************************/
/* Last modified on 5/18/92 by DBS@CWU for X11 and GRASS use. */

#include <string.h>
#include <termio.h>
#include <math.h>
#include <stdio.h>                       /*  needed if define.h is included   */
#include <signal.h>                      /*  used for signal handling         */
#include "../../incl/define.h"           /*  all standard #define statements  */
#include "../../incl/control.h"          /*  info from the prog calling ltp   */
#include "../../incl/errors.h"           /*  a list of error messages         */
#include "../../incl/fc.h"               /*  <fc> function definitions        */
#include "../../incl/fm.h"               /*  <fm> function definitions        */
#include "../../incl/font.h"             /*  graphics font                    */
#include "../../incl/func.h"             /*  names & addresses of FMs         */
#include "../../incl/graphics.h"       	 /*  graphics variables               */
#include "../../incl/in.h"               /*  <in> function definitions        */
#include "../../incl/input.h"            /*  variables about input modes, etc.*/
#include "../../incl/lists.h"            /*  prompting & display text         */
#include "../../incl/map.h"              /*  map file globals                 */
#include "../../incl/mouse.h"            /*  mouse input buffer info          */
#include "../../incl/paths.h"            /*  directory paths, ltp & map files */
#include "../../incl/pens.h"             /*  contents of current pen file     */
#include "../../incl/prompts.h"          /*  a list of available prompts      */
#include "../../incl/set.h"              /*  list of current set criteria     */
#include "../../incl/tables.h"           /*  lookup tables tlocale & movement */
#define GRAPHICS_ON
#include "../../incl/x11.h"              /* the X11 global variables          */


/*-------------------- function defn, parameters -----------------------------*/

int main(argc, argv)
	int argc;  
	char *argv[];
/*	int argc	The number of parameters passed to main.  All 
			parameters passed (beyond 1 [main]) are assumed
			to be macro names.  These macros (if any) are 
			executed before beginning user input.  
	char *argv[]	These are all assumed to be macro names.             */

{
/*--------------------    internal variables     -----------------------------*/

	long i;
	long malloc_option;
	int size;

	char *l;
	char cwd[256];

	char string[STRLIST];

/*
printf("argc = %d\n",argc);
for(i=0; i<argc; i++)
	printf("argv[%d] = %s\n",i,argv[i]);
printf("\n\n");
*/


/*-------------   see if the LTPQID environment variable exists.     ---------*/
	if( !getenv(LTPQID) )
	{	printf("\n\nWARNING:  You have invoked this program incorrectly!\n\
Execute the program 'ltpgo' to run LTPlus.  Do it again and I'll format\n\
your hard disk and call you bad names!  Ha!\n\n\7");
		exit(0);
	}

/*-------------   look for -b/-m input parameter (batch/malloc switches) -----*/

	malloc_option = 0;
	for(i=1;  i<argc;  i++)
	{	if(!strcmp("-b", trim(lowerc(argv[i]))))
		{	batch_mode = 1;
			malloc_option = 1;
		}
		if(!strcmp("-m", trim(lowerc(argv[i]))))
			malloc_option = 1;
	}


/*---------------------    set the paths in paths.h     ----------------------*/
	if(!batch_mode)
		printf("\nsetting internal variables:  paths.h\n");
		/*    first, get the complete call path to this program (from '/') */
	l=argv[0];

#ifdef MULTIUSER
		strcpy(ltp_path, LTP_BASE_DIR);
#else
	if(l[0] != '/') 
	{	size = 258;
		if( !getcwd(cwd,size) )
		{	if(!batch_mode)
				printf("errored out on getcwd in main.\n\n");
			system_ltp("LTP_EXIT");
			return;
		}
		strcpy(ltp_path,cwd);
		strcat(ltp_path,"/");
		strcat(ltp_path,l);
	}
	else  strcpy(ltp_path,l);

		/*  next, snip it off after 'ltp/'                    */
	l = strstr( ltp_path, "ltp/");

	if(l == (char *) 0)
	{	
#if (SV32)
		strcpy(ltp_path,"/usr/ltp");
#else
		strcpy(ltp_path,"/u/ltp");
#endif

/*#if (ULTRIX || SUNOS) */
#ifdef SCO_UNIX
#define         RUN_LS_STRING "lc -d %s"
#else
#define         RUN_LS_STRING "ls -d %s"
#endif
		sprintf(string,RUN_LS_STRING,ltp_path);
		if(system_ltp(string))
		{	strcpy(ltp_path,"/u/ltp");
			sprintf(string,RUN_LS_STRING,ltp_path);
			if(system_ltp(string))
			{	if(!batch_mode)
				{	printf("\n     The 'ltp' directory was not found when setting paths in main.\n\n");
					printf("Note:  To run Line Trace Plus, you should use the install package to make a\n");
					printf("directory called 'ltp' under the 'u' or 'usr' file system.\n");
					printf("Please check your installation instructions and re-install the system or\n");
					printf("contact your system manager.  LTPlus now ending.\n");
					printf("\n");
				}
				system_ltp("LTP_EXIT");
				return(1);
			}
		}
	}
	else
		l[3] = 0;    /** trim string after 'ltp/'  **/
#endif /*MULTIUSER*/

	if(!batch_mode)
		printf("      from:  %s\n",ltp_path);
	else
		printf("\n\n\n      LTPlus batch mode begun successfully.\n\n");

#ifdef MULTIUSER
	sprintf(home_path, "%s", getenv("HOME"));
	if (!home_path[0]) {
	  if(!batch_mode) {
	    printf("\n  Error while trying to get the user's home directory");
	    printf("\n  from the HOME environment variable.  \n");
	  }
	  system_ltp("LTP_EXIT");
	  return(1);
	}
	else {
	  char fullpath[512];
	  sprintf(fullpath, "%s/.ltp", home_path);
	  makedir(fullpath);
	}
#endif

#ifdef GRASS
	sprintf(grass_path, "%s", getenv("LOCATION"));
	if (!grass_path[0]) {
	  if(!batch_mode) {
	    printf("\n  Error while trying to read the user's current GRASS LOCATION");
	    printf("\n  and mapset.  \n");
	  }
	  system_ltp("LTP_EXIT");
	  return(1);
	}
	sprintf(maps_path, "%s/ltp_maps", grass_path);
	makedir(maps_path);
#else
	sprintf(maps_path, "%s/maps", ltp_path);
#endif

	sprintf(digmenus_path, "%s/digmenus", ltp_path);
	sprintf(export_path, "%s/export", ltp_path);
	sprintf(import_path, "%s/import", ltp_path);
	sprintf(macros_path, "%s/macros", ltp_path);
	sprintf(menus_path, "%s/menus", ltp_path);

/*--------------------- set globals & put in graphics mode ------------------*/
	if(initialize(malloc_option))
	{
                printf("Line Trace Plus terminating.\n\n");
		system_ltp("LTP_EXIT");
		exit(1);
	}

/*--------------------- dispaly execution indicator (lower rt) --------------*/
	swplane(0);
	bar_on(626,413,637,403);


/*--------------------- Trap death of children  -----------------------------*/
	signal(SIGCLD, SIG_IGN);
	signal(SIGINT, SIG_IGN);

/*--------------------- execute entry parameters (commands)    --------------*/
/*         (processes all commands, ignores exit.)                           */

	for(i=1;  i<argc;  i++)
	{	if(strcmp("-b", trim(lowerc(argv[i]))))
		{	if(strcmp("-m", trim(lowerc(argv[i]))))
			{	strcpy(raw_buf,argv[i]);
				message(raw_buf);
				if(fj() == -1)
					message("fatal error on execution, LTPlus is aborting.");
		    	exit_macro = 0;    /*   reset abort macro flag   */
			}
		}
	}


/*--------------------- remove  execution indicator (lower rt) --------------*/
	swplane(0);
	bar_off(626,413,637,403);


/*--------------------- get user input until exit is executed  --------------*/
/*                         (if not in batch mode)                            */
	
	if(!batch_mode)
	{	for(; !(exit_program) ;)
		{	if(im() == -1)
			{
				message("error on input.");
				sleep(5);
				exit_program = 1;
			}
			else
			{	swplane(0);
				bar_on(626,413,637,403);  /* display execution indicator */
#ifdef LTP_X11
				XFlush(xdisplay);
#endif
			 	if(fj() == -1)
				{
					message("error on execution.");
					sleep(5);
					exit_program = 1;
				}
				exit_macro = 0;    /*   reset abort macro flag   */
				swplane(0);
				bar_off(626,413,637,403);  /* remove exection indicator */
#ifdef LTP_X11
				XFlush(xdisplay);
#endif
			}
		}
	}
	else      /*-------- if in batch mode, execute exit routine -----*/
	{	swplane(0);
		bar_on(626,413,637,403);  /* display execution indicator */
		sprintf(raw_buf,"exit 1");
		fj();
		swplane(0);
		bar_off(626,413,637,403);  /* remove exection indicator */
	}


/*--------------------- reset screen to previous mode ------------------------*/
	if(!batch_mode)
		graphics_off();

/*--------------------- stop trapping graphics input  ------------------------*/
	if(!batch_mode)
		ginput_reset();

/*--------------------- free malloc'ed space          ------------------------*/
	if(malloc_ptr[0] != (char *) 0)
	{	free(malloc_ptr[0]);
		malloc_ptr[0] = (char *) 0;
	}

/*--------------------- stop trapping keyboard input  ------------------------*/
	if(!batch_mode)
		kb_reset();

/*--------------------- reset the function key values on AT&T ----------------*/
#ifdef SV32
	if(!batch_mode)
		reset_fkeys();
#endif

/*--------------------- tell the user goodby          ------------------------*/
	printf("\n\n\n\n\n\n\n\n\n");
	printf("                  Thank you for shopping at Line Trace Plus.\n\n");
	printf("                              Please come again.\n");
	printf("\n\n\n\n\n\n\n\n\n\n\n\n\n");

	system_ltp("LTP_EXIT");

	return(0);
}
