/*******************************************************************************
*       malloc_ltp      (see manual, Internal Functions)                <in>   *
*                                                                              *
*            In LTPlus there are 2 options for acquiring RAM:                  *
*       1)  Use the malloc/calloc/realloc/free commands as needed to get it.   *
*       2)  Use the malloc command once at the beginning to get a large block  *
*           and control its allocation internally within the program.          *
*                                                                              *
*           Both of these options are available.   By default, when running    *
*       the program in interactive (non-batch) mode option 1) is used:  Let    *
*       the operating system provide RAM as needed.  The default (and only     *
*       option) when in batch mode (ltpgo -b ...) is option 2:  Get a block    *
*       of RAM at the beginning and control its use internally.  Option 2)     *
*       may also be used in interactive mode when ltpgo is executed with the   *
*       the -m option:   ltpgo -m.                                             *
*           Putting batch runs in option 2 means that it won't compete as much *
*       with an interactive forground partition running ltpgo or other         *
*       software.  If users want to avoid using swap space then they should    *
*       use the -m option also.                                                *
*           Option 2 looks at the config 'malloc = ' value to find out how     *
*       much RAM to grab.  This value should not exceede 3 megs less than      *
*       the total available (RAM + swap).  If foreground partitions are to     *
*       be run at the same time, then 1 meg + ram for it should be left out    *
*       also.                                                                  *
*                                                                              *
*           This program checks the malloc_ptr (char *) to find out which      *
*       option is being used.  If option 2 is being used the the malloc_ptr    *
*       is non-null and memory is managed internally.  If the malloc_ptr is    *
*       null, then the internal calls for memeory (malloc/calloc/realloc/free) *
*       become transparant calls to the operating system.                      *
*                                                                              *
*                                                                              *
*                       only memory control (mcontrol) globals are changed.    *
*                                                                              *
*                       Returns are the same as the corrisponding system calls.*
*                      j.dabritz  1/4/90                                       *
*******************************************************************************/

#include <malloc.h>
#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <string.h>



/*WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW*/

char *malloc_ltp(size)
unsigned int size;
{
	char *ptr;
	long i, j;
	long space,testspace;
	char *next_ptr;
char string[256];
	

		/*** check malloc_ptr for memory option in effect. ***/
	if(malloc_ptr[0] == (char *) 0)
	{		/**** note pointer position in mcontrol array ****/
		for(i=0; (i<100) && (mcontrol[i][0] != (char *) 0); i++);
		i= (i<100) ? i : 99;
		mcontrol[i][0] = malloc(size);
		mcontrol[i][1] = mcontrol[i][0] + size;
/* sprintf(string,"----------------> malloc_ltp returning %d",mcontrol[i][0]);*/
/* message(string);*/
		return(mcontrol[i][0]);
	}

		/****************** check input parameter ************/
	if(size == 0)
		return(0);

		/****************** round size up to full word *******/
	if(size % 4)
		size = size + (4 - (size % 4));
	if(size > (malloc_ptr[1]-malloc_ptr[0]))
		return(0);

		/****************** find next free pointer ***********/
	for(i=0; (i<100)  &&  (mcontrol[i][0]); i++);
	if(i>=100)
		return(0);
	
		/****************** look for a free block big enough ******/
		/*  check each free block in ptr order  */
	for(ptr = malloc_ptr[0];  ptr<malloc_ptr[1]  ; ptr = next_ptr)
	{	next_ptr = malloc_ptr[1];
		space = malloc_ptr[1] - ptr;
		for(j=0; j<100; j++)           /*  check vs all allocated blocks */
		{	if(mcontrol[j][0] >= ptr)  /*  if j block is >= test ptr     */
			{	testspace = mcontrol[j][0] - ptr;
				if(space > testspace)
				{	space = testspace;
					next_ptr = mcontrol[j][1];
				}
			}
		}
		if(space >= size)     /*  found enough space, set control variables */
		{	mcontrol[i][0] = ptr;
			mcontrol[i][1] = ptr + size;
			return(ptr);
		}
	}

	return((char *) 0);

}


/*WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW*/

long free_ltp(ptr)
char * ptr;
{
	char string[256];
	long i;

		/*** check malloc_ptr for memory option in effect. ***/
	if(malloc_ptr[0] == (char *) 0)
	{	for(i=0; (i<100) && (mcontrol[i][0] != ptr); i++);
		if(i < 100)
		{	mcontrol[i][0] = (char *) 0;
			mcontrol[i][1] = (char *) 0;
/*sprintf(string," ||||||||||||||| free_ltp, freeing %d",ptr);*/
/*message(string);*/
			free(ptr);
		}
		else
		{	sprintf(string,"ERROR:  free_ltp, attempting to free pointer not allocated: %d",ptr);
			message(string);
			wait_ltp(string);
		}
		return(0);
	}

		/****************** find pointer in list *************/
	for(i=0; i< 100; i++)
	{	if(mcontrol[i][0] == ptr)
			break;
	}

		/****************** free space ***********************/
	if(i<100)
	{	mcontrol[i][0] = (char *) 0;
		mcontrol[i][1] = (char *) 0;
	}

	return(0);
		


}


/*WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW*/

char *realloc_ltp(ptr,size)
char *ptr;
unsigned int size;
{

	long i;
	char *old0, *old1;
	char *new0;
	char *pto, *pfrom;
char string[256];


		/****************** find old space ************************/
	for(i=0; i<100; i++)
	{	if(mcontrol[i][0] == ptr)
			break;
	}

		/*** check malloc_ptr for memory option in effect. ***/
	if(malloc_ptr[0] == (char *) 0)
	{	if(i >= 100)
			i = 99;
		new0 = realloc(ptr,size);
		if(new0 != (char *) 0)
		{	mcontrol[i][0] = new0;
			mcontrol[i][1] = new0 + size;
		}
/*sprintf(string,"----------------> realloc_ltp returning %d",new0);*/
/*message(string);*/
		return(new0);
	}

		/****************** check old space ************************/
	if(i >= 100)
		return((char *) 0);

		/****************** free old space ************************/
	old0 = mcontrol[i][0];
	old1 = mcontrol[i][1];
	mcontrol[i][0] = (char *) 0;
	mcontrol[i][1] = (char *) 0;

		/****************** malloc new space **********************/
	if(size % 4)            /*  round size up to full word   */
		size = size + (4 - (size % 4));
	new0 = malloc_ltp(size);
	if(!new0)
	{	mcontrol[i][0] = old0;
		mcontrol[i][1] = old1;
		return((char *) 0);
	}

		/****************** move data into new space **************/
	if(new0 != old0)
	{	pto = new0;
		pfrom = old0;
		for( ; (pfrom <= old1) && (pto <= mcontrol[i][1]) ; pto++, pfrom++)
			*pto = *pfrom;
	}


	return(new0);



}


/*WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW*/

char *calloc_ltp(nelem,elsize)
unsigned int nelem;
unsigned int elsize;
{
	unsigned int size;
	char *ptr0;
	char *ptr,*end;
	long i;
char string[256];

		/*** check malloc_ptr for memory option in effect. ***/
	size = nelem * elsize;

	if(malloc_ptr[0] == (char *) 0)
	{	ptr0 = (char *) calloc(nelem,elsize);
		if(ptr0 != (char *) 0)
		{	for(i=0; i<100; i++)
			{	if(mcontrol[i][0] == (char *) 0)
					break;
			}
			if(i >= 100)
				i = 99;
			mcontrol[i][0] = ptr0;
			mcontrol[i][1] = ptr0 + size;
		}
/*sprintf(string,"----------------> calloc_ltp returning %d",ptr0);*/
/*message(string);*/
		return(ptr0);
	}

	if(!(ptr0 = malloc_ltp(size)))
		return((char *) 0);

	end = (char *) (ptr0 + size);
	for(ptr = (char *) ptr0; ptr<end; ptr++)
		*ptr = 0;

	return(ptr0);
	
}


/*WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW*/
