/*******************************************************************************
*     mat_solve                                                                *
*         This program reduces the square left-hand portion of a 2-dimentional *
*     matrix of doubles to the identity.  The portion to the right is treated  *
*     identically and will have whatever values it should.  The number of      *
*     rows must be less than or equal to the number of colums.  There must     *
*     be at least 2 rows.                                                      *
*                                                                              *
*     Parameters:  long rows, cols:   number of rows & columns (rows <= cols)  *
*                                                              (rows >= 2)     *
*                  double *matrix:    pointer to first double of matrix.       *
*                                                                              *
*     Globals changed:  none                                                   *
*                                                                              *
*     Returned:   long 0 = matrix reduced                                      *
*                      1 = solution failed  (rows not unique etc.)             *
*                                                                              *
*     J.Dabritz  8/4/89                                                        *
*******************************************************************************/


long mat_solve(rows, cols, matrix)
	long rows, cols;
	double *matrix;
{
	long c;
	long row, col;
	
	double factor;
	double save;


		/*** check rows & cols ***/
	if((rows < 2) || (cols < rows))
		return(1);

		/*** solve for each column ****/
	for(col = 0; col < rows; col++)
	{		/** find with non-zero value in column col **/
		for(row = col; row < rows; row++)
		{	if(matrix[row * cols + col] != 0.0)
				break;
		}
			/*** check to see if one found ***/
		if(row >= rows)
			return(1);

			/*** switch rows if row != col ***/
		if(row != col)
		{	for(c=0; c<cols; c++)
			{	save = matrix[col * cols + c];
				matrix[col * cols + c] = matrix[row * cols + c];
				matrix[row * cols + c] = save;
			}
		}
			/*** set column value to 1 for diagonal element ***/
		factor = 1.0 / matrix[col * cols + col];
		matrix[col * cols + col] = 1.0;
		for(c = col + 1; c<cols; c++)
			matrix[col * cols + c] = matrix[col * cols + c] * factor;

			/*** 0 out all other rows for this column ***/
		for(row=0; row<rows; row++)
		{	if(row != col)
			{	factor = matrix[row * cols + col];
				matrix[row * cols + col] = 0.0;
				for(c=col+1; c<cols; c++)
					matrix[row * cols + c] -= matrix[col * cols + c] * factor;
			}
		}

	}

		/*** return ***/
	return(0);
}
