/*******************************************************************************
*    move_vertex   (see manual, Command Ref., Internal Functions)   <in>       *
*            This program finds a vertex to move & sets anchorx,y if           *
*       first_point == 1.  If not, it moves the vertex to x,y & snaps it       *
*       (up to 1 pixel) to level entry_level.                                  *
*                                                                              *
*       files/globals changed:  coord_list, objects, anchorx, anchory          *
*                                                                              *
*       values returned:  long 0= ok,  1=failure                               *
*                                                                              *
*       parameters:    long x,y    raster coordinates                          *
*                                                                              *
*            J.Dabritz   10/16/89                                              *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long move_vertex(x,y)
   long x,y;
{
   OBJECTS object,poly,parent,segment,node;
   COORD_LIST coordlist;

   double dr,dx,dy;
   double rr,rx,ry;
   double length, area;
   double last_x, last_y;

   static long obj_num;
   static long vertex;
   long l1,l2;
   long ro,rv1,rv2,rl,rf,vf;
   long recalc[5];
   long i,j,n,p,v,v2;
   long sg,seg;
   long begin, end;
   long rtn;


      /************* check for map loaded ************************/
   if(!job_stats[0])
   {   message("You must Create or Acquire a map befor using the vertex_move command.");
      return(1);
   }

      /************* check level number **************************/
   if((entry_level < 32) || (entry_level > 95))
   {   message("Illegal level number.  Aborting.");
      return(1);
   }


      /************* check for first point ***********************/
   if(first_point)
   {      /********** find map vertex to be moved *****************/
      dx = (double) x;
      dy = (double) y;
      dr = search_radius;
      l1 = 0;
      l2 = 0;
      vf = 2;
      if(find_vertex(dx,dy,dr,l1,l2,vf,&rr,&rx,&ry,&ro,&rv1,&rv2,&rl,&rf))
         return(1);


         /********** set object number & vertex ******************/
      obj_num = ro;
			/*** check vertex rv1 ***/
      if(get_coordlst(&coordlist,rv2))
      {   message("move_vertex, abort, location 5.");
         return(1);
      }
		dx = coordlist.x;
		dy = coordlist.y;
		if((dx == rx) && (dy = ry))
      	vertex = rv2;   /*** changed ** fix it here if need be.  **/
		else
			vertex = rv1;

			/********** check object ********/
      if(get_objects(&object,obj_num))
      {   message("move_vertex, abort, location 1.");
         return(1);
      }

			/********** check object type *****/
		if(object.i12[0] > 7)    /** segment **/
		{		/** check for endpoint **/
			if(vertex == object.i12[5])    /** vertex is beginning vertex **/
				obj_num = object.i12[9];    /** set obj to beginning node **/
			else if(vertex == object.i12[6])    /** vertex is ending vertex **/
				obj_num = object.i12[10];    /** set obj to ending node **/
		}

         /********** set anchorx,y *******************************/
      anchorx = (rx + 0.5);
      anchory = (ry + 0.5);

         /********** return **************************************/
      return(0);
   }
   else
   {      /********** find position to move to    *****************/
      dx = (double) x;
      dy = (double) y;
      dr = 1.414;
      l1 = entry_level;
      l2 = entry_level;
      vf = 1;
      rtn = find_vertex(dx,dy,dr,l1,l2,vf,&rr,&rx,&ry,&ro,&rv1,&rv2,&rl,&rf);
      if(rtn == -1)
      {   message("move_vertex abort, location X");
         return(1);
      }
      if(rtn == 1)      /** no item found **/
      {   rx = dx;
         ry = dy;
      }
         

         /************* init recalc array ***********************/
      for(i=0; i<4; i++)
         recalc[i] = -1;

         /************* get object record ***********************/
      if(get_objects(&object,obj_num))
      {   message("move_vertex, abort, location 1.");
         return(1);
      }

         /************* switch by object type ***************/
      if(object.i12[0] < 8)       /** node **/
      {      /************* remove object from index_list ***********/
         index_obj(0,obj_num);

         /************* NODES: move node & segments *********/
         object.f4[0] = rx;
         object.f4[1] = ry;
         if(put_objects(&object,obj_num))
         {   message("move_vertex, abort, location 2.");
            return(1);
         }
            /************* re-index object *************************/
         index_obj(0,obj_num);

            /************* then    move branch endpoints *******/
         for(j=6; j<10; j++)
         {   seg = object.i12[j];
            if(seg)
            {   seg = (seg < 0) ?  -seg: seg;
                  /************* remove object from index_list ***********/
               index_obj(0,seg);

               if(get_objects(&segment,seg))
               {   message("move_vertex, abort, location 3.");
                  return(1);
               }
               if(object.i12[j] > 0)
                  v = segment.i12[5];
               else
                  v = segment.i12[6];
               coordlist.x = rx;
               coordlist.y = ry;
               if(put_coordlst(&coordlist,v))
               {   message("move_vertex, abort, location 4.");
                  return(1);
               }
               recalc[j - 6] = seg;
                  /************* re-index object  ***********************/
               index_obj(1,seg);
            }
            else
               break;
         }
      }
      else
      {   /************* SEGMENTS: move vertex ***************/
         if(get_coordlst(&coordlist,vertex))
         {   message("move_vertex, abort, location 5.");
            return(1);
         }

            /****** if end vertex, move node ********/
         if(vertex == object.i12[5])   /*** beginning vertex **/
            n = object.i12[9];
         else if(vertex == object.i12[6])  /** ending vertex **/
            n = object.i12[10];
         else                       /** middle vertex **/
            n = -1;

         if(n < 0)     /** just move vertex ***/
         {      /********* remove object from index_list ****/
            index_obj(0,obj_num);
               /********* move vertex ******************/
            coordlist.x = rx;
            coordlist.y = ry;
            if(put_coordlst(&coordlist,vertex))
            {   message("move_vertex, abort, location 6.");
               return(1);
            }
            recalc[0] = obj_num;
               /******** re-index object into index_array ***/
            index_obj(1,obj_num);
         }
         else         /* move node & all attached segments **/
         {      /************* remove object from index_list ***********/
            index_obj(0,n);
               /************* get node record *************************/
            if(get_objects(&node,n))
            {   message("move_vertex, abort, location 7.");
               return(1);
            }
               /** move node *******************/
            node.f4[0] = rx;
            node.f4[1] = ry;
            if(put_objects(&node,n))
            {   message("move_vertex, abort, location 8.");
               return(1);
            }
               /************* re-index node ***************************/
            index_obj(1,n);
               /***** then move branch endpoints *******/
            for(j=6; j<10; j++)
            {   seg = node.i12[j];
               if(seg)
               {   seg = (seg < 0) ?  -seg: seg;
                     /************* remove segment from index ***************/
                  index_obj(0,seg);

                  if(get_objects(&segment,seg))
                  {   message("move_vertex, abort, location 9.");
                     return(1);
                  }
                  if(node.i12[j] > 0)
                     v2 = segment.i12[5];
                  else
                     v2 = segment.i12[6];
                  coordlist.x = rx;
                  coordlist.y = ry;
                  if(put_coordlst(&coordlist,v2))
                  {   message("move_vertex, abort, location 10.");
                     return(1);
                  }
                     /************* re-index segment ************************/
                  index_obj(1,seg);

                  recalc[j - 6] = seg;
               }
               else
                  break;
            }
         }
      }
   }
      
      /************* recalculate lengths & areas *****************/
   for(i=0; (i<4) && (recalc[i] >= 0); i++)
   {      /******* get object **************/
      if(get_objects(&object,recalc[i]))
      {   message("move_vertex, abort, location 11.");
         return(1);
      }

      length = 0.0;
      area = 0.0;
      begin = object.i12[5];
      end = object.i12[6];
      for(v = begin;  v<= end;  v++)
      {   if(get_coordlst(&coordlist,v))
         {   message("move_vertex, abort, location 12.");
            return(1);
         }
         if(v > begin)
         {   dx = coordlist.x - last_x;
            dy = coordlist.y - last_y;
            length += sqrt((dx*dx) + (dy*dy));
            area += dx * (coordlist.y + last_y);  /* double area */
         }
         last_x = coordlist.x;
         last_y = coordlist.y;
      }
      area = area / 2;
      object.f4[2] = length;
      object.f4[3] = area;

      if(put_objects(&object,recalc[i]))
      {   message("move_vertex, abort, location 13.");
         return(1);
      }

         /**** update polygon on both sides ***/
      for(j=7; j<9; j++)
      {   p = object.i12[j];    /** polygon number **/
         if(p <= 0)
            break;

         if(get_objects(&poly,p))
         {   message("move_vertex, abort, location 14.");
            return(1);
         }
            /** calculate polygon perimeter & area **/
         begin = poly.i12[5];
         end = poly.i12[6];
         area = 0.0;
         length = 0.0;
         for(sg=begin; sg<= end; sg++)
         {   if(get_seglist(&seg,sg))
            {   message("move_vertex, abort, location 15.");
               return(1);
            }
            if(seg > 0)
            {   if(get_objects(&segment,seg))
               {   message("move_vertex, abort, location 16.");
                  return(1);
               }
               length += segment.f4[2];
               area += segment.f4[3];
            }
            else
            {   if(get_objects(&segment,-seg))
               {   message("move_vertex, abort, location 17.");
                  return(1);
               }
               length += segment.f4[2];
               area += -segment.f4[3];
            }
         }

         if(poly.i12[0] == 15)         /* island polygon */
         {   if(get_objects(&parent,poly.i12[7]))
            {   message("move_vertex, abort, location 18.");
               return(1);
            }
      
            parent.f4[3] += poly.f4[3];   /** subtract island area **/

            if(put_objects(&parent,poly.i12[7]))
            {   message("move_vertex, abort, location 19.");
               return(1);
            }
         }
         if(put_objects(&poly,p))
         {   message("move_vertex, abort, location 20.");
            return(1);
         }
      }
   }

      /************* recalculate line centroids ******************/
      /************* recalculate polygon centroids ***************/

      /*** redisplay the map ****/
   if((disp_hglight || disp_normal))
   {      /************* call dispgraphics to re-display data *********/
      i = (rwindowxmin + rwindowxmax) / 2;
      j = (rwindowymin + rwindowymax) / 2;
      if(dispgraphics(i,j,resolution_code))
         return(1);
   }

      /************* return **************************************/
   return(0);
}
