/*******************************************************************************
*	mse_input     (see manual, Internal Functions)            	<in>   *
*	    		Collects input from mouse and cooks it.  Sets the      *
*			ginfo structure and the raw_buf string if appropriate. *
*			returns a 1 for a button press, otherwise 0.           *
*           Updates the cursor display.                                        *
*			Mice handeled are:                                     *
*			    Logitech serial mouse                              *
*               ATT bus mouse (on ATT machines only)                           *
*               Microsoft mouse [in future ]                                   *
*               X11 Pointer (DBS@CWU)                                          *
*                                                                              *
*			Globals Modified:                                      *
*               ginfo structure                                                *
*               raw_buf                                                        *
*                                                                              *
*			Returns 1 if raw_buf is set (button press, if defined).*
*			        0 for no command input.                        *
*              J.dabritz  6/21/89                                              *
*******************************************************************************/
/* Modified 4/92 by DBS@CWU for X11 support */

#include <string.h>
#include "../../incl/globals.h" 

long mse_input()
{
	long i;
	long longx, longy;


		/***************** null out raw input variables ************/
	ginfo.rawcoord[0] = 0;
	ginfo.rawcoord[1] = 0;
	ginfo.button = 0;

		/***************** get mouse raw input *********************/
		/*   ( button & raw x & y )                                */
	longx = 0;
	longy = 0;
	for(i = 0;;)
	{	switch( gr_dev ) {
#ifndef LTP_X11
		case LTSMSE:		/*  logitech serial mouse  */
			i = ltsmse_input();
			break;
		case MSSMSE:		/*  microsoft serial mouse */
			i = mssmse_input();
			i = (cursor_mode & 2) ? i : 1;
			break;
		case ATTBMSE:		/*  ATT bus mouse          */
			attmse_input();
			if( cursor_mode * cursor_mode * 3)
				i = 1;
			break;
#else
		case X11MSE:
			i = x11mse_input();
			break;
#endif
		default:
			return(0);
		}
#ifdef LTP_X11  /* the x11mse provides absolute coords, not relative */
		longx = ginfo.rawcoord[0];
		longy = ginfo.rawcoord[1];
#else
		longx += ginfo.rawcoord[0];
		longy += ginfo.rawcoord[1];
#endif
		if(ginfo.button || !cursor_mode || i)
			break;
	}
	ginfo.rawcoord[0] = longx;
	ginfo.rawcoord[1] = longy;

		/************** check for input *********************************/
	if((ginfo.rawcoord[0] == 0)  &&  (ginfo.rawcoord[1] == 0) && !(ginfo.button))
	{	/***************** display cursor *****************************/
		cursor(ginfo.scoord[0], ginfo.scoord[1]);
		return(0);
	}

		/************** calc x & y screen coords ************************/
		/*   calc screen coords */
#ifdef LTP_X11  /* the x11mse provides absolute coords */
	ginfo.scoord[0] = ginfo.rawcoord[0];
#else
	ginfo.scoord[0] = ginfo.scoord[0] + ginfo.rawcoord[0];
#endif
	ginfo.scoord[0] = (ginfo.scoord[0] > 639) ? 639 : ginfo.scoord[0];
	ginfo.scoord[0] = (ginfo.scoord[0] < 0) ? 0 : ginfo.scoord[0];

#ifdef LTP_X11  /* the x11mse provides absolute coords */
	ginfo.scoord[1] = ginfo.rawcoord[1];
#else
	ginfo.scoord[1] = ginfo.scoord[1] - ginfo.rawcoord[1];
#endif
	ginfo.scoord[1] = (ginfo.scoord[1] > 415) ? 415 : ginfo.scoord[1];
	ginfo.scoord[1] = (ginfo.scoord[1] < 0) ? 0 : ginfo.scoord[1];

		/************** find which tile we are on ***********************/
	longx =  ginfo.scoord[0];
	longy =  ginfo.scoord[1];
	if((longx >= windowxmin)  &&  (longx <= windowxmax)  &&
	   			(longy >= windowymin)  &&  (longy <= windowymax) )
		ginfo.tile_num = 0;   /*  on the graphics tile  */
	else if((longx >= windowxmin)  &&  
			 (longx < tcoords[8].x[1])  &&
	   		 (longy >= tcoords[8].y[0])  &&  
			 (longy < tcoords[8].y[1]) )
		ginfo.tile_num = 8;
	else
	{	for(i=0; i<12; i++)
		{	if((longx >= tcoords[i].x[0])  &&  
			 (longx < tcoords[i].x[1])  &&
	   		 (longy >= tcoords[i].y[0])  &&  
			 (longy < tcoords[i].y[1]) )
				break;
		}
		ginfo.tile_num = i;    /*  note:  12 is default tile */
	}

		/************** calculate tile coords ***************************/
	switch( ginfo.tile_num )   {
	case 0:         /*   raster graphics tile      */
		scrn_to_rstr(&longx, &longy);
		ginfo.tcoord[0] = longx;
		ginfo.tcoord[1] = longy;
		break;
	case 3:         /*   permanent functions tile  */
	case 5:         /*   menu names tile           */
	case 6:         /*   functions tile            */
			/*   calc row number from the top   */
		i = ginfo.tile_num;
		ginfo.tcoord[1] = (ginfo.scoord[1] - tcoords[i].y[0] ) / 16;
		ginfo.tcoord[0] = ginfo.tcoord[1];
		break;
	case 8:         /*   horizontal position tile  */
		if(((long) ginfo.scoord[0]) == horz_marker)
		{	ginfo.tcoord[0] = 0;
			ginfo.tcoord[1] = 0;
		}
		else if(((long) ginfo.scoord[0]) > horz_marker)
		{	ginfo.tcoord[0] = 1;
			ginfo.tcoord[1] = 1;
		}
		else
		{	ginfo.tcoord[0] = -1;
			ginfo.tcoord[1] = -1;
		}
		break;
	case 9:         /*   vertical position tile    */
		if(((long) ginfo.scoord[1]) == vert_marker)
		{	ginfo.tcoord[0] = 0;
			ginfo.tcoord[1] = 0;
		}
		else if(((long) ginfo.scoord[1]) < vert_marker)
		{	ginfo.tcoord[0] = 1;
			ginfo.tcoord[1] = 1;
		}
		else
		{	ginfo.tcoord[0] = -1;
			ginfo.tcoord[1] = -1;
		}
		break;
	case 10:        /*   compress/uncompress graphics tile  */
		ginfo.tcoord[0] = 1;
		ginfo.tcoord[1] = 1;
		break;
	default:        /*   any other tile, don't care.  */
		ginfo.tcoord[0] = 0;
		ginfo.tcoord[1] = 0;
		break;
	}

		/***************** display cursor *****************************/
	longx = ginfo.scoord[0];
	longy = ginfo.scoord[1];
	cursor(longx, longy);

		/***************** if button press, set string ****************/
	if(ginfo.button)
	{	switch(ginfo.tile_num)   {
		case 3:         /*   permanent functions tile  */
			i = (ginfo.tcoord[1]);
			strncpy(raw_buf,perm_item[i],95);
			raw_buf[95] = 0;
			trim(raw_buf);
			break;
		case 5:         /*   menu names tile           */
			i = (ginfo.tcoord[1]);
			strncpy(raw_buf,menu_name[i],95);
			raw_buf[95] = 0;
			trim(raw_buf);
			break;
		case 6:         /*   current menu functions tile   */
			i = (ginfo.tcoord[1]);
			strncpy(raw_buf,menu_item[current_menu][i],95);
			raw_buf[95] = 0;
			trim(raw_buf);
			break;
		case 8:         /*   horizontal position tile  */
			if(ginfo.tcoord[0] < 0)
				strcpy(raw_buf,"step left");
			else if(ginfo.tcoord[0] > 0)
				strcpy(raw_buf,"step right");
			else
				raw_buf[0] = 0;
			break;
		case 9:         /*   vertical position tile    */
			if(ginfo.tcoord[1] < 0)
				strcpy(raw_buf,"step down");
			else if(ginfo.tcoord[1] > 0)
				strcpy(raw_buf,"step up");
			else
				raw_buf[0] = 0;
			break;
		case 10:        /*   compress/uncompress graphics tile  */
			if(windowxmin)      /*  graphics window is currently 'small' */
				strcpy(raw_buf,"graphics +f");  /*  change to large */
			else                /*  graphics window is currently 'large' */
				strcpy(raw_buf,"graphics -f");  /*  change to small */
			break;
		default:        /*   process button through ckeys array */
			strncpy(raw_buf,cursor_keys[(unsigned char) ginfo.button],95);
			raw_buf[95] = 0;
			trim(raw_buf);
			break;
		}
	}
	else
		raw_buf[0] = 0;
	
		/****************** return flag dependent on string input **/
	if(raw_buf[0])
		return(1);
	else
		return(0);

}
