/*******************************************************************************
*	mssmse_input   (see manual, Internal Functions)            	<in>   *
*	    		Checks for microsoft serial mouse input.  the raw       *
*			input is put into the ginfo global data structure.     *
*			button presses are de-bounced.                         *
*                                                                              *
*			Globals Modified:                                      *
*			        ginfo.button                               *
*			        ginfo.rawcoord[0]                          *
*			        ginfo.rawcoord[1]                          *
*                                                                              *
*			Returns 0 if input is read, 1 if none there.           *
*			J.Dabritz  8/1/89                                      *
*******************************************************************************/

#include <stdio.h>
#include <sys/ioctl.h>
#include "../../incl/globals.h"

long mssmse_input()
{
/*  Automatics  */

	static char mse_buf[5];
	static long debounce;
	long i,j;
	char dx,dy,c0,c1,c2;

/*  Mainline  */

		/************** read until a byte is read with bit #6 set. ******/
	ginfo.button = 0;
	ginfo.rawcoord[0] = 0;
	ginfo.rawcoord[1] = 0;
	for(;;)
	{	if( read(fd_ginput, mse_buf, 1))
		{	if(*mse_buf & 64)
				break;
		}
		else
			return(1);
	}


		/************** read next 2 bytes into mse_buf  *****************/
	for(i=1, j=0; i<3 && j<100; j++)
	{	if( read(fd_ginput, (mse_buf + i), 1))
			i++;
		else
			sleep(0);
	}
	if(j>=100)
		return(1);


		/************** check bytes 1-2 for bit 6 = 0   *****************/
	for(i=1; i<3 ; i++)
	{	if(mse_buf[i] & 64)
			return(1);
	}


		/************** build ginfo button press ********************/
	if(debounce)
	{	ginfo.button = 0;         /* set to null */
		if(!((*mse_buf) & 48))
			debounce = 0;
	}
	else
	{	if(*mse_buf & 32)
		{	ginfo.button = 'L';
			debounce = 1;
		}
		else if(*mse_buf & 16)
		{	ginfo.button = 'R';
			debounce = 1;
		}
		else
			debounce = 0;
	}


		/************** build ginfo x & y movement ******************/
	c0 = *mse_buf;
	c1 = 63 & *(mse_buf + 1);
	c2 = 63 & *(mse_buf + 2);
	dx = ((c0 & 3) << 6) | c1;
	dy = -(((c0 & 12) << 4) | c2);
	ginfo.rawcoord[0] = dx;
	ginfo.rawcoord[1] = dy;


	return(0);
}
