/*******************************************************************************
*       pixel           (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Turn a pixel 'on' or 'off' on the map raster.  Updates the    *
*    screen display and the mini-raster global.                                *
*                                                                              *
*    Arguments:    long x,y:   raster coordinates of pixel.                    *
*                  long on:    flag:  0 = erase, 1 = draw, 2 = recall          *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified: Updates:  lraster (craster), mini-raster;      *
*                                                                              *
*                      j.dabritz  6/26/89                                      *
*******************************************************************************/
/* Modified 5/92 by DBS@CWU for X11 support */

#include "../../incl/globals.h"
#ifdef LTP_X11
#include "../../incl/x11.h"
#endif

long pixel(x, y, on)
	long x, y;
	long on;
{	
	long word, byte, bit;
	long row, col;
	long divisor;
	char *ptr;



		/***************** check for map present *********************/
	if(!job_stats[0])
		return(0);


		/***************** if flag is recall, then check bit & set on */
	if(on == 2)
	{	if(!recall_xy(x,y))
			return(0);
	}

		/***************** check input paramters *********************/
	if((x<0) || (x>job_stats[1]) || (y<0) || (y>job_stats[2]))
		return(0);


		/***************** change raster pixel ***********************/
	word = x >> 5;
	bit = (7 & ~x) | (x & 24);
	if(on)
		lraster[y][word] |= (1 << bit);
	else
		lraster[y][word] &= (~(1 << bit));


		/***************** update mini-raster ************************/
	divisor = (job_stats[1] > job_stats[2]) ? (job_stats[1]+1):(job_stats[2]+1);
	row = (y * 400) / divisor ;
	col = (x * 400) / divisor;
	byte = col >> 3;
	bit = (~col) & 7;
	if(on)
		mini_raster[row][byte] |= (1 << bit);
	else
		mini_raster[row][byte] &= (255 & (~(1 << bit)));


		/***************** update display ****************************/
	if(!batch_mode)
	{	if(!rstr_to_scrn(&x,&y))   /*  converts to screen, & 0 if on window */
		{	swplane(1);               /*  switch to raster plane  */
			if(resolution_code != 3) {     /*  not fat pixels  */
#ifndef LTP_X11              /***** first the VGA code *****/
				ptr = vga_plane + (y * BITPLN_WIDTH + ((x >> 3)));
				bit = (~x) & 7;
				if(on)
					*ptr |= (1 << bit);
				else
					*ptr &= (255 & (~(1 << bit)));
#else			     /***** now the X11 code *****/
				if(on)
					XDrawPoint(xdisplay,xmainwin,xgccopy,x,y);
				else
					XDrawPoint(xdisplay,xmainwin,xgcclear,x,y);
#endif			     /***** end of X11 code *****/
			}
			else             /*  fat pixel display mode */
			{	x &= ~7;     /*  round to nearest byte  */
				y &= ~7;
				if(on)
					bar_on(x,y,x+7,y+7);
				else
					bar_off(x,y,x+7,y+7);
			}
		}
	}
	

		/***************** update edit flag **************************/
	if((job_stats[40] < job_stats[45]) || !job_stats[45])
	{	job_stats[45]++;
		job_stats[40] = job_stats[45];
	}

		/***************** return ************************************/
	return(0);
}
