/*******************************************************************************
*       plot_object     (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Determines if object should be plotted in the current pen.    *
*                                                                              *
*    Parameters:   OBJECTS *optr    - pointer to objects record                *
*                  long     penmask - bit mask of which sets are to be plotted *
*                                     in a specific pen number.                *
*                                                                              *
*    Returned Values: (long) display_code:  0 = Do not plot this object        *
*                                           1 = Plot this object               *
*                                          -1 = Error during processing        *
*                                                                              *
*    Global Variables Modified: none                                           *
*    Files modified           : none                                           *
*                                                                              *
*                      C. Jonson  9/5/89                                       *
*******************************************************************************/
#include "../../incl/globals.h"

long plot_object(optr, penmask)
	OBJECTS *optr;
	unsigned long     penmask;
{
	OBJECTS  orec;

/*** Check for polygons.  If found, return.   ***/
	if( optr->i12[0] > 10 )
		return(0);

/*** If penmask is 0xffffffff, plot all MAP-TYPE objects ***/
	if( penmask == (unsigned long)0xffffffff )
	{	if( optr->i12[0] < 8 )		/* True if its a node */
		{	if( map_type & 1 )      /* Its a node map     */
				return(1);
			else                    /* Not a node map     */
				return(0);
		}
		else if((map_type & 4) || (map_type & 2))   /* Seg/poly type map   */
			return(1);
		else 
			return(0);
	}
	else if( optr->i12[0] >= 8 && optr->i12[0] <= 10 )    /* segment */
	{	if( optr->i12[4] & penmask )
			return(1);
		else
		{	if( get_objects( &orec, optr->i12[8] ) )
				return(-1);
			if( orec.i12[4] & penmask )
				return(1);
			else
			{	if( get_objects( &orec, optr->i12[7] ) )
					return(-1);
				if( orec.i12[4] & penmask )
					return(1);
				else
					return(0);
			}
		}
	}
	return(0);
}
