/*******************************************************************************
*	plot_setup      (see manual, Internal Functions)            	<in>   *
*           Set up line for HP plotter.                                        *
*           Switch settings for HP 7580B:                                      *
*                  EXPAND __  XX  NORMAL                                       *
*                 EMULATE __  XX  NORMAL                                       *
*             STAND ALONE XX  __  EAVESDROP                                    *
*            MONITOR MODE __  XX  NORMAL                                       *
*                   LOCAL __  XX  NORMAL                                       *
*                                                                              *
*           Line is no parity, full duplex, hardwire, dtr-bypass.              *
*           Baud rate has been set at 9600.                                    *
*			Globals Modified:                                      *
*			        fd_plot                                        *
*                                                                              *
*			Returns 0 for success, -1 for failure.                 *
*			C.Jonson 8/28/89                                       *
*******************************************************************************/
#include <sys/ioctl.h>  
#include <string.h>  
#include "../../incl/globals.h"     /** Contains termio.h  **/

long plot_init();
char *get_plot();

long plot_setup(p_dev)
long p_dev;
{
	char          *ioctl();
	long          i;
	char          p_get[STRLIST];     /** String written to plotter **/
	char          p_put[STRLIST];     /** String read by plotter **/
	char          p_type[STRLIST];
	char          resp[4];
	static char p_msg[][STRLIST] = 
{"Is the plotter turned ON?  Press RETURN to continue.",
 "Is the plotter in REMOTE state?  Verify, then press RETURN to continue.",
 "Cannot syncronize with the plotter.  Three strikes and your out!" };

/*** Set config variable based on desired plotter type ***/
	if( p_dev == 1)
		strcpy( p_type, config.plt1_adr);
	else if( p_dev == 2 )
		strcpy( p_type, config.plt2_adr);

/***** Open the port with plotter attached to it  */
	if( plot_init( p_type ) == -1 )
	{	message("Cannot initialize plotter. See config.pltX_adr");
		return(-1);
	}

/**** Give time for plotter to syncronize to new baud rate *****/
	sleep(0); 

/**** Establish XON/XOFF handshake for plotter  ***/
	strcpy(p_put, "\033.\0100:");
	if( put_plot(p_put, fd_plot) == -1)
	{	message("Cannot send message to plotter.");
		return(-1);
	}
	strcpy(p_put, "\033.I96;0;17:");
	if( put_plot(p_put, fd_plot) == -1)
	{	message("Cannot send message to plotter.");
		return(-1);
	}
	strcpy(p_put, "\033.N0;19:");
	if( put_plot(p_put, fd_plot) == -1)
	{	message("Cannot send message to plotter.");
		return(-1);
	}

/**** "esc.A" asks HP plotter to return what it is. 7580B or 7585B expected */
	strcpy(p_put, "\033.A");
	if( put_plot(p_put, fd_plot) == -1)
	{	message("Cannot send message to plotter.");
		return(-1);
	}

/**** Read from plotter.  NULL string means sync didn't occur  ***/
	for( i = 0; i < 2; i++ )
	{	if( strlen(get_plot(p_get)) == 0)
		{	message(p_msg[i]);
			get_string(resp);
		}
		else
			break;
	}
	if( i == 2 )		/*** Cannot sync up.  Return ***/
	{	message(p_msg[i]);
		return(-1);
	}

/**** "esc.K" asks HP plotter to clear its buffer               ***/
	strcpy(p_put, "\033.K");
	if( put_plot(p_put, fd_plot) == -1)
	{	message("Cannot send message to plotter.");
		return(-1);
	}
	sleep(1);

/**** Flush the input buffer prior to any more line activity ***/
	ioctl( fd_plot, TCFLSH, 2);

/*** Outta here  ***/
	return(0);
}
/*******************************************************************************
*	plot_init      (see manual, Internal Functions)            	<in>   *
*           Starts output for a serial plotter as specified in the             *
*           plotter config file.                                               *
*           The original line attributes are not saved as the port is          *
*           closed after processing.                                           *
*                                                                              *
*		Globals Modified:                                              *
*			fd_plot                                                *
*                                                                              *
*	    Returns 0 for success, -1 for failure.                             *
*               C. Jonson 8/28/89                                              *
*******************************************************************************/

long plot_init(path)
char *path;
{
	extern int errno;
	struct termio termvar;

/***** Open the plotter file device ***********/
	if( (fd_plot = openf( path, O_RDWR | O_NDELAY )) == -1 )
	{	message("PLOTSETUP: Cannot open plotter.");
		return(-1);
	}

/**** Get and set input attributes of port for plotter */
	if( ioctl( fd_plot, TCGETA, &termvar) == (char *)-1 )
	{	message("PLOTSETUP: Cannot do ioctl #1 on plotter.");
		return(-1);
	}

/***  Hardcode the plotter for 9600 baud operation  ***/
	termvar.c_cflag = CS8 | CREAD | CLOCAL | HUPCL | B9600; 
/***  Enable XON processing                         ***/
	termvar.c_iflag = IXON;
	if( ioctl( fd_plot, TCSETAF, &termvar) == (char *)-1 )
	{	message("PLOTSETUP: Cannot do ioctl #2 on plotter.");
		return(-1);
	}

/**** Allow additional time for the plotter to sync  */
	sleep(0);

	return(0);
}
