/*******************************************************************************
*       polycentroid    (see manual, Internal Functions)                <in>   *
*                                                                              *
*        Centroids all polygons                                                *
*    This centroid routine relies on the existing raster to get visually       *
*    centered locations for labling etc.  For 'exterior' (outside of islands)  *
*    polygons, the centroid is just 1 pixel below their lowest point.  All     *
*    'interior' (normal) polygons are centroided as follows:                   *
*        1.  go around the polygon perimeter stopping at regular intervals.    *
*            a.  at each stop, step 1 pixel inside the polygon.                *
*            b.  blow up a square 'bubble' at that location, moving and        *
*                enlarging it until it is 'trapped' (toucing the polygon       *
*                edge on 3+ sides of the square);                              *
*            c.  store these trial centroids (up to 100) & their sizes.        *
*        2.  sort trial centroids by size.                                     *
*        3.  take the largest as the primary centroid.                         *
*        4.  take up to 4 more secondary centroids (checking in decreasing     *
*            size order) if they are 8 pixels+ in size and their centers       *
*            are at least 1.5 times primary centroid diameter appart.          *
*                                                                              *
*                                                                              *
*    Arguments:    none                                                        *
*                                                                              *
*    Returned Values: (long) 1 = failure                                       *
*                            0 = normal end of job.                            *
*                                                                              *
*    Global Variables Modified:  none, the objects file is modified.           *
*                                                                              *
*                      j.dabritz  7/19/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"

OBJECTS poly, segment;
double step_divisor;
long step_length;
void qsort();

long polycentroid()
{
	COORD_LIST coord_list;

	long polygon;                 /*  polygon (objects) record number */
	long first_pnt;               /*  flag for first centroid point of poly */
	long c, i, j, s;
	long step2;                   /*  new check length for step_length */
	long rmax;                    /*  maximum of all centroid radii    */
	long rmax0;                   /*  set by map size */
	long min_dist;                /*  check dist between centroids     */
	long keeper;                  /*  flag for centroid to be kept     */
	long x,y;
	long dx,dy;
	long dist;
	long centroid[100][3];        /*  array of 100 possible centroids:
                                      [0] = diameter, [1] = x, [2] = y   */
	long final_cent[10][3];       /*  array of up to 10 selected centroids;
                                      [0] = diameter, [1] = x, [2] = y   */
	long get_start_point();
	long blow_bubble();
	int compair_l();

	/** double xxx; **/

	char string[256];

		/**************** check for map & polygons ********************/
	if(!(job_stats[0])  ||  !(job_stats[7]))
	{	message("abort, polycentroid, location 1");
		return(1);
	}

		/**************** initialize variables ************************/
	step_divisor = (job_stats[1] + job_stats[2]) / 200.0;
	rmax0 = (job_stats[1] + job_stats[2]) / 60;


		/**************** for each polygon, find centroid(s) **********/
	for(polygon = job_stats[7]; polygon < job_stats[10]; polygon++)
	{
			/** check for user abort **/
		if(abort_cmd())
			return(1);

		if((polygon % 10) == 0)
		{	sprintf(string,"centroiding polygon %d  . . .  ",job_stats[10]-polygon);
			message(string);
		}

			/**************** get polygon object **********************/
		if(get_objects(&poly, polygon))
		{	message("abort, polycentroid, location 2");
			return(1);
		}

			/**************** switch on interior/exterior type ********/
		switch(poly.i12[0])  {
		case 14:            /** interior polygon **/

				/**************** initialize variables ****************/
			first_pnt = 1;
			for(i=0; i<100; i++)
			{	centroid[i][0] = 0;
				centroid[i][1] = 0;
				centroid[i][2] = 0;
			}

				/**************** go around perimeter blowing bubbles */
			for(c=0; c<100; c++)
			{		/**************** get next starting point for bubble */
				if(get_start_point(&x,&y,first_pnt))
					break;
				centroid[c][1] = x;
				centroid[c][2] = y;
					/**************** blow up bubble & store centroid */
				if(blow_bubble(centroid[c]))  /* poly has no open interior */
				{	poly.f4[0] = x + 0.5;
					poly.f4[1] = y + 0.5;
				}

					/************* reset step_lenght & first point variables*/
/*				if(centroid[c][0] && ( (long) step_divisor))
				{
					xxx = centroid[c][0] / step_divisor;
					step2 = xxx * xxx * (xxx + 1) * 0.5;
				}
				else
				{
					step2 = 0;
				}           
*/
step2 = centroid[c][0];     /* set to the centroid diameter */
				step_length = (step2 > step_length) ?  step2 : step_length;
				first_pnt = 0;
			}

				/**************** sort centroids by diameter **********/
			qsort((char *)centroid, (unsigned)100, 12, compair_l);

				/**************** store primary centroid **************/
			if(centroid[99][0])
			{	poly.f4[0] = centroid[99][1];
				poly.f4[1] = centroid[99][2];
			}

				/**************** store secondary centroids by diameter */
			final_cent[0][0] = centroid[99][0];
			final_cent[0][1] = centroid[99][1];
			final_cent[0][2] = centroid[99][2];
			for(i=1; i<10; i++)         /* initialize rest of final_cent array*/
			{	final_cent[i][0] = 0;
				final_cent[i][1] = 0;
				final_cent[i][2] = 0;
			}
			rmax = centroid[99][0];
			if(rmax < rmax0)
				rmax = rmax0;
			
				/** check centroids from greatest down & build list to keep */
			for(j=1, c=98; (j<10) && (c >= 0) ; c--)
			{		/*  check to see if centroid diameter is >= 8  */
				if(centroid[c][0] < 8)
					break;
					/*** check against all previously accepted centroids **/
/*				min_dist = 1.5 * (rmax + centroid[c][0]);        */
				min_dist = 2.0 * (rmax);
				min_dist *= min_dist;
				for(i=0, keeper=1; i<j; i++)
				{	dx = centroid[c][1] - final_cent[i][1];
					dy = centroid[c][2] - final_cent[i][2];
					dist = (dx*dx) + (dy*dy);
					if(dist < min_dist)
					{	keeper = 0;
						break;
					}
				}
				if(keeper)
				{		/*** put centroid in final_cent list **/
					final_cent[j][0] = centroid[c][0];
					final_cent[j][1] = centroid[c][1];
					final_cent[j][2] = centroid[c][2];
						/*** increment counter of centroids kept **/
					j++;
				}
			}
				/** store final list (up to 10) of centroids ***/
/*			poly.f4[0] = final_cent[0][1];   /** primary centroid */
/*			poly.f4[1] = final_cent[0][2];                        */
			if(j > 1)                /* if there are secondary centroids */
			{	poly.i12[3] = job_stats[11];
				for(i=1; i<j; i++)
				{	coord_list.x = (float) final_cent[i][1];
					coord_list.y = (float) final_cent[i][2];
					if(put_coordlst(&coord_list,job_stats[11]))
					{	message("abort, polycentroid, location 3");
						return(1);
					}
				}
				poly.i12[11] = job_stats[11] - 1;
			}

			break;
		case 15:            /** exterior polygon, not map border  */
		case 16:            /** map border (exterior) polygon     */
				/**************** find a low point of the polygon *****/
			if(get_seglist(&s,poly.i12[5])) /* first seg always has low point */
			{	message("abort, polycentroid, location 4");
				return(1);
			}
			s = abs( (int)s);
			if(get_objects(&segment, (long)s))
			{	message("abort, polycentroid, location 5");
				return(1);
			}
				/**************** store pixel below it as the centroid **/
			poly.f4[0] = (float) segment.i12[1];        /*  x of low point */
			poly.f4[1] = (float) (segment.i12[2] - 1);  /*  y-1 of low point */

			break;
		}

			/**************** store polygon object data ***************/
		if(put_objects(&poly,polygon))
		{	message("abort, polycentroid, location 6");
			return(1);
		}

		if((polygon % 10) == 0)
			append_msg("done.");

	}


		/**************** return **************************************/
	return(0);
}



/*******************************************************************************
*	get_start_point                                                 
*	      		uses polygon, segment, and vertex data to set x & y    *
*			to the interior point beside the polygon perimeter a distance      *
*           of step_length around the perimeter from the last point.           *
*                                                                              *
*           returns 1 if at end of perimeter.   0 otherwise.                   *
*           J.Dabritz  7/19/89                                                 *
*******************************************************************************/


long get_start_point(x,y,first_pnt)
long *x, *y, first_pnt;
{
	static long seg_list_rec, seg_listn;  /*  record numbers in seg_list file */
	static long coord_list_rec, coord_listn; /* rec numbers in coord_list file*/
	static long coord_list_incr;          /*  increment value for records     */
	static long x1, y1, x2, y2;           
	static long vect_len;                 /*  length of vector remaining      */
	static long step_len_rem;             /*  length of step remaining        */
	static long dir;                      /*  direction of vector (0-7)       */
	static COORD_LIST coord_list;         /*  coord_list record contents      */
	static long move_x[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
	static long move_y[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };

	long s;                               /*  signed segment (objects) rec #  */
	long dx, dy; 
	long inside_dir;                      /*  direction to move to get inside */
	long *word_ptr;
	long bit;


	if(first_pnt)
	{		/*****    set markers to seg_list records for this polygon **/
		seg_list_rec = poly.i12[5];
		seg_listn = poly.i12[6];
			/***** get first seg_list record */
		if(get_seglist(&s,seg_list_rec))
		{	message("abort, polycentroid, location 7");
			return(1);
		}
		if(seg_list_rec != seg_listn)    /* increment if not at last rec */
			seg_list_rec++;
			/***** get first segment object record **/
		if(get_objects(&segment, (long)abs( (int)s)))
		{	message("abort, polycentroid, location 8");
			return(1);
		}
			/***** set markers & increment value for coord_list records **/
		if(s > 0)         /*  trace segment in forward direction */
		{	coord_list_rec = segment.i12[5];
			coord_listn = segment.i12[6];
			coord_list_incr = 1;
		}
		else              /*  trace segment backward   */
		{	coord_list_rec = segment.i12[6];
			coord_listn = segment.i12[5];
			coord_list_incr = -1;
		}
			/***** get the first coordinate pair **/
		if(get_coordlst(&coord_list,coord_list_rec))
		{	message("abort, polycentroid, location 9");
			return(1);
		}
			/***** set the beginning x & y values **/
		x2 = coord_list.x;
		y2 = coord_list.y;
			/***** set the length of vect. to 0 to make the following code
					do the work of reading the next vertex, setting vect_len,
					and setting the vector direction etc.   ***/
		vect_len = 0;
			/***** set the first step to 1 pixel to ensure that there will 
					be at least 1 centroid **/
		step_len_rem = 1;         /* only go 1 pixel the first time */
			/***** calc the step_length to be used the rest of the time **/
		step_length = ( poly.f4[2] / 98 ) + 1;	
		step_length = (step_length < 10) ?  10 : step_length;
	}
	else
		step_len_rem = step_length;

		/** trace around poly until we have gone a distance of step_length */
	for( ; step_len_rem ;  )
	{	if(vect_len >= step_len_rem)      /*  next step is on this vector */
		{	x1 += step_len_rem * move_x[dir];
			y1 += step_len_rem * move_y[dir];
			vect_len -= step_len_rem;
			step_len_rem = 0;
		}
		else                              /*  next step is beyond this vector */
		{	step_len_rem -= vect_len;
			x1 = x2;
			y1 = y2;
			if(coord_list_rec != coord_listn)  /* don't need next segment */
			{	coord_list_rec += coord_list_incr;
				if(get_coordlst(&coord_list,coord_list_rec))
				{	message("abort, polycentroid, location 10");
					return(1);
				}
			}
			else                               /* we do need the next segment */
			{
				if(seg_list_rec != seg_listn)  /* not to end of polygon  */
				{		/** get next seg_list record */
					if(get_seglist(&s,seg_list_rec++))
					{	message("abort, polycentroid, location 11");
						return(1);
					}
						/** get next segment object record **/
					if(get_objects(&segment, (long)abs( (int)s)))
					{	message("abort, polycentroid, location 12");
						return(1);
					}
					if(s > 0)         /*  trace segment in forward direction */
					{	coord_list_rec = segment.i12[5];
						coord_listn = segment.i12[6];
						coord_list_incr = 1;
					}
					else              /*  trace segment backward   */
					{	coord_list_rec = segment.i12[6];
						coord_listn = segment.i12[5];
						coord_list_incr = -1;
					}
				}
				else                           /*  at end of polygon  */
					return(1);

				if(get_coordlst(&coord_list,coord_list_rec))
				{	message("abort, polycentroid, location 14");
					return(1);
				}
			}
				/****** set initial values for vect_len, x2,y2 & dir */
			x2 = coord_list.x;
			y2 = coord_list.y;
			dx = x2 - x1;
			dy = y2 - y1;
			vect_len = (dx) ?  abs( (int)dx) : abs( (int)dy);
			if(dx > 0)
			{	if(dy > 0)
					dir = 1;
				else if(dy == 0)
					dir = 2;
				else
					dir = 3;
			}
			else if(dx < 0)
			{	if(dy > 0)
					dir = 7;
				else if(dy == 0)
					dir = 6;
				else
					dir = 5;
			}
			else
			{	if(dy > 0)
					dir = 0;
				else
					dir = 4;
			}
		}
	}
		/*** step 1 pixel inside polygon ***/
	inside_dir = (dir + 3) & 7;
	*x = x1 + move_x[inside_dir];
	*y = y1 + move_y[inside_dir];
		/*** test for an open pixel & move 1 if necessary **/
	word_ptr = (long *) (lraster[*y] + ((*x) >> 5));
	bit  = (7 & ~(*x)) | ((*x) & 24);
	if(*word_ptr & (1 << bit))
	{	inside_dir = (dir + 2) & 7;
		*x = x1 + move_x[inside_dir];
		*y = y1 + move_y[inside_dir];
	}

		/*** return ************************/
	return(0);
}


/*******************************************************************************
*	blow_bubble                                                                *
*	      		uses the initial centroid data (d, x, y) and blows up a        *
*			square bubble on the raster until it becomes trapped.              *
*           returns the position and diameter (d, x, y) in the slots           *
*           of the parameter array.                                            *
*                                                                              *
*           not globals or files changed.                                      *
*                                                                              *
*           parameters:  long centroid[3];   [0] = d (set by this program)     *
*                                            [1] = x (start pt & final center) *
*                                            [2] = y (start pt & final center) *
*                                                                              *
*           returns 1 if initial pt is 'on', 0 otherwise.                      *
*           J.Dabritz  7/20/89                                                 *
*******************************************************************************/

long touch[8];            /** array signaling sides touched:
                              [0] = top, [1] = top rt corner,
                              [2] = right, [3] = bottom rt corner,
                              [4] = bottom, [5] = bottom lt corner,
                              [6] = left, [7] = top lt corner         ***/
long touched;             /** number of sides touched     */
long bubble[4];           /** raster coordinates of bubble limits:
                              [0] = top, [1] = rt, [2] = bottom, [3] = left **/
long movement;            /** direction of movement:
                                          7 0 1
                                          6 X 2
                                          5 4 3          ***/


long blow_bubble(centroid)
long *centroid;  
{
	long x,y;
	long word, bit;
	long i;
	long side;                      /** side number (bubble) of back side **/
	long back_side, front_side;     /** bubble values of sides  **/
	long movement2;                 /** flag for reversing direction **/

	long try_it();


		/*********** check for 'on' pixel at starting location ******/
	x = centroid[1];
	y = centroid[2];
	centroid[0] = 0;
	word = x >> 5;
	bit = (7 & ~x) | (x & 24);
	if(lraster[y][word] & (1 << bit))
		return(1);

		/*********** initialize 'touch' matrix **********************/
	for(i=0; i<8; touch[i++] = 0);

		/*********** initiailize bubble array ***********************/
	bubble[0] = bubble[2] = y;
	bubble[1] = bubble[3] = x;

		/*********** enlarge & move bubble until trapped ************/
	for(;;)
	{	/*********** puff up bubble until it touches on 3-4 sides  **/
			/** change which side is expanding loop ***/
		for(movement = 1; movement < 8; movement += 2)
		{		/***  change diameter loop **/
			for(;;)
			{	/***  test bubble in current location **/
				try_it(); /** sets touch array and touched (# sides touched) */
					/** if a side touched, hop out to change exp. side loop */
				if(touched)
					break;
					/*** otherwise, increase the expanding sides by 1 */
				switch(movement)   {
				case 1:
					bubble[0]++;
					bubble[1]++;
					break;
				case 3:
					bubble[1]++;
					bubble[2]--;
					break;
				case 5:
					bubble[2]--;
					bubble[3]--;
					break;
				case 7:
					bubble[3]--;
					bubble[0]++;
					break;
				}
			}
		}
			
			/*********** check to see if bubble trapped *******/
		if(touch[0] && touch[2] && touch[4] && touch[6])
		{	centroid[0] = bubble[0] - bubble[2];
			if(centroid[0] == 0)
				centroid[0] = 1;       /* let min radius = 1 (for flag)  */
			centroid[1] = (bubble[1] + bubble[3]) / 2;
			centroid[2] = (bubble[0] + bubble[2]) / 2;
			return(0);         /* <<<<<<<<<<<<<<<<<<<<<<<<  RETURN  */
		}
			/*------------------------------------------------------------*/

			/********* Now, move bubble until no 2 opposite sides are touched*/
			/*** select first open cardinal direction to move bubble **/
		for(i=0; touch[i] ; i += 2);
		movement = i;

			/*********** save information on front & back side *****/
		side = ((movement+4) & 7) / 2;       /*  side number of back side */
		back_side = bubble[side];            /*  value of back side */
		front_side = bubble[(side + 2) & 3]; /*  value of front side */

			/*********** check back side to see if it is open also  */
		movement2 = (movement + 4) & 7;
		if(touch[movement2])
			movement2 = -1;

			/*********** move & test bubble until open on a side or trapped */
		for(;;)
		{		/*********** init touch matrix *********************/
			for(i=0; i<8; touch[i++] = 0);
				/*********** check bubble for touches ***************/
			try_it();    /** sets touch array and touched (# sides touched) */
				/*********** check for a touch in direction of movement */
			if(touch[movement])
			{		/******* see if back side is open ***/
				if(movement2 >= 0)
				{		/** reverse directions **/
					bubble[side] = back_side;
					side = (side + 2) & 3;
					back_side = bubble[side];
					bubble[side] = front_side;
					movement = movement2;
					movement2 = -1;
				}
				else
					bubble[side] = back_side;
			}

				/*********** check for a touch ahead ****************/
			if(touch[movement])
			{	centroid[0] = bubble[0] - bubble[2];
				if(centroid[0] == 0)
					centroid[0] = 1;       /* let min radius = 1 (for flag)  */
				centroid[1] = (bubble[1] + bubble[3]) / 2;
				centroid[2] = (bubble[0] + bubble[2]) / 2;
				return(0);         /* <<<<<<<<<<<<<<<<<<<<<<<<  RETURN  */
			}

				/*********** check for open on rt or left (open ahead) */
			if(touch[(movement+6)&7] + touch[(movement+7)&7] == touched)
				break;
			if(touch[(movement+2)&7] + touch[(movement+1)&7] == touched)
				break;

				/*********** otherwise move some more ***************/
			switch(movement)   {
			case 0:
				bubble[0]++;
				bubble[2]++;
				break;
			case 2:
				bubble[1]++;
				bubble[3]++;
				break;
			case 4:
				bubble[2]--;
				bubble[0]--;
				break;
			case 6:
				bubble[3]--;
				bubble[1]--;
				break;
			}
		}
	}
		/*********** ( returns from within the loop ) ***************/
}




/*******************************************************************************
*	try_it                                                                     *
*	      		tests 2 or 3 sides of a bubble for 'on' pixel.                 *
*			sets the touch array accordingly (1 = side touched, 0=untouched)   *
*           and the varaible touched (number of sides touched) .               *
*                                                                              *
*           not globals or files changed.                                      *
*                                                                              *
*           parameters:  none                                                  *
*                                                                              *
*           returns 0                                                          *
*           J.Dabritz  7/20/89                                                 *
*******************************************************************************/

long try_it()
{
	long j;            /** direction of trial movement:
                                          7 0 1
                                          6 X 2
                                          5 4 3          ***/
	long dj;           /** amount to vary (+/-) j **/
	long start, finish;   /** start and end values for varying j **/

	long test_it();


		/************** initialize touched variable ******************/
	touched = 0;

		/************** test 2 or 3 sides for an 'on' pixel **********/
	if(movement & 1)     /** diagonal, test 2 sides and 1 corner */
		dj = 1;
	else                 /** cardinal, test 3 sides and 2 corners */
		dj = 2;
	start = (movement + 8 - dj) & 7;
	finish = (movement + dj) & 7;
	for(j = start; j != finish; j = (j+1) & 7)
		test_it(j);
	test_it(j);

		/************** return ***************************************/
	return(0);
}



/*******************************************************************************
*	test                                                                       *
*	      		checks 1 side or corner of the bubble for 'on' pixels &        *
*			sets the touch array accordingly (1 = side touched, 0=untouched)   *
*           and increments the touched variable (number of sides touched).     *
*                                                                              *
*           not globals or files changed.                                      *
*                                                                              *
*           parameters:  long j:     side/corner to check:                     *
*                                    sides (top clockwise) = 0, 2, 4, 6.       *
*                                    corners (top right cw) = 1, 3, 5, 7.      *
*                        (These are the corrisponding touch array element      *
*                        meanings).                                            *
*                                                                              *
*           returns 0                                                          *
*           J.Dabritz  7/20/89                                                 *
*******************************************************************************/

long test_it(j)
long j;
{
	long n,s,e,w,w2;            /* north etc limits of trial side */
	long *word_ptr, *end_ptr;   /*  pointers to raster words (for testing) */
	long bit;


		/************* reset touched ********************************/
	touched += touch[j];

		/************* check to see if side already touching ********/
	if(touch[j])
		return(0);

		/************* see if test is corner or side ****************/
	if(j & 1)          /*** test is on a corner **/
	{		/************* set n, e on raster coords of corner ******/
		switch(j)  {
		case 1:              /*  top right corner */
			n = bubble[0] + 1;
			e = bubble[1] + 1;
			break;
		case 3:              /*  bottom right corner  */
			n = bubble[2] - 1;
			e = bubble[1] + 1;
			break;
		case 5:              /*  bottom left corner   */
			n = bubble[2] - 1;
			e = bubble[3] - 1;
			break;
		case 7:              /*  top left corner */
			n = bubble[0] + 1;
			e = bubble[3] - 1;
			break;
		}
			/************* check pixel ******************************/
		word_ptr = (long *) (lraster[n] + (e >> 5));
		bit = (7 & ~e) | (e & 24);
		if(*word_ptr & (1 << bit))
		{	touch[j] = 1;
			touched++;
		}
	}
	else if(j & 2)              /*** test is on a side 2 or 6 (vertical) ****/
	{		/************* find colum & begin & end rows to test ****/
		n = bubble[0];
		s = bubble[2];
		if(j == 2)
			e = bubble[1] + 1;
		else
			e = bubble[3] - 1;
			/************* test a colum of pixels *******************/
		word_ptr = (long *) (lraster[s] + (e >> 5));
		end_ptr = (long *) (lraster[n] + (e >> 5) + 1);
		bit = (7 & ~e) | (e & 24);
		for(; word_ptr < end_ptr; word_ptr += raster_wincrement)
		{	if(*word_ptr & (1 << bit))
			{	touch[j] = 1;
				touched++;
				break;
			}
		}
	}
	else                        /*** test is on a horizontal side ***/
	{		/************* find row and begin & end columns to test */
		e = bubble[1];
		w = bubble[3];
		if(j == 0)
			n = bubble[0] + 1;
		else
			n = bubble[2] - 1;
			/************* find begin and end words to check ********/
		word_ptr = (long *) (lraster[n] + (w >> 5));
		end_ptr = (long *) (lraster[n] + (e >> 5));
			/************* see if all bits in 1 word ****************/
		if(word_ptr == end_ptr)       /** all bits in 1 word, check it */
		{	for(; w <= e; w++)
			{	bit = (7 & ~w) | (w & 24);
				if(*word_ptr & (1 << bit))
				{	touch[j] = 1;
					touched++;
					break;
				}
			}
		}
		else                          /** all bits not in 1 word, check them */
		{	/************* check first word *************************/
			w2 = (w + 32) & ~31;
			for(;w < w2 ; w++)
			{	bit = (7 & ~w) | (w & 24);
				if(*word_ptr & (1 << bit))
				{	touch[j] = 1;
					touched++;
					break;
				}
			}
			if(touch[j])
				return(0);
			/************* check middle words ***********************/
			for(word_ptr++; word_ptr < end_ptr; word_ptr++)
			{	if(*word_ptr)
				{	touch[j] = 1;
					touched++;
					break;
				}
			}
			if(touch[j])
				return(0);
			/************* check last word **************************/
			w = e & ~31;
			for(; w <= e; w++)
			{	bit = (7 & ~w) | (w & 24);
				if(*word_ptr & (1 << bit))
				{	touch[j] = 1;
					touched++;
					break;
				}
			}
		}
	}
			/************* return ***********************************/
	return(0);
}

