/*******************************************************************************
*       put_errors      (see manual, Internal Functions)                <in>   *
*                       This program stores the information in one errors      *
*                       structure in either malloc space or the temp file.     *
*                       (whichever is being used as data storage)              *
*                                                                              *
*                       The malloc space/file is changed.                      *
*                       The malloc struct may be changed.                      *
*                       The job_stats[14] entry is updated.                    *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  8/11/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <string.h>
#include <malloc.h>


long put_errors(ptr,rec_no)
ERRORS *ptr;                /*  pointer to errors structure to be saved  */
long rec_no;          /*  one greater than record number for save    */
{
	char *new_ptr;
	char path[512];
	char *to_ptr;
char string[256];

	long offset;
	long put_on_disc;
	long size;
long i;

		/******************** check for legitimate parameters ***********/
	if(!ptr  ||  rec_no < 0 || (rec_no > job_stats[14]))
	{	message("put_errors, bad parameters received, rejected.");
		return(1);
	}

		/******************** calculate offset **************************/
	offset = rec_no * errorsm.rec_size;

		/******************** see if current storage is in RAM  *********/
	put_on_disc = 0;    /** default **/
	if(errorsm.malloc  != (char *) 0)       
	{		/******************** check for enough existing malloc'd space **/
		if((errorsm.rec0 + offset + errorsm.rec_size) > errorsm.recn)
		{		/******************** reallocate enough space ***************/
			size = errorsm.recn - errorsm.rec0;
			size += 2000 * errorsm.rec_size;     /*** add 2000 records ***/
				/*   try to realloc space  */
			new_ptr = (char *) realloc_ltp(errorsm.malloc,size);
			if(new_ptr != (char *) 0)
			{	errorsm.malloc = new_ptr;
				errorsm.rec0 = new_ptr;
				errorsm.recn = new_ptr + size;
					/*   check for a meg free    */
				if(!ramfree(1000000))
					put_on_disc = 1;
			}
			else
				put_on_disc = 1;

			if(put_on_disc)         /*  dump file to disc   */
			{	sprintf(path,"%s/errors",temp_map_path);
				if(0 > (errors_fd = openf(path,RDWR)))
				{	message("put_errors, bad file integer.  rejected.");
					return(1);
				}
				if(0 > writeb(errorsm.rec0,errorsm.rec_size,job_stats[14],errors_fd))
				{	message("put_errors, couldn't write to errors file.  aborting.");
					return(1);
				}
				free_ltp(errorsm.malloc);
				errorsm.malloc = (char *) 0;
				errorsm.rec0 = (char *) 0;
				errorsm.recn = (char *) 0;
			}
		}
			/******************** store in ram ******************************/
		if(!put_on_disc)
		{	to_ptr = errorsm.rec0 + offset;
			memcpy(to_ptr,(char *)ptr,errorsm.rec_size);
		}
	}

	if(errorsm.malloc == 0)
	{		/******************** check for file pointer ********************/
		if(0 >= errors_fd)
		{	message("put_errors, there is no file pointer, rejecting.");
			return(1);
		}
			/******************** position file write pointer ***************/
		if(0 > errors_fd)
		{	message("put_errors, cannot open errors map file, rejecting.");
			return(1);
		}

		if(0 > movefp(errors_fd,offset,0))
		{	message("put_errors, cannot move file pointer, rejecting.");
			return(1);
		}

			/******************** write record ******************************/
		if(0 > writeb((char *)ptr, errorsm.rec_size ,1 ,errors_fd))
		{	message("put_errors, can't write record.  rejecting.");
			return(1);
		}
	}

		/******************** update job_stats file *********************/
	job_stats[14] = (rec_no < job_stats[14]) ? job_stats[14] : (rec_no + 1);

		/******************** return ************************************/
	return(0);
}
