/*******************************************************************************
*       put_indexl      (see manual, Internal Functions)                <in>   *
*                       This program stores the information in one index_list  *
*                       structure in either malloc space or the temp file.     *
*                       (whichever is being used as data storage)              *
*                                                                              *
*                       The malloc space/file is changed.                      *
*                       The malloc struct may be changed.                      *
*                       The job_stats[15] entry is updated.                    *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  6/12/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <string.h>
#include <malloc.h>


long put_indexl(ptr,rec_no)
INDEX_LIST *ptr;      /*  pointer to index_list structure to be saved  */
long rec_no;          /*  one greater than record number for save    */
{
	char *new_ptr;
	char path[512];
	char *to_ptr;

	long offset;
	long put_on_disc;
	long size;

		/******************** check for legitimate parameters ***********/
	if(!ptr  ||  rec_no < 0 || (rec_no > job_stats[15]))
	{	message("put_indexl, bad parameters received, rejected.");
		return(1);
	}

		/******************** calculate offset **************************/
	offset = rec_no * index_listm.rec_size;

		/******************** see if current storage is in RAM  *********/
	put_on_disc = 0;    /** default **/
	if(index_listm.malloc != (char *) 0)       
	{		/******************** check for enough existing malloc'd space **/
		if((index_listm.rec0 + offset + index_listm.rec_size) > index_listm.recn)
		{		/******************** reallocate enough space ***************/
			size = index_listm.recn - index_listm.rec0;
			size += 2000 * index_listm.rec_size;     /*** add 2000 records ***/
				/*   try to realloc space  */
			new_ptr = (char *) realloc_ltp(index_listm.malloc,size);
			if(new_ptr != (char *) 0)
			{	index_listm.malloc = new_ptr;
				index_listm.rec0 = new_ptr;
				index_listm.recn = new_ptr + size;
					/*   check for a meg free    */
				if(!ramfree(1000000))
					put_on_disc = 1;
			}
			else
				put_on_disc = 1;

			if(put_on_disc)         /*  dump file to disc   */
			{	sprintf(path,"%s/index_list",temp_map_path);
				if(0 > (index_list_fd = openf(path,RDWR)))
					return(1);
				if(0 > writeb(index_listm.rec0,index_listm.rec_size,job_stats[15],index_list_fd))
					return(1);
				free_ltp(index_listm.malloc);
				index_listm.malloc = (char *) 0;
				index_listm.rec0 = (char *) 0;
				index_listm.recn = (char *) 0;
			}
		}
			/******************** store in ram ******************************/
		if(!put_on_disc)
		{	to_ptr = index_listm.rec0 + offset;
			memcpy(to_ptr,(char *)ptr,index_listm.rec_size);
		}
	}

	if(index_listm.malloc == 0)
	{		/******************** check for file pointer ********************/
		if(0 > index_list_fd)
		{	message("put_indexl, there is no file pointer, rejecting.");
			return(1);
		}
			/******************** position file write pointer ***************/
		if(0 > movefp(index_list_fd,offset,0))
			return(1);
			/******************** write record ******************************/
		if(0 > writeb((char *)ptr, index_listm.rec_size ,1 ,index_list_fd))
			return(1);
	}

		/******************** update job_stats file *********************/
	job_stats[15] = (rec_no < job_stats[15]) ? job_stats[15] : (rec_no + 1);

		/******************** return ************************************/
	return(0);
}
