/*******************************************************************************
*       put_objects     (see manual, Internal Functions)                <in>   *
*                       This program stores the information in one objects     *
*                       structure in either malloc space or the temp file.     *
*                       (whichever is being used as data storage)              *
*                                                                              *
*                       The malloc space/file is changed.                      *
*                       The malloc struct may be changed.                      *
*                       The job_stats[10] entry is updated.                    *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  8/11/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <string.h>
#include <malloc.h>
#include <errno.h>


long put_objects(ptr,rec_no)
OBJECTS *ptr;                /*  pointer to object structure to be saved  */
long rec_no;          /*  one greater than record number for save    */
{
	extern int errno;
	char *new_ptr;
	char path[512];
	char string[STRLIST];
	char *to_ptr;

	long offset;
	long put_on_disc;
	long size;

		/******************** check for legitimate parameters ***********/
	if(!ptr  ||  rec_no < 0 || (rec_no > job_stats[10]))
	{	message("put_objects, bad parameters received, rejected.");
		return(1);
	}

		/******************** calculate offset **************************/
	offset = rec_no * objectsm.rec_size;

		/******************** see if current storage is in RAM  *********/
	put_on_disc = 0;    /** default **/
	if(objectsm.malloc != (char *) 0)       
	{		/******************** check for enough existing malloc'd space **/
		if((objectsm.rec0 + offset + objectsm.rec_size) > objectsm.recn)
		{		/******************** reallocate enough space ***************/
			size = objectsm.recn - objectsm.rec0;
			size += 2000 * objectsm.rec_size;     /*** add 2000 records ***/
				/*   try to realloc space  */
			new_ptr = (char *) realloc_ltp(objectsm.malloc,size);
			if(new_ptr != (char *) 0)
			{	objectsm.malloc = new_ptr;
				objectsm.rec0 = new_ptr;
				objectsm.recn = new_ptr + size;
					/*   check for a meg free    */
				if(!ramfree(1000000))
					put_on_disc = 1;
			}
			else
				put_on_disc = 1;

			if(put_on_disc)         /*  dump file to disc   */
			{	sprintf(path,"%s/objects",temp_map_path);
				if(0 > (objects_fd = openf(path,RDWR)))
				{	sprintf(string, "put_objects: on open.  UNIX errno = %d", 
							errno);
					message(string);
					return(1);
				}
				if(0 > writeb(objectsm.rec0,objectsm.rec_size,job_stats[10],objects_fd))
				{	sprintf(string, "put_objects: 1st write.  UNIX errno = %d", 
							errno);
					message(string);
					return(1);
				}
				free_ltp(objectsm.malloc);
				objectsm.malloc = (char *) 0;
				objectsm.rec0 = (char *) 0;
				objectsm.recn = (char *) 0;
			}
		}
			/******************** store in ram ******************************/
		if(!put_on_disc)
		{	to_ptr = objectsm.rec0 + offset;
			memcpy(to_ptr,(char *)ptr,objectsm.rec_size);
		}
	}

	if(objectsm.malloc == 0)
	{		/******************** check for file pointer ********************/
		if(0 > objects_fd)
		{	message("put_objects, there is no file pointer, rejecting.");
			return(1);
		}
			/******************** position file write pointer ***************/
		if(0 > movefp(objects_fd,offset,0))
		{	sprintf(string, "put_objects: on movefp.  UNIX errno = %d",errno);
			message(string);
			return(1);
		}
			/******************** write record ******************************/
		if(0 > writeb((char *)ptr, objectsm.rec_size ,1 ,objects_fd))
		{	sprintf(string, "put_objects: 2nd write.  UNIX errno = %d",errno);
			message(string);
			return(1);
		}
	}

		/******************** update job_stats file *********************/
	job_stats[10] = (rec_no < job_stats[10]) ? job_stats[10] : (rec_no + 1);

		/******************** return ************************************/
	return(0);
}
