/*******************************************************************************
*       put_v_coord     (see manual, Internal Functions)                <in>   *
*                       This program stores the information in one v_coordlist *
*                       structure in either malloc space or the temp file.     *
*                       (whichever is being used as data storage)              *
*                                                                              *
*                       The malloc space/file is changed.                      *
*                       The malloc struct may be changed.                      *
*                       The job_stats[18] entry is updated.                    *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  9/18/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <string.h>
#include <malloc.h>


long put_v_coord(ptr,rec_no)
COORD_LIST *ptr;         /*  pointer to coord_list structure to be saved  */
long rec_no;          /*  record number of item to save              */
{
	char *new_ptr;
	char path[512];
	char *to_ptr;

	long offset;
	long put_on_disc;
	long size;

		/******************** check for legitimate parameters ***********/
	if(!ptr  ||  rec_no < 0 || (rec_no > job_stats[18]))
	{	message("put_v_coord, bad parameters received, rejected.");
		return(1);
	}

		/******************** calculate offset **************************/
	offset = rec_no * v_coordlistm.rec_size;

		/******************** see if current storage is in RAM  *********/
	put_on_disc = 0;    /** default **/
	if(v_coordlistm.malloc != (char *) 0)       
	{		/******************** check for enough existing malloc'd space **/
		if((v_coordlistm.rec0 + offset + v_coordlistm.rec_size) > v_coordlistm.recn)
		{		/******************** reallocate enough space ***************/
			size = v_coordlistm.recn - v_coordlistm.rec0;
			size += 2000 * v_coordlistm.rec_size;     /*** add 2000 records ***/
				/*   try to realloc space  */
			new_ptr = (char *) realloc_ltp(v_coordlistm.malloc,size);
			if(new_ptr != (char *) 0)
			{	v_coordlistm.malloc = new_ptr;
				v_coordlistm.rec0 = new_ptr;
				v_coordlistm.recn = new_ptr + size;
					/*   check for a meg free    */
				if(!ramfree(1000000))
					put_on_disc = 1;
			}
			else
				put_on_disc = 1;

			if(put_on_disc)         /*  dump file to disc   */
			{	sprintf(path,"%s/coord_list",temp_map_path);
				if(0 > (v_coordlist_fd = openf(path,RDWR)))
					return(1);
				if(0 > writeb(v_coordlistm.rec0,v_coordlistm.rec_size,job_stats[18],v_coordlist_fd))
					return(1);
				free_ltp(v_coordlistm.malloc);
				v_coordlistm.malloc = (char *) 0;
				v_coordlistm.rec0 = (char *) 0;
				v_coordlistm.recn = (char *) 0;
			}
		}
			/******************** store in ram ******************************/
		if(!put_on_disc)
		{	to_ptr = v_coordlistm.rec0 + offset;
			memcpy(to_ptr,(char *)ptr,v_coordlistm.rec_size);
		}
	}

	if(v_coordlistm.malloc == 0)
	{		/******************** check for file pointer ********************/
		if(0 > v_coordlist_fd)
		{	message("put_v_coord, there is no file pointer, rejecting.");
			return(1);
		}
			/******************** position file write pointer ***************/
		if(0 > movefp(v_coordlist_fd,offset,0))
			return(1);
			/******************** write record ******************************/
		if(0 > writeb((char *)ptr, v_coordlistm.rec_size ,1 ,v_coordlist_fd))
			return(1);
	}

		/******************** update job_stats file *********************/
	job_stats[18] = (rec_no < job_stats[18]) ? job_stats[18] : (rec_no + 1);

		/******************** return ************************************/
	return(0);
}
