/*******************************************************************************
*       put_vobjects    (see manual, Internal Functions)                <in>   *
*                       This program stores the information in one v_objects   *
*                       structure in either malloc space or the temp file.     *
*                       (whichever is being used as data storage)              *
*                                                                              *
*                       The malloc space/file is changed.                      *
*                       The malloc struct may be changed.                      *
*                       The job_stats[17] entry is updated.                    *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  9/18/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <string.h>
#include <malloc.h>


long put_vobjects(ptr,rec_no)
V_OBJECTS *ptr;                /*  pointer to object structure to be saved  */
long rec_no;          /*  one greater than record number for save    */
{
	char *new_ptr;
	char path[512];
	char *to_ptr;

	long offset;
	long put_on_disc;
	long size;

		/******************** check for legitimate parameters ***********/
	if(!ptr  ||  rec_no < 0 || (rec_no > job_stats[17]))
	{	message("put_vobjects, bad parameters received, rejected.");
		return(1);
	}

		/******************** calculate offset **************************/
	offset = rec_no * v_objectsm.rec_size;

		/******************** see if current storage is in RAM  *********/
	put_on_disc = 0;    /** default **/
	if(v_objectsm.malloc != (char *) 0)       
	{		/******************** check for enough existing malloc'd space **/
		if((v_objectsm.rec0 + offset + v_objectsm.rec_size) > v_objectsm.recn)
		{		/******************** reallocate enough space ***************/
			size = v_objectsm.recn - v_objectsm.rec0;
			size += 2000 * v_objectsm.rec_size;     /*** add 2000 records ***/
				/*   try to realloc space  */
			new_ptr = (char *) realloc_ltp(v_objectsm.malloc,size);
			if(new_ptr != (char *) 0)
			{	v_objectsm.malloc = new_ptr;
				v_objectsm.rec0 = new_ptr;
				v_objectsm.recn = new_ptr + size;
					/*   check for a meg free    */
				if(!ramfree(1000000))
					put_on_disc = 1;
			}
			else
				put_on_disc = 1;

			if(put_on_disc)         /*  dump file to disc   */
			{	sprintf(path,"%s/v_objects",temp_map_path);
				if(0 > (v_objects_fd = openf(path,RDWR)))
					return(1);
				if(0 > writeb(v_objectsm.rec0,v_objectsm.rec_size,job_stats[17],v_objects_fd))
					return(1);
				free_ltp(v_objectsm.malloc);
				v_objectsm.malloc = (char *) 0;
				v_objectsm.rec0 = (char *) 0;
				v_objectsm.recn = (char *) 0;
			}
		}
			/******************** store in ram ******************************/
		if(!put_on_disc)
		{	to_ptr = v_objectsm.rec0 + offset;
			memcpy(to_ptr, (char *)ptr, (int)v_objectsm.rec_size);
		}
	}

	if(v_objectsm.malloc == 0)
	{		/******************** check for file pointer ********************/
		if(0 > v_objects_fd)
		{	message("put_vobjects, there is no file pointer, rejecting.");
			return(1);
		}
			/******************** position file write pointer ***************/
		if(0 > movefp(v_objects_fd,offset,0))
			return(1);
			/******************** write record ******************************/
		if(0 > writeb((char *)ptr, v_objectsm.rec_size ,1 ,v_objects_fd))
			return(1);
	}

		/******************** update job_stats file *********************/
	job_stats[17] = (rec_no < job_stats[17]) ? job_stats[17] : (rec_no + 1);

		/******************** return ************************************/
	return(0);
}
