/*******************************************************************************
*       query_xy        (see manual, Internal Functions)                <in>   *
*    Finds objects using the input x,y location & search radius.  Displays     *
*    the object number, serial number & first 2 attributes on the screen.      *
*                                                                              *
*    Arguments:    float x,y,r:   raster coordinates and search radius.        *
*                                 (search radius used for segs & nodes)        *
*                  long otype:    0 = node,  1 = segment,  2 = polygon         *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified:  none.                                         *
*                                                                              *
*                      j.dabritz  8/13/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"

long query_xy(x,y,r,otype)
float x,y,r;
long otype;
{	
	OBJECTS object;
	ATTRIBUTES atr_rec;
	V_OBJECTS v_obj;

	long obj_no;
	long ot10, level;

	double length, area;

	char string[256];
	char string2[256];

		/********* check for map ****************************************/
	if(!job_stats[0])
	{	message("no current map.");
		return(1);
	}

		/********* check for objects ************************************/
	ot10 = otype % 10;
	level = otype / 10;
	if((ot10 <0) || (ot10 > 2))
	{	message("Illegal object type.  query aborted.");
		return(1);
	}
	if((level != 0) && ((level < 32) || (level > 95)))
	{	message("Illegal level.  query aborted.");
		return(1);
	}

		/********* find object ******************************************/
	switch(ot10)     {
	case 0:           /** nodes ***/
		obj_no = find_vnode(x,y,r,level);
		break;
	case 1:           /** segments  */
		obj_no = find_vseg(x,y,r,level);
		break;
	case 2:           /** polygons  */
		obj_no = find_vpoly(x,y,level);
		break;
	default:
		message("Illegal object type.  Query not performed.");
		return(0);
	}


		/********* display object data **********************************/
	if(!level)   /** map object **/
	{	if((obj_no >= 0)  && (obj_no < job_stats[10]))
		{		/**** get object record *****/
			if(get_objects(&object,obj_no))
				return(1);
				/**** get attribute record ****/
			if(object.i12[2])
			{	if(get_attr(&atr_rec,object.i12[2]))
					return(0);
			}	
			else
			{	atr_rec.string[0] = 0;
				atr_rec.next = 0;
			}
	
				/**** get unit conversion factors ***/
			get_units();

				/**** calculate lenght/perimeter & area ****/
			if(unit_factor[2][0] > 0.0)
			{	length = object.f4[2] * unit_factor[2][0] + 0.5;/* pixels to feet */
				if(object.i12[0] > 10)    /** polygon **/
					area = object.f4[3] * unit_factor[2][1] + 0.005;  /* acres*/
				else
					area = 0.0;
				sprintf(string2,"len/perim= %.0f ft, area= %.2f acres",length,area);
			}
			else
			{	length = object.f4[2] + 0.5;  /* pixels */
				if(object.i12[0] > 10)    /** polygon **/
					area = object.f4[3] + 0.005;    /* sq pix */
				else
					area = 0.0;
				sprintf(string2,"len/perim= %.0f pix, area= %.2f pix",length,area);
			}
	
				/**** display message ***/
			sprintf(string,"object no:%7d,  serial:%7d,  1st atrbt:  %s",
		                   	obj_no,object.i12[1],atr_rec.string);
			message(string);
			if(atr_rec.next)
			{	if(get_attr(&atr_rec,atr_rec.next))
					return(0);
				sprintf(string,"%s,  2nd atrbt:  %s",string2 , atr_rec.string);
			}
			else
				sprintf(string,"%s,  2nd atrbt:",string2);
	
			message(string);
		}
		else
			message("  Object not found within search radius.");
	}
	else            /*** imported vector object ***/
	{	if((obj_no >= 0)  && (obj_no < job_stats[17]))
		{		/**** get object record *****/
			if(get_vobjects(&v_obj,obj_no))
				return(1);
				/**** get attribute record ****/
			if(v_obj.atr)
			{	if(get_v_attr(&atr_rec,v_obj.atr))
					return(0);
			}	
			else
			{	atr_rec.string[0] = 0;
				atr_rec.next = 0;
			}
	
	
				/**** display message ***/
			sprintf(string,"v_obj no:%7d,  1st atrbt:  %s",
		                   	obj_no,atr_rec.string);
			message(string);
			if(atr_rec.next)
			{	if(get_v_attr(&atr_rec,atr_rec.next))
					return(0);
				sprintf(string,"              2nd atrbt:  %s",atr_rec.string);
			}
			else
				sprintf(string,"              2nd atrbt:");
			message(string);
		}
		else
			message("  Object not found within search radius.");
	}

		/********* return ***********************************************/
	return(0);
}
