/*******************************************************************************
*	reacquire(see manual, Command Ref., User Functions)	<fc>   *
*	         	closes files, frees                space, and re-acquires      *
*			the temp directory for use as the current map.         *
*			                                                       *
*			Returns 1 if successful, 0 if not.                     *
*			j.dabritz, 5/31/89                                     *
*******************************************************************************/

#include <string.h>
#include "../../incl/globals.h"
#include "../../incl/in.h"
#include <malloc.h>

long reacquire( )
{	
	
	int filefd;

	long h,i,w;
	long hold_version;
	long row;
	unsigned long raster_size, size;

	char path[512];
	
	OBJECTS objects;
	ATTRIBUTES v_attributes;


		/******** close files & free space *************/
	message("re-acquiring temporary map file . . . ");
	close_files();
	free_malloc();


		/*  load fixed map globals (header/registration etc)               */
			/* header */
message("getting header, history, level_names job_stats & registration.");
	sprintf(path,"%s/header",temp_map_path);
	if(get_header(path,&header,1))
	{	message("  <<  cannot read map header file.");
		return(1);
	}


			/* history */
	sprintf(path,"%s/history",temp_map_path);
	if(get_history(path))
	{	message("  <<  cannot read map history file.");
		return(1);
	}


			/* level_names */
	sprintf(path,"%s/level_names",temp_map_path);
	if(get_levelnms(path))
	{	message("  <<  cannot read map level_names file.");
		return(1);
	}


			/* job_stats */
	sprintf(path,"%s/job_stats",temp_map_path);
	if(get_jobstats(path))
	{	message("  <<  cannot read job_stats map file.");
		job_stats[0] = 0;
		return(1);
	}
	hold_version = job_stats[0];
	job_stats[0] = 0;                   /* set to 0 until all file is loaded */
	job_stats[78] = 0;        /*** reset VIEW ONLY flag ****/
		/*  set the raster_wincrement and raster_bincrement globals  */
	raster_wincrement = (job_stats[1] + 1) >> 5;
	raster_bincrement = (job_stats[1] + 1) >> 3;


			/* register */
	sprintf(path,"%s/register",temp_map_path);
	if(get_register(path))
	{	message("  <<  cannot read register map file.");
		return(1);
	}


/********** load raster ******************************************************/
message("getting raster.");
		/***********  calculate sizes **************/
	h = job_stats[2] + 1;             /* height of raster in rows  */
	w = (job_stats[1] + 1) / 8;       /* width of raster in bytes  */
	raster_size = h*w;                /* size of raster in bytes   */
	
	size = raster_size + 4*w + 8*h;   /* raster+4lines + space for lraster/r */
		/***********  allocate memory **************/
	rasterm.malloc = (char *) calloc_ltp(size,1);
	if(!rasterm.malloc)
	{	message("  <<  cannot get enough RAM for raster.");
		return(1);
	}
		/********  calculate & set global pointers to raster ****************/

	rasterm.rec0 = (char *)(rasterm.malloc + 2*w + 8*h);
	rasterm.recn = rasterm.rec0 + raster_size;
	craster = (unsigned char **)(rasterm.malloc);
	lraster = (unsigned long **)(rasterm.malloc + 4*h);
	for(i=0; i<h; i++)
	{	craster[i] = (unsigned char *)rasterm.rec0 + w*i;
		lraster[i] = (unsigned long *) (craster[i]);
	}

		/***********  read raster from file *********************/
	sprintf(path,"%s/raster",temp_map_path);
	filefd = openf(path,READ);
	if(0 > filefd)
	{	message("  <<  cannot open raster map file.");
		return(1);
	}
	if(0 >= readb(rasterm.rec0,raster_size,1,filefd))
	{	message("  <<  cannot read raster map file.");
		closef(filefd);
		return(1);
	}


		/*  load mini-raster                         */
message("getting mini-raster.");
	sprintf(path,"%s/mini_raster",temp_map_path);
	filefd = openf(path,READ);
	if(0 > filefd)
	{	message("  <<  cannot open mini_raster map file.");
		return(1);
	}
	if(0 >= readb(mini_raster[0],sizeof(mini_raster),1,filefd))
	{	message("  <<  cannot read mini_raster map file.");
		return(1);
	}
	closef(filefd);


		/*  load index_array                    */
message("getting index_array.");
		/***********  calculate sizes **************/
	h = job_stats[2] / 32 + 1;        /* height of index_array (rows) */
	w = (job_stats[1] / 32 + 1) * 4;  /* width of index_array (bytes)*/
	size = (h*w + 4*h);               /* size to malloc            */
		/***********  allocate memory **************/
	index_arraym.malloc = (char *) calloc_ltp(size,1);
	if(!index_arraym.malloc)
	{	message("  <<  cannot get enough RAM for index_array.");
		return(1);
	}
		/********  calculate & set global pointers to index_array ***********/
	index_arraym.rec0 = index_arraym.malloc;
	index_arraym.recn = (char *)(index_arraym.rec0 + h*w);
	index_array = (long **)index_arraym.recn;
	for(i=0; i<h; i++)
		index_array[i] = (long *) (index_arraym.malloc + w*i);

		/***********  read index_array from file *********************/
	sprintf(path,"%s/index_array",temp_map_path);
	filefd = openf(path,READ);
	if(0 > filefd)
	{	message("  <<  cannot open index_array map file.");
		return(1);
	}
	if(0 > readb(index_arraym.rec0,h*w,1,filefd))
	{	message("  <<  cannot read index_array map file.");
		closef(filefd);
		return(1);
	}
	closef(filefd);


		/*  load record type files if space allows or set up file pointers */
		/********* objects ***********/
		/*    open file        */
message("getting objects.");
	sprintf(path,"%s/objects",temp_map_path);
	objects_fd = openf(path,RDWR);
	if(0 > objects_fd)
	{	message("  <<  cannot open objects map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = objectsm.rec_size * job_stats[10];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(objectsm.malloc = (malloc_ltp(size)))
			{   objectsm.rec0 = objectsm.malloc;
				objectsm.recn = objectsm.rec0 + size;
				if(!readb(objectsm.rec0,size,1,objects_fd))
				{	message("  <<  cannot read objects map file.");
					return(1);
				}
			}
		}
	}
		/***** build node_lookup (node lookup array) *********/
		/*  free old space if any */
	if(node_lookup_ptr != (char *) 0)
		free_ltp(node_lookup_ptr);
		/*  calloc new space  */
	node_lookup_ptr = calloc_ltp(job_stats[2]+1,4);
	if(node_lookup_ptr == (char *) 0)
	{	message("  <<  cannot calloc new space for node_lookup table.");
		return(1);
	}
	else
	{	node_lookup = (long *) node_lookup_ptr;
		for(i=0; i<job_stats[10]; i++)
		{	get_objects(&objects,i);
			if((objects.i12[0] < 8) && (objects.i12[0] != 5)) /* non-is node */
			{	row = objects.f4[1];
				if((i < node_lookup[row])  ||  !(node_lookup[row]))
					node_lookup[row] = i;
			}
		}
	}	

		/********* seg_list ***********/
		/*    open file        */
message("getting seg_list.");
	sprintf(path,"%s/seg_list",temp_map_path);
	seg_list_fd = openf(path,RDWR);
	if(0 > seg_list_fd)
	{	message("  <<  cannot open seg_list map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = seg_listm.rec_size * job_stats[12];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(seg_listm.malloc = (malloc_ltp(size)))
			{   seg_listm.rec0 = seg_listm.malloc;
				seg_listm.recn = seg_listm.rec0 + size;
				if(!readb(seg_listm.rec0,size,1,seg_list_fd))
				{	message("  <<  cannot read seg_list map file.");
					return(1);
				}
			}
		}
	}


		/********* coord_list ***********/
		/*    open file        */
message("getting coord_list.");
	sprintf(path,"%s/coord_list",temp_map_path);
	coord_list_fd = openf(path,RDWR);
	if(0 > coord_list_fd)
	{	message("  <<  cannot open coord_list map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = coord_listm.rec_size * job_stats[11];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(coord_listm.malloc = (malloc_ltp(size)))
			{   coord_listm.rec0 = coord_listm.malloc;
				coord_listm.recn = coord_listm.rec0 + size;
				if(!readb(coord_listm.rec0,size,1,coord_list_fd))
				{	message("  <<  cannot read coord_list map file.");
					return(1);
				}
			}
		}
	}


		/********* index_list ***********/
		/*    open file        */
message("getting index_list.");
	sprintf(path,"%s/index_list",temp_map_path);
	index_list_fd = openf(path,RDWR);
	if(0 > index_list_fd)
	{	message("  <<  cannot open index_list map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = index_listm.rec_size * job_stats[15];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{
		if(ramfree(size + RAMSPACE))
		{
			if(index_listm.malloc = (malloc_ltp(size)))
			{
			   index_listm.rec0 = index_listm.malloc;
				index_listm.recn = index_listm.rec0 + size;
				if(!readb(index_listm.rec0,size,1,index_list_fd))
				{
					message("  <<  cannot read index_list map file.");
					return(1);
				}
			}
		}
	}


		/********* v_objects ***********/
		/*    open file        */
message("getting v_objects.");
	sprintf(path,"%s/v_objects",temp_map_path);
	v_objects_fd = openf(path,RDWR);
	if(0 > v_objects_fd)
	{	message("  <<  cannot open v_objects map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = v_objectsm.rec_size * job_stats[17];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(v_objectsm.malloc = (malloc_ltp(size)))
			{   v_objectsm.rec0 = v_objectsm.malloc;
				v_objectsm.recn = v_objectsm.rec0 + size;
				if(!readb(v_objectsm.rec0,size,1,v_objects_fd))
				{	message("  <<  cannot read v_objects map file.");
					return(1);
				}
			}
		}
	}


		/********* v_coordlist ***********/
		/*    open file        */
message("getting v_coordlist.");
	sprintf(path,"%s/v_coordlist",temp_map_path);
	v_coordlist_fd = openf(path,RDWR);
	if(0 > v_coordlist_fd)
	{	message("  <<  cannot open v_coordlist map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = v_coordlistm.rec_size * job_stats[18];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(v_coordlistm.malloc = (malloc_ltp(size)))
			{   v_coordlistm.rec0 = v_coordlistm.malloc;
				v_coordlistm.recn = v_coordlistm.rec0 + size;
				if(!readb(v_coordlistm.rec0,size,1,v_coordlist_fd))
				{	message("  <<  cannot read v_coordlist map file.");
					return(1);
				}
			}
		}
	}


		/********* attributes ***********/
		/*    open file        */
message("getting attributes.");
	sprintf(path,"%s/attributes",temp_map_path);
	attributes_fd = openf(path,RDWR);
	if(0 > attributes_fd)
	{	message("  <<  cannot open attributes map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = attributesm.rec_size * job_stats[13];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(attributesm.malloc = (malloc_ltp(size)))
			{   attributesm.rec0 = attributesm.malloc;
				attributesm.recn = attributesm.rec0 + size;
				if(!readb(attributesm.rec0,size,1,attributes_fd))
				{	message("  <<  cannot read attributes map file.");
					return(1);
				}
			}
		}
	}
		/** make sure attributes has at least one record **/
	if(!job_stats[13])
	{	*v_attributes.string = 0;
		v_attributes.next = 0;
		if(put_atrbt(&v_attributes,0))
			return(1);
	}


		/********* v_attributes ***********/
		/*    open file        */
message("getting v_attributes.");
	sprintf(path,"%s/v_attributes",temp_map_path);
	v_attributes_fd = openf(path,RDWR);
	if(0 > v_attributes_fd)
	{	message("  <<  cannot open v_attributes map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = v_attributesm.rec_size * job_stats[16];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(v_attributesm.malloc = (malloc_ltp(size)))
			{   v_attributesm.rec0 = v_attributesm.malloc;
				v_attributesm.recn = v_attributesm.rec0 + size;
				if(!readb(v_attributesm.rec0,size,1,v_attributes_fd))
				{	message("  <<  cannot read v_attributes map file.");
					return(1);
				}
			}
		}
	}
		/** make sure v_attributes has at least one record **/
	if(!job_stats[16])
	{	*v_attributes.string = 0;
		v_attributes.next = 0;
		if(put_v_attr(&v_attributes,0))
			return(1);
	}


		/********* errors ***********/
		/*    open file        */
message("getting errors.");
	sprintf(path,"%s/errors",temp_map_path);
	errors_fd = openf(path,RDWR);
	if(0 > errors_fd)
	{	message("  <<  cannot open errors map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = errorsm.rec_size * job_stats[14];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(errorsm.malloc = (malloc_ltp(size)))
			{   errorsm.rec0 = errorsm.malloc;
				errorsm.recn = errorsm.rec0 + size;
				if(!readb(errorsm.rec0,size,1,errors_fd))
				{	message("  <<  cannot read errors map file.");
					return(1);
				}
			}
		}
	}


		/*** reset undo array    **********/
	for(i=0; i<32; i++)
		undo_array[i].mode = 0;

		/*********  set up graphics & display     */
message("setting up graphics.");
	job_stats[0] = hold_version;
	for(i=0; i<4; i++)
	{	view[i].lower_leftx = 0;
		view[i].lower_lefty = 0;
	}
	view[0].zoom_factor = mind(2,  (double) 1.0, 
			(double) (400.0/(maxl(2,job_stats[1]+1,job_stats[2]+1))));
	view[1].zoom_factor = 0.25;
	view[2].zoom_factor = 1.0;
	view[3].zoom_factor = 8.0;


message("re-displaying map.");
	redisp_map();

	cursor(561,80);
	cursor(560,80);


		/*   tell user what is going on  */
	append_msg(" done.");

    return(0);
}
