/*******************************************************************************
*    recalc_area   (see manual, Command Ref., Internal Functions)   <in>       *
*            This program recalculates the lenght & area for all map           *
*       segments & polygons.                                                   *
*                                                                              *
*       files/globals changed:  objects                                        *
*                                                                              *
*       values returned:  long 0= ok,  1=failure                               *
*                                                                              *
*                                                                              *
*            J.Dabritz   8/8/90                                                *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>


long recalc_area()
{
	OBJECTS object,segment,parent;
	COORD_LIST coordlist;

	char string[STRLIST];

	double area,length;
	double dx,dy;
	double last_x,last_y;

	long i;
	long begin,end;
	long v;
	long sg;
	long seg;


		/************* recalculate segments first *******************/
	for(i=0; i<job_stats[10]; i++)
	{		/**** inform user ****/
		if(!(i % 100))
		{	sprintf(string,"Recalculating lengths, object %d.",i);
			message(string);
		}

		if(get_objects(&object,i))
		{	message("level_snap, abort, location 12.");
			return(1);
		}
		if((object.i12[0] < 14) && (object.i12[0] >  7))      /**  segment **/
		{	length = 0.0;
			area = 0.0;
			begin = object.i12[5];
			end = object.i12[6];
			for(v = begin;  v<= end;  v++)
			{
				if(get_coordlst(&coordlist,v))
				{	message("level_snap, abort, location 13.");
					return(1);
				}
				if(v > begin)
				{	dx = coordlist.x - last_x;
					dy = coordlist.y - last_y;
					length += sqrt((dx*dx) + (dy*dy));
					area += dx * (coordlist.y + last_y);  /* double area */
				}
				last_x = coordlist.x;
				last_y = coordlist.y;
			}
			area = area / 2;
			object.f4[2] = length;
			object.f4[3] = area;

			if(put_objects(&object,i))
			{	message("level_snap, abort, location 18.");
				return(1);
			}
		}
	}

		/************* recalculate polygons next **********************/
	for(i=0; i<job_stats[10]; i++)
	{		/**** inform user ****/
		if(!(i % 100))
		{	sprintf(string,"Recalculating areas, object %d.",i);
			message(string);
		}

		if(get_objects(&object,i))
		{	message("level_snap, abort, location 12.");
			return(1);
		}

		if(object.i12[0] >= 14)      /**  polygon **/
		{		/** calculate polygon perimeter & area **/
			begin = object.i12[5];
			end = object.i12[6];
			area = 0.0;
			length = 0.0;
			for(sg=begin; sg<= end; sg++)
			{	if(get_seglist(&seg,sg))
				{	message("level_snap, abort, location 14.");
					return(1);
				}
				if(seg > 0)
				{	if(get_objects(&segment,seg))
					{	message("level_snap, abort, location 15.");
						return(1);
					}
					length += segment.f4[2];
					area += segment.f4[3];
				}
				else
				{	if(get_objects(&segment,-seg))
					{	message("level_snap, abort, location 15.");
						return(1);
					}
					length += segment.f4[2];
					area += -segment.f4[3];
				}
			}
			object.f4[2] = length;
			object.f4[3] = area;
			if(put_objects(&object,i))
			{	message("level_snap, abort, location 18.");
				return(1);
			}
		}
	}

		/************* subtract out islands last *******************/
	for(i=0; i<job_stats[10]; i++)
	{		/**** inform user ****/
		if(!(i % 1000))
		{	sprintf(string,"Subtracting islands, object %d.",i);
			message(string);
		}

		if(get_objects(&object,i))
		{	message("level_snap, abort, location 12.");
			return(1);
		}
		if(object.i12[0] == 15)         /* island polygon */
		{	if(get_objects(&parent,object.i12[7]))
			{	message("level_snap, abort, location 16.");
				return(1);
			}
	
			parent.f4[3] += object.f4[3];   /** subtract island area **/

			if(put_objects(&parent,object.i12[7]))
			{	message("level_snap, abort, location 17.");
				return(1);
			}
		}
	}

	return(0);
}
