/*******************************************************************************
*       recall_mode     (see manual, Internal Functions)                <in>   *
*                                                                              *
*            This serves as the continue (&sometimes start) program for        *
*       when the input mode is one of the recall modes.                        *
*            modes served:          18 = recall point                          *
*                                   19 = recall line                           *
*                                   20 = recall fat-line                       *
*                                   21 = recall rectangle border               *
*                                   22 = recall rectangle area                 *
*                                   23 = recall wedge                          *
*                                   24 = recall separate                       *
*                                   25 = recall connect                        *
*                                                                              *
*        parameters:  long mode;     mode of recall                            *
*                     long x2,y2;    oldest coordinate pair, raster coords     *
*                     long x1,y1;    old coordinate pair, raster coords        *
*                     long x,y;      newest coordinate pair, raster coords     *
*                                                                              *
*       globals/files changed:  (globals) lraster, mini-raster                 *
*                                                                              *
*       returns:   (long) 0 = successful,  1 = failure                         *
*                                                                              *
*                      j.dabritz  1/4/90                                       *
*******************************************************************************/

#include "../../incl/globals.h"

int raster_fd;
long braster;


long recall_mode(mode,x2,y2,x1,y1,x,y)
	long mode,x2,y2,x1,y1,x,y;
{
	char string[512];

	long dx,dy;
	long i,j;
	long px1,px2,px3;
	long ymin,ymax;
	long xmin,xmax;
	long hit;

	double slope;


		/******************* check for map loaded **********************/
	if(!job_stats[0])
		return(1);

		/***************** open raster file ****************************/
	if(braster && israsterb())
		sprintf(string,"%s/rasterb",temp_map_path);
	else
		sprintf(string,"%s/raster",temp_map_path);

	raster_fd = openf(string,READ);
	if(raster_fd == -1)
		return(1);


		/***************** make sure x & y are on raster ***************/
	if(x < 0)
		x = 0;
	else if(x > job_stats[1])
		x = job_stats[1];

	if(y < 0)
		y = 0;
	else if(y > job_stats[2])
		y = job_stats[2];


	switch(mode)  {
 	case 25:          /* recall connect                 */
		dx = x - x1;
		dx = (dx > -dx)  ?  dx : -dx;
		dy = y - y1;
		dy = (dy > -dy)  ?  dy : -dy;
		if(dy > dx)
		{	dy = 0;
			dx = 1;
		}
		else
		{	dy = 1;
			dx = 0;
		}
		/** erase diagonal lines on both edges **/
		line_diag(x1+dx,y1+dy,x+dx,y+dy,2);
		line_diag(x1-dx,y1-dy,x-dx,y-dy,2);
			/** draw diagonal line separating edges **/
		line_diag(x1,y1,x,y,2);
			/** fill in the erased line corners to make them solid **/
		fill_concnrs(x1,y1,x,y);
		break;
 	case 18:         /* recall point            */
		pixel(x,y,2);
		break;
 	case 19:         /* recall line             */
		line_solid(x1,y1,x,y,2);
		break;
 	case 20:         /* recall fat-line         */
		dx = x - x1;
		dx = (dx > -dx)  ?  dx : -dx;
		dy = y - y1;
		dy = (dy > -dy)  ?  dy : -dy;
		if(dy > dx)
		{	dy = 0;
			dx = 1;
		}
		else
		{	dy = 1;
			dx = 0;
		}
		for(i= -1; i<2; i++)
			line_solid(x1+i*dx,y1+i*dy,x+i*dx,y+i*dy,2);
		break;
 	case 21:         /* recall rectangle border */
		line_diag(x1,y1,x1,y,2);
		line_diag(x1,y,x,y,2);
		line_diag(x,y,x,y1,2);
		line_diag(x,y1,x1,y1,2);
		break;
	case 22:          /* recall rectangle area   */
		if(x<x1)
		{	xmin = x;
			xmax = x1;
		}
		else
		{	xmin = x1;
			xmax = x;
		}
		if(y<y1)
		{	ymin = y;
			ymax = y1;
		}
		else
		{	ymin = y1;
			ymax = y;
		}
		for(i=ymin; i<=ymax; i++)
		{	for(j=xmin; j<=xmax; j++)
				pixel(j,i,2);
			if(abort_cmd())
				break;
		}
		break;
	case 24:          /* recall separate         */
		dx = x - x1;
		dx = (dx > -dx)  ?  dx : -dx;
		dy = y - y1;
		dy = (dy > -dy)  ?  dy : -dy;
		if(dy > dx)
		{	dy = 0;
			dx = 1;
		}
		else
		{	dy = 1;
			dx = 0;
		}

			/** draw solid lines on both edges **/
		line_solid(x1+dx,y1+dy,x+dx,y+dy,2);
		line_solid(x1-dx,y1-dy,x-dx,y-dy,2);
			/** erase solid line separating edges **/
		line_solid(x1,y1,x,y,2);
			
		break;
	case 23:          /* recall wedge        */
		line_diag(x1,y1,x,y,2);
		line_diag(x2,y2,x,y,2);
		line_diag(x2,y2,x1,y1,2);
			/* find y limits of triangle */
		ymin = ymax = y;
		if(y1 > ymax)
			ymax = y1;
		if(y2 > ymax)
			ymax = y2;

		if(y1 < ymin)
			ymin = y1;
		if(y2 < ymin)
			ymin = y2;


			/** erase horizontal lines within triangle from bottom to top */
		for(ymin++;  ymin < ymax; ymin++)
		{		/** find intersection of row with triangle sides **/
			hit = 0;
				/** side anchor to anchor2 **/
			if(((ymin <= y1) && (ymin >= y2))   ||
			   ((ymin <= y2) && (ymin >= y1)))
			{	dx = x2 - x1;
				dy = y2 - y1;
				if(dy)
				{	slope = dx / (double) dy;
					px1 = px2 = x1 + (ymin - y1) * slope + 0.5;
					hit = 1;
				}
			}
				/** side anchor to x,y     **/
			if(((ymin <= y1) && (ymin >= y))   ||
			   ((ymin <= y) && (ymin >= y1)))
			{	dx = x - x1;
				dy = y - y1;
				if(dy)
				{	slope = dx / (double) dy;
					px3 = x1 + (ymin - y1)* slope + 0.5;
					if(!hit)
					{	px1 = px2 = px3;
						hit = 1;
					}
					else
					{	px1 = (px3 < px1) ?   px3 : px1;
						px2 = (px3 > px2) ?   px3 : px2;
					}
				}
			}
				/** side anchor2 to x,y    **/
			if(((ymin <= y2) && (ymin >= y))   ||
			   ((ymin <= y) && (ymin >= y2)))
			{	dx = x - x2;
				dy = y - y2;
				if(dy)
				{	slope = dx / (double) dy;
					px3 = x2 + (ymin - y2)* slope + 0.5;
					if(!hit)
					{	px1 = px2 = px3;
						hit = 1;
					}
					else
					{	px1 = (px3 < px1) ?   px3 : px1;
						px2 = (px3 > px2) ?   px3 : px2;
					}
				}
			}
				/** erase row between intersects **/
			if(hit)
			{	for(px3=px1; px3 <= px2; px3++)
					pixel(px3,ymin,2);
			}
			if(abort_cmd())
				break;
		}
		break;
	}

		/***************** close raster file *************************/
	closef(raster_fd);

		/***************** return ************************************/
	return(0);
}

/*******************************************************************************
*       recall_xy       (see manual, Internal Functions)                <in>   *
*                                                                              *
*            This program gets an xy location passed to it, looks up the       *
*       bit in the temp/raster  or temp/rasterb file and returns a boolean     *
*       telling if it is on.                                                   *
*                                                                              *
*       parameters long x,y;    raster coordinates of location to check        *
*                                                                              *
*       globals/files changed:  none                                           *
*                                                                              *
*       returns:   (long) value of bit:  0 = off,  1 = on                      *
*                                                                              *
*                      j.dabritz  1/4/90                                       *
*******************************************************************************/


long recall_xy(x,y)
long x,y;
{
	static long last_y = -1;
	static long last_word;
	static long rword;
	long word, bit;
	long offset;


		/************** check to see if xy is on the raster ************/
	if(x<0  ||  x>job_stats[1]  ||  y<0  ||  y>job_stats[2])
		return(0);

		/************** calc word & bit ********************************/
	word = x >> 5;
	bit  = (7 & ~x) | (x & 24);

		/************** see if we already have the right word **********/
	if((y==last_y)  &&  (word==last_word))
		return((rword >> bit) & 1);

		/************** calculate the offset in the raster file ********/
	offset = y * ((job_stats[1] + 1) >> 3) + 4 * word;

		/******************** position file read pointer ***************/
	if(0 > movefp(raster_fd,offset,0))
		return(0);

		/************** read word **************************************/
	if(0 > readb((char *) &rword, 4, 1 ,raster_fd))
		return(0);

		/************** set last values  *******************************/
	last_y = y;
	last_word = word;

		/************** return bit value *******************************/
	return((rword >> bit) & 1);
}





/*******************************************************************************
*    recall_rset                                                               *
*        This program sets the braster local/global to 0 (for recall from      *
*    the raster) or to 1 (for recall from the braster).                        *
*                                                                              *
*    it returns the prior value of the braster local-global                    *
*                                                                              *
*    J.Dabritz  9/22/90                                                        *
*******************************************************************************/

recall_rset(b)
long b;
{
	long last_braster;

	last_braster = braster;
	braster = b;
	return(last_braster);
}
