/*******************************************************************************
*       reduce_rdata    (see manual, Internal Functions)                <in>   *
*                                                                              *
*        Reduces raster vertex data.                                           *
*    This program reduces the number of (and changes the values of) vertex     *
*    data that has just been vectorized.  It assumes that all verticies        *
*    corrispond to vectors in even 45 degree increments and are whole pixel    *
*    values.  This program can only reduce vectorized data once.  It is        *
*    designed to remove verticies which are artifacts of the rastorization     *
*    process and which do not contribute to the macroscopic shape of the       *
*    line.  This is not a generalized data reduction routine.  It is very      *
*    specialized to deal with vectorized raster data.                          *
*        After data reduction, it recalcuates lengths and areas of the         *
*    segments & polygons.                                                      *
*                                                                              *
*    Arguments:    long reduce;  1= reduce vertex data, 0 = don't reduce       *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*    Global Variables Modified:  none, the objects and coord_list file are     *
*                                      changed.                                *
*                      j.dabritz  7/25/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long x, y;                     /*  points to move along segment vectors */
long vector[6][2];             /*  array of 6 current vectors (dir,len) */
long v1,v2,v3,v4,v5,v6;        /*  index to vector array (above)        */
long next_coord_out;           /*  index to reduced verticies written   */
long last_x, last_y, last_dir; /*  last values written/used             */
long move_x[8] = { 0,1,1,1,0,-1,-1,-1 };   /* x movement                */
long move_y[8] = { 1,1,0,-1,-1,-1,0,1 };   /* y movement                */
long next_vertex, end_vertex;  /*  index to reading verticies           */
long last_x_read, last_y_read; /*  last values read                     */
long start, twostep, finish;   /*  index values for segments/objects file */

double segment_area;           /*  area contribution of current segment */
double segment_length;         /*  length of current segment            */

COORD_LIST coord_list;         /*  x & y coord_list (file) structure     */

long defaultwrite();
long write_reduce();
long check3();
long check4();
long reduce3();
long reduce4();
long next_vector();



long reduce_rdata(reduce)
long reduce;
{
	long b,i,e,o,o2,s;         /*  counters                             */
	long o_end, o_start;       /*  limits to read from the objects file */
	long rec;
	long new_first_vertex;     /*  index to new first vertex record     */

	double area, perimeter;
	double x1,y1,x2,y2;

	char string[256];

	OBJECTS object,segment,island;  /* objects (file) structures        */


		/************* check for map & segments ***********************/
	if(!job_stats[0] || (!job_stats[5] && !job_stats[6]) )
		return(0);

		/************* set loop limits ********************************/
	start = (job_stats[5]) ? job_stats[5] : job_stats[6]; 
	twostep = (job_stats[6]) ?  job_stats[6] - 2: job_stats[10];
	finish = (job_stats[7]) ?  job_stats[7] : job_stats[10];

		/************* set initial index values ***********************/
	v1 = 0;
	v2 = 1;
	v3 = 2;
	v4 = 3;
	v5 = 4;
	v6 = 5;
	next_coord_out = 0;

	if(reduce)
		message("reducing data & recalculating lengths & area contributions . . .");
	else
		message("calculating length & area contributions . . .");
		/************* reduce vertex data on all segments *************/
	for(s = start;  s < finish  ; s += (s < twostep) ? 1 : 2)
	{		/** check for user abort **/
		if(abort_cmd())
			return(1);

			/************* get segment data ***************************/
		if(get_objects(&segment,s))
		{	message("abort, reduce_rdata, location 1");
			return(1);
		}

			/************* setup vector array *************************/
		next_vertex = segment.i12[5];
		new_first_vertex = next_coord_out;
		end_vertex = segment.i12[6];
		for(i=0; i<7; i++)
		{	if(next_vector())
			{	message("abort, reduce_rdata, location 2");
				return(1);
			}
		}
		last_dir = -1;

			/************* write initial vertex ***********************/
		if(get_coordlst(&coord_list,segment.i12[5]))
		{	message("abort, reduce_rdata, location 3");
			return(1);
		}
		if(put_coordlst(&coord_list,next_coord_out++))
		{	message("abort, reduce_rdata, location 4");
			return(1);
		}

			/************* initialize reduction variables *************/
		segment_length = 0.0;
		segment_area = 0.0;
		x = last_x = coord_list.x;
		y = last_y = coord_list.y;

			/************* reduce data through all vectors of segment */
			/*** check for reduction parameter **/
		if(reduce)
		{	for(;vector[v1][1];)
			{		/************* check for & do 4+ vector reduction *****/
				if(check4())
					reduce4();
					/************* otherwise check for & do 3 vector red. */
				else if(check3())
					reduce3();
					/************* otherwise write unreduced vector *******/
				else
				{	if(defaultwrite())
					{	message("abort, reduce_rdata, location 5");
						return(1);
					}
				}
			}
		}
		else
		{	for(;vector[v1][1];)
			{	if(defaultwrite())
				{	message("abort, reduce_rdata, location 6");
					return(1);
				}
			}
		}

			/************* update segment info ************************/
		segment.i12[5] = new_first_vertex;
		segment.i12[6] = next_coord_out - 1;
		segment.f4[2] = segment_length;
		segment.f4[3] = segment_area * 0.5;
			/************* store segment ******************************/
		if(put_objects(&segment,s))
		{	message("abort, reduce_rdata, location 7");
			return(1);
		}
	}
	append_msg("done.");
	message("packing centroids . . . ");
		/********** pack all secondary centroids & reclac poly perim & area  */
		/*  note:  to avoid overwritting good data, poly centroids must be   */
		/*  packed first, then segment centroids.    */
	o_end = (job_stats[7]) ?  job_stats[7]-1 : job_stats[10] - 1;
	if(job_stats[7])
		o_start = job_stats[7];
	else if(job_stats[5])
		o_start = job_stats[5];
	else 
		o_start = job_stats[6];
	o2 = (job_stats[5]) ?  job_stats[5] : job_stats[6];
	for( o = o_start ; ; o = (o == job_stats[10]-1) ? o2 : o + 1 )
	{
		if(get_objects(&object,o))
		{	message("abort, reduce_rdata, location 8");
			return(1);
		}
		if(object.i12[0] > 13)        /* polygon  */
		{	if(object.i12[3])         /** has 2ndry centroids, pack 'em **/
			{	b = object.i12[3];
				object.i12[3] = next_coord_out;
				e = object.i12[11];
				for(i=b; i<= e; i++)
				{	if(get_coordlst(&coord_list,i))
					{	message("abort, reduce_rdata, location 9");
						return(1);
					}
					if(put_coordlst(&coord_list,next_coord_out++))
					{	message("abort, reduce_rdata, location 10");
						return(1);
					}
				}
				object.i12[11] = next_coord_out - 1;
			}
				/** calc new area **/
			area = 0.0;
			perimeter = 0.0;
			for(rec = object.i12[5]; rec <= object.i12[6]; rec++)
			{	if(get_seglist(&s,rec))
				{	message("abort, reduce_rdata, location 11");
					return(1);
				}
				if(get_objects(&segment,abs(s)))
				{	message("abort, reduce_rdata, location 12");
					return(1);
				}
				if(s > 0)          /* segment used in forward direction  */
					area += segment.f4[3];
				else               /* segment used in reverse direction  */
					area -= segment.f4[3];
				perimeter += segment.f4[2];
			}
			object.f4[3] = area;
			object.f4[2] = perimeter;
			if(area == 1.0)           /* 1 pixel polygon, recalc centroid */
			{
				for(rec = object.i12[5]; rec <= object.i12[6]; rec += 2)
				{	if(get_seglist(&s,rec))
					{	message("abort, reduce_rdata, location 13");
						return(1);
					}
					if(get_objects(&segment,abs(s)))
					{	message("abort, reduce_rdata, location 14");
						return(1);
					}
					for(i=segment.i12[5]; i<= segment.i12[6]; i++)
					{	if(get_coordlst(&coord_list,i))
						{	message("abort, reduce_rdata, location 15");
							return(1);
						}
						if((rec == object.i12[5]) && (i == segment.i12[5]))
						{	x1 = x2 = coord_list.x;  /* first point */
							y1 = y2 = coord_list.y;
						}
						else
						{	x1 = (x1 < coord_list.x) ?  x1 : coord_list.x;
							x2 = (x2 > coord_list.x) ?  x2 : coord_list.x;
							y1 = (y1 < coord_list.y) ?  y1 : coord_list.y;
							y2 = (y2 > coord_list.y) ?  y2 : coord_list.y;
						}
					}
				}
					/**  set new (fractional) centroid value  */
				object.f4[0] = (x1 + x2) / 2.0;  /* new cent. x (xxx.5)  */
				object.f4[1] = (y1 + y2) / 2.0;  /* new cent. y (yyy.5)  */
			}
			if(put_objects(&object,o))
			{	message("abort, reduce_rdata, location 16");
				return(1);
			}
		}
		else if(object.i12[0] > 7)       /* segment  */
		{	if(object.i12[3])         /** has 2ndry centroids, pack 'em **/
			{	b = object.i12[3];
				object.i12[3] = next_coord_out;
				e = object.i12[11];
				for(i=b; i<= e; i++)
				{	if(get_coordlst(&coord_list,i))
					{	message("abort, reduce_rdata, location 17");
						return(1);
					}
					if(put_coordlst(&coord_list,next_coord_out++))
					{	message("abort, reduce_rdata, location 18");
						return(1);
					}
				}
				object.i12[11] = next_coord_out - 1;
				if(put_objects(&object,o))
				{	message("abort, reduce_rdata, location 19");
					return(1);
				}
			}
		}
		if(o == o_end)
			break;
	}

	append_msg("done.");
	message("subtracting island areas from parent polygons . . .");
		/************* subtract off island areas from parent polygons */
	o = (job_stats[7]) ? job_stats[7] : job_stats[10];
	for( ; o < job_stats[10]; o++)
	{	if(get_objects(&object,o))
		{	message("abort, reduce_rdata, location 20");
			return(1);
		}
		if(object.i12[8] && (object.i12[0] == 14))  /* polygon has islands */
		{	for(rec = object.i12[8]; rec; rec = island.i12[8])
			{	if(get_objects(&island,rec))
				{	message("abort, reduce_rdata, location 21");
					return(1);
				}
				object.f4[3] += island.f4[3];
			}
			if(put_objects(&object,o))
			{	message("abort, reduce_rdata, location 22");
				return(1);
			}
		}
	}
	append_msg("done.");

		/************* reset job_stats[11] (number of coord_list recs)*/
	if(reduce && job_stats[11])
	{	x2 = 100 * ((next_coord_out / (double) job_stats[11]));
		sprintf(string,"Reduced verticies occupy %.0f%% of the old space.",x2);
		message(string);
		sleep(2);
	}
	job_stats[11] = next_coord_out;

	
		/************* return *****************************************/
	return(0);
}


/*******************************************************************************
*       next_vector                                                            *
*                                                                              *
*        gets the next vertex & calculates the vector to it, updates           *
*    the vector array.                                                         *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*                      j.dabritz  7/25/89                                      *
*******************************************************************************/

long next_vector()
{
	long dx,dy;

		/**** increment index pointers **/
	v1 = (v1 + 1) % 6;
	v2 = (v2 + 1) % 6;
	v3 = (v3 + 1) % 6;
	v4 = (v4 + 1) % 6;
	v5 = (v5 + 1) % 6;
	v6 = (v6 + 1) % 6;
		/**** read next vertex (if any are left) ***/
	if(next_vertex <= end_vertex)
	{	if(get_coordlst(&coord_list,next_vertex++))
		{	message("abort, reduce_rdata, location 23");
			return(1);
		}
			/*** calc vector ***/
		dx = coord_list.x - last_x_read;
		dy = coord_list.y - last_y_read;
		last_x_read = coord_list.x;
		last_y_read = coord_list.y;
			/*** set direction ***/
		if(dx > 0)
		{	if(dy > 0)
				vector[v6][0] = 1;
			else if(dy == 0)
				vector[v6][0] = 2;
			else
				vector[v6][0] = 3;
		}
		else if(dx == 0)
		{	if(dy > 0)
				vector[v6][0] = 0;
			else
				vector[v6][0] = 4;
		}
		else
		{	if(dy > 0)
				vector[v6][0] = 7;
			else if(dy == 0)
				vector[v6][0] = 6;
			else
				vector[v6][0] = 5;
		}
			/*** set length **/
		vector[v6][1] = (abs(dx) > abs(dy)) ?  abs(dx): abs(dy);
	}
	else
	{		/*** set direction & length to 0 ***/
		vector[v6][0] = 0;
		vector[v6][1] = 0;
	}
	
	return(0);
}


/*******************************************************************************
*       check3                                                                 *
*                                                                              *
*        checks the next 3 vectors to see if 3 vector vector-reduction         *
*    is possible.                                                              *
*                                                                              *
*    Returned Values: (long) 0 = not possible,   1 = is possible               *
*                                                                              *
*                      j.dabritz  7/25/89                                      *
*******************************************************************************/

long check3()
{
	long dd;         /** difference in direction **/

		/******** the following constitute the rules for 3 vector reduction */
		/******** 1. must be at least 3 vectors left ********************/
	if(vector[v3][1] == 0)    
		return(0);

		/******** 2. direction of vector 1 & 3 must be equal ************/
	if(vector[v1][0] != vector[v3][0])
		return(0);

		/******** 3. length of vector 2 must be exactly 1 ***************/
	if(vector[v2][1] != 1)
		return(0);

		/******** 4. direction of vector 2 must be +-1 of first vector **/
	dd = (vector[v2][0] - vector[v1][0] + 8) % 8;
	if((dd != 1) && (dd != 7))
		return(0);

		/******** 5. if there is a vector4, it must not mirror vector 2 */
	if((vector[v4][0] == vector[v2][0]) && (vector[v4][1] == vector[v2][1]))
		return(0);

		/******** 6. the previous vector (if any) must not mirror vec2 **/
	if(last_dir == vector[v2][0])
		return(0);

		/*********** if we got this far, then 3 vector reduction is poss */
	return(1);
}


/*******************************************************************************
*       check4                                                                 *
*                                                                              *
*        checks the next 4 vectors to see if 4 vector vector-reduction         *
*    is possible.                                                              *
*                                                                              *
*    Returned Values: (long) 0 = not possible,   1 = is possible               *
*                                                                              *
*                      j.dabritz  7/25/89                                      *
*******************************************************************************/

long check4()
{	
	long dd;          /** difference in direction */

		/******** the following constitute the rules for 4 vector reduction */
		/******** 1. must be at least 4 vectors left ********************/
	if(vector[v4][1] == 0)    
		return(0);

		/******** 2. direction of vector 1 & 3 must be equal ************/
	if(vector[v1][0] != vector[v3][0])
		return(0);

		/******** 3. direction of vector 2 & 4 must be equal ************/
	if(vector[v2][0] != vector[v4][0])
		return(0);

		/******** 4. direction of vector 2 must be +-1 of first vector **/
	dd = (vector[v2][0] - vector[v1][0] + 8) % 8;
	if((dd != 1) && (dd != 7))
		return(0);

		/******** 5. both (v1 & v3) or (v2 & v4) must have length 1 *****/
	if((vector[v2][1] != 1) || (vector[v4][1] != 1))
	{	if((vector[v1][1] != 1) || (vector[v3][1] != 1))
			return(0);
	}

		/******** 6. if length v1 = 1, then length v2 must be <= 3 ******/
	if(vector[v1][1] == 1)
	{	if(vector[v2][1] > 3)
			return(0);
	}

		/*********** if we got this far, then 4 vector reduction is poss */
	return(1);
}


/*******************************************************************************
*       reduce3                                                                *
*                                                                              *
*        finds endpoint for 3-vector vector-reduction & outputs coordinates.   *
*    updates vector array.                                                     *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*                      j.dabritz  7/25/89                                      *
*******************************************************************************/

long reduce3()
{
	long length;

		/********** 1. if v1 > v3+1 then move up v1 before shortcutting ***/
	if(vector[v1][1] > (vector[v3][1] + 1))
	{	length = vector[v1][1] - vector[v3][1];
		vector[v1][1] = vector[v1][1] - length;
		x += length * move_x[vector[v1][0]];
		y += length * move_y[vector[v1][0]];
		if(write_reduce())
		{	message("abort, reduce_rdata, location 24");
			return(1);
		}
	}

		/********** 2. move up vector 1 and 2 *****************************/
	x += vector[v1][1] * move_x[vector[v1][0]];
	y += vector[v1][1] * move_y[vector[v1][0]];
	x += vector[v2][1] * move_x[vector[v2][0]];
	y += vector[v2][1] * move_y[vector[v2][0]];

		/********** 3. move up vector 3 to a point not exceeding vector1+1 */
	length = (vector[v3][1] < vector[v1][1]+1) ? vector[v3][1] : vector[v1][1];
	vector[v3][1] -= length;
	x += length * move_x[vector[v3][0]];
	y += length * move_y[vector[v3][0]];
	
		/********** 4. write ending coords ********************************/
	if(write_reduce())
	{	message("abort, reduce_rdata, location 25");
		return(1);
	}

		/********** 5. increment v depending of portion of v3 used ********/
	if(next_vector())
	{	message("abort, reduce_rdata, location 26");
		return(1);
	}
	if(next_vector())
	{	message("abort, reduce_rdata, location 27");
		return(1);
	}
	if(!vector[v1][1])
	{	if(next_vector())
		{	message("abort, reduce_rdata, location 28");
			return(1);
		}
	}
	else
		last_dir = vector[v1][0];

		/********** return ************************************************/
	return(0);
}


/*******************************************************************************
*       reduce4                                                                *
*                                                                              *
*        finds endpoint for 4-vector vector-reduction & outputs coordinates.   *
*    updates vector array.                                                     *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*                      j.dabritz  7/25/89                                      *
*******************************************************************************/

long reduce4()
{
	long type;            /*  shortcut type, see 1. below                    */
	long primary_length;  /*  of zigzag pairs                                */
	long length;
	long primary_dir;     /*  of zigzag vector pairs                         */
	long secondary_dir;   /*  of zigzag vector pairs                         */
	long x0, y0;          
	long end_vec;         /*  index */
	long i;
	long dx, dy;
	long x1,y1,x2,y2;
	long test1;           /*  test flag */
	long first_leg;       /*  length of first vector */

	double slope, slope1, slope2;


		/********* 1. determine shortcut type:                         ********/
		/*********     type 0 = odd number vectors have length 1, primary     */
		/*********              direction is direction of second vector.      */
		/*********     type 1 = even numbers have length 1, length v1 near 0  */
		/*********     type 2 = even numbers have length 1, length v1 is near */
		/*********              0.5 times v3                                  */
		/*********     type 3 = even numbers have length 1, length v1 is near */
		/*********              to v3 or greater                              */
	if((vector[v2][1] == 1) && (vector[v4][1] == 1))
	{	type = (vector[v1][1] / (double) vector[v3][1]) + 1.5;
		type = (type > 3) ?  3 : type;
	}
	else
		type = 0;

	first_leg = vector[v1][1];


		/********* 2. set primary length (length of first full long vector) ***/
	if(type == 0)
		primary_length = vector[v2][1];
	else
		primary_length = vector[v3][1];

		/********* 3. if type = 3, then check to see if length v1 > primary   */
		/*********    length, if so then move up v1 until type = 2         ****/
	if((type == 3)  && (vector[v1][1] > primary_length + 1))
	{	length = vector[v1][1] - primary_length;
		vector[v1][1] = vector[v1][1] - length;
		x += length * move_x[vector[v1][0]];
		y += length * move_y[vector[v1][0]];
		if(write_reduce())
		{	message("abort, reduce_rdata, location 29");
			return(1);
		}
	}

		/********* 4. set primary & secondary direction variables & begin x,y */
	if(type == 0)
	{	primary_dir = vector[v2][0];
		secondary_dir = vector[v1][0];
	}
	else
	{	primary_dir = vector[v1][0];
		secondary_dir = vector[v2][0];
	}
	x0 = x;
	y0 = y;

		/********* 5. move down zigzag until next unchecked vector in the     */
		/*********    primary direction is at v2, setting benchmark slopes as */
		/*********    you go                                                  */
	end_vec = (type == 0) ?  2 : 3;
	for(i=0; i<end_vec; i++)
	{	x += vector[v1][1] * move_x[vector[v1][0]];
		y += vector[v1][1] * move_y[vector[v1][0]];
		dx = x - x0;
		dy = y - y0;
		if(dx)
			slope = dy / (double) dx;
		else
			slope = (dy < 0) ?  -10000 : 10000;
		if(i == end_vec - 1)
			slope2 = slope;
		else if(i == end_vec - 2)
			slope1 = slope;
		if(next_vector())
		{	message("abort, reduce_rdata, location 30");
			return(1);
		}
	}
	
		/********* 6. loop test the next 2 vectors until zigzag shortcut ends */
	for(;;)
	{	test1 = 0;
			/*** test for 2 more vectors available **/
		if(vector[v2][1] == 0)
			break;

			/*** test is dir v1 correct           **/
		if((vector[v1][0] != secondary_dir) || (vector[v1][1] == 0))
			break;

			/*** is dir v2 correct                **/
		if(vector[v2][0] != primary_dir)      break;

			/*** is length v1 == 1                **/
		if(vector[v1][1] != 1)                break;

			/*** do endpoints of v2 fall between slopes 1 and 2 ***/
		x1 = x + vector[v1][1] * move_x[vector[v1][0]];
		y1 = y + vector[v1][1] * move_y[vector[v1][0]];
		x2 = x1 + vector[v2][1] * move_x[vector[v2][0]];
		y2 = y1 + vector[v2][1] * move_y[vector[v2][0]];
		dx = x1 - x0;
		dy = y1 - y0;
		if(dx)
			slope = dy / (double) dx;
		else
			slope = (dy < 0) ?  -10000000.0 : 10000000.0;
		if(((slope > slope1) && (slope > slope2)) ||
			((slope < slope1) && (slope < slope2)))
			break;      /**** exit loop if outside slope range **/
		slope1 = slope;
			
		test1 = 1;

		dx = x2 - x0;
		dy = y2 - y0;
		if(dx)
			slope = dy / (double) dx;
		else
			slope = (dy < 0) ?  -10000000.0 : 10000000.0;
		if(((slope > slope1) && (slope > slope2)) ||
			((slope < slope1) && (slope < slope2)))
			break;      /**** exit loop if outside slope range **/
		slope2 = slope;
		
			/**** increment x & y and get next two vectors **/
		x = x2;
		y = y2;
		if(next_vector())
		{	message("abort, reduce_rdata, location 31");
			return(1);
		}
		if(next_vector())
		{	message("abort, reduce_rdata, location 32");
			return(1);
		}
	}

		/********* 7. find correct endpoint for shortcut, calculate coords &  */
		/*********    adjust/increment vectors.  Remember, we are at the end  */
		/*********    of a long vector.                                       */
		/***** move in dir1 up to dist of first_lsg (if everything is     */
		/*     copasetic), increment vectors                              */
	if(test1 && type) /*vector exists, dir1&2 correct, len v1=1, slope1 ok */
	{		/***** move up vector 1 (1 pixel) ****/
		x += move_x[vector[v1][0]];
		y += move_y[vector[v1][0]];
		if(next_vector())
		{	message("abort, reduce_rdata, location 33");
			return(1);
		}
			/***** move up a distance of first_leg up vector 2 (1 now) **/
		length = (vector[v1][1] < first_leg) ? vector[v1][1] : first_leg;
		length = (length > primary_length) ? primary_length : length;
		x += length * move_x[vector[v1][0]];
		y += length * move_y[vector[v1][0]];
		vector[v1][1] -= length;
		if(vector[v1][1] == 0)     /* used all of vector 1  */
		{	if(next_vector())
			{	message("abort, reduce_rdata, location 33");
				return(1);
			}
		}
	}
	
		/********* 8. finally, write coords for end of shortcut ***************/
	if(write_reduce())
	{	message("abort, reduce_rdata, location 34");
		return(1);
	}

	
		/********* return *****************************************************/
	return(0);
}
	

/*******************************************************************************
*       write_reduce                                                           *
*                                                                              *
*        write the coordinates (passed) to the next slot in the coord_list     *
*    file.  accumulate segment length & area.                                  *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*                      j.dabritz  7/25/89                                      *
*******************************************************************************/

long write_reduce()
{
	long dx,dy;
	double square;


	coord_list.x = x;
	coord_list.y = y;

	if(put_coordlst(&coord_list,next_coord_out++))
	{	message("abort, reduce_rdata, location 35");
		return(1);
	}

	segment_area += (y + last_y) * (x - last_x);
	dx = x - last_x;
	dy = y - last_y;
	square = (dx * dx) + (dy * dy);
	segment_length += sqrt(square);

	last_x = x;
	last_y = y;
	
	return(0);
}


/*******************************************************************************
*       defaultwrite                                                           *
*                                                                              *
*        calculate the coordinates for v1 & write them out, increment the      *
*    vector array.                                                             *
*                                                                              *
*    Returned Values: (long) 0                                                 *
*                                                                              *
*                      j.dabritz  7/25/89                                      *
*******************************************************************************/

long defaultwrite()
{
	x += vector[v1][1] * move_x[vector[v1][0]];
	y += vector[v1][1] * move_y[vector[v1][0]];
	if(write_reduce())
	{	message("abort, reduce_rdata, location 36");
		return(1);
	}
	if(next_vector())
	{	message("abort, reduce_rdata, location 37");
		return(1);
	}
	return(0);
}
