/*******************************************************************************
*       rstr_to_scrn    (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Converts raster coordinates (longs) to screen coordinates.    *
*       Uses the input parameters and the current view and window globals      *
*       for the conversion.                                                    *
*                                                                              *
*    Returned Values: 0 if screen coordinates are in the graphics window.      *
*                     1 if screen coordinates are outside the graphics window. *
*                    -1 on error or bad parameters or no map loaded            *
*                                                                              *
*    Global Variables Modified: Updates:  none                                 *
*                                                                              *
*                      j.dabritz  6/16/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"

long rstr_to_scrn(xptr,yptr)
	long *xptr, *yptr;    /*  input (raster) & output (screen) registers  */
{
	long dx_raster;
	long dx_screen;
	long dy_raster;
	long dy_screen;


		/******************** check for legitimate pointers **********/
	if( xptr == (long *)NULL  ||  yptr == (long *)NULL )
		return(-1);

	
		/******************** check for map loaded *******************/
	if(!job_stats[0])
		return(-1);


		/******************** check for good resolution code *********/
	if(resolution_code<0  ||  resolution_code>3)
		return(-1);


		/******************** convert x ******************************/
	dx_raster = *xptr - view[resolution_code].lower_leftx;
	dx_screen = dx_raster * view[resolution_code].zoom_factor;
	*xptr = windowxmin + dx_screen;


		/******************** convert y ******************************/
	dy_raster = *yptr - view[resolution_code].lower_lefty;
	dy_screen = dy_raster * view[resolution_code].zoom_factor;
	*yptr = windowymax - dy_screen;     /* (screen y, positive is down )  */


		/******************** center on fat pixels *******************/
	if(resolution_code == 3)
	{	*xptr = *xptr + 3;
		*yptr = *yptr - 3;
	}


		/******************** return appropriate code ****************/
	if((*xptr >= windowxmin) && (*xptr <= windowxmax) && (*yptr >= windowymin)
		 	&& (*yptr <= windowymax) )
		return(0);
	else
		return(1);

}
