/*******************************************************************************
*       scrn_to_rstr    (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Purpose:    Converts screen coordinates (longs) to raster coordinates.    *
*       Uses the input parameters and the current view and window globals      *
*       for the conversion.                                                    *
*                                                                              *
*    Returned Values: 0 if raster coordinates are on the raster.               *
*                     1 if raster coordinates are off the raster.              *
*                    -1 on error or bad parameters or no map loaded            *
*                                                                              *
*    Global Variables Modified: Updates:  none                                 *
*                                                                              *
*                      j.dabritz  6/16/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"

long scrn_to_rstr(xptr,yptr)
	long *xptr, *yptr;    /*  input (screen) & output (raster) registers  */
{
	long dx_raster;
	long dx_screen;
	long dy_raster;
	long dy_screen;


		/******************** check for legitimate pointers **********/
	if((xptr<=0)  ||  (yptr<=0))
		return(-1);

	
		/******************** check for map loaded *******************/
	if(!job_stats[0])
		return(-1);


		/******************** check for good resolution code *********/
	if(resolution_code<0  ||  resolution_code>3)
		return(-1);


		/******************** check for non-zero zoom factor *********/
	if(0.0 == view[resolution_code].zoom_factor)
		return(-1);


		/******************** convert x ******************************/
	dx_screen = *xptr - windowxmin;
	if(resolution_code > 1)
		dx_raster = dx_screen / view[resolution_code].zoom_factor;
	else
		dx_raster = (0.5 + dx_screen) / view[resolution_code].zoom_factor;
	*xptr = view[resolution_code].lower_leftx + dx_raster;
	

		/******************** convert y ******************************/
	dy_screen = windowymax - *yptr;       /*  screen y, positive is down */
	if(resolution_code > 1)
		dy_raster = dy_screen / view[resolution_code].zoom_factor;
	else
		dy_raster = (0.5 + dy_screen) / view[resolution_code].zoom_factor;
	*yptr = view[resolution_code].lower_lefty + dy_raster;
	

		/******************** return appropriate code ****************/
	if((*xptr >= 0) && (*xptr <= job_stats[1]) && (*yptr >= 0)
		 	&& (*yptr <= job_stats[2]) )
		return(0);
	else
		return(1);

}
