/*******************************************************************************
*       seg_breaker     (see manual, Internal Functions)                <in>   *
*    Searches the vector file for nearest part of nearest segment (input       *
*    parmater 0 = 0),  breaks the segment into two segments (input parameter   *
*    0 = 1), sets cursor anchors for highlighting segment.                     *
*                                                                              *
*    Arguments:    long  action:  0 = find & highlight nearest verticies/seg   *
*                                 1 = break highlighted segment                *
*                  float x,y:     raster coordinates            (pixels)       *
*                                                                              *
*    Returned Values:        0 = action completed                              *
*                            1 = object not found or not broken / failure.     *
*                                                                              *
*    Global Variables Modified:  job_stats, objects, index, index_list etc.    *
*                                                                              *
*                      j.dabritz  5/9/90                                       *
*******************************************************************************/

#include <math.h>
#include "../../incl/globals.h"

long seg_update() ;

long seg_breaker(action,lx,ly)
long action;
long lx,ly;
{   
   INDEX_LIST index_list;
   OBJECTS object1, object2, node;
   COORD_LIST coord_list;

   long go_on;
   long row, col;
   long x1,y1,x2,y2;
   long i, v;
   static long record_no;
   static long vertex1, vertex2;
   long obj1, obj2;
   long node1, node2;

   float  r;
	float x,y;

   double vx, vy;
   double last_vx, last_vy;
   double array[2][3];
   double dx, dy;
   double dist_squared, min_square;


      /*********** check for map **************************************/
   if(!job_stats[0])
      return(1); 

      /*********** check for file type needed *************************/
   if(job_stats[10] == 0)
      return(1); 
   if(job_stats[49])        /*  map needs to be re-indexed */
   {   if(index_map())      /*  1 = index failed           */
         return(1); 
   }
   
   switch(action)  {
   case 0:
         /********* find nearest segment *********************************/
		x = (float) lx;
		y = (float) ly;

         /*********** check r ********************************************/
      r = search_radius;
      if(r < 0.0)
         return(1); 
   
         /*********** find index_array window ****************************/
      x1 = (x - r + 0.5);    /* x min (pixels) */
      x1 = x1 >> 5;          /* x min (index_array coords) */
      x2 = (x + r + 0.5);    /* x max (pixels) */
      x2 = x2 >> 5;          /* x max (index_array coords) */
      y1 = (y - r + 0.5);    /* y min (pixels) */
      y1 = y1 >> 5;          /* y min (index_array coords) */
      y2 = (y + r + 0.5);    /* y max (pixels) */
      y2 = y2 >> 5;          /* y max (index_array coords) */

         /** test for raster coverage *****************************/
      if((x2 < 0)  || (x1 > (job_stats[1] >> 5)))
         return(1); 
      if((y2 < 0)  || (y1 > (job_stats[2] >> 5)))
         return(1); 
         /** limit to range of index_array ***/
      if(x1 < 0) 
         x1 = 0;
      if(y1 < 0)
         y1 = 0;
      if(x2 > (job_stats[1] >> 5))
         x2 = (job_stats[1] >> 5);
      if(y2 > (job_stats[2] >> 5))
         y2 = (job_stats[2] >> 5);
      
         /*********** search for segment within r dist **********************/
         /** for each row & col of index array, search objects */
      min_square = (r + .01) * 1.01;        /** in case r = 0 or very large **/
      min_square = min_square * min_square; /** compair squares to save time **/
      for(row = y1; row <= y2; row++)
      {   for(col = x1; col <= x2; col++)
         {      /** search all indicated objects **/
            index_list.next = index_array[row][col];
            for(; index_list.next;)
            {      /** get index_list record **/
               if(get_indexl(&index_list,index_list.next))
                  return(1); 
                 /*** check for object type == segment ***/
               go_on = 1;
               if(index_list.type != 1)
                  go_on = 0;
   
                  /**** test every vertex & side for distance **/
               i = (index_list.v1 < index_list.v2)  ?  1 : -1;
               for(v = index_list.v1; go_on ; v += i)
               {      /*** get vertex coords ***/
                  if(get_coordlst(&coord_list,v))
                     return(1); 
                  vx = coord_list.x;
                  vy = coord_list.y;
            
                     /*** calculate distance to input point **/
                  dx = x - vx;
                  dy = y - vy;
                  dist_squared = (dx * dx) + (dy * dy);
                     
                     /*** check against min dist ***/
                  if(dist_squared < min_square)
                  {   min_square = dist_squared;
                     record_no = index_list.obj_rec;
                     vertex1 = v;
                     if(v == index_list.v1)
                        vertex2 = v + i;
                     else
                        vertex2 = v - i;
                  }
               
                     /*** if not first point, then get dist to vector **/
                  if(v != index_list.v1)
                  {   dx = vx - last_vx;
                     dy = vy - last_vy;
                     if((dx != 0.0) || (dy != 0.0))
                     {   array[0][0] = dx;
                        array[0][1] = dy;
                        array[0][2] = x - last_vx;
                        array[1][0] = dy;
                        array[1][1] = -dx;
                        array[1][2] = y - last_vy;
                        if(!mat_solve(2,3,array))
                        {      /*** see if pt perp. off side of vector **/
                           if((array[0][2] > 0.0) && (array[0][2] < 1.0))
                           {   dist_squared = array[1][2] * array[1][2] * 
                                        ((dx * dx) + (dy * dy));
                                 /*** check against min dist ***/
                              if(dist_squared < min_square)
                              {   min_square = dist_squared;
                                 record_no = index_list.obj_rec;
                                 vertex1 = v;
                                 vertex2 = v - i;
                              }
                           }
                        }
                     }
                  }
               
                     /*** check to see if verticies done ***/
                  if(v == index_list.v2)
                     break;
   
                     /*** reset last_v coords ***/
                  last_vx = vx;
                  last_vy = vy;
               }
            }
         }
      }
   
         /*********** return closest segment record number or -1 ************/
      if(min_square <= (r * r))
      {      /**** set anchor & anchor2 ****************************/
         if(get_coordlst(&coord_list,vertex1))
            return(1); 
         anchorx = coord_list.x;
         anchory = coord_list.y;
         if(get_coordlst(&coord_list,vertex2))
            return(1); 
         anchor2x = coord_list.x;
         anchor2y = coord_list.y;
      }
      else
      {   message("Seg_break:  No segment found within search radius.");
         return(1);
      }

      break;
   case 1:
         /************************************************************/
         /***** action 2 = break segment *****************************/
         /************************************************************/

			/***** remove attribute(s) from current object   ************/
		asgn_atrbt(record_no,7,"  ");

         /***** remove object from current vector display ************/
      display_obj(0,record_no);

         /***** make vertex1 < vertex2 *******************************/
      if(vertex1 > vertex2)
      {   i = vertex1;
         vertex1 = vertex2;
         vertex2 = i;
      }

      obj1 = record_no;
         /***** un-index segment *************************************/
      if(index_obj(0,obj1))       /** index first segment     ****/
         return(1);

         /***** read vertex1 *****************************************/
      if(get_coordlst(&coord_list,vertex1))
         return(1); 

         /***** make node record for seg1 ****************************/
      node.i12[0] = 4;
      node.i12[1] = job_stats[74];
      job_stats[74]++;
      node.i12[2] = 0;
      node.i12[3] = 0;
      node.i12[4] = 0;
      node.i12[5] = 1;
      node.i12[6] = obj1;
      node.i12[7] = 0;
      node.i12[8] = 0;
      node.i12[9] = 0;
      node.i12[10] = 0;
      node.i12[11] = 0;
      node.f4[0] = coord_list.x;
      node.f4[1] = coord_list.y;
      node.f4[2] = 0.0;
      node.f4[3] = 0.0;
         /***** store node record for seg1 ***************************/
      node1 = job_stats[10];
      if(put_objects(&node,job_stats[10]))
         return(1);

         /***** read vertex2 *****************************************/
      if(get_coordlst(&coord_list,vertex2))
         return(1); 

         /***** make node record for seg2 ****************************/
      node.i12[1] = job_stats[74];
      job_stats[74]++;
      node.i12[6] = obj2 = job_stats[10] + 1;
      node.f4[0] = coord_list.x;
      node.f4[1] = coord_list.y;
         /***** store node record for seg1 ***************************/
      node2 = job_stats[10];
      if(put_objects(&node,job_stats[10]))
         return(1);

         /***** read object record1 **********************************/
      if(get_objects(&object1,obj1))
         return(1);
         /***** read object record2 **********************************/
      if(get_objects(&object2,obj1))
         return(1);
      object2.i12[2] = 0;  /** no attributes **/

         /***** change object record vertex   range *******************/
      object1.i12[6] = vertex1;
      object2.i12[5] = vertex2;

         /***** update object types, lengths, areas, centroids ******/
      if(seg_update(&object1,obj1))
         return(1);
      if(seg_update(&object2,obj2))
         return(1);

         /***** change obj2 ending node record for seg rec no. change */
      if(get_objects(&node,object2.i12[10]))
         return(1);
      for(i=6; i<10; i++)
      {   if(node.i12[i] == obj1)
            node.i12[i] = obj2;
      }
      if(put_objects(&node,object2.i12[10]))
         return(1);
      

         /***** re-index objects *************************************/
      if(index_obj(1,obj1))       /** index first segment     ****/
         return(1);
      if(index_obj(1,obj2))       /** index second segment    ****/
         return(1);
      if(index_obj(1,node1))      /** index first new node    ****/
         return(1);
      if(index_obj(1,node2))      /** index second new node   ****/
         return(1);

         /***** redisplay both new nodes *****************************/
      display_obj(1,node1);
      display_obj(1,node2);

         /***** redisplay both new segments **************************/
      display_obj(1,obj1);
      display_obj(1,obj2);

         /***** return ****/
      break;
   }



   return(0);
}

/*******************************************************************************
*   seg_update                                                                 *
*       updates the object type, length, area, & centoroid.                    *
*                                                                              *
*******************************************************************************/
         /***** update object types, lengths, areas, centroids ******/


long seg_update(obj_ptr,obj_no) 
OBJECTS *obj_ptr;
long obj_no;
{
   OBJECTS node;
   COORD_LIST coord_list;

   long free_ends;
   long v, v1, v2;

   double lastx, lasty;
   double length, area;
   double vx, vy;
   double dx, dy;
   double d;


      /********** find/update object type ******************************/
      /*** look up beginning node ***/
   free_ends = 0;
   if(get_objects(&node,obj_ptr->i12[9]))
      return(1);
   if(node.i12[0] == 4)
      free_ends++;

      /*** look up ending node    ***/
   if(get_objects(&node,obj_ptr->i12[9]))
      return(1);
   if(node.i12[0] == 4)
      free_ends++;

      /*** store segment type     ***/
   obj_ptr->i12[0] = 10 - free_ends;

      /********** read all verticies, calculating length, area *********/
   v1 = obj_ptr->i12[5];
   v2 = obj_ptr->i12[6];
   if(get_coordlst(&coord_list,v1))
      return(1); 
   lastx = coord_list.x;
   lasty = coord_list.y;
   area = 0.0;
   length = 0.0;
   for(v = v1 ; v <= v2; v++)
   {   if(get_coordlst(&coord_list,v))
         return(1); 
      vx = coord_list.x;
      vy = coord_list.y;
      dx = vx - lastx;
      dy = vy - lasty;
      length += sqrt(dx * dx + dy * dy);
		area += dx * ((vy + lasty) / 2.0);
      lastx = vx;
      lasty = vy;
   }
   obj_ptr->f4[2] = length;
   obj_ptr->f4[3] = area;


      /********** position new centroid ********************************/
   length = length / 2.0;      /** length to centroid point **/
   if(get_coordlst(&coord_list,v1))
      return(1); 
   lastx = coord_list.x;
   lasty = coord_list.y;
   for(v = v1 ; v <= v2; v++)
   {   if(get_coordlst(&coord_list,v))
         return(1); 
      vx = coord_list.x;
      vy = coord_list.y;
      dx = vx - lastx;
      dy = vy - lasty;
      d  = sqrt(dx * dx + dy * dy);

      if((length - d) <= 0.0)
      {   if( d != 0.0)
         {   obj_ptr->f4[0] = lastx + dx * (length / d);
            obj_ptr->f4[1] = lasty + dy * (length / d);
         }
         else
         {   obj_ptr->f4[0] = lastx;
            obj_ptr->f4[1] = lasty;
         }

         break;
      }
      length = length - d;

      lastx = vx;
      lasty = vy;
   }
   

      /********** store object record **********************************/
   if(put_objects(obj_ptr,obj_no))
      return(1);

      /********** return ***********************************************/
   return(0);
}
