/*******************************************************************************
*       segcentroid     (see manual, Internal Functions)                <in>   *
*                                                                              *
*        Centroids all segments                                                *
*    This program puts a primary centroid near the middle of all segments,     *
*    and secondary centroids allong all segments over 20% of the maximum       *
*    map dimention.                                                            *
*                                                                              *
*    Arguments:    none                                                        *
*                                                                              *
*    Returned Values: (long) 0   ok,   1= abort                                *
*                                                                              *
*    Global Variables Modified:  none, the objects and coord_list file are     *
*                                      changed.                                *
*                      j.dabritz  7/24/89                                      *
*******************************************************************************/

#include "../../incl/globals.h"


long segcentroid()
{
	OBJECTS segment;
	COORD_LIST coord_list;

	long start, finish;          /* record numbers to read in coord_list file */
	long twostep;                /* record number at which to increment by 2  */
	long mult_step_len;          /* min length requiring multiple centroids  */
	long c,s;
	long length;                 /* segment length */
	long no_centroids;           /* number of centroids for a given segment */
	long primary_cent;           /* centroid number of the primary centroid */
	long between_length;         /* length between centroids                */
	long x1,y1;                  /* trailing point                          */
	long x2,y2;                  /* next vertex / leading point             */
	long step_length_rem;        /* step length remaining to next vertex    */
	long vect_len;               /* unused length of this vector            */
	long coord_list_rec;         /* record number of coord_list file read   */
	long dx,dy;                  /* differences between verticies           */
	long dir = 0;                /* direction code for vector:
                                             7 0 1
                                             6 X 2
                                             5 4 3              */
	static long move_x[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
	static long move_y[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	

		/*************** initialize local variables *********************/
	x1 = y1 = 0;

		/*************** check for map & segments ***********************/
	if(!job_stats[0])
		return(0);

		/*************** centroid all segments **************************/
	message("centroiding all segments  . . .  ");

	start = (job_stats[5]) ? job_stats[5] : job_stats[6]; 
	twostep = (job_stats[6]) ?  job_stats[6] - 2: job_stats[10];
	finish = (job_stats[7]) ?  job_stats[7] : job_stats[10];
	if(job_stats[1] > job_stats[2])
		mult_step_len = job_stats[1] * 0.2;
	else
		mult_step_len = job_stats[2] * 0.2;
	for(s = start; s < finish; s += (s < twostep) ? 1 : 2)
	{
			/*** check for user abort **/
		if(abort_cmd())
			return(1);

			/*************** get segment record *****************************/
		if(get_objects(&segment,abs(s)))
		{	message("abort, segcentroid, location Z");
			return(1);
		}
			/*************** erase temp low-point stuff from vectorize ******/
		segment.i12[1] = 0;
		segment.i12[2] = 0;
		segment.i12[3] = 0;
			/*************** calc number of centroids needed ****************/
		length = segment.f4[2];
		if(length > mult_step_len)
		{	no_centroids = length / mult_step_len + 1;
			no_centroids = (no_centroids > 10) ?  10 : no_centroids;
			primary_cent = no_centroids / 2;
			between_length = length / no_centroids;
			
		}
		else
		{	no_centroids = 1;
			primary_cent = 0;
			between_length = length;
		}
			/*************** loop through setting centroids *****************/
		if(length == 1)       /** 1 pixel segment, put cent between pixels */
		{	if(get_coordlst(&coord_list,segment.i12[5]))
			{	message("abort, segcentroid, location Z");
				return(1);
			}
			segment.f4[0] = coord_list.x;
			segment.f4[1] = coord_list.y;
			if(get_coordlst(&coord_list,segment.i12[6]))
			{	message("abort, segcentroid, location Z");
				return(1);
			}
			segment.f4[0] = (segment.f4[0] + coord_list.x) / 2;
			segment.f4[1] = (segment.f4[1] + coord_list.y) / 2;
		}
		else
		{
			coord_list_rec = segment.i12[5];
			if(get_coordlst(&coord_list,coord_list_rec++))
			{	message("abort, segcentroid, location Z");
				return(1);
			}
				/***** set the beginning x & y values **/
			x2 = coord_list.x;
			y2 = coord_list.y;
			step_length_rem = between_length / 2;
			vect_len = 0;
			for(c = 0; c < no_centroids;   )
			{		/*********** see if next step is on this vector *********/
				if(vect_len >= step_length_rem)
				{	x1 += step_length_rem * move_x[dir];
					y1 += step_length_rem * move_y[dir];
					vect_len -= step_length_rem;
					step_length_rem = 0;
						/***** set centroid ***********/
					if(c == primary_cent)
					{	segment.f4[0] = x1;
						segment.f4[1] = y1;
					}
					else
					{	if(!segment.i12[3])
							segment.i12[3] = job_stats[11];
						coord_list.x = x1;
						coord_list.y = y1;
						segment.i12[11] = job_stats[11];
						if(put_coordlst(&coord_list,job_stats[11]))
						{	message("abort, segcentroid, location Z");
							return(1);
						}
					}
						/**** set step_length_rem for next loop *************/
					step_length_rem = between_length;
						/**** increment centroid number ***/
					c++;
				}
				else            /*  next step is beyond this vector */
				{	step_length_rem -= vect_len;
					x1 = x2;
					y1 = y2;
					if(get_coordlst(&coord_list,coord_list_rec++))
					{	message("abort, segcentroid, location Z");
						return(1);
					}
						/****** set initial values for vect_len, x2,y2 & dir */
					x2 = coord_list.x;
					y2 = coord_list.y;
					dx = x2 - x1;
					dy = y2 - y1;
					vect_len = (dx) ?  abs(dx) : abs(dy);
					if(dx>0)
					{	if(dy>0)
							dir = 1;
						else if(dy == 0)
							dir = 2;
						else
							dir = 3;
					}
					else if(dx==0)
					{	if(dy>0)
							dir = 0;
						else
							dir = 4;
					}
					else
					{	if(dy>0)
							dir = 7;
						else if(dy == 0)
							dir = 6;
						else
							dir = 5;
					}
				}
			}
		}

				/*********** store segment record ***************************/
		if(put_objects(&segment,abs(s)))
		{	message("abort, segcentroid, location Z");
			return(1);
		}
	}

	append_msg("done.");
		/*** return ************************/
	return(0);
}
