/*******************************************************************************
*       set_mapin       (see manual, Internal Functions)                <in>   *
*                Translates other registration to map inch registration        *
*                                                                              *
*                Returns 0 upon successful EOJ, else returns -1.               *
*                      C.E.Jonson 7/17/89 (EDS)  modified 6/20/90 JPD          *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <math.h>

long set_mapin(c)
long c;            /*** c: 2=utm, 3=st pln, 4=dig1, 5=dig2 **/
{
	double abcd[4];               /** Holds abcd values from get_params **/
	double dx, dy;                /** Delta x and Delta y               **/
	double length_raw;            /** Length of map side                **/
	double maplen_raw;            /** Length of map in scale            **/
	double maplen_in;             /** Length of map in feet             **/
	long   i;                     /** Run of the mill for-loop counter  **/

		/**** check input parameters ****/
	switch(c) {
	case 2:    /** utm           **/
	case 3:    /** state plane   **/
	case 4:    /** dig1          **/
	case 5:    /** dig2          **/
		break;       /** continue **/
	default:
		return(-1);
	}

	if(c < 4)       /*** utm or state plane ***/
	{	if(map_scale <= 0.0 )
		{	message("Map scale invalid.  Check header \"scale\" value.");
			return(-1);
		}
	}
	else            /*** digitizer registration ***/
	{	if(diginfo.units_per_inch <= (double) 0.0)
		{	message("digitizer units_per_inch value invalid.  Check digitizer file.");
			return(-1);
		}
	}


/*** Calculate West edge length using UTM meters.      ***/
	dx = corner[c][0][0] - corner[c][1][0];
	dy = corner[c][0][1] - corner[c][1][1];
	length_raw = sqrt( (dx * dx) + (dy * dy) );
	if(c < 4)     /** utm or state plane ***/
	{	maplen_raw = length_raw / map_scale;
		if(c == 2)       /*** utm         ***/
			maplen_in = maplen_raw * 39.37;
		else             /*** state plane ***/
			maplen_in = maplen_raw * 12.0;
	}
	else          /*** digitizer ***/
		maplen_in = length_raw / diginfo.units_per_inch;
	
/*** Assign SW and NW corners initial values  ***/
	corner[6][0][0] = corner[6][0][1] = corner[6][1][0] = 1.0;
	corner[6][1][1] = 1.0 + maplen_in;

/*** Get translation/rotation/scale parameters   ***/
	if(get_params(abcd, corner[6], corner[c], 2L))
	{	message("Cannot calculate trans/rotat/scale params.");
		return(-1);
	}
	
	for( i = 2; i < 4; i++ )
	{	corner[6][i][0] = abcd[0] * corner[c][i][0] - 
				abcd[1] * corner[c][i][1] + abcd[2];
		corner[6][i][1] = abcd[1] * corner[c][i][0] + 
				abcd[0] * corner[c][i][1] + abcd[3];
	}
	return(0);
}
