/*******************************************************************************
*    setup_ofile                                                               *
*        This program trys to set up a file for output based on the input      *
*    fname string.  fname may be a DOS filename, or a Unix/Xenix file name     *
*    (absolute or relative to the default export path), or a Unix/Xenix        *
*    directory path (absolute or relative to the default export path.          *
*        If fname contains ':', then it is considered to be a DOS filename.    *
*    In this case a temporary Unix/Xenix file is created and returned in the   *
*    (char *) actual string.                                                   *
*        If fname is a directory path, then a file is created in it which      *
*    has the same name as the map (relative paths are taken to be under the    *
*    the default export path [in config]).                                     *
*        If fname is not a directory then it is created.  (relative names      *
*    are treated as being under the default export path).                      *
*                                                                              *
*    Globals/files changed:  the indicated file is created, no other changes.  *
*                                                                              *
*    Parameters:  char fname[]      desired filename string                    *
*                 char actual[]     actual absolute file path created          *
*                                                                              *
*    Returned values:  (long)  0 = file created                                *
*                              1 = couldn't create file                        *
*                                                                              *
*    J.Dabritz  8/23/89                                                        *
*******************************************************************************/
/* Modified 6/92 by DBS@CWU for multi user support */

#include <string.h>
#include "../../incl/globals.h"
#include <sys/stat.h>
#include <errno.h>

long setup_ofile(fname,actual)
	char fname[];
	char actual[];
{
	extern int  errno;
	struct stat buf;

	char tmp[L_tmpnam];
	char wrk[256];
	char string[256];


		/************** check input & output parameters *************/
	if(!fname  ||  !actual)
		return(1);


		/************** check for map loaded ************************/
	if(!job_stats[0])
		return(1);


		/************** check for absolute or relative path *********/
		/************** & add default export path if necessary ******/
	trim(fname);
	strcpy(actual,fname);
	if((*actual != '/')  &&  (actual[1] != ':'))      /** a relative name **/
	{	trim(config.export_path);
		if(*config.export_path == (char) 0)
#ifdef MULTIUSER
			sprintf(config.export_path,"%s/.ltp/export", home_path); /* default path */
			if(!file_exists(config.export_path,""))
				makedir(config.export_path);
#else
			sprintf(config.export_path,"%s/export", ltp_path); /* default path */
#endif
		sprintf(wrk,"%s/%s",config.export_path,actual);
		strcpy(actual,wrk);
	}


		/************** check file for ':' (DOS file) ***************/
		/************** & get a temp file name (absolute) ***********/
	if(strchr(actual,':'))      /** if true, we have a DOS file name **/
	{	strcpy(fname,actual);
		tmpnam(tmp);
		strcpy(actual,tmp);
	}

	

		/************** check to see if it is a directory ***********/
		/************** & add map_name to end of directory path *****/
	if( stat(actual, &buf) )
	{	if(errno == ENOTDIR) /** a component of the path prefix is not a dir*/
		{	sprintf(string,"bad directory path:  %s",actual);
			message(string);
			return(1);
		}
	}


		/************** create file & load name into actual parameter  */
	if(creatf(actual) < 0)
	{	sprintf(wrk,"%s/%s",actual,map_name);
		strcpy(actual,wrk);
		if(creatf(actual) < 0)
		{	sprintf(string,"cannot create file:  %s  or  %s",actual,wrk);
			message(string);
			return(1);
		}
	}


		/************** return **************************************/
	return(0);
}
