/*******************************************************************************
*       stpln_data      (see manual, Internal Functions)                <in>   *
*                 This program sets the stpln_params[16] global to the         *
*                 values determined by the input state plane zone.             *
*                 It also resets the header.stplzn global.                     *
*                                                                              *
*                 Data in the register.h global is modified:                   *
*                      stpln_params[0] -  zone type:  1 = lambert              *
*                           (projection type)         2 = transverse mercator  *
*                                                     3 = Alaska zones 2-9     *
*                                                     4 = Alaska zone 1        *
*                                                     5 = azmthl equdist (Guam)*
*                      stpln_params[1]-[15] - projection parameters.  The      *
*                          number of parameters vary with the projection type. *
*                 The header.stplzn global is reset.                           *
*                                                                              *
*                 Note:  These values and the formulae to use them were taken  *
*                 from"STATE PLANE COORDINATES BY AUTOMATIC DATA PROCESSING"  *
*                 by Charles N Claire, Geodesist (retired)                     *
*                 published by the Department of Commerce, Environmental       *
*                 Science Services Administration, Coast and Geodetic Survey,  *
*                 Publication 62-4                                             *
*                                                                              *
*                 Returned values:  0 (if zone is found)  or -1 (if zone not   *
*                                   is not found)                              *
*                                                                              *
*     j.dabritz  3/7/89 - 5/8/89  , Data converted from LIDES code (circa 86)  *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long stpln_data(zone)
long zone;
{	long i;

		/*   set all parameters to 0 or blank  */
	for(i=0; i<16; i++)
		stpln_params[i] = 0.0;        /*  set all parameters to 0    */

		/*   set new values depending on input zone   */
	switch (zone)  {
	case 101:
		strcpy(header.stplzn,"101     ALABAMA EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 309000.00;
		stpln_params[3] = 1822;
		stpln_params[4] = 21.00903;
		stpln_params[5] = .9999600000;
		stpln_params[6] = .3817065;
		break;
	case 102:
		strcpy(header.stplzn,"102     ALABAMA WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 315000.00;
		stpln_params[3] = 1792;
		stpln_params[4] = 25.53386;
		stpln_params[5] = .9999333333;
		stpln_params[6] = .3817477;
		break;
	case 201:
		strcpy(header.stplzn,"201     ARIZONA EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 396600.00;
		stpln_params[3] = 1852;
		stpln_params[4] = 16.62358;
		stpln_params[5] = .9999000000;
		stpln_params[6] = .3816485;
		break;
	case 202:
		strcpy(header.stplzn,"202     ARIZONA CENTRAL ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 402900.00;
		stpln_params[3] = 1852;
		stpln_params[4] = 16.62358;
		stpln_params[5] = .9999000000;
		stpln_params[6] = .3816485;
		break;
	case 203:
		strcpy(header.stplzn,"203     ARIZONA WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 409500.00;
		stpln_params[3] = 1852;
		stpln_params[4] = 16.62358;
		stpln_params[5] = .9999333333;
		stpln_params[6] = .3815948;
		break;
	case 301:
		strcpy(header.stplzn,"301     ARKANSAS NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 331200.00;
		stpln_params[3] = 29277593.61;
		stpln_params[4] = 29732882.87;
		stpln_params[5] = .9999359370;
		stpln_params[6] = .5818991407;
		stpln_params[7] = 2126;
		stpln_params[8] = 46.35656;
		stpln_params[9] = 3.81452;
		stpln_params[10] = 3.26432;
		stpln_params[11] = 0;
		break;
	case 302:
		strcpy(header.stplzn,"302     ARKANSAS SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 331200.00;
		stpln_params[3] = 31014039.23;
		stpln_params[4] = 31511724.20;
		stpln_params[5] = .9999184698;
		stpln_params[6] = .5596906871;
		stpln_params[7] = 2033;
		stpln_params[8] = 56.94711;
		stpln_params[9] = 3.81550;
		stpln_params[10] = 3.08256;
		stpln_params[11] = 0;
		break;
	case 401:
		strcpy(header.stplzn,"401     CALIFORNIA FAR NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 439200.00;
		stpln_params[3] = 24245358.05;
		stpln_params[4] = 24792436.23;
		stpln_params[5] = .9998946358;
		stpln_params[6] = .6538843192;
		stpln_params[7] = 2441;
		stpln_params[8] = 26.75847;
		stpln_params[9] = 3.80992;
		stpln_params[10] = 3.93575;
		stpln_params[11] = 0;
		break;
	case 402:
		strcpy(header.stplzn,"402     CALIFORNIA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 439200.00;
		stpln_params[3] = 25795850.31;
		stpln_params[4] = 26312257.65;
		stpln_params[5] = .9999146793;
		stpln_params[6] = .6304679732;
		stpln_params[7] = 2336;
		stpln_params[8] = 30.81964;
		stpln_params[9] = 3.81147;
		stpln_params[10] = 3.70114;
		stpln_params[11] = 0;
		break;
	case 403:
		strcpy(header.stplzn,"403     CALIFORNIA NORTH CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 433800.00;
		stpln_params[3] = 27057475.85;
		stpln_params[4] = 27512992.04;
		stpln_params[5] = .9999291792;
		stpln_params[6] = .6122320427;
		stpln_params[7] = 2256;
		stpln_params[8] = 35.52018;
		stpln_params[9] = 3.81265;
		stpln_params[10] = 3.52998;
		stpln_params[11] = 0;
		break;
	case 404:
		strcpy(header.stplzn,"404     CALIFORNIA CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 428400.00;
		stpln_params[3] = 28182405.33;
		stpln_params[4] = 28652931.96;
		stpln_params[5] = .9999407628;
		stpln_params[6] = .5965871443;
		stpln_params[7] = 2189;
		stpln_params[8] = 10.35494;
		stpln_params[9] = 3.81362;
		stpln_params[10] = 3.39020;
		stpln_params[11] = 0;
		break;
	case 405:
		strcpy(header.stplzn,"405     CALIFORNIA SOUTH CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 424800.00;
		stpln_params[3] = 30194145.54;
		stpln_params[4] = 30649424.27;
		stpln_params[5] = .9999221277;
		stpln_params[6] = .5700119219;
		stpln_params[7] = 2076;
		stpln_params[8] = 52.10305;
		stpln_params[9] = 3.81523;
		stpln_params[10] = 3.16593;
		stpln_params[11] = 0;
		break;
	case 406:
		strcpy(header.stplzn,"406     CALIFORNIA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 418500.00;
		stpln_params[3] = 31846570.92;
		stpln_params[4] = 32271267.72;
		stpln_params[5] = .9999541438;
		stpln_params[6] = .5495175982;
		stpln_params[7] = 1992;
		stpln_params[8] = 00.16335;
		stpln_params[9] = 3.81642;
		stpln_params[10] = 3.00292;
		stpln_params[11] = 0;
		break;
	case 407:
		strcpy(header.stplzn,"407     CALIFORNIA FAR SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 4186692.58;
		stpln_params[2] = 426000.00;
		stpln_params[3] = 30891382.10;
		stpln_params[4] = 35055396.31;
		stpln_params[5] = .9999885350;
		stpln_params[6] = .5612432071;
		stpln_params[7] = 2040;
		stpln_params[8] = 22.88096;
		stpln_params[9] = 3.81572;
		stpln_params[10] = 3.09520;
		stpln_params[11] = 0;
		break;
	case 501:
		strcpy(header.stplzn,"501     COLORADO NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 379800.00;
		stpln_params[3] = 24751897.68;
		stpln_params[4] = 25086068.20;
		stpln_params[5] = .9999568475;
		stpln_params[6] = .6461334829;
		stpln_params[7] = 2406;
		stpln_params[8] = 24.62308;
		stpln_params[9] = 3.81044;
		stpln_params[10] = 3.85610;
		stpln_params[11] = 0;
		break;
	case 502:
		strcpy(header.stplzn,"502     COLORADO CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 379800.00;
		stpln_params[3] = 25781376.91;
		stpln_params[4] = 26243052.74;
		stpln_params[5] = .9999359117;
		stpln_params[6] = .6306895773;
		stpln_params[7] = 2337;
		stpln_params[8] = 29.65162;
		stpln_params[9] = 3.81146;
		stpln_params[10] = 3.70326;
		stpln_params[11] = 0;
		break;
	case 503:
		strcpy(header.stplzn,"503     COLORADO SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 379800.00;
		stpln_params[3] = 26977133.89;
		stpln_params[4] = 27402231.82;
		stpln_params[5] = .9999453995;
		stpln_params[6] = .6133780528;
		stpln_params[7] = 2261;
		stpln_params[8] = 34.26662;
		stpln_params[9] = 3.81257;
		stpln_params[10] = 3.54046;
		stpln_params[11] = 0;
		break;
	case 600:
		strcpy(header.stplzn,"600     CONNECTICUT");
		stpln_params[0] = 1;
		stpln_params[1] = 600000.00;
		stpln_params[2] = 261900.00;
		stpln_params[3] = 23659233.56;
		stpln_params[4] = 23914389.02;
		stpln_params[5] = .9999831405;
		stpln_params[6] = .6630594147;
		stpln_params[7] = 2483;
		stpln_params[8] = 19.67980;
		stpln_params[9] = 3.80929;
		stpln_params[10] = 4.03278;
		stpln_params[11] = 0;
		break;
	case 700:
		strcpy(header.stplzn,"700     DELAWARE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 271500.00;
		stpln_params[3] = 2271;
		stpln_params[4] = 30.53702;
		stpln_params[5] = .9999950281;
		stpln_params[6] = .3811454;
		break;
	case 901:
		strcpy(header.stplzn,"901     FLORIDA EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 291600.00;
		stpln_params[3] = 1453;
		stpln_params[4] = 26.09287;
		stpln_params[5] = .9999411765;
		stpln_params[6] = .3821090;
		break;
	case 902:
		strcpy(header.stplzn,"902     FLORIDA WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 295200.00;
		stpln_params[3] = 1453;
		stpln_params[4] = 26.09287;
		stpln_params[5] = .9999411765;
		stpln_params[6] = .3821090;
		break;
	case 903:
		strcpy(header.stplzn,"903     FLORIDA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 304200.00;
		stpln_params[3] = 36030443.05;
		stpln_params[4] = 36454924.53;
		stpln_params[5] = .9999484343;
		stpln_params[6] = .5025259000;
		stpln_params[7] = 1802;
		stpln_params[8] = 26.11701;
		stpln_params[9] = 3.81898;
		stpln_params[10] = 2.65643;
		stpln_params[11] = 0;
		break;
	case 1001:
		strcpy(header.stplzn,"1001     GEORGIA EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 295800.00;
		stpln_params[3] = 1792;
		stpln_params[4] = 25.53386;
		stpln_params[5] = .9999000000;
		stpln_params[6] = .3817593;
		break;
	case 1002:
		strcpy(header.stplzn,"1002     GEORGIA WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 303000.00;
		stpln_params[3] = 1792;
		stpln_params[4] = 25.53386;
		stpln_params[5] = .9999000000;
		stpln_params[6] = .3817593;
		break;
	case 1101:
		strcpy(header.stplzn,"1101     IDAHO EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 403800.00;
		stpln_params[3] = 2491;
		stpln_params[4] = 18.35156;
		stpln_params[5] = .9999473684;
		stpln_params[6] = .3807624;
		break;
	case 1102:
		strcpy(header.stplzn,"1102     IDAHO CENTRAL ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 410400.00;
		stpln_params[3] = 2491;
		stpln_params[4] = 18.35156;
		stpln_params[5] = .9999473684;
		stpln_params[6] = .3807624;
		break;
	case 1103:
		strcpy(header.stplzn,"1103     IDAHO WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 416700.00;
		stpln_params[3] = 2491;
		stpln_params[4] = 18.35156;
		stpln_params[5] = .9999333333;
		stpln_params[6] = .3806227;
		break;
	case 1201:
		strcpy(header.stplzn,"1201     ILLINOIS EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 318000.00;
		stpln_params[3] = 2191;
		stpln_params[4] = 37.04639;
		stpln_params[5] = .9999750000;
		stpln_params[6] = .3811074;
		break;
	case 1202:
		strcpy(header.stplzn,"1202     ILLINOIS WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 324600.00;
		stpln_params[3] = 2191;
		stpln_params[4] = 37.04639;
		stpln_params[5] = .9999411765;
		stpln_params[6] = .3811332;
		break;
	case 1301:
		strcpy(header.stplzn,"1301     INDIANA EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 308400.00;
		stpln_params[3] = 2241;
		stpln_params[4] = 32.84965;
		stpln_params[5] = .9999666667;
		stpln_params[6] = .3811064;
		break;
	case 1302:
		strcpy(header.stplzn,"1302     INDIANA WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 313500.00;
		stpln_params[3] = 2241;
		stpln_params[4] = 32.84965;
		stpln_params[5] = .9999666667;
		stpln_params[6] = .3811064;
		break;
	case 1401:
		strcpy(header.stplzn,"1401     IOWA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 336600.00;
		stpln_params[3] = 22736950.34;
		stpln_params[4] = 23162461.59;
		stpln_params[5] = .9999453686;
		stpln_params[6] = .6777445518;
		stpln_params[7] = 2551;
		stpln_params[8] = 20.02265;
		stpln_params[9] = 3.80827;
		stpln_params[10] = 4.19479;
		stpln_params[11] = 0;
		break;
	case 1402:
		strcpy(header.stplzn,"1402     IOWA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 336600.00;
		stpln_params[3] = 23936585.11;
		stpln_params[4] = 24374096.67;
		stpln_params[5] = .9999483705;
		stpln_params[6] = .6587010213;
		stpln_params[7] = 2463;
		stpln_params[8] = 22.59905;
		stpln_params[9] = 3.80959;
		stpln_params[10] = 3.98630;
		stpln_params[11] = 0;
		break;
	case 1501:
		strcpy(header.stplzn,"1501     KANSAS NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 352800.00;
		stpln_params[3] = 25644959.12;
		stpln_params[4] = 25979068.57;
		stpln_params[5] = .9999568556;
		stpln_params[6] = .6327148646;
		stpln_params[7] = 2346;
		stpln_params[8] = 27.97215;
		stpln_params[9] = 3.81133;
		stpln_params[10] = 3.72376;
		stpln_params[11] = 0;
		break;
	case 1502:
		strcpy(header.stplzn,"1502     KANSAS SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 354600.00;
		stpln_params[3] = 26896024.48;
		stpln_params[4] = 27351521.50;
		stpln_params[5] = .9999359200;
		stpln_params[6] = .6145281068;
		stpln_params[7] = 2266;
		stpln_params[8] = 34.41020;
		stpln_params[9] = 3.81250;
		stpln_params[10] = 3.55102;
		stpln_params[11] = 0;
		break;
	case 1601:
		strcpy(header.stplzn,"1601     KENTUCKY NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 303300.00;
		stpln_params[3] = 26371820.68;
		stpln_params[4] = 26724051.82;
		stpln_params[5] = .9999620817;
		stpln_params[6] = .6220672671;
		stpln_params[7] = 2299;
		stpln_params[8] = 30.63364;
		stpln_params[9] = 3.81202;
		stpln_params[10] = 3.62113;
		stpln_params[11] = 0;
		break;
	case 1602:
		strcpy(header.stplzn,"1602     KENTUCKY SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 308700.00;
		stpln_params[3] = 27467860.75;
		stpln_params[4] = 27832235.64;
		stpln_params[5] = .9999453808;
		stpln_params[6] = .6064623718;
		stpln_params[7] = 2231;
		stpln_params[8] = 36.57874;
		stpln_params[9] = 3.81301;
		stpln_params[10] = 3.47771;
		stpln_params[11] = 0;
		break;
	case 1701:
		strcpy(header.stplzn,"1701     LOUISIANA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 333000.00;
		stpln_params[3] = 33624568.36;
		stpln_params[4] = 34079629.33;
		stpln_params[5] = .9999147417;
		stpln_params[6] = .5287006734;
		stpln_params[7] = 1907;
		stpln_params[8] = 12.68515;
		stpln_params[9] = 3.81758;
		stpln_params[10] = 2.84511;
		stpln_params[11] = 0;
		break;
	case 1702:
		strcpy(header.stplzn,"1702     LOUISIANA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 328800.00;
		stpln_params[3] = 36271389.35;
		stpln_params[4] = 36756553.45;
		stpln_params[5] = .9999257458;
		stpln_params[6] = .5000126971;
		stpln_params[7] = 1792;
		stpln_params[8] = 28.55026;
		stpln_params[9] = 3.81911;
		stpln_params[10] = 2.63885;
		stpln_params[11] = 0;
		break;
	case 1703:
		strcpy(header.stplzn,"1703     LOUISIANA OFFSHORE ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 328800.00;
		stpln_params[3] = 41091749.54;
		stpln_params[4] = 41576762.39;
		stpln_params[5] = .9998947956;
		stpln_params[6] = .4540068519;
		stpln_params[7] = 1612;
		stpln_params[8] = 59.30342;
		stpln_params[9] = 3.82138;
		stpln_params[10] = 2.27436;
		stpln_params[11] = 25;
		break;
	case 1801:
		strcpy(header.stplzn,"1801     MAINE EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 246600.00;
		stpln_params[3] = 2621;
		stpln_params[4] = 15.15187;
		stpln_params[5] = .9999000000;
		stpln_params[6] = .3806180;
		break;
	case 1802:
		strcpy(header.stplzn,"1802     MAINE WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 252600.00;
		stpln_params[3] = 2561;
		stpln_params[4] = 16.25668;
		stpln_params[5] = .9999666667;
		stpln_params[6] = .3806575;
		break;
	case 1900:
		strcpy(header.stplzn,"1900     MARYLAND");
		stpln_params[0] = 1;
		stpln_params[1] = 800000.00;
		stpln_params[2] = 277200.00;
		stpln_params[3] = 25989474.99;
		stpln_params[4] = 26369112.76;
		stpln_params[5] = .9999498485;
		stpln_params[6] = .6276341196;
		stpln_params[7] = 2323;
		stpln_params[8] = 59.69369;
		stpln_params[9] = 3.81166;
		stpln_params[10] = 3.67392;
		stpln_params[11] = 0;
		break;
	case 2001:
		strcpy(header.stplzn,"2001     MASSACHUSETTS MAINLAND ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 600000.00;
		stpln_params[2] = 257400.00;
		stpln_params[3] = 23111975.14;
		stpln_params[4] = 23549477.32;
		stpln_params[5] = .9999645506;
		stpln_params[6] = .6717286561;
		stpln_params[7] = 2523;
		stpln_params[8] = 19.53138;
		stpln_params[9] = 3.80870;
		stpln_params[10] = 4.12738;
		stpln_params[11] = 0;
		break;
	case 2002:
		strcpy(header.stplzn,"2002     MASSACHUSETTS ISLAND ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 200000.00;
		stpln_params[2] = 253800.00;
		stpln_params[3] = 23784678.44;
		stpln_params[4] = 23924398.02;
		stpln_params[5] = .9999984844;
		stpln_params[6] = .6610953994;
		stpln_params[7] = 2474;
		stpln_params[8] = 19.47463;
		stpln_params[9] = 3.80943;
		stpln_params[10] = 4.01174;
		stpln_params[11] = 0;
		break;
	case 2101:
		strcpy(header.stplzn,"2101     MICHIGAN EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 301200.00;
		stpln_params[3] = 2481;
		stpln_params[4] = 18.72150;
		stpln_params[5] = .9999428571;
		stpln_params[6] = .3807283;
		break;
	case 2102:
		strcpy(header.stplzn,"2102     MICHIGAN CENTRAL ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 308700.00;
		stpln_params[3] = 2481;
		stpln_params[4] = 18.72150;
		stpln_params[5] = .9999090909;
		stpln_params[6] = .3807541;
		break;
	case 2103:
		strcpy(header.stplzn,"2103     MICHIGAN WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 319500.00;
		stpln_params[3] = 2481;
		stpln_params[4] = 18.72150;
		stpln_params[5] = .9999090909;
		stpln_params[6] = .3805361;
		break;
	case 2111:
		strcpy(header.stplzn,"2111     MICHIGAN NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 313200.00;
		stpln_params[3] = 20041716.18;
		stpln_params[4] = 20589420.09;
		stpln_params[5] = .9999410344;
		stpln_params[6] = .7227899381;
		stpln_params[7] = 2768;
		stpln_params[8] = 22.25085;
		stpln_params[9] = 3.80501;
		stpln_params[10] = 4.68430;
		stpln_params[11] = 36;
		break;
	case 2112:
		strcpy(header.stplzn,"2112     MICHIGAN CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 303600.00;
		stpln_params[3] = 21001715.22;
		stpln_params[4] = 21594768.40;
		stpln_params[5] = .9999509058;
		stpln_params[6] = .7064074100;
		stpln_params[7] = 2687;
		stpln_params[8] = 50.76661;
		stpln_params[9] = 3.80622;
		stpln_params[10] = 4.46875;
		stpln_params[11] = 35;
		break;
	case 2113:
		strcpy(header.stplzn,"2113     MICHIGAN SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 303600.00;
		stpln_params[3] = 22564848.51;
		stpln_params[4] = 23069597.22;
		stpln_params[5] = .9999450783;
		stpln_params[6] = .6805292633;
		stpln_params[7] = 2564;
		stpln_params[8] = 22.23938;
		stpln_params[9] = 3.80808;
		stpln_params[10] = 4.15706;
		stpln_params[11] = 33;
		break;
	case 2201:
		strcpy(header.stplzn,"2201     MINNESOTA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 335160.00;
		stpln_params[3] = 18984319.62;
		stpln_params[4] = 19471398.75;
		stpln_params[5] = .9999028166;
		stpln_params[6] = .7412196637;
		stpln_params[7] = 2861;
		stpln_params[8] = 24.63011;
		stpln_params[9] = 3.80362;
		stpln_params[10] = 5.01609;
		stpln_params[11] = 0;
		break;
	case 2202:
		strcpy(header.stplzn,"2202     MINNESOTA CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 339300.00;
		stpln_params[3] = 20006679.72;
		stpln_params[4] = 20493457.15;
		stpln_params[5] = .9999220223;
		stpln_params[6] = .7233880702;
		stpln_params[7] = 2771;
		stpln_params[8] = 20.89747;
		stpln_params[9] = 3.80497;
		stpln_params[10] = 4.76197;
		stpln_params[11] = 0;
		break;
	case 2203:
		strcpy(header.stplzn,"2203     MINNESOTA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 338400.00;
		stpln_params[3] = 21327006.06;
		stpln_params[4] = 21874349.14;
		stpln_params[5] = .9999220448;
		stpln_params[6] = .7009277824;
		stpln_params[7] = 2661;
		stpln_params[8] = 20.12517;
		stpln_params[9] = 3.80662;
		stpln_params[10] = 4.46959;
		stpln_params[11] = 0;
		break;
	case 2301:
		strcpy(header.stplzn,"2301     MISSISSIPPI EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 319800.00;
		stpln_params[3] = 1772;
		stpln_params[4] = 28.62716;
		stpln_params[5] = .9999600000;
		stpln_params[6] = .3817257;
		break;
	case 2302:
		strcpy(header.stplzn,"2302     MISSISSIPPI WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 325200.00;
		stpln_params[3] = 1822;
		stpln_params[4] = 21.00903;
		stpln_params[5] = .9999411765;
		stpln_params[6] = .3816986;
		break;
	case 2401:
		strcpy(header.stplzn,"2401     MISSOURI EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 325800.00;
		stpln_params[3] = 2141;
		stpln_params[4] = 41.66790;
		stpln_params[5] = .9999333333;
		stpln_params[6] = .3812643;
		break;
	case 2402:
		strcpy(header.stplzn,"2402     MISSOURI CENTRAL ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 333000.00;
		stpln_params[3] = 2141;
		stpln_params[4] = 41.66790;
		stpln_params[5] = .9999333333;
		stpln_params[6] = .3812422;
		break;
	case 2403:
		strcpy(header.stplzn,"2403     MISSOURI WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 340200.00;
		stpln_params[3] = 2161;
		stpln_params[4] = 39.76857;
		stpln_params[5] = .9999411765;
		stpln_params[6] = .3812362;
		break;
	case 2501:
		strcpy(header.stplzn,"2501     MONTANA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 394200.00;
		stpln_params[3] = 18689498.40;
		stpln_params[4] = 19157874.26;
		stpln_params[5] = .9999714855;
		stpln_params[6] = .7464518080;
		stpln_params[7] = 2888;
		stpln_params[8] = 20.21285;
		stpln_params[9] = 3.80322;
		stpln_params[10] = 5.09490;
		stpln_params[11] = 0;
		break;
	case 2502:
		strcpy(header.stplzn,"2502     MONTANA CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 394200.00;
		stpln_params[3] = 19432939.76;
		stpln_params[4] = 19919806.36;
		stpln_params[5] = .9999220151;
		stpln_params[6] = .7333538278;
		stpln_params[7] = 2821;
		stpln_params[8] = 21.96779;
		stpln_params[9] = 3.80422;
		stpln_params[10] = 4.90135;
		stpln_params[11] = 0;
		break;
	case 2503:
		strcpy(header.stplzn,"2503     MONTANA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 394200.00;
		stpln_params[3] = 20500650.51;
		stpln_params[4] = 21096820.93;
		stpln_params[5] = .9999107701;
		stpln_params[6] = .7149012442;
		stpln_params[7] = 2729;
		stpln_params[8] = 21.15820;
		stpln_params[9] = 3.80560;
		stpln_params[10] = 4.64814;
		stpln_params[11] = 0;
		break;
	case 2601:
		strcpy(header.stplzn,"2601     NEBRASKA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 360000.00;
		stpln_params[3] = 23004346.29;
		stpln_params[4] = 23368977.46;
		stpln_params[5] = .9999645501;
		stpln_params[6] = .6734507906;
		stpln_params[7] = 2531;
		stpln_params[8] = 19.30504;
		stpln_params[9] = 3.80858;
		stpln_params[10] = 4.14653;
		stpln_params[11] = 0;
		break;
	case 2602:
		strcpy(header.stplzn,"2602     NEBRASKA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 358200.00;
		stpln_params[3] = 24104561.06;
		stpln_params[4] = 24590781.86;
		stpln_params[5] = .9999220725;
		stpln_params[6] = .6560764003;
		stpln_params[7] = 2451;
		stpln_params[8] = 24.68139;
		stpln_params[9] = 3.80977;
		stpln_params[10] = 3.95865;
		stpln_params[11] = 0;
		break;
	case 2701:
		strcpy(header.stplzn,"2701     NEVADA EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 416100.00;
		stpln_params[3] = 2076;
		stpln_params[4] = 48.30429;
		stpln_params[5] = .9999000000;
		stpln_params[6] = .3812311;
		break;
	case 2702:
		strcpy(header.stplzn,"2702     NEVADA CENTRAL ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 420000.00;
		stpln_params[3] = 2076;
		stpln_params[4] = 48.30429;
		stpln_params[5] = .9999000000;
		stpln_params[6] = .3812311;
		break;
	case 2703:
		strcpy(header.stplzn,"2703     NEVADA WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 426900.00;
		stpln_params[3] = 2076;
		stpln_params[4] = 48.30429;
		stpln_params[5] = .9999000000;
		stpln_params[6] = .3812311;
		break;
	case 2800:
		strcpy(header.stplzn,"2800     NEW HAMPSHIRE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 258000.00;
		stpln_params[3] = 2541;
		stpln_params[4] = 16.76677;
		stpln_params[5] = .9999666667;
		stpln_params[6] = .3807327;
		break;
	case 2900:
		strcpy(header.stplzn,"2900     NEW JERSEY");
		stpln_params[0] = 2;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 268800.00;
		stpln_params[3] = 2321;
		stpln_params[4] = 27.02745;
		stpln_params[5] = .9999750295;
		stpln_params[6] = .3810845;
		break;
	case 3001:
		strcpy(header.stplzn,"3001     NEW MEXICO EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 375600.00;
		stpln_params[3] = 1852;
		stpln_params[4] = 16.62358;
		stpln_params[5] = .9999090909;
		stpln_params[6] = .3816135;
		break;
	case 3002:
		strcpy(header.stplzn,"3002     NEW MEXICO CENTRAL ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 382500.00;
		stpln_params[3] = 1852;
		stpln_params[4] = 16.62358;
		stpln_params[5] = .9999000000;
		stpln_params[6] = .3816204;
		break;
	case 3003:
		strcpy(header.stplzn,"3003     NEW MEXICO WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 388200.00;
		stpln_params[3] = 1852;
		stpln_params[4] = 16.62358;
		stpln_params[5] = .9999166667;
		stpln_params[6] = .3816288;
		break;
	case 3101:
		strcpy(header.stplzn,"3101     NEW YORK EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 267600.00;
		stpln_params[3] = 2391;
		stpln_params[4] = 22.84247;
		stpln_params[5] = .9999666667;
		stpln_params[6] = .3808377;
		break;
	case 3102:
		strcpy(header.stplzn,"3102     NEW YORK CENTRAL ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 275700.00;
		stpln_params[3] = 2391;
		stpln_params[4] = 22.84247;
		stpln_params[5] = .9999375000;
		stpln_params[6] = .3808450;
		break;
	case 3103:
		strcpy(header.stplzn,"3103     NEW YORK WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 282900.00;
		stpln_params[3] = 2391;
		stpln_params[4] = 22.84247;
		stpln_params[5] = .9999375000;
		stpln_params[6] = .3808750;
		break;
	case 3104:
		strcpy(header.stplzn,"3104     NEW YORK LONG ISLAND ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 266400.00;
		stpln_params[3] = 24235000.80;
		stpln_params[4] = 24462545.30;
		stpln_params[5] = .9999949000;
		stpln_params[6] = .6540820950;
		stpln_params[7] = 2442;
		stpln_params[8] = 20.64240;
		stpln_params[9] = 3.80990;
		stpln_params[10] = 3.93780;
		stpln_params[11] = 0;
		break;
	case 3200:
		strcpy(header.stplzn,"3200     NORTH CAROLINA");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 284400.00;
		stpln_params[3] = 29637059.47;
		stpln_params[4] = 30183611.25;
		stpln_params[5] = .9998725510;
		stpln_params[6] = .5771707700;
		stpln_params[7] = 2106;
		stpln_params[8] = 51.60353;
		stpln_params[9] = 3.81480;
		stpln_params[10] = 3.22483;
		stpln_params[11] = 0;
		break;
	case 3301:
		strcpy(header.stplzn,"3301     NORTH DAKOTA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 361800.00;
		stpln_params[3] = 18819849.05;
		stpln_params[4] = 19215516.01;
		stpln_params[5] = .9999358426;
		stpln_params[6] = .7441333961;
		stpln_params[7] = 2876;
		stpln_params[8] = 22.57950;
		stpln_params[9] = 3.80339;
		stpln_params[10] = 5.05972;
		stpln_params[11] = 0;
		break;
	case 3302:
		strcpy(header.stplzn,"3302     NORTH DAKOTA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 361800.00;
		stpln_params[3] = 19661027.79;
		stpln_params[4] = 20086977.18;
		stpln_params[5] = .9999358523;
		stpln_params[6] = .7293826040;
		stpln_params[7] = 2801;
		stpln_params[8] = 20.45445;
		stpln_params[9] = 3.80452;
		stpln_params[10] = 4.84504;
		stpln_params[11] = 0;
		break;
	case 3401:
		strcpy(header.stplzn,"3401     OHIO NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 297000.00;
		stpln_params[3] = 24048738.51;
		stpln_params[4] = 24559158.47;
		stpln_params[5] = .9999391411;
		stpln_params[6] = .6569503193;
		stpln_params[7] = 2455;
		stpln_params[8] = 23.48125;
		stpln_params[9] = 3.80971;
		stpln_params[10] = 3.96783;
		stpln_params[11] = 0;
		break;
	case 3402:
		strcpy(header.stplzn,"3402     OHIO SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 297000.00;
		stpln_params[3] = 25522875.81;
		stpln_params[4] = 26027071.12;
		stpln_params[5] = .9999359346;
		stpln_params[6] = .6345195439;
		stpln_params[7] = 2354;
		stpln_params[8] = 28.63705;
		stpln_params[9] = 3.81121;
		stpln_params[10] = 3.74048;
		stpln_params[11] = 0;
		break;
	case 3501:
		strcpy(header.stplzn,"3501     OKLAHOMA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 352800.00;
		stpln_params[3] = 28657871.66;
		stpln_params[4] = 29082831.70;
		stpln_params[5] = .9999454101;
		stpln_params[6] = .5901470744;
		stpln_params[7] = 2161;
		stpln_params[8] = 42.56887;
		stpln_params[9] = 3.81402;
		stpln_params[10] = 3.33440;
		stpln_params[11] = 0;
		break;
	case 3502:
		strcpy(header.stplzn,"3502     OKLAHOMA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 352800.00;
		stpln_params[3] = 30382831.06;
		stpln_params[4] = 30838032.96;
		stpln_params[5] = .9999359432;
		stpln_params[6] = .5676166827;
		stpln_params[7] = 2066;
		stpln_params[8] = 52.48935;
		stpln_params[9] = 3.81537;
		stpln_params[10] = 3.14645;
		stpln_params[11] = 0;
		break;
	case 3601:
		strcpy(header.stplzn,"3601     OREGON NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 433800.00;
		stpln_params[3] = 20836250.94;
		stpln_params[4] = 21383852.48;
		stpln_params[5] = .9998945810;
		stpln_params[6] = .7091860222;
		stpln_params[7] = 2701;
		stpln_params[8] = 22.08858;
		stpln_params[9] = 3.80602;
		stpln_params[10] = 4.57382;
		stpln_params[11] = 0;
		break;
	case 3602:
		strcpy(header.stplzn,"3602     OREGON SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 433800.00;
		stpln_params[3] = 22341309.43;
		stpln_params[4] = 22888667.15;
		stpln_params[5] = .9998946058;
		stpln_params[6] = .6841473833;
		stpln_params[7] = 2581;
		stpln_params[8] = 22.74104;
		stpln_params[9] = 3.80782;
		stpln_params[10] = 4.26823;
		stpln_params[11] = 0;
		break;
	case 3701:
		strcpy(header.stplzn,"3701     PENNSYLVANIA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 279900.00;
		stpln_params[3] = 23755351.27;
		stpln_params[4] = 24211050.37;
		stpln_params[5] = .9999568410;
		stpln_params[6] = .6615397363;
		stpln_params[7] = 2476;
		stpln_params[8] = 21.57953;
		stpln_params[9] = 3.80940;
		stpln_params[10] = 4.01753;
		stpln_params[11] = 0;
		break;
	case 3702:
		strcpy(header.stplzn,"3702     PENNSYLVANIA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 279900.00;
		stpln_params[3] = 24577800.67;
		stpln_params[4] = 24984826.43;
		stpln_params[5] = .9999595012;
		stpln_params[6] = .6487931668;
		stpln_params[7] = 2418;
		stpln_params[8] = 23.87979;
		stpln_params[9] = 3.81026;
		stpln_params[10] = 3.88319;
		stpln_params[11] = 0;
		break;
	case 3800:
		strcpy(header.stplzn,"3800     RHODE ISLAND");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 257400.00;
		stpln_params[3] = 2456;
		stpln_params[4] = 19.72344;
		stpln_params[5] = .9999937500;
		stpln_params[6] = .3809220;
		break;
	case 3901:
		strcpy(header.stplzn,"3901     SOUTH CAROLINA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 291600.00;
		stpln_params[3] = 30630125.53;
		stpln_params[4] = 31127724.75;
		stpln_params[5] = .9999454207;
		stpln_params[6] = .5644973800;
		stpln_params[7] = 2053;
		stpln_params[8] = 53.44099;
		stpln_params[9] = 3.81555;
		stpln_params[10] = 3.12127;
		stpln_params[11] = 0;
		break;
	case 3902:
		strcpy(header.stplzn,"3902     SOUTH CAROLINA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 291600.00;
		stpln_params[3] = 32252126.30;
		stpln_params[4] = 32676887.65;
		stpln_params[5] = .9999326284;
		stpln_params[6] = .5446515700;
		stpln_params[7] = 1972;
		stpln_params[8] = 3.57839;
		stpln_params[9] = 3.81669;
		stpln_params[10] = 2.94381;
		stpln_params[11] = 0;
		break;
	case 4001:
		strcpy(header.stplzn,"4001     SOUTH DAKOTA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 360000.00;
		stpln_params[3] = 20922704.09;
		stpln_params[4] = 21366697.03;
		stpln_params[5] = .9999391116;
		stpln_params[6] = .7077381841;
		stpln_params[7] = 2694;
		stpln_params[8] = 18.93392;
		stpln_params[9] = 3.80612;
		stpln_params[10] = 4.55529;
		stpln_params[11] = 0;
		break;
	case 4002:
		strcpy(header.stplzn,"4002     SOUTH DAKOTA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 361200.00;
		stpln_params[3] = 21993575.61;
		stpln_params[4] = 22461937.05;
		stpln_params[5] = .9999068931;
		stpln_params[6] = .6898519579;
		stpln_params[7] = 2608;
		stpln_params[8] = 21.54370;
		stpln_params[9] = 3.80742;
		stpln_params[10] = 4.33519;
		stpln_params[11] = 0;
		break;
	case 4100:
		strcpy(header.stplzn,"4100     TENNESSEE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 309600.00;
		stpln_params[3] = 29010231.09;
		stpln_params[4] = 29535149.91;
		stpln_params[5] = .9999484030;
		stpln_params[6] = .5854397296;
		stpln_params[7] = 2141;
		stpln_params[8] = 44.28313;
		stpln_params[9] = 3.81431;
		stpln_params[10] = 3.29422;
		stpln_params[11] = 0;
		break;
	case 4201:
		strcpy(header.stplzn,"4201     TEXAS NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 365400.00;
		stpln_params[3] = 29456907.29;
		stpln_params[4] = 29972959.94;
		stpln_params[5] = .9999108771;
		stpln_params[6] = .5795358654;
		stpln_params[7] = 2116;
		stpln_params[8] = 48.58548;
		stpln_params[9] = 3.81466;
		stpln_params[10] = 3.24452;
		stpln_params[11] = 0;
		break;
	case 4202:
		strcpy(header.stplzn,"4202     TEXAS NORTH CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 351000.00;
		stpln_params[3] = 32187809.58;
		stpln_params[4] = 32691654.54;
		stpln_params[5] = .9998726224;
		stpln_params[6] = .5453944146;
		stpln_params[7] = 1975;
		stpln_params[8] = 5.95074;
		stpln_params[9] = 3.81665;
		stpln_params[10] = 2.97107;
		stpln_params[11] = 0;
		break;
	case 4203:
		strcpy(header.stplzn,"4203     TEXAS CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 361200.00;
		stpln_params[3] = 34851703.46;
		stpln_params[4] = 35337121.23;
		stpln_params[5] = .9998817443;
		stpln_params[6] = .5150588857;
		stpln_params[7] = 1852;
		stpln_params[8] = 21.62181;
		stpln_params[9] = 3.81832;
		stpln_params[10] = 2.74550;
		stpln_params[11] = 0;
		break;
	case 4204:
		strcpy(header.stplzn,"4204     TEXAS SOUTH CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 356400.00;
		stpln_params[3] = 37261509.20;
		stpln_params[4] = 37807440.38;
		stpln_params[5] = .9998632433;
		stpln_params[6] = .4899126408;
		stpln_params[7] = 1752;
		stpln_params[8] = 37.19059;
		stpln_params[9] = 3.81962;
		stpln_params[10] = 2.56899;
		stpln_params[11] = 0;
		break;
	case 4205:
		strcpy(header.stplzn,"4205     TEXAS SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 354600.00;
		stpln_params[3] = 41091749.54;
		stpln_params[4] = 41576762.39;
		stpln_params[5] = .9998947956;
		stpln_params[6] = .4540068519;
		stpln_params[7] = 1612;
		stpln_params[8] = 59.30342;
		stpln_params[9] = 3.82138;
		stpln_params[10] = 2.33094;
		stpln_params[11] = 0;
		break;
	case 4301:
		strcpy(header.stplzn,"4301     UTAH NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 401400.00;
		stpln_params[3] = 23894872.45;
		stpln_params[4] = 24229110.29;
		stpln_params[5] = .9999568422;
		stpln_params[6] = .6593554910;
		stpln_params[7] = 2466;
		stpln_params[8] = 21.96231;
		stpln_params[9] = 3.80955;
		stpln_params[10] = 3.99323;
		stpln_params[11] = 0;
		break;
	case 4302:
		strcpy(header.stplzn,"4302     UTAH CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 401400.00;
		stpln_params[3] = 25117176.75;
		stpln_params[4] = 25664114.42;
		stpln_params[5] = .9998988207;
		stpln_params[6] = .6405785926;
		stpln_params[7] = 2381;
		stpln_params[8] = 29.30066;
		stpln_params[9] = 3.81081;
		stpln_params[10] = 3.80024;
		stpln_params[11] = 0;
		break;
	case 4303:
		strcpy(header.stplzn,"4303     UTAH SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 401400.00;
		stpln_params[3] = 27025955.35;
		stpln_params[4] = 27432812.88;
		stpln_params[5] = .9999512939;
		stpln_params[6] = .6126873424;
		stpln_params[7] = 2258;
		stpln_params[8] = 34.16878;
		stpln_params[9] = 3.81262;
		stpln_params[10] = 3.53414;
		stpln_params[11] = 0;
		break;
	case 4400:
		strcpy(header.stplzn,"4400     VERMONT");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 261000.00;
		stpln_params[3] = 2541;
		stpln_params[4] = 16.76677;
		stpln_params[5] = .9999642857;
		stpln_params[6] = .3807420;
		break;
	case 4501:
		strcpy(header.stplzn,"4501     VIRGINIA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 282600.00;
		stpln_params[3] = 26230200.09;
		stpln_params[4] = 26576444.45;
		stpln_params[5] = .9999483859;
		stpln_params[6] = .6241178597;
		stpln_params[7] = 2308;
		stpln_params[8] = 30.78682;
		stpln_params[9] = 3.81189;
		stpln_params[10] = 3.64047;
		stpln_params[11] = 0;
		break;
	case 4502:
		strcpy(header.stplzn,"4502     VIRGINIA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 282600.00;
		stpln_params[3] = 27434800.06;
		stpln_params[4] = 27811312.71;
		stpln_params[5] = .9999454027;
		stpln_params[6] = .6069248249;
		stpln_params[7] = 2233;
		stpln_params[8] = 36.41072;
		stpln_params[9] = 3.81298;
		stpln_params[10] = 3.48187;
		stpln_params[11] = 0;
		break;
	case 4601:
		strcpy(header.stplzn,"4601     WASHINGTON NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 435000.00;
		stpln_params[3] = 18798081.67;
		stpln_params[4] = 19205863.43;
		stpln_params[5] = .9999422551;
		stpln_params[6] = .7445203390;
		stpln_params[7] = 2878;
		stpln_params[8] = 22.15711;
		stpln_params[9] = 3.80336;
		stpln_params[10] = 5.06556;
		stpln_params[11] = 0;
		break;
	case 4602:
		strcpy(header.stplzn,"4602     WASHINGTON SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 433800.00;
		stpln_params[3] = 19832653.52;
		stpln_params[4] = 20289119.60;
		stpln_params[5] = .9999145875;
		stpln_params[6] = .7263957947;
		stpln_params[7] = 2786;
		stpln_params[8] = 21.72121;
		stpln_params[9] = 3.80474;
		stpln_params[10] = 4.80336;
		stpln_params[11] = 0;
		break;
	case 4701:
		strcpy(header.stplzn,"4701     WEST VIRGINIA NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 286200.00;
		stpln_params[3] = 25305029.12;
		stpln_params[4] = 25715126.55;
		stpln_params[5] = .9999407460;
		stpln_params[6] = .6377729696;
		stpln_params[7] = 2368;
		stpln_params[8] = 57.52979;
		stpln_params[9] = 3.81099;
		stpln_params[10] = 3.77244;
		stpln_params[11] = 0;
		break;
	case 4702:
		strcpy(header.stplzn,"4702     WEST VIRGINIA SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 291600.00;
		stpln_params[3] = 26639323.45;
		stpln_params[4] = 27070620.78;
		stpln_params[5] = .9999256928;
		stpln_params[6] = .6181953936;
		stpln_params[7] = 2282;
		stpln_params[8] = 33.82207;
		stpln_params[9] = 3.81227;
		stpln_params[10] = 3.58491;
		stpln_params[11] = 0;
		break;
	case 4801:
		strcpy(header.stplzn,"4801     WISCONSIN NORTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 324000.00;
		stpln_params[3] = 20124133.05;
		stpln_params[4] = 20489179.67;
		stpln_params[5] = .9999453461;
		stpln_params[6] = .7213707913;
		stpln_params[7] = 2761;
		stpln_params[8] = 19.04034;
		stpln_params[9] = 3.80511;
		stpln_params[10] = 4.73451;
		stpln_params[11] = 0;
		break;
	case 4802:
		strcpy(header.stplzn,"4802     WISCONSIN CENTRAL ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 324000.00;
		stpln_params[3] = 21050746.99;
		stpln_params[4] = 21430913.91;
		stpln_params[5] = .9999407059;
		stpln_params[6] = .7055766312;
		stpln_params[7] = 2683;
		stpln_params[8] = 48.81363;
		stpln_params[9] = 3.80628;
		stpln_params[10] = 4.52782;
		stpln_params[11] = 0;
		break;
	case 4803:
		strcpy(header.stplzn,"4803     WISCONSIN SOUTH ZONE");
		stpln_params[0] = 1;
		stpln_params[1] = 2000000.00;
		stpln_params[2] = 324000.00;
		stpln_params[3] = 22161432.25;
		stpln_params[4] = 22672134.66;
		stpln_params[5] = .9999325474;
		stpln_params[6] = .6871032423;
		stpln_params[7] = 2595;
		stpln_params[8] = 20.01691;
		stpln_params[9] = 3.80761;
		stpln_params[10] = 4.30274;
		stpln_params[11] = 0;
		break;
	case 4901:
		strcpy(header.stplzn,"4901     WYOMING EAST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 378600.00;
		stpln_params[3] = 2431;
		stpln_params[4] = 20.83533;
		stpln_params[5] = .9999411765;
		stpln_params[6] = .3808422;
		break;
	case 4902:
		strcpy(header.stplzn,"4902     WYOMING EAST CENTRAL ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 386400.00;
		stpln_params[3] = 2431;
		stpln_params[4] = 20.83533;
		stpln_params[5] = .9999411765;
		stpln_params[6] = .3808422;
		break;
	case 4903:
		strcpy(header.stplzn,"4903     WYOMING WEST CENTRAL ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 391500.00;
		stpln_params[3] = 2431;
		stpln_params[4] = 20.83533;
		stpln_params[5] = .9999411765;
		stpln_params[6] = .3808422;
		break;
	case 4904:
		strcpy(header.stplzn,"4904     WYOMING WEST ZONE");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 396300.00;
		stpln_params[3] = 2431;
		stpln_params[4] = 20.83533;
		stpln_params[5] = .9999411765;
		stpln_params[6] = .3808422;
		break;
	case 5001:
		strcpy(header.stplzn,"5001     ALASKA SE ZONE (LAT<60)");
		stpln_params[0] = 4;
		stpln_params[1] = 6388906.01513;
		stpln_params[2] = 1.00029977273;
		stpln_params[3] = 0.00447599131;
		stpln_params[4] = 6386352.67013;
		stpln_params[5] = 0.327015517176;
		stpln_params[6] = 0.945018968871;
		stpln_params[7] = 0.3460412203;
		stpln_params[8] = 1.0015773595;
		stpln_params[9] = 0.082271854223003;
		stpln_params[10] = 0.006768657997291;
		stpln_params[11] = M_PI;
		stpln_params[12] = M_E;
		stpln_params[13] = 0.999900000000;
		stpln_params[14] = 6378206.40000;
		stpln_params[15] = 101.514031441667;
		break;
	case 5002:
		strcpy(header.stplzn,"5002     ALASKA LAT>=60 LON<=144");
		stpln_params[0] = 3;
		stpln_params[1] = 500000.000;
		stpln_params[2] = 511200.00000;
		break;
	case 5003:
		strcpy(header.stplzn,"5003     ALASKA LAT>=60 LON<=148");
		stpln_params[0] = 3;
		stpln_params[1] = 500000.000;
		stpln_params[2] = 525600.00000;
		break;
	case 5004:
		strcpy(header.stplzn,"5004     ALASKA LAT>=60 LON<=152");
		stpln_params[0] = 3;
		stpln_params[1] = 500000.000;
		stpln_params[2] = 540000.00000;
		break;
	case 5005:
		strcpy(header.stplzn,"5005     ALASKA LAT>=60 LON<=156");
		stpln_params[0] = 3;
		stpln_params[1] = 500000.000;
		stpln_params[2] = 554400.00000;
		break;
	case 5006:
		strcpy(header.stplzn,"5006     ALASKA LAT>=60 LON<=160");
		stpln_params[0] = 3;
		stpln_params[1] = 500000.000;
		stpln_params[2] = 568800.00000;
		break;
	case 5007:
		strcpy(header.stplzn,"5007     ALASKA LAT>=60 LON<=164");
		stpln_params[0] = 3;
		stpln_params[1] = 700000.000;
		stpln_params[2] = 583200.00000;
		break;
	case 5008:
		strcpy(header.stplzn,"5008     ALASKA LAT>=60 LON<=168");
		stpln_params[0] = 3;
		stpln_params[1] = 500000.000;
		stpln_params[2] = 597600.00000;
		break;
	case 5009:
		strcpy(header.stplzn,"5009     ALASKA LAT>=60 LON>168");
		stpln_params[0] = 3;
		stpln_params[1] = 600000.000;
		stpln_params[2] = 612000.00000;
		break;
	case 5010:
		strcpy(header.stplzn,"5010     ALASKA PENINSULA (LAT<60)");
		stpln_params[0] = 1;
		stpln_params[1] = 3000000.00;
		stpln_params[2] = 633600.00;
		stpln_params[3] = 15893950.36;
		stpln_params[4] = 16564628.77;
		stpln_params[5] = .9998480641;
		stpln_params[6] = .7969223940;
		stpln_params[7] = 3161;
		stpln_params[8] = 47.87068;
		stpln_params[9] = 3.79919;
		stpln_params[10] = 5.91550;
		stpln_params[11] = 44;
		break;
	case 5101:
		strcpy(header.stplzn,"5101     HAWAII ZONE 1");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 559800.00;
		stpln_params[3] = 1124;
		stpln_params[4] = 39.52714;
		stpln_params[5] = .9999666667;
		stpln_params[6] = .3826496;
		break;
	case 5102:
		strcpy(header.stplzn,"5102     HAWAII ZONE 2");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 564000.00;
		stpln_params[3] = 1214;
		stpln_params[4] = 18.21554;
		stpln_params[5] = .9999666667;
		stpln_params[6] = .3825762;
		break;
	case 5103:
		strcpy(header.stplzn,"5103     HAWAII ZONE 3");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 568800.00;
		stpln_params[3] = 1264;
		stpln_params[4] = 6.77497;
		stpln_params[5] = .9999900000;
		stpln_params[6] = .3825176;
		break;
	case 5104:
		strcpy(header.stplzn,"5104     HAWAII ZONE 4");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 574200.00;
		stpln_params[3] = 1303;
		stpln_params[4] = 57.83623;
		stpln_params[5] = .9999900000;
		stpln_params[6] = .3824812;
		break;
	case 5105:
		strcpy(header.stplzn,"5105     HAWAII ZONE 5");
		stpln_params[0] = 2;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 576600.00;
		stpln_params[3] = 1294;
		stpln_params[4] = 0.05280;
		stpln_params[5] = .9999999999;
		stpln_params[6] = .3824867;
		break;
	case 5201:
		strcpy(header.stplzn,"5201     PUERTO RICO AND VIRGIN ISLANDS");
		stpln_params[0] = 1;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 239160.0000;
		stpln_params[3] = 63542221.66;
		stpln_params[4] = 63687479.44;
		stpln_params[5] = .9999939449;
		stpln_params[6] = .3128882281;
		stpln_params[7] = 1088;
		stpln_params[8] = 48.44933;
		stpln_params[9] = 3.82699;
		stpln_params[10] = 1.51030;
		stpln_params[11] = 0;
		break;
	case 5202:
		strcpy(header.stplzn,"5202     ST. CROIX");
		stpln_params[0] = 1;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 239160.0000;
		stpln_params[3] = 63542221.66;
		stpln_params[4] = 63787479.44;
		stpln_params[5] = .9999939449;
		stpln_params[6] = .3128882281;
		stpln_params[7] = 1088;
		stpln_params[8] = 48.44933;
		stpln_params[9] = 3.82699;
		stpln_params[10] = 1.51030;
		stpln_params[11] = 0;
		break;
	case 5300:
		strcpy(header.stplzn,"5300     SAMOA");
		stpln_params[0] = 1;
		stpln_params[1] = 500000.00;
		stpln_params[2] = 612000.0000;
		stpln_params[3] = -82312234.65;
		stpln_params[4] = -82000000.00;
		stpln_params[5] = .9999999999;
		stpln_params[6] = -.2464352205;
		stpln_params[7] = -851;
		stpln_params[8] = -49.53291;
		stpln_params[9] = 3.82892;
		stpln_params[10] = -1.16664;
		stpln_params[11] = 0;
		break;
	case 9999:
		strcpy(header.stplzn,"5400     GUAM");
		stpln_params[0] = 5;
		stpln_params[1] = 50000.0000;
		stpln_params[2] = 50000.0000;
		break;
	default:
		return -1;
		break;
	}

	return 0;
}
