/*******************************************************************************
*       table_maker     (see manual, Internal Functions)                <in>   *
*                       This program builds the global array tlocale[256].     *
*                                                                              *
*                       This table is the heart of line trace, as it           *
*                       contains all the information about each                *
*                       neiborhood/locale/configuration of pixels surrounding  *
*                       a given raster location.  It is used extensively       *
*                       by the thin, vectorize, assemble, and other programs.  *
*                                                                              *
*       JPD 3/15/88                                                            *
*******************************************************************************/


#include "../../incl/globals.h"


long table_maker()
{

/*******************************************************************************
*   This program is adapted from the TABLE_MAKER program used to               *
*   write the code for LIDES which loads values into the lookup                *
*   table which is the heart of LIDES.  A similar table is the                 *
*   heart of Line Trace Plus.                                                  *
*                                                                              *
*   This program loads the global array tlocale[256] when line trace is        *
*   started. The lookup table is used for thinning, vectorizing, editing and   *
*   assembling polygons from raster-pixel raw data.                            *
*                                                                              *
*   When considering a given pixel, it is useful to look at the adjacent       *
*   pixels.  For this program and the data it generates, these 8 pixels        *
*   are numbered as follows (around the X location):                           *
*               7 0 1                                                          *
*               6 X 2                                                          *
*               5 4 3                                                          *
*                                                                              *
*   1.  To use the look-up table, you would first calcualte a variable         *
*   which I shall call locale.  locale is the (long) integer that you get      *
*   when you use the pixel numbers of the 'on' pixels as bits.                 *
*   For example:                                                               *
*                   0 1 0                                                      *
*   the pixels:     0 X 1   would generate a local = 5                         *
*                   0 0 0   (because pixels/bits 0 and 2 are 'on')             *
*                                                                              *
*   Note that the variable locale will be between 0 and 255 inclusive.         *
*                                                                              *
*   2.  Now, you can access a lot of information about the pixel by using      *
*   the variable locale to access the lookup table (long tlocale[256])         *
*   as follows:	tlocale[locale]                                                *
*                                                                              *
*   The number returned is coded as follows:                                   *
*                                                                              *
*        bits   meaning                                                        *
*         0-7  The 'adjusted' Locale variable.  The adjustment is made by      *
*              turning off all corner bits which have an 'on' side bit         *
*              beside them.  This represents the directions in which a         *
*              pixel arrangement should be traced.                             *
*          8   bit flag, 1 indicates isolated point (0 branch node)            *
*          9   bit flag, 1 indicates a spur end     (1 branch node)            *
*         10   bit flag, 1 indicates a segment   (or 2 branch node)            *
*         11   bit flag, 1 indicates a 3 branch node                           *
*         12   bit flag, 1 indicates a 4 branch node                           *
*         13   bit flag, 1 indicates that a node (any kind) at this pixel      *
*         14   bit flag, 1 indicates that thinning is complete at this pixel   *
*         15   bit flag, 1 indicates remove pixel for type 0 thinning          *
*              (nondestructive, leave spur ends, remove square corners and     *
*              stairsteps.)                                                    *
*         16   bit flag, 1 indicates remove pixel for type 1 thinning          *
*              (nondestructive, leave spur ends, leave square corners and      *
*              stairsteps.)                                                    *
*         17   bit flag, 1 indicates remove pixel for type 2 thinning          *
*              (DESTRUCTIVE, remove all pixels which are not part of a solid   *
*              interior.)  This flag indicates all pixels which are not        *
*              part of a solid interior (with a 1), and conversely pixels      *
*              which are part of a solid interior (with a 0).                  *
*         18   bit flag, 1 indicates that this pixel configuration in the      *
*              unthinned (old) raster requires an additional check (due to     *
*              directionality of thinning).  Check the 'new' (thinned above)   *
*              pixel locale against the value below, and thin if it matches.   *
*        19-26 Test 'new' locale to be used in conjunction with 18 above.  If  *
*              18 is flagged and there is a match between this and the 'new'   *
*              locale, then the item should be type 0 or type 1 thinned.       *
*              Not used on type 2 (destructive) thinning.                      *
*       27-28  unused                                                          *
*       29-31  a 3 bit integer (unsigned) indicating the number of branches    *
*              from the center pixel location.  For Line Trace use this will   *
*              always be a number from 0 to 4 since lines are not traced to    *
*              corners if the side pixel is on.                                *
*                                                                              *
*   Note:  The adjusted locale variable (bits 7-0 of the lookup table) is      *
*   adjusted by turning off bits which represent corner pixels if an           *
*   adjacent side pixel is on.  For example:                                   *
*                                                                              *
*   1 1 1               0 1 0                                                  *
*   1 X 1  adjusts to   1 X 1  (locale=255  becomes node_type=85)              *
*   1 1 1               0 1 0                                                  *
*                                                                              *
*   1 1 1               0 1 0                                                  *
*   0 X 0  adjusts to   0 X 0  (locale=171  becomes node_type=41)              *
*   1 0 1               1 0 1                                                  *
*                                                                              *
*   (The branches are counted after adjustment.  The thinning information      *
*   is calculated on the unadjusted locale.)                                   *
*                                                                              *
*   The logic is this:                                                         *
*     (for each locale, 0-255)                                                 *
*    1.  Generate all reflections and rotations relative to the pixel          *
*        positions (0-8 unique are possible).                                  *
*    -do the following for only the least locale value of all rotations and    *
*     reflections, then copy thier bit flag data into all the others.          *
*    2.  Start the lookup table with the adjusted locale number.               *
*    3.  Calculate the information held in bits 8-31 of the table:             *
*       a.  Count branches & load in bits 10-8.                                *
*       b.  calculate the number of 'breaks' & set flags, bits 14-17.          *
*       c.  calculate & set the 'additional check' variables, bits 18-26.      *
*    4.  Load the information into the appropriate slots in tlocale[].         *
*    5.  Add directional flags & variables and load accordingly.               *
*    - do the following once:                                                  *
*    6.  write of file of 'C' code which loads the lookup table accordingly    *
*        and adds comments for each line.                                      *
*******************************************************************************/

	long raw[8];
	long adj[8];
	long i,j,k;
	long locale;
	long branches;
	long breaks;
	long on;
	long least;

		/*  for each pixel configuration about a given pixel  */
	for(locale=0;locale<256;locale++)
	{
			/*  reset all 8 rotations/reflections to 0     */
		for(i=0;i<8;i++)   
		{	raw[i]=0;
			adj[i]=0;
		}

			/*  set the first raw to the locale variable   */
		raw[0]=locale;

			/*   generate all rotations                    */
		for(i=1;i<=3;i++)
			raw[i]=(raw[i-1] >> 2) | ((raw[i-1] << 6) & 255);

			/*   generate all reflections  (about a vertical plane)   */
		for(i=4;i<8;i++)
			for(j=0;j<8;j++)
				raw[i]=(raw[i] | (((raw[i-4] >> j) & 1) << ((8-j) & 7)));

			/*  calculate adjusted locale numbers by setting corner bits 
				adjoining 'on'side bits to 0     */
		for(i=0;i<8;i++)
		{	adj[i]=raw[i];
			for(j=0;j<7;j=j+2)     /*  for each 'side' bit     */
				if((raw[i] >> j) & 1)
				{	k=255 ^ (1 << ((j+1) & 7));
					k=k ^ (1 << ((j+7) & 7));
					adj[i] = adj[i] & k;
				}
		}


			/* find if initial value is least and continue only if it is */
		for(i=1, least=1;i<8;i++)
			if(raw[0] > raw[i])     least=0;

			/* continue only if it is the least                            */
		if(least)
		{
			/*  count branches                             */
			branches=0;
			for(i=0;i<8;i++)     /* for each possible branch direction (0-7)  */
				branches = branches + (1 & (adj[0] >> i));

			/*   add branch count to adj[0] (bits 29-31)                   */
			adj[0] = adj[0] | (branches << 29);

			/*   set flag for branch type (0,1,2,3,4 branches), bits 8-12    */
			adj[0] = adj[0] | (1 << (8 + branches));  /* individual node flag */

			/*   set flag for any node (0,1,3,4 branches), bit 13            */
			switch(branches)	
			{
			case 0: case 1: case 3: case 4:   /* 0,1,3,4 branch node flag    */
				adj[0] = adj[0] | (1 << 13);
			}

			/*    calculate thinning complete & type 0 & 1  (bits 14-16)    */
			if((adj[0] & 255) != 85)
			{		/* set thinning complete flag               */
				adj[0] = adj[0] | (1 << 14);
					/*  count the number of breaks around the raw locale    */
				on = (raw[0] >> 7) & 1;     /* initial value = max bit 7,6   */
				on = on | (raw[0] >> 6) & 1;   

					/*  count the breaks around the central pixel       */
				breaks = 0;
				for(i=0;i<8;i++)     /* go entirely around pixel         */ 
					if((((raw[0] >> i) & 1) !=on) && ((on==0) || ((i & 1) ==0)))
					{	breaks = breaks + 1;
						on = (on+1) & 1;
				 	}

					/* set types 0 and 1 thinning, generally thin if breaks<3 */
				if(breaks < 3)
				{	switch(raw[0])
					{
					case 0:                               /* isolated pixel   */
					case 1: case 2: case 3:               /* spur end         */
					case 21: case 53: case 91:            /* T type nodes     */
						break;	                          /* do not thin      */
					case 5:                               /* square corners   */
						adj[0] = adj[0] | (1 << 15);      /* thin type 0      */
						break;
					default:
						adj[0] = adj[0] | (1 << 15);      /* thin type 0      */
						adj[0] = adj[0] | (1 << 16);      /* thin type 1      */
						break;
					}
				}
			}
				/*  calculate destructive thinning flag (not solid interior)  */
			if((raw[0]!=85) && ((adj[0] & 255) == 85));  /* solid interior */
			else    adj[0] = adj[0] | (1<<17);       /* destructive thin flag*/


				/*  add bit flags etc. to all rotations, reflections      */ 
			for(i=1;i<8;i++)
				adj[i] = adj[i] | ((adj[0] >> 8) << 8);

				/*  store these in the lookup table                        */
			for(i=0;i<8;i++)
				tlocale[raw[i]] = adj[i];

		}	/*  end least business					*/
	}

		/*   add flags for directional thinning check                      */
		/*  (assumes top left beginning, thin row by row to lower right.   */

	/*      old=7           new=6   lower left corner, thin it!            */
    /*		0 1 1           0 0 1                                          */
	/*		0 X 1           0 X 1  <--- this one is the check integer      */
	/*		0 0 0           0 0 0                                          */
	tlocale[7] = tlocale[7] | (1 << 18);   /*   set directional check flag */
	tlocale[7] = tlocale[7] | (6 << 19);   /*   add check integer          */

	/*      old=14          new=12   one pixel bump on left, thin it!      */
    /*		0 0 1           0 0 0                                          */
	/*		0 X 1           0 X 1  <--- this one is the check integer      */
	/*		0 0 1           0 0 1                                          */
	tlocale[14] = tlocale[14] | (1 << 18);   /*   set directional check flag */
	tlocale[14] = tlocale[14] | (12 << 19);   /*   add check integer       */

	/*      old=112         new=48   top right corner, thin it!            */
    /*		0 0 0           0 0 0                                          */
	/*		1 X 0           0 X 0  <--- this one is the check integer      */
	/*		1 1 0           1 1 0                                          */
	tlocale[112] = tlocale[112] | (1 << 18);  /*  set directional check flag */
	tlocale[112] = tlocale[112] | (48 << 19);   /*   add check integer       */

	/*      old=131         new=1   bump on bottom, thin it!              */
    /*		1 1 1           0 1 0                                          */
	/*		0 X 0           0 X 0  <--- this one is the check integer      */
	/*		0 0 0           0 0 0                                          */
	tlocale[131] = tlocale[131] | (1 << 18);  /*  set directional check flag */
	tlocale[131] = tlocale[131] | (1 << 19);   /*   add check integer      */

	/*      old=193         new=128  bottom right corner, thin it!         */
    /*		1 1 0           1 0 0                                          */
	/*		1 X 0           0 X 0  <--- this one is the check integer      */
	/*		0 0 0           0 0 0                                          */
	tlocale[193] = tlocale[193] | (1 << 18);  /*  set directional check flag */
	tlocale[193] = tlocale[193] | (128 << 19);   /*   add check integer      */

	/*      old=195         new=129  next to btm right corner, thin it!    */
    /*		1 1 1           1 1 0                                          */
	/*		1 X 0           0 X 0  <--- this one is the check integer      */
	/*		0 0 0           0 0 0                                          */
	tlocale[195] = tlocale[195] | (1 << 18);  /*  set directional check flag */
	tlocale[195] = tlocale[195] | (129 << 19);   /*   add check integer      */

	/*      old=224         new=96  bump on rt side, thin it!              */
    /*		1 0 0           0 0 0                                          */
	/*		1 X 0           1 X 0  <--- this one is the check integer      */
	/*		1 0 0           1 0 0                                          */
	tlocale[224] = tlocale[224] | (1 << 18);  /*  set directional check flag */
	tlocale[224] = tlocale[224] | (96 << 19);   /*   add check integer      */


	return(0); 
}
