/*******************************************************************************
*      update_atrbt.c                                                          *
*          This program updates the newly assembled map with the old           *
*      attributes and serial numbers.  Each new object is given the attributes *
*      of the 'old' object of the same type (point, line, polygon) that lies   *
*      'under' it.  Many new objects may be given the attributes from the      *
*      same 'old' obect.  For example if an old polygon were divided up into   *
*      many 'new' polygons, all of the new ones would have the same            *
*      attributes as the old one.  Serial numbers are given to the largest     *
*      'new' object that lies over the 'old' object.  (biggest polygon,        *
*      longest line, closest node)                                             *
*                                                                              *
*      parameters:  long old_objects, old_coordlst;  the number of records     *
*                        in the old_objects and old_coordlst files.            *
*                                                                              *
*      Globals changed:  none                                                  *
*                                                                              *
*      Returned values:  long 0 = update done                                  *
*                             1 = update failed                                *
*                                                                              *
*      J.Dabritz   8/5/89                                                      *
*******************************************************************************/

#include <string.h>
#include "../../incl/globals.h"
#include <math.h>

int nobjects_fd;          /* passes file pointer to get & put */


long update_atrbt(old_objects,old_coordlst)
	long old_objects, old_coordlst;
{
	OBJECTS newobject, oldobject, bobject;

	long no_objects;       /** number of records in the objects file    */
	long no_coord_list;    /** number of records in the coord_list file */
	long size;	           /** size in bytes to malloc                  */
	long n,o,p,s;          /** record numbers for nodes, objects, etc   */
	long baby;             /** from which to take candy                 */
	long i;

	char string1[256];
	char path[256], string[256];

	float node_r;          /** search radius for nodes                  */
	float seg_r;           /** search radius for segments               */

	double dx,dy;
	double measure;        /** for compairing who gets the old serial nos. */

	char *malloc_ltp();
	long get_new();
	long put_new();


	message("Updating attributes & serial numbers, updating disc files . . . ");
		/************* update current data to temp path *****************/
	if(update_temp())
	{	message("abort, update_atrbt, location 1");
		return(1);
	}
	append_msg("done.");

	message("Update attributes, cont.  setting up files for update . . . ");
		/************* close & free objects & coord_list files **********/
	if(objects_fd > 0)
	{	if(closef(objects_fd))
		{	message("Cannot close objects_fd.  Aborting.");
			return(1);
		}
		objects_fd = -1;
	}

	if(coord_list_fd > 0)
	{	if(closef(coord_list_fd))
		{	message("Cannot close coord_list_fd.  Aborting.");
			return(1);
		}
		coord_list_fd = -1;
	}
 
	if(objectsm.malloc)
	{	free_ltp(objectsm.malloc);
		objectsm.malloc = (char *) 0;            
	}
 
	if(coord_listm.malloc)
	{	free_ltp(coord_listm.malloc);
		coord_listm.malloc = (char *) 0;           
	}

		/************* move object & coord_list files for safe keeping **/
		/**	                 (save # of records in each)               **/
	no_objects = job_stats[10];
	no_coord_list = job_stats[11];
		/** move objects file **/
	sprintf(string,"%s/nobjects",temp_map_path);
	purgef(string);

	sprintf(string,"%s/objects",temp_map_path);
	sprintf(string1,"%s/nobjects",temp_map_path);
	if(move_file(string, string1))
	{	message("abort, update_atrbt, location 3");
		return(1);
	}

		/** move coord_list file **/
	sprintf(string,"%s/ncoord_list",temp_map_path);
	purgef(string);

	sprintf(string,"%s/coord_list",temp_map_path);
	sprintf(string1,"%s/ncoord_list",temp_map_path);
	if(move_file(string, string1))
	{	message("abort, update_atrbt, location 5");
		return(1);
	}

		/************* open objects file ********************************/
	sprintf(path,"%s/nobjects",temp_map_path);
	nobjects_fd = openf(path,RDWR);
	if(0 > nobjects_fd)
	{	message("  <<  cannot open objects map file.");
		return(1);
	}
	
		/** check for user abort **/
	if(abort_cmd())
		return(1);

		/************* move old_objects & old_coordlst into new space ***/
		/** move objects file **/
	sprintf(string,"%s/old_objects",temp_map_path);
	sprintf(string1,"%s/objects",temp_map_path);
	if(move_file(string, string1))
	{	message("abort, update_atrbt, location 6");
		return(1);
	}

	
		/** check for user abort **/
	if(abort_cmd())
		return(1);

		/** move coord_list file **/
	sprintf(string,"%s/old_coordlst",temp_map_path);
	sprintf(string1,"%s/coord_list",temp_map_path);
	if(move_file(string, string1))
	{	message("abort, update_atrbt, location 7");
		return(1);
	}

		/************* open & malloc space for files ********************/
	job_stats[10] = old_objects;
	job_stats[11] = old_coordlst;
		/********* objects ***********/
		/*    open file        */
	sprintf(path,"%s/objects",temp_map_path);
	objects_fd = openf(path,RDWR);
	if(0 > objects_fd)
	{	message("  <<  cannot open objects map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = objectsm.rec_size * job_stats[10];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(objectsm.malloc = (malloc_ltp(size)))
			{   objectsm.rec0 = objectsm.malloc;
				objectsm.recn = objectsm.rec0 + size;
				if(!readb(objectsm.rec0,size,1,objects_fd))
				{	message("  <<  cannot read objects map file.");
					return(1);
				}
			}
		}
	}
	
		/** check for user abort **/
	if(abort_cmd())
		return(1);

		/********* coord_list ***********/
		/*    open file        */
	sprintf(path,"%s/coord_list",temp_map_path);
	coord_list_fd = openf(path,RDWR);
	if(0 > coord_list_fd)
	{	message("  <<  cannot open coord_list map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = coord_listm.rec_size * job_stats[11];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(coord_listm.malloc = (malloc_ltp(size)))
			{   coord_listm.rec0 = coord_listm.malloc;
				coord_listm.recn = coord_listm.rec0 + size;
				if(!readb(coord_listm.rec0,size,1,coord_list_fd))
				{	message("  <<  cannot read coord_list map file.");
					return(1);
				}
			}
		}
	}
	
		/** check for user abort **/
	if(abort_cmd())
		return(1);

		/** for all objects, set object.f4[2] = 0.0
			and object.i12[0] = -1 as flags for the update routine below */
	for(i=0; i<job_stats[10]; i++)
	{	if(get_objects(&oldobject,i))
		{	message("abort, update_atrbt, location 8");
			return(1);
		}
		oldobject.i12[0] = -1;
		oldobject.f4[2] = 0.0;
		if(put_objects(&oldobject,i))
		{	message("abort, update_atrbt, location 9");
			return(1);
		}
	}
	append_msg("done.");

		/************* update serials & attributes for all objects ******/
	node_r = (float) 1.5 + job_stats[77] / 100.0;
	seg_r = (float) 1.0 + job_stats[77] / 100.0;
	for(o = 0; o<no_objects ; o++)
	{		/** get new object ***/
		if(get_new(&newobject,o))
		{	message("abort, update_atrbt, location 10");
			return(1);
		}

			/*** inform user *******/
		if(!(o % 50))
		{	sprintf(string,"Updating attributes and serial numbers, at object %d",o);
			message(string);
		}
	
			/** check for user abort **/
		if(abort_cmd())
			return(1);


			/** find corrisponding old object **/
			/*---------------------- node -----------------------------*/
		if(newobject.i12[0] < 8)        /** node **/
		{
			if(job_stats[41])     /* map previously assembled */
				n = find_vnode(newobject.f4[0],newobject.f4[1],node_r,0);
			else
				n = -1;

			if(n >= 0)
			{	if(get_objects(&oldobject,n))
				{	message("abort, update_atrbt, location 11");
					return(1);
				}
					/** give new object the attributes of the old **/
				newobject.i12[2] = oldobject.i12[2];
					/** give new object serial, if it's the closest **/
				dx = oldobject.f4[0] - newobject.f4[0];
				dy = oldobject.f4[1] - newobject.f4[1];
				measure = node_r * node_r - (dx*dx + dy*dy);
				if(measure > oldobject.f4[2])
				{		/** take serial away from previous new owner (if any) */
					if(oldobject.i12[0] >= 0)
					{	baby = oldobject.i12[0];
						if(get_new(&bobject,baby))
						{	message("abort, update_atrbt, location 12");
							return(1);
						}
						bobject.i12[1] = job_stats[74]++;
						if(put_new(&bobject,baby))
						{	message("abort, update_atrbt, location 13");
							return(1);
						}
					}
						/** give it to new owner  **/
					newobject.i12[1] = oldobject.i12[1];
						/** leave trail to new owner in old object **/
					oldobject.i12[0] = o;
					oldobject.f4[2] = measure;
						/** store so that trail will remain **/
					if(put_objects(&oldobject,n))
					{	message("abort, update_atrbt, location 14");
						return(1);
					}
				}
				else     /** assign a new serial number **/
					newobject.i12[1] = job_stats[74]++;
			}
			else     /** assign a new serial number **/
				newobject.i12[1] = job_stats[74]++;
		}

			/*---------------------- segment --------------------------*/
		else if(newobject.i12[0] < 14)  /** segment **/
		{
			if(job_stats[41])     /* map previously assembled */
				s = find_vseg(newobject.f4[0],newobject.f4[1],seg_r,0);
			else
				s = -1;

			if(s >= 0)
			{	if(get_objects(&oldobject,s))
				{	message("abort, update_atrbt, location 15");
					return(1);
				}
					/** give new object the attributes of the old **/
				newobject.i12[2] = oldobject.i12[2];
					/** give new object serial, if it's the longest **/
				measure = newobject.f4[2];
				if(measure > oldobject.f4[2])
				{		/** take serial away from previous new owner (if any) */
					if(oldobject.i12[0] >= 0)
					{	baby = oldobject.i12[0];
						if(get_new(&bobject,baby))
						{	message("abort, update_atrbt, location 16");
							return(1);
						}
						bobject.i12[1] = job_stats[74]++;
						if(put_new(&bobject,baby))
						{	message("abort, update_atrbt, location 17");
							return(1);
						}
					}
						/** give it to new owner  **/
					newobject.i12[1] = oldobject.i12[1];
						/** leave trail to new owner in old object **/
					oldobject.i12[0] = o;
					oldobject.f4[2] = measure;
						/** store so that trail will remain **/
					if(put_objects(&oldobject,s))
					{	message("abort, update_atrbt, location 18");
						return(1);
					}
				}
				else     /** assign a new serial number **/
					newobject.i12[1] = job_stats[74]++;
			}
			else     /** assign a new serial number **/
				newobject.i12[1] = job_stats[74]++;
		}

			/*---------------------- polygon --------------------------*/
		else                            /** polygon **/
		{
			if(job_stats[41])     /* map previously assembled */
				p = find_vpoly(newobject.f4[0],newobject.f4[1],0);
			else
				p = -1;

/*			if(p >= 0)    */
			if(p > 0)
			{	if(get_objects(&oldobject,p))
				{	message("abort, update_atrbt, location 19");
					return(1);
				}
				if(newobject.i12[0] == 15)       /** new object is an island **/
				{	for(; oldobject.i12[8];)
					{	p = oldobject.i12[8];
						if(get_objects(&oldobject,oldobject.i12[8]))
						{	message("abort, update_atrbt, location 19.1");
							return(1);
						}
						if((newobject.f4[0] == oldobject.f4[0]) &&
						   (newobject.f4[1] == oldobject.f4[1]))
							break;
					}
					if((newobject.f4[0] != oldobject.f4[0]) ||
					   (newobject.f4[1] != oldobject.f4[1]) ||
					   (newobject.i12[0] != 15) )
						p = -1;
				}
			}
			if(p >= 0)
			{		/** give new object the attributes of the old **/
				newobject.i12[2] = oldobject.i12[2];
					/** give new object serial, if it's the largest **/
				measure = newobject.f4[3];
				if(newobject.i12[0] == 16)    /** map border polygon **/
					measure = 0.1;           /** has precedence over islands **/
				if(measure > oldobject.f4[2])
				{		/** take serial away from previous new owner (if any) */
					if(oldobject.i12[0] >= 0)
					{	baby = oldobject.i12[0];
						if(get_new(&bobject,baby))
						{	message("abort, update_atrbt, location 20");
							return(1);
						}
						bobject.i12[1] = job_stats[74]++;
						if(put_new(&bobject,baby))
						{	message("abort, update_atrbt, location 21");
							return(1);
						}
					}
						/** give it to new owner  **/
					newobject.i12[1] = oldobject.i12[1];
						/** leave trail to new owner in old object **/
					oldobject.i12[0] = o;
					oldobject.f4[2] = measure;
						/** store so that trail will remain **/
					if(put_objects(&oldobject,p))
					{	message("abort, update_atrbt, location 22");
						return(1);
					}
				}
				else     /** assign a new serial number **/
					newobject.i12[1] = job_stats[74]++;
			}
			else     /** assign a new serial number **/
				newobject.i12[1] = job_stats[74]++;
		}

			/***** store new object ***/
		if(put_new(&newobject,o))
		{	message("abort, update_atrbt, location 23");
			return(1);
		}

	}
	append_msg("done.");

	message("Update attributes, cont.  resetting files & packing . . . ");
		/************* free, close, & purge (old) objects & coord_list **/
		/** free malloc space **/
	if(objectsm.malloc)
	{	free_ltp(objectsm.malloc);
		objectsm.malloc = (char *) 0;            
	}
	if(coord_listm.malloc)
	{	free_ltp(coord_listm.malloc);
		coord_listm.malloc = (char *) 0;           
	}
 		/** close files **/
	if(objects_fd >= 0)
	{	if(closef(objects_fd))
		{	message("Cannot close objects_fd.  Aborting.");
			return(1);
		}
		objects_fd = -1;
	}
	if(coord_list_fd >= 0)
	{	if(closef(coord_list_fd))
		{	message("Cannot close coord_list_fd.  Aborting.");
			return(1);
		}
		coord_list_fd = -1;
	}
		/*** purge files ***/
	sprintf(string,"%s/objects",temp_map_path);
	purgef(string);

	sprintf(string,"%s/coord_list",temp_map_path);
	purgef(string);

		/************* close real objects file **************************/
	if(closef(nobjects_fd))
	{	message("Cannot close coord_list_fd.  Aborting.");
		return(1);
	}
	nobjects_fd = -1;

		/** check for user abort **/
	if(abort_cmd())
		return(1);
	
		/************* move real objects & coord_list files back ********/
		/**	     reset job_stats record numbers                        **/
	job_stats[10] = no_objects;
	job_stats[11] = no_coord_list;
		/** move objects file **/
	sprintf(string,"%s/nobjects",temp_map_path);
	sprintf(string1,"%s/objects",temp_map_path);
	if(move_file(string, string1))
	{	message("abort, update_atrbt, location 26");
		return(1);
	}


		/** check for user abort **/
	if(abort_cmd())
		return(1);
	
		/** move coord_list file **/
	sprintf(string,"%s/ncoord_list",temp_map_path);
	sprintf(string1,"%s/coord_list",temp_map_path);
	if(move_file(string, string1))
	{	message("abort, update_atrbt, location 27");
		return(1);
	}

		/** check for user abort **/
	if(abort_cmd())
		return(1);
	

		/************* open & malloc objects & coord_list files *********/
		/********* objects ***********/
		/*    open file        */
	sprintf(path,"%s/objects",temp_map_path);
	for(i=0; (i<20) && 0>(objects_fd = openf(path,RDWR)) ; sleep(1), i++);
	if(0 > objects_fd)
	{	message("  <<  cannot open objects map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = objectsm.rec_size * job_stats[10];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(objectsm.malloc = (malloc_ltp(size)))
			{   objectsm.rec0 = objectsm.malloc;
				objectsm.recn = objectsm.rec0 + size;
				if(!readb(objectsm.rec0,size,1,objects_fd))
				{	message("  <<  cannot read objects map file.");
					return(1);
				}
			}
		}
	}

		/** check for user abort **/
	if(abort_cmd())
		return(1);
	
		/********* coord_list ***********/
		/*    open file        */
	sprintf(path,"%s/coord_list",temp_map_path);
	for(i=0; (i<20) && 0>(coord_list_fd = openf(path,RDWR));sleep(1), i++);
	if(0 > coord_list_fd)
	{	message("  <<  cannot open coord_list map file.");
		return(1);
	}
		/*  calculate size to malloc */
	size = coord_listm.rec_size * job_stats[11];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(coord_listm.malloc = (malloc_ltp(size)))
			{   coord_listm.rec0 = coord_listm.malloc;
				coord_listm.recn = coord_listm.rec0 + size;
				if(!readb(coord_listm.rec0,size,1,coord_list_fd))
				{	message("  <<  cannot read coord_list map file.");
					return(1);
				}
			}
		}
	}

		/** check for user abort **/
	if(abort_cmd())
		return(1);
	

		/************* re-set the job_stats[77] variable ****************/
	job_stats[77] = 0;

		/************* pack the attributes file *************************/
	if(job_stats[13]>1)
	{	if(pack_atrbts())
			return(1);
	}
	append_msg("done.");

		/************* return *******************************************/
	return(0);
}



/*******************************************************************************
*      get_new                                                                 *
*                       This program fills the input structre with object      *
*                       information for the input object record number.        *
*                                                                              *
*                       no global or file data is changed.                     *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  8/5/89                                       *
*******************************************************************************/


long get_new(ptr,record_number)
OBJECTS *ptr;                /*  pointer to object structure to be got    */
long record_number;          /*  record number for get.                   */
{
	long offset;

		/******************** calculate offset **************************/
	offset = record_number * objectsm.rec_size;

		/******************** position file write pointer ***************/
	if(0 > movefp(nobjects_fd,offset,0))
	{	message(" cannot position read/write pointer in objects file.  aborting.");
		return(1);
	}
		/******************** read  record ******************************/
	if(0 > readb((char *)ptr, objectsm.rec_size ,1 ,nobjects_fd))
	{	message(" cannot read new objects file.  aborting.");
		return(1);
	}

		/******************** return ************************************/
	return(0);
}


/*******************************************************************************
*      put_new                                                                 *
*                       This program stores the input structure into the       *
*                       record indicated in the nobjects file pointer.         *
*                                                                              *
*                       only the file written is changed.                      *
*                                                                              *
*                       Returns zero upon normal end-of-job (EOJ).             *
*                       returns 1 upon failure.                                *
*                      j.dabritz  8/5/89                                       *
*******************************************************************************/


long put_new(ptr,record_number)
OBJECTS *ptr;                /*  pointer to object structure to be put    */
long record_number;          /*  record number for put.                   */
{
	long offset;

		/******************** calculate offset **************************/
	offset = record_number * objectsm.rec_size;

		/******************** position file write pointer ***************/
	if(0 > movefp(nobjects_fd,offset,0))
	{	message(" cannot position read/write pointer in objects file.  aborting.");
		return(1);
	}
		/******************** read  record ******************************/
	if(0 > writeb((char *)ptr, objectsm.rec_size ,1 ,nobjects_fd))
	{	message(" cannot write in new objects file.  aborting.");
		return(1);
	}

		/******************** return ************************************/
	return(0);
}
