/*******************************************************************************
*       utm_to_geo      (see manual, Internal Functions)                <in>   *
*                This program converted from HP Extended Basic.                *
*                From CT5UMI:  J. Quinones DMA/CSC Report 946.                 *
*                Original program name was utm_latlon.                         *
*                                                                              *
*                Returns 0 upon successful EOJ, else returns 1.                *
*                      C.E.Jonson 7/17/89 (EDS)                                *
*******************************************************************************/

#include "../../incl/globals.h"  /*  <stdio.h> & all LTP defines & globals  */
#include <math.h>

long utm_to_geo(sph_code, xutm, yutm, xgeo, ygeo)
	long sph_code;		/*** Set to 2 for Clarke 1866 ***/
	double *xutm, *yutm, *xgeo, *ygeo;
{
	double	xk[4], c[3], s[3], a[5], b[5], w[7];
	double	sf = 0.9996,
		fe = 500000.0,
		phio = 0.0;
	double	fn, deg, xlamo;
	long	ih, indx, izone;
	double	cc, ss, cc1, cc2, const1, aa, fio, ak, r, xlat, f, denom;
	double	ts, aj, ajd, xnpv, sin1, a12, dphi, p;
	double	t, vii, viii, d6, xix, dx, e5, e7, xp, q, q2, dlam, xlong;
	double 	an, Tan, ap, bp, cp, dp, so, pb = 0.0;

/***  check lat_ref & lon_ref ***/
	if( check_llref() )
	{	message("Lat and lon references missing.  See header.");
		return(1);
	}

/***  Check that the northing is  0 < x <= 10,000,000    ***/
	if( *yutm < 0.0 || *yutm > 10000000.0 )
	{	message("NORTHING is NEGATIVE or GREATER than TEN MILLION.");
		*xutm = *yutm = 0.0;
		return(1);
	} 
	
/***  Set every variable known to mankind.   ***/
	ih = lat_ref >= 0.0 ? 1 : 2;
	fn = (ih == 1) ? 0.0 : ((ih == 2) ? 100000000.0 : -1);

/***  Set the izone  ***/
	if(lon_ref < 0.0)          /*** take both + & - lon ref as West lon */
		izone = (186.0 + lon_ref ) / 6.0;
	else
		izone = (186.0 - lon_ref ) / 6.0;
	
/***  Set more variables  ***/
	deg = 1.0 * (6 * izone - 183);
	xlamo = deg * 0.017453292519943;

/***  xk[0] = Semi-major axis  xk[1] = Semi-minor axis  ***/
/***  xk[2] = E ^2             xk[3] = E - Prime ^ 2    ***/
	switch(sph_code)
	{
	case 0:			/***  This is Clarke 1880   ***/
		xk[0] = 6378249.145;
		xk[1] = 6356515.8696;
		xk[2] = .006803511283;
		xk[3] = .006850116125;
		break;
	case 1:			/***  This is International  ***/
		xk[0] = 6378388.0;
		xk[1] = 6356911.9461;
		xk[2] = .006722670022;
		xk[3] = .006768170197;
		break;
	case 2:			/***  This is Clarke 1866   ***/
		xk[0] = 6378206.4;
		xk[1] = 6356583.8;
		xk[2] = .006768657997;
		xk[3] = .006814784946;
		break;
	case 3:			/***  This is Bessel         ***/
		xk[0] = 6377397.155;
		xk[1] = 6356078.9628;
		xk[2] = .006674372232;
		xk[3] = .006719218799;
		break;
	case 4:			/***  This is Everest        ***/
		xk[0] = 6377276.3452;
		xk[1] = 6356075.4133;
		xk[2] = .00663784663;
		xk[3] = .006682202063;
		break;
	case 5:			/***  This is Walbeck        ***/
		xk[0] = 6376896.0;
		xk[1] = 6355834.8467;
		xk[2] = .006594548094;
		xk[3] = .006638324847;
		break;
	case 6:			/***  This is Southeast Asia ***/
		xk[0] = 6378155.0;
		xk[1] = 6356773.3205;
		xk[2] = .006693421623;
		xk[3] = .006738525415;
		break;
	case 7:			/***  This is Krasovsky      ***/
		xk[0] = 6378245.0;
		xk[1] = 6356863.0188;
		xk[2] = .006693421623;
		xk[3] = .006738525415;
		break;
	default:		/***  This ain't happy  ***/
		message("SPHEROID CODE is LESS than 0 or GREATER than 7.");
		*xgeo = *ygeo = 0.0;
		return(1);
		break;
	}

/***  Compute the spheroid constants  a, b, c, d  ***/
	an = (xk[0] - xk[1]) / (xk[0] + xk[1]);
	ap = xk[0] * (1.0 - an + 1.25 * (pow(an,2.0) - pow(an,3.0))
		 + 1.265625 * (pow(an, 4.0) - pow(an,5.0)) );
	bp = 1.5 * xk[0] * (an - pow(an, 2.0) + .875 * 
		(pow(an, 3.0) - pow(an,4.0)) + .859375 * pow(an, 5.0) );
	cp = .9375 * xk[0] * (pow(an, 2.0) - pow(an, 3.0) + .75 *
		(pow(an, 4.0) - pow(an, 5.0)) );
	dp = (35.0/48.0) * xk[0] * (pow(an, 3.0) - pow(an, 4.0) + 
		.6875 * pow(an, 5.0) );

/***  Compute length of meridional arc of origin  ***/
	so = ap * phio - pb * sin(2.0 * phio) + cp * sin(4.0 * phio) -
		dp * sin(6.0 * phio);

/***  Compute function i or origin  ***/
	fio = so * sf;

/***  Compute foot point latitude of station  ***/
	ak = 57.2957795131;
	aa = ap/ak;
	r = *yutm - fn + fio;
	xlat = r/(sf * aa);
	f = 2.0 * xlat/ak;
	c[0] = cos(f);
	c[1] = cos(2.0 * f);
	c[2] = cos(3.0 * f);
	s[0] = sin(f);
	s[1] = sin(2.0 * f);
	s[2] = sin(3.0 * f);
	w[0] = bp * s[0] - cp * s[1] + dp * s[2];
	w[1] = 2.0 * bp * c[0] - 4.0 * cp * c[1] + 6.0 * dp * c[2];
	w[2] = 2.0 * bp * s[0] - 8.0 * cp * s[1] + 18.0 * dp * s[2];
	w[3] = 4.0/3.0 * bp * c[0] - 32.0/3.0 * cp * c[1] + 36.0 * dp * c[2];
	w[4] = 2.0 /3.0 * bp * s[0] - 32.0/3.0 * cp * s[1] + 54.0 * dp * s[2];
	w[5] = 4.0/15.0 * bp * c[0] - 128.0/15.0  * cp * c[1] + 324.0/5.0 *
		dp * c[2];
	w[6] = 4.0/45.0 * bp * s[0] - 256.0/45.0 * cp * s[1] + 324.0/5.0 *
		dp * s[2];
	denom = aa * ak - w[1];
	t = w[0]/denom;
	for( indx = 0; indx < 5; indx++ )
		a[indx] = -w[indx + 2]/denom;
	a12 = a[0] * a[0];
	/**a3 = -a[2];
	a4 = -a[3]; **/
	b[0] = a[0];
	b[1] = a[1] + 2.0 * a12;
	b[2] = a[2] + 5.0 * (a[1] + a12) * a[0];
	b[3] = a[3] + a[0] * (6.0 * a[2] + a[0] * (b[1] + 8.0 * a[1]) ) +
		3.0 * b[1] * b[1];
	b[4] = a[4] + 7.0 * (a[1] * a[2] + a[0] * (b[3] - 2.0 * b[2] *
		a[0] + a12 * b[1] + pow(a[1], 2.0)) );

	dphi = t;
	for( indx = 0; indx < 5; indx++ )
		dphi += b[indx] * pow(t, (double)(indx + 2));
	p = (xlat + ak * dphi)/ak;

/***  Compute radius of curvature in the prime vertical of foot path  ***/
	xlat = p;
	xnpv = xk[0]/sqrt(1.0 - xk[2] * pow(sin(fabs(xlat)), 2.0));
	xlat = 0.0;
	aj = xnpv;
	ss = sin(p) * sin(p);
	cc = cos(p) * cos(p);
	ajd = aj * aj * sf * sf;
	
	sin1 = sin( 1.0/(3600.0 * ak) );
	cc1 = 1.0/cc;
	cc2 = cos(2.0 * p);
	Tan = ss/cc;
	ts = sin(p) * 1000000000000.0/(cos(p) * sin1 * 2.0 * ajd);
	const1 = 1000000000000.0/ (12.0 * ajd);
	
/***  Compute functs vii viii d6 ix x e5 e7 of foot point lat of station  ***/
	vii = ts * (1.0 + xk[3] * cc);
	viii = ts * const1 * (2.0 + 3.0 * (cc1 + xk[3] * (2.0 * cc2 - xk[3] *
		(cc + ss * cc * 2.0))) );
	d6 = ts * pow(const1, 2.0) * (32.0 + 90.0 * cc1 * (cc1 - xk[3] * 
		ss) + xk[3] * (214.0 * cc2 - 20.0 * ss))/5.0;
	xix = pow(10.0, 6.0)/(cos(p) * sin1 * aj * sf);
	dx = 2.0 * xix * const1 * (cc1 + Tan + xk[3] * cc);
	e5 = 1.2 * xix * const1 * const1 * (1.0 + 4.0 * cc1 * (1.0 + 6.0 * Tan) +
		2.0 * xk[3] * (3.0 + ss) );
	e7 = xix * pow(const1, 3.0) * 2.4/7.0 * cc * (61.0 + Tan *
		(662.0 + Tan * (1320.0 + 720.0 * Tan)));

/***  Compute grid distance of station from central meridian of TM system ***/
	xp = *xutm - fe;
	
/***  Compute quantity q  ***/
	q = fabs(xp/1000000.0);

/***  Compute difference of longitude of station from longitude of origin ***/
	q2 = q * q;
	dlam = q * (xix - q2 * (dx - q2 * (e5 - e7 * q2))) * .48481368111E-5;

/***  Compute geographic latitude of station  ***/
	xlat = p - q2 * (vii - q2 * (viii - d6 * q2)) * 0.48481368111E-5;

/***  Compute geographic longitude of station  ***/
	if( xp < 0 )
		xlong = xlamo - dlam;
	else
		xlong = xlamo + dlam;

/***  Test x latitude  ***/
	if( (fabs(xlat) * ak) < 80.5 )
	{	*ygeo = xlat * ak;
		*xgeo = xlong * ak;
		return(0);
	}
	else
	{	message("GEOGRAPHIC LATITUDE of STATION is ABOVE 80 degrees\
 and 30 minutes NORTH or SOUTH.");
		*xgeo = *ygeo = 0.0;
		return(1);
	}
}
