/*******************************************************************************
*       vectorize       (see manual, Internal Functions)                <in>   *
*                                                                              *
*    Vectorize all lines from the raster into the objects and coord_list       *
*    files/objects.                                                            *
*                                                                              *
*    Arguments:    none                                                        *
*                                                                              *
*    Returned Values: (long) 1 = failure                                       *
*                            0 = normal end of job.                            *
*                                                                              *
*    Global Variables Modified:  job_stats[10] & [11], node_lookup array,      *
*    the objects and coord_list files.                                         *
*                                                                              *
*                      j.dabritz  7/11/89                                      *
*******************************************************************************/
/* Modified 7/92 by DBS@CWU for X11 and big endian support */

#include "../../incl/globals.h"
void sync();
#ifdef LTP_X11
#include "../../incl/x11.h"
#endif
#ifdef BIG_ENDIAN
#include "../../incl/big_end.h"
#endif

long vectorize()
{  
  OBJECTS node, node2;          /*  object structures for nodes  */
  OBJECTS segment;              /*  object structure for segment */
  COORD_LIST coord_list;        /*  coord_list structure for coordlist rec */
  COORD_LIST coord_list2;       /*  coord_list structure for coordlist rec */

  int filefd;                   /*  file int for getting raster back  */

  long *end_ptr, *wrd_ptr;      /*  long pointers for examining raster */
  long bit, word;               /*  bit number (in word), word no. (in row) */
  long b, b2;                   /*  node branch numbers  */
  long branches;                /*  counter for number of branches */
  long i, j;                       
  long no_nodes;                /*  toatal number of nodes */
  long last_node;               /*  marker for end of node loop */
  long n, n2;                   /*  node numbers         */
  long node_dir;                /*  branch direction from node:  
                                                7 0 1
                                                6 X 2
                                                5 4 3     */
  long d, d2, last_d;           /*  vector direction (see above)  */
  long key;                     /*  tlocale element (see tlocale global) */
  long pixels;                  /*  pixel count of vector length  */
  long raster_size;             /*  size in bytes */
  long s,v;                     /*  segment & vertex numbers */
  long x,y;                     /*  raster coordinates         */
  long x_begin, y_begin;        /*  raster coordinates         */
  long dx,dy;                   /*  x & y displacement          */
  long x_increment, y_increment;/*  x & y increment values       */
  long low_yx, low_y;           /*  x & y of lowest (y) of segment (temp use) */
  long low_d;                   /*  forward direction at low point  */
  long spur_ends;               /*  counts spur ends for segment type */
  long length;                  /*  segment length in pixels */
  static long movex[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
  static long movey[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
  static long bit_pos[] = {7, 6, 5, 4, 3, 2, 1, 0, 15, 14, 13, 12, 11, 10, 
    9, 8, 23, 22, 21, 20, 19, 18, 17, 16, 31, 30, 29, 28, 27, 26, 25, 24};

  char *ptr;
  char string[256];

    /***************** check for map present *********************/
  if(!job_stats[0])
  {  message("abort, vectorize, location 1");
    return(1);
  }

    /***************** init node & segment object structures *****/
  for(j=0; j<12; j++)
    segment.i12[j] = 0;
  for(j=0; j<4; j++)
    segment.f4[j] = 0.0;

    /***************** trace every segment from every node *******/
    /*** inform user ***/
  job_stats[5] = job_stats[10];        /*  default rec of first segment */
  message("tracing lines which are not part of simple islands . . .");
  no_nodes = job_stats[10];


  sync();

  for(n=job_stats[3]; n<no_nodes; n++)
  {    /*** check for user abort **/
    if(abort_cmd())
      return(1);

       /****  get node object  */
    if(get_objects(&node,n))
    {  message("abort, vectorize, location 2");
      return(1);
    }
    if(!(n % 250))
    {  sprintf(string,"vectorizing from node %d, total objects = %d.",n,job_stats[10]);
      message(string);
#ifdef LTP_X11
      XFlush(xdisplay);
#endif
    }
      /**** trace each branch ***/
    branches = node.i12[0] - 3;
    for(b = 0, node_dir = -1; b < branches; b++)
    {    /**** get starting direction  ***/
      for(node_dir++; !(node.i12[5] & (1<<node_dir)) ; node_dir++);
        /**** check to see if it has already been traced ***/
      if(!(node.i12[b+6]))
      {    /**** store segment number in node structure ***/
        node.i12[b+6] = job_stats[10];
        if(put_objects(&node,n))
        {  message("abort, vectorize, location 3");
          return(1);
        }
          /**** store node number in segment structure ***/
        segment.i12[9] = n;
          /**** store record number of first coord_list rec in segment struct */
        segment.i12[5] = job_stats[11];
          /**** store beginning coordinates ****/
        coord_list.x = node.f4[0];
        coord_list.y = node.f4[1];
        if(put_coordlst(&coord_list,job_stats[11]))
        {  message("abort, vectorize, location 4");
          return(1);
        }
          /**** trace segment storing as we go *****/
        x = node.f4[0];
        y = node.f4[1];
        low_yx = x;
        low_y = y;
        low_d = 0;     /* flag for beginning node */
        d = node_dir;
        last_d = d;
        length = 0;
        for(;;)
        {    /**** move 1 pixel ****/
          x += movex[d];
          y += movey[d];
          length++;
            /**** check for node ***/
          key = tlocale[locale_xy(x,y)];
          if(!(key & (1<<13)))            /*  not a node  */
          {    /**** get next direction ****/
            for(d = (d+5) & 7; !(key & (1<<d)) ; d = (d+1) & 7);
              /**** store coordinates if direction has changed. ***/
            if(d != last_d)
            {  coord_list.x = x;
              coord_list.y = y;
              if(put_coordlst(&coord_list,job_stats[11]))
              {  message("abort, vectorize, location 5");
                return(1);
              }
                /**** check & reset low point ***/
              if(y < low_y)
              {  low_y = y;
                low_yx = x;
                low_d = (d < 4) ? 1 : 2;   /** (1=rt, 2=lt) ***/
              }
                /**** update last_d ***/
              last_d = d;
            }
          }
          else                            /*  hit a node  */
          {    /**** store record number of ending coordinates ****/
            segment.i12[6] = job_stats[11];
              /**** store low point (for use in assemble) ***/
            segment.i12[1] = low_yx;
            segment.i12[2] = low_y;
            segment.i12[3] = low_d;
              /**** store segment length ***/
            segment.f4[2] = (float) length;
              /**** store ending coordinates ****/
            coord_list.x = x;
            coord_list.y = y;
            if(put_coordlst(&coord_list,job_stats[11]))
            {  message("abort, vectorize, location 6");
              return(1);
            }
              /**** find node record number *****/
            n2 = find_node(x,y);
              /**** store node number in segment structure ****/
            segment.i12[10] = n2;
              /**** find branch number at ending node ****/
            if(get_objects(&node2,n2))
            {  message("abort, vectorize, location 7");
              return(1);
            }
            d = (d + 4) & 7;
            for(b2 = 0, d2 = 0;  d2 != d;  d2++)
            { if(node2.i12[5] & (1<<d2))
                b2++;
            }
              /**** store segment number (negative) at ending node ****/
            node2.i12[6+b2] = -job_stats[10];
            if(put_objects(&node2,n2))
            {  message("abort, vectorize, location 8");
              return(1);
            }
            if(n == n2)         /** if same as beginning node, update data **/
            {  if(get_objects(&node,n))
              {  message("abort, vectorize, location 9");
                return(1);
              }
            }
              /**** store segment type in strucure ***/
            spur_ends = 0;
            if(node.i12[0] == 4)       /*  a spur end  */
              spur_ends++;
            if(node2.i12[0] == 4)       /*  a spur end  */
              spur_ends++;
            segment.i12[0] = 10 - spur_ends;
              /**** store segment ***/
            if(put_objects(&segment,job_stats[10]))
            {  message("abort, vectorize, location 10");
              return(1);
            }
              /**** exit trace loop ***/
            break;
          }
        }
      }
    }
  }
  sprintf(string,"done, %d lines traced.",job_stats[10]-job_stats[5]);
  append_msg(string);
  if(job_stats[5] == job_stats[10])
    job_stats[5] = 0;       /** no non-island segments **/

  message("saving raster in temporary file . . . ");
    /***************** save raster in temp file **************************/
    /** set the path ***/
  sprintf(string,"%s/raster",temp_map_path);
    /** save the raster **/
  if(put_raster(string))
  {  message("abort, vectorize, location 11");
    return(1);
  }
  append_msg("done.");

  message("Erasing all objects traced so far . . . ");

#ifdef LTP_X11
  XFlush(xdisplay);
#endif

    /***************** erase all vectors & nodes traced so far ***********/
    /** erase all nodes ***/
  last_node = (job_stats[5]) ?  job_stats[5] : job_stats[10];
  for(n = job_stats[3]; n<last_node; n++)
  {    /*  get node */
    if(get_objects(&node,n))
    {  message("abort, vectorize, location 12");
      return(1);
    }
      /*  find pointer & bit  */
    x = node.f4[0];
    y = node.f4[1];
#ifndef BIG_ENDIAN 
    ptr = (char *) (craster[y] + (x>>3));
    bit = 7 & ~x;
      /*  turn off bit        */
    *ptr &= (char) ~(1 << bit);
#else
	lraster[y][x>>5] &= (~(1<<((7 & ~x) | (x & 24))));
#endif

  }


    /** erase all vectors **/
  for(s = job_stats[5]; job_stats[5]  &&  s<job_stats[10]; s++)
  {    /**** get segment  ****/
    if(get_objects(&segment,s))
    {  message("abort, vectorize, location 13");
      return(1);
    }
      /**** get first vertex ****/
    if(get_coordlst(&coord_list2,segment.i12[5]))
    {  message("abort, vectorize, location 14");
      return(1);
    }
      /**** for each subsequent vertex ****/
    for(v = segment.i12[5] + 1;  v <= segment.i12[6]; v++)
    {    /**** get next vertex  ****/
      if(get_coordlst(&coord_list,v))
      {  message("abort, vectorize, location 15");
        return(1);
      }
        /**** erase line between verticies ****/
      
      x = coord_list2.x;
      dx = coord_list.x - x;
      x_increment = 0;
      if(dx)
        x_increment = (dx > 0) ?  1 : -1;

      y = coord_list2.y;
      dy = coord_list.y - y;
      y_increment = 0;
      if(dy)
        y_increment = (dy > 0) ?  1 : -1;
      pixels = (dx*x_increment>dy*y_increment) ? dx*x_increment: dy*y_increment;
      
      for(i = 0; i < pixels; i++)
      {  x += x_increment;
        y += y_increment;
#ifndef BIG_ENDIAN 
	ptr = (char *) (craster[y] + (x>>3));
        bit = 7 & ~x;
          /*  turn off bit        */
        *ptr &= (char) ~(1 << bit);
#else
	lraster[y][x>>5] &= (~(1<<((7 & ~x) | (x & 24))));
#endif

      }
        /**** update last vertex ****/
      coord_list2.x = coord_list.x;
      coord_list2.y = coord_list.y;
      /*   memcpy(&coord_list2,&coord_list,coord_listm.rec_size);  */
    }
  }
  append_msg("done.");

#ifdef LTP_X11
  XFlush(xdisplay);
#endif

    /***************** init node & segment object structures *****/
  for(j=0; j<12; j++)
  { node.i12[j] = 0;
    segment.i12[j] = 0;
  }
  for(j=0; j<4; j++)
  { node.f4[j] = 0.0;
    segment.f4[j] = 0.0;
  }

  message("tracing all simple islands . . . ");
#ifdef LTP_X11
  XFlush(xdisplay);
#endif

    /***************** search for, trace, & erase islands as found  */
  end_ptr = (long *) (lraster[job_stats[2]] + raster_wincrement);
    /***** search each word  *****/
  for( wrd_ptr = (long *) lraster[0]; wrd_ptr < end_ptr; wrd_ptr++ )
  {
    if( *wrd_ptr )
    {   /****** search each bit of word, left to right **/
      for( i = 0; i < 32; i++ )
      { if(*wrd_ptr & (1 << bit_pos[i]) )        /*** found an 'on' bit **/
        {   /***  set job_stats recs to first island node/seg if necess **/
          if(!job_stats[6])
          { job_stats[4] = job_stats[10];
            job_stats[6] = job_stats[4] + 1;
          }
            /***  Find coordinates and direction.  ***/ 
          word = (wrd_ptr - (long *) lraster[0]) % raster_wincrement;
          x = (word << 5) | i;
          y = (wrd_ptr - (long *) lraster[0]) / raster_wincrement;
            /**** store node type in node structure ***/
          node.i12[0] = 5;
            /**** store node adjusted locale in node structure ****/
          key = tlocale[locale_ptr(wrd_ptr,bit_pos[i])];
          node.i12[5] = key & 255;
            /**** store segment number in node structure ***/
            /** (note we will start vectorizing on 2nd branch if first
                 branch is straight up.  this keeps the exterior poly
                 on the right always.)    */
          if(key & 1)
          { node.i12[7] = job_stats[10] + 1;
            node.i12[6] = - node.i12[7];
          }
          else
          { node.i12[6] = job_stats[10] + 1;
            node.i12[7] =  - node.i12[6];
          }
            /**** store x,y position in node structure ***/
          node.f4[0] = (float) x;
          node.f4[1] = (float) y;
            /**** store node number in segment structure ***/
          segment.i12[9] = job_stats[10];
          segment.i12[10] = job_stats[10];
            /**** store segment type in segment structure ****/
          segment.i12[0] = 10;
            /**** store node ***/
          if(put_objects(&node,job_stats[10]))
          {  message("abort, vectorize, location 16");
            return(1);
          }

            /** store record number of first coord_list rec in segment struct */
          segment.i12[5] = job_stats[11];
            /**** store beginning coordinates ****/
          coord_list.x = node.f4[0];
          coord_list.y = node.f4[1];
          if(put_coordlst(&coord_list,job_stats[11]))
          {  message("abort, vectorize, location 17");
            return(1);
          }
            /**** find beginning direction ***********/
            /*   note:  don't look in direction because then the exterior
                 polygon would be on the left     */
          for(d = 2; !(key & (1<<d)) ; d = (d+7) & 7); /*  find beginning dir */
          last_d = d;
            /**** store low point (for use in assemble) in segment structure **/
          segment.i12[1] = x;
          segment.i12[2] = y;
          segment.i12[3] = 1;   /** right **/
            /**** trace segment storing as we go *****/
          x_begin = x;
          y_begin = y;
          length = 0;
          for(;;)
          {   /*** erase pixel ***/
#ifdef BIG_ENDIAN
	    lraster[y][x>>5] &= (~(1<<((7 & ~x) | (x & 24))));
#else
            ptr = (char *) (craster[y] + (x>>3));
            bit = 7 & ~x;
              /*  turn off bit        */
            *ptr &= (char) ~(1 << bit);
#endif
              /**** move 1 pixel ****/
            x += movex[d];
            y += movey[d];
            length++;
              /**** get locale key ***/
            key = tlocale[locale_xy(x,y)];
              /**** check to see if we are done with island  ****/
            if(key & 255)                   /*  not back to begin node */
            {   /**** get next direction ****/
              for(d = (d+3) & 7; !(key & (1<<d)) ; d = (d+7) & 7);
                  /**** store coordinates if direction has changed. ***/
              if(d != last_d)
              { coord_list.x = x;
                coord_list.y = y;
                if(put_coordlst(&coord_list,job_stats[11]))
                {  message("abort, vectorize, location 18");
                  return(1);
                }
                   /**** update last_d ***/
                last_d = d;
              }
            }
            else          /* at end of island loop          */
            {   /*** erase pixel ***/
#ifdef BIG_ENDIAN
	      lraster[y][x>>5] &= (~(1<<((7 & ~x) | (x & 24))));
#else
              ptr = (char *) (craster[y] + (x>>3));
              bit = 7 & ~x;
                /*  turn off bit        */
              *ptr &= (char) ~(1 << bit);
#endif
                /**** get next direction ****/
              dx = x_begin - x;
              dy = y_begin - y;
              if(dx == 1)
              {  if(dy == 1)   d = 1;
                else if(dy)   d = 3;
                else          d = 2;
              }
              else if(dx)
              {  if(dy == 1)   d = 7;
                else if(dy)   d = 5;
                else          d = 6;
              }
              else
              {  if(dy == 1)   d = 0;
                else          d = 4;
              }
                  /**** store coordinates if direction has changed. ***/
              if(d != last_d)
              { coord_list.x = x;
                coord_list.y = y;
                if(put_coordlst(&coord_list,job_stats[11]))
                {  message("abort, vectorize, location 18");
                  return(1);
                }
                   /**** update last_d ***/
                last_d = d;
              }
              length++;

                /**** store record number of ending coordinates ****/
              segment.i12[6] = job_stats[11];
                /**** store segment length ******/
              segment.f4[2] = (float) length;
                 /**** store ending coordinates ****/
              coord_list.x = x_begin;
              coord_list.y = y_begin;
              if(put_coordlst(&coord_list,job_stats[11]))
              {  message("abort, vectorize, location 19");
                return(1);
              }
                /**** store segment ***/
              if(put_objects(&segment,job_stats[10]))
              {  message("abort, vectorize, location 20");
                return(1);
              }
                /**** exit trace loop ***/
              break;
            }
          }
        }
      }
    }
  }

  if(job_stats[6])
    sprintf(string,"done, %d islands traced.",((job_stats[10]-job_stats[4])/2));
  else
    sprintf(string,"done (none found).");
  append_msg(string);

#ifdef LTP_X11
  XFlush(xdisplay);
#endif

    /***************** get the un-erased raster back *************/
  message("reloading raster from temp file . . .");
  sprintf(string,"%s/raster",temp_map_path);
  raster_size = rasterm.recn - rasterm.rec0;
  filefd = openf(string,READ);
  if(0 > filefd)
  { message("  <<  cannot open raster map file.");
    return(1);
  }
  if(0 >= readb(rasterm.rec0,raster_size,1,filefd))
  { message("  <<  cannot read raster map file.");
    closef(filefd);
    return(1);
  }
  append_msg("done.");
  
  
    /***************** return ************************************/
  return(0);

}
