/*******************************************************************************
*       vertex_pack     (see manual, Internal Functions)                <in>   *
*                                                                              *
*        Rewrites the map (coord_list) file packing all verticies/centroids.   *
*                                                                              *
*    Arguments:    none                                                        *
*                                                                              *
*    Returned Values: (long) 0 = ok,  1 = failure.                             *
*                                                                              *
*    Global Variables Modified:  job_stats[11], coord_list_fd, coord_listm     *
*                                                                              *
*    NOTE:  This routine does NOT re-index the map vectors.  Use the           *
*    index_map() internal function to do that befor returning control to the   *
*    user.           IMPORTANT!!!!!                                            *
*                                                                              *
*                      j.dabritz  8/1/90                                       *
*******************************************************************************/

#include "../../incl/globals.h"
#include "../../incl/in.h"

long new_js11;

long write_new_cl();

long vertex_pack()
{
	OBJECTS object;

	char new_path[256];
	char old_path[256];
	char path[256];
	char string[STRLIST];

	long size;
	long i;
	long aborted;
	long begin, end;
	long write_obj;

	int fd;


		/********* check for map ****************************************/
	if(!job_stats[0])
	{	message("abort, vertex_pack, location 1");
		return(1);
	}

		/********* create new file **************************************/
		/* attempt to remove file if it exists */
	message("building new index_list file . . .");
	sprintf(new_path,"%s/coord_listb",temp_map_path);
	purgef(new_path);
		/* create new file */
	if(creatf(new_path) < 0)
	{	message("cannot create new coord_list file.  aborting process.");
		return(1);
	}
		/* open new file */
	fd = -1;
	fd = openf(new_path,RDWR);
	if(fd < 0)
	{	message("cannot open new coord_list file.  aborting process.");
		return(1);
	}

		/************** move all verticies/centroids to new file **/
	aborted = 0;
	new_js11 = 0;
	for(i=0; i<job_stats[10]; i++)
	{		/****** read object record *****************/
		if(get_objects(&object,i))
		{	aborted = 1;
			break;
		}

			/****** inform user ***********************/
		if(!(i%100))
		{	sprintf(string,"packing verticies, at object %d.",i);
			message(string);
		}

			/****** if node, do nothing ****************/
		if(object.i12[0] < 8)
			write_obj = 0;

			/****** if segment, move verticies & 2ndry centroids to new file ***/
		else if(object.i12[0] < 14) 
		{	write_obj = 1;
			begin = object.i12[5];
			end = object.i12[6];
			if(write_new_cl(begin,end,&object.i12[5],&object.i12[6],1,fd))
			{	aborted = 1;
				break;
			}
		}

			/****** if polygon or segment, move 2ndry centroids to new file. ***/
		if(object.i12[0] > 7)
		{	if(object.i12[3])
			{	write_obj = 1;
				begin = object.i12[3];
				end = object.i12[11];
				if(write_new_cl(begin,end,&object.i12[3],&object.i12[11],0,fd))
				{	aborted = 1;
					break;
				}
			}
		}

			/****** if object changed, write object record **********/
		if(write_obj)
		{	if(put_objects(&object,i))
			{	aborted = 1;
				break;
			}
		}
	}

		/************** close new file ****************************/
	closef(fd);
	if(aborted)
	{	message("vertex_pack aborted.");
		return(1);
	}

		/************** close old file & free malloc space ********/
	if(coord_listm.malloc != (char *) 0)
	{	free_ltp(coord_listm.malloc);
		coord_listm.malloc = (char *) 0;           
	}
	if(coord_list_fd >= 0)
	{	closef(coord_list_fd);
		coord_list_fd = -1;
	}

		/************** overwrite new file on old  ****************/
	sprintf(old_path,"%s/coord_list",temp_map_path);
	if(move_file(new_path,old_path))
		message("Cannot overwrite old file, coord_list file not packed.");
	job_stats[11] = new_js11;


		/************** open new file & malloc space **************/
		/*    open file        */
	sprintf(path,"%s/coord_list",temp_map_path);
	coord_list_fd = openf(path,RDWR);
	if(0 > coord_list_fd)
	{	message("  <<  cannot open coord_list map file.");
		pause_ltp(8,"  >> you should re-acquire the map befor continuing. <<");
		return(1);
	}
		/*  calculate size to malloc */
	size = coord_listm.rec_size * job_stats[11];
		/*  malloc space & set pointer or leave file (stream) pointer  */
	if(size)
	{	if(ramfree(size + RAMSPACE))
		{	if(coord_listm.malloc = (malloc_ltp(size)))
			{   coord_listm.rec0 = coord_listm.malloc;
				coord_listm.recn = coord_listm.rec0 + size;
				if(!readb(coord_listm.rec0,size,1,coord_list_fd))
				{	message("  <<  cannot read coord_list map file.");
					pause_ltp(8,"  >> you should re-acquire the map befor continuing. <<");
					return(1);
				}
			}
		}
	}

		/************** return ************************************/
	return(0);
}


/*******************************************************************************
*   write_new_cl                                                               *
*         writes new coord_list file records.                                  *
*         parameters:  long begin;         1st record to copy from old file    *
*                      long end;           last record to copy from old file   *
*                      long *bptr;         pointer to long for new begin rec.  *
*                      long *eptr;         pointer to long for new end rec.    *
*                      long seg_flag;      flag: 1=segment, write min 2 vert.  *
*                      int  fd;            file pointer                        *
*         values returned:  0=successful, 1=failed                             *
*******************************************************************************/


long write_new_cl(begin,end,bptr,eptr,seg_flag,fd)
	long begin,end;
	long *bptr, *eptr;
	long seg_flag;
	int  fd;
{
	COORD_LIST coord_list;

	long v;
	long count;

	float last_x, last_y;


		/*************** initialize **********************/
	count = 0;           /* init count of verticies writen to new file */
	*bptr = new_js11;    /* set new begin record number */

		/*************** read through list, writing new verticies ********/
	for(v=begin; v<=end; v++)
	{
			/******************** read vertex record *********************/
		if(get_coordlst(&coord_list,v))
			return(1);

			/******************** check for different vertex *************/
		if((coord_list.x != last_x) || (coord_list.y != last_y) ||
		  (v == begin) || ((v == end)  && (count < 2)  && seg_flag))
		{		/******************** reset last_x,y **********************/
			last_x = coord_list.x;
			last_y = coord_list.y;

				/******************** write record ************************/
			if(0 > writeb((char *)&coord_list, coord_listm.rec_size ,1 ,fd))
				return(1);
			count++;

				/******************** set new end record number ***********/
			*eptr = new_js11;

				/******************** increment new_js11 counter **********/
			new_js11++;
		}
	}

		/******************** return ************************************/
	return(0);
}
