/*******************************************************************************
*       vertex_smooth   (see manual, Internal Functions)                <in>   *
*                                                                              *
*        Adds verticies to smooth map vectors.                                 *
*                                                                              *
*    Arguments:    double r;     smoosmoothg radius, fractional pixels           *
*                  long s;       set number of objects to be smoothed.         *
*                                                                              *
*    Returned Values: (long) 0 = ok,  1 = failure.                             *
*                                                                              *
*    Global Variables Modified:  objects file, coord_list file, job_stats[11]  *
*                                                                              *
*    NOTE:  This routine does NOT re-calc segment lengths, polygon perimeters  *
*    or areas.      This must be done by the calling function. IMPORTANT!!!    *
*                                                                              *
*    NOTE:  This routine does NOT re-index the map vectors.  Use the           *
*    index_map() internal function to do that befor returning control to the   *
*    user.           IMPORTANT!!!!!                                            *
*                                                                              *
*                      j.dabritz  8/1/90                                       *
*******************************************************************************/

#include "../../incl/globals.h"
#include <math.h>

long vertex_smooth(r,s)
	double r;
	long s;
{
	OBJECTS object,poly;
	COORD_LIST coordlist;

	char string[256];

	long i;
	long in_set;
	long j,v,v_end;
	long rt_turn;
	long no_points;
	long count;

	double pi,twopi;
	double x0,y0;
	double x1,y1;
	double x2,y2;
	double cx,cy;
	double x12,y12;
	double len1,len2,len12,minlen;
	double r1,r2,r3;
	double dx1,dy1;
	double dx2,dy2;
	double dx_rt,dy_rt;
	double dx0c,dy0c;
	double dx2c,dy2c;
	double alpha,omega,delta,absdelta,theta;    
	double increment;
	double mat[2][3];
	double a,b;


		/************* check set number ****************************/
	if((s < 0) || (s > 31))
	{	message("Illegal set number.  Aborting.");
		return(1);
	}


		/************* check radius & units code *******************/
	if(r < (double) 0.0)
	{	message("Snap radius must be greater than zero.  Sorry Charlie.");
		return(1);
	}


		/************* for each object in the map ******************/
	pi = 3.1415927;
	twopi = 2 * pi;
	for(i=0; i<job_stats[10]; i++)
	{		/************* inform user *****************************/
		if(!(i % 100))
		{	sprintf(string,"Vertex_smooth:  checking verticies on object %d.",i);
			message(string);
		}

			/************* check for user abort ********************/
		if(abort_cmd())
			break;

			/************* see if object is in set *****************/
		if(get_objects(&object,i))
		{	message("vertex_smooth, abort, location 1.");
			return(1);
		}

		in_set = 0;                      /** default **/
		if((object.i12[0] > 7) && (object.i12[0] < 14))      /** segment **/
		{	if(object.i12[4] & (1 << s))
				in_set = 1;
			else
			{	for(j=7; j<9; j++)
				{	if(object.i12[j])
					{	if(get_objects(&poly,object.i12[j]))
						{	message("vertex_smooth, abort, location 2.");
							return(1);
						}
						if(poly.i12[4] & (1 << s))
							in_set = 1;
					}
				}
			}
		}			/** polygons checked as part of segments **/

		if(in_set)
		{
			v = object.i12[5];
			v_end = object.i12[6];
			/************* SEGMENTS: check all verticies *******/
			if(get_coordlst(&coordlist,v))
			{	message("vertex_smooth, abort, location 3.");
				return(1);
			}

				/******* store beginning vertex, set beginning rec **/
			object.i12[5] = job_stats[11];
			if(put_coordlst(&coordlist,job_stats[11]))
			{	message("vertex_smooth, abort, location 3.");
				return(1);
			}
	
				/******* set x0, beginning vertex coordinates *******/
			x0 = coordlist.x; 
			y0 = coordlist.y; 

				/******* set x1, center vertex coordinates **********/
			v++;
			if(get_coordlst(&coordlist,v))
			{	message("vertex_smooth, abort, location 3.");
				return(1);
			}
			x1 =  coordlist.x; 
			y1 =  coordlist.y; 
			x2 = x1;
			y2 = y1;

			for(v++;  v <= v_end; v++ )
			{	if(get_coordlst(&coordlist,v))
				{	message("vertex_smooth, abort, location 3.");
					return(1);
				}
				x2 = coordlist.x;
				y2 = coordlist.y;


					/******* find & normalize vectors out ********/
				dx1 = x0 - x1;
				dy1 = y0 - y1;
				len1 = sqrt(dx1*dx1 + dy1*dy1);
				if(len1 > 0.0)
				{	dx1 = dx1 / len1;
					dy1 = dy1 / len1;
				}
				dx2 = x2 - x1;
				dy2 = y2 - y1;
				len2 = sqrt(dx2*dx2 + dy2*dy2);
				if(len2 > 0.0)
				{	dx2 = dx2 / len2;
					dy2 = dy2 / len2;
				}

					/******* divide process by len of v1 + v2 ****/
				x12 = dx1 + dx2;
				y12 = dy1 + dy2;
				len12 = sqrt(x12*x12 + y12*y12);
				if((len12 < 0.5) || (len1 == 0.0) || (len2 == 0.0))
				{		/******* re-write one vertex *****************/
					coordlist.x = x1;
					coordlist.y = y1;
					if(put_coordlst(&coordlist,job_stats[11]))
					{	message("vertex_smooth, abort, location 3.");
						return(1);
					}
					x0 = x1;
					y0 = y1;
				}
				else  /******* add verticies ***********************/
				{		/******* find spread radius (r1) *************/	
						/* now take half of shortest vector length */
					if(len1 < len2)
						minlen = len1 / 3.0;
					else
						minlen = len2 / 3.0;

						/* now compare with r1 (between 1 & 0.5 times r) */
					r1 = r * (1 - len12 / 4);
					if(r1 > minlen)
						r1 = minlen;

						/******* find center of spread circle cx,xy **/
						/* first, normalize x12,y12 **/
					x12 = x12 / len12;
					y12 = y12 / len12;
						/* now, move distance r1 from pt 1. *****/
					cx = x1 + r1 * x12;
					cy = y1 + r1 * y12;

						/******* find if turn is left or right *******/
					dx_rt = -dy1;
					dy_rt = dx1;
					mat[0][0] = dx1;
					mat[0][1] = dx_rt;
					mat[0][2] = cx - x1;
					mat[1][0] = dy1;
					mat[1][1] = dy_rt;
					mat[1][2] = cy - y1;
					if(mat_solve(2,3,mat))
					{	message("vertex_smooth, abort, location 7.");
						return(1);
					}
					if(mat[1][2] > 0.0)
						rt_turn = 1;
					else
						rt_turn = 0;


						/******* find starting angle of corner *******/
					dx0c = cx - x0;
					dy0c = cy - y0;
					r2 = sqrt(dx0c*dx0c + dy0c*dy0c);
					if(r2 == 0.0)
					{	message("vertex_smooth, abort, location 7.");
						return(1);
					}
					theta = asin((double) (r1 / r2));
					r3 = r2 * cos(theta);
					mat[0][0] = r1;
					mat[0][2] = x0 - cx;
					mat[1][1] = r1;
					mat[1][2] = y0 - cy;
					if(rt_turn)
					{	mat[0][1] = -r3;
						mat[1][0] = r3;
					}
					else
					{	mat[0][1] = r3;
						mat[1][0] = -r3;
					}
					if(mat_solve(2,3,mat))
					{	message("vertex_smooth, abort, location 7.");
						return(1);
					}
					a = mat[0][2];
					b = mat[1][2];
					if(fabs(a) < 0.00000000001)
						alpha = pi / 2.0;
					else
						alpha = atan(b/a);
					if(a < 0.0)
						alpha = alpha + pi;

						/******* find ending angle of corner *******/
					dx2c = cx - x2;
					dy2c = cy - y2;
					r2 = sqrt(dx2c*dx2c + dy2c*dy2c);
					if(r2 == 0.0)
					{	message("vertex_smooth, abort, location 7.");
						return(1);
					}
					theta = asin((double) (r1 / r2));
					r3 = r2 * cos(theta);
					mat[0][0] = r1;
					mat[0][2] = x2 - cx;
					mat[1][1] = r1;
					mat[1][2] = y2 - cy;
					if(rt_turn)
					{	mat[0][1] = r3;
						mat[1][0] = -r3;
					}
					else
					{	mat[0][1] = -r3;
						mat[1][0] = r3;
					}
					if(mat_solve(2,3,mat))
					{	message("vertex_smooth, abort, location 7.");
						return(1);
					}
					a = mat[0][2];
					b = mat[1][2];
					if(fabs(a) < 0.00000000001)
						omega = pi / 2.0;
					else
						omega = atan(b/a);
					if(a < 0.0)
						omega = omega + pi;

						/******* find how many pts to put on corner **/
					if(rt_turn)
					{	for(;omega < alpha; omega += twopi );
						for(;alpha < omega; alpha += twopi );
					}
					else
					{	for(;alpha < omega; alpha += twopi);
						for(;omega < alpha; omega += twopi);
					}
					delta = omega - alpha;
					absdelta = fabs(delta);
					no_points = absdelta * 1.637;  /* (one every 35 degrees or so) */
					if(!no_points)
						no_points = 1;
					increment = delta / no_points;


						/******* put points on corner ****************/
					for(count = 0; count <= no_points; count++, alpha += increment)
					{	coordlist.x = cx + r1 * cos(alpha);
						coordlist.y = cy + r1 * sin(alpha);
						if(put_coordlst(&coordlist,job_stats[11]))
						{	message("vertex_smooth, abort, location 3.");
							return(1);
						}
					}
					x0 = coordlist.x;
					y0 = coordlist.y;
				}
				x1 = x2;
				y1 = y2;
			}
				/******* save final vertex *******************/
			coordlist.x = x2;
			coordlist.y = y2;
			object.i12[6] = job_stats[11];
			if(put_coordlst(&coordlist,job_stats[11]))
			{	message("vertex_smooth, abort, location 3.");
				return(1);
			}
			
				/******** re-save object ********************/
			if(put_objects(&object,i))
			{	message("vertex_smooth, abort, location 7.");
				return(1);
			}
		}
	}
		

		/************* return **************************************/
	return(0);
}
