/*******************************************************************************
*    ltpgo                                                                     *
*        This program starts up the ltpgo executable and then will handle      *
*        all of the system calls made by ltpgo.  This allows more memory       *
*        to be utilized, as a small process is fork'ed to perform the system   *
*        calls.  Interprocess communication is achieved through the use of     *
*        message UNIX/XENIX system calls.  IPC_PRIVATE is used to get a        *
*        unique queue.  Two message types are used.  Type 1 (TODAEMON) is used *
*        to access messages going to this program.  Type 2 (FROMDAEMON) is     *
*        used to access messages from this program.  The message queue is      *
*        removed during exit processing. The message queue ID is placed in the *
*        environment and accessed as 'LTPQID'.                                 *
*        Globals modified:   qid is modified and placed in the environment     *
*        Parameters:         array of vectors passed to ltpgo.                 *
*        Returns 0 if processing was OK, else returns -1 to the environment    *
*        C. Jonson   12/27/89                                                  *
*******************************************************************************/
#include "../../incl/define.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <signal.h>
#include <stdio.h>
#include <errno.h>

static int qid;                        /*** Queue ID ***/
static int pid;                        /*** Process ID of child ***/
static int exitval;                    /*** return value ***/

long main(argc, argv)
	int argc;
	char *argv[];
{
	extern int errno;
	long i;
	long no_hit = 0;
	long offset;
	MSG_ST  mrcv, msnd;
	char envname[32];
	char tmsg[128];
	char *ptr;
	char orig_rec[256];
	char record[256];
	char t_path[L_tmpnam];

	char *strchr();
#ifdef MULTIUSER
	static char plus_path[100];
#else
#ifdef SV32
	static char plus_path[2][32] =	{"/usr/ltp/bin/ltplus", 
									"/usr2/ltp/bin/ltplus"};
#else
	static char plus_path[2][32] =	{"/u/ltp/bin/ltplus", 
									"/u2/ltp/bin/ltplus"};
#endif
#endif /* MULTIUSER */
	FILE *fp;
	void no_child(), ltpgo_clean();
	struct stat buf;



/*** Create msg queue.  IPC_PRIVATE is used to get unique queue ***/
	if( (qid = msgget(IPC_PRIVATE, 0666)) == -1)
	{	printf("LTPEXEC: Cannot create msg queue.  errno is %d", errno);
		exitval = -1;
		no_child();
	}

/*** Load qid into environment ***/
	sprintf(envname, "%s=%d", LTPQID, qid);
	if( putenv(envname) )
	{	printf("LTPEXEC: Cannot export %s to environment", envname);
		exitval = -1;
		no_child();
	}

	signal(SIGINT, ltpgo_clean );
	signal(SIGQUIT, SIG_IGN);
	/*** signal(SIGCLD, SIG_IGN); ***/

#ifdef MULTIUSER
	
	sprintf (plus_path, "%s/%s", LTP_BIN_DIR, "ltplus");
	if( !((stat(plus_path, &buf) == -1) && (errno == ENOENT)) )
		strcpy(orig_rec, plus_path);
	else
	{
	    printf("No such command '%s'", plus_path);
	    exitval = -1;
	    no_child();
	}

#else
/*** Look for ltplus in selected directories first. If not found, find it ***/
	for( i = 0; i < 2; i++ )
	{	if( (stat(plus_path[i], &buf) == -1) && (errno == ENOENT) )
			continue;
		else
		{	strcpy(orig_rec, plus_path[i]);
			break;
		}
	}

/*** If ltplus not found above, use 'find' command to locate it ***/
	if( i == 2 )
	{	printf("\nSearching for \'ltplus\' executable code....\n");

	/*** Figure out what directory contains ltplus ***/
		if( tmpnam(t_path) == (char *)NULL )
		{	printf("Cannot generate a temp file.");
			exitval = -1;
			no_child();
		}

		sprintf(tmsg, "find / -name ltplus -print > %s", t_path);

		if( system(tmsg) == -1 )
		{	printf("Cannot perform \'find\' command.");
			exitval = -1;
			no_child();
		}

	/*** Open the file created in system call above ***/
		if( (fp = fopen(t_path, "r")) == (FILE *)NULL )
		{	printf("Cannot open %s.", t_path);
			exitval = -1;
			no_child();
		}

	/*** Read the list of files with 'ltplus' in the path.  Check the files ***/
	/*** for 'bin'. If found, execute this program                          ***/
		for(i = 0; fgets(record, sizeof(record) - 1, fp) != (char *)NULL; )
		{	record[strlen(record) - 1] = (char)NULL;
			strcpy(orig_rec, record);			/** Store original record **/
	
			for( no_hit = offset = 0;; )			/** 47 is a '/' **/
			{	if( (ptr = strchr((record + offset), 47)) == (char *)NULL )
				{	no_hit = 1;
					break;
				}
				else 
				{	ptr++;
					if( !strncmp(ptr, "bin", 3) )
						break;
					else
						offset = ptr - record;
				}
			}

			if( no_hit )
				continue;
		}
	}
#endif  /* MULTIUSER */

/*** execute the first program. pass the argv too ***/
	argv[0] = orig_rec;
/*	argv[1] = 0;  */
	if( (pid = fork() ) == 0 )	/* in child */
	{	execvp(argv[0], argv);
		printf("LTPEXEC: Cannot exec %s  errno is %d", argv[0], errno);
		exitval = -1;
		no_child();
	}
	else				/* in parent */
	{	for(;;)
		{	mrcv.mtype = TODAEMON;
			if( msgrcv(qid, &mrcv, MTEXT_SZ, TODAEMON, 0) == -1) 
			{	sprintf(tmsg, "LTPEXEC: Cannot get msg from queue. errno is %d",
						errno);
				exitval = -1;
				no_child();
			}

			/*** Check mtext for exit message ***/
			if( !strncmp(mrcv.mtext, "LTP_EXIT", 8) )
			{	exitval = 0;
				no_child();				/* exit */ 
			}
			else if( fork_execvp(mrcv.mtext) )	/* true if system fails */
				msnd.mtext[0] = 0;
			else
				msnd.mtext[0] = 1;

			msnd.mtype = FROMDAEMON;
			if( msgsnd(qid, &msnd, FROMDAEMON, 0) )
			{	sprintf(tmsg, "LTPEXEC: Cannot send msg to queue.  errno is %d",
						errno);
				exitval = -1;
				no_child();
			}
		}
	}
	ltpgo_clean();
}

/*******************************************************************************
*   ltpgo_clean                                                                *
*        Kills the child process if it hangs.                                  *
*******************************************************************************/

void ltpgo_clean()
{
	char cmd[64];

/*** Kill the 'ltplus' process ***/
	sprintf(cmd, "kill -9 %d", pid);
	
	if( system(cmd) == -1 )
		printf( "Cannot perform \'%s\'.", cmd );

	exitval = -1;
	no_child();
}

/*******************************************************************************
*   no_child                                                                   *
*        Removes the message queue and exits.                                  *
*******************************************************************************/

void no_child()
{
	msgctl(qid, IPC_RMID, 0);
	exit(exitval);
}
