/*****************************************************************************
*    change_perms                                                            *
*        This program changes the permissions on ALL files in the 'ltp'      *
*        directory.  The user is prompted to select one user's name and      *
*        group as the default for ltp files.   This is done to assure        *
*        compatability with GRASS.  The /u[sr]/ltp/config/tolerance file     *
*        is changed to root ownership with 744 permissions.                  *
*                                                                            *
*        C. Jonson   10/89                                                   *
*****************************************************************************/
#include <stdio.h>
#include <grp.h>
#include <pwd.h>
#include <ftw.h>
#include <sys/types.h>
#include <sys/stat.h>

static char  grp_user[32];
static char  grp_name[32];

int  change_perms(path, uname)
	char *path, *uname;
{
	int set_perms();
	long lognum, i;
	struct passwd *pw;
	struct group *gp, *getgrnam();
	char dflt_path[256];
	char name_list[64][32];
	char cmd[256];

/***  Have user select the login to use for file ownership ***/
	if( *uname == (char)NULL )
	{	user_list(name_list);
		printf("\nSelect one of the following user logins to be the owner\
 of the LTPlus files.\n\
If GRASS is run from this machine, you should select the login that\n\
owns grass.\n");
		for( i = 0; name_list[i][0]; i++ )
			printf("\n\t%d.  %s", i, name_list[i]);
		printf("\n\n\t\tEnter the number of a login \7(0 - %d)--> ", i - 1 );
		for( ;; )
		{	scanf("%d", &lognum);
			if( lognum >= 0 && lognum < i )
				break;
			printf("\n\7\t\tYour choice is invalid.  Chose again --> ");
		}
		strcpy(uname, name_list[lognum]);
	}

/***  Read passwd file again and get gid of selected user ***/
	pw = (struct passwd *)getpwnam(uname);
	gp = (struct group *)getgrgid(pw->pw_gid);
	strcpy(grp_name, gp->gr_name);
	strcpy(grp_user, uname);

/***  Change the perms on files that were loaded.  ***/
	printf("\nFile permissions and ownership are being set...");
	sprintf(dflt_path, "%s/ltp", path);
		/*** first change the permissions of the primary path ***/
	sprintf(cmd, "chown %s %s/*", grp_user, dflt_path);
	if( system(cmd) )
	{	printf("ltpinst:change_perms: \'chown\' failed.\n\7");
		return(-1);
	}
	sprintf(cmd, "chmod 0775 %s/*", dflt_path);
	if( system(cmd) )
	{	printf("ltpinst:change_perms: \'chmod\' failed.\n\7");
		return(-1);
	}
	sprintf(cmd, "chgrp %s %s/*", grp_name, dflt_path);
	if( system(cmd) )
	{	printf("ltpinst:change_perms: \'chgrp\' failed.\n\7");
		return(-1);
	}
	
		/*** now change permissions of all directories below it. **/
	printf("\n . . . please remain calm . . . do NOT panic . . .\n\n");
	if( ftw(dflt_path, set_perms, 7) == -1 )
	{	printf("Unable to change permissions.  The ltplus files have\
 been loaded, but it will be necessary to manually change the group and \
the permissions.  This must be done, else LTPlus will not work correctly!\n\n");
		return(0);
	}

/***  Set the /u[sr]/ltp/config/tolerance file to root ownership ***/
	sprintf(cmd, "chown root %s/ltp/config/tolerance", path);
	if( system(cmd) )
	{	printf("ltpinst:change_perms:set_perms: \'chown\' failed.\n\7");
		return(-1);
	}
	sprintf(cmd, "chmod 0644 %s/ltp/config/tolerance", path);
	if( system(cmd) )
	{	printf("ltpinst:change_perms:set_perms: \'chmod\' failed.\n\7");
		return(-1);
	}

/***** let the user know about system error messages *****/
	printf("\n\nYOU MAY IGNORE screen messages above about  :arg list too long\n");
	printf("            or   :no such file or directory\n\n");
	return(0);
}

int set_perms(fname, buf, flag)
	char *fname;
	struct stat *buf;
	int flag;
{
	static long cnt = 0;
	char cmd[256];
	
		/**** test for directory or not ****/
	if(flag != FTW_D)
		return(0);

	sprintf(cmd, "chown %s %s/*", grp_user, fname);
	system(cmd);

	sprintf(cmd, "chmod 0775 %s/*", fname);
	system(cmd);

	sprintf(cmd, "chgrp %s %s/*", grp_name, fname);
	system(cmd);

	if( !(cnt % 40) )
	{	printf(".");
		fflush(stdout);
	}

	cnt++;
	return(0);
}
