/*******************************************************************************
*   dir_list                                                                   *
*     Reads a specified directory and loads the names of the active inodes     *
*     into a passed array.  The second parameter is the base address of a      *
*     two dimensional array that is [lcount][96] in size.                      *
*     For XENIX systems, DOS directories may be searched too.                  *
*                                                                              *
*     Returns the number of items on the list if successful,                   *
*     else returns -1 in case of error.                                        *
*******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include "../../incl/in.h"  
#define ARRSZ 96 

long dir_list( dir_path, list, lcount )
	char *dir_path;
	char *list;
	long lcount;
{
	extern int errno;
	unsigned int indx = 0;
	int str_comp();                 /*  Compare function declaration  */ 
	long i;
	int fd;
	FILE *fp;
	char cmd[256];
	char t_name[L_tmpnam];
	struct direct st_dir;           /*  Directory structure  */
	void qsort();

/*** Check for a colon in the name.  If present, search that DOS directory ***/
/*** If no colon, simply process a UNIX/XENIX directory                    ***/
	if( strchr(dir_path, ':') != (char *)NULL )
	{	if( tmpnam(t_name) == (char *)NULL )
		{	printf("Cannot generate a temp file name\n");
			return(-1);
		}
			
		if( creat(t_name, 0664) == -1)
		{	printf("Cannot create %s", t_name);
			return(-1);
		}

		sprintf(cmd, "dosls %s > %s", dir_path, t_name);
		if( system(cmd) == -1 )
		{	printf("Cannot perform--> %s   UNIX errno is %d", cmd, errno);
			return(-1);
		}
		
	/*** Open the temp file ***/
		if( (fp = fopen(t_name, "r")) == (FILE *)NULL )
		{	printf("Cannot open %s.  UNIX errno is %d\n", t_name, errno);
			return(-1);
		}

	/*** Read the temp file and copy the names into the list ***/
		for(i = 0; i < lcount; i++)
		{	if( fgets(list, (ARRSZ - 1), fp) == (char *)NULL )
				break;
			list[strlen(list) - 1] = (char)NULL;
			list += ARRSZ;
		}

	/*** All set.  Return to caller ***/
		fclose(fp);
		indx = i;
	}
	else		/*** Its a non-DOS path ***/
	{	if((fd = open( dir_path, O_RDONLY ))  == -1 )
		{	printf("Can't open file: %s   Errno is %d\n", dir_path, errno );
			return(-1);
		}

	/***  Load names from dir file if inode is not null  ***/
		while( read(fd, (char *)&st_dir, sizeof(st_dir)) && (indx < lcount) )
		{	if( st_dir.d_ino && st_dir.d_name[0] != '.' )
			{	strncpy((list + (indx++ * ARRSZ)), st_dir.d_name,
							sizeof(st_dir.d_name) );
					/*** truncate at 14 characters */
				*(list + (indx * ARRSZ) + 14) = (char)NULL;
			}
		}
	
	/***  Sort the array into lexicographical order  ***/
		if( indx > 1 )
			qsort( list, indx, ARRSZ, str_comp );
	
	/***  Close directory ***/
		close(fd);
	}

/*** Return 0 if nothing in list, else return number of elements in list ***/
	if( !indx )
		return(0);
	else
		return( indx );
}

/******************************************************************
*    str_comp                                                     *
*       Compares characters. Returns an int < 0 if a < b,         *
*       or an int == 0 if a == b, or an int > 0 if a > b.         *
******************************************************************/

int str_comp( a, b )
	char a[],
	     b[];
{
	/** return( a < b? -1: (a == b? 0: 1) ); **/
	return( strcmp(a, b) > 0 ? 1 : (strcmp(a, b) ? -1 : 0) );
}
