/****************************************************************************
*     mk_list                                                               *
*           This function creates a list of file names found along the      *
*           path denoted by the first passed parameter (normally 'ltp').    *
*           This file is used in a cpio command to generate the distri-     *
*           bution package.                                                 *
*           Parameters:  path - name of search directory for find command.  *
*                        fname - filename of name list.                     *
*                        bin_only - flag indicating binary only package.    *
*                                   Nonzero indicates true.                 *
*           C. Jonson  3/2/90                                               *
****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#define SASIZE 32

extern int errno;

long  mk_list( path, fname, bin_only )
	char *path, *fname;
	long bin_only;
{
	long rm_dir_names();
	long i, num_dirs;
	extern long osnum;
	char cmd[256];
	static char no_source[][SASIZE] = {"bin", "config", "cursor_keys", 
		"digitizers", "help", "macros", "menus" };
	static char source_bin[][SASIZE] = {"bin", "config", "cursor_keys", "help", 
		"digitizers", "incl", "lib", "macros", "menus", "source" };

/*** Ensure the target list file is not present ***/
	errno = 0;
	if( unlink(fname) == -1 && errno != 2 )
	{	printf("Cannot unlink %s.  UNIX errno is %d\n", fname, errno);
		return(-1);
	}

/*** Figure out the number of directories for source or bin only dist ***/
	if( bin_only )
		num_dirs = sizeof(no_source) / SASIZE;
	else
		num_dirs = sizeof(source_bin) / SASIZE;

/*** Change directory to path ***/
	if( chdir(path) == -1 )
	{	printf("Cannot change directory to %s. UNIX errno is %d.\n", path, errno);
		return(-1);
	}

/*** Loop thourgh the directory lists and generate file list ***/
	for( i = 0; i < num_dirs; i++ )
	{	if( bin_only )
			sprintf(cmd, "find ltp/%s -depth -type f -print >> %s",
						no_source[i], fname);
		else
			sprintf(cmd, "find ltp/%s -depth -type f -print >> %s",
						source_bin[i], fname);

		if( system(cmd) == -1 )
		{	printf("Cannot %s.  UNIX errno is %d\n", cmd, errno);
			return(-1);
		}
	}

/*** Go through fname and remove directory names so tar won't unload twice ***/
	if( rm_dir_names( fname, path ) == -1 )
		printf("Cannot remove directory names from %s.  Do so manually before\n\
executing the archiving command!!\n\n\7", fname );

/*** Return to ltpdist ***/
	return(0);
}

/*******************************************************************************
*  rm_dir_names                                                                *
*     This function will remove directory name records from the list of files  *
*     created previously.                                                      *
*******************************************************************************/

long rm_dir_names( fname, path )
	char *fname, *path;
{
	FILE *fp_in, *fp_out;
	char cmd[256];
	char record[256];
	char t_file[L_tmpnam + 1];
	struct stat buf;

/*** Open the files ***/
	if( (fp_in = fopen(fname, "r")) == (FILE *)NULL)
	{	printf("Cannot open %s\n", fname);
		return(-1);
	}
	
	tmpnam(t_file);
	if( (fp_out = fopen(t_file, "w")) == (FILE *)NULL)
	{	printf("Cannot open %s\n", t_file);
		fclose(fp_in);
		return(-1);
	}

/*** Read entire file.  If name is a directory, do NOT write it to temp file **/
	for( ; fgets( record, sizeof(record) + 1, fp_in) != (char *)NULL; )
	{	record[strlen(record) - 1] = (char)NULL;
		if( stat(record, &buf) == -1 )
			printf("Cannot stat %s.  UNIX errno is %d.\n", record, errno);
		
		if( buf.st_mode & 040000 ) 		/** true if its a directory **/
			continue;

		record[strlen(record)] = '\n';
		if( fputs( record, fp_out ) == EOF )
		{	printf("Cannot write to %s. \n");
			fclose(fp_in);
			fclose(fp_out);
			return(-1);
		}
	}

/*** Close files and move the new file to the ltplist file ***/
	fclose(fp_in);
	fclose(fp_out);

	sprintf(cmd, "mv %s %s", t_file, fname);
	if( system(cmd) == -1)
		printf("Cannot move %s over %s.  Use %s as the list file.\n\7",
					t_file, fname, t_file);


/*** return ***/
	return(0);
}
