/*******************************************************************************
*    moss_check    (see manual, Command Ref., User Functions)    <fc>          *
*            This checks a moss file for consistancy.                          *
*                                                                              *
*                                                                              *
*            J.Dabritz   1/29/90                                               *
*******************************************************************************/

#include <fcntl.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

long mat_solve();
long check_dbl();

long main(argc,argv)
int argc;
char *argv[];
{
	char actual[256];
	char string[1056];
	char readblock[1056];
	char *ptr, *readptr, *endptr;
	char *atrbt;

	int fd;

	long line;
	long v;
	long objects;
	long brd;
	long item;
	long n;
	long v_count;
	long e_count;

	double x,y;
	double x1,y1;
	double x2,y2;
	double x_begin,y_begin;

	
		/************* check arg count *********************************/
	if(argc < 2)
	{	printf("USAGE:   moss_check fname\n");
printf("      This program checks a single MOSS import/export file of POLYGON data.\n");
printf("      It checks for POLYGON CLOSURE, REPETE VERTICIES, DOUBLE-BACKS, and for\n");
printf("      accuracy of header (NUMBER of VERTICIES) data, and general header\n");
printf("      and vertex LINE FORMAT.  All occurrances of these errors will be noted,\n");
printf("      and their file line number noted on the screen.  The total number of \n");
printf("      errors in the file will be displayed when the check is complete.\n");
printf("      NOTE:  A repete of a coordinate pair will generate a repete error and a\n");
printf("      AND a double-back error.  A double-back error is a case where a line \n");
printf("      doubles back on itself to within 1 unit (foot/meter).  This program \n");
printf("      DOES NOT check topology.\n");
		return(0);
	}

		/************* open file ***************************************/
	sprintf(actual,"%s",argv[1]);

	fd = open(actual,O_RDONLY);
	if(fd <= 0)
	{	printf("Cannot open file  %s  , aborting.\n",actual);
		return(1);
	}
printf("\nmoss_check, for POLYGON files.   checking file %s . . . \n",actual);

		/************* read file ********************************/
	v=0;
	objects = 0;
	endptr = (char *) 0;
	readptr = endptr;
	line = 0;
	e_count = 0;
	for(;;)
	{		/*** read the next line of the file ***/
		
		ptr = string;
		*ptr = (char) 0;
		for(;;ptr++)
		{		/**** make sure that there is enough data ***/
			if(readptr >= endptr)
			{		/*** read another block of data ***/
				brd = read(fd, readblock,1024);
				if(brd <= 0)
				{	if(v)
					{	printf("--- line %d, end of file before last vertex read.\n",line);
						e_count++;
					}
					printf("==== done, total errors = %d   ====\n\n",e_count);
					return(0); 
				}
					/*** set pointers ***/
				readptr = readblock;
				endptr = readptr + brd;
			}

				/**** transfer one character ****/
			*ptr = *readptr;
			readptr++;
		
				/**** check for lf ****/
			if(*ptr == '\n')  /** line feed (lf) **/
			{	ptr++;             /** move past lf & return string **/
				*ptr = (char) 0;
				break;
			}
		}
		line++;

			/*** check end of string ***/
		n = strlen(string);
			/*** check char before lf ***/
		if(n >= 2)
		{	if(string[n-2] < ' ')
			{	printf("--- line %d, doesn't end in a simple lf.  next-to-last char is an ascii %d.\n",line,(unsigned char) string[n-2]);
				e_count++;
			}
		}

			/*** see if it should be a header line or a vertex line ***/
		if(!v)     /*** header line ***/
		{	item = 0;
			string[5] = (char) 0;
			string[45] = (char) 0;
			atrbt= string + 15;
			v = 0;
			sscanf(string,"%ld",&item);
			n = sscanf(string+46,"%ld",&v);
			if(n < 1)
			{	printf("--- line %d, inadquate header:\n%s\n",line,string);
				e_count++;
				v = 0;
			}
			v_count = 0;
		}
		else       /*** vertex line ***/
		{	string[11] = (char) 0;
			n = sscanf(string,"%lf",&x);
			n += sscanf(string+12,"%lf",&y);
			if(n < 2) 
			{	printf("--- line %d, inadquate vertex:\n%s\n",line,string);
				e_count++;
			}
			else
			{	
				v_count++;
				v--;

					/*** check for island vertex ***/
				if(x < 0.0)
				{	if(v_count > 1)
					{	if((x1 != x_begin) || (y1 != y_begin)) /* check closing */
						{	printf("--- line %d, closing coords not equal to begin coords.\n",line-1);
							e_count++;
						}
					}
					x = -x;
					v_count = 1;
				}

					/*** check for repete vertex ***/
				if(v_count > 1)
				{	if((x == x1) && (y == y1))
					{	printf("--- line %d, repete vertex\n",line);
						e_count++;
					}
						/*** check for double-back ***/
					else if(v_count > 2)
					{	if(check_dbl(x,y,x1,y1,x2,y2))
						{	printf("--- line %d, center of double-back (0).\n",line-1);
							e_count++;
						}
						else if(check_dbl(x2,y2,x,y,x1,y1))
						{	printf("--- line %d, center of double-back (2).\n",line-1);
							e_count++;
						}
					}
				}

					/*** check for last coords == first coords ***/
				if(v_count == 1)   /*** set begin coords ***/
				{	x_begin = x;
					y_begin = y;
				}
				else if(v == 0)    /*** check end/begin coords ***/
				{	if((x != x_begin) || (y != y_begin))
					{	printf("--- line %d, closing coords not equal to begin coords.\n",line);
						e_count++;
					}
				}

					/** update xy1,xy2 **/
				x2 = x1;
				x1 = x;
				y2 = y1;
				y1 = y;
			}
		}
	}
}



/*******************************************************************************
*   check_dbl checks to see if point x,y is within 1 unit of x1y1-->x2y2 along *
*   the line.  If so, it returns a 1, otherwise a 0.                           *
*******************************************************************************/
long check_dbl(x,y,x1,y1,x2,y2)
double x,y,x1,y1,x2,y2;
{
	double dx,dy,dxp,dyp,length;
	double mat[2][3];


		/*** check x,y against pt1-pt2 **/
		/*** find pt1-2 **/
	dx = x2-x1;
	dy = y2-y1;

		/*** make a perpendicular normalized copy ***/
	length = sqrt(dx * dx  +  dy * dy);
	if(length == 0.0)
		return(0);
	dx = dx / length;
	dy = dy / length;
	dxp = dy;
	dyp = -dx;

		/*** calc use of vectors from x,y to x1,y1 along dx,dy & dxp,dyp **/
	mat[0][0] = dx;
	mat[0][1] = dxp;
	mat[0][2] = x - x1;
	mat[1][0] = dy; 
	mat[1][1] = dyp; 
	mat[1][2] = y - y1;


	if(mat_solve(2,3,&mat[0][0]))
		return(0);


	if((mat[0][2] >= 0.0) && (mat[0][2] <= length))
	{	if((mat[1][2] > -1.0) && (mat[1][2] < 1.0))
			return(1);
	}

	return(0);
}

/*******************************************************************************
*     mat_solve                                                                *
*         This program reduces the square left-hand portion of a 2-dimentional *
*     matrix of doubles to the identity.  The portion to the right is treated  *
*     identically and will have whatever values it should.  The number of      *
*     rows must be less than or equal to the number of colums.  There must     *
*     be at least 2 rows.                                                      *
*                                                                              *
*     Parameters:  long rows, cols:   number of rows & columns (rows <= cols)  *
*                                                              (rows >= 2)     *
*                  double *matrix:    pointer to first double of matrix.       *
*                                                                              *
*     Globals changed:  none                                                   *
*                                                                              *
*     Returned:   long 0 = matrix reduced                                      *
*                      1 = solution failed  (rows not unique etc.)             *
*                                                                              *
*     J.Dabritz  8/4/89                                                        *
*******************************************************************************/


long mat_solve(rows, cols, matrix)
	long rows, cols;
	double *matrix;
{
	long c;
	long row, col;
	
	double factor;
	double save;


		/*** check rows & cols ***/
	if((rows < 2) || (cols < rows))
		return(1);

		/*** solve for each column ****/
	for(col = 0; col < rows; col++)
	{		/** find with non-zero value in column col **/
		for(row = col; row < rows; row++)
		{	if(matrix[row * cols + col] != 0.0)
				break;
		}
			/*** check to see if one found ***/
		if(row >= rows)
			return(1);

			/*** switch rows if row != col ***/
		if(row != col)
		{	for(c=0; c<cols; c++)
			{	save = matrix[col * cols + c];
				matrix[col * cols + c] = matrix[row * cols + c];
				matrix[row * cols + c] = save;
			}
		}
			/*** set column value to 1 for diagonal element ***/
		factor = 1.0 / matrix[col * cols + col];
		matrix[col * cols + col] = 1.0;
		for(c = col + 1; c<cols; c++)
			matrix[col * cols + c] = matrix[col * cols + c] * factor;

			/*** 0 out all other rows for this column ***/
		for(row=0; row<rows; row++)
		{	if(row != col)
			{	factor = matrix[row * cols + col];
				matrix[row * cols + col] = 0.0;
				for(c=col+1; c<cols; c++)
					matrix[row * cols + c] -= matrix[col * cols + c] * factor;
			}
		}

	}

		/*** return ***/
	return(0);
}
