
/*****************************************************************************
*    set_configs                                                             *
*              This program will set many values of the /u[sr]/ltp/config    *
*              files including tolerance, config, and header.  The user is   *
*              prompted for values.  If default values are available, it is  *
*              listed as such.                                               *
*                                                                            *
*        C. Jonson   10/89                                                   *
*****************************************************************************/
#include <stdio.h>
#include <string.h> 
#include <malloc.h> 
#define STRLEN 96
#define TOLERANCE 50000
#define RASTERH 2400
#define RASTERW 1800
#define CONTOURINT 40
#define MAPH 7.5
#define MAPW 7.5
#define SCALE 24000

long set_configs(path, unix_sys)
	char *path;
	long unix_sys;
{
	double scan_double;
	long offset = 0;
	long tolerance = 0;
	long scan_long = 0;
	long set_long_parm(), set_str_parm();
	FILE *fp;
	char config_path[256];
	char *conf_ptr;
	char resp[256];
	char scan_str[256];
	
/*** Initialize the strings ***/
	*resp = *scan_str = 0;

/*** Modify the tolerance file ***/
	sprintf(config_path, "%s/ltp/config/tolerance", path);
	if( (fp = fopen(config_path, "w")) == (FILE *)NULL)
	{	printf("Unable to open the %s file.  This file should be set\n\
by your systems administrator before using LTPlus.\n\n");
	}

	printf("\n\nEnter the value in feet for the digitizing tolerance\n\
\t\t(default value is %ld feet)  -->  ", TOLERANCE);
	fflush(stdin);
	gets(resp);
	sscanf(resp, "%ld", &tolerance);
	if( !tolerance )
		tolerance = TOLERANCE;

	fprintf(fp, "%ld\n", tolerance);
	fclose(fp);
	printf("The %s file now has the value of %d feet.\n", config_path, tolerance);
	*resp = 0;

/*** Modify the config file ***/
	printf("\nDo you wish to modify parameters of the config file or would\n\
 you prefer to use the default values? [yes / no (default)] --> ");
	fflush(stdin);
	gets(resp);
	if( tolower((int)resp[0]) == 121 )             /* 121 is 'y' */
	{	sprintf(config_path, "%s/ltp/config/config", path);
		if( (fp = fopen(config_path, "r")) == (FILE *)NULL)
		{	printf("Unable to open the %s file.  This file should be set\n\
by your systems administrator before using LTPlus.\n\n", config_path);
		}
	
	/*** Copy the config file into memory ***/
		if( (conf_ptr = (char *)calloc( 150, STRLEN )) == (char *)NULL)
		{	printf("\nUnable to allocate space for config file updating.\n\
Contact your LTPlus support person.\n\n");
			fclose(fp);
			return(-1);
		}
		while(fgets((conf_ptr + offset), STRLEN - 1, fp) )
			offset += STRLEN;
		*(conf_ptr + offset) = 0;          /* set last entry to NULL */
		fclose(fp);

	/*** Modify the raster height ***/
		printf("\n\nEnter the raster height (default is %ld pixels) -->  ", 
					RASTERH);
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = RASTERH;
		if( set_long_parm("rasterh", scan_long, conf_ptr) )
			printf("Cannot set rasterh value in %s.\n", config_path);
		scan_long = *resp = 0;
		
	/*** Modify the raster width ***/
		printf("\n\nEnter the raster width (default is %ld pixels) -->  ",
					RASTERW);
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = RASTERW;
		if( set_long_parm("rasterw", scan_long, conf_ptr) )
			printf("Cannot set rasterw value in %s.\n", config_path);
		scan_long = *resp = 0;
		
	/*** Modify the mouse type ***/
		if( unix_sys )
			printf("\n\nEnter the mouse type:\n\t0.  AT&T Bus Mouse (default)\
\n\t1.  Microsoft Serial Mouse\n\t2.  Logitech Serial Mouse\n\t\t\
Select mouse type [0/1/2] -->  ");
		else
			printf("\n\nEnter the mouse type:\n\t0.  Logitech Serial Mouse \
(default)\n\t1.  Microsoft Serial Mouse\n\t2.  Other Serial Mouse\n\t\t\
Select mouse type [0/1/2] -->  ");

		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = 0;
		switch(scan_long)
		{	case 1:
				strcpy(scan_str, "mssmse");
				break;
			case 2:
				if( unix_sys )
					strcpy(scan_str, "ltsmse");
				else
				{	printf("Sorry, no other serial mice supported.\n");
					scan_str[0] = 0;
				}
				break;
			case 0:
			default:
				if( unix_sys )
					strcpy(scan_str, "attbmse");
				else
					strcpy(scan_str, "ltsmse");
				break;
		}
		if( set_str_parm("mouse_type", scan_str, conf_ptr) )
			printf("Cannot set mouse_type value in %s.\n", config_path);
		scan_long = *scan_str = *resp = 0;

	/*** Modify the mouse address (unless "attbmse" chosen) ***/
		if( strcmp(scan_str, "attbmse") )		/** Not an ATT Bus Mouse **/
		{	if( unix_sys )
				printf("\n\nEnter the mouse address:\n\t0.  /dev/tty00 \
(default)\n\t1.  /dev/tty1a\n\t2.  /dev/tty2a\n\t3.  Other address\n\t\t\
Select mouse address [0/1/2/3] -->  ");
			else
				printf("\n\nEnter the mouse address:\n\t0.  /dev/tty1a \
(default)\n\t1.  /dev/tty2a\n\t2.  /dev/tty00\n\t3.  Other address\n\t\t\
Select mouse address [0/1/2/3] -->  ");

			fflush(stdin);
			gets(resp);
			sscanf(resp, "%ld", &scan_long);
			if( !scan_long )
				scan_long = 0;
			switch(scan_long)
			{	case 1:
					if( unix_sys )
						strcpy(scan_str, "/dev/tty1a");
					else
						strcpy(scan_str, "/dev/tty2a");
					break;
				case 2:
					if( unix_sys )
						strcpy(scan_str, "/dev/tty2a");
					else
						strcpy(scan_str, "/dev/tty00");
					break;
				case 3:
					printf("\n\t\t\tEnter mouse address  --> ");
					fflush(stdin);
					gets(resp);
					strcpy(scan_str, resp);
					break;
				case 0:
				default:
					if( unix_sys )
						strcpy(scan_str, "/dev/tty00");
					else
						strcpy(scan_str, "/dev/tty1a");
					break;
			}
			if( set_str_parm("mouse_adr", scan_str, conf_ptr) )
				printf("Cannot set mouse_adr value in %s.\n", config_path);
		}
		scan_long = *scan_str = *resp = 0;

		printf("\n\nEnter the digitizer 1 type:\n\
\t0.  CalComp 9100 using format 2 with 16 button puck (default)\n\
\t1.  CalComp 9100 using format 5 with 16 button puck\n\
\t2.  CalComp 9000 using format 2 with 12 button puck\n\
\t3.  Altek 30 using format 8 with 16 button puck\n\
\t4.  Altek 40 using format 8 (binary format 1) with 16 button puck\n");
printf("\t5.  Altek 40 using format 8 (binary format 2) with 16 button puck\n\
\t6.  Kurta ONE format 2b with 4 button puck\n\
\t7.  Kurta ONE format 3 with 4 button puck\n\
\t8.  Kurta THREE format 2 with 4 button puck\n\
\t9.  None\n\
\t\tSelect digitizer type [0/1/2/3/4/5/6/7/8/9] -->  ");
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = 0;
		switch(scan_long)
		{	case 1:
				strcpy(scan_str, "cc9100f5");
				break;
			case 2:
				strcpy(scan_str, "cc9000f2");
				break;
			case 3:
				strcpy(scan_str, "al30f8");
				break;
			case 4:
				strcpy(scan_str, "al40f81");
				break;
			case 5:
				strcpy(scan_str, "al40f82");
				break;
			case 6:
				strcpy(scan_str, "ku1f2b");
				break;
			case 7:
				strcpy(scan_str, "ku1f3");
				break;
			case 8:
				strcpy(scan_str, "ku3f2");
				break;
			case 0:
			default:
				strcpy(scan_str, "cc9100f2");
				break;
		}
		if( set_str_parm("dig1_type", scan_str, conf_ptr) )
			printf("Cannot set dig1_type value in %s.\n", config_path);
		scan_long = *scan_str = *resp = 0;

		printf("\n\nEnter the digitizer 1 address (no default value) -->  "); 
		fflush(stdin);
		gets(resp);
		if( strlen(resp) )
			if( set_str_parm("dig1_adr", resp, conf_ptr) )
				printf("Cannot set dig1_adr value in %s.\n", config_path);
		*scan_str = *resp = 0;

		printf("\n\nEnter the plotter 1 type (HP7580 default) -->  ");
		fflush(stdin);
		gets(resp);
		if( strlen(resp) == 0 )
			strcpy(resp, "HP7580");
		if( set_str_parm("plt1_type", resp, conf_ptr) )
			printf("Cannot set plt1_type value in %s.\n", config_path);
		*resp = 0;

		printf("\n\nEnter the plotter 1 address (no default value) -->  "); 
		fflush(stdin);
		gets(resp);
		if( strlen(resp) )
			if( set_str_parm("plt1_adr", resp, conf_ptr) )
				printf("Cannot set mouse_adr value in %s.\n", config_path);
		*scan_str = *resp = 0;

		printf("\n\nEnter the menuset type:\n\t0.  poly_menu (default)\
\n\t1.  contour_menu\n\t2.  line_menu\n\t3.  point_menu\n\
\t\tSelect menuset type [0/1/2/3] -->  ");
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = 0;
		switch(scan_long)
		{	case 1:
				strcpy(scan_str, "contour_menu");
				break;
			case 2:
				strcpy(scan_str, "line_menu");
				break;
			case 3:
				strcpy(scan_str, "point_menu");
				break;
			case 0:
			default:
				strcpy(scan_str, "poly_menu");
				break;
		}
		if( set_str_parm("menuset", scan_str, conf_ptr) )
			printf("Cannot set menuset value in %s.\n", config_path);
		scan_long = *scan_str = *resp = 0;

		printf("\n\nSelect the map path:\n\t0.  %s/ltp/maps (default)\
\n\t1.  Other\n\t\tSelect map path [0/1] -->  ", path);
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = 0;
		switch(scan_long)
		{	case 1:
				printf("\nEnter map path --> ");
				fflush(stdin);
				gets(resp);
				strcpy(scan_str, resp);
				break;
			case 0:
			default:
				sprintf(scan_str, "%s/ltp/maps", path);
				break;
		}
		if( set_str_parm("maps_path", scan_str, conf_ptr) )
			printf("Cannot set maps_path value in %s.\n", config_path);
		scan_long = *scan_str = *resp = 0;

		printf("\n\nSelect the import path:\n\t0.  %s/ltp/import (default)\
\n\t1.  Other\n\t\tSelect import path [0/1] -->  ", path);
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = 0;
		switch(scan_long)
		{	case 1:
				printf("\nEnter import path --> ");
				fflush(stdin);
				gets(resp);
				strcpy(scan_str, resp);
				break;
			case 0:
			default:
				sprintf(scan_str, "%s/ltp/import", path);
				break;
		}
		if( set_str_parm("import_path", scan_str, conf_ptr) )
			printf("Cannot set import_path value in %s.\n", config_path);
		scan_long = *scan_str = *resp = 0;


		printf("\n\nSelect the export path:\n\t0.  %s/ltp/export (default)\
\n\t1.  Other\n\t\tSelect export path [0/1] -->  ", path);
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = 0;
		switch(scan_long)
		{	case 1:
				printf("\nEnter export path --> ");
				fflush(stdin);
				gets(resp);
				strcpy(scan_str, resp);
				break;
			case 0:
			default:
				sprintf(scan_str, "%s/ltp/export", path);
				break;
		}
		if( set_str_parm("export_path", scan_str, conf_ptr) )
			printf("Cannot set export_path value in %s.\n", config_path);
		scan_long = *scan_str = *resp = 0;

	/*** Write config array to the config file ***/
		if( (fp = fopen(config_path, "w")) == (FILE *)NULL)
		{	printf("Unable to open the %s file.  This file should be set\n\
by your systems administrator before using LTPlus.\n\n");
		}
		for( offset = 0; *(conf_ptr + offset); offset += 96)
			fputs( (conf_ptr + offset), fp);
	
		fclose(fp);               /* Close the config file */
		free(conf_ptr);           /* Free memory */
		printf("The config file has been updated.\n");	
	}
	else
		printf("\nThe %s file should have certian values set \n\
before running LTPlus.  See your LTPlus User's Guide.\n\n", config_path);

/*** Modify the header file ***/
	printf("\nDo you wish to modify parameters of the header file or would\n\
 you prefer to use the default values? [yes / no (default)] --> ");
	fflush(stdin);
	resp[0] = 0;              /* NULL out resp array */
	gets(resp);
	if( tolower((int)resp[0]) == 121 )             /* 121 is 'y' */
	{	sprintf(config_path, "%s/ltp/config/header", path);
		if( (fp = fopen(config_path, "r")) == (FILE *)NULL)
		{	printf("Unable to open the %s file.  This file should be set\n\
by your systems administrator before using LTPlus.\n\n", config_path);
		}
	
	/*** Copy the header file into memory ***/
		offset = 0;
		if( (conf_ptr = (char *)calloc( 150, STRLEN )) == (char *)NULL)
		{	printf("\nUnable to allocate space for header file updating.\n\
Contact your LTPlus support person.\n\n");
			fclose(fp);
			return(-1);
		}
		offset = 0;
		while(fgets((conf_ptr + offset), STRLEN - 1, fp) )
			offset += STRLEN;
		*(conf_ptr + offset) = 0;          /* set last entry to NULL */
		fclose(fp);

	/*** Modify the contour interval ***/
		printf("\n\nEnter the contour interval (default is %ld feet) -->  ",
					CONTOURINT);
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = CONTOURINT;
		if( set_long_parm("contour_interval", scan_long, conf_ptr) )
			printf("Cannot set contour_interval value in %s.\n", config_path);
		scan_long = *resp = 0;
		
	/*** Modify the layer name  ***/
		printf("\n\nEnter the layer name (no default value) -->  "); 
		fflush(stdin);
		gets(resp);
		if( strlen(resp) )
			if( set_str_parm("layer_name", resp, conf_ptr) )
				printf("Cannot set layer_name in %s.\n", config_path);
		*scan_str = *resp = 0;

	/*** Modify the map height (in minutes)  ***/
		printf("\n\nEnter the map height in minutes  \
(default is %.1f minutes) -->  ", MAPH);
		fflush(stdin);
		gets(resp);
		scan_double = 0.0;
		sscanf(resp, "%lf", &scan_double);
		if( scan_double == 0.0 )
			scan_double = MAPH;
		sprintf(scan_str, "%.3lf", scan_double);
		if( set_str_parm("map_ht", scan_str, conf_ptr) )
			printf("Cannot set map height value in %s.\n", config_path);
		*resp = 0;
		scan_double = 0.0;
		
	/*** Modify the map width (in minutes)  ***/
		printf("\n\nEnter the map width in minutes  \
(default is %.1f minutes) -->  ", MAPW);
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%lf", &scan_double);
		if( scan_double == 0.0 )
			scan_double = MAPW;
		sprintf(scan_str, "%.3lf", scan_double);
		if( set_str_parm("map_wd", scan_str, conf_ptr) )
			printf("Cannot set map width value in %s.\n", config_path);
		*resp = 0;
		scan_double = 0.0;
		
	/*** Modify the map type ***/
		printf("\n\nEnter the map type:\n\t0.  Polygon map (default)\
\n\t1.  Line map\n\t2.  Point map\n\t\t\
Select map type [0/1/2] -->  ");
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = 0;
		switch(scan_long)
		{	case 1:
				strcpy(scan_str, "line");
				break;
			case 2:
				strcpy(scan_str, "point");
				break;
			case 0:
			default:
				strcpy(scan_str, "poly");
				break;
		}
		if( set_str_parm("maptype", scan_str, conf_ptr) )
			printf("Cannot set map type value in %s.\n", config_path);
		scan_long = *scan_str = *resp = 0;

	/*** Modify the map scale  ***/
		printf("\n\nEnter the map scale  (default is %ld) -->  ", SCALE);
		fflush(stdin);
		gets(resp);
		sscanf(resp, "%ld", &scan_long);
		if( !scan_long )
			scan_long = SCALE;
		if( set_long_parm("scale", scan_long, conf_ptr) )
			printf("Cannot set map scale value in %s.\n", config_path);
		scan_long = *resp = 0;

	/*** Write header array to the header file ***/
		if( (fp = fopen(config_path, "w")) == (FILE *)NULL)
		{	printf("Unable to open the %s file.  This file should be set\n\
by your systems administrator before using LTPlus.\n\n");
		}
		for( offset = 0; *(conf_ptr + offset); offset += 96)
			fputs( (conf_ptr + offset), fp);
	
		fclose(fp);               /* Close header file */
		free(conf_ptr);           /* Free memory */
		printf("The header file has been updated.\n");	
	}
	else
		printf("\nThe %s file should have certian values set \n\
before running LTPlus.  See your LTPlus User's Guide.\n\n", config_path);
		
	return(0);
}

long set_str_parm(parm, value, ptr)
	char *parm, *value;
	char *ptr;
{
	long len;
	long offset;

/*** Calculate the parm length ***/
	len = strlen(parm);
	
/*** Read array until the desired match is found ***/
	for(offset = 0; strncmp((ptr + offset), parm, len); offset += STRLEN );
	sprintf((ptr + offset), "%s = %s\n", parm, value);
	
	return(0);
}

long set_long_parm(parm, value, ptr)
	char *parm;
	long value;
	char *ptr;
{
	long len;
	long offset;

/*** Calculate parm length  ***/
	len = strlen(parm);
	
/*** Read array until the desired match is found ***/
	for(offset = 0; strncmp((ptr + offset), parm, len); offset += STRLEN );
	sprintf((ptr + offset), "%s = %ld\n", parm, value);
	
	return(0);
}
