/*******************************************************************************
*  update_ltp                                                                  *
*     This program updates the following directories of LTPlus except for:     *
*     1.   config                                                              *
*     2.   import and export                                                   *
*     3.   maps                                                                *
*     To determine the permissions and ownership to set downloaded files to,   *
*     two LTPlus files (ltp/bin/ltplus ltp/config/config) are examined for     *
*     their ownership.  If they are the same as each other, the downloaded     *
*     files are set to that ownership.  Otherwise, the user is prompted to     *
*     select one of the owners and then the rest of the files are set.         *
*         C. Jonson 3/2/90                                                     *
*******************************************************************************/
#include <stdio.h>
#include <pwd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
struct passwd *getpwuid();

long update_ltp( unix_sys )
	long unix_sys;
{
	long i;
	long no_hit;
	long offset;
	long user_uid;
	unsigned short old_uid[2];
	char cmd[256];
	char orig_rec[256];
	char path[256];
	char *ptr;
	char record[256];
	char t_path[256];
	char tmp_msg[256];
	char user_name[96];
	char schitzos[2][96];
	FILE *fp;
	struct stat buf;
	struct passwd *pw;

	printf("\n\nUpdate selected.  This will update all of the LTPlus files \
with the\nexception of the config, import, export, and maps directories.");

	printf("\n\nSince there are many files that can be user customized, \
it is prudent\nto backup the files you have modified before continuing.\n");

	printf("\nPress RETURN to continue, or DELETE to quit\7 ");
	
	fflush(stdin);
	getchar();

/*** Figure out what directory contains ltp software ***/
	if( tmpnam(t_path) == (char *)NULL )
	{	printf("Cannot generate a temp file.");
		return(-1);
	}

	sprintf(cmd, "find / -name config -print > %s", t_path);

	if( system(cmd) == -1 )
	{	printf("Cannot perform \'find\' command.");
		return(-1);
	}

	if( (fp = fopen(t_path, "r")) == (FILE *)NULL )
	{	printf("Cannot open %s.", t_path);
		return(-1);
	}

/*** Read the list of files with 'config' in the path.  Check the files ***/
/*** for 'ltp'. If found, compare owners.                               ***/
	for(i = 0; fgets(record, sizeof(record) - 1, fp) != (char *)NULL; )
	{	record[strlen(record) - 1] = (char)NULL;
		strcpy(orig_rec, record);			/** Store original record **/

		for( no_hit = offset = 0;; )			/** 47 is a '/' **/
		{	if( (ptr = strchr((record + offset), 47)) == (char *)NULL )
			{	no_hit = 1;
				break;
			}
			else 
			{	ptr++;
				if( !strncmp(ptr, "ltp", 3) )
				{	*(ptr - 1) = (char)NULL;		/** clip end of path **/
					break;
				}
				else
					offset = ptr - record;
			}
		}

		if( no_hit )
			continue;

		if( stat(orig_rec, &buf) )
		{	sprintf(tmp_msg, "Cannot stat %s .", orig_rec);
			return(-1);
		}

		old_uid[i++] = buf.st_uid;

		if( i == 2 )
			break;
	}

/*** If no name found, tell installer they are up the creek w/o a paddle ***/
	if( i == 0 )
	{	printf("No config files found. ");
		return(-1);
	}

/*** Take the name of the config file and get the path name preceding 'ltp' ***/
	*(ptr - 1) = (char)NULL;
	strcpy(path, record);

/*** Compare user ids.  If equal, set the downloaded files to this owner.  ***/
/*** Else, ask installer what ownership they want for the files.           ***/
	if( i == 2 )
	{	if( old_uid[0] == old_uid[1] )		/** Same user for config files **/
		{	user_uid = old_uid[0];
			if( (pw = getpwuid(user_uid)) == (struct passwd *)NULL )
				*user_name = (char)NULL;
			else
				strcpy(user_name, pw->pw_name);
		}
		else
		{	*user_name = (char)NULL;		/** Different owners in LTPlus **/
			for( i = 0; i < 2; i++ )
			{	if( (pw = getpwuid(user_uid)) == (struct passwd *)NULL )
				{	*user_name = (char)NULL;
					break;
				}
				else 
					strcpy(schitzos[i], pw->pw_name);
			}
			printf("\n\tNOTE:  I have noticed that %s and %s own LTPlus files. \
As only ONE owner\n\tis allowed for LTPlus files (except for the tolerance \
file), you\n\twill be asked to choose an owner for LTPlus files in a few minutes.\n\7", schitzos[0], schitzos[1]);
			sleep(4);
		}
	}
	else					/** Only one 'config' found. Use its owner **/
	{	user_uid = old_uid[0];
		if( (pw = getpwuid(user_uid)) == (struct passwd *)NULL )
			*user_name = (char)NULL;
		else
			strcpy(user_name, pw->pw_name);
	}

/*** Copy the config directory to a temp location ***/
	sprintf(cmd, "copy -r -o %s/ltp/config %s/ltp/config.bkup", path, path );
	if( system(cmd) == -1)
	{	printf("Cannot backup config directory to %s/ltp/config.bkup ", path);
		return(-1);
	}	
	printf("\nCurrent config directory backed up as %s/ltp/config.bkup\n",path);

/*** Download the files ***/
	if( load_ltp(path, unix_sys ) )
	{	printf("Problem loading files.  ");
		return(-1);
	}

	if( change_perms(path, user_name) )
	{	printf("\nCannot change permissions.  ");
		return(-1);
	}

/*** Unlink the temp file.  Don't bother checking return code, as its ***/
/*** in the temp directory cleared upon boot-up                       ***/
	unlink(t_path);

/*** Copy original config directory over top of new config directory ***/
	sprintf(cmd, "copy -r -o %s/ltp/config.bkup %s/ltp/config", path, path);
	if( system(cmd) == -1)
	{	printf("Cannot overlay the new config directory.\n");
		return(-1);
	}
	printf("\nOriginal config directory is now restored!\n");

	sprintf(cmd, "rm -r %s/ltp/config.bkup", path );
	if( system(cmd) == -1)
	{	printf("Cannot remove the config.bkup directory. ");
		return(-1);
	}
	printf("\n%s/ltp/config.bkup now removed.\n", path );

	return(0);
}
